@@ -1,7 +1,7 @@
 ;;;
 ;;; extract - filter bilingual texinfo document
 ;;;
-;;;  Copyright (c) 2001-2018  Shiro Kawai  <shiro@acm.org>
+;;;  Copyright (c) 2001-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;  Permission to use, copy, modify, distribute this software and
 ;;;  accompanying documentation for any purpose is hereby granted,
@@ -15,9 +15,9 @@
 
 (use gauche.parseopt)
 (use gauche.charconv)
-(use gauche.parameter)
 (use srfi-13)
 (use file.util)
+(use util.levenshtein)
 
 (define *version*
   (string-trim-both (file->string (or (find (cut sys-access <> R_OK)
@@ -52,6 +52,9 @@
      read-line)))
 
 (define (filter pattern-in pattern-out)
+  ;; This regexp picks potential typo of extract directives.
+  (define suspicious
+    #/^@c ([A-Za-z]{2,7}(?![A-Za-z:', -])([^A-Za-z:', -])?)/)
   (define (in line)
     (rxmatch-case line
       [test eof-object?]
@@ -73,6 +76,7 @@
       [#/^@c MOD\s+(\S+)$/ (#f module)
              (display #"@{@t{~|module|}@}\n")
              (in (read-line))]
+      [suspicious (#f word) (check-typo word line) (in (read-line))]
       [test (^_ (eq? (lang) 'en))
             (display (regexp-replace-all #/@VERSION@/ line *version*))
             (newline) (in (read-line))]
@@ -96,10 +100,23 @@
       [test eof-object?]
       [pattern-in ()  (in (read-line))]
       [#/^@c COMMON$/ () (in (read-line))]
+      [suspicious (#f word) (check-typo word line) (out (read-line))]
       [else (out (read-line))]))
 
   (in (read-line)))
 
+;; Detect potential typo of extract directives.
+;; I tend to miss the warnings, so make it an error.  Add allowed words
+;; as needed.
+(define (check-typo word line)
+  (when (and (string-any char-upper-case? word)
+             (let1 ds (re-distances word '("EN" "JP" "COMMON" "NODE" "MOD"
+                                           "TODO" "NOTE" "NB"))
+               (<= 1 (apply min ds) 2)))
+    (errorf "Unrecognized extract directive ~s at or near ~s:~d\n"
+            word (port-name (current-input-port))
+            (port-current-line (current-input-port)))))
+
 ;; We search relative to the current directory first, then
 ;; relative to $srcdir; for FILE may be the generated one.
 (define (find-included-file file)
