@@ -3,8 +3,10 @@
   :description "Calculate pi and e in arbitrary precision\n\
            This is a sample package to show how to write Gauche extension.\n\
            The C code implements Spigot algorithm."
-  :require (("Gauche" (>= "0.9.5_")))
+  :require (("Gauche" (>= "0.9.7")))
+  :providing-modules (spigot)
   :authors ("Shiro Kawai <shiro@acm.org>")
-  :repository "https://github.com/Gauche"
   :licenses ("Public Domain")
+  :homepage "https://practical-scheme.net/gauche"
+  :repository "https://github.com/Gauche"
   )
