@@ -1,7 +1,7 @@
 /*
  * binary.c - Binary I/O routines
  *
- *   Copyright (c) 2004-2018  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2004-2020  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -33,7 +33,7 @@
 
 #include <gauche.h>
 #include <gauche/extend.h>
-#include <gauche/bytes_inline.h>
+#include <gauche/priv/bytesP.h>
 #include "binary.h"
 
 #define ENSURE_IPORT(var)  if (!var) var = SCM_CURIN
@@ -44,27 +44,27 @@
 #define SWAP_64(e, v) do { if (SWAP_REQUIRED(e)) SWAP_8(v); } while (0)
 
 #ifdef DOUBLE_ARMENDIAN
-#define SWAP_D(e, v)                            \
-    do {                                        \
-        if (IS_ARM_LE(Scm_NativeEndian())) {    \
-            if (IS_BE(e)) SWAP_ARM2BE(v);       \
-            else if (IS_LE(e)) SWAP_ARM2LE(v);  \
-        } else {                                \
-            if (IS_ARM_LE(e)) SWAP_ARM2LE(v);   \
-            else if (IS_BE(e)) SWAP_8(v);       \
-        }                                       \
+#define SWAP_D(e, v)                                    \
+    do {                                                \
+        if (SCM_IS_ARM_LE(Scm_NativeEndian())) {        \
+            if (SCM_IS_BE(e)) SWAP_ARM2BE(v);           \
+            else if (SCM_IS_LE(e)) SWAP_ARM2LE(v);      \
+        } else {                                        \
+            if (SCM_IS_ARM_LE(e)) SWAP_ARM2LE(v);       \
+            else if (SCM_IS_BE(e)) SWAP_8(v);           \
+        }                                               \
     } while (0)
 #elif WORDS_BIGENDIAN
-#define SWAP_D(e, v)                            \
-    do {                                        \
-        if (IS_LE(e)) SWAP_8(v);                \
-        else if (IS_ARM_LE(e)) SWAP_ARM2BE(v);  \
+#define SWAP_D(e, v)                                    \
+    do {                                                \
+        if (SCM_IS_LE(e)) SWAP_8(v);                    \
+        else if (SCM_IS_ARM_LE(e)) SWAP_ARM2BE(v);      \
     } while (0)
 #else /*!WORDS_BIGENDIAN*/
-#define SWAP_D(e, v)                            \
-    do {                                        \
-        if (IS_BE(e)) SWAP_8(v);                \
-        else if (IS_ARM_LE(e)) SWAP_ARM2LE(v);  \
+#define SWAP_D(e, v)                                    \
+    do {                                                \
+        if (SCM_IS_BE(e)) SWAP_8(v);                    \
+        else if (SCM_IS_ARM_LE(e)) SWAP_ARM2LE(v);      \
     } while (0)
 #endif /*!WORDS_BIGENDIAN*/
 
@@ -91,7 +91,7 @@ ScmObj Scm_ReadBinaryU8(ScmPort *iport, ScmSymbol *endian)
 {
     int b;
     ENSURE_IPORT(iport);
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     if ((b = Scm_Getb(iport)) == EOF) return SCM_EOF;
     else return SCM_MAKE_INT(b);
 }
@@ -100,7 +100,7 @@ ScmObj Scm_ReadBinaryS8(ScmPort *iport, ScmSymbol *endian)
 {
     int b;
     ENSURE_IPORT(iport);
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     if ((b = Scm_Getb(iport)) == EOF) return SCM_EOF;
     if (b >= 128) b -= 256;
     return SCM_MAKE_INT(b);
@@ -109,7 +109,7 @@ ScmObj Scm_ReadBinaryS8(ScmPort *iport, ScmSymbol *endian)
 ScmObj Scm_ReadBinaryU16(ScmPort *iport, ScmSymbol *endian)
 {
     swap_u16_t v;
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     if (getbytes(v.buf, 2, iport) == EOF) return SCM_EOF;
     SWAP_16(endian, v);
     return SCM_MAKE_INT(v.val);
@@ -118,7 +118,7 @@ ScmObj Scm_ReadBinaryU16(ScmPort *iport, ScmSymbol *endian)
 ScmObj Scm_ReadBinaryS16(ScmPort *iport, ScmSymbol *endian)
 {
     swap_s16_t v;
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     if (getbytes(v.buf, 2, iport) == EOF) return SCM_EOF;
     SWAP_16(endian, v);
     return SCM_MAKE_INT(v.val);
@@ -127,7 +127,7 @@ ScmObj Scm_ReadBinaryS16(ScmPort *iport, ScmSymbol *endian)
 ScmObj Scm_ReadBinaryU32(ScmPort *iport, ScmSymbol *endian)
 {
     swap_u32_t v;
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     if (getbytes(v.buf, 4, iport) == EOF) return SCM_EOF;
     SWAP_32(endian, v);
     return Scm_MakeIntegerFromUI(v.val);
@@ -136,7 +136,7 @@ ScmObj Scm_ReadBinaryU32(ScmPort *iport, ScmSymbol *endian)
 ScmObj Scm_ReadBinaryS32(ScmPort *iport, ScmSymbol *endian)
 {
     swap_s32_t v;
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     if (getbytes(v.buf, 4, iport) == EOF) return SCM_EOF;
     SWAP_32(endian, v);
     return Scm_MakeInteger(v.val);
@@ -145,7 +145,7 @@ ScmObj Scm_ReadBinaryS32(ScmPort *iport, ScmSymbol *endian)
 ScmObj Scm_ReadBinaryU64(ScmPort *iport, ScmSymbol *endian)
 {
     swap_u64_t v;
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     if (getbytes(v.buf, 8, iport) == EOF) return SCM_EOF;
     SWAP_64(endian, v);
     return Scm_MakeIntegerU64(v.val);
@@ -154,7 +154,7 @@ ScmObj Scm_ReadBinaryU64(ScmPort *iport, ScmSymbol *endian)
 ScmObj Scm_ReadBinaryS64(ScmPort *iport, ScmSymbol *endian)
 {
     swap_s64_t v;
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     if (getbytes(v.buf, 8, iport) == EOF) return SCM_EOF;
     SWAP_64(endian, v);
     return Scm_MakeInteger64(v.val);
@@ -163,7 +163,7 @@ ScmObj Scm_ReadBinaryS64(ScmPort *iport, ScmSymbol *endian)
 ScmObj Scm_ReadBinaryF16(ScmPort *iport, ScmSymbol *endian)
 {
     swap_f16_t v;
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     if (getbytes(v.buf, 2, iport) == EOF) return SCM_EOF;
     SWAP_16(endian, v);
     return Scm_MakeFlonum(Scm_HalfToDouble(v.val));
@@ -172,7 +172,7 @@ ScmObj Scm_ReadBinaryF16(ScmPort *iport, ScmSymbol *endian)
 ScmObj Scm_ReadBinaryF32(ScmPort *iport, ScmSymbol *endian)
 {
     swap_f32_t v;
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     if (getbytes(v.buf, 4, iport) == EOF) return SCM_EOF;
     SWAP_32(endian, v);
     return Scm_MakeFlonum((double)v.val);
@@ -181,7 +181,7 @@ ScmObj Scm_ReadBinaryF32(ScmPort *iport, ScmSymbol *endian)
 ScmObj Scm_ReadBinaryF64(ScmPort *iport, ScmSymbol *endian)
 {
     swap_f64_t v;
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     if (getbytes(v.buf, 8, iport) == EOF) return SCM_EOF;
     SWAP_D(endian, v);
     return Scm_MakeFlonum(v.val);
@@ -194,7 +194,7 @@ ScmObj Scm_ReadBinaryF64(ScmPort *iport, ScmSymbol *endian)
 void Scm_WriteBinaryU8(ScmObj sval, ScmPort *oport, ScmSymbol *endian)
 {
     int val = Scm_GetIntegerU8Clamp(sval, SCM_CLAMP_NONE, NULL);
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     ENSURE_OPORT(oport);
     Scm_Putb(val, oport);
 }
@@ -202,7 +202,7 @@ void Scm_WriteBinaryU8(ScmObj sval, ScmPort *oport, ScmSymbol *endian)
 void Scm_WriteBinaryS8(ScmObj sval, ScmPort *oport, ScmSymbol *endian)
 {
     int val = Scm_GetInteger8Clamp(sval, SCM_CLAMP_NONE, NULL);
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     ENSURE_OPORT(oport);
     Scm_Putb(val, oport);
 }
@@ -211,7 +211,7 @@ void Scm_WriteBinaryU16(ScmObj sval, ScmPort *oport, ScmSymbol *endian)
 {
     swap_u16_t v;
     ENSURE_OPORT(oport);
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     v.val = Scm_GetIntegerU16Clamp(sval, SCM_CLAMP_NONE, NULL);
     SWAP_16(endian, v);
     Scm_Putz(v.buf, 2, oport);
@@ -221,7 +221,7 @@ void Scm_WriteBinaryS16(ScmObj sval, ScmPort *oport, ScmSymbol *endian)
 {
     swap_s16_t v;
     ENSURE_OPORT(oport);
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     v.val = Scm_GetInteger16Clamp(sval, SCM_CLAMP_NONE, NULL);
     SWAP_16(endian, v);
     Scm_Putz(v.buf, 2, oport);
@@ -231,7 +231,7 @@ void Scm_WriteBinaryU32(ScmObj sval, ScmPort *oport, ScmSymbol *endian)
 {
     swap_u32_t v;
     ENSURE_OPORT(oport);
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     v.val = Scm_GetIntegerU32Clamp(sval, FALSE, FALSE);
     SWAP_32(endian, v);
     Scm_Putz(v.buf, 4, oport);
@@ -241,7 +241,7 @@ void Scm_WriteBinaryS32(ScmObj sval, ScmPort *oport, ScmSymbol *endian)
 {
     swap_s32_t v;
     ENSURE_OPORT(oport);
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     v.val = Scm_GetInteger32Clamp(sval, FALSE, FALSE);
     SWAP_32(endian, v);
     Scm_Putz(v.buf, 4, oport);
@@ -251,7 +251,7 @@ void Scm_WriteBinaryU64(ScmObj sval, ScmPort *oport, ScmSymbol *endian)
 {
     swap_u64_t v;
     ENSURE_OPORT(oport);
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     v.val = Scm_GetIntegerU64Clamp(sval, FALSE, FALSE);
     SWAP_64(endian, v);
     Scm_Putz(v.buf, 8, oport);
@@ -261,7 +261,7 @@ void Scm_WriteBinaryS64(ScmObj sval, ScmPort *oport, ScmSymbol *endian)
 {
     swap_s64_t v;
     ENSURE_OPORT(oport);
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     v.val = Scm_GetInteger64Clamp(sval, FALSE, FALSE);
     SWAP_64(endian, v);
     Scm_Putz(v.buf, 8, oport);
@@ -271,7 +271,7 @@ void Scm_WriteBinaryF16(ScmObj sval, ScmPort *oport, ScmSymbol *endian)
 {
     swap_f16_t v;
     ENSURE_OPORT(oport);
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     v.val = Scm_DoubleToHalf(Scm_GetDouble(sval));
     SWAP_16(endian, v);
     Scm_Putz(v.buf, 2, oport);
@@ -281,7 +281,7 @@ void Scm_WriteBinaryF32(ScmObj sval, ScmPort *oport, ScmSymbol *endian)
 {
     swap_f32_t v;
     ENSURE_OPORT(oport);
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     v.val = (float)Scm_GetDouble(sval);
     SWAP_32(endian, v);
     Scm_Putz(v.buf, 4, oport);
@@ -291,7 +291,7 @@ void Scm_WriteBinaryF64(ScmObj sval, ScmPort *oport, ScmSymbol *endian)
 {
     swap_f64_t v;
     ENSURE_OPORT(oport);
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     v.val = Scm_GetDouble(sval);
     SWAP_D(endian, v);
     Scm_Putz(v.buf, 8, oport);
@@ -318,7 +318,7 @@ static void extract(ScmUVector *uv, char *buf, int off, int eltsize)
 ScmObj Scm_GetBinaryU8(ScmUVector *uv, int off, ScmSymbol *endian)
 {
     unsigned char b;
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     extract(uv, (char *)&b, off, 1);
     return SCM_MAKE_INT(b);
 }
@@ -326,7 +326,7 @@ ScmObj Scm_GetBinaryU8(ScmUVector *uv, int off, ScmSymbol *endian)
 ScmObj Scm_GetBinaryS8(ScmUVector *uv, int off, ScmSymbol *endian)
 {
     unsigned char b;
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     extract(uv, (char *)&b, off, 1);
     int r = b;
     if (r >= 128) r -= 256;
@@ -336,7 +336,7 @@ ScmObj Scm_GetBinaryS8(ScmUVector *uv, int off, ScmSymbol *endian)
 ScmObj Scm_GetBinaryU16(ScmUVector *uv, int off, ScmSymbol *endian)
 {
     swap_u16_t v;
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     extract(uv, v.buf, off, 2);
     SWAP_16(endian, v);
     return SCM_MAKE_INT(v.val);
@@ -345,7 +345,7 @@ ScmObj Scm_GetBinaryU16(ScmUVector *uv, int off, ScmSymbol *endian)
 ScmObj Scm_GetBinaryS16(ScmUVector *uv, int off, ScmSymbol *endian)
 {
     swap_s16_t v;
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     extract(uv, v.buf, off, 2);
     SWAP_16(endian, v);
     return SCM_MAKE_INT(v.val);
@@ -354,7 +354,7 @@ ScmObj Scm_GetBinaryS16(ScmUVector *uv, int off, ScmSymbol *endian)
 ScmObj Scm_GetBinaryU32(ScmUVector *uv, int off, ScmSymbol *endian)
 {
     swap_u32_t v;
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     extract(uv, v.buf, off, 4);
     SWAP_32(endian, v);
     return Scm_MakeIntegerFromUI(v.val);
@@ -363,7 +363,7 @@ ScmObj Scm_GetBinaryU32(ScmUVector *uv, int off, ScmSymbol *endian)
 ScmObj Scm_GetBinaryS32(ScmUVector *uv, int off, ScmSymbol *endian)
 {
     swap_s32_t v;
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     extract(uv, v.buf, off, 4);
     SWAP_32(endian, v);
     return Scm_MakeInteger(v.val);
@@ -372,7 +372,7 @@ ScmObj Scm_GetBinaryS32(ScmUVector *uv, int off, ScmSymbol *endian)
 ScmObj Scm_GetBinaryU64(ScmUVector *uv, int off, ScmSymbol *endian)
 {
     swap_u64_t v;
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     extract(uv, v.buf, off, 8);
     SWAP_64(endian, v);
     return Scm_MakeIntegerU64(v.val);
@@ -381,7 +381,7 @@ ScmObj Scm_GetBinaryU64(ScmUVector *uv, int off, ScmSymbol *endian)
 ScmObj Scm_GetBinaryS64(ScmUVector *uv, int off, ScmSymbol *endian)
 {
     swap_s64_t v;
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     extract(uv, v.buf, off, 8);
     SWAP_64(endian, v);
     return Scm_MakeInteger64(v.val);
@@ -390,7 +390,7 @@ ScmObj Scm_GetBinaryS64(ScmUVector *uv, int off, ScmSymbol *endian)
 ScmObj Scm_GetBinaryF16(ScmUVector *uv, int off, ScmSymbol *endian)
 {
     swap_f16_t v;
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     extract(uv, v.buf, off, 2);
     SWAP_16(endian, v);
     return Scm_MakeFlonum(Scm_HalfToDouble(v.val));
@@ -399,7 +399,7 @@ ScmObj Scm_GetBinaryF16(ScmUVector *uv, int off, ScmSymbol *endian)
 ScmObj Scm_GetBinaryF32(ScmUVector *uv, int off, ScmSymbol *endian)
 {
     swap_f32_t v;
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     extract(uv, v.buf, off, 4);
     SWAP_32(endian, v);
     return Scm_MakeFlonum((double)v.val);
@@ -408,7 +408,7 @@ ScmObj Scm_GetBinaryF32(ScmUVector *uv, int off, ScmSymbol *endian)
 ScmObj Scm_GetBinaryF64(ScmUVector *uv, int off, ScmSymbol *endian)
 {
     swap_f64_t v;
-    CHECK_ENDIAN(endian);
+    SCM_CHECK_ENDIAN(endian);
     extract(uv, v.buf, off, 8);
     SWAP_D(endian, v);
     return Scm_MakeFlonum(v.val);
@@ -436,21 +436,21 @@ static void inject(ScmUVector *uv, char *buf, int off, int eltsize)
 void Scm_PutBinaryU8(ScmUVector *uv, int off, ScmObj val, ScmSymbol *e)
 {
     u_char v = (u_char)Scm_GetIntegerU8Clamp(val, SCM_CLAMP_NONE, NULL);
-    CHECK_ENDIAN(e);
+    SCM_CHECK_ENDIAN(e);
     inject(uv, (char *)&v, off, 1);
 }
 
 void Scm_PutBinaryS8(ScmUVector *uv, int off, ScmObj val, ScmSymbol *e)
 {
     u_char v = (u_char)Scm_GetInteger8Clamp(val, SCM_CLAMP_NONE, NULL);
-    CHECK_ENDIAN(e);
+    SCM_CHECK_ENDIAN(e);
     inject(uv, (char *)&v, off, 1);
 }
 
 void Scm_PutBinaryU16(ScmUVector *uv, int off, ScmObj val, ScmSymbol *e)
 {
     swap_u16_t v;
-    CHECK_ENDIAN(e);
+    SCM_CHECK_ENDIAN(e);
     v.val = Scm_GetIntegerU16Clamp(val, SCM_CLAMP_NONE, NULL);
     SWAP_16(e, v);
     inject(uv, v.buf, off, 2);
@@ -459,7 +459,7 @@ void Scm_PutBinaryU16(ScmUVector *uv, int off, ScmObj val, ScmSymbol *e)
 void Scm_PutBinaryS16(ScmUVector *uv, int off, ScmObj val, ScmSymbol *e)
 {
     swap_s16_t v;
-    CHECK_ENDIAN(e);
+    SCM_CHECK_ENDIAN(e);
     v.val = Scm_GetInteger16Clamp(val, SCM_CLAMP_NONE, NULL);
     SWAP_16(e, v);
     inject(uv, v.buf, off, 2);
@@ -468,7 +468,7 @@ void Scm_PutBinaryS16(ScmUVector *uv, int off, ScmObj val, ScmSymbol *e)
 void Scm_PutBinaryU32(ScmUVector *uv, int off, ScmObj val, ScmSymbol *e)
 {
     swap_u32_t v;
-    CHECK_ENDIAN(e);
+    SCM_CHECK_ENDIAN(e);
     v.val = Scm_GetIntegerU32Clamp(val, FALSE, FALSE);
     SWAP_32(e, v);
     inject(uv, v.buf, off, 4);
@@ -477,7 +477,7 @@ void Scm_PutBinaryU32(ScmUVector *uv, int off, ScmObj val, ScmSymbol *e)
 void Scm_PutBinaryS32(ScmUVector *uv, int off, ScmObj val, ScmSymbol *e)
 {
     swap_s32_t v;
-    CHECK_ENDIAN(e);
+    SCM_CHECK_ENDIAN(e);
     v.val = Scm_GetInteger32Clamp(val, FALSE, FALSE);
     SWAP_32(e, v);
     inject(uv, v.buf, off, 4);
@@ -486,7 +486,7 @@ void Scm_PutBinaryS32(ScmUVector *uv, int off, ScmObj val, ScmSymbol *e)
 void Scm_PutBinaryU64(ScmUVector *uv, int off, ScmObj val, ScmSymbol *e)
 {
     swap_u64_t v;
-    CHECK_ENDIAN(e);
+    SCM_CHECK_ENDIAN(e);
     v.val = Scm_GetIntegerU64Clamp(val, FALSE, FALSE);
     SWAP_64(e, v);
     inject(uv, v.buf, off, 8);
@@ -495,7 +495,7 @@ void Scm_PutBinaryU64(ScmUVector *uv, int off, ScmObj val, ScmSymbol *e)
 void Scm_PutBinaryS64(ScmUVector *uv, int off, ScmObj val, ScmSymbol *e)
 {
     swap_s64_t v;
-    CHECK_ENDIAN(e);
+    SCM_CHECK_ENDIAN(e);
     v.val = Scm_GetInteger64Clamp(val, FALSE, FALSE);
     SWAP_64(e, v);
     inject(uv, v.buf, off, 8);
@@ -504,7 +504,7 @@ void Scm_PutBinaryS64(ScmUVector *uv, int off, ScmObj val, ScmSymbol *e)
 void Scm_PutBinaryF16(ScmUVector *uv, int off, ScmObj val, ScmSymbol *e)
 {
     swap_f16_t v;
-    CHECK_ENDIAN(e);
+    SCM_CHECK_ENDIAN(e);
     v.val = Scm_DoubleToHalf(Scm_GetDouble(val));
     SWAP_16(e, v);
     inject(uv, v.buf, off, 2);
@@ -513,7 +513,7 @@ void Scm_PutBinaryF16(ScmUVector *uv, int off, ScmObj val, ScmSymbol *e)
 void Scm_PutBinaryF32(ScmUVector *uv, int off, ScmObj val, ScmSymbol *e)
 {
     swap_f32_t v;
-    CHECK_ENDIAN(e);
+    SCM_CHECK_ENDIAN(e);
     v.val = (float)Scm_GetDouble(val);
     SWAP_32(e, v);
     inject(uv, v.buf, off, 4);
@@ -522,7 +522,7 @@ void Scm_PutBinaryF32(ScmUVector *uv, int off, ScmObj val, ScmSymbol *e)
 void Scm_PutBinaryF64(ScmUVector *uv, int off, ScmObj val, ScmSymbol *e)
 {
     swap_f64_t v;
-    CHECK_ENDIAN(e);
+    SCM_CHECK_ENDIAN(e);
     v.val = Scm_GetDouble(val);
     SWAP_D(e, v);
     inject(uv, v.buf, off, 8);
