@@ -5,7 +5,6 @@
 (use gauche.test)
 (use gauche.uvector)
 (use gauche.collection)
-(use gauche.parameter)
 (use srfi-1)
 
 (test-start "binary")
@@ -728,7 +727,7 @@
       (with-error-handler
           (lambda (err)
             (and (cond ((regexp? expected)
-                        (rxmatch expected (slot-ref err 'message)))
+                        (rxmatch expected (condition-message err "")))
                        ((procedure? expected)
                         (expected err))
                        (else
