@@ -1,7 +1,7 @@
 ;;;
 ;;; convaux - auxiliary charconv routines
 ;;;
-;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -113,6 +113,34 @@
 
     (values ces-equivalent? ces-upper-compatible?)))
 
+;; Returns a list of octets for replacement characters.
+;; This is called from jconv_set_replacement() internal C function.
+(define (%ces-replacement ces)
+  (assume-type ces <string>)
+  (cond
+   ;; For unicode, we use U+FFFD Replacement Character.
+   [(ces-equivalent? "utf8" ces)  '(#xef #xbf #xbd)]
+   ;; For jis-family, we use U+3013 Geta Mark (JIS 020E)
+   ;; NB: iso-2022-jp is generated via eucjp, so it is suffice to give
+   ;; the replacement char in eucjp.
+   [(ces-equivalent? "eucjp" ces) '(#xa2 #xae)]
+   [(ces-equivalent? "sjis" ces) '(#x81 #xac)]
+   [(ces-equivalent? "iso2022jp" ces) '(#xa2 #xae)]
+   ;; 1-byte encoding, we use '?'
+   [(ces-equivalent? "ascii" ces) '(#x3f)]
+   [(ces-equivalent? "iso88591" ces) '(#x3f)]
+   [(ces-equivalent? "iso88592" ces) '(#x3f)]
+   [(ces-equivalent? "iso88593" ces) '(#x3f)]
+   [(ces-equivalent? "iso88594" ces) '(#x3f)]
+   [(ces-equivalent? "iso88595" ces) '(#x3f)]
+   [(ces-equivalent? "iso88596" ces) '(#x3f)]
+   [(ces-equivalent? "iso88597" ces) '(#x3f)]
+   [(ces-equivalent? "iso88598" ces) '(#x3f)]
+   [(ces-equivalent? "iso88599" ces) '(#x3f)]
+   ;; we should detect some more unicode family to pass U+FFFD, but
+   ;; for now, we return '?'.
+   [else '(#x3f)]))
+
 ;; returns appropriate input port and size
 (define (%ces-input input)
   (cond [(string? input)
@@ -132,19 +160,24 @@
    [else (error "Only <string> or <u8vector> is supported, but got:" class)]))
 
 ;; Convert string or uvector -> string or uvector
-(define (ces-convert-to class input fromcode :optional (tocode #f))
+(define (ces-convert-to class input fromcode
+                        :optional (tocode #f) 
+                        :key illegal-output)
   (receive (inp isize) (%ces-input input)
     (receive (out get) (%ces-output class)
       (let1 in ($ open-input-conversion-port inp fromcode
-                  :to-code tocode :buffer-size isize :owner? #t)
+                  :to-code tocode :buffer-size isize 
+                  :owner? #t :illegal-output illegal-output)
         (copy-port in out :unit 'byte)
         (close-input-port in)
         (flush out)
         (begin0 (get out)
           (close-output-port out))))))
 
-(define (ces-convert input fromcode :optional (tocode #f))
-  (ces-convert-to <string> input fromcode tocode))
+(define (ces-convert input fromcode 
+                     :optional (tocode #f)
+                     :key illegal-output)
+  (ces-convert-to <string> input fromcode tocode :illegal-output illegal-output))
 
 ;; "Wrap" the given port for convering to/from native encoding if needed.
 ;; Unlike open-*-conversion-port, these return port itself if the conversion
@@ -169,22 +202,26 @@
 (define (call-with-input-conversion port proc
                                     :key ((:encoding from-code)
                                           (gauche-character-encoding))
-                                         ((:conversion-buffer-size bufsiz) 0))
+                                         ((:conversion-buffer-size bufsiz) 0)
+                                         illegal-output)
   (if (ces-upper-compatible? (gauche-character-encoding) from-code)
     (proc port)
     (let1 cvp (open-input-conversion-port port from-code
-                                          :owner? #f :buffer-size bufsiz)
+                                          :owner? #f :buffer-size bufsiz
+                                          :illegal-output illegal-output)
       (unwind-protect (proc cvp)
         (close-input-port cvp)))))
 
 (define (call-with-output-conversion port proc
                                      :key ((:encoding to-code)
                                            (gauche-character-encoding))
-                                          ((:conversion-buffer-size bufsiz) 0))
+                                          ((:conversion-buffer-size bufsiz) 0)
+                                          illegal-output)
   (if (ces-upper-compatible? (gauche-character-encoding) to-code)
     (proc port)
     (let1 cvp (open-output-conversion-port port to-code
-                                           :owner? #f :buffer-size bufsiz)
+                                           :owner? #f :buffer-size bufsiz
+                                           :illegal-output illegal-output)
       (unwind-protect (proc cvp)
         (close-output-port cvp)))))
 
@@ -199,59 +236,84 @@
          opts))
 
 ;; Inserts conversion port.  These are called from system's
-;; open-{input|output}-port when :encoding argument is given.
+;; open-{input|output}-file when :encoding argument is given.
 (define (%open-input-file/conv name :key (encoding #f)
                                          ((:conversion-buffer-size bufsiz) 0)
+                                         ((:conversion-illegal-output illegal-output) 'raise)
                                     :allow-other-keys rest)
   (and-let* ([port (apply (with-module gauche.internal %open-input-file)
                           name rest)])
     (wrap-with-input-conversion port encoding
-                                :buffer-size bufsiz :owner? #t)))
+                                :buffer-size bufsiz
+                                :owner? #t
+                                :illegal-output illegal-output)))
 
 (define (%open-output-file/conv name :key (encoding #f)
                                           ((:conversion-buffer-size bufsiz) 0)
+                                          ((:conversion-illegal-output illegal-output) 'raise)
                                      :allow-other-keys rest)
   (and-let* ([port (apply (with-module gauche.internal %open-output-file)
                           name rest)])
     (wrap-with-output-conversion port encoding
-                                 :buffer-size bufsiz :owner? #t)))
+                                 :buffer-size bufsiz :owner? #t
+                                 :illegal-output illegal-output)))
 
 ;;
 ;; Low-level API
 ;;
 
 (inline-stub
- "#include \"charconv.h\""
+ (declcode
+  (.include "charconv.h"))
 
- (define-cproc ces-conversion-supported? (from to) ::<boolean>
+ (define-cproc ces-conversion-supported? (from to)
+   ::<boolean>
    (let* ([cfrom::(const char*) (Scm_GetCESName from "from-code")]
-          [cto  ::(const char*) (Scm_GetCESName to "to-code")])
-     (return (Scm_ConversionSupportedP cfrom cto))))
+          [cto  ::(const char*) (Scm_GetCESName to "to-code")]
+          [flags::u_long 0])
+     (return (Scm_ConversionSupportedP cfrom cto flags))))
 
- ;; NB: :handler interface is experimental.  Do not use it.
  (define-cproc open-input-conversion-port (source::<input-port>
                                            from-code
                                            :key (to-code #f)
                                                 (buffer-size::<fixnum> 0)
                                                 (owner? #f)
-                                                (handler #f))
+                                                illegal-output)
    (let* ([fc::(const char*) (Scm_GetCESName from_code "from-code")]
-          [tc::(const char*) (Scm_GetCESName to_code "to-code")])
-     (return (Scm_MakeInputConversionPort source fc tc handler buffer_size
-                                          (not (SCM_FALSEP ownerP))))))
+          [tc::(const char*) (Scm_GetCESName to_code "to-code")]
+          [flags::u_long 0])
+     (unless (SCM_FALSEP ownerP)
+       (logior= flags CVPORT_OWNER))
+     (cond [(SCM_EQ illegal-output 'replace) (logior= flags CVPORT_REPLACE)]
+           [(or (SCM_UNBOUNDP illegal-output)
+                (SCM_UNDEFINEDP illegal-output)
+                (SCM_EQ illegal-output 'raise))]
+           [else (Scm_Error ":illegal-output argument must be either raise or \
+                             replace, but got: %S" illegal-output)])
+     (return (Scm_MakeInputConversionPort source fc tc buffer_size
+                                          flags))))
 
  (define-cproc open-output-conversion-port (sink::<output-port>
                                             to-code
                                             :key (from-code #f)
                                                  (buffer-size::<fixnum> 0)
-                                                 (owner? #f))
+                                                 (owner? #f)
+                                                 illegal-output)
    (let* ([fc::(const char*) (Scm_GetCESName from_code "from-code")]
-          [tc::(const char*) (Scm_GetCESName to_code "to-code")])
-     (return (Scm_MakeOutputConversionPort sink tc fc buffer_size
-                                           (not (SCM_FALSEP ownerP))))))
+          [tc::(const char*) (Scm_GetCESName to_code "to-code")]
+          [flags::u_long 0])
+     (unless (SCM_FALSEP ownerP)
+       (logior= flags CVPORT_OWNER))
+     (cond [(SCM_EQ illegal-output 'replace) (logior= flags CVPORT_REPLACE)]
+           [(or (SCM_UNBOUNDP illegal-output)
+                (SCM_UNDEFINEDP illegal-output)
+                (SCM_EQ illegal-output 'raise))]
+           [else (Scm_Error ":illegal-output argument must be either raise or \
+                             replace, but got: %S" illegal-output)])
+     (return (Scm_MakeOutputConversionPort sink tc fc buffer_size flags))))
 
  (define-cproc ces-guess-from-string (string::<string> scheme::<string>)
-   (let* ([size::u_int]
+   (let* ([size::ScmSmallInt]
           [s::(const char*) (Scm_GetStringContent string (& size) NULL NULL)]
           [guessed::(const char*)
                     (Scm_GuessCES (Scm_GetStringConst scheme) s size)])
