@@ -12,19 +12,20 @@
   (string-complete->incomplete
    (call-with-input-file file port->byte-string)))
 
-(define (file->string-conv/in file from . to)
+(define (file->string-conv/in file from . args)
   (string-complete->incomplete
    (call-with-input-file file
      (lambda (f)
-       (port->byte-string  (apply open-input-conversion-port f from to))))))
+       (port->byte-string  (apply open-input-conversion-port f from args))))))
 
-(define (file->string-conv/out file to from reader writer)
+(define (file->string-conv/out file to from reader writer . args)
   (string-complete->incomplete
    (call-with-output-string
      (lambda (out)
        (call-with-input-file file
          (lambda (in)
-           (let ((cv (open-output-conversion-port out to :from-code from)))
+           (let ((cv (apply open-output-conversion-port out to
+                            :from-code from args)))
              (let loop ((data (reader in)))
                (if (eof-object? data)
                    (close-output-port cv)
@@ -71,27 +72,29 @@
 ;;--------------------------------------------------------------------
 (test-section "input conversion")
 
-(define (test-input file from to . guesser)
-  (let* ((realfrom (if (null? guesser) from (car guesser)))
+(define (test-input file from to :key (guesser #f) illegal-output)
+  (let* ((realfrom (or guesser from))
          (infostr  (format #f "~a.~a (~a) => ~a" file from realfrom to))
          (fromfile (format #f "~a.~a" file from))
          (tofile   (format #f "~a.~a" file to)))
     (if (ces-conversion-supported? from to)
-        (if (supported-character-encoding? to)
-            (test infostr
-                  (file->string tofile)
-                  (lambda () (file->string-conv/in fromfile realfrom)))
-            (test infostr
-                  (file->string tofile)
-                  (lambda () (file->string-conv/in fromfile realfrom
-                                                   :to-code to))))
-        (test infostr "(not supported)"
-              (lambda () "(not supported)")))
+      (if (supported-character-encoding? to)
+        (test infostr
+              (file->string tofile)
+              (lambda () (file->string-conv/in fromfile realfrom
+                                               :illegal-output illegal-output)))
+        (test infostr
+              (file->string tofile)
+              (lambda () (file->string-conv/in fromfile realfrom
+                                               :to-code to
+                                               :illegal-output illegal-output))))
+      (test infostr "(not supported)"
+            (lambda () "(not supported)")))
     ))
 
 (map-test test-input "data/jp1"
-          '("EUCJP" "UTF-8" "SJIS" "ISO2022JP")
-          '("EUCJP" "UTF-8" "SJIS" "ISO2022JP"))
+          '("EUCJP" "UTF-8" "SJIS" "ISO2022JP" "UTF-16" "UTF-16BE" "UTF-16LE")
+          '("EUCJP" "UTF-8" "SJIS" "ISO2022JP" "UTF-16BE" "UTF-16LE"))
 (map-test test-input "data/jp2"
           '("EUCJP" "UTF-8" "SJIS" "ISO2022JP")
           '("EUCJP" "UTF-8" "SJIS" "ISO2022JP"))
@@ -105,23 +108,63 @@
           '("EUCKR" "UTF-8" "ISO2022KR")
           '("EUCKR" "UTF-8" "ISO2022KR"))
 
+;; illegal-output replace
+(map-test (lambda (file from to)
+            (test-input file from to :illegal-output 'replace))
+          "data/jp1"
+          '("ASCII" "EUCJP" "UTF-8" "SJIS" "ISO2022JP")
+          '("ASCII"))
+(map-test (lambda (file from to)
+            (test-input file from to :illegal-output 'replace))
+          "data/jp2"
+          '("EUCJP" "UTF-8" "SJIS" "ISO2022JP")
+          '("ASCII"))
+(map-test (lambda (file from to)
+            (test-input file from to :illegal-output 'replace))
+          "data/lat1"
+          '("ISO8859-1")
+          '("ASCII" "ISO8859-1" "EUCJP" "SJIS" "ISO2022JP" "UTF-8"
+            "UTF-16" "UTF-16BE" "UTF-16LE" 
+            "UTF-32" "UTF-32BE" "UTF-32LE"))
+(map-test (lambda (file from to)
+            (test-input file from to :illegal-output 'replace))
+          "data/lat1x"
+          '("ISO8859-1" "EUCJP" "UTF-8" "SJIS" 
+            "UTF-16" "UTF-16BE" "UTF-16LE"
+            "UTF-32" "UTF-32BE" "UTF-32LE")
+          '("ISO8859-1"))
+(dolist [n '(2 3 4 5 6 7 8 9 10 11 13 14 15 16)]
+  (map-test test-input #"data/lat~|n|"
+            `(,#"ISO8859-~|n|" "UTF-16BE")
+            `(,#"ISO8859-~|n|" "UTF-16BE")))
+
 ;; autodetect tester
 (map-test (lambda (file from to)
-            (test-input file from to "*JP"))
+            (test-input file from to :guesser "*JP"))
           "data/jp1"
           '("EUCJP" "UTF-8" "SJIS" "ISO2022JP")
           '("EUCJP" "UTF-8" "SJIS" "ISO2022JP"))
 (map-test (lambda (file from to)
-            (test-input file from to "*JP"))
+            (test-input file from to :guesser "*JP"))
           "data/jp2"
           '("EUCJP" "UTF-8" "SJIS" "ISO2022JP")
           '("EUCJP" "UTF-8" "SJIS" "ISO2022JP"))
 (map-test (lambda (file from to)
-            (test-input file from to "*JP"))
+            (test-input file from to :guesser "*JP"))
           "data/jp3"
           '("EUCJP" "UTF-8" "SJIS" "ISO2022JP")
           '("EUCJP" "UTF-8" "SJIS" "ISO2022JP"))
 
+;; UTF BOM illegal-output
+(let ()
+  (define (f n) #"~(sys-dirname (current-load-path))/~n")
+
+  (define (bom-test bom-src)
+    (equal? (call-with-input-file (f "data/jp1.UTF-8") port->string)
+            (call-with-input-file (f bom-src) port->string)))
+  (bom-test "data/jp1.UTF-16-BOMBE")
+  (bom-test "data/jp1.UTF-16-BOMLE"))
+
 ;;--------------------------------------------------------------------
 (test-section "output conversion")
 
@@ -158,8 +201,8 @@
     (else '())))
 
 (map-test test-output/byte "data/jp1"
-          '("EUCJP" "UTF-8" "SJIS" "ISO2022JP")
-          '("EUCJP" "UTF-8" "SJIS" "ISO2022JP"))
+          '("EUCJP" "UTF-8" "SJIS" "ISO2022JP" "UTF-16" "UTF-16BE" "UTF-16LE")
+          '("EUCJP" "UTF-8" "SJIS" "ISO2022JP" "UTF-16BE" "UTF-16LE"))
 (map-test test-output/chunk256 "data/jp1"
           '("EUCJP" "UTF-8" "SJIS" "ISO2022JP")
           '("EUCJP" "UTF-8" "SJIS" "ISO2022JP"))
@@ -219,6 +262,11 @@
           '("EUCKR" "UTF-8" "ISO2022KR")
           '("EUCKR" "UTF-8" "ISO2022KR"))
 
+(dolist [n '(1 2 3 4 5 6 7 8 9 10 11 13 14 15 16)]
+  (map-test test-output/chunk20 #"data/lat~|n|"
+            `(,#"ISO8859-~|n|" "UTF-16BE")
+            `(,#"ISO8859-~|n|" "UTF-16BE")))
+
 ;;--------------------------------------------------------------------
 (test-section "open -> #f")
 
@@ -286,6 +334,29 @@
           '("EUCJP" "UTF-8" "SJIS" "ISO2022JP")
           '("EUCJP" "UTF-8" "SJIS" "ISO2022JP"))
 
+;;-------------------------------------------------------------------
+(test-section "replacement illegal-output")
+(test* "utf-8 -> ascii noreplacement" (test-error <io-decoding-error>)
+       (ces-convert #u8(#x61 #xe3 #x81 #x82 #x62 #xe3 #x81
+                       #x84 #x63 #xe3 #x81 #x86)
+                    'utf-8 'ascii))
+(test* "utf-8 -> ascii replacement" "a?b?c?"
+       (ces-convert #u8(#x61 #xe3 #x81 #x82 #x62 #xe3 #x81
+                        #x84 #x63 #xe3 #x81 #x86)
+                    'utf-8 'ascii :illegal-output 'replace))
+
+(let ([src #u8(#x61 #xe2 #x98 #xba #x62 #xe2 #x9b #xb1 #x63)])
+  ;; src is #\a + snowman + #\b + umbrella + #\c
+  (test* "utf-8 -> eucjp noreplacement" (test-error <io-decoding-error>)
+       (ces-convert-to <u8vector> src 'utf-8 'eucjp))
+  (test* "utf-8 -> eucjp replacement" '#u8(#x61 #xa2 #xae #x62 #xa2 #xae #x63)
+       (ces-convert-to <u8vector> src 'utf-8 'eucjp :illegal-output 'replace))
+  (test* "utf-8 -> sjis noreplacement" (test-error <io-decoding-error>)
+       (ces-convert-to <u8vector> src 'utf-8 'sjis))
+  (test* "utf-8 -> sjis replacement" '#u8(#x61 #x81 #xac #x62 #x81 #xac #x63)
+       (ces-convert-to <u8vector> src 'utf-8 'sjis :illegal-output 'replace))
+  )
+
 ;;--------------------------------------------------------------------
 (test-section "wrapping conversion")
 
