@@ -1,7 +1,7 @@
 ;;;
 ;;; data.queue - queue (fifo) implementation
 ;;;
-;;;   Copyright (c) 2010-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2010-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -67,30 +67,31 @@
 ;;;  Data structures
 ;;;
 (inline-stub
- "#include <gauche/class.h>"
+ (declcode
+  (.include <gauche/class.h>))
 
  ;;
  ;; <queue>
  ;;
- "typedef struct QueueRec {"
- "  SCM_INSTANCE_HEADER;"
- "  long len;"     ;; lazily calc'd.  -1 for 'to be calculated'
- "  ScmObj head;"
- "  ScmObj tail;"
- "} Queue;"
+ (define-ctype Queue::(.struct
+                       (SCM_INSTANCE_HEADER :: ""
+                        len::ScmSize ;; lazily calc'd.  -1 for 'to be calculated'
+                        head
+                        tail)))
 
  "SCM_CLASS_DECL(QueueClass);"
- "#define QP(obj)          SCM_ISA(obj, &QueueClass)"
- "#define Q(obj)           ((Queue*)(obj))"
- "#define Q_HEAD(obj)      (Q(obj)->head)"
- "#define Q_TAIL(obj)      (Q(obj)->tail)"
- "#define Q_LENGTH(obj)    (Q(obj)->len)"  ; can be -1; should use %qlength().
- "#define Q_EMPTY_P(obj)   (SCM_NULLP(Q_HEAD(obj)))"
 
- (define-cfn %qlength (q::Queue*) ::u_long  ; must be called with lock held
+ (.define QP (obj) (SCM_ISA obj (& QueueClass)))
+ (.define Q (obj) (cast Queue* obj))
+ (.define Q_HEAD (obj) (-> (Q obj) head))
+ (.define Q_TAIL (obj) (-> (Q obj) tail))
+ (.define Q_LENGTH (obj) (-> (Q obj) len)) ; can be -1; should use %qlength().
+ (.define Q_EMPTY_P (obj) (SCM_NULLP (Q_HEAD obj)))
+
+ (define-cfn %qlength (q::Queue*) ::ScmSize  ; must be called with lock held
    (when (< (Q_LENGTH q) 0)
      (set! (Q_LENGTH q) (Scm_Length (Q_HEAD q))))
-   (return (cast u_long (Q_LENGTH q))))
+   (return (cast ScmSmallInt (Q_LENGTH q))))
 
  (define-cfn makeq (klass::ScmClass*)
    (let* ([z::Queue* (SCM_NEW_INSTANCE Queue klass)])
@@ -101,35 +102,34 @@
  (define-cclass <queue>
    "Queue*" "QueueClass" ()
    ((length :type <uint> :c-name "len" :setter #f))
-   (allocator (return (makeq klass)))
+   (allocator (begin (cast void initargs) (return (makeq klass))))
    (printer (Scm_Printf port "#<queue %d @%p>" (%qlength (Q obj)) obj)))
 
  ;;
  ;; <mtqueue>
  ;;
- "typedef struct MtQueueReq {"
- "  Queue q;"
- "  int maxlen;"     ;negative if unlimited
- "  ScmInternalMutex mutex;"
- "  ScmObj locker;"  ;thread holding the lock.  see the comment above.
- "  ScmInternalCond lockWait;"
- "  ScmInternalCond readerWait;"
- "  ScmInternalCond writerWait;"
- "  int readerSem;" ;used by zero-length queue.  # of waiting reader
- "} MtQueue;"
+ (define-ctype MtQueue::(.struct
+                         (q::Queue
+                          maxlen::ScmSmallInt ; negative if unlimited
+                          mutex::ScmInternalMutex
+                          locker ; thread holding the lock.  see the comment above.
+                          lockWait::ScmInternalCond
+                          readerWait::ScmInternalCond
+                          writerWait::ScmInternalCond
+                          readerSem::int ; used by zero-length queue.  # of waiting reader
+                          )))
 
  "SCM_CLASS_DECL(MtQueueClass);"
- "#define MTQP(obj)       SCM_ISA(obj, &MtQueueClass)"
- "#define MTQ(obj)        ((MtQueue*)(obj))"
- "#define MTQ_MAXLEN(obj) (MTQ(obj)->maxlen)"
- "#define MTQ_MUTEX(obj)  (MTQ(obj)->mutex)"
- "#define MTQ_LOCKER(obj) (MTQ(obj)->locker)"
-
- "#define MTQ_LOCK(q)   (SCM_INTERNAL_MUTEX_LOCK (MTQ_MUTEX q))"
- "#define MTQ_UNLOCK(q) (SCM_INTERNAL_MUTEX_UNLOCK (MTQ_MUTEX q))"
- "#define MTQ_CV(q, kind) (MTQ(q)->kind)"
- "#define MTQ_READER_SEM(q) (MTQ(q)->readerSem)"
 
+ (.define MTQP (obj) (SCM_ISA obj (& MtQueueClass)))
+ (.define MTQ (obj) (cast MtQueue* obj))
+ (.define MTQ_MAXLEN (obj) (-> (MTQ obj) maxlen))
+ (.define MTQ_MUTEX (obj) (-> (MTQ obj) mutex))
+ (.define MTQ_LOCKER (obj) (-> (MTQ obj) locker))
+ (.define MTQ_LOCK (obj) (SCM_INTERNAL_MUTEX_LOCK "MTQ_MUTEX q"))
+ (.define MTQ_UNLOCK (obj) (SCM_INTERNAL_MUTEX_UNLOCK "MTQ_MUTEX q"))
+ (.define MTQ_CV (q kind) (-> (MTQ q) kind))
+ (.define MTQ_READER_SEM (q) (-> (MTQ q) readerSem))
 
  (define-cfn makemtq (klass::ScmClass* maxlen::int)
    (let* ([z::MtQueue* (SCM_NEW_INSTANCE MtQueue klass)])
@@ -144,7 +144,7 @@
      (return (SCM_OBJ z))))
 
  (define-cfn mtq-maxlen-get (mtq::MtQueue*)
-   (let* ([ml::int (MTQ_MAXLEN mtq)])
+   (let* ([ml::ScmSmallInt (MTQ_MAXLEN mtq)])
      (if (< ml 0) (return '#f) (return (SCM_MAKE_INT ml)))))
 
  (define-cfn mtq-maxlen-set (mtq::MtQueue* maxlen) ::void
@@ -197,8 +197,9 @@
  (define-cise-stmt notify-readers
    [(_ q) `(SCM_INTERNAL_COND_BROADCAST (MTQ_CV ,q readerWait))])
 
- "#define CW_TIMEDOUT 1"
- "#define CW_INTR     2"
+ (.define CW_TIMEDOUT 1)
+ (.define CW_INTR 2)
+
  ;; (wait-cv Q SLOT PTIMESPEC STATUS)
  ;;   Wait on Q's condition variable in SLOT.  PTIMESPEC is a pointer
  ;;   to ScmTimeSpec or NULL, specifies timeout.  Must be called
@@ -229,8 +230,10 @@
  ;;   TIMEOUT-VAL.  (DO-OK is supposed to set RETVAL in it).
  (define-cise-stmt do-with-timeout
    [(_ q retval timeout timeout-val cv-slot init wait-check do-ok)
-    (let ([ts (gensym)] [pts (gensym)] [status (gensym)] [inited (gensym)])
+    (let ([ts (gensym)] [pts (gensym)] [tv (gensym)]
+          [status (gensym)] [inited (gensym)])
       `(let* ([,ts :: (ScmTimeSpec)] [,status :: int 0]
+              [,tv :: (volatile ScmObj) ,timeout-val]
               [,pts :: (ScmTimeSpec*) (Scm_GetTimeSpec ,timeout (& ,ts))]
               [,inited :: int FALSE])
          (while TRUE
@@ -245,7 +248,7 @@
                (notify-lockers ,q)
                (break)))
            (case ,status
-             [(CW_TIMEDOUT) (set! ,retval ,timeout-val)]
+             [(CW_TIMEDOUT) (set! ,retval ,tv)]
              [(CW_INTR)     (Scm_SigCheck (Scm_VM)) (continue)]) ;restart op
            (break))))])
 
@@ -334,7 +337,7 @@
 
  ;; API
  (define-cproc mtqueue-room (q::<mtqueue>) ::<number>
-   (let* ([room::int -1])
+   (let* ([room::ScmSmallInt -1])
      (with-mtq-light-lock q
        (when (>= (MTQ_MAXLEN q) 0)
          (set! room (- (MTQ_MAXLEN q) (%qlength (Q q))))))
@@ -354,13 +357,15 @@
    (return TRUE))
 
  (define-cproc %queue-peek (q::<queue> :optional fallback) ::(<top> <top>)
-   (let* ([ok::int FALSE] [h] [t])
+   (let* ([ok::int FALSE] [fb::(volatile ScmObj) fallback] [h] [t])
      (if (not (MTQP q))
        (set! ok (queue-peek-both-int q (& h) (& t)))
        (with-mtq-light-lock q (set! ok (queue-peek-both-int q (& h) (& t)))))
      (cond [ok (return h t)]
-           [(SCM_UNBOUNDP fallback) (Scm_Error "queue is empty: %S" q)]
-           [else (return fallback fallback)])))
+           [(SCM_UNBOUNDP fb)
+            (Scm_Error "queue is empty: %S" q)
+            (return SCM_UNDEFINED SCM_UNDEFINED)] ;dummy
+           [else (return fb fb)])))
  )
 
 ;; APIs
@@ -392,7 +397,7 @@
 
 (inline-stub
  ;; internal enqueue - lock must be held.
- (define-cfn enqueue_int (q::Queue* cnt::u_int head tail) ::void
+ (define-cfn enqueue_int (q::Queue* cnt::ScmSmallInt head tail) ::void
    (when (>= (Q_LENGTH q) 0)
      (set! (Q_LENGTH q) (+ (Q_LENGTH q) cnt)))
    (cond [(Q_EMPTY_P q) (set! (Q_HEAD q) head (Q_TAIL q) tail)]
@@ -400,7 +405,7 @@
                         (set! (Q_TAIL q) tail)]))
 
  ;; to call internal enqueue from Scheme.  lock must be held.
- (define-cproc %enqueue! (q::<queue> cnt::<uint> head tail) ::<void>
+ (define-cproc %enqueue! (q::<queue> cnt::<fixnum> head tail) ::<void>
    (enqueue_int q cnt head tail))
 
  ;; (q-write-op OP Q CNT HEAD TAIL)
@@ -420,7 +425,7 @@
 
  ;; API
  (define-cproc enqueue! (q::<queue> obj :rest more-objs)
-   (let* ([head (Scm_Cons obj more-objs)] [tail] [cnt::u_int])
+   (let* ([head (Scm_Cons obj more-objs)] [tail] [cnt::ScmSmallInt])
      (if (SCM_NULLP more-objs)
        (set! tail head cnt 1)
        (set! tail (Scm_LastPair more-objs) cnt (Scm_Length head)))
@@ -431,7 +436,7 @@
  (define-cproc enqueue/wait! (q::<mtqueue> obj :optional (timeout #f)
                                                          (timeout-val #f))
    (let* ([cell (SCM_LIST1 obj)] [retval (SCM_OBJ q)])
-     (.if "defined(GAUCHE_HAS_THREADS)"
+     (.if (defined GAUCHE_HAS_THREADS)
           (do-with-timeout q retval timeout timeout-val writerWait
                            (begin)
                            (?: (!= (MTQ_MAXLEN q) 0)
@@ -462,7 +467,7 @@
 
 (inline-stub
  ;; queue-push! - add item(s) to the head
- (define-cfn queue-push-int (q::Queue* cnt::u_int head tail) ::void
+ (define-cfn queue-push-int (q::Queue* cnt::ScmSmallInt head tail) ::void
    (SCM_SET_CDR tail (Q_HEAD q))
    (set! (Q_HEAD q) head
          (Q_TAIL q) (Scm_LastPair tail))
@@ -470,7 +475,7 @@
      (set! (Q_LENGTH q) (+ (Q_LENGTH q) cnt))))
 
  (define-cproc queue-push! (q::<queue> obj :rest more-objs)
-   (let* ([objs (Scm_Cons obj more-objs)] [head] [tail] [cnt::u_int])
+   (let* ([objs (Scm_Cons obj more-objs)] [head] [tail] [cnt::ScmSmallInt])
      (if (SCM_NULLP more-objs)
        (set! head objs tail objs cnt 1)
        (set! head (Scm_ReverseX objs)
@@ -482,7 +487,7 @@
  (define-cproc queue-push/wait! (q::<mtqueue> obj :optional (timeout #f)
                                                             (timeout-val #f))
    (let* ([cell (SCM_LIST1 obj)] [retval (SCM_OBJ q)])
-     (.if "defined(GAUCHE_HAS_THREADS)"
+     (.if (defined GAUCHE_HAS_THREADS)
           (do-with-timeout q retval timeout timeout-val writerWait
                            (begin)
                            (?: (!= (MTQ_MAXLEN q) 0)
@@ -523,21 +528,21 @@
             (return FALSE))]))
 
  (define-cproc dequeue! (q::<queue> :optional fallback)
-   (let* ([empty::int FALSE] [r SCM_UNDEFINED])
+   (let* ([empty::int FALSE] [fb::(volatile ScmObj) fallback] [r SCM_UNDEFINED])
      (if (not (MTQP q))
        (set! empty (dequeue-int q (& r)))
        (with-mtq-light-lock q (set! empty (dequeue-int q (& r)))))
      (if empty
-       (if (SCM_UNBOUNDP fallback)
+       (if (SCM_UNBOUNDP fb)
          (Scm_Error "queue is empty: %S" q)
-         (set! r fallback))
+         (set! r fb))
        (when (MTQP q) (notify-writers q)))
      (return r)))
 
  (define-cproc dequeue/wait! (q::<mtqueue> :optional (timeout #f)
                                                      (timeout-val #f))
    (let* ([retval SCM_UNDEFINED])
-     (.if "defined(GAUCHE_HAS_THREADS)"
+     (.if (defined GAUCHE_HAS_THREADS)
           (do-with-timeout q retval timeout timeout-val readerWait
                            (begin (post++ (MTQ_READER_SEM q))
                                   (notify-writers (Q q)))
