@@ -1,7 +1,7 @@
 ;;;
 ;;; gdbm - gdbm interface
 ;;;
-;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -172,14 +172,15 @@
 ;;;
 
 (inline-stub
- "#include <gdbm.h>"
- "#include <stdlib.h>"
+ (declcode
+  (.include <gdbm.h>)
+  (.include <stdlib.h>))
 
- "typedef struct ScmGdbmFileRec {
-    SCM_HEADER;
-    ScmObj name;
-    GDBM_FILE dbf;              /* NULL if closed */
-  } ScmGdbmFile;"
+ (define-ctype ScmGdbmFile::(.struct
+                             (SCM_HEADER :: ""
+                              name
+                              dbf::GDBM_FILE ; NULL if closed
+                              )))
 
  (define-cclass <gdbm-file> :private ScmGdbmFile* "Scm_GdbmFileClass" ()
    ()
@@ -187,6 +188,7 @@
     (Scm_Printf port "#<gdbm-file %S>" (-> (SCM_GDBM_FILE obj) name))])
 
  (define-cfn gdbm_finalize (obj data::void*) ::void :static
+   (cast void data)                     ; suppress unused var warning
    (let* ((g :: ScmGdbmFile* (SCM_GDBM_FILE obj)))
      (when (-> g dbf)
        (gdbm_close (-> g dbf))
@@ -214,21 +216,11 @@
     `(unless (-> ,g dbf) (Scm_Error "gdbm file already closed: %S" ,g))])
 
  ;; Those symbols may not be defined in the older gdbm
- "#ifndef GDBM_SYNC"
- "#define GDBM_SYNC 0"
- "#endif"
- "#ifndef GDBM_NOLOCK"
- "#define GDBM_NOLOCK 0"
- "#endif"
- "#ifndef GDBM_SYNCMODE"
- "#define GDBM_SYNCMODE 0"
- "#endif"
- "#ifndef GDBM_CENTFREE"
- "#define GDBM_CENTFREE 0"
- "#endif"
- "#ifndef GDBM_COALESCEBLKS"
- "#define GDBM_COALESCEBLKS 0"
- "#endif"
+ (.unless (defined GDBM_SYNC) (.define GDBM_SYNC 0))
+ (.unless (defined GDBM_NOLOCK) (.define GDBM_NOLOCK 0))
+ (.unless (defined GDBM_SYNCMODE) (.define GDBM_SYNCMODE 0))
+ (.unless (defined GDBM_CENTFREE) (.define GDBM_CENTFREE 0))
+ (.unless (defined GDBM_COALESCEBLKS) (.define GDBM_COALESCEBLKS 0))
 
  (define-cproc gdbm-open
    (name::<string> :optional (size::<fixnum> 0)
