@@ -1,7 +1,7 @@
 ;;;
 ;;; odbm - original dbm interface
 ;;;
-;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -161,14 +161,12 @@
 ;;;
 
 (inline-stub
- "#include \"dbmconf.h\""
- "#if HAVE_DBM_H"
- "#include <dbm.h>"
- "#elif HAVE_GDBM_SLASH_DBM_H"
- "#include <gdbm/dbm.h>"
- "#elif HAVE_GDBM_MINUS_DBM_H"
- "#include <gdbm-dbm.h>"
- "#endif"
+ (declcode
+  (.include "dbmconf.h")
+  (.cond
+   ["HAVE_DBM_H" (.include <dbm.h>)]
+   ["HAVE_GDBM_SLASH_DBM_H" (.include <gdbm/dbm.h>)]
+   ["HAVE_GDBM_MINUS_DBM_H" (.include <gdbm-dbm.h>)]))
 
  ;; data conversion macros
  (define-cise-stmt TO_DATUM
@@ -193,7 +191,7 @@
  ;; Original dbm allows to open only one file at a time.
  ;; The static variable odbm_opened tracks the status.
  ;; TODO: MT SAFENESS
- "static int odbm_opened = FALSE;"
+ (define-cvar odbm_opened::int :static FALSE)
 
  ;; bindings
  (define-cproc odbm-init (name::<string>) ::<int>
