@@ -65,7 +65,7 @@
  * Please make sure that your system defines BYTE_ORDER.  If your
  * architecture is little-endian, make sure it also defines
  * LITTLE_ENDIAN and that the two (BYTE_ORDER and LITTLE_ENDIAN) are
- * equivilent.
+ * equivalent.
  *
  * If your system does not define the above, then you can do so by
  * hand like this:
@@ -247,7 +247,7 @@ void SHA512_Internal_Transform(SHA_CTX*, const sha_word64*);
 #define K1_60_TO_79	0xca62c1d6UL
 
 /* Initial hash value H for SHA-1: */
-const static sha_word32 sha1_initial_hash_value[5] = {
+static const sha_word32 sha1_initial_hash_value[5] = {
 	0x67452301UL,
 	0xefcdab89UL,
 	0x98badcfeUL,
@@ -256,7 +256,7 @@ const static sha_word32 sha1_initial_hash_value[5] = {
 };
 
 /* Hash constant words K for SHA-224 and SHA-256: */
-const static sha_word32 K256[64] = {
+static const sha_word32 K256[64] = {
 	0x428a2f98UL, 0x71374491UL, 0xb5c0fbcfUL, 0xe9b5dba5UL,
 	0x3956c25bUL, 0x59f111f1UL, 0x923f82a4UL, 0xab1c5ed5UL,
 	0xd807aa98UL, 0x12835b01UL, 0x243185beUL, 0x550c7dc3UL,
@@ -276,7 +276,7 @@ const static sha_word32 K256[64] = {
 };
 
 /* Initial hash value H for SHA-224: */
-const static sha_word32 sha224_initial_hash_value[8] = {
+static const sha_word32 sha224_initial_hash_value[8] = {
 	0xc1059ed8UL,
 	0x367cd507UL,
 	0x3070dd17UL,
@@ -288,7 +288,7 @@ const static sha_word32 sha224_initial_hash_value[8] = {
 };
 
 /* Initial hash value H for SHA-256: */
-const static sha_word32 sha256_initial_hash_value[8] = {
+static const sha_word32 sha256_initial_hash_value[8] = {
 	0x6a09e667UL,
 	0xbb67ae85UL,
 	0x3c6ef372UL,
@@ -300,7 +300,7 @@ const static sha_word32 sha256_initial_hash_value[8] = {
 };
 
 /* Hash constant words K for SHA-384 and SHA-512: */
-const static sha_word64 K512[80] = {
+static const sha_word64 K512[80] = {
 	0x428a2f98d728ae22ULL, 0x7137449123ef65cdULL,
 	0xb5c0fbcfec4d3b2fULL, 0xe9b5dba58189dbbcULL,
 	0x3956c25bf348b538ULL, 0x59f111f1b605d019ULL,
@@ -344,7 +344,7 @@ const static sha_word64 K512[80] = {
 };
 
 /* Initial hash value H for SHA-384 */
-const static sha_word64 sha384_initial_hash_value[8] = {
+static const sha_word64 sha384_initial_hash_value[8] = {
 	0xcbbb9d5dc1059ed8ULL,
 	0x629a292a367cd507ULL,
 	0x9159015a3070dd17ULL,
@@ -356,7 +356,7 @@ const static sha_word64 sha384_initial_hash_value[8] = {
 };
 
 /* Initial hash value H for SHA-512 */
-const static sha_word64 sha512_initial_hash_value[8] = {
+static const sha_word64 sha512_initial_hash_value[8] = {
 	0x6a09e667f3bcc908ULL,
 	0xbb67ae8584caa73bULL,
 	0x3c6ef372fe94f82bULL,
@@ -734,7 +734,8 @@ void SHA1_Final(sha_byte digest[], SHA_CTX* context) {
 	/* Convert FROM host byte order */
 	REVERSE64(context->s1.bitcount,context->s1.bitcount);
 #endif
-	*(sha_word64*)&context->s1.buffer[56] = context->s1.bitcount;
+        void *buf56 = &context->s1.buffer[56];
+	*(sha_word64*)buf56 = context->s1.bitcount;
 
 	/* Final transform: */
 	SHA1_Internal_Transform(context, (sha_word32*)context->s1.buffer);
@@ -1061,7 +1062,8 @@ void SHA256_Internal_Last(SHA_CTX* context) {
 		*context->s256.buffer = 0x80;
 	}
 	/* Set the bit count: */
-	*(sha_word64*)&context->s256.buffer[56] = context->s256.bitcount;
+        void *buf56 = &context->s256.buffer[56];
+	*(sha_word64*)buf56 = context->s256.bitcount;
 
 	/* Final transform: */
 	SHA256_Internal_Transform(context, (sha_word32*)context->s256.buffer);
@@ -1469,8 +1471,10 @@ void SHA512_Internal_Last(SHA_CTX* context) {
 		*context->s512.buffer = 0x80;
 	}
 	/* Store the length of input data (in bits): */
-	*(sha_word64*)&context->s512.buffer[112] = context->s512.bitcount[1];
-	*(sha_word64*)&context->s512.buffer[120] = context->s512.bitcount[0];
+        void *buf112 = &context->s512.buffer[112];
+	*(sha_word64*)buf112 = context->s512.bitcount[1];
+        void *buf120 = &context->s512.buffer[120];
+	*(sha_word64*)buf120 = context->s512.bitcount[0];
 
 	/* Final transform: */
 	SHA512_Internal_Transform(context, (sha_word64*)context->s512.buffer);
