@@ -174,6 +174,32 @@
        (rlet1 p '()
          (for-each (^x (push! p x)) (sseq 1 2 3 4 5))))
 
+(test* "map$" '(#\A #\B #\C #\D #\E)
+       ((map$ char-upcase) "abcde"))
+(test* "for-each$" '(#\A #\B #\C #\D #\E)
+       (rlet1 r '()
+         ((for-each$ (^x (push! r x))) "EDCBA")))
+(test* "fold$"  '(#\A #\B #\C #\D #\E . #t)
+       ((fold$ cons) #t "EDCBA"))
+(test* "fold$"  '(#\A #\B #\C #\D #\E . #f)
+       ((fold$ cons #f) "EDCBA"))
+(test* "fold2$"  '(195 (5 15 25 4 14 24 3 13 23 2 12 22 1 11 21))
+       (values->list
+        ((fold2$ (^[n0 n1 n2 s m]
+                   (values (+ n0 n1 n2 s)
+                           (list* n0 n1 n2 m)))
+                 0 '())
+         '(1 2 3 4 5 6) '(11 12 13 14 15) '(21 22 23 24 25 26))))
+(test* "fold3$" '(195 275701345920000
+                      (5 15 25 4 14 24 3 13 23 2 12 22 1 11 21))
+       (values->list
+        ((fold3$ (^[n0 n1 n2 s m l]
+                   (values (+ n0 n1 n2 s)
+                           (* n0 n1 n2 m)
+                           (list* n0 n1 n2 l)))
+                 0 1 '())
+         '(1 2 3 4 5 6) '(11 12 13 14 15) '(21 22 23 24 25 26))))
+
 (test-section "searching and selection")
 
 (test* "find (list)" 4
@@ -387,26 +413,22 @@
        (subseq '(1 2 3 4 5) 2))
 (test* "subseq (list)" '(3 4)
        (subseq '(1 2 3 4 5) 2 4))
-(test* "subseq (list)" '(1 2 3 4)
-       (subseq '(1 2 3 4 5) 0 -1))
 (test* "subseq (vector)" '#(3 4 5)
        (subseq '#(1 2 3 4 5) 2))
 (test* "subseq (vector)" '#(3 4)
        (subseq '#(1 2 3 4 5) 2 4))
-(test* "subseq (vector)" '#(1 2 3 4)
-       (subseq '#(1 2 3 4 5) 0 -1))
+(test* "subseq (bitvector)" '#*0011001
+       (subseq '#*1010011001 3))
+(test* "subseq (bitvector)" '#*0011
+       (subseq '#*1010011001 3 7))
 (test* "subseq (string)" "345"
        (subseq "12345" 2))
 (test* "subseq (string)" "34"
        (subseq "12345" 2 4))
-(test* "subseq (string)" "1234"
-       (subseq "12345" 0 -1))
 (test* "subseq (custom)" '("3" "4" "5")
        (slot-ref (subseq (sseq 1 2 3 4 5) 2) 'strings))
 (test* "subseq (custom)" '("3" "4")
        (slot-ref (subseq (sseq 1 2 3 4 5) 2 4) 'strings))
-(test* "subseq (custom)" '("1" "2" "3" "4")
-       (slot-ref (subseq (sseq 1 2 3 4 5) 0 -1) 'strings))
 
 (test* "for-each-with-index (list)" '((2 . c) (1 . b) (0 . a))
        (rlet1 r '()
@@ -673,14 +695,15 @@
   (define (tester !? in needle before after)
     (test* (format "break-list-by-sequence~a ~s" (if !? '! "") in)
            (list before after)
-           (receive (a b) ((if !?
-                             break-list-by-sequence!
-                             break-list-by-sequence)
-                           in needle)
-             (and (or (not !?)
-                      (null? a)
-                      (eq? a in))  ; check if head part is shared
-                  (list a b)))))
+           (let ([data (list-copy in)])
+             (receive (a b) ((if !?
+                               break-list-by-sequence!
+                               break-list-by-sequence)
+                             data needle)
+               (and (or (not !?)
+                        (null? a)
+                        (eq? a data))  ; check if head part is shared
+                    (list a b))))))
     
   (dolist [datum data]
     (apply tester #f datum)
