@@ -50,6 +50,13 @@
 (test* "grange + generator->list" '(1.0 1.5 2.0 2.5)
        (generator->list (grange 1.0 3 1/2)))
 
+(test* "grange + generator->list" '()
+       (generator->list (grange 5 0)))
+(test* "grange + generator->list" '(5 4 3 2 1)
+       (generator->list (grange 5 0 -1)))
+(test* "grange + generator->list" '(5.0 4.0 3.0 2.0 1.0)
+       (generator->list (grange 5 0 -1.0)))
+
 ;; converters
 (let-syntax ((t (syntax-rules ()
                   [(t dir fn cv data)
@@ -141,14 +148,40 @@
                   (^[yield] (let loop ([i 0]) (yield i) (loop (+ i 1))))))
   )
 
-;; test x->generator <collection>.  this needs to be tested after
-;; the test of 'generate'.
-(test* "x->generator <collection>" '((0 . a) (1 . b) (2 . c))
+(test* "x->generator <hash-table>" '((0 . a) (1 . b) (2 . c))
        (sort-by (generator->list (x->generator (hash-table 'eqv?
                                                            '(0 . a)
                                                            '(1 . b)
                                                            '(2 . c))))
                 car))
+(test* "x->generator <tree-map>" '((0 . a) (1 . b) (2 . c))
+       (generator->list (x->generator (alist->tree-map 
+                                       '((0 . a)
+                                         (1 . b)
+                                         (2 . c))
+                                       = <))))
+(test* "x->generator <tree-map>" '((2 . c) (1 . b) (0 . a))
+       (generator->list (x->generator (alist->tree-map 
+                                       '((0 . a)
+                                         (1 . b)
+                                         (2 . c))
+                                       = >))))
+
+;; test x->generator <collection>.  this needs to be tested after
+;; the test of 'generate'.
+(define-class <mycoll> (<collection>)
+  ((elts :init-value '() :init-keyword :elts)) ;list
+  )
+(define-method call-with-iterator ((coll <mycoll>) proc)
+  (define z (~ coll 'elts))
+  (proc (cut eq? z '())
+        (^[] (pop! z))))
+(test* "x->generator <collection>" '((0 . a) (1 . b) (2 . c))
+       (sort-by (generator->list (x->generator (make <mycoll>
+                                                 :elts '((0 . a)
+                                                         (1 . b)
+                                                         (2 . c)))))
+                car))
 
 (test* "gappend" '(0 1 2 3 a b c d A B C D)
        (generator->list (gappend (giota 4)
@@ -204,6 +237,11 @@
                                  0
                                  (^s (giota 10)))))
 
+(test* "giterate" '(0 2 4 6 8 10 12 14 16 18)
+       (generator->list (giterate (pa$ + 2) 0) 10))
+(test* "giterate1" '(2 4 6 8 10 12 14 16 18 20)
+       (generator->list (giterate1 (pa$ + 2) 0) 10))
+
 (define-syntax test-list-like
   (syntax-rules ()
     [(_  gfn lfn src ...)
@@ -408,4 +446,24 @@
                                    (giota 20)
                                    :open #t :repeat #t)))
 
+;; srfi-42+generator
+;; NB: This test should be moved to ext/srfi when srfi-42 becomes precompiled.
+(use srfi-42)
+
+(test* "srfi-42 + generator" '(0 2 4 6 8 10)
+       (list-ec (:while (:generator x (giota)) (<= x 5))
+                (* x 2)))
+
+(define-class <my-coll> (<collection>)
+  ((elements :init-keyword :elements)))
+(define-method call-with-iterator ((c <my-coll>) proc)
+  (define xs (~ c'elements))
+  (proc (^[] (null? xs))
+        (^[] (pop! xs))))
+
+(test* "srfi-42 + collection" '(2 4 6 8 10)
+       (list-ec (: x (make <my-coll> :elements '(1 2 3 4 5)))
+                (+ x x)))
+
+
 (test-end)
