@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.unicode - Various Unicode-specific operations
 ;;;
-;;;   Copyright (c) 2011-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2011-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -40,6 +40,8 @@
   (export ucs4->utf8  utf8-length  utf8->ucs4
           ucs4->utf16 utf16-length utf16->ucs4
           utf8->string string->utf8
+          utf16->string string->utf16
+          utf32->string string->utf32
 
           make-word-breaker
           make-word-reader
@@ -83,6 +85,7 @@
 ;;   'strict     - default, throws an error.
 ;;   'permissive - convert as if it is a valid value.
 ;;   'ignore     - drop the data.  behaves as if invalid data weren't there.
+;;   'replace    - replace the output char with U+FFFD
 
 (define-macro (utf8-2 c)  `(logior (ash ,c -6) #xc0))
 (define-macro (utf8-3 c)  `(logior (ash ,c -12) #xe0))
@@ -95,13 +98,17 @@
 (define-macro (utf8-w c)  `(logior (logand (ash ,c -18) #x3f) #x80))
 (define-macro (utf8-v c)  `(logior (logand (ash ,c -24) #x3f) #x80))
 
+(define-constant *utf8-replacement* '(#xef #xbf #xbd))
+(define-constant *utf16-replacement* '(#xfffd))
+(define-constant *ucs4-replacement*  #xfffd)
 
 ;; Returns list of integers
 (define (ucs4->utf8 code :optional (strictness 'strict))
   (cond [(< code 0)
          (ecase strictness
            [(strict permissive) (errorf "Invalid unicode codepoint: ~x" code)]
-           [(ignore) '()])]
+           [(ignore) '()]
+           [(replace) *utf8-replacement*])]
         [(< code #x80)     `(,code)]
         [(< code #x800)    `(,(utf8-2 code) ,(utf8-z code))]
         [(< code #x10000)
@@ -110,7 +117,8 @@
              [(strict)
               (errorf "Codepoint doesn't encode a character: ~4,'0x" code)]
              [(permissive) `(,(utf8-3 code) ,(utf8-y code) ,(utf8-z code))]
-             [(ignore) '()])
+             [(ignore) '()]
+             [(replace) *utf8-replacement*])
            `(,(utf8-3 code) ,(utf8-y code) ,(utf8-z code)))]
         [(< code #x110000) `(,(utf8-4 code)
                              ,(utf8-x code) ,(utf8-y code) ,(utf8-z code))]
@@ -129,27 +137,31 @@
                    `(,(utf8-6 code) ,(utf8-v code) ,(utf8-w code)
                      ,(utf8-x code) ,(utf8-y code) ,(utf8-z code))]
                   [else (errorf "Codepoint outside domain: #x~x" code)]
-                  )])]
+                  )]
+           [(replace) *utf8-replacement*])]
         ))
 
 (define (utf8-length octet :optional (strictness 'strict))
+  (define (err-oob octet)
+    (errorf "Argument out of range as an octet: ~s" octet))
   (define (err-illegal octet)
     (errorf "Illegal utf-8 octet: ~2,'0x" octet))
-  (cond [(< octet 0) (err-illegal octet)]
+  (cond [(< octet 0) (err-oob octet)]
         [(< octet #x80) 1]
         [(< octet #xc0) (ecase strictness
-                          [(strict permissive) (err-illegal octet)]
+                          [(strict) (err-illegal octet)]
+                          [(permissive replace) 1]
                           [(ignore) 0])]
         [(< octet #xe0) 2]
         [(< octet #xf0) 3]
         [(< octet #xf8) 4]
-        [(> octet #xff) (err-illegal octet)]
+        [(> octet #xff) (err-oob octet)]
         [else (ecase strictness
                 [(strict) (err-illegal octet)]
                 [(ignore) 0]
-                [(permissive) (cond [(< octet #xfc) 5]
-                                    [(< octet #xfe) 6]
-                                    [else (err-illegal octet)])])]))
+                [(permissive replace) (cond [(< octet #xfc) 5]
+                                            [(< octet #xfe) 6]
+                                            [else 1])])]))
 
 ;; decoding utf8
 ;;  To avoid code redundancy and runtime overhead, we generate
@@ -172,7 +184,11 @@
        (values b #f)
        (case (utf8-length b ',strictness)
          [(0) (values 0 ,(name "u1"))]         ;ignore
-         [(1) (values b #f)]
+         [(1) ,(if (eq? strictness 'replace)
+                 '(if (>= b #x80)
+                    (values *ucs4-replacement* #f)
+                    (values b #f))
+                 '(values b #f))]
          [(2) (values (logand #x1f b) ,(name "u2-1"))]
          [(3) (values (logand #x0f b) ,(name "u3-1"))]
          [(4) (values (logand #x07 b) ,(name "u4-1"))]
@@ -211,14 +227,17 @@
     (case strictness
       [(strict)     `(values 'bad #f)]
       [(permissive) `(values v #f)]
+      [(replace)    `(values ,*ucs4-replacement* #f)]
       [(ignore)     `(values 0 ,(string->symbol #"u1-~|strictness|"))]))
   (define (bad-octet strictness)
     (case strictness
       [(strict permissive) `(values 'bad #f)]
+      [(replace)           `(values ,*ucs4-replacement* #f)]
       [(ignore)            `(values b #t)]))
   (define (end-input strictness)
     (case strictness
       [(strict permissive) `(values 'bad #f)]
+      [(replace)           `(values ,*ucs4-replacement* #f)]
       [(ignore)            `(values eof #f)]))
   `(begin ,@(rec n 1)))
 
@@ -231,131 +250,137 @@
      (decode-utf8-gen-n 5 #x200000 ,strictness)
      (decode-utf8-gen-n 6 #x4000000 ,strictness)))
 
-(define utf8->ucs4 #f)
+(define utf8->ucs4
+  (let ()
+    (decode-utf8-gen strict)
+    (decode-utf8-gen permissive)
+    (decode-utf8-gen replace)
+    (decode-utf8-gen ignore)
 
-(let ()
-  (decode-utf8-gen strict)
-  (decode-utf8-gen permissive)
-  (decode-utf8-gen ignore)
+    (define (utf8->ucs4 octets :optional (strictness 'strict))
+      (define initial-state (ecase strictness
+                              [(strict)     u1-strict]
+                              [(permissive) u1-permissive]
+                              [(replace)    u1-replace]
+                              [(ignore)     u1-ignore]))
+      (if (null? octets)
+        (values eof '())
+        (let loop ([state initial-state]
+                   [val 0]
+                   [b (car octets)]
+                   [bs (cdr octets)])
+          (receive (val next) (state b val)
+            (cond [(eq? val 'bad)
+                   ;; TODO: better error message
+                   (error "invalid utf8 sequence")]
+                  [(not next)    (values val bs)]
+                  [(eq? next #t) (loop initial-state 0 val bs)]
+                  [(null? bs)    (loop next val eof '())]
+                  [else          (loop next val (car bs) (cdr bs))])))))
 
-  (define (%utf8->ucs4 octets :optional (strictness 'strict))
-    (define initial-state (ecase strictness
-                            [(strict)     u1-strict]
-                            [(permissive) u1-permissive]
-                            [(ignore)     u1-ignore]))
-    (if (null? octets)
-      (values eof '())
-      (let loop ([state initial-state]
-                 [val 0]
-                 [b (car octets)]
-                 [bs (cdr octets)])
-        (receive (val next) (state b val)
-          (cond [(eq? val 'bad)
-                 ;; TODO: better error message
-                 (error "invalid utf8 sequence")]
-                [(not next)    (values val bs)]
-                [(eq? next #t) (loop initial-state 0 val bs)]
-                [(null? bs)    (loop next val eof '())]
-                [else          (loop next val (car bs) (cdr bs))])))))
-
-  (set! utf8->ucs4 %utf8->ucs4)
-  )
+    utf8->ucs4))
 
 (define (ucs4->utf16 code :optional (strictness 'strict))
-  (cond [(< code #xd800) `(,code)]
-        [(< code #xe000)
-         (ecase strictness
-           [(strict)     (errorf "Converting codepoint ~x to utf-16 \
+  (define (error-oob strictness)
+    (ecase strictness
+      [(strict permissive)
+       (errorf "Input outside of Unicode codepoint range: #x~x" code)]
+      [(replace) *utf16-replacement*]
+      [(ignore) '()]))
+  (cond  [(< code 0) (error-oob strictness)]
+         [(< code #xd800) `(,code)]
+         [(< code #xe000)
+          (ecase strictness
+            [(strict)     (errorf "Converting codepoint ~x to utf-16 \
                                   will lose information" code)]
-           [(ignore)     '()]
-           [(permissive) `(,code)])]
-        [(< code #x10000) `(,code)]
-        [(< code #x110000) (let1 code (- code #x10000)
-                             `(,(logior (ash code -10) #xd800)
-                               ,(logior (logand code #x3ff) #xdc00)))]
-        [else
-         (ecase strictness
-           [(strict permissive)
-            (errorf "Input outside of Unicode codepoint range: #x~x" code)]
-           [(ignore) '()])]))
+            [(ignore)     '()]
+            [(replace)    *utf16-replacement*]
+            [(permissive) `(,code)])]
+         [(< code #x10000) `(,code)]
+         [(< code #x110000) (let1 code (- code #x10000)
+                              `(,(logior (ash code -10) #xd800)
+                                ,(logior (logand code #x3ff) #xdc00)))]
+         [else (error-oob strictness)]))
 
 (define (utf16-length word :optional (strictness 'strict))
   (cond [(<= #xd800 word #xdbff) 2]
         [(<= #xdc00 word #xdfff)
          (ecase strictness
            [(strict)    (errorf "unpaired surrogate: #x~x" word)]
-           [(permissive) 1]
+           [(permissive replace) 1]
            [(ignore)     0])]
         [(<= 0 word #xffff) 1]
         [else (ecase strictness
                 [(strict permissive) (errorf "Illegal utf-16 word: #x~x" word)]
+                [(replace) 1]
                 [(ignore) 0])]))
 
-(define utf16->ucs4 #f)
+(define utf16->ucs4
+  (let ()
+    ;; The state machine for decoding utf16.  Each state takes the current
+    ;; 16bit word, previous word, and stricness parameter.  Return values are
+    ;; the value to be passed around, and the action.
+    ;; Possible actions:
+    ;;   reset  - reset to initial state
+    ;;   surrogate - switch to secondary state
+    ;;   unget  - don't consume the current word, and use val as a return value.
+    ;;   skip   - don't take the next word and move to init
+    ;;   lone   - lone surrogate.  val indicates bad word.
+    ;;   range  - range error
+    ;;   end    - end
 
-(let ()
-  ;; The state machine for decoding utf16.  Each state takes the current
-  ;; 16bit word, previous word, and stricness parameter.  Return values are
-  ;; the value to be passed around, and the action.
-  ;; Possible actions:
-  ;;   reset  - reset to initial state
-  ;;   surrogate - switch to secondary state
-  ;;   unget  - don't consume the current word, and use val as a return value.
-  ;;   skip   - don't take the next word and move to init
-  ;;   lone   - lone surrogate.  val indicates bad word.
-  ;;   range  - range error
-  ;;   end    - end
+    ;; Common routine.  Surrogate? is #t if w is supposed to be the second
+    ;; word of the surrogated pair.
+    (define (dispatch surrogate? w prev strictness)
+      (if (not surrogate?)
+        ;; first word
+        (cond [(eof-object? w) (values eof 'end)]
+              [(< w 0) (values w 'range)]
+              [(< w #xd800) (values w 'end)]
+              [(< w #xdc00) (values w 'surrogate)]
+              [(< w #xe000) (ecase strictness
+                              [(strict)     (values w 'lone)]
+                              [(permissive) (values w 'end)]
+                              [(replace)    (values *ucs4-replacement* 'end)]
+                              [(ignore) (values 0 'reset)])]
+              [(< w #x10000) (values w 'end)]
+              [else (values w 'range)])
+        ;; second word
+        (cond [(eof-object? w) (ecase strictness
+                                 [(strict)     (values prev 'lone)]
+                                 [(permissive) (values prev 'end)]
+                                 [(replace)    (values *ucs4-replacement* 'end)]
+                                 [(ignore)     (values eof 'end)])]
+              [(<= #xdc00 w #xdfff)
+               (values (+ #x10000 (ash (logand #x3ff prev) 10) (logand #x3ff w))
+                       'end)]
+              [(<= 0 w #xffff)
+               (ecase strictness
+                 [(strict)     (values prev 'lone)]
+                 [(permissive) (values prev 'unget)]  ; don't consume w
+                 [(replace)    (values *ucs4-replacement* 'unget)] ; ditto
+                 [(ignore)     (values w 'skip)])]
+              [else (values w 'range)])))
 
-  ;; Common routine.  Surrogate? is #t if w is supposed to be the second
-  ;; word of the surrogated pair.
-  (define (dispatch surrogate? w prev strictness)
-    (if (not surrogate?)
-      ;; first word
-      (cond [(eof-object? w) (values eof 'end)]
-            [(< w 0) (values w 'range)]
-            [(< w #xd800) (values w 'end)]
-            [(< w #xdc00) (values w 'surrogate)]
-            [(< w #xe000) (ecase strictness
-                            [(strict) (values w 'lone)]
-                            [(permissive) (values w 'end)]
-                            [(ignore) (values 0 'reset)])]
-            [(< w #x10000) (values w 'end)]
-            [else (values w 'range)])
-      ;; second word
-      (cond [(eof-object? w) (ecase strictness
-                               [(strict)     (values prev 'lone)]
-                               [(permissive) (values prev 'end)]
-                               [(ignore)     (values eof 'end)])]
-            [(<= #xdc00 w #xdfff)
-             (values (+ #x10000 (ash (logand #x3ff prev) 10) (logand #x3ff w))
-                     'end)]
-            [(<= 0 w #xffff)
-             (ecase strictness
-               [(strict)     (values prev 'lone)]
-               [(permissive) (values prev 'unget)]  ; don't consume w
-               [(ignore)     (values w 'skip)])]
-            [else (values w 'range)])))
+    (define (utf16->ucs4 words :optional (strictness 'strict))
+      (let loop ([words words]
+                 [lookahead #f]
+                 [surrogate #f]
+                 [val 0])
+        (receive (w ws) (cond [lookahead (values lookahead words)]
+                              [(null? words) (values eof '())]
+                              [else (values (car words) (cdr words))])
+          (receive (val action) (dispatch surrogate w val strictness)
+            (ecase action
+              [(reset)  (loop ws #f #f 0)]
+              [(surrogate) (loop ws #f #t val)]
+              [(unget)  (values val words)] ; not consuming w
+              [(skip)   (loop ws val #f 0)]
+              [(lone)   (errorf "lone surrogate in utf-16 stream: #x~4,'0x" val)]
+              [(range)  (errorf "out-of-range word in utf-16 stream: ~x" val)]
+              [(end)    (values val ws)])))))
 
-  (define (%utf16->ucs4 words :optional (strictness 'strict))
-    (let loop ([words words]
-               [lookahead #f]
-               [surrogate #f]
-               [val 0])
-      (receive (w ws) (cond [lookahead (values lookahead words)]
-                            [(null? words) (values eof '())]
-                            [else (values (car words) (cdr words))])
-        (receive (val action) (dispatch surrogate w val strictness)
-          (ecase action
-            [(reset)  (loop ws #f #f 0)]
-            [(surrogate) (loop ws #f #t val)]
-            [(unget)  (values val words)] ; not consuming w
-            [(skip)   (loop ws val #f 0)]
-            [(lone)   (errorf "lone surrogate in utf-16 stream: #x~4,'0x" val)]
-            [(range)  (errorf "out-of-range word in utf-16 stream: ~x" val)]
-            [(end)    (values val ws)])))))
-
-  (set! utf16->ucs4 %utf16->ucs4)
-  )
+    utf16->ucs4))
 
 ;; R7RS procedures
 
@@ -363,12 +388,26 @@
 ;; at runtime.
 ;; At this moment, we don't worry much about performance when the native
 ;; encoding isn't utf8.
+;; If the input contains invalid utf-8 encoding, we replace those parts with
+;; U+FFFD.  That is a slow path.
 (define (utf8->string bvec :optional (start 0) (end -1))
   (assume-type bvec <u8vector>)
   (if (eq? (gauche-character-encoding) 'utf-8)
-    (rlet1 s (u8vector->string bvec start end)
-      (when (string-incomplete? s)
-        (error "invalid utf-8 sequence in u8vector:" bvec)))
+    (let1 s (u8vector->string bvec start end)
+      (if (string-incomplete? s)
+        ;; we have to go through every char
+        (with-output-to-string
+          (^[]
+            (let loop ([in ($ generator->lseq
+                              $ uvector->generator bvec start
+                              $ if (< end 0) (u8vector-length bvec) end)])
+              (receive (ch next) (utf8->ucs4 in 'replace)
+                (unless (eof-object? ch)
+                  (write-char (ucs->char ch))
+                  (loop next))))))
+        s))
+    ;; If intenral encoding isn't utf-8, we just let ces-convert handle it.
+    ;; (invalid char replacement may not be handled well).
     (ces-convert (u8vector->string bvec start end) 'utf-8)))
 
 (define (string->utf8 str :optional (start 0) (end -1))
@@ -381,6 +420,127 @@
                                      (gauche-character-encoding)
                                      'utf-8)))))
 
+;; ;; R7RS scheme.bytevector
+
+(define (utf16->string bvec :optional (endian #f) (ignore-bom? #f)
+                       (start 0) (end -1))
+  (assume-type bvec <u8vector>)
+  (let1 end (cond [(< end 0) (u8vector-length bvec)]
+                  [(or (< start 0) (< end start))
+                   (error "invalid start/end positions:" (list start end))]
+                  [else end])
+    (receive (start endian)
+        (if (or ignore-bom? (< (- end start) 2))
+          (values start (or endian 'bid-endian))
+          (let ([b0 (u8vector-ref bvec start)]
+                [b1 (u8vector-ref bvec (+ start 1))])
+            (cond [(and (eqv? b0 #xfe) (eqv? b1 #xff))
+                   (values (+ start 2) 'big-endian)]
+                  [(and (eqv? b0 #xff) (eqv? b1 #xfe))
+                   (values (+ start 2) 'little-endian)]
+                  [else (values start (or endian 'big-endian))])))
+      (unless (even? (- end start))
+        (error "number of input octets for utf16 isn't even:" (- end start)))
+      (with-output-to-string
+        (^[] (let loop ([in ($ generator->lseq
+                               $ %u8->u16-generator bvec start end endian)])
+               (receive (u32 next) (utf16->ucs4 in 'replace)
+                 (or (eof-object? u32)
+                     (begin (write-char (ucs->char u32))
+                            (loop next))))))))))
+
+(define (%u8->u16-generator bvec start end endian)
+  ;; We don't want to depend on binary.io
+  (define combine
+    (if (memq endian '(big big-endian))
+      (^[v k] (logior (ash (u8vector-ref v k) 8)
+                      (u8vector-ref v (+ k 1))))
+      (^[v k] (logior (u8vector-ref v k)
+                      (ash (u8vector-ref v (+ k 1)) 8)))))
+  (^[] (if (= start end)
+         (eof-object)
+         (begin0 (combine bvec start) (inc! start 2)))))
+
+(define (string->utf16 str :optional (endian 'big-endian) 
+                                     (add-bom? #f)
+                                     (start 0) end)
+  (with-builder (<u8vector> add! get)
+    (define add16!
+      (if (memq endian '(big big-endian))
+        (^u (add! (ash u -8)) (add! (logand u #xff)))
+        (^u (add! (logand u #xff)) (add! (ash u -8)))))
+    (when add-bom? (add16! #xfeff))
+    (generator-for-each (^[ch] (for-each add16! (ucs4->utf16 (char->ucs ch))))
+                        (string->generator str start end))
+    (get)))
+
+(define (utf32->string bvec :optional (endian #f) (ignore-bom? #f)
+                                      (start 0) (end -1))
+  (assume-type bvec <u8vector>)
+  (let1 end (cond [(< end 0) (u8vector-length bvec)]
+                  [(or (< start 0) (< end start))
+                   (error "invalid start/end positions:" (list start end))]
+                  [else end])
+    (receive (start endian)
+        (if (or ignore-bom? (< (- end start) 4))
+          (values start (or endian 'bid-endian))
+          (let ([b0 (u8vector-ref bvec start)]
+                [b1 (u8vector-ref bvec (+ start 1))]
+                [b2 (u8vector-ref bvec (+ start 2))]
+                [b3 (u8vector-ref bvec (+ start 3))])
+            (cond [(and (eqv? b0 0) (eqv? b1 0) (eqv? b2 #xfe) (eqv? b3 #xff))
+                   (values (+ start 4) 'big-endian)]
+                  [(and (eqv? b0 #xff) (eqv? b1 #xfe) (eqv? b2 0) (eqv? b3 0))
+                   (values (+ start 4) 'little-endian)]
+                  [else (values start (or endian 'big-endian))])))
+      (unless (zero? (modulo (- end start) 4))
+        (error "number of input octets for utf32 isn't multiple of 4"
+               (- end start)))
+      (if (zero? (mod start 4))
+        (u32vector->string (uvector-alias <u32vector> bvec start end)
+                           0 -1 #f endian)
+        (map-to <string> ucs->char 
+                ($ generator->lseq
+                   $ %u8->u32-generator bvec start end endian))))))
+
+(define (%u8->u32-generator bvec start end endian)
+  ;; We don't want to depend on binary.io
+  (define combine
+    (if (memq endian '(big big-endian))
+      (^[v k] (logior (ash (u8vector-ref v k) 24)
+                      (ash (u8vector-ref v (+ k 1)) 16)
+                      (ash (u8vector-ref v (+ k 2)) 8)
+                      (u8vector-ref v (+ k 3))))
+      (^[v k] (logior (u8vector-ref v k)
+                      (ash (u8vector-ref v (+ k 1)) 8)
+                      (ash (u8vector-ref v (+ k 2)) 16)
+                      (ash (u8vector-ref v (+ k 3)) 24)))))
+  (^[] (if (= start end)
+         (eof-object)
+         (begin0 (combine bvec start) (inc! start 4)))))
+
+(define (string->utf32 str :optional (endian 'big-endian)
+                                     (add-bom? #f)
+                                     (start 0) end)
+  (let* ([s ((with-module gauche.internal %maybe-substring) str start end)]
+         [len (string-length s)]
+         [r (make-u8vector (* 4 (if add-bom? (+ len 1) len)))]
+         [rr (uvector-alias <u32vector> r (if add-bom? 4 0))])
+    (string->u32vector! rr 0 s 0 -1 endian)
+    (when add-bom?
+      (case endian
+        [(big big-endian) 
+         (u8vector-set! r 0 #x00)
+         (u8vector-set! r 1 #x00)
+         (u8vector-set! r 2 #xfe)
+         (u8vector-set! r 3 #xff)]
+        [else 
+         (u8vector-set! r 0 #xff)
+         (u8vector-set! r 1 #xfe)
+         (u8vector-set! r 2 #x00)
+         (u8vector-set! r 3 #x00)]))
+    r))
+
 ;;;
 ;;;  Character properties
 ;;;
@@ -390,7 +550,8 @@
 ;;
 
 (inline-stub
- "#include \"gauche/priv/unicode_attr.h\""
+ (declcode
+  (.include "gauche/priv/unicode_attr.h"))
 
  (initcode "init_GB_symbols(Scm_CurrentModule());")
  (initcode "init_WB_symbols(Scm_CurrentModule());")
@@ -409,7 +570,7 @@
              [else (SCM_TYPE_ERROR scode "char or fixnum")]))])
 
  (define-cproc gb-property (scode) ::<int>
-   (let* ([ch::int])
+   (let* ([ch::int SCM_CHAR_INVALID])
      (get-arg ch scode)
      (cond [(== ch #x0a) (return GB_LF)]
            [(== ch #x0d) (return GB_CR)]
@@ -425,8 +586,8 @@
            [else (return GB_Other)])))
 
  (define-cproc wb-property (scode) ::<int>
-   (let* ([ch::int])
-     (get-arg ch scode) 
+   (let* ([ch::int SCM_CHAR_INVALID])
+     (get-arg ch scode)
      (cond [(== ch #x0a) (return WB_LF)]
            [(== ch #x0d) (return WB_CR)]
            [(== ch #x22) (return WB_Double_Quote)]
@@ -443,7 +604,7 @@
            [else (return WB_Other)])))
 
  (define-cproc width-property (scode) ::<int>
-   (let* ([ch::int])
+   (let* ([ch::int SCM_CHAR_INVALID])
      (get-arg ch scode)
      (cond [(< ch #x20000)
             (let*([k::u_char (aref width_table (>> ch 8))])
@@ -511,7 +672,8 @@
 (define (make-sequence-splitter cluster-reader-maker)
   (^[seq]
     (let1 gen (x->generator seq)
-      (generator->list (cluster-reader-maker gen identity)))))
+      (generator->list 
+       (cluster-reader-maker gen (cut coerce-to (class-of seq) <>))))))
 
 ;;=========================================================================
 ;; State transition compiler
@@ -540,7 +702,7 @@
                       [_ (list p)]))
                 alist))
   (let ([default-next-state* (expand-or default-next-state)]
-        [input-index-size 
+        [input-index-size
          (+ (dict-fold input-index-map (^[k v s] (max v s)) 0) 1)]
         [state-size (length state-desc)]
         [states (map (^s (cons (car s) (expand-or (cdr s)))) state-desc)]
@@ -675,7 +837,7 @@
     (:other
      (:else               -> #t :default))       ; WB14
     ))
-  
+
 (define *word-break-fa*
   (compile-state-transition-description *word-break-states*
                                         *word-break-default-next-states*
@@ -695,7 +857,9 @@
                                                     '(MidNumLet . 9)
                                                     '(Numeric . 10)
                                                     '(ExtendNumLet . 11)
-                                                    '(Other . 12)
+                                                    '(WSegSpace . 12)
+                                                    '(ZWJ . 13)
+                                                    '(Other . 14)
                                                     '(CR . 16)
                                                     '(LF . 17)
                                                     '(Single_Quote . 18)
@@ -826,7 +990,8 @@
                                                     '(T . 7)
                                                     '(LV . 8)
                                                     '(LVT . 9)
-                                                    '(Other . 10)
+                                                    '(ZWJ . 10)
+                                                    '(Other . 11)
                                                     '(CR . 16)
                                                     '(LF . 17))))
 
@@ -866,10 +1031,12 @@
 ;;;
 
 (inline-stub
- "#include <gauche/char_attr.h>"
- "#define CHAR_UPCASE 0"
- "#define CHAR_DOWNCASE 1"
- "#define CHAR_TITLECASE 2"
+ (declcode
+  (.include <gauche/char_attr.h>)
+  (.define CHAR_UPCASE 0)
+  (.define CHAR_DOWNCASE 1)
+  (.define CHAR_TITLECASE 2))
+
  (define-enum CHAR_UPCASE)
  (define-enum CHAR_DOWNCASE)
  (define-enum CHAR_TITLECASE)
@@ -896,7 +1063,7 @@
  (define-cproc %char-xcase-extended (scode buf::<vector>
                                      kind::<int> charp::<boolean>)
    ::<int>
-   (let* ([ch::int]
+   (let* ([ch::int SCM_CHAR_INVALID]
           [cm::ScmCharCaseMap]
           [pcm::(const ScmCharCaseMap*)]
           [i::int 0])
@@ -908,7 +1075,7 @@
        [(CHAR_UPCASE) (fill-result to_upper_full to_upper_simple)] ;toupper
        [(CHAR_DOWNCASE) (fill-result to_lower_full to_lower_simple)] ;tolower
        [(CHAR_TITLECASE) (fill-result to_title_full to_title_simple)] ;totitle
-       )))
+       [else (return 0)])))
 
  (define-enum SCM_CHAR_FULL_CASE_MAPPING_SIZE)
  )
