@@ -1,7 +1,7 @@
 /*
  * addr.c - socket address
  *
- *  Copyright (c) 2001-2018  Shiro Kawai  <shiro@acm.org>
+ *  Copyright (c) 2001-2020  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -64,7 +64,8 @@ SCM_DEFINE_BUILTIN_CLASS(Scm_SockAddrClass, sockaddr_print,
                          NULL, NULL, sockaddr_allocate,
                          NULL);
 
-void sockaddr_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
+void sockaddr_print(ScmObj obj, ScmPort *port, 
+                    ScmWriteContext *ctx SCM_UNUSED)
 {
     Scm_Printf(port, "#<sockaddr %S %S>",
                Scm_SockAddrFamily(SCM_SOCKADDR(obj)),
@@ -90,7 +91,8 @@ ScmObj Scm_SockAddrFamily(ScmSockAddr *addr)
 }
 
 /* Fallback of allocation method */
-static ScmObj sockaddr_allocate(ScmClass *klass, ScmObj initargs)
+static ScmObj sockaddr_allocate(ScmClass *klass SCM_UNUSED, 
+                                ScmObj initargs SCM_UNUSED)
 {
     Scm_Error("you can't directly instantiate the abstract class <sockaddr>");
     return SCM_UNDEFINED;       /* dummy */
@@ -129,7 +131,8 @@ ScmObj Scm_MakeSockAddr(ScmClass *klass, struct sockaddr *saddr, int len)
  * Unix domain socket
  */
 
-static ScmObj sockaddr_un_allocate(ScmClass *klass, ScmObj initargs)
+static ScmObj sockaddr_un_allocate(ScmClass *klass SCM_UNUSED,
+                                   ScmObj initargs)
 {
     ScmObj path = Scm_GetKeyword(key_path, initargs, SCM_FALSE);
     if (!SCM_FALSEP(path) && !SCM_STRINGP(path)) {
@@ -144,10 +147,10 @@ static ScmObj sockaddr_un_allocate(ScmClass *klass, ScmObj initargs)
 #endif
     addr->addr.sun_family = AF_UNIX;
     if (SCM_STRINGP(path)) {
-        u_int size;
+        ScmSmallInt size;
         const char *cpath = Scm_GetStringContent(SCM_STRING(path), &size,
                                                  NULL, NULL);
-        if (size >= sizeof(addr->addr.sun_path)-1) {
+        if ((size_t)size >= sizeof(addr->addr.sun_path)-1) {
             Scm_Error("path too long: %S", path);
         }
         memcpy(addr->addr.sun_path, cpath, size);
@@ -180,7 +183,7 @@ SCM_DEFINE_BUILTIN_CLASS(Scm_SockAddrUnClass, sockaddr_print,
  * Inet domain socket
  */
 
-static ScmObj sockaddr_in_allocate(ScmClass *klass, ScmObj initargs)
+static ScmObj sockaddr_in_allocate(ScmClass *klass SCM_UNUSED, ScmObj initargs)
 {
     ScmObj host = Scm_GetKeyword(key_host, initargs, key_any);
     ScmObj port = Scm_GetKeyword(key_port, initargs, SCM_MAKE_INT(0));
@@ -267,7 +270,7 @@ SCM_DEFINE_BUILTIN_CLASS(Scm_SockAddrInClass, sockaddr_print,
 
 #ifdef HAVE_IPV6
 
-static ScmObj sockaddr_in6_allocate(ScmClass *klass, ScmObj initargs)
+static ScmObj sockaddr_in6_allocate(ScmClass *klass SCM_UNUSED, ScmObj initargs)
 {
     ScmObj host = Scm_GetKeyword(key_host, initargs, key_any);
     ScmObj port = Scm_GetKeyword(key_port, initargs, SCM_MAKE_INT(0));
@@ -351,8 +354,6 @@ SCM_DEFINE_BUILTIN_CLASS(Scm_SockAddrIn6Class, sockaddr_print,
  * Parse internet addresses
  */
 
-#define S6_ADDR32(in6, offset) (*(uint32_t*)&(((in6).s6_addr)[(offset)*4]))
-
 ScmObj Scm_InetStringToAddress(const char *s,
                                int *proto,     /*out*/
                                ScmUVector *buf /*out*/)
@@ -367,7 +368,8 @@ ScmObj Scm_InetStringToAddress(const char *s,
             if (Scm_UVectorSizeInBytes(buf) < 4) {
                 Scm_Error("uniform vector buffer isn't big enough to hold IPv4 address: %S", buf);
             }
-            SCM_U32VECTOR_ELEMENTS(buf)[0] = in4.s_addr;
+            ScmU32Vector *u32 = (ScmU32Vector*)buf;
+            SCM_U32VECTOR_ELEMENTS(u32)[0] = in4.s_addr;
             return SCM_TRUE;
         } else {
             return Scm_MakeIntegerU(ntohl(in4.s_addr));
@@ -381,16 +383,16 @@ ScmObj Scm_InetStringToAddress(const char *s,
             if (Scm_UVectorSizeInBytes(buf) < 16) {
                 Scm_Error("uniform vector buffer isn't big enough to hold IPv6 address: %S", buf);
             }
-            SCM_U32VECTOR_ELEMENTS(buf)[0] = S6_ADDR32(in6, 0);
-            SCM_U32VECTOR_ELEMENTS(buf)[1] = S6_ADDR32(in6, 1);
-            SCM_U32VECTOR_ELEMENTS(buf)[2] = S6_ADDR32(in6, 2);
-            SCM_U32VECTOR_ELEMENTS(buf)[3] = S6_ADDR32(in6, 3);
+            ScmU8Vector *u8 = (ScmU8Vector*)buf;
+            for (int i=0; i<16; i++) {
+                SCM_U8VECTOR_ELEMENTS(u8)[i] = in6.s6_addr[i];
+            }
             return SCM_TRUE;
         } else {
             ScmObj s = SCM_MAKE_INT(0);
-            for (int i=0; i<4; i++) {
-                s = Scm_Add(Scm_Ash(s, 32),
-                            Scm_MakeIntegerU(ntohl(S6_ADDR32(in6, i))));
+            for (int i=0; i<16; i++) {
+                s = Scm_Add(Scm_Ash(s, 8),
+                            SCM_MAKE_INT(in6.s6_addr[i]));
             }
             return s;
         }
@@ -427,20 +429,20 @@ ScmObj Scm_InetAddressToString(ScmObj addr,  /* integer or uvector */
         char buf[INET6_ADDRSTRLEN];
         struct in6_addr in6;
         if (SCM_INTEGERP(addr)) {
-            ScmObj mask = Scm_MakeIntegerU(0xffffffffUL);
-            for (int i=0; i<4; i++) {
+            ScmObj mask = SCM_MAKE_INT(0xff);
+            for (int i=0; i<16; i++) {
                 u_long a = Scm_GetIntegerU(Scm_LogAnd(addr, mask));
-                S6_ADDR32(in6, 3-i) = htonl(a);
-                addr = Scm_Ash(addr, -32);
+                in6.s6_addr[15-i] = a;
+                addr = Scm_Ash(addr, -8);
             }
         } else if (SCM_UVECTORP(addr)) {
             if (Scm_UVectorSizeInBytes(SCM_UVECTOR(addr)) < 16) {
                 Scm_Error("uvector too short for IPv6 address: %S", addr);
             }
-            S6_ADDR32(in6, 0) = SCM_U32VECTOR_ELEMENTS(addr)[0];
-            S6_ADDR32(in6, 1) = SCM_U32VECTOR_ELEMENTS(addr)[1];
-            S6_ADDR32(in6, 2) = SCM_U32VECTOR_ELEMENTS(addr)[2];
-            S6_ADDR32(in6, 3) = SCM_U32VECTOR_ELEMENTS(addr)[3];
+            ScmU8Vector *u8 = SCM_U8VECTOR(addr);
+            for (int i=0; i<16; i++) {
+                in6.s6_addr[i] = SCM_U8VECTOR_ELEMENTS(u8)[i];
+            }
         } else {
             Scm_TypeError("address", "integer or uvector", addr);
         }
