@@ -1,7 +1,7 @@
 ;;;
 ;;; netlib.stub - network interface
 ;;;
-;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -35,9 +35,10 @@
 
 (inline-stub
 
- (declcode "#include \"gauche-net.h\"")
- (declcode "#include <gauche/class.h>")
- (declcode "#include <gauche/bignum.h>")
+ (declcode
+  (.include "gauche-net.h")
+  (.include <gauche/class.h>)
+  (.include <gauche/bignum.h>))
 
  (define-type <socket-address> "ScmSockAddr*" "socket address"
    "Scm_SockAddrP" "SCM_SOCKADDR")
@@ -56,20 +57,27 @@
  (define-cgeneric sockaddr-port "Scm_GenericSockAddrPort" (extern))
 
  (define-cmethod sockaddr-name ((addr "Scm_SockAddrClass"))
+   (cast void addr)                     ; suppress unused var warning
    (return '"unknown"))
 
  (define-cmethod sockaddr-family ((addr "Scm_SockAddrClass"))
+   (cast void addr)                     ; suppress unused var warning
    (return 'unknown))
 
  (define-cmethod sockaddr-name ((addr "Scm_SockAddrUnClass"))
    (return
-    (?: (> (-> (cast ScmSockAddr* addr) addrlen) (sizeof (struct sockaddr)))
+    (?: (> (cast unsigned (-> (cast ScmSockAddr* addr) addrlen))
+           (sizeof (struct sockaddr)))
         (SCM_MAKE_STR (ref (-> (cast ScmSockAddrUn* addr) addr) sun_path))
         (SCM_MAKE_STR ""))))
 
- (define-cmethod sockaddr-family ((addr "Scm_SockAddrUnClass")) (return 'unix))
+ (define-cmethod sockaddr-family ((addr "Scm_SockAddrUnClass"))
+   (cast void addr)                     ; suppress unused var warning
+   (return 'unix))
 
- (define-cmethod sockaddr-family ((addr "Scm_SockAddrInClass")) (return 'inet))
+ (define-cmethod sockaddr-family ((addr "Scm_SockAddrInClass")) 
+   (cast void addr)                     ; suppress unused var warning
+   (return 'inet))
 
  (define-cmethod sockaddr-addr ((addr "Scm_SockAddrInClass")) ::<ulong>
    (let* ([a::ScmSockAddrIn* (cast ScmSockAddrIn* addr)])
@@ -79,8 +87,9 @@
    (let* ([a::ScmSockAddrIn* (cast ScmSockAddrIn* addr)])
      (return (ntohs (ref (-> a addr) sin_port)))))
 
- (when "defined HAVE_IPV6"
+ (.when (defined HAVE_IPV6)
    (define-cmethod sockaddr-family ((addr "Scm_SockAddrIn6Class"))
+     (cast void addr)                     ; suppress unused var warning
      (return 'inet6))
 
    (define-cmethod sockaddr-addr ((addr "Scm_SockAddrIn6Class"))
@@ -196,7 +205,9 @@
     (return (Scm_SocketOutputPort sock bufmode))))
 
 (inline-stub 
- (if "defined(SHUT_RD) && defined(SHUD_WR) && defined(SHUT_RDWR)"
+ (.if (and (defined SHUT_RD)
+           (defined SHUT_WR)
+           (defined SHUT_RDWR))
    (begin
      (define-enum SHUT_RD)
      (define-enum SHUT_WR)
@@ -405,9 +416,9 @@
 ;; IPv6 routines
 
 (inline-stub 
-(declcode "extern ScmObj addrinfo_allocate(ScmClass *klass, ScmObj initargs);")
+ (declare-cfn addrinfo_allocate (klass::ScmClass* intargs))
 
-(if "defined HAVE_IPV6"
+ (.if (defined HAVE_IPV6)
     (begin
       (define-type <sys-addrinfo> "ScmSysAddrinfo*" #f
         "SCM_SYS_ADDRINFO_P" "SCM_SYS_ADDRINFO")
@@ -491,7 +502,7 @@
       (Scm_Error "uniform vector buffer too short: %S" buf))
     (for [() (> size 0) (-= size 2)]
          (when (== size 1)
-           (.if "WORDS_BIGENDIAN"
+           (.if WORDS_BIGENDIAN
                 (+= sum (<< (* (cast u_char* wp)) 8))
                 (+= sum (* (cast u_char* wp))))
            (break))
@@ -499,7 +510,7 @@
     (set! sum (+ (>> sum 16) (logand sum #xffff)))
     (+= sum (>> sum 16))
     (set! result (cast uint16_t (lognot sum)))  ;truncate to 16bit
-    (.if "!WORDS_BIGENDIAN"
+    (.unless WORDS_BIGENDIAN
          (set! result (logior (>> result 8) (<< result 8))))
     (return result)))
 
