@@ -2,7 +2,7 @@
 ;;; peg.scm - Parser Expression Grammar Parser
 ;;;
 ;;;   Copyright (c) 2006 Rui Ueyama (rui314@gmail.com)
-;;;   Copyright (c) 2008-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2008-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -48,26 +48,26 @@
           peg-parser->generator ;experimental
 
           parse-success?
-          return-result return-failure/expect return-failure/unexpect
+          return-result return-failure
+          return-failure/expect return-failure/unexpect
           return-failure/message return-failure/compound
           
-          $return $fail $expect $lift $lift* $debug
-          $do $try $seq $or $fold-parsers $fold-parsers-right
-          $many $many1 $skip-many $skip-many1
-          $repeat $optional
-          $alternate
+          $bind $return $fail $expect $lift $lift* $debug
+          $let $let* $try $assert $not
+          $or $fold-parsers $fold-parsers-right
+          $seq $seq0 $between
+          $many $many1 $many_ $many1_ $repeat $repeat_
+          $many-till $many-till_
+          $optional
           $sep-by $end-by $sep-end-by
-          $count $between $followed-by
-          $not $many-till $chain-left $chain-right
-          $lazy
+          $chain-left $chain-right
+          $lazy $parameterize
 
-          $s $c $y
-          $string $string-ci
-          $char $one-of $none-of $many-chars
-          $satisfy
-
-          anychar upper lower letter alphanum digit
-          hexdigit newline tab space spaces eof
+          $any $eos $.
+          
+          $symbol $string $string-ci
+          $char $one-of $none-of
+          $satisfy $match1 $match1*
 
           $->rope $->string $->symbol rope->string rope-finalize
           )
@@ -80,16 +80,16 @@
 ;;;
 ;;;   A ::= B C
 ;;;     => (define a ($seq b c))
-;;;    If you need values of B and C, use monadic macro $do, or applicative
+;;;    If you need values of B and C, use monadic macro $let, or applicative
 ;;;    combinator $lift
-;;;     => (define a ($do [x b] [y c] ($return (cons x y))))
+;;;     => (define a ($let ([x b] [y c]) ($return (cons x y))))
 ;;;     => (define a ($lift cons b c)
 ;;;
 ;;;   A :: B | C
 ;;;     => (define a ($or b c))
 ;;;     To be precise, this actually mean B / C in PEG; if B fails without
 ;;;     consuming input, we try C.  But if B ever consumes input, we don't
-;;;     backtrack and A failes.  If you want backtracking, use $try as well.
+;;;     backtrack and A fails.  If you want backtracking, use $try as well.
 ;;;     => (define a ($or ($try b) c))
 ;;;
 ;;;   A :: B*
@@ -102,14 +102,14 @@
 ;;;     is that you want a string out of gathered values.  $->string is a
 ;;;     combinator that just translates the semantic value.
 ;;;     => (define a ($->string ($many b)))
-;;;     If the result of a is futher gathered, it is wasteful to make it
+;;;     If the result of a is further gathered, it is wasteful to make it
 ;;;     a string, since it will be thrown away soon.  We have an efficient
 ;;;     intermediate string representation called rope.  Here you can make
 ;;;     the result of a as rope:
 ;;;     => (define a ($->rope ($many b)))
 ;;;     Later, you can concatenate and convert ropes to a string by
 ;;;     rope->string.
-;;;     If you don't need the values, you can use $skip-many instead, which
+;;;     If you don't need the values, you can use $many_ instead, which
 ;;;     is more efficient.
 ;;;
 ;;;   A ::= B?
@@ -118,7 +118,7 @@
 ;;;
 
 
-;;;  In this module, we provide various consturctors of PARSERs,
+;;;  In this module, we provide various constructors of PARSERs,
 ;;;  and DRIVER procedures.
 ;;;
 ;;;  A PARSER is merely a closure that takes a list of input (most
@@ -148,40 +148,51 @@
 ;;;  status into <parser-error> object and raises it.
 ;;;
 
-;; NB: Currently we're in the transition process of reimplementing this
-;; module for better performance *and* cleaner API.  You may see some
-;; code that doesn't make sense below, which may be just a remaining
-;; of the old code and waiting to be removed.  The whole module is not
-;; 'official' yet---do not rely on the the current behavior unless you
-;; are experimenting.
-;; The rfc.json module is built on top of this module and is serving as
-;; one of the benchmark/testbed.  The API of rfc.json is official; though
-;; its implementation may be changed as parser.peg is changed.
-
 ;;;============================================================
 ;;; Parse result types
 ;;;
 
+;; An error object for the external API.  Note that a parser won't raise
+;; this error; it just returns a 'failure' value, so that other options
+;; can be tried if there's any.   Creating, throwing, and catching an error
+;; is expesive operation compared to the simple call/return.  It is the
+;; parser driver that sees the failure as the final result, then constructs
+;; and creates <parse-error> to raise.
 (define-condition-type <parse-error> <error> #f
   (position)                            ;stream position
-  (objects))                            ;offending object(s) or messages
+  (type)                                ;fail type
+  (objects)                             ;expecting/unexpecting items
+  (token))                              ;token that caused error
 
 (define-method write-object ((o <parse-error>) out)
   (format out "#<<parse-error> ~S>" (~ o 'message)))
 
+;; Primitive parts to be used inside custom parser.
+;; The user usually doesn't need to use them, as long as he construct
+;; parsers by parser combinators.
 (define-inline (parse-success? x) (not x))
 
-(define-macro (return-result value s)
-  `(values #f ,value ,s))
-
-(define-macro (return-failure/message m s)
-  `(values 'fail-message ,m ,s))
-(define-macro (return-failure/expect m s)
-  `(values 'fail-expect ,m ,s))
-(define-macro (return-failure/unexpect m s)
-  `(values 'fail-unexpect ,m ,s))
-(define-macro (return-failure/compound m s)
-  `(values 'fail-compound ,m ,s))
+(define-inline (return-result v s) (values #f v s))
+(define-hybrid-syntax return-failure
+  (^[t v s] (values t v s))
+  ;; If type is given as a literal symbol, we statically check it.
+  (er-macro-transformer
+   (^[f r c]
+     (match f
+       [(_ ('quote x) v s)
+        (unless (memq x '(fail-message fail-expect fail-unexpect fail-compound))
+          (error "Invalid failure type; must be one of fail-message, \
+                  fail-expect, fail-unexpect or fail-compund, but got: " x))
+        (quasirename r
+          `(values ',x ,v ,s))]
+       [(_ t v s)
+        (quasirename r
+          `(values ,t ,v ,s))]
+       [_ f]))))
+(define-inline (return-failure/message v s)  (values 'fail-message v s))
+(define-inline (return-failure/expect v s)   (values 'fail-expect v s))
+(define-inline (return-failure/unexpect v s) (values 'fail-unexpect v s))
+(define-inline (return-failure/compound v s) (values 'fail-compound v s))
 
 (define (make-peg-parse-error type objs pos seq)
   (define (flatten-compound-error objs)
@@ -192,30 +203,36 @@
   (define (analyze-compound-error objs pos)
     (let1 grps (map (^g (cons (caar g) (map cdr g)))
                     (group-collection objs :key car))
+      ;; If we have explicit $fail, we just use it.  Other info is
+      ;; preserved in objects slot.
       (let ([msgs (assoc-ref grps 'fail-message)]
             [exps (assoc-ref grps 'fail-expect)]
             [unexps (assoc-ref grps 'fail-unexpect)])
         (tree->string
-         (cons (or-concat (cond-list
-                           [exps (compound-exps exps)]
-                           [unexps (compound-unexps unexps)]
-                           [msgs @ msgs]))
-               (format " at ~s" pos))))))
+         (if msgs
+           (cons (or-concat msgs) (format " at ~s" pos))
+           (cons (or-concat (cond-list
+                             [exps (compound-exps exps)]
+                             [unexps (compound-unexps unexps)]))
+                 (format " at ~s" pos)))))))
   (define (or-concat lis)
+    (define (rec lis)
+      (match lis
+        [(x y) `("(",x") or (",y")")]
+        [(x . more) `("(",x"), ",@(rec more))]))
     (match lis
       [() '()]
       [(x) `(,x)]
-      [(x y) `(,x " or " ,y)]
-      [(x . more) `(,x ", " ,@(or-concat more))]))
+      [xs (rec xs)]))
   (define (compound-exps exps)
-    (match exps
+    (match (delete-duplicates exps equal?)
       [(x) (format "expecting ~s" x)]
       [(xs ...) (format "expecting one of ~s" xs)]))
   (define (compound-unexps unexps)
-    (match unexps
+    (match (delete-duplicates unexps equal?)
       [(x) (format "not expecting ~s" x)]
       [(xs ...) (format "not expecting any of ~s" xs)]))
-  (define (message pos nexttok)
+  (define (message objs pos nexttok)
     (case type
       [(fail-message)  (format "~a at ~s" objs pos)] ;objs is a string message
       [(fail-expect)
@@ -226,14 +243,16 @@
        (if (char? objs)
          (format "expecting but ~s at ~a, and got ~s" objs pos nexttok)
          (format "expecting but ~a at ~a, and got ~s" objs pos nexttok))]
-      [(fail-compound)
-       (analyze-compound-error (flatten-compound-error objs) pos)]
+      [(fail-compound) (analyze-compound-error objs pos)]
       [else (format "unknown parser error at ~a: ~a" pos objs)]  ;for safety
       ))
-  (let1 nexttok (if (pair? seq) (car seq) (eof-object))
+  (let ([nexttok (if (pair? seq) (car seq) (eof-object))]
+        [objs (if (eq? type 'fail-compound)
+                (flatten-compound-error objs)
+                objs)])
     (make-condition <parse-error>
-                    'position pos 'objects objs
-                    'message (message pos nexttok))))
+                    'position pos 'type type 'objects objs 'token nexttok
+                    'message (message objs pos nexttok))))
 
 (define (construct-peg-parser-error r v s s1)
   (make-peg-parse-error r v
@@ -267,13 +286,19 @@
 ;; API
 ;;   NB: We can consolidate peg-parse-string and peg-parse-port via
 ;;   x->generator, but should we?
-(define (peg-parse-string parser str)
+(define (peg-parse-string parser str :optional (cont #f))
   (check-arg string? str)
-  (values-ref (peg-run-parser parser (x->lseq str)) 0))
+  (receive (r rest) (peg-run-parser parser (x->lseq str))
+    (if cont 
+      (cont r rest)
+      r)))
 ;; API
-(define (peg-parse-port parser port)
+(define (peg-parse-port parser port :optional (cont #f))
   (check-arg input-port? port)
-  (values-ref (peg-run-parser parser (x->lseq port)) 0))
+  (receive (r rest) (peg-run-parser parser (x->lseq port))
+    (if cont
+      (cont r rest)
+      r)))
 
 ;; API
 ;;  Returns a generator
@@ -332,13 +357,9 @@
 (define ($expect parse msg)
   (^s (receive (r v ss) (parse s)
         (if (parse-success? r)
-          (values r v ss)
+          (return-result v ss)
           (return-failure/expect msg s)))))
 
-;; a parser that merely returns 'fail-unexpect with MSG.
-(define ($unexpect msg s) (^_ (return-failure/unexpect msg s)))
-
-
 ;; A convenience utility to check the upper bound, allowing unlimited
 ;; upper bound by #f.
 (define-inline (>=? count max) (and max (>= count max)))
@@ -355,13 +376,13 @@
   (^s (receive [r v s1] (p s)
         (if (parse-success? r)
           ((f v) s1)
-          (values r v s1)))))
+          (return-failure r v s1)))))
 
 ;; API
 ;; $lift :: (a,...) -> b, (Parser,..) -> Parser
-;; ($lift f parser) == ($do [x parser] ($return (f x)))
+;; ($lift f parser) == ($let ([x parser]) ($return (f x)))
 (define-inline ($lift f . parsers)
-  ;; We don't use the straightforward definition (using $do or $bind)
+  ;; We don't use the straightforward definition (using $let or $bind)
   ;; to reduce closure construction.
   (^s (let accum ([s s] [parsers parsers] [vs '()])
         (if (null? parsers)
@@ -369,7 +390,7 @@
           (receive [r v s1] ((car parsers) s)
             (if (parse-success? r)
               (accum s1 (cdr parsers) (cons v vs))
-              (values r v s1)))))))
+              (return-failure r v s1)))))))
 
 ;; API
 ;; Like $lift, but f gets single list argument
@@ -380,7 +401,7 @@
           (receive [r v s1] ((car parsers) s)
             (if (parse-success? r)
               (accum s1 (cdr parsers) (cons v vs))
-              (values r v s1)))))))
+              (return-failure r v s1)))))))
 
 ;; API
 ;; For debugging
@@ -392,21 +413,71 @@
         (values r v s))))
 
 ;; API
-;; $do clause ... body
+;; $let (bind ...) body ...
 ;;   where
-;;     clause := (var parser)
-;;            |  (parser)
-;;            |  parser
-(define-syntax $do
-  (syntax-rules ()
-    [(_ body) body]
-    [(_ [var parser] clause . rest)
-     ($bind parser (^[var] ($do clause . rest)))]
-    [(_ [parser] clause . rest)
-     ($bind parser (^_ ($do clause . rest)))]
-    [(_ parser clause . rest)
-     ($bind parser (^_ ($do clause . rest)))]
-    [(_  . other) (syntax-error "malformed $do" ($do . other))]))
+;;     bind := (var parser)
+;;          |  (parser)
+;;          |  parser
+;; var's are visible from body ... (but not from parser)
+(define-syntax $let
+  (er-macro-transformer
+   (^[f r c]
+     (match f
+       [(_ (bind ...) body ...)
+        (let1 vars&parsers 
+            (map (^b (match b
+                       [(var parser) `(,var ,(gensym "parser") ,parser)]
+                       [(parser) `(,(gensym "_") ,(gensym "parser") ,parser)]
+                       [parser `(,(gensym "_") ,(gensym "parser") ,parser)]))
+                 bind)
+          (quasirename r
+            `(let (,@(map (^b `(,(cadr b) ,(caddr b))) vars&parsers))
+               ,@(let loop ([vars&parsers vars&parsers])
+                   (if (null? vars&parsers)
+                     body
+                     (match-let1 [(var pvar _) . rest] vars&parsers
+                       (quasirename r
+                         `(($bind ,pvar (^[,var] ,@(loop rest)))))))))))]
+       [_ (error "Malformed $let:" f)]))))
+
+;; API
+;; $let* (bind ...) body ...
+;;   where
+;;     bind := (var parser)
+;;          |  (parser)
+;;          |  parser
+;; var's are visible from subsequent bind and body
+(define-syntax $let*
+  (er-macro-transformer
+   (^[f r c]
+     (match f
+       [(_ () body ...) (quasirename r `(begin ,@body))]
+       [(_ ((var parser) bind ...) body ...)
+        (quasirename r
+          `($bind ,parser (^[,var] ($let* ,bind ,@body))))]
+       [(_ ((parser) bind ...) body ...)
+        (quasirename r
+          `($bind ,parser (^[,(gensym "_")] ($let* ,bind ,@body))))]
+       [(_ (parser bind ...) body ...)
+        (quasirename r
+          `($bind ,parser (^[,(gensym "_")] ($let* ,bind ,@body))))]
+       [_ (error "Malformed $let*:" f)]))))
+
+;; API
+;; ($parameterize ((param expr) ..) parser ...)
+;; Returns a parser that run parser ... while altering the parameter values
+;; like parameterize.  The parser ... are run as if in $seq.
+;; Suggested by Saito Atsushi
+(define-syntax $parameterize
+  (er-macro-transformer
+   (^[f r c]
+     (match f
+       [(_ ((p e) ...) parser ...)
+        (let1 tmp (gensym)
+          (quasirename r
+            `(let1 ,tmp ($seq ,@parser)
+               (^[s] (parameterize (,@(map list p e))
+                       (,tmp s))))))]))))
 
 ;; API
 ;; $or p1 p2 ...
@@ -435,7 +506,7 @@
 ;;   Apply parsers sequentially, passing around seed value.
 ;;   Note: $fold-parsers can be written much simpler (only shown in
 ;;   recursion branch):
-;;     ($do [v (car ps)] ($fold-parsers proc (proc v seed) (cdr ps)))
+;;     ($let ([v (car ps)]) ($fold-parsers proc (proc v seed) (cdr ps)))
 ;;   but it needs to create closures at parsing time, rather than construction
 ;;   time.  Interestingly, $fold-parsers-right can be written simply
 ;;   without this disadvantage.
@@ -449,24 +520,27 @@
           (receive (r1 v1 s1) ((car ps) s)
             (if (parse-success? r1)
               (loop s1 (cdr ps) (proc v1 seed))
-              (values r1 v1 s1))))))))
+              (return-failure r1 v1 s1))))))))
 
 ;; API
 (define ($fold-parsers-right proc seed ps)
   (match ps
     [()       ($return seed)]
-    [(p . ps) ($do [v    p]
-                   [seed ($fold-parsers-right proc seed ps)]
-                   ($return (proc v seed)))]))
+    [(p . ps) ($lift proc p ($fold-parsers-right proc seed ps))]))
 
 ;; API
-;; $seq p1 p2 ...
-;;   Match p1, p2 ... sequentially.  On success, returns the semantic
-;;   value of the last parser.
-;;   To get all the results of p1, p2, ... in a list, use $lift* list p1 p2 ...
+;; $seq P ... Pz
+;;   Matches P ... Pz sequentially, and returns the result of Pz.
+;;   To get all the results of p1, p2, ... in a list, use $lift list p1 p2 ...
 (define ($seq . parsers)
   ($fold-parsers (^[v s] v) #f parsers))
 
+;; API
+;; $seq0 P0 P ...
+;;   Matches P0 P ..., and returns the result of P.
+(define ($seq0 parse . followers)
+  (apply $lift (^[v . _] v) parse followers))
+
 ;; API
 ;; $try parser
 ;;   Try to match parsers.  If it fails, backtrack to
@@ -477,9 +551,28 @@
 ;;   would try a, b, ... even some of them consumes the input.
 (define-inline ($try p)
   (^[s0] (receive (r v s) (p s0)
-           (if (not r)
+           (if (parse-success? r)
              (return-result v s)
-             (values r v s0)))))
+             (return-failure r v s0)))))
+
+;; API
+;; $assert parser
+;;   Match parser, but never consumes the result.
+;;   On success, the value of the parser is returned.
+(define-inline ($assert p)
+  (^s (receive (r v s1) (p s)
+        (values r v s))))
+
+;; API
+;; $not parser
+;;   Succeeds when the input does not matches parser.  The value is #t.
+;;   If the input matches P, unexpected failure results.
+;;   Never consumes input.
+(define ($not p)
+  (^s (receive (r v s1) (p s)
+        (if (parse-success? r)
+          (return-failure/unexpect v s)
+          (return-result #f s)))))
 
 ;; API
 (define-syntax $lazy
@@ -501,23 +594,11 @@
             (and max (> min max)))
     (error "invalid argument:" min max)))
 
-;; API
-;; $count p n
-;;   Exactly n times of p.  Returns the list.
-(define ($count parse n)
-  ($many parse n n))
-
-;; API
-;; $skip-count p n
-;;   Exactly n times of p, discarding the results but the last.
-(define ($skip-count parse n)
-  (if (= n 1)
-    parse
-    ($do parse ($skip-count parse (- n 1)))))
-
 ;; API
 ;; $many p :optional min max
+;; $many_ p :optional min max
 ;; $many1 p :optional max
+;; $many1_ p :optional max
 (define-inline ($many parse :optional (min 0) (max #f))
   (%check-min-max min max)
   (lambda (s)
@@ -528,31 +609,37 @@
           (cond [(parse-success? r) (loop (cons v vs) s1 (+ count 1))]
                 [(and (eq? s s1) (<= min count))
                  (return-result (reverse! vs) s1)]
-                [else (values r v s1)]))))))
+                [else (return-failure r v s1)]))))))
 
-(define ($many1 parse :optional (max #f))
-  (if max
-    ($do [v parse] [vs ($many parse 0 (- max 1))] ($return (cons v vs)))
-    ($do [v parse] [vs ($many parse)] ($return (cons v vs)))))
+(define-inline ($many_ parse :optional (min 0) (max #f))
+  (%check-min-max min max)
+  (lambda (s)
+    (let loop ([s s] [count 0])
+      (if (>=? count max)
+        (return-result #t s)
+        (receive (r v s1) (parse s)
+          (cond [(parse-success? r) (loop s1 (+ count 1))]
+                [(and (eq? s s1) (<= min count))
+                 (return-result #t s1)]
+                [else (return-failure r v s1)]))))))
+
+(define-inline ($many1 parse :optional (max #f)) ($many parse 1 max))
+(define-inline ($many1_ parse :optional (max #f)) ($many_ parse 1 max))
 
 ;; API
-;; $skip-many p :optional min max
-;; $skip-many1 p :optional max
-;;   Like $many, but does not keep the results. Always returns #f.
-;;   This should be optimized; we don't need to retain intermediate values
-(define ($skip-many parse :optional (min 0) (max #f))
-  (%check-min-max min max)
-  (if (= min 0)
-    ($do [($many parse min max)]
-         ($return #f))
-    ($do [($skip-count parse min)]
-         [($skip-many parse 0 (and max (- max min)))]
-         ($return #f))))
+;; $repeat p n
+;; $repeat_ p n
+;;   Exactly n time of P.  Same as ($many p n n)
+(define ($repeat parse n) ($many parse n n))
+(define ($repeat_ parse n) ($many_ parse n n))
 
-(define ($skip-many1 parse :optional (max #f))
-  (if max
-    ($do parse [($skip-many parse 0 (- max 1))] ($return #f))
-    ($do parse [($skip-many parse)] ($return #f))))
+;; API
+;; $many-till P E :optional min max
+;; $many-till_ P E :optional min max
+(define ($many-till parse end . args)
+  (apply $many ($seq ($not end) parse) args))
+(define ($many-till_ parse end . args)
+  (apply $many_ ($seq ($not end) parse) args))
 
 ;; API
 ;; $optional p :optional fallback
@@ -562,12 +649,6 @@
 (define ($optional parse :optional (fallback #f))
   ($or parse ($return fallback)))
 
-;; API
-;; $repeat p n
-;;   Exactly n time of P.  Same as ($many p n n)
-(define ($repeat parse n)
-  ($many parse n n))
-
 ;; API
 ;; $sep-by p sep :optional min max
 ;;   P sparated by SEP, e.g. P SEP P SEP P.  Returns list of values of P.
@@ -575,39 +656,28 @@
 ;;   entire $sep-by fails.
 (define ($sep-by parse sep :optional (min 0) (max #f))
   (define rep
-    ($do [x parse]
-         [xs ($many ($seq sep parse)
-                    (clamp (- min 1) 0)
-                    (and max (- max 1)))]
-         ($return (cons x xs))))
+    ($let ([x parse]
+           [xs ($many ($seq sep parse)
+                      (clamp (- min 1) 0)
+                      (and max (- max 1)))])
+      ($return (cons x xs))))
   (cond
    [(and max (zero? max)) ($return '())]
    [(> min 0) rep]
    [else ($or rep ($return '()))]))
 
-;; API
-;; $alternate p sep
-;;   P separated by SEP.  Returns list of values of P.
-;;   Unline $sep-by, this one sets backtrack point before each SEP.  So,
-;;   for example, $sep-by failes with input P SEP P SEP P SEP Q, but
-;;   $alternate returns three results from SEP, leaving SEP Q in the input.
-(define ($alternate parse sep)
-  ($or ($do [h parse]
-            [t ($many ($try ($do [v1 sep] [v2 parse] ($return (list v1 v2)))))]
-            ($return (cons h (apply append! t))))
-       ($return '())))
-
 ;; API
 ;; $end-by p sep :optional min max
 ;;   Matches repetition of P SEP.  Returns a list of values of P.
 ;;   This one doesn't set backtrack point, so for example the input is
 ;;   P SEP P SEP P Q, the entire match fails.
 (define ($end-by parse sep . args)
-  (apply $many ($try ($do [v parse] sep ($return v))) args))
+  (apply $many ($try ($let ([v parse] sep) ($return v))) args))
 
 ;; API
 ;; $sep-end-by p sep min max
-;;   
+;;   The last SEP is optional.  The definition is a bit involved
+;;   for performance.
 (define ($sep-end-by parse sep :optional (min 0) (max #f))
   (%check-min-max min max)
   (^s (let loop ([vs '()] [s s] [count 0])
@@ -620,40 +690,16 @@
                             (loop (cons v vs) s.. (+ count 1))]
                            [(and (eq? s.. s.) (<= min (+ count 1)))
                             (return-result (reverse (cons v vs)) s.)]
-                           [else (values r. v. s..)]))]
+                           [else (return-failure r. v. s..)]))]
                   [(and (eq? s s.) (<= min count))
                    (return-result (reverse vs) s)]
-                  [else (values r v s.)]))))))
+                  [else (return-failure r v s.)]))))))
 
 ;; API
 ;; $between A B C
 ;;   Matches A B C, and returns the result of B.
 (define ($between open parse close)
-  ($do open [v parse] close ($return v)))
-
-;; API
-;; $followed-by P S ...
-;;   Matches P S ..., and returns the result of P.
-(define ($followed-by parse . followers)
-  (apply $lift (^[v . _] v) parse followers))
-
-;; API
-;; $not P
-;;   Succeeds when the input does not matches P.  The value is #f.
-;;   If the input matches P, unexpected failure results.
-;;   Unlike other parsers, input may be consumed when this parser succeeds.
-(define ($not parse)
-  (lambda (s0)
-    (receive (r v s) (parse s0)
-      (if r
-        (return-result #f s)
-        (return-failure/unexpect v s0)))))
-
-;; API
-;; $many-till P E :optional min max
-;;
-(define ($many-till parse end . args)
-  (apply $many ($do [($not end)] parse) args))
+  ($let (open [v parse] close) ($return v)))
 
 ;; API
 ;; $chain-left P OP
@@ -662,39 +708,142 @@
     (receive (r v s) (parse st)
       (if (parse-success? r)
         (let loop ([r1 r] [v1 v] [s1 s])
-          (receive (r2 v2 s2) (($do [proc op] [v parse]
-                                    ($return (proc v1 v)))
+          (receive (r2 v2 s2) (($let ([proc op] [v parse])
+                                 ($return (proc v1 v)))
                                s1)
             (if (parse-success? r2)
               (loop r2 v2 s2)
-              (values r1 v1 s1))))
-        (values r v s)))))
+              (return-failure r1 v1 s1))))
+        (return-failure r v s)))))
 
 ;; API
 ;; $chain-right P OP
 (define ($chain-right parse op)
   (rec (loop s)
-    (($do (h parse)
-          ($or ($try ($do [proc op]
-                          [t loop]
-                          ($return (proc h t))))
-               ($return h)))
+    (($let ([h parse])
+       ($or ($try ($let ([proc op]
+                         [t loop])
+                    ($return (proc h t))))
+            ($return h)))
      s)))
 
 ;; API
-;; $satisfy
-(define-syntax $satisfy
-  (syntax-rules (cut <>)
-    [(_ (cut p x <>) expect)            ;TODO: hygiene!
-     (lambda (s)
-       (if (and (pair? s) (p x (car s)))
-         (return-result (car s) (cdr s))
-         (return-failure/expect expect s)))]
-    [(_ pred expect)
-     (lambda (s)
-       (if (and (pair? s) (pred (car s)))
-         (return-result (car s) (cdr s))
-         (return-failure/expect expect s)))]))
+;; $satisfy PRED EXPECT [RESULT])
+;;   - Returns a parser such that ...
+;;   - If the head of input stream satisfies PRED, 
+;;     call (RESULT head (PRED head)) and let its result
+;;     as the value of successulf parsing.
+;;   - Otherwise, returns failure with EXPECT as the expected input.
+(define-inline ($satisfy pred expect :optional (result #f))
+  (^s (if-let1 v (and (pair? s) (pred (car s)))
+        (return-result (if result
+                         (result (car s) v) 
+                         (car s))
+                       (cdr s))
+        (return-failure/expect expect s))))
+
+;; API
+;; $match1 PATTERN [(=> FAIL)] [RESULT]
+;; $match1* PATTERN [(=> FAIL)] [RESULT]
+;;   - Run util.match#match against the input stream.
+;;   - $match1 takes one item from stream and see if it matches
+;;     with PATTERN.
+;;   - $match1* applys PATTERN on the entire input stream.
+;;   - If matched, RESULT is evaluated in the environment where pattern 
+;;     variables in PATTERN are bound, and its result becomes the result 
+;;     value of the parser.
+;;   - If RESULT is omitted, the matched item is returned.
+;;   - If (=> FAIL) is given, FAIL (must be an identifier) is bound to
+;;     a closure to be called (FAIL message) to return a failure.
+;;     NB: Unlinke the (=> FAIL) in match (util.match), FAIL is not
+;;     a continuation but just a procedure, so it must be called at
+;;     the tail position of RESULT.
+
+(define-syntax $match1*
+  (er-macro-transformer
+   (^[f r c]
+     (define (=>? x) (c (r'=>) (r x)))
+     (define fail-mark (gensym))
+     (define (build-fail-decl fail)
+       (if fail
+         (quasirename r
+           `((define (,fail msg) (cons ',fail-mark msg))))
+         '()))
+     (define (build-return result fail tail)
+       (if fail
+         (quasirename r
+           `(let ((r ,result))
+              (if (and (pair? r) (eq? (car r) ',fail-mark))
+                (return-failure/message (cdr r) s)
+                (return-result r ,tail))))
+         (quasirename r
+           `(return-result ,result ,tail))))
+     ;; If pat is (x ...), add a rest var: (x ... . rest)
+     ;; Returns the pattern and the rest var.
+     (define (make-match-pattern pat)
+       (if (pair? pat)
+         (if (null? (cdr (last-pair pat)))
+           (let1 rest (gensym)
+             (values (append pat rest) rest))
+           (values pat '()))
+         (values pat '())))
+     (define (build-parser pat result fail)
+       (receive (match-pat tail) (make-match-pattern pat)
+         (quasirename r
+           `(lambda (s)
+              ,@(build-fail-decl fail)
+              (match s
+                [,match-pat ,(build-return result fail tail)]
+                [_ (return-failure/expect ,(write-to-string pat) s)])))))
+     (match f
+       [(_ pat ((? =>?) fail) result)
+        (build-parser pat result fail)]
+       [(_ pat result)
+        (build-parser pat result #f)]
+       [(_ pat)
+        (build-parser pat
+                      (if-let1 n (length+ pat)
+                        (quasirename r
+                          `(take s ,n))
+                        (r 's))
+                      #f)]))))
+
+(define-syntax $match1
+  (er-macro-transformer
+   (^[f r c]
+     (define (=>? x) (c (r'=>) (r x)))
+     (define fail-mark (gensym))
+     (define (build-fail-decl fail)
+       (if fail
+         (quasirename r
+           `((define (,fail msg) (cons ',fail-mark msg))))
+         '()))
+     (define (build-return result fail)
+       (if fail
+         (quasirename r
+           `(let ((r ,result))
+              (if (and (pair? r) (eq? (car r) ',fail-mark))
+                (return-failure/message (cdr r) s)
+                (return-result r (cdr s)))))
+         (quasirename r
+           `(return-result ,result (cdr s)))))
+     (define (build-parser pat result fail)
+       (quasirename r
+         `(lambda (s)
+            ,@(build-fail-decl fail)
+            (if (pair? s)
+              (match (car s)
+                [,pat ,(build-return result fail)]
+                [_ (return-failure/expect ,(write-to-string pat) s)])
+              (return-failure/expect ,(write-to-string pat) s)))))
+     (match f
+       [(_ pat ((? =>?) fail) result)
+        (build-parser pat result fail)]
+       [(_ pat result)
+        (build-parser pat result #f)]
+       [(_ pat)
+        (build-parser pat (quasirename r `(car s)) #f)]
+       ))))
 
 ;;;============================================================
 ;;; Intermediate structure constructor
@@ -728,77 +877,74 @@
              (cons ca cd)))]
         [else obj]))
 
-(define-values ($string $string-ci)
-  (let-syntax
-      ([expand
-        (syntax-rules ()
-          ((_ char=)
-           (lambda (str)
-             (let1 lis (string->list str)
-               (lambda (s0)
-                 (let loop ((r '()) (s s0) (lis lis))
-                   (if (null? lis)
-                     (return-result (make-rope (reverse! r)) s)
-                     (if (and (pair? s)
-                              (char= (car s) (car lis)))
-                       (loop (cons (car s) r) (cdr s) (cdr lis))
-                       (return-failure/expect str s0)))))))))])
-    (values (expand char=?)
-            (expand char-ci=?))))
+;; NB: On success, we know the matched input is the same as STR,
+;; so we don't need to bother to collect matched chars.
+(define-inline ($string str)
+  (if (= (string-length str) 1)
+    (let1 ch (string-ref str 0)
+      (^s
+        (if (and (pair? s) (eqv? (car s) ch))
+          (return-result str (cdr s))
+          (return-failure/expect str s))))
+    (let1 lis (string->list str)
+      (^[s0]
+        (let loop ([s s0] [lis lis])
+          (if (null? lis)
+            (return-result str s)
+            (if (and (pair? s) (eqv? (car s) (car lis)))
+              (loop (cdr s) (cdr lis))
+              (return-failure/expect str s0))))))))
 
-(define ($char c)
-  ($satisfy (cut char=? c <>) c))
+(define ($string-ci str)
+  (let1 lis (string->list str)
+    (^[s0]
+      (let loop ([r '()] [s s0] [lis lis])
+        (if (null? lis)
+          (return-result (make-rope (reverse! r)) s)
+          (if (and (pair? s)
+                   (char? (car s))
+                   (char-ci=? (car s) (car lis)))
+            (loop (cons (car s) r) (cdr s) (cdr lis))
+            (return-failure/expect str s0)))))))
+   
+(define-inline ($char c)
+  (assume-type c <char>) 
+  ($satisfy (cut eqv? c <>) c))
 
 (define ($char-ci c)
-  ($satisfy (cut char-ci=? c <>)
+  (assume-type c <char>) 
+  ($satisfy (^x (and (char? x) (char-ci=? c x)))
             (list->char-set c (char-upcase c) (char-downcase c))))
 
 (define ($one-of charset)
-  ($satisfy (cut char-set-contains? charset <>)
+  (assume-type charset <char-set>)
+  ($satisfy (^x (and (char? x) (char-set-contains? charset x)))
             charset))
 
-(define ($s x) ($string x))
-
-(define ($c x) ($char x))
-
-(define ($y x) ($lift ($ string->symbol $ rope->string $) ($s x)))
-
-;; ($many-chars charset [min [max]]) == ($many ($one-of charset) [min [max]])
-;;   with possible optimization.
-(define-syntax $many-chars
-  (syntax-rules ()
-    [(_ parser) ($many ($one-of parser))]
-    [(_ parser min) ($many ($one-of parser) min)]
-    [(_ parser min max) ($many ($one-of parser) min max)]))
+(define ($symbol sym) 
+  (assume-type sym <symbol>)
+  ($seq ($string (symbol->string sym)) ($return sym)))
 
 (define ($none-of charset)
   ($one-of (char-set-complement charset)))
 
-(define (anychar s)
-  (if (pair? s)
-    (return-result (car s) (cdr s))
-    (return-failure/expect "character" s)))
+;; Anything except end of stream.
+(define-inline ($any)
+  (^s (if (pair? s)
+        (return-result (car s) (cdr s))
+        (return-failure/unexpect "end of input" s))))
 
-(define-syntax define-char-parser
-  (syntax-rules ()
-    ((_ proc charset expect)
-     (define proc
-       ($expect ($one-of charset) expect)))))
-
-(define-char-parser upper    #[A-Z]         "upper case letter")
-(define-char-parser lower    #[a-z]         "lower case letter")
-(define-char-parser letter   #[A-Za-z]      "letter")
-(define-char-parser alphanum #[A-Za-z0-9]   "letter or digit")
-(define-char-parser digit    #[0-9]         "digit")
-(define-char-parser hexdigit #[0-9A-Fa-f]   "hexadecimal digit")
-(define-char-parser newline  #[\n]          "newline")
-(define-char-parser tab      #[\t]          "tab")
-(define-char-parser space    #[\s]          "space")
-
-(define spaces ($lift make-rope ($many space)))
-
-(define (eof s)
-  (if (pair? s)
-    (return-failure/expect "end of input" s)
-    (return-result (eof-object) s)))
+(define-inline ($eos)
+  (^s (if (pair? s)
+        (return-failure/expect "end of input" s)
+        (return-result (eof-object) s))))
 
+;; Parse one item---a char, a string or a charset.
+(define-inline ($. item)
+  (cond
+   [(char-set? item) ($one-of item)]
+   [(char? item) ($char item)]
+   [(string? item) ($string item)]
+   [(symbol? item) ($symbol item)]
+   [else (error "Bad item: $. requires a char, a char-set, a string, \
+                 or a symbol, but got:" item)]))
