@@ -1,7 +1,7 @@
 /*
  * ctrie.h - Compact Trie
  *
- *   Copyright (c) 2009-2018  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2009-2020  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -98,7 +98,8 @@ typedef struct LeafRec {
 #define LEAF_KEY_MASK   ((1UL<<LEAF_KEY_BITS)-1)
 
 /* managing key in the leaf */
-static u_long leaf_key(Leaf *leaf)
+
+static inline u_long leaf_key(Leaf *leaf)
 {
     return (((leaf->key1&LEAF_KEY_MASK) << LEAF_KEY_BITS)
             + (leaf->key0&LEAF_KEY_MASK));
