@@ -1,7 +1,7 @@
 ;;;
 ;;; data.sparse - sparse data structures
 ;;;
-;;;   Copyright (c) 2007-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2007-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -75,11 +75,11 @@
 (select-module data.sparse)
 
 (inline-stub
- (declcode "#include \"ctrie.h\""
-           "#include \"spvec.h\""
-           "#include \"sptab.h\""
-           "#include <gauche/bits_inline.h>")
- )
+ (declcode
+  (.include "ctrie.h")
+  (.include "spvec.h")
+  (.include "sptab.h")
+  (.include <gauche/bits_inline.h>)))
 
 (define-macro (define-stuff type class iter ref set)
   (let ([x-fold     (string->symbol #"~|type|-fold")]
@@ -113,7 +113,7 @@
        (define (,x-update! sv k proc . fallback)
          (rlet1 tmp (proc (apply ,ref sv k fallback))
            (,set sv k tmp)))
-       ;; Treatment of defauleValue differ between sptab and spvec atm,
+       ;; Treatment of defaultValue differ between sptab and spvec atm,
        ;; so we define push! separately.
        ;; (define (,x-push! sv k val)
        ;;   (,set sv k (cons val (,ref sv k '()))))
@@ -189,6 +189,7 @@
  (define-cproc sparse-table-copy (sv::<sparse-table>) SparseTableCopy)
 
  (define-cfn sparse-table-iter (args::ScmObj* nargs::int data::void*) :static
+   (cast void nargs)                    ; suppress unused var warning
    (let* ([iter::SparseTableIter* (cast SparseTableIter* data)]
           [r (SparseTableIterNext iter)]
           [eofval (aref args 0)])
@@ -261,6 +262,7 @@
    "SPARSE_VECTOR_BASE_P" "SPARSE_VECTOR")
 
  (define-cproc %make-sparse-vector (type default-value flags::<ulong>)
+   (cast void flags)                     ; suppress unused var warning
    (let* ([klass::ScmClass* NULL])
      (cond [(SCM_CLASSP type)  (set! klass (SCM_CLASS type))]
            [(SCM_FALSEP type)  (set! klass SCM_CLASS_SPARSE_VECTOR)]
@@ -301,8 +303,9 @@
      (when (not oor)
        (set! r (SparseVectorRef sv i fallback)))
      (if (SCM_UNBOUNDP r)
-       (if (SCM_UNDEFINEDP (-> sv defaultValue))
-         (Scm_Error "%S doesn't have an entry at index %S" (SCM_OBJ sv) index)
+       (begin
+         (when (SCM_UNDEFINEDP (-> sv defaultValue))
+           (Scm_Error "%S doesn't have an entry at index %S" (SCM_OBJ sv) index))
          (return (-> sv defaultValue)))
        (return r))))
 
@@ -334,6 +337,7 @@
    SparseVectorInc)
 
  (define-cfn sparse-vector-iter (args::ScmObj* nargs::int data::void*) :static
+   (cast void nargs)                    ; suppress unused var warning
    (let* ([iter::SparseVectorIter* (cast SparseVectorIter* data)]
           [r (SparseVectorIterNext iter)]
           [eofval (aref args 0)])
@@ -405,8 +409,8 @@
   (define-type <sparse-matrix> "SparseVector*" "sparse matrix"
     "SPARSE_MATRIX_BASE_P" "SPARSE_MATRIX")
 
-  "#define X_OOR  1"
-  "#define Y_OOR  2"
+  (.define X_OOR 1)
+  (.define Y_OOR 2)
 
   (define-cise-stmt oor-check
     [(_ oor which oorval)
@@ -422,8 +426,8 @@
   ;; 2 or 4 bits in 32bit/64bit integer.  Also, if key is distributed
   ;; enough, we wonder if there's much sense aligning to the node boundary.
   ;; So we adopt 4 bits interleaving instead.
-  "#define INTERLEAVE_SHIFT 4"
-  "#define INTERLEAVE_MASK ((1UL<<INTERLEAVE_SHIFT)-1)"
+  (.define INTERLEAVE_SHIFT 4)
+  (.define INTERLEAVE_MASK (- (<< 1UL INTERLEAVE_SHIFT) 1))
   
   (define-cfn index-combine-2d (x y oor::int*)
     ::u_long :static
@@ -508,9 +512,10 @@
     (when (not oor)
       (set! r (SparseVectorRef sv i fallback)))
     (if (SCM_UNBOUNDP r)
-      (if (SCM_UNDEFINEDP (-> sv defaultValue))
-        (Scm_Error "%S doesn't have an entry at index (%S %S)"
-                   (SCM_OBJ sv) x y)
+      (begin
+        (when (SCM_UNDEFINEDP (-> sv defaultValue))
+          (Scm_Error "%S doesn't have an entry at index (%S %S)"
+                     (SCM_OBJ sv) x y))
         (return (-> sv defaultValue)))
       (return r))))
 
@@ -542,6 +547,7 @@
 
 (inline-stub
  (define-cfn sparse-matrix-iter (args::ScmObj* nargs::int data::void*) :static
+   (cast void nargs)                    ; suppress unused var warning
    (let* ([iter::SparseVectorIter* (cast SparseVectorIter* data)]
           [r (SparseVectorIterNext iter)]
           [eofval (aref args 0)])
