@@ -31,7 +31,7 @@
   (use srfi-13)
   (export time-tai time-utc time-monotonic time-thread
           time-process time-duration current-time time-resolution
-          make-time time-type time-second time-nanosecond
+          make-time time? time-type time-second time-nanosecond
           set-time-type! set-time-second! set-time-nanosecond! copy-time
           time=? time<? time<=? time>? time>=?
           time-difference time-difference! add-duration add-duration!
@@ -109,7 +109,8 @@
 ;; note they go higher to lower, and end in 1972.
 ;; See srfi-19/read-tai.scm to update this list.
 (define-constant tm:leap-second-table
-  '((1341100800 . 35)
+  '((1435708800 . 36)
+    (1341100800 . 35)
     (1230768000 . 34)
     (1136073600 . 33)
     (915148800 . 32)
