@@ -14,8 +14,10 @@
 (use srfi-1)
 (test-module 'srfi-1)
 
+(define Apply apply)                    ;avoic inline expansion
+
 (test* "xcons" '(a . b) (xcons 'b 'a))
-(test* "cons*" (test-error) (apply cons* '())) ; use apply to avoid compile error during inlining
+(test* "cons*" (test-error) (Apply cons* '())) ; use apply to avoid compile error during inlining
 (test* "cons*" 'o  (cons* 'o))
 (test* "cons*" '(1 2 3 . 4) (cons* 1 2 3 4))
 (test* "make-list" 5 (length (make-list 5)))
@@ -129,7 +131,7 @@
                    '(0 1 2 3 4)
                    (circular-list 0 1))))
 (test* "map!" '(4 1 5 1)
-       (map! + '(3 1 4 1) (circular-list 1 0)))
+       (map! + (list 3 1 4 1) (circular-list 1 0)))
 (test* "map-in-order"  '(4 1 5 1)
        (map-in-order + '(3 1 4 1) (circular-list 1 0)))
 (test* "pair-for-each" '((c) (b c) (a b c))
@@ -171,6 +173,19 @@
        (list-index = '(3 1 4 1 5 9) '(2 7 1)))
 ;; TODO: lset stuff
 
+(test* "lset=" #t (lset= char-ci=? '(#\a #\B #\c) '(#\C #\A #\b)))
+(test* "lset=" #f (lset= char-ci=? '(#\a #\B #\c #\d) '(#\C #\A #\b)))
+(test* "lset=" #f (lset= char-ci=? '(#\a #\B #\c) '(#\C #\d #\A #\b)))
+
+;; This tests the argument order of '=' predicate
+(test* "lset= (order)" #t
+       (lset= (^[x y] (assume (char<=? x y)) (char-ci=? x y))
+              '(#\A #\B #\C) '(#\c #\b #\a)))
+
+(test* "lset<=" #t (lset<= char-ci=? '(#\a #\B #\c) '(#\C #\A #\b)))
+(test* "lset<=" #f (lset<= char-ci=? '(#\a #\B #\c #\d) '(#\C #\A #\b)))
+(test* "lset<=" #t (lset<= char-ci=? '(#\a #\B #\c) '(#\C #\d #\A #\b)))
+
 ;;;
 ;;; test srfi-13
 ;;;
@@ -1398,4 +1413,79 @@
 (chibi-test
  (include "vectors-test.scm"))
 
+;;
+;; Test srfi-144
+;;
+
+(test-section "srfi-144")
+(use srfi-144)
+(test-module 'srfi-144)
+
+(define-module srfi-144-test
+
+  (define-module srfi.144 (extend srfi-144))
+  (define-module tests.scheme.test
+    (use gauche.test)
+    (export test test/one-of test/unspec-or-exn test/approx test/approx-1ulp)
+    (define (f=? a b)
+      (cond [(flonum? a)
+             (and (flonum? b)
+                  (cond [(nan? a) (nan? b)]
+                        [(-zero? a) (-zero? b)]
+                        [else (eqv? a b)]))]
+            [(list? a)
+             (and (list? b) (= (length a) (length b))
+                  (every f=? a b))]
+            [else (equal? a b)]))
+    (define-syntax test
+      (syntax-rules ()
+        [(_ expr expected)
+         (test* #"~'expr" expected expr f=?)]))
+    (define-syntax test/one-of
+      (syntax-rules ()
+        [(_ expr expected-list)
+         (test* #"~'expr" expected-list expr
+                (^[elis r] (any (cut f=? <> r) elis)))]))
+    (define-syntax test/unspec-or-exn
+      (syntax-rules ()
+        [(_ expr _)
+         (test* #"~'expr (error)" (test-error) expr)]))
+    (define-syntax test/approx-1ulp
+      (syntax-rules ()
+        [(_ expr expected)
+         (test* #"~'expr (approx 1ulp)" expected expr
+                (^[x y]
+                  (if (list? x)
+                    (and (list? y) (every approx=? x y))
+                    (approx=? x y))))]))
+    ;; NB: MinGW's bessel function jn has greater errors and we need
+    ;; to relax approx check.
+    (define rel-tolerance
+      (cond-expand
+       [gauche.os.windows 5e-7]
+       [else 1e-9]))
+    (define-syntax test/approx
+      (syntax-rules ()
+        [(_ expr expected)
+         (test* #"~'expr (approx)" expected expr
+                (cut approx=? <> <> rel-tolerance 1e-15))])))
+  (use scheme.base)
+  (include "../../test/include/srfi-144-tests.scm")
+  (with-module tests.scheme.flonum
+    (run-flonum-tests)))
+
+;;
+;; Test srfi-178
+;;
+
+(define-module srfi-178-test
+  (use gauche.test)
+  (use srfi-178)
+  (test-module 'srfi-178)
+  (use srfi-78)
+  
+  (define-syntax import (syntax-rules () [(_ _) #f]))
+  (include "../../test/include/srfi-178-tests.scm")
+  )
+
 (test-end)
