@@ -1,7 +1,7 @@
 /*
  * termios.c - termios interface
  *
- *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -50,7 +50,8 @@ SCM_DEFINE_BUILTIN_CLASS(Scm_SysTermiosClass,
                          termios_allocate,
                          NULL);
 
-static ScmObj termios_allocate(ScmClass *klass, ScmObj initargs)
+static ScmObj termios_allocate(ScmClass *klass SCM_UNUSED,
+                               ScmObj initargs SCM_UNUSED)
 {
     ScmSysTermios *t = SCM_NEW(ScmSysTermios);
     SCM_SET_CLASS(t, SCM_CLASS_SYS_TERMIOS);
@@ -186,6 +187,7 @@ void Scm_Init_termios(void)
 {
     SCM_INIT_EXTENSION(gauche__termios);
     ScmModule *mod = SCM_FIND_MODULE("gauche.termios", SCM_FIND_MODULE_CREATE);
+    (void)mod; /* suppress unused var warning */
 
 #if !defined(GAUCHE_WINDOWS)
     Scm_InitStaticClass(&Scm_SysTermiosClass, "<sys-termios>", mod,
