@@ -252,7 +252,7 @@
 (test* "make-condition-variable" #t
        (condition-variable? (make-condition-variable)))
 
-(test* "condition-varaible-name" 'foo
+(test* "condition-variable-name" 'foo
        (condition-variable-name (make-condition-variable 'foo)))
 
 (test* "condition-variable-specific" "hello"
@@ -441,8 +441,6 @@
 ;;---------------------------------------------------------------------
 (test-section "thread-local parameters")
 
-(use gauche.parameter)
-
 (define *thr1-val* #f)
 (define *thr2-val* #f)
 
@@ -504,6 +502,12 @@
          (for-each thread-join! ts)
          (atom-ref a)))
 
+(test* "atomic-update! extra values" '(2 3 4)
+       (let ([a (atom 1 2)])
+         (values->list
+          (atomic-update! a
+                          (^[x y] (values (+ x 1) (+ y 1) (+ x y 1)))))))
+
 ;;---------------------------------------------------------------------
 (test-section "threads and promise")
 
