@@ -1,7 +1,7 @@
 /*
  * thread.c - Scheme thread API
  *
- *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -35,6 +35,7 @@
 #include <gauche/vm.h>
 #include <gauche/extend.h>
 #include <gauche/exception.h>
+#include <gauche/priv/vmP.h>
 #include "threads.h"
 
 #ifdef HAVE_UNISTD_H
@@ -131,11 +132,12 @@ static SCM_INTERNAL_THREAD_PROC_RETTYPE thread_entry(void *data)
     return SCM_INTERNAL_THREAD_PROC_RETVAL;
 }
 
+#if defined(GAUCHE_USE_PTHREADS)
 /* The default signal mask on the thread creation */
 static struct threadRec {
-    int dummy;                  /* required to place this in data area */
     sigset_t defaultSigmask;
-} threadrec = { 0 };
+} threadrec;
+#endif /* GAUCHE_USE_PTHREADS */
 #endif /* defined(GAUCHE_HAS_THREADS) */
 
 /* Start a thread.  If the VM is in "NEW" state, create a new thread and
@@ -147,7 +149,7 @@ static struct threadRec {
    deconstruction phase, the handler thread_cleanup() runs and saves the
    thread result to the ScmVM structure.  If nobody cares about the
    result of the thread, ScmVM structure will eventually be GCed.
-   This is to prevent exitted thread's system resources from being
+   This is to prevent exited thread's system resources from being
    uncollected.
  */
 ScmObj Scm_ThreadStart(ScmVM *vm)
@@ -200,12 +202,12 @@ ScmObj Scm_ThreadStart(ScmVM *vm)
 }
 
 /* Thread join */
-ScmObj Scm_ThreadJoin(ScmVM *target, ScmObj timeout, ScmObj timeoutval)
+ScmObj Scm_ThreadJoin(ScmVM *target, ScmObj timeout, volatile ScmObj timeoutval)
 {
 #ifdef GAUCHE_HAS_THREADS
     ScmTimeSpec ts;
-    ScmObj result = SCM_FALSE, resultx = SCM_FALSE;
-    int intr = FALSE, tout = FALSE;
+    volatile ScmObj result = SCM_FALSE, resultx = SCM_FALSE;
+    volatile int intr = FALSE, tout = FALSE;
 
     ScmTimeSpec *pts = Scm_GetTimeSpec(timeout, &ts);
     SCM_INTERNAL_MUTEX_SAFE_LOCK_BEGIN(target->vmlock);
@@ -227,12 +229,12 @@ ScmObj Scm_ThreadJoin(ScmVM *target, ScmObj timeout, ScmObj timeoutval)
     if (tout) {
         if (SCM_UNBOUNDP(timeoutval)) {
             ScmObj e = Scm_MakeThreadException(SCM_CLASS_JOIN_TIMEOUT_EXCEPTION, target);
-            result = Scm_Raise(e);
+            result = Scm_Raise(e, 0);
         } else {
             result = timeoutval;
         }
     } else if (SCM_CONDITIONP(resultx)) {
-        result = Scm_Raise(resultx);
+        result = Scm_Raise(resultx, 0);
     }
     return result;
 #else  /*!GAUCHE_HAS_THREADS*/
@@ -388,7 +390,7 @@ ScmObj Scm_ThreadSleep(ScmObj timeout)
    We try to terminate the thread gracefully as possible.
    First, we use vm->stopRequest mechanism.  If the target thread is
    in VM loop, it responds to the flag and terminates itself.
-   If that fails, then we use more agressive means.
+   If that fails, then we use more aggressive means.
 
    TODO: We should probably make it configurable whether to use the
    forcible termination---it is too dangerous.
@@ -468,7 +470,7 @@ ScmObj Scm_ThreadTerminate(ScmVM *target)
 /*
  * Initialization.
  */
-void Scm_Init_threads(ScmModule *mod)
+void Scm_Init_threads(ScmModule *mod SCM_UNUSED)
 {
 #ifdef GAUCHE_USE_PTHREADS
     sigfillset(&threadrec.defaultSigmask);
