@@ -1,7 +1,7 @@
 ;;;
 ;;; threads.scm - thread related procedures.  to be autoloaded
 ;;;
-;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -57,15 +57,15 @@
 (select-module gauche.threads)
 
 (inline-stub
- "#include \"threads.h\""
-
  (declcode
-  "extern void Scm_Init_mutex(ScmModule*);"
-  "extern void Scm_Init_threads(ScmModule*);")
+  (.include "threads.h"))
+
+ (declare-cfn Scm_Init_mutex (mod::ScmModule*) ::void)
+ (declare-cfn Scm_Init_threads (mod::ScmModule*) ::void)
 
  (initcode
-  "Scm_Init_threads(Scm_CurrentModule());"
-  "Scm_Init_mutex(Scm_CurrentModule());"))
+  (Scm_Init_threads (Scm_CurrentModule))
+  (Scm_Init_mutex (Scm_CurrentModule))))
 
 ;;===============================================================
 ;; System query
@@ -75,8 +75,8 @@
 ;; it should return 'pthreads' instead of 'pthread' on pthreads platform.
 (inline-stub
  (define-cproc gauche-thread-type ()
-   (.cond ["defined(GAUCHE_USE_PTHREADS)" (return 'pthread)]
-          ["defined(GAUCHE_USE_WTHREADS)" (return 'win32)]
+   (.cond [(defined GAUCHE_USE_PTHREADS) (return 'pthread)]
+          [(defined GAUCHE_USE_WTHREADS) (return 'win32)]
           [else (return 'none)])))
 
 ;;===============================================================
@@ -112,7 +112,8 @@
      [(SCM_VM_STOPPED)   (return 'stopped)]
      [(SCM_VM_TERMINATED)(return 'terminated)]
      [else (Scm_Error "[internal] thread state has invalid value: %d"
-                      (-> vm state))]))
+                      (-> vm state))
+           (return SCM_UNDEFINED)]))   ;dummy
 
  (define-cproc %make-thread (thunk::<procedure> name) Scm_MakeThread)
 
@@ -272,14 +273,13 @@
        (with-lock timeout timeout-val timeout-vals (apply proc vals)))
      (^[proc timeout timeout-val timeout-vals]
        (with-lock timeout timeout-val timeout-vals
-                  (call-with-values (cut apply proc vals)
-                    (^ vs
-                      (unless (= (length vs) (length vals))
-                        (errorf "atomic-update!: procedure returned wrong \
+                  (receive newvals (apply proc vals)
+                    (unless (>= (length newvals) (length vals))
+                      (errorf "atomic-update!: procedure returned too few \
                                    number of values (~a, while ~a expected)"
-                                (length vs) (length vals)))
-                      (set! vals vs)
-                      (apply values vals))))))))
+                              (length newvals) (length vals)))
+                    (set! vals (take newvals (length vals)))
+                    (apply values newvals)))))))
 
 (define (atomic atom proc :optional (timeout #f) (timeout-val #f) :rest vals)
   (unless (atom? atom) (error "atom required, but got:" atom))
