@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007-2015, Cameron Rich
+ * Copyright (c) 2007-2019, Cameron Rich
  * 
  * All rights reserved.
  * 
@@ -92,13 +92,13 @@ static void ensure_mutex_initialization()
 #endif /* !(defined(__MINGW64_VERSION_MAJOR) && (_WIN32_WINNT >= 0x0600)) */
 #endif /* GAUCHE_WINDOWS */
 
-#ifndef WIN32
+#if !defined(WIN32) && defined(CONFIG_USE_DEV_URANDOM)
 static int rng_fd = -1;
-#elif defined(CONFIG_WIN32_USE_CRYPTO_LIB)
+#elif defined(WIN32) && defined(CONFIG_WIN32_USE_CRYPTO_LIB)
 static HCRYPTPROV gCryptProv;
 #endif
 
-#if (!defined(CONFIG_USE_DEV_URANDOM) && !defined(CONFIG_WIN32_USE_CRYPTO_LIB)) || !defined(ENTROPY_POOL_SIZE) /* The last condition added to compile on MinGW */
+#if !((!defined(WIN32) && defined(CONFIG_USE_DEV_URANDOM)) || (defined(WIN32) && defined(CONFIG_WIN32_USE_CRYPTO_LIB)))
 /* change to processor registers as appropriate */
 #define ENTROPY_POOL_SIZE 32
 #define ENTROPY_COUNTER1 ((((uint64_t)tv.tv_sec)<<32) | tv.tv_usec)
@@ -161,27 +161,27 @@ EXP_FUNC void STDCALL RNG_initialize()
         SCM_INTERNAL_MUTEX_UNLOCK(mutex);
         return;
     }
-    
+
 #if !defined(WIN32) && defined(CONFIG_USE_DEV_URANDOM)
     rng_fd = open("/dev/urandom", O_RDONLY);
 #elif defined(WIN32) && defined(CONFIG_WIN32_USE_CRYPTO_LIB)
     if (!CryptAcquireContext(&gCryptProv, 
                       NULL, NULL, PROV_RSA_FULL, 0))
     {
-        if (GetLastError() == NTE_BAD_KEYSET &&
+        if (GetLastError() == (DWORD)NTE_BAD_KEYSET &&
                 !CryptAcquireContext(&gCryptProv, 
                        NULL, 
                        NULL, 
                        PROV_RSA_FULL, 
                        CRYPT_NEWKEYSET))
         {
-            printf("CryptoLib: %x\n", unsupported_str, GetLastError());
+            printf("CryptoLib: %s(%lx)\n", unsupported_str, GetLastError());
             exit(1);
         }
     }
 #else
     /* start of with a stack to copy across */
-    /* int i; */
+    /* int i = rand(); */
     /* memcpy(entropy_pool, &i, ENTROPY_POOL_SIZE); */
     uint8_t arr[ENTROPY_POOL_SIZE];
     memcpy(entropy_pool, arr, ENTROPY_POOL_SIZE);
@@ -196,7 +196,9 @@ EXP_FUNC void STDCALL RNG_initialize()
  */
 EXP_FUNC void STDCALL RNG_custom_init(const uint8_t *seed_buf, int size)
 {
-#if defined(WIN32) || defined(CONFIG_WIN32_USE_CRYPTO_LIB)
+    (void)seed_buf;
+    (void)size;
+#if !((!defined(WIN32) && defined(CONFIG_USE_DEV_URANDOM)) || (defined(WIN32) && defined(CONFIG_WIN32_USE_CRYPTO_LIB)))
     int i;
 
     for (i = 0; i < ENTROPY_POOL_SIZE && i < size; i++)
@@ -215,9 +217,9 @@ EXP_FUNC void STDCALL RNG_terminate(void)
         return;
     }
     
-#ifndef WIN32
+#if !defined(WIN32) && defined(CONFIG_USE_DEV_URANDOM)
     close(rng_fd);
-#elif defined(CONFIG_WIN32_USE_CRYPTO_LIB)
+#elif defined(WIN32) && defined(CONFIG_WIN32_USE_CRYPTO_LIB)
     CryptReleaseContext(gCryptProv, 0);
 #endif
 
