@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007-2017, Cameron Rich
+ * Copyright (c) 2007-2019, Cameron Rich
  * 
  * All rights reserved.
  * 
@@ -656,7 +656,7 @@ error:
 }
 
 /** 
- * Increment the read sequence number (as a 64 bit endian indepenent #)
+ * Increment the read sequence number (as a 64 bit endian independent #)
  */     
 static void increment_read_sequence(SSL *ssl)
 {
@@ -670,7 +670,7 @@ static void increment_read_sequence(SSL *ssl)
 }
             
 /**
- * Increment the read sequence number (as a 64 bit endian indepenent #)
+ * Increment the read sequence number (as a 64 bit endian independent #)
  */      
 static void increment_write_sequence(SSL *ssl)
 {        
@@ -1086,7 +1086,6 @@ static int send_raw_packet(SSL *ssl, uint8_t protocol)
         }
     }
 
-    SET_SSL_FLAG(SSL_NEED_RECORD);  /* reset for next time */
     ssl->bm_index = 0;
 
     if (protocol != PT_APP_PROTOCOL_DATA)  
@@ -1347,6 +1346,7 @@ int basic_read(SSL *ssl, uint8_t **in_data)
         /* do we violate the spec with the message size?  */
         if (ssl->need_bytes > RT_MAX_PLAIN_LENGTH+RT_EXTRA-BM_RECORD_OFFSET)
         {
+            ssl->need_bytes = SSL_RECORD_SIZE;
             ret = SSL_ERROR_RECORD_OVERFLOW;              
             goto error;
         }
@@ -1655,6 +1655,7 @@ int send_alert(SSL *ssl, int error_code)
  */
 int process_finished(SSL *ssl, uint8_t *buf, int hs_len)
 {
+    (void)hs_len;
     int ret = SSL_OK;
     int is_client = IS_SET_SSL_FLAG(SSL_IS_CLIENT);
     int resume = IS_SET_SSL_FLAG(SSL_SESSION_RESUME);
@@ -2415,8 +2416,9 @@ EXP_FUNC void STDCALL ssl_display_error(int error_code) {}
 
 #ifdef CONFIG_BINDINGS
 #if !defined(CONFIG_SSL_ENABLE_CLIENT)
-EXP_FUNC SSL * STDCALL ssl_client_new(SSL_CTX *ssl_ctx, int client_fd, const
-        uint8_t *session_id, uint8_t sess_id_size)
+EXP_FUNC SSL * STDCALL ssl_client_new(SSL_CTX *ssl_ctx, int client_fd, 
+        const uint8_t *session_id, uint8_t sess_id_size, 
+        SSL_EXTENSIONS* ssl_ext)
 {
     printf("%s", unsupported_str);
     return NULL;
