@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007-2016, Cameron Rich
+ * Copyright (c) 2007-2017, Cameron Rich
  * 
  * All rights reserved.
  * 
@@ -92,8 +92,10 @@ int do_svr_handshake(SSL *ssl, int handshake_type, uint8_t *buf, int hs_len)
             if (ret == SSL_OK)    /* verify the cert */
             { 
                 int cert_res;
-                cert_res = x509_verify(
-                        ssl->ssl_ctx->ca_cert_ctx, ssl->x509_ctx);
+                int pathLenConstraint = 0;
+
+                cert_res = x509_verify(ssl->ssl_ctx->ca_cert_ctx, 
+                        ssl->x509_ctx, &pathLenConstraint);
                 ret = (cert_res == 0) ? SSL_OK : SSL_X509_ERROR(cert_res);
             }
             break;
@@ -157,7 +159,7 @@ static int process_client_hello(SSL *ssl)
 
     offset += id_len;
     cs_len = (buf[offset]<<8) + buf[offset+1];
-    offset += 3;        /* add 1 due to all cipher suites being 8 bit */
+    offset += 2;
 
     PARANOIA_CHECK(pkt_size, offset + cs_len);
 
@@ -165,9 +167,13 @@ static int process_client_hello(SSL *ssl)
        the preference */
     for (i = 0; i < cs_len; i += 2)
     {
+        /* only support ciphersuites with the form (0x00, xxxx) */
+        if (buf[offset+i])
+            continue;
+
         for (j = 0; j < NUM_PROTOCOLS; j++)
         {
-            if (ssl_prot_prefs[j] == buf[offset+i])   /* got a match? */
+            if (ssl_prot_prefs[j] == buf[offset+i+1])   /* got a match? */
             {
                 ssl->cipher = ssl_prot_prefs[j];
                 goto do_compression;
@@ -206,7 +212,7 @@ do_compression:
         ext_len += buf[offset++];
         PARANOIA_CHECK(pkt_size, offset + ext_len);
         
-        if (ext == SIG_ALG_EXTENSION)
+        if (ext == SSL_EXT_SIG_ALG)
         {
             while (ext_len > 0)
             {
