@@ -43,6 +43,15 @@
 #include <gauche/extern.h>
 #include <gauche/class.h>
 
+#define GAUCHE_CA_SYSTEM "system"
+
+/* We try these paths to find ca certs on Unix-like platforms */
+#define SYSTEM_CA_CERT_PATHS                                            \
+    "/etc/ssl/certs/ca-certificates.crt", /* ubnutu */                  \
+    "/usr/share/pki/ca-trust-source/ca-bundle.trust.crt", /* fedora */  \
+    "/etc/pki/tls/certs/ca-budle.crt",    /* fedora (compat) */         \
+    "/usr/local/etc/openssl/cert.pem"     /* osx homebrew openssl */
+
 #if defined(GAUCHE_USE_AXTLS)
 #include "axTLS/ssl/ssl.h"
 #endif  /* GAUCHE_USE_MBEDTLS */
@@ -58,12 +67,16 @@
 #include <mbedtls/net.h>
 #endif
 
-#ifndef GAUCHE_CA_BUNDLE
-#define GAUCHE_CA_BUNDLE "ca-cert.crt"
-#endif
-
 #endif  /*GAUCHE_USE_MBEDTLS*/
 
+/* This is a 'success' code of AxTLS.  mbedtls x509 routine returns 0
+   on success, which happens to be the same as SSL_OK.
+   We just need this in case we're configured with mbedtls and without axtls.
+ */
+#ifndef SSL_OK
+#define SSL_OK 0
+#endif
+
 #ifndef GAUCHE_USE_AXTLS
 /* dummy symbols */
 #define SSL_CLIENT_AUTHENTICATION               0x00010000
@@ -89,6 +102,7 @@ typedef struct ScmTLSRec ScmTLS;
 
 struct ScmTLSRec {
     SCM_HEADER;
+    ScmObj sock;                /* <socket> */
     ScmObj in_port;
     ScmObj out_port;
 
@@ -118,15 +132,18 @@ extern ScmObj Scm_TLSDestroy(ScmTLS* t);
 extern ScmObj Scm_TLSLoadObject(ScmTLS* t, ScmObj obj_type,
                                 const char *filename,
                                 const char *password);
-extern ScmObj Scm_TLSConnect(ScmTLS* t, int fd);
-extern ScmObj Scm_TLSAccept(ScmTLS* t, int fd);
+extern ScmObj Scm_TLSConnect(ScmTLS* t, ScmObj sock, int fd);
+extern ScmObj Scm_TLSAccept(ScmTLS* t, ScmObj sock, int fd);
 extern ScmObj Scm_TLSClose(ScmTLS* t);
+extern ScmObj Scm_TLSSocket(ScmTLS *t);
+
+extern int    Scm_TLSSystemCABundleAvailable(void);
 
 /*
    KZ: presumably due to block sizes imposed by the crypto algorithms
    used, TLSRead() doesn't take a desired size and instead returns
    whatever the underlying TLS layer was able to read and
-   decrypt. size accomodation is implemented in tls.scm.
+   decrypt. size accommodation is implemented in tls.scm.
  */
 extern ScmObj Scm_TLSRead(ScmTLS* t);
 extern ScmObj Scm_TLSWrite(ScmTLS* t, ScmObj msg);
