@@ -13,6 +13,8 @@
 ;;  Read input data from stdin, write modified data to stdout.
 ;;  $ gosh ssltest-mod.scm SRCDIR BUILDDIR < ssltest.c > ssltest.mod.c
 
+(use gauche.process)
+(use gauche.version)
 (use file.util)
 (use file.filter)
 (use util.match)
@@ -20,7 +22,7 @@
 (define (p . args) (for-each print args))
 
 (define (usage)
-  (p #"Usage: ,*program-name* $srcdir $builddir < ssltest.c > ssltest.mod.c"
+  (p #"Usage: ~*program-name* $srcdir $builddir < ssltest.c > ssltest.mod.c"
      "  Transforms axTLS's ssltest.c to the suitable form."
      "  Give absolute pathname of $srcdir and $builddir.")
   (exit 1))
@@ -44,16 +46,50 @@
      (define srcpath-replace #"~|srcpath|/")
      (define kicker-replace  #"~kicker ")
      ])
+  ;; Caveat: 
+  ;;  Some axTLS tests use 1024bit keys.  Recent Debian sets openssl SECLEVEL to
+  ;;  2 by default, which disables 1024bit keys and make tests fail.
+  ;;  We ensure SECLEVEL=1 with the command line.
+  ;;  Note that @SECLEVEL thing isn't supported in openssl 1.0.x.
+  ;;  https://sourceforge.net/p/gauche/mailman/gauche-devel/thread/87tvew1hri.fsf%40karme.de/
+  (define openssl-1.1>=?
+    (let1 ssl-version ($ rxmatch-substrings
+                         $ rxmatch #/(\w+)\s+([\d\.]+)/
+                         $ process-output->string
+                         (cond-expand [gauche.os.windows
+                                       ;; Running openssl directly fails
+                                       ;; deep in mingw runtime.
+                                       ;; https://github.com/shirok/Gauche/pull/467
+                                       '("cmd.exe" "/c" openssl version)]
+                                      [else
+                                       '(openssl version)]))
+      (and (list? ssl-version)
+           (string=? (~ ssl-version 1) "OpenSSL")
+           (version>=? (~ ssl-version 2) "1.1"))))
+
+  (define add-seclevel-client
+    (if openssl-1.1>=?
+      (^m #"~(m 0)@SECLEVEL=1")
+      (^m (m 0))))
+  (define add-seclevel-server
+    (if openssl-1.1>=?
+      (^m #"~(m 0) -cipher DEFAULT@SECLEVEL=1")
+      (^m (m 0))))
 
   (p "/* This is generated file. Don't edit! */"
      "static int safe_system(const char *);")
 
   (file-filter-for-each
    (^[line seed]
+     ;; The order of replacement is sensitive - add-seclevel-server must come
+     ;; before kicker-replace.
      ($ format #t "~a\n" $ regexp-replace-all* line
+        #/-cipher [\w-]+/ add-seclevel-client
+        #/openssl s_server/ add-seclevel-server
         #/\.\.\/ssl\// srcpath-replace
         #/openssl /    kicker-replace
-        #/system\s*\(/      "safe_system(")))
+        #/system\s*\(/      "safe_system("
+        )))
 
   (p "#include <errno.h>"
      "int safe_system(const char *commands)"
