@@ -27,11 +27,35 @@
     (warn #"~|msg|: some tests are skipped.\n")
     (set! openssl-cmd #f))
 
+  ;; MinGW's openssl command needs winpty.
+  ;; (MSYS's openssl command doesn't need it.)
+  (define mingw-detected
+    (cond-expand
+     [gauche.os.windows
+      (and-let1 msystem (sys-getenv "MSYSTEM")
+        (boolean (#/MINGW(64|32)/ msystem)))]
+     [else #f]))
+  (define openssl-path
+    (and mingw-detected
+         openssl-cmd
+         (find-file-in-paths openssl-cmd :extensions '("exe"))))
+  (define winpty-needed
+    (and mingw-detected
+         openssl-path
+         (boolean (#/mingw(64|32)/ openssl-path))))
+
   (sys-unlink "axTLS/ssl/openssl.pid")
   (sys-unlink "kick_openssl.sh")
 
-  (if (not openssl-cmd)
-    (no-openssl "openssl command not available")
+  (cond
+   [(not openssl-cmd)
+    (no-openssl "openssl command not available")]
+   [(and mingw-detected (not openssl-path))
+    (no-openssl "couldn't get openssl command path")]
+   [(and winpty-needed
+         (not (find-file-in-paths "winpty" :extensions '("exe"))))
+    (no-openssl "winpty not found. (MinGW's openssl command needs it.)")]
+   [else
     ;; Check openssl version.  OSX and MinGW32 ship with old openssl
     ;; that's unusable.
     (guard (e [(<process-abnormal-exit> e)
@@ -45,18 +69,26 @@
         (let1 vers (m 1)
           (unless (version>=? vers "1.0.1")
             (no-openssl #"openssl version is too old (~vers)")))
-        (no-openssl "couldn't get openssl version"))))
+        (no-openssl "couldn't get openssl version")))])
 
   (when openssl-cmd
-    ;; kick_openssl.sh is called from ssltest to run openssl command;
-    ;; It records process id of the invoked command so that it can be
-    ;; killed later by killopenssh.sh.
     (with-output-to-file "kick_openssl.sh"
       (^[]
         (print "#!/bin/sh")
         (print "set -e")
         (print #"echo \"$$\" \"~|openssl-cmd|\" >> openssl.pid")
-        (print #"exec \"~|openssl-cmd|\" \"$@\"")))
+        (cond
+         [winpty-needed
+          ;; MinGW's openssl command needs winpty only when stdin is terminal.
+          ;; (MSYS's openssl command doesn't need this workaround.)
+          (print  "if [ -t 0 ]; then")
+          (print #"    exec winpty -Xallow-non-tty -Xplain \"~|openssl-cmd|\" \"$@\"")
+          (print  "else")
+          (print #"    exec \"~|openssl-cmd|\" \"$@\"")
+          (print  "fi")]
+         [else
+          (print "echo \"Starting openssl in `pwd`\" >&2")
+          (print #"exec \"~|openssl-cmd|\" \"$@\"")])))
     (sys-chmod "kick_openssl.sh" #o755))
 
   (test* "ssltest" 0
@@ -64,8 +96,13 @@
           (run-process `(./ssltest
                          ,@(cond-list [openssl-cmd "--exttest"]))
                        :directory "axTLS/ssl"
-                       :output "ssltest.log"
+                       ;:output "ssltest.log"
                        :wait #t)))
+
+  ;; On MSYS (mintty), winpty with '-Xallow-non-tty' option changes tty
+  ;; setting, so that we should reset it.
+  (when (and openssl-cmd winpty-needed)
+    (sys-system "stty sane"))
   ]
  [else])
 
