@@ -55,7 +55,7 @@
         c))))
 
 (define (array-transpose a :optional (dim1 0) (dim2 1))
-  (let* ([sh (copy-object (array-shape a))]
+  (let* ([sh (array-copy (array-shape a))]
          [rank (array-rank a)]
          [tmp0 (array-ref sh dim1 0)]
          [tmp1 (array-ref sh dim1 1)])
@@ -63,7 +63,7 @@
     (array-set! sh dim1 1 (array-ref sh dim2 1))
     (array-set! sh dim2 0 tmp0)
     (array-set! sh dim2 1 tmp1)
-    (rlet1 res (make-array sh)
+    (rlet1 res (make-array-internal (class-of a) sh)
       (array-for-each-index a
         (^[vec1] (let* ([vec2 (vector-copy vec1)]
                         [tmp (vector-ref vec2 dim1)])
@@ -73,7 +73,7 @@
         (make-vector rank)))))
 
 (define (array-rotate-90 a :optional (dim1 0) (dim2 1))
-  (let* ([sh (copy-object (array-shape a))]
+  (let* ([sh (array-copy (array-shape a))]
          [rank (array-rank a)]
          [tmp0 (array-ref sh dim1 0)]
          [tmp1 (array-ref sh dim1 1)]
@@ -82,7 +82,7 @@
     (array-set! sh dim1 1 (array-ref sh dim2 1))
     (array-set! sh dim2 0 tmp0)
     (array-set! sh dim2 1 tmp1)
-    (rlet1 res (make-array sh)
+    (rlet1 res (make-array-internal (class-of a) sh)
       (array-for-each-index a
         (^[vec1] (let* ([vec2 (vector-copy vec1)]
                         [tmp (vector-ref vec2 dim1)])
@@ -108,10 +108,11 @@
             (vector-set! vec2 dimension j)
             (array-set! a vec1 (array-ref a vec2))
             (array-set! a vec2 tmp))))
-      (make-vector rank))))
+      (make-vector rank))
+    a))
 
 (define (array-flip a . args)
-  (rlet1 res (copy-object a)
+  (rlet1 res (array-copy a)
     (apply array-flip! res args)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -238,7 +239,7 @@
     (if (or (eq? class <f32array>)
             (eq? class <f64array>)
             (eq? class <array>))
-      (determinant! (copy-object a))
+      (determinant! (array-copy a))
       (let* ([rank (s32vector-length (start-vector-of a))]
              [b (tabulate-array (array-shape a)
                                 (^[ind] (array-ref a ind))
@@ -249,7 +250,73 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; matrix arithmetic
 
-(define (array-mul a b) ; NxM * MxP => NxP
+(define (array-mul2 a b) ; NxM * MxP => NxP
+  (let ([a-start (start-vector-of a)]
+        [a-end (end-vector-of a)]
+        [b-start (start-vector-of b)]
+        [b-end (end-vector-of b)])
+    (unless (= 2 (s32vector-length a-start) (s32vector-length b-start))
+      (error "array-mul matrices must be of rank 2"))
+    (let ([n (- (s32vector-ref a-end 0) (s32vector-ref a-start 0))]
+          [m (- (s32vector-ref a-end 1) (s32vector-ref a-start 1))]
+          [p (- (s32vector-ref b-end 1) (s32vector-ref b-start 1))]
+          [a-col-b-row-off (- (s32vector-ref a-start 1)
+                              (s32vector-ref b-start 0))])
+      (let1 res (make-minimal-backend-array (list a b) (shape 0 n 0 p))
+        (unless (= m (- (s32vector-ref b-end 0) (s32vector-ref b-start 0)))
+          (errorf "dimension mismatch: can't mul shapes ~S and ~S"
+                  (array-shape a) (array-shape b)))
+        (do ([i (s32vector-ref a-start 0) (+ i 1)])       ; for-each row of a
+            [(= i (s32vector-ref a-end 0)) res]
+          (do ([k (s32vector-ref b-start 1) (+ k 1)])     ; for-each col of b
+              [(= k (s32vector-ref b-end 1))]
+            (let1 tmp 0
+              (do ([j (s32vector-ref a-start 1) (+ j 1)]) ; for-each col of a & row of b
+                  [(= j (s32vector-ref a-end 1))]
+                (inc! tmp (* (array-ref a i j)
+                             (array-ref b (- j a-col-b-row-off) k))))
+              (array-set! res 
+                          (- i (s32vector-ref a-start 0))
+                          (- k (s32vector-ref b-start 1)) tmp))))))))
+
+(define (%array-mul r a b) ; NxM * MxP => NxP
+  (let ([a-start (start-vector-of a)]
+        [a-end (end-vector-of a)]
+        [b-start (start-vector-of b)]
+        [b-end (end-vector-of b)])
+    (unless (= 2 (s32vector-length a-start) (s32vector-length b-start))
+      (error "array-mul matrices must be of rank 2"))
+    (let ([a-start-row (s32vector-ref a-start 0)]
+          [a-end-row (s32vector-ref a-end 0)]
+          [a-start-col (s32vector-ref a-start 1)]
+          [a-end-col (s32vector-ref a-end 1)]
+          [b-start-col (s32vector-ref b-start 1)]
+          [b-end-col (s32vector-ref b-end 1)])
+      (let ([n (- a-end-row a-start-row)]
+            [m (- a-end-col a-start-col)]
+            [p (- b-end-col b-start-col)]
+            [a-col-b-row-off (- a-start-col (s32vector-ref b-start 0))])
+        (unless (= m (- (s32vector-ref b-end 0) (s32vector-ref b-start 0)))
+          (errorf "dimension mismatch: can't multiply shapes ~S and ~S"
+                  (array-shape a) (array-shape b)))
+        '(when (and r (not (and (= n (array-length r 0))
+                               (= p (array-length r 1)))))
+          (errof "result array can't hold the result of multiplication"))
+        (let1 res (or r (make-minimal-backend-array (list a b) (shape 0 n 0 p)))
+          (do ([i a-start-row (+ i 1)])       ; for-each row of a
+              [(= i a-end-row) res]
+            (do ([k b-start-col (+ k 1)])     ; for-each col of b
+                [(= k b-end-col)]
+              (let1 tmp 0
+                (do ([j a-start-col (+ j 1)]) ; for-each col of a & row of b
+                    [(= j a-end-col)]
+                  (inc! tmp (* (array-ref a i j) 
+                               (array-ref b (- j a-col-b-row-off) k))))
+                (array-set! res (- i a-start-row) (- k b-start-col) tmp)))))))))
+
+(define (array-mul a b) (%array-mul #f a b))
+
+(define (array-mul3 a b) ; NxM * MxP => NxP
   (let ([a-start (start-vector-of a)]
         [a-end (end-vector-of a)]
         [b-start (start-vector-of b)]
@@ -280,6 +347,8 @@
               (inc! tmp (* (array-ref a i j) (array-ref b (- j a-col-b-row-off) k))))
             (array-set! res (- i a-start-row) (- k b-start-col) tmp)))))))
 
+
+
 (define (array-div-left a b)
   (if-let1 b-1 (array-inverse b)
     (array-mul b-1 a)
@@ -293,7 +362,7 @@
 (define (array-expt ar pow)
   (let loop ([a ar] [n pow])
     (case n
-      [(0) (identity-array (s32vector-length (start-vector-of a)))]
+      [(0) (identity-array (s32vector-length (start-vector-of a)) (class-of a))]
       [(1) a]
       [(2) (array-mul a a)]
       [(3) (array-mul (array-mul a a) a)]
@@ -318,39 +387,50 @@
   a)
 
 (define-method array-add-elements! ((a <array-base>) (b <number>))
-  (array-map! a (^i (+ b i)) a))
+  (array-map! a (^i (+ b i)) a)
+  a)
 
 (define-method array-add-elements! ((a <number>) (b <array-base>))
-  (array-map! (copy-object b) (^i (+ a i)) b))
+  (rlet1 res (array-copy b)
+    (array-map! res (^i (+ a i)) b)))
 
 (define-method array-add-elements! ((a <array-base>) (b <array-base>))
-  (array-map! a (^[i j] (+ i j)) a b))
+  (array-map! a (^[i j] (+ i j)) a b)
+  a)
 
 (define (array-add-elements a . rest)
-  (rlet1 res (copy-object a)
-    (apply array-add-elements! res rest)))
+  (if (null? rest)
+    (begin (assume (array? a)) a)
+    (apply array-add-elements! (array-copy a) rest)))
 
 ;; sub
 
-(define-method array-sub-elements! ((a <array-base>))
-  (array-sub-elements! 1 a))
+(define-method array-sub-elements! ((a <array-base>)) a)
+
 
 (define-method array-sub-elements! ((a <array-base>) b c . rest)
   (array-sub-elements! a b)
   (apply array-sub-elements! a c rest))
 
 (define-method array-sub-elements! ((a <array-base>) (b <number>))
-  (array-map! a (^i (- i b)) a))
+  (array-map! a (^i (- i b)) a)
+  a)
 
 (define-method array-sub-elements! ((a <number>) (b <array-base>))
-  (array-map! (copy-object b) (^i (- a i)) b))
+  (rlet1 res (array-copy b)
+    (array-map! res (^i (- a i)) b)))
 
 (define-method array-sub-elements! ((a <array-base>) (b <array-base>))
-  (array-map! a (^[i j] (- i j)) a b))
+  (array-map! a (^[i j] (- i j)) a b)
+  a)
 
 (define (array-sub-elements a . rest)
-  (rlet1 res (copy-object a)
-    (apply array-sub-elements! res rest)))
+  (if (null? rest)
+    (begin (assume (array? a)) a)
+    (apply array-sub-elements! (array-copy a) rest)))
+
+(define (array-negate-elements! a) (array-sub-elements! 0 a))
+(define (array-negate-elements a) (array-sub-elements! 0 (array-copy a)))
 
 ;; mul
 
@@ -362,17 +442,21 @@
   a)
 
 (define-method array-mul-elements! ((a <array-base>) (b <number>))
-  (array-map! a (^i (* b i)) a))
+  (array-map! a (^i (* b i)) a)
+  a)
 
 (define-method array-mul-elements! ((a <number>) (b <array-base>))
-  (array-map! (copy-object b) (^i (* a i)) b))
+  (rlet1 res (array-copy b)
+    (array-map! res (^i (* a i)) b)))
 
 (define-method array-mul-elements! ((a <array-base>) (b <array-base>))
-  (array-map! a (^[i j] (* i j)) a b))
+  (array-map! a (^[i j] (* i j)) a b)
+  a)
 
 (define (array-mul-elements a . rest)
-  (rlet1 res (copy-object a)
-    (apply array-mul-elements! res rest)))
+  (if (null? rest)
+    (begin (assume (array? a)) a)
+    (apply array-mul-elements! (array-copy a) rest)))
 
 ;; div
 
@@ -384,18 +468,24 @@
   a)
 
 (define-method array-div-elements! ((a <array-base>) (b <number>))
-  (array-map! a (^i (/ i b)) a))
+  (array-map! a (^i (/ i b)) a)
+  a)
 
 (define-method array-div-elements! ((a <number>) (b <array-base>))
-  (array-map! (copy-object b) (^i (/ a i)) b))
+  (rlet1 res (array-copy b)
+    (array-map! res (^i (/ a i)) b)))
 
 (define-method array-div-elements! ((a <array-base>) (b <array-base>))
-  (array-map! a (^[i j] (/ i j)) a b))
+  (array-map! a (^[i j] (/ i j)) a b)
+  a)
 
 (define (array-div-elements a . rest)
-  (rlet1 res (copy-object a)
-    (apply array-div-elements! res rest)))
+  (if (null? rest)
+    (begin (assume (array? a)) a)
+    (apply array-div-elements! (array-copy a) rest)))
 
+(define (array-reciprocate-elements! a) (array-div-elements! 1 a))
+(define (array-reciprocate-elements a) (array-div-elements! 1 (array-copy a)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; utilities
