@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.uvector - uniform vectors
 ;;;
-;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -44,143 +44,36 @@
 (define-module gauche.uvector
   (use gauche.collection)
   (use gauche.sequence)
-  (use data.queue)
-  (export f16vector f16vector->list f16vector->vector
-          f16vector-add f16vector-add! f16vector-append
-          f16vector-clamp f16vector-clamp! f16vector-compare
-          f16vector-copy f16vector-copy! f16vector-div f16vector-div!
-          f16vector-dot f16vector-fill! f16vector-length
-          f16vector-mul f16vector-mul!
-          f16vector-multi-copy! f16vector-range-check
-          f16vector-ref f16vector-set! f16vector-sub f16vector-sub!
-          f16vector-swap-bytes f16vector-swap-bytes!
-          f16vector=? f16vector?
 
-          f32vector f32vector->list f32vector->vector
-          f32vector-add f32vector-add! f32vector-append
-          f32vector-clamp f32vector-clamp! f32vector-compare
-          f32vector-copy f32vector-copy! f32vector-div f32vector-div!
-          f32vector-dot f32vector-fill! f32vector-length
-          f32vector-mul f32vector-mul!
-          f32vector-multi-copy! f32vector-range-check
-          f32vector-ref f32vector-set! f32vector-sub f32vector-sub!
-          f32vector-swap-bytes f32vector-swap-bytes!
-          f32vector=? f32vector?
+  (include "./exports")
+  (export make-uvector
 
-          f64vector f64vector->list f64vector->vector f64vector-add
-          f64vector-add! f64vector-append f64vector-clamp f64vector-clamp!
-          f64vector-compare f64vector-copy f64vector-copy! f64vector-div
-          f64vector-div! f64vector-dot f64vector-fill! f64vector-length
-          f64vector-mul f64vector-mul! f64vector-multi-copy!
-          f64vector-range-check f64vector-ref f64vector-set! f64vector-sub
-          f64vector-sub! f64vector-swap-bytes f64vector-swap-bytes! f64vector=?
-          f64vector?
+          open-output-uvector get-output-uvector port->uvector
+          read-block! read-uvector read-uvector!
+          read-bytevector read-bytevector!
+          referencer
 
-          get-output-uvector
-
-          list->f16vector list->f32vector
-          list->f64vector list->s16vector list->s32vector list->s64vector
-          list->s8vector list->u16vector list->u32vector list->u64vector
-          list->u8vector
-
-          make-f16vector make-f32vector make-f64vector
-          make-s16vector make-s32vector make-s64vector make-s8vector
-          make-u16vector make-u32vector make-u64vector make-u8vector
-          make-uvector
-
-          open-output-uvector port->uvector read-block!
-          read-uvector read-uvector! referencer
-
-          s16vector s16vector->list
-          s16vector->vector s16vector-add s16vector-add! s16vector-and
-          s16vector-and! s16vector-append s16vector-clamp s16vector-clamp!
-          s16vector-compare s16vector-copy s16vector-copy! s16vector-dot
-          s16vector-fill! s16vector-ior s16vector-ior! s16vector-length
-          s16vector-mul s16vector-mul! s16vector-multi-copy!
-          s16vector-range-check s16vector-ref s16vector-set! s16vector-sub
-          s16vector-sub! s16vector-swap-bytes s16vector-swap-bytes!
-          s16vector-xor s16vector-xor! s16vector=? s16vector?
-
-          s32vector s32vector->list s32vector->string
-          s32vector->vector s32vector-add s32vector-add! s32vector-and
-          s32vector-and! s32vector-append s32vector-clamp s32vector-clamp!
-          s32vector-compare s32vector-copy s32vector-copy! s32vector-dot
-          s32vector-fill! s32vector-ior s32vector-ior! s32vector-length
-          s32vector-mul s32vector-mul! s32vector-multi-copy!
-          s32vector-range-check s32vector-ref s32vector-set! s32vector-sub
-          s32vector-sub! s32vector-swap-bytes s32vector-swap-bytes!
-          s32vector-xor s32vector-xor! s32vector=? s32vector?
-
-          s64vector s64vector->list
-          s64vector->vector s64vector-add s64vector-add! s64vector-and
-          s64vector-and! s64vector-append s64vector-clamp s64vector-clamp!
-          s64vector-compare s64vector-copy s64vector-copy! s64vector-dot
-          s64vector-fill! s64vector-ior s64vector-ior! s64vector-length
-          s64vector-mul s64vector-mul! s64vector-multi-copy!
-          s64vector-range-check s64vector-ref s64vector-set! s64vector-sub
-          s64vector-sub! s64vector-swap-bytes s64vector-swap-bytes!
-          s64vector-xor s64vector-xor! s64vector=? s64vector?
-
-          s8vector s8vector->list s8vector->string s8vector->vector
-          s8vector-add s8vector-add! s8vector-and s8vector-and! s8vector-append
-          s8vector-clamp s8vector-clamp! s8vector-compare s8vector-copy
-          s8vector-copy! s8vector-dot s8vector-fill! s8vector-ior s8vector-ior!
-          s8vector-length s8vector-mul s8vector-mul! s8vector-multi-copy!
-          s8vector-range-check s8vector-ref s8vector-set! s8vector-sub
-          s8vector-sub! s8vector-xor s8vector-xor! s8vector=? s8vector?
-
-          string->s32vector string->s32vector! string->s8vector
-          string->s8vector! string->u32vector string->u32vector!
+          string->s8vector string->s8vector!
           string->u8vector string->u8vector!
-
-          u16vector u16vector->list
-          u16vector->vector u16vector-add u16vector-add! u16vector-and
-          u16vector-and! u16vector-append u16vector-clamp u16vector-clamp!
-          u16vector-compare u16vector-copy u16vector-copy! u16vector-dot
-          u16vector-fill! u16vector-ior u16vector-ior! u16vector-length
-          u16vector-mul u16vector-mul! u16vector-multi-copy!
-          u16vector-range-check u16vector-ref u16vector-set! u16vector-sub
-          u16vector-sub! u16vector-swap-bytes u16vector-swap-bytes!
-          u16vector-xor u16vector-xor! u16vector=? u16vector?
-
-          u32vector u32vector->list u32vector->string
-          u32vector->vector u32vector-add u32vector-add! u32vector-and
-          u32vector-and! u32vector-append u32vector-clamp u32vector-clamp!
-          u32vector-compare u32vector-copy u32vector-copy! u32vector-dot
-          u32vector-fill! u32vector-ior u32vector-ior! u32vector-length
-          u32vector-mul u32vector-mul! u32vector-multi-copy!
-          u32vector-range-check u32vector-ref u32vector-set! u32vector-sub
-          u32vector-sub! u32vector-swap-bytes u32vector-swap-bytes!
-          u32vector-xor u32vector-xor! u32vector=? u32vector?
-
-          u64vector u64vector->list u64vector->vector u64vector-add
-          u64vector-add! u64vector-and u64vector-and! u64vector-append
-          u64vector-clamp u64vector-clamp! u64vector-compare u64vector-copy
-          u64vector-copy! u64vector-dot u64vector-fill! u64vector-ior
-          u64vector-ior! u64vector-length u64vector-mul u64vector-mul!
-          u64vector-multi-copy! u64vector-range-check u64vector-ref
-          u64vector-set! u64vector-sub u64vector-sub! u64vector-swap-bytes
-          u64vector-swap-bytes! u64vector-xor u64vector-xor! u64vector=?
-          u64vector?
-
-          u8vector u8vector->list u8vector->string u8vector->vector
-          u8vector-add u8vector-add! u8vector-and u8vector-and! u8vector-append
-          u8vector-clamp u8vector-clamp! u8vector-compare u8vector-copy
-          u8vector-copy! u8vector-dot u8vector-fill! u8vector-ior u8vector-ior!
-          u8vector-length u8vector-mul u8vector-mul! u8vector-multi-copy!
-          u8vector-range-check u8vector-ref u8vector-set! u8vector-sub
-          u8vector-sub! u8vector-xor u8vector-xor! u8vector=? u8vector?
+          string->s32vector string->s32vector! 
+          string->u32vector string->u32vector!
+          s8vector->string u8vector->string
+          s32vector->string u32vector->string
 
           uvector-alias uvector-binary-search uvector-class-element-size
           uvector-copy uvector-copy! uvector-ref uvector-set! uvector-size
-          uvector-swap-bytes uvector-swap-bytes!
+          uvector->list uvector->vector uvector-swap-bytes uvector-swap-bytes!
 
-          vector->f16vector vector->f32vector vector->f64vector
-          vector->s16vector vector->s32vector vector->s64vector
-          vector->s8vector vector->u16vector vector->u32vector
-          vector->u64vector vector->u8vector
+          write-block write-uvector write-bytevector
 
-          write-block write-uvector))
+          ;; R7RS compatibility (scheme base) and (scheme bytevector)
+          bytevector bytevector? make-bytevector bytevector-fill!
+          bytevector-length bytevector-u8-ref bytevector-u8-set!
+          bytevector-s8-ref bytevector-s8-set!
+          bytevector->u8-list u8-list->bytevector
+          bytevector-copy bytevector-copy! bytevector-copy!-r6
+          bytevector-append bytevector=?
+          ))
 (select-module gauche.uvector)
 
 ;; gauche.vport is used by port->uvector.  Technically it's on top
@@ -188,19 +81,20 @@
 ;; it is expected to belong gauche.uvector.
 (autoload gauche.vport open-output-uvector get-output-uvector)
 
+;; referenced from srfi-160 make-TAGvector-generator
+(autoload gauche.generator uvector->generator)
+
 (inline-stub
- "#include <math.h>"
- "#define EXTUVECTOR_EXPORTS"
- "#include \"gauche/uvector.h\""
- "#include \"gauche/priv/vectorP.h\""
- "#include \"uvectorP.h\""
- )
+ (declcode
+  (.include <math.h>)
+  (.define EXTUVECTOR_EXPORTS)
+  (.include "gauche/uvector.h")
+  (.include "gauche/priv/vectorP.h")
+  (.include "gauche/priv/bytesP.h")
+  (.include "uvectorP.h")))
 
 ;; uvlib.scm is generated by uvlib.scm.tmpl
-(inline-stub
- (include "./uvlib.scm")
- )
-
+(include "./uvlib.scm")
 
 ;;;
 ;;; Generic procedures
@@ -209,7 +103,7 @@
 ;; uvector-alias
 (inline-stub
  (define-cproc uvector-alias
-   (klass::<class> v::<uvector> :optional (start::<int> 0) (end::<int> -1))
+   (klass::<class> v::<uvector> :optional (start::<fixnum> 0) (end::<fixnum> -1))
    Scm_UVectorAlias)
  )
 
@@ -249,6 +143,78 @@
      (return r)))
  )
 
+;; uvector->list
+;; uvector->vector
+(inline-stub
+ (define-cproc uvector->list (v::<uvector> 
+                              :optional (start::<fixnum> 0)
+                                        (end::<fixnum> -1))
+   (case (Scm_UVectorType (Scm_ClassOf (SCM_OBJ v)))
+     [(SCM_UVECTOR_S8) (return (Scm_S8VectorToList (SCM_S8VECTOR v)
+                                                   start end))]
+     [(SCM_UVECTOR_U8) (return (Scm_U8VectorToList (SCM_U8VECTOR v)
+                                                   start end))]
+     [(SCM_UVECTOR_S16) (return (Scm_S16VectorToList (SCM_S16VECTOR v)
+                                                     start end))]
+     [(SCM_UVECTOR_U16) (return (Scm_U16VectorToList (SCM_U16VECTOR v)
+                                                     start end))]
+     [(SCM_UVECTOR_S32) (return (Scm_S32VectorToList (SCM_S32VECTOR v)
+                                                     start end))]
+     [(SCM_UVECTOR_U32) (return (Scm_U32VectorToList (SCM_U32VECTOR v)
+                                                     start end))]
+     [(SCM_UVECTOR_S64) (return (Scm_S64VectorToList (SCM_S64VECTOR v)
+                                                     start end))]
+     [(SCM_UVECTOR_U64) (return (Scm_U64VectorToList (SCM_U64VECTOR v)
+                                                     start end))]
+     [(SCM_UVECTOR_F16) (return (Scm_F16VectorToList (SCM_F16VECTOR v)
+                                                     start end))]
+     [(SCM_UVECTOR_F32) (return (Scm_F32VectorToList (SCM_F32VECTOR v)
+                                                     start end))]
+     [(SCM_UVECTOR_F64) (return (Scm_F64VectorToList (SCM_F64VECTOR v)
+                                                     start end))]
+     [(SCM_UVECTOR_C32) (return (Scm_C32VectorToList (SCM_C32VECTOR v)
+                                                     start end))]
+     [(SCM_UVECTOR_C64) (return (Scm_C64VectorToList (SCM_C64VECTOR v)
+                                                     start end))]
+     [(SCM_UVECTOR_C128) (return (Scm_C128VectorToList (SCM_C128VECTOR v)
+                                                       start end))]
+     [else (Scm_Error "[internal] Invalid uvector type: %S" v)
+           (return SCM_UNDEFINED)]))
+ (define-cproc uvector->vector (v::<uvector> 
+                                :optional (start::<fixnum> 0)
+                                          (end::<fixnum> -1))
+   (case (Scm_UVectorType (Scm_ClassOf (SCM_OBJ v)))
+     [(SCM_UVECTOR_S8) (return (Scm_S8VectorToVector (SCM_S8VECTOR v)
+                                                     start end))]
+     [(SCM_UVECTOR_U8) (return (Scm_U8VectorToVector (SCM_U8VECTOR v)
+                                                     start end))]
+     [(SCM_UVECTOR_S16) (return (Scm_S16VectorToVector (SCM_S16VECTOR v)
+                                                       start end))]
+     [(SCM_UVECTOR_U16) (return (Scm_U16VectorToVector (SCM_U16VECTOR v)
+                                                       start end))]
+     [(SCM_UVECTOR_S32) (return (Scm_S32VectorToVector (SCM_S32VECTOR v)
+                                                       start end))]
+     [(SCM_UVECTOR_U32) (return (Scm_U32VectorToVector (SCM_U32VECTOR v)
+                                                       start end))]
+     [(SCM_UVECTOR_S64) (return (Scm_S64VectorToVector (SCM_S64VECTOR v)
+                                                       start end))]
+     [(SCM_UVECTOR_U64) (return (Scm_U64VectorToVector (SCM_U64VECTOR v)
+                                                       start end))]
+     [(SCM_UVECTOR_F16) (return (Scm_F16VectorToVector (SCM_F16VECTOR v)
+                                                       start end))]
+     [(SCM_UVECTOR_F32) (return (Scm_F32VectorToVector (SCM_F32VECTOR v)
+                                                       start end))]
+     [(SCM_UVECTOR_F64) (return (Scm_F64VectorToVector (SCM_F64VECTOR v)
+                                                       start end))]
+     [(SCM_UVECTOR_C32) (return (Scm_C32VectorToVector (SCM_C32VECTOR v)
+                                                       start end))]
+     [(SCM_UVECTOR_C64) (return (Scm_C64VectorToVector (SCM_C64VECTOR v)
+                                                       start end))]
+     [(SCM_UVECTOR_C128) (return (Scm_C128VectorToVector (SCM_C128VECTOR v)
+                                                         start end))]
+     [else (Scm_Error "[internal] Invalid uvector type: %S" v)
+           (return SCM_UNDEFINED)])))   
+
 ;; allocation by class
 (inline-stub
  (define-cproc make-uvector (klass::<class> size::<fixnum>
@@ -300,7 +266,14 @@
         (Scm_F64VectorFill (SCM_F64VECTOR v)
                            (Scm_GetDouble init)
                            0 -1)]
-       [else SCM_UNDEFINED]) ; can't happen
+       [(SCM_UVECTOR_C32)
+        (Scm_C32VectorFill (SCM_C32VECTOR v) (Scm_GetHalfComplex init) 0 -1)]
+       [(SCM_UVECTOR_C64)
+        (Scm_C64VectorFill (SCM_C64VECTOR v) (Scm_GetFloatComplex init) 0 -1)]
+       [(SCM_UVECTOR_C128)
+        (Scm_C128VectorFill (SCM_C128VECTOR v) 
+                            (Scm_GetDoubleComplex init) 0 -1)]
+       [else (Scm_Error "[internal] Invalid uvector class: %S" klass)])
      (return v))))
 
 ;; generic copy
@@ -308,15 +281,7 @@
  (define-cproc uvector-copy (v::<uvector>
                              :optional (start::<fixnum> 0)
                                        (end::<fixnum> -1))
-   (let* ([len::int (SCM_UVECTOR_SIZE v)]
-          [klass::ScmClass* (Scm_ClassOf (SCM_OBJ v))]
-          [eltsize::int (Scm_UVectorElementSize klass)]
-          [src::(const char *) (cast (const char *) (SCM_UVECTOR_ELEMENTS v))])
-     (SCM_CHECK_START_END start end len)
-     (let* ([newsize::int (* (- end start) eltsize)]
-            [dst::char* (SCM_NEW_ATOMIC_ARRAY (char) newsize)])
-       (memcpy dst (+ src (* start eltsize)) newsize)
-       (return (Scm_MakeUVector klass (- end start) dst)))))
+   (return (Scm_UVectorCopy v start end)))
  )
 
 ;; search
@@ -339,7 +304,7 @@
                                                    (end   #f)
                                                    (skip  #f)
                                                    (rounding #f))
-   (let* ([len::size_t (SCM_UVECTOR_SIZE v)]
+   (let* ([len::ScmSize (SCM_UVECTOR_SIZE v)]
           [s::ScmSmallInt (get-fixnum-arg start 0 "start")]
           [e::ScmSmallInt (get-fixnum-arg end -1 "end")]
           [p::ScmSmallInt (get-fixnum-arg skip 0 "skip")])
@@ -382,12 +347,12 @@
             (set! r (Scm_BinarySearchU32 (+ (SCM_U32VECTOR_ELEMENTS v) s)
                                          (- e s) k p (& lb) (& ub))))]
          [(SCM_UVECTOR_S64)
-          (let* ([k::ScmInt64
+          (let* ([k::int64_t
                   (Scm_GetInteger64Clamp key SCM_CLAMP_ERROR NULL)])
             (set! r (Scm_BinarySearchS64 (+ (SCM_S64VECTOR_ELEMENTS v) s)
                                          (- e s) k p (& lb) (& ub))))]
          [(SCM_UVECTOR_U64)
-          (let* ([k::ScmUInt64
+          (let* ([k::uint64_t
                   (Scm_GetIntegerU64Clamp key SCM_CLAMP_ERROR NULL)])
             (set! r (Scm_BinarySearchU64 (+ (SCM_U64VECTOR_ELEMENTS v) s)
                                          (- e s) k p (& lb) (& ub))))]
@@ -403,7 +368,19 @@
           (let* ([k::double (Scm_GetDouble key)])
             (set! r (Scm_BinarySearchF64 (+ (SCM_F64VECTOR_ELEMENTS v) s)
                                          (- e s) k p (& lb) (& ub))))]
-         [else (SCM_ASSERT "Invalid uvector type")]
+         [(SCM_UVECTOR_C32)
+          (let* ([k::ScmHalfComplex (Scm_GetHalfComplex key)])
+            (set! r (Scm_BinarySearchC32 (+ (SCM_C32VECTOR_ELEMENTS v) s)
+                                         (- e s) k p (& lb) (& ub))))]         
+         [(SCM_UVECTOR_C64)
+          (let* ([k::(ScmFloatComplex) (Scm_GetFloatComplex key)])
+            (set! r (Scm_BinarySearchC64 (+ (SCM_C64VECTOR_ELEMENTS v) s)
+                                         (- e s) k p (& lb) (& ub))))]         
+         [(SCM_UVECTOR_C128)
+          (let* ([k::(ScmDoubleComplex) (Scm_GetDoubleComplex key)])
+            (set! r (Scm_BinarySearchC128 (+ (SCM_C128VECTOR_ELEMENTS v) s)
+                                          (- e s) k p (& lb) (& ub))))] 
+         [else (Scm_Error "[internal] Invalid uvector type: %S" v)]
          )
        (when (== r (cast (size_t) -1))
          (cond
@@ -437,7 +414,7 @@
        (return r)
        (begin
          (SCM_ASSERT (SCM_INTP r))
-         (let* ([n::long (SCM_INT_VALUE r)])
+         (let* ([n::ScmSmallInt (SCM_INT_VALUE r)])
            (SCM_ASSERT (and (<= n size) (<= 0 n)))
            ;; NB: If read size is a lot shorter than requested size, we may
            ;; want to copy it instead of just keeping the rest of vector
@@ -454,19 +431,33 @@
    Scm_WriteBlock)
  )
 
+;; R7RS bytevectors
+(define (read-bytevector k :optional (port (current-input-port)))
+  (read-uvector <u8vector> k port))
+(define (read-bytevector! bv :optional (port (current-input-port))
+                          (start 0)
+                          (end (uvector-length bv)))
+  (assume-type bv <u8vector>)
+  (read-uvector! bv port start end))
+(define (write-bytevector bv :optional (port (current-output-port))
+                                       (start 0)
+                                       (end (uvector-length bv)))
+  (assume-type bv <u8vector>)
+  (write-uvector bv port start end))
+
 ;; copy
 (inline-stub
  (define-cproc uvector-copy! (dest::<uvector> dstart::<int> src::<uvector>
-                              :optional (sstart::<int> 0)
-                                        (send::<int> -1))
+                              :optional (sstart::<fixnum> 0)
+                                        (send::<fixnum> -1))
    ::<void>
    (SCM_UVECTOR_CHECK_MUTABLE dest)
    (SCM_CHECK_START_END sstart send (SCM_UVECTOR_SIZE src))
    (let* ([deltsize::int (Scm_UVectorElementSize (Scm_ClassOf (SCM_OBJ dest)))]
-          [doff::int (* dstart deltsize)]
+          [doff::ScmSmallInt (* dstart deltsize)]
           [seltsize::int (Scm_UVectorElementSize (Scm_ClassOf (SCM_OBJ src)))]
-          [soff::int (* sstart seltsize)]
-          [size::int (- (* send seltsize) soff)])
+          [soff::ScmSmallInt (* sstart seltsize)]
+          [size::ScmSmallInt (- (* send seltsize) soff)])
      (memmove (+ (cast char* (SCM_UVECTOR_ELEMENTS dest)) doff)
               (+ (cast (const char*) (SCM_UVECTOR_ELEMENTS src)) soff)
               size)))
@@ -481,8 +472,8 @@
    [(_ (s start end sp ep) . body)
     (let ([sb (gensym)] [size (gensym)] [len (gensym)] [ss (gensym)])
       `(let* ([,sb :: (const ScmStringBody*) (SCM_STRING_BODY ,s)]
-              [,size :: u_int (SCM_STRING_BODY_SIZE ,sb)]
-              [,len :: u_int (SCM_STRING_BODY_LENGTH ,sb)]
+              [,size :: ScmSize (SCM_STRING_BODY_SIZE ,sb)]
+              [,len :: ScmSize (SCM_STRING_BODY_LENGTH ,sb)]
               [,ss :: (const char*) (SCM_STRING_BODY_START ,sb)])
          (SCM_CHECK_START_END ,start ,end (cast int ,len))
          (let* ([,sp :: (const char*)
@@ -495,8 +486,12 @@
                          (Scm_StringBodyPosition ,sb ,end))])
            ,@body)))])
  
- (define-cfn string->bytevector
-   (klass::ScmClass* s::ScmString* start::int end::int immutable::int) :static
+ (define-cfn string->bytevector (klass::ScmClass* 
+                                 s::ScmString* 
+                                 start::ScmSmallInt
+                                 end::ScmSmallInt
+                                 immutable::int)
+   :static
    (with-input-string-pointers (s start end sp ep)
      (let* ([buf::char* NULL])
        (if immutable
@@ -504,7 +499,7 @@
          (begin
            (set! buf (SCM_NEW_ATOMIC2 (char*) (- ep sp)))
            (memcpy buf sp (- ep sp))))
-       (return (Scm_MakeUVectorFull klass (cast int (- ep sp)) buf
+       (return (Scm_MakeUVectorFull klass (cast ScmSmallInt (- ep sp)) buf
                                     immutable NULL)))))
 
  (define-cproc string->s8vector
@@ -517,9 +512,13 @@
     :optional (start::<fixnum> 0) (end::<fixnum> -1) (immutable?::<boolean> #f))
    (return (string->bytevector SCM_CLASS_U8VECTOR s start end immutable?)))
 
- (define-cfn string->bytevector!
-   (v::ScmUVector* tstart::int s::ScmString* start::int end::int) :static
-   (let* ([tlen::int (SCM_UVECTOR_SIZE v)])
+ (define-cfn string->bytevector! (v::ScmUVector* 
+                                  tstart::ScmSmallInt
+                                  s::ScmString*
+                                  start::ScmSmallInt 
+                                  end::ScmSmallInt)
+   :static
+   (let* ([tlen::ScmSmallInt (SCM_UVECTOR_SIZE v)])
      (when (and (>= tstart 0) (< tstart tlen))
        (SCM_UVECTOR_CHECK_MUTABLE v)
        (with-input-string-pointers (s start end sp ep)
@@ -530,22 +529,25 @@
      (return (SCM_OBJ v))))
 
  (define-cproc string->s8vector! (v::<s8vector>
-                                  tstart::<int>
+                                  tstart::<fixnum>
                                   s::<string>
                                   :optional (start::<fixnum> 0)
                                   (end::<fixnum> -1))
    (return (string->bytevector! (SCM_UVECTOR v) tstart s start end)))
 
  (define-cproc string->u8vector! (v::<u8vector>
-                                  tstart::<int>
+                                  tstart::<fixnum>
                                   s::<string>
                                   :optional (start::<fixnum> 0)
                                   (end::<fixnum> -1))
    (return (string->bytevector! (SCM_UVECTOR v) tstart s start end)))
 
- (define-cfn bytevector->string (v::ScmUVector* start::int end::int term)
+ (define-cfn bytevector->string (v::ScmUVector* 
+                                 start::ScmSmallInt 
+                                 end::ScmSmallInt
+                                 term)
    :static
-   (let* ([len::int (SCM_UVECTOR_SIZE v)])
+   (let* ([len::ScmSmallInt (SCM_UVECTOR_SIZE v)])
      ;; We automatically avoid copying the string contents when the
      ;; following conditions are met:
      ;; * The source vector is immutable
@@ -568,8 +570,8 @@
                             0
                             SCM_STRING_COPYING)])
        (when (SCM_INTP term)
-         (let* ([terminator::u_int (logand #xff (SCM_INT_VALUE term))]
-                [i::int])
+         (let* ([terminator::u_char (logand #xff (SCM_INT_VALUE term))]
+                [i::ScmSmallInt])
            (for [(set! i start) (< i end) (post++ i)]
              (when (== terminator
                        (aref (cast u_char* (SCM_UVECTOR_ELEMENTS v)) i))
@@ -590,41 +592,66 @@
                                            (terminator #f))
    (return (bytevector->string (SCM_UVECTOR v) start end terminator)))
 
- (define-cfn string->wordvector
-   (klass::ScmClass* s::ScmString* start::int end::int) :static
+ (define-cfn string->wordvector (klass::ScmClass* s::ScmString*
+                                 start::ScmSmallInt end::ScmSmallInt
+                                 endian::ScmObj)
+   :static
+   (unless (SCM_SYMBOLP endian)
+     (set! endian (Scm_DefaultEndian)))
    (with-input-string-pointers (s start end sp ep)
      (let* ([v (Scm_MakeUVector klass (- end start) NULL)]
-            [eltp::int32_t* (cast int32_t* (SCM_UVECTOR_ELEMENTS v))]
-            [i::int 0])
+            [eltp::uint32_t* (cast uint32_t* (SCM_UVECTOR_ELEMENTS v))]
+            [i::ScmSmallInt 0]
+            [do_swap::int (SWAP_REQUIRED endian)])
        (for [() (< sp ep) (post++ i)]
          (let* ([ch::ScmChar])
            (SCM_CHAR_GET sp ch)
-           (set! (aref eltp i) ch)
+           (if do_swap
+             (let* ([v::swap_u32_t])
+               (set! (ref v val) (cast uint32_t ch))
+               (SWAP_4 v)
+               (set! (aref eltp i) (ref v val)))
+             (set! (aref eltp i) (cast uint32_t ch)))
            (+= sp (SCM_CHAR_NBYTES ch))))
        (return v))))
 
  (define-cproc string->s32vector (s::<string>
                                   :optional (start::<fixnum> 0)
-                                  (end::<fixnum> -1))
-   (return (string->wordvector SCM_CLASS_S32VECTOR s start end)))
+                                            (end::<fixnum> -1)
+                                            endian)
+   (return (string->wordvector SCM_CLASS_S32VECTOR s start end endian)))
 
  (define-cproc string->u32vector (s::<string>
                                   :optional (start::<fixnum> 0)
-                                  (end::<fixnum> -1))
-   (return (string->wordvector SCM_CLASS_U32VECTOR s start end)))
+                                            (end::<fixnum> -1)
+                                            endian)
+   (return (string->wordvector SCM_CLASS_U32VECTOR s start end endian)))
 
- (define-cfn string->wordvector!
-   (v::ScmUVector* tstart::int s::ScmString* start::int end::int) :static
-   (let* ([tlen::int (SCM_UVECTOR_SIZE v)])
+ (define-cfn string->wordvector! (v::ScmUVector*
+                                  tstart::ScmSmallInt
+                                  s::ScmString*
+                                  start::ScmSmallInt
+                                  end::ScmSmallInt
+                                  endian::ScmObj)
+   :static
+   (unless (SCM_SYMBOLP endian)
+     (set! endian (Scm_DefaultEndian)))
+   (let* ([tlen::ScmSmallInt (SCM_UVECTOR_SIZE v)])
      (when (and (>= tstart 0) (< tstart tlen))
        (SCM_UVECTOR_CHECK_MUTABLE v)
        (with-input-string-pointers (s start end sp ep)
-         (let* ([buf::int32_t* (cast int32_t* (SCM_UVECTOR_ELEMENTS v))]
-                [i::int tstart])
+         (let* ([buf::uint32_t* (cast uint32_t* (SCM_UVECTOR_ELEMENTS v))]
+                [i::ScmSmallInt tstart]
+                [do_swap::int (SWAP_REQUIRED endian)])
            (for [() (and (< sp ep) (< i tlen)) (post++ i)]
              (let* ([ch::ScmChar])
                (SCM_CHAR_GET sp ch)
-               (set! (aref buf i) ch)
+               (if do_swap
+                 (let* ([v::swap_u32_t])
+                   (set! (ref v val) (cast uint32_t ch))
+                   (SWAP_4 v)
+                   (set! (aref buf i) (ref v val)))
+                 (set! (aref buf i) (cast uint32_t ch)))
                (+= sp (SCM_CHAR_NBYTES ch)))))))
      (return (SCM_OBJ v))))
 
@@ -632,24 +659,40 @@
                                    tstart::<fixnum>
                                    s::<string>
                                    :optional (start::<fixnum> 0)
-                                             (end::<fixnum> -1))
-   (return (string->wordvector! (SCM_UVECTOR v) tstart s start end)))
+                                             (end::<fixnum> -1)
+                                             endian)
+   (return (string->wordvector! (SCM_UVECTOR v) tstart s start end endian)))
 
  (define-cproc string->u32vector! (v::<u32vector>
                                    tstart::<fixnum>
                                    s::<string>
                                    :optional (start::<fixnum> 0)
-                                             (end::<fixnum> -1))
-   (return (string->wordvector! (SCM_UVECTOR v) tstart s start end)))
+                                             (end::<fixnum> -1)
+                                             endian)
+   (return (string->wordvector! (SCM_UVECTOR v) tstart s start end endian)))
 
- (define-cfn wordvector->string (v::ScmUVector* start::int end::int term)
+ (define-cfn wordvector->string (v::ScmUVector* 
+                                 start::ScmSmallInt
+                                 end::ScmSmallInt
+                                 term
+                                 endian)
    :static
-   (let* ([len::int (SCM_UVECTOR_SIZE v)]
-          [s (Scm_MakeOutputStringPort FALSE)])
+   (unless (SCM_SYMBOLP endian)
+     (set! endian (Scm_DefaultEndian)))
+   (let* ([len::ScmSmallInt (SCM_UVECTOR_SIZE v)]
+          [s (Scm_MakeOutputStringPort FALSE)]
+          [do_swap::int (SWAP_REQUIRED endian)])
      (SCM_CHECK_START_END start end len)
      (let* ([eltp::int32_t* (cast int32_t* (SCM_UVECTOR_ELEMENTS v))])
        (while (< start end)
-         (let* ([ch::ScmChar (cast ScmChar (aref eltp (post++ start)))])
+         (let* ([ch::ScmChar]
+                [val::uint32_t (aref eltp (post++ start))])
+           (if do_swap
+             (let* ([v::swap_u32_t])
+               (set! (ref v val) val)
+               (SWAP_4 v)
+               (set! ch (cast ScmChar (ref v val))))
+             (set! ch (cast ScmChar val)))
            (when (and (SCM_INTP term)
                       (== (SCM_INT_VALUE term) ch))
              (break))
@@ -659,14 +702,16 @@
  (define-cproc s32vector->string (v::<s32vector>
                                   :optional (start::<fixnum> 0)
                                             (end::<fixnum> -1)
-                                            (terminator #f))
-   (return (wordvector->string (SCM_UVECTOR v) start end terminator)))
+                                            (terminator #f)
+                                            endian)
+   (return (wordvector->string (SCM_UVECTOR v) start end terminator endian)))
 
  (define-cproc u32vector->string (v::<u32vector>
                                   :optional (start::<fixnum> 0)
                                             (end::<fixnum> -1)
-                                            (terminator #f))
-   (return (wordvector->string (SCM_UVECTOR v) start end terminator)))
+                                            (terminator #f)
+                                            endian)
+   (return (wordvector->string (SCM_UVECTOR v) start end terminator endian)))
  )
 
 ;; for the bakcward compatibility
@@ -683,90 +728,61 @@
     (get-output-uvector p :shared #t)))
 
 ;;-------------------------------------------------------------
-;; Appending vectors
+;; Element range check (srfi-160)
 ;;
 
-(define-macro (define-appender tag)
-  (let ([app   (string->symbol #"~|tag|vector-append")]
-        [len   (string->symbol #"~|tag|vector-length")]
-        [make  (string->symbol #"make-~|tag|vector")]
-        [copy! (string->symbol #"~|tag|vector-copy!")])
-    `(define (,app . vs)
-       (let* ([size (apply + (map ,len vs))]
-              [dest (,make size)])
-         (do ([vs vs (cdr vs)]
-              [k  0  (+ k (,len (car vs)))])
-             [(null? vs) dest]
-           (,copy! dest k (car vs)))))
-    ))
+(define-cproc u8? (v) ::<boolean>
+  (return (and (SCM_INTP v)
+               (<= 0 (SCM_INT_VALUE v))
+               (<  (SCM_INT_VALUE v) 256))))
 
-(define-appender s8)
-(define-appender u8)
-(define-appender s16)
-(define-appender u16)
-(define-appender s32)
-(define-appender u32)
-(define-appender s64)
-(define-appender u64)
-(define-appender f16)
-(define-appender f32)
-(define-appender f64)
+(define-cproc s8? (v) ::<boolean>
+  (return (and (SCM_INTP v)
+               (<= -128 (SCM_INT_VALUE v))
+               (<  (SCM_INT_VALUE v) 128))))
+
+(define-cproc u16? (v) ::<boolean>
+  (return (and (SCM_INTP v)
+               (<= 0 (SCM_INT_VALUE v))
+               (<  (SCM_INT_VALUE v) 65536))))
+
+(define-cproc s16? (v) ::<boolean>
+  (return (and (SCM_INTP v)
+               (<= -32768 (SCM_INT_VALUE v))
+               (<  (SCM_INT_VALUE v) 32768))))
+
+(define-cproc u32? (v) ::<boolean>
+  (let* ([oor::int])
+    (cast void (Scm_GetIntegerU32Clamp v SCM_CLAMP_NONE (& oor)))
+    (return oor)))
+
+(define-cproc s32? (v) ::<boolean>
+  (let* ([oor::int])
+    (cast void (Scm_GetInteger32Clamp v SCM_CLAMP_NONE (& oor)))
+    (return oor)))
+
+(define-cproc u64? (v) ::<boolean>
+  (let* ([oor::int])
+    (cast void (Scm_GetIntegerU64Clamp v SCM_CLAMP_NONE (& oor)))
+    (return oor)))
+
+(define-cproc s64? (v) ::<boolean>
+  (let* ([oor::int])
+    (cast void (Scm_GetInteger64Clamp v SCM_CLAMP_NONE (& oor)))
+    (return oor)))
+
+(define-cproc f16? (v) ::<boolean> :fast-flonum (return (SCM_REALP v)))
+(define-cproc f32? (v) ::<boolean> :fast-flonum (return (SCM_REALP v)))
+(define-cproc f64? (v) ::<boolean> :fast-flonum (return (SCM_REALP v)))
+
+(define-cproc c32? (v) ::<boolean> :fast-flonum (return (SCM_NUMBERP v)))
+(define-cproc c64? (v) ::<boolean> :fast-flonum (return (SCM_NUMBERP v)))
+(define-cproc c128? (v) ::<boolean> :fast-flonum (return (SCM_NUMBERP v)))
 
 ;;-------------------------------------------------------------
-;; Sequence protocol implementation
+;; special coercers (most sequence methods are in uvlib.scm.tmpl
 ;;
 
-(define-macro (%define-srfi-4-collection-interface tag)
-  (let* ([tagvector (string->symbol #"~|tag|vector")]
-         [class     (string->symbol #"<~|tagvector|>")]
-         [meta      (string->symbol #"<~|tagvector|-meta>")]
-         [len       (string->symbol #"~|tagvector|-length")]
-         [ref       (string->symbol #"~|tagvector|-ref")]
-         [set       (string->symbol #"~|tagvector|-set!")]
-         [copy      (string->symbol #"~|tagvector|-copy")]
-         [->list    (string->symbol #"~|tagvector|->list")]
-         [list->    (string->symbol #"list->~|tagvector|")]
-         [->vec     (string->symbol #"~|tagvector|->vector")]
-         [vec->     (string->symbol #"vector->~|tagvector|")]
-         [make      (string->symbol #"make-~|tagvector|")])
-    `(begin
-       (define-method call-with-iterator ((v ,class) proc :key (start #f))
-         (let* ([len (,len v)] [i (or start 0)])
-           (proc (^[] (>= i len))
-                 (^[] (rlet1 r (,ref v i) (inc! i))))))
-       (define-method call-with-builder ((c ,meta) proc :key (size #f))
-         (if size
-           (let ([v (,make size)] [i 0])
-             (proc (^[item] (,set v i item) (inc! i))
-                   (^[] v)))
-           (let1 q (make-queue)
-             (proc (^[item] (enqueue! q item))
-                   (^[] (,list-> (dequeue-all! q)))))
-           ))
-       (define-method referencer ((v ,class)) ,ref)
-       (define-method modifier   ((v ,class)) ,set)
-       (define-method size-of ((v ,class)) (,len v))
-       (define-method coerce-to ((c <list-meta>) (v ,class)) (,->list v))
-       (define-method coerce-to ((c ,meta) (v <list>)) (,list-> v))
-       (define-method coerce-to ((c <vector-meta>) (v ,class)) (,->vec v))
-       (define-method coerce-to ((c ,meta) (v <vector>)) (,vec-> v))
-       (define-method coerce-to ((c ,meta) (v ,class)) (,copy v))
-       (define-method subseq ((v ,class) . args) (apply ,copy v args))
-       )))
-
-(%define-srfi-4-collection-interface s8)
-(%define-srfi-4-collection-interface u8)
-(%define-srfi-4-collection-interface s16)
-(%define-srfi-4-collection-interface u16)
-(%define-srfi-4-collection-interface s32)
-(%define-srfi-4-collection-interface u32)
-(%define-srfi-4-collection-interface s64)
-(%define-srfi-4-collection-interface u64)
-(%define-srfi-4-collection-interface f16)
-(%define-srfi-4-collection-interface f32)
-(%define-srfi-4-collection-interface f64)
-
-;; some special cases
 (define-method coerce-to ((dst <string-meta>) (src <u8vector>))
   (u8vector->string src))
 (define-method coerce-to ((dst <string-meta>) (src <s8vector>))
@@ -784,3 +800,42 @@
 (define-method coerce-to ((dst <s32vector-meta>) (src <string>))
   (string->s32vector src))
 
+;;-------------------------------------------------------------
+;; Bytevector aliases (R7RS compatibility)
+;;
+
+(define (%adjust-fill-arg fill)
+  (cond [(<= 0 fill 255) fill]
+        [(<= -128 fill -1) (logand fill #xff)]
+        [else (error "fill argument out of range" fill)]))
+
+(define (make-bytevector len :optional (fill 0))
+  (make-u8vector len (%adjust-fill-arg fill)))
+(define (bytevector-fill! v fill)       ; scheme.bytevector
+  (u8vector-fill! v (%adjust-fill-arg fill)))
+
+(define-inline bytevector         u8vector)
+(define-inline bytevector?        u8vector?)
+(define-inline bytevector-length  u8vector-length)
+(define-inline bytevector-u8-ref  u8vector-ref)
+(define-inline bytevector-u8-set! u8vector-set!)
+(define-inline bytevector-copy    u8vector-copy)
+(define-inline bytevector-copy!   u8vector-copy!)
+(define-inline bytevector-append  u8vector-append)
+(define-inline bytevector=?       u8vector=?)
+
+(define (bytevector-s8-set! v k b)      ; scheme.bytevector
+  (bytevector-u8-set! v k (logand b #xff)))
+(define bytevector-s8-ref         ; scheme.bytevector
+  (letrec ([bytevector-s8-ref 
+            (^[v k]
+              (let1 b (bytevector-u8-ref v k)
+                (if (>= b 128) (- b 256) b)))])
+    (getter-with-setter bytevector-s8-ref bytevector-s8-set!)))
+   
+(define (bytevector-copy!-r6 src sstart target tstart len) ; scheme.bytevector
+  (u8vector-copy! target tstart src sstart (+ sstart len)))
+
+(define (bytevector->u8-list v) (u8vector->list v))     ; scheme.bytevector
+(define (u8-list->bytevector lis) (list->u8vector lis)) ; scheme.bytevector
+
