@@ -1,5 +1,5 @@
 ;;
-;; Generates uvect.c from uvect.c.tmpl
+;; Generates uvector.c, uvector.h and uvlib.scm the corresponding *.tmpl file
 ;;
 
 (use srfi-13)
@@ -54,16 +54,14 @@
              (p "\"" (regexp-replace-all #/[\\\"]/ line "\\\\\\0") "\"")
              (loop (read-line))])))
 
-  (receive (out name)
-      (sys-mkstemp (build-path (temporary-directory) (sys-basename tmpl-file)))
-    (unwind-protect
-     (begin
-       (with-output-to-port out
-         (cut with-input-from-file tmpl-file
-              (cut translate)))
-       (close-output-port out)
-       (call-with-input-file name load-from-port))
-     (sys-unlink name))))
+  (call-with-temporary-file
+   (^[out name]
+     (with-output-to-port out
+       (cut with-input-from-file tmpl-file
+            (cut translate)))
+     (close-output-port out)
+     (call-with-input-file name load-from-port))
+   :prefix (sys-basename tmpl-file)))
 
 (define (getval rules key)
   (cond [(assq key rules) => cadr] [else #f]))
@@ -72,7 +70,7 @@
 ;; rules :: ((<string> <string-or-proc>) ...)
 (define (substitute line rules)
   (define (sub key . args)
-    (cond [(getval rules key)
+    (cond [(any (cut getval rules <>) (cons key (compatible-keys key)))
            => (^v (if (string? v)
                     (display v)
                     (display (apply v args))))]
@@ -89,6 +87,40 @@
                   (write-char ch)
                   (loop (read-char))])))))
 
+(define (compatible-keys key)
+  (case key
+    [(ntype) '(etype)]
+    [(NUNBOX) '(UNBOX)]
+    [(VMBOX) '(BOX)]
+    [(NBOX) '(BOX)]
+    [(VMNBOX) '(NBOX BOX)]
+    [else '()]))
+
+;; keys ------------------------------------------------------------
+;;
+;;  t           : lowercase tag, e.g. "s16"
+;;  T           : uppercase tag, e.g. "S16"
+;;  etype       : element C type, e.g. int16_t for s16vector
+;;  ntype       : natural C type to operate on each element. (optional)
+;;                e.g. long for s16vector
+;;  ZERO        : 'zero' value of etype (optional)
+;;  (REF_NTYPE v i) 
+;;              : Reference v[i] as ntype
+;;  (CAST_N2E exp)
+;;              : Cast etype's expression exp to ntype (optional)
+;;  (UNBOX dst src clamp)
+;;              : etype dst = unbox(ScmObj src, clamp)
+;;  (NUNBOX dst src clamp)
+;;              : ntype dst = unbox(ScmObj src, clamp)  (optional)
+;;  (BOX dst src)
+;;              : ScmObj dst = box(etype src)
+;;  (NBOX dst src)
+;;              : ScmObj dst = box(ntype src)   (optional)
+;;  (VMBOX dst src)
+;;              : ScmObj dst = box(etype src) // dst is VM register (optional)
+;;  (NVMBOX dst src)
+;;              : ScmObj dst = box(ntype src) // dst is VM register (optional)
+
 ;; substitution rules ----------------------------------------------
 
 (define (make-integer-rules)
@@ -106,15 +138,25 @@
         (make-f32rules)
         (make-f64rules)))
 
-(define (make-rules)
+(define (make-complex-rules)
+  (list (make-c32rules)
+        (make-c64rules)
+        (make-c128rules)))
+
+(define (make-scalar-rules)
   (append (make-integer-rules) (make-flonum-rules)))
 
+(define (make-rules)
+  (append (make-integer-rules) (make-flonum-rules) (make-complex-rules)))
+
 ;; common stuff
 (define (common-rules tag)
   (let* ([stag  (symbol->string tag)]
          [STAG  (string-upcase stag)])
     `((t   ,stag)
-      (T   ,STAG))))
+      (T   ,STAG)
+      (CAST_N2E ,identity)
+      (ZERO  "0"))))
 
 (define (make-s8rules)
   `((etype     "int8_t")
@@ -126,11 +168,7 @@
     (NUNBOX    ,(^[dst src clamp]
                   #"~dst = Scm_GetInteger8Clamp(~src, ~clamp, NULL)"))
     (BOX       ,(^[dst src] #"~dst = SCM_MAKE_INT(~src)"))
-    (VMBOX     ,(^[dst src] #"~dst = SCM_MAKE_INT(~src)"))
     (NBOX      ,(^[dst src] #"~dst = Scm_MakeInteger(~src)"))
-    (VMNBOX    ,(^[dst src] #"~dst = Scm_MakeInteger(~src)"))
-    (EQ        ,(^[x y]     #"(~x == ~y)"))
-    (PRINT     ,(^[out elt] #"Scm_Printf(~out, \"%d\", ~elt)"))
     ,@(common-rules 's8)))
 
 (define (make-u8rules)
@@ -143,11 +181,7 @@
     (NUNBOX    ,(^[dst src clamp]
                   #"~dst = Scm_GetIntegerU8Clamp(~src, ~clamp, NULL)"))
     (BOX       ,(^[dst src] #"~dst = SCM_MAKE_INT(~src)"))
-    (VMBOX     ,(^[dst src] #"~dst = SCM_MAKE_INT(~src)"))
     (NBOX      ,(^[dst src] #"~dst = Scm_MakeIntegerU(~src)"))
-    (VMNBOX    ,(^[dst src] #"~dst = Scm_MakeIntegerU(~src)"))
-    (EQ        ,(^[x y]     #"(~x == ~y)"))
-    (PRINT     ,(^[out elt] #"Scm_Printf(~out, \"%d\", ~elt)"))
     ,@(common-rules 'u8)))
 
 (define (make-s16rules)
@@ -160,11 +194,7 @@
     (NUNBOX    ,(^[dst src clamp]
                   #"~dst = Scm_GetInteger16Clamp(~src, ~clamp, NULL)"))
     (BOX       ,(^[dst src] #"~dst = SCM_MAKE_INT(~src)"))
-    (VMBOX     ,(^[dst src] #"~dst = SCM_MAKE_INT(~src)"))
     (NBOX      ,(^[dst src] #"~dst = Scm_MakeInteger(~src)"))
-    (VMNBOX    ,(^[dst src] #"~dst = Scm_MakeInteger(~src)"))
-    (EQ        ,(^[x y]     #"(~x == ~y)"))
-    (PRINT     ,(^[out elt] #"Scm_Printf(~out, \"%d\", ~elt)"))
     ,@(common-rules 's16)))
 
 (define (make-u16rules)
@@ -177,11 +207,7 @@
     (NUNBOX    ,(^[dst src clamp]
                   #"~dst = Scm_GetIntegerU16Clamp(~src, ~clamp, NULL)"))
     (BOX       ,(^[dst src] #"~dst = SCM_MAKE_INT(~src)"))
-    (VMBOX     ,(^[dst src] #"~dst = SCM_MAKE_INT(~src)"))
     (NBOX      ,(^[dst src] #"~dst = Scm_MakeIntegerU(~src)"))
-    (VMNBOX    ,(^[dst src] #"~dst = Scm_MakeIntegerU(~src)"))
-    (EQ        ,(^[x y]     #"(~x == ~y)"))
-    (PRINT     ,(^[out elt] #"Scm_Printf(~out, \"%d\", ~elt)"))
     ,@(common-rules 'u16)))
 
 (define (make-s32rules)
@@ -194,11 +220,6 @@
     (NUNBOX    ,(^[dst src clamp]
                   #"~dst = Scm_GetInteger32Clamp(~src, ~clamp, NULL)"))
     (BOX       ,(^[dst src] #"~dst = Scm_MakeInteger(~src)"))
-    (VMBOX     ,(^[dst src] #"~dst = Scm_MakeInteger(~src)"))
-    (NBOX      ,(^[dst src] #"~dst = Scm_MakeInteger(~src)"))
-    (VMNBOX    ,(^[dst src] #"~dst = Scm_MakeInteger(~src)"))
-    (EQ        ,(^[x y]     #"(~x == ~y)"))
-    (PRINT     ,(^[out elt] #"Scm_Printf(~out, \"%d\", ~elt)"))
     ,@(common-rules 's32)))
 
 (define (make-u32rules)
@@ -211,45 +232,22 @@
     (NUNBOX    ,(^[dst src clamp]
                   #"~dst = Scm_GetIntegerU32Clamp(~src, ~clamp, NULL)"))
     (BOX       ,(^[dst src] #"~dst = Scm_MakeIntegerU(~src)"))
-    (VMBOX     ,(^[dst src] #"~dst = Scm_MakeIntegerU(~src)"))
-    (NBOX      ,(^[dst src] #"~dst = Scm_MakeIntegerU(~src)"))
-    (VMNBOX    ,(^[dst src] #"~dst = Scm_MakeIntegerU(~src)"))
-    (EQ        ,(^[x y]     #"(~x == ~y)"))
-    (PRINT     ,(^[out elt] #"Scm_Printf(~out, \"%u\", ~elt)"))
     ,@(common-rules 'u32)))
 
 (define (make-s64rules)
-  `((etype     "ScmInt64")
-    (ntype     "ScmInt64")
+  `((etype     "int64_t")
     (REF_NTYPE ,(^[v i] #"SCM_S64VECTOR_ELEMENTS(~v)[~i]"))
-    (CAST_N2E  ,identity)
     (UNBOX     ,(^[dst src clamp]
                   #"~dst = Scm_GetInteger64Clamp(~src, ~clamp, NULL)"))
-    (NUNBOX    ,(^[dst src clamp]
-                  #"~dst = Scm_GetInteger64Clamp(~src, ~clamp, NULL)"))
     (BOX       ,(^[dst src] #"~dst = Scm_MakeInteger64(~src)"))
-    (VMBOX     ,(^[dst src] #"~dst = Scm_MakeInteger64(~src)"))
-    (NBOX      ,(^[dst src] #"~dst = Scm_MakeInteger64(~src)"))
-    (VMNBOX    ,(^[dst src] #"~dst = Scm_MakeInteger64(~src)"))
-    (EQ        ,(^[x y]     #"int64eqv(~x, ~y)"))
-    (PRINT     ,(^[out elt] #"int64print(~out, ~elt)"))
     ,@(common-rules 's64)))
 
 (define (make-u64rules)
-  `((etype     "ScmUInt64")
-    (ntype     "ScmUInt64")
+  `((etype     "uint64_t")
     (REF_NTYPE ,(^[v i] #"SCM_U64VECTOR_ELEMENTS(~v)[~i]"))
-    (CAST_N2E  ,identity)
     (UNBOX     ,(^[dst src clamp]
                   #"~dst = Scm_GetIntegerU64Clamp(~src, ~clamp, NULL)"))
-    (NUNBOX    ,(^[dst src clamp]
-                  #"~dst = Scm_GetIntegerU64Clamp(~src, ~clamp, NULL)"))
     (BOX       ,(^[dst src] #"~dst = Scm_MakeIntegerU64(~src)"))
-    (VMBOX     ,(^[dst src] #"~dst = Scm_MakeIntegerU64(~src)"))
-    (NBOX      ,(^[dst src] #"~dst = Scm_MakeIntegerU64(~src)"))
-    (VMNBOX    ,(^[dst src] #"~dst = Scm_MakeIntegerU64(~src)"))
-    (EQ        ,(^[x y]     #"uint64eqv(~x, ~y)"))
-    (PRINT     ,(^[out elt] #"uint64print(~out, ~elt)"))
     ,@(common-rules 'u64)))
 
 (define (make-f16rules)
@@ -265,8 +263,6 @@
     (VMBOX     ,(^[dst src] #"~dst = Scm_VMReturnFlonum(Scm_HalfToDouble(~src))"))
     (NBOX      ,(^[dst src] #"~dst = Scm_MakeFlonum(~src)"))
     (VMNBOX    ,(^[dst src] #"~dst = Scm_VMReturnFlonum(~src)"))
-    (EQ        ,(^[x y]     #"SCM_HALF_FLOAT_CMP(==, ~x, ~y)"))
-    (PRINT     ,(^[out elt] #"Scm_PrintDouble(~out, Scm_HalfToDouble(~elt), 0)"))
     ,@(common-rules 'f16)))
 
 (define (make-f32rules)
@@ -280,29 +276,42 @@
     (VMBOX     ,(^[dst src] #"~dst = Scm_VMReturnFlonum((double)~src)"))
     (NBOX      ,(^[dst src] #"~dst = Scm_MakeFlonum(~src)"))
     (VMNBOX    ,(^[dst src] #"~dst = Scm_VMReturnFlonum(~src)"))
-    (EQ        ,(^[x y]     #"(~x == ~y)"))
-    (PRINT     ,(^[out elt] #"Scm_PrintDouble(~out, (double)~elt, 0)"))
     ,@(common-rules 'f32)))
 
 (define (make-f64rules)
   `((etype     "double")
-    (ntype     "double")
     (REF_NTYPE ,(^[v i] #"SCM_F64VECTOR_ELEMENTS(~v)[~i]"))
-    (CAST_N2E  ,identity)
     (UNBOX     ,(^[dst src clamp] #"~dst = Scm_GetDouble(~src)"))
-    (NUNBOX    ,(^[dst src clamp] #"~dst = Scm_GetDouble(~src)"))
     (BOX       ,(^[dst src]       #"~dst = Scm_MakeFlonum(~src)"))
     (VMBOX     ,(^[dst src]       #"~dst = Scm_VMReturnFlonum(~src)"))
-    (NBOX      ,(^[dst src]       #"~dst = Scm_MakeFlonum(~src)"))
-    (VMNBOX    ,(^[dst src]       #"~dst = Scm_VMReturnFlonum(~src)"))
-    (EQ        ,(^[x y]           #"(~x == ~y)"))
-    (PRINT     ,(^[out elt]  #"Scm_PrintDouble(~out, (double)~elt, 0)"))
     ,@(common-rules 'f64)))
 
+(define (make-c32rules)
+  `((etype     "ScmHalfComplex")
+    (REF_NTYPE ,(^[v i] #"SCM_C32VECTOR_ELEMENTS(~v)[~i]"))
+    (UNBOX     ,(^[dst src clamp] #"~dst = Scm_GetHalfComplex(~src)"))
+    (BOX       ,(^[dst src]       #"~dst = Scm_HalfComplexToComplex(~src)"))
+    (ZERO      "(ScmHalfComplex){0, 0}")
+    ,@(common-rules 'c32)))
+
+(define (make-c64rules)
+  `((etype     "ScmFloatComplex")
+    (REF_NTYPE ,(^[v i] #"SCM_C64VECTOR_ELEMENTS(~v)[~i]"))
+    (UNBOX     ,(^[dst src clamp] #"~dst = Scm_GetFloatComplex(~src)"))
+    (BOX       ,(^[dst src]       #"~dst = Scm_FloatComplexToComplex(~src)"))
+    ,@(common-rules 'c64)))
+
+(define (make-c128rules)
+  `((etype     "ScmDoubleComplex")
+    (REF_NTYPE ,(^[v i] #"SCM_C128VECTOR_ELEMENTS(~v)[~i]"))
+    (UNBOX     ,(^[dst src clamp] #"~dst = Scm_GetDoubleComplex(~src)"))
+    (BOX       ,(^[dst src]       #"~dst = Scm_DoubleComplexToComplex(~src)"))
+    ,@(common-rules 'c128)))
+
 (define (dummy . _) "/* not implemented */")
 
 ;;===============================================================
-;; Uvector opertaion generator
+;; Uvector operation generator
 ;;
 
 (define (generate-numop)
@@ -316,7 +325,7 @@
             '("add" "sub" "mul")
             '("Add" "Sub" "Mul")
             '("Add" "Sub" "Mul"))
-  (dolist [rule (make-flonum-rules)]
+  (dolist [rule (append (make-flonum-rules) (make-complex-rules))]
     (for-each (cute substitute <> `((opname  "div")
                                     (Opname  "Div")
                                     (Sopname  "Div")
@@ -350,23 +359,13 @@
 
 (define (generate-dotop)
   (dolist [rule (make-rules)]
-    (let1 tag (string->symbol (getval rule 't))
-      (define (ZERO r)
-        (case tag
-          [(s64 u64) #"SCM_SET_INT64_ZERO(~r)"]
-          [else #"~r = 0"]))
-      (for-each (cute substitute <> `((ZERO  ,ZERO) ,@rule))
-                *tmpl-dotop*))))
+    (for-each (cute substitute <> rule) *tmpl-dotop*)))
 
 (define (generate-rangeop)
-  (dolist [rule (make-rules)]
+  (dolist [rule (make-scalar-rules)]
     (let ([tag (string->symbol (getval rule 't))]
           [TAG (getval rule 'T)]
           [cast (getval rule 'CAST_N2E)])
-      (define (ZERO r)
-        (case tag
-          [(s64 u64) #"SCM_SET_INT64_ZERO(~r)"]
-          [else #"~r = 0"]))
       (define (GETLIM r dc v)
         (let1 getter
             (case tag
@@ -401,7 +400,6 @@
                       "SCM_OBJ(x)")
                      )]
         (for-each (cute substitute <> `((GETLIM  ,GETLIM)
-                                        (ZERO  ,ZERO)
                                         (LT  ,LT)
                                         (opname   ,(ref ops 0))
                                         (Opname   ,(ref ops 1))
@@ -412,7 +410,7 @@
                   *tmpl-rangeop*)))))
 
 (define (generate-swapb)
-  (dolist [rule (make-rules)]
+  (dolist [rule (make-scalar-rules)]
     (let1 tag (string->symbol (getval rule 't))
       (define (SWAPB)
         (case tag
