@@ -172,6 +172,77 @@
              (display "xyz" p)
              (get-output-string o)))))
 
+;;-----------------------------------------------------------
+(test-section "virtual-io-port")
+
+(let ([data (make-vector 100)]
+      [pos 0]
+      [end 0])
+  (define p (make <virtual-io-port>
+              :getc (^[]
+                      (if (>= pos end)
+                        (eof-object)
+                        (begin0 (vector-ref data pos)
+                          (inc! pos))))
+              :putc (^c
+                     (vector-set! data pos c)
+                     (inc! pos)
+                     (set! end (max pos end)))
+              :seek (^[off whence]
+                      (cond
+                       [(= whence SEEK_SET) (set! pos off)]
+                       [(= whence SEEK_CUR) (inc! pos off)]
+                       [(= whence SEEK_END) (set! pos (+ end off))])
+                      pos)))
+  (vector-copy! data 0 '#(#\a #\b #\c #\d #\e))
+  (inc! end 5)
+  (test* "read from io port" "abcde" (port->string p))
+  (test* "port-tell" 5 (port-tell p))
+  (write-char #\X p)
+  (write-char #\Y p)
+  (write-char #\Z p)
+  (port-seek p 0)
+  (test* "write, then read" "abcdeXYZ" (port->string p))
+  (port-seek p -8 SEEK_END)
+  (write-string "PQR" p)
+  (port-seek p 2 SEEK_CUR)
+  (write-string "12345" p)
+  (port-seek p 0)
+  (test* "write, then read 2" "PQRde12345" (port->string p))
+  )
+
+(let ([data (make-vector 100)]
+      [pos 0]
+      [end 0])
+  (define p (make <virtual-io-port>
+              :getc (^[]
+                      (if (>= pos end)
+                        (eof-object)
+                        (begin0 (vector-ref data pos)
+                          (inc! pos))))
+              :putc (^c
+                     (vector-set! data pos c)
+                     (inc! pos)
+                     (set! end (max pos end)))
+              :getpos (^[] pos)
+              :setpos (^p (set! pos p))))
+  (vector-copy! data 0 '#(#\a #\b #\c #\d #\e))
+  (inc! end 5)
+  (test* "read from io port" "abcde" (port->string p))
+  (test* "port-tell" 5 (port-position p))
+  (write-char #\X p)
+  (write-char #\Y p)
+  (write-char #\Z p)
+  (set-port-position! p 0)
+  (test* "write, then read" "abcdeXYZ" (port->string p))
+  (set-port-position! p 5)
+  (write-string "PQR" p)
+  (set-port-position! p 0)
+  (write-string "12345" p)
+  (set-port-position! p 0)
+  (test* "write, then read 2" "12345PQR" (port->string p))
+  )
+
 ;;-----------------------------------------------------------
 (test-section "buffered-input-port")
 
@@ -222,10 +293,11 @@
                   size))
         )))
   (test* "buffered-input-port w/o seeker"
-         '(0 1 #f 2)
+         '(0 1 #t 2)
          (let* ([a (read-byte in)]
                 [b (read-byte in)]
-                [s (port-seek in 5 SEEK_SET)] ;this shouldn't move the point
+                [s (guard (e [(<port-error> e) #t])
+                     (port-seek in 5 SEEK_SET))] ;this shouldn't move the point
                 [c (read-byte in)])
            (list a b s c)))
   )
@@ -478,4 +550,153 @@
                       (^p (generator->list
                            (get-remaining-input-generator p))))
 
+;;-----------------------------------------------------------
+(test-section "output-accumulator-port")
+
+(let ([data '()]
+      [finished #f])
+  (define (acc x)
+    (if (eof-object? x)
+      (begin (set! finished (reverse-list->string data))
+             (set! data '()))
+      (push! data x)))
+
+  (define accout (open-output-char-accumulator acc))
+
+  (test* "output char accumulator" "abcdef"
+         (begin
+           (write-char #\a accout)
+           (write 'bcd accout)
+           (display "ef" accout)
+           (close-output-port accout)
+           finished)))
+
+;;-----------------------------------------------------------
+(test-section "srfi-181 and 192")
+
+(define-module srfi-181-tests
+  (use gauche.test)
+  (use gauche.uvector)
+  (use srfi-1)
+  (use srfi-13)
+  (use srfi-64)
+  (use srfi-181)
+  (use srfi-192)
+  (use scheme.base :only (bytevector 
+                          bytevector-copy
+                          bytevector-u8-ref
+                          flush-output-port))
+
+  (test-module 'srfi-181)
+  (test-module 'srfi-192)
+
+  (define-syntax import (syntax-rules () ((_ . _) #f)))
+
+  (include "../../test/include/srfi-181-192-tests.scm")
+  )
+
+;; Auxiliary srfi-181 tests
+
+;; i/o-invalid-position-error
+(define-module srfi-181-tests-2
+  (use gauche.test)
+  (use srfi-181)
+  (use srfi-192)
+  (define p (make-custom-binary-input-port 
+             'invalid-position-error-test
+             (^ _ 1)
+             (^[] 0)
+             (^[pos] (raise (make-i/o-invalid-position-error pos)))
+             #f))
+  (test* "invalid-position-error"
+         "Invalid position object: foo"
+         (guard (e [else
+                    (and (condition-has-type? e <io-invalid-position-error>)
+                         (eq? (condition-ref e 'port) p)
+                         (~ e'message))])
+           (set-port-position! p 'foo)))
+  )
+
+(define-module srfi-181-transcoder
+  (use gauche.test)
+  (use gauche.vport)
+  (use gauche.charconv) ; chaconv is tested before vport
+  (use gauche.uvector)  ; uvector is tested before vport
+  (use srfi-181)
+
+  (define (import-test codec external-data internal-string handling)
+    (let1 native-codec (~ (native-transcoder)'codec)
+      (test* `(bytevector->string (,(~ codec'name) -> ,(~ native-codec'name)))
+             internal-string
+             (bytevector->string external-data 
+                                 (make-transcoder codec
+                                                  (native-eol-style)
+                                                  handling)))
+      (test* `(port (,(~ codec'name) -> ,(~ native-codec'name)))
+             internal-string
+             (let1 p
+                 (transcoded-port (open-input-bytevector external-data)
+                                  (make-transcoder codec
+                                                   (native-eol-style)
+                                                   handling))
+               (begin0 (port->string p)
+                 (close-port p))))))
+  (define (export-test codec internal-string external-data handling)
+    (let1 native-codec (~ (native-transcoder)'codec)
+      (test* `(string->bytevector (,(~ native-codec'name) -> ,(~ codec'name)))
+             external-data
+             (string->bytevector internal-string
+                                 (make-transcoder codec
+                                                  (native-eol-style)
+                                                  handling)))
+      (test* `(port (,(~ native-codec'name) -> ,(~ codec'name)))
+             external-data
+             (let* ([buf (open-output-bytevector)]
+                    [p (transcoded-port buf
+                                        (make-transcoder codec
+                                                         (native-eol-style)
+                                                         handling))])
+               (display internal-string p)
+               (flush p)
+               (begin0 (get-output-bytevector buf)
+                 (close-port p))))))
+
+  (define (roundtrip-test codec external-data internal-string handling)
+    (import-test codec external-data internal-string handling)
+    (export-test codec internal-string external-data handling))
+                   
+  (cond-expand
+   [gauche.ces.utf8
+    (roundtrip-test (make-codec 'latin1)
+                    '#u8(#x41 #xc2 #x42)
+                    "A\u00c2B" 'replace)
+    (roundtrip-test (make-codec 'utf-16be)
+                    '#u8(#x00 #x41 #x00 #xc2 #x00 #x42)
+                    "A\u00c2B" 'replace)
+    (roundtrip-test (make-codec 'utf-16le)
+                    '#u8(#x41 #x00 #xc2 #x00 #x42 #x00)
+                    "A\u00c2B" 'replace)
+    (import-test (make-codec 'utf-16)
+                 '#u8(#xfe #xff #x00 #x41 #x00 #xc2 #x00 #x42)
+                 "A\u00c2B" 'replace)
+    (import-test (make-codec 'utf-16)
+                 '#u8(#xff #xfe #x41 #x00 #xc2 #x00 #x42 #x00)
+                 "A\u00c2B" 'replace)
+    (export-test (make-codec 'utf-16)
+                 "A\u00c2B"
+                 (test-one-of
+                  '#u8(#xff #xfe #x41 #x00 #xc2 #x00 #x42 #x00)
+                  '#u8(#xfe #xff #x00 #x41 #x00 #xc2 #x00 #x42))
+                 'replace)
+    
+    (import-test (make-codec 'utf-16be)
+                 '#u8(#xdc #x00)
+                 (test-error <io-decoding-error>) 'raise)
+    (export-test (make-codec 'ascii)
+                 "A\u3000"
+                 (test-error <io-encoding-error>) 'raise)
+    ]
+   [else])
+  )
+
 (test-end)
