@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.vport - virtual port
 ;;;
-;;;   Copyright (c) 2004-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -37,14 +37,19 @@
   (use util.match)
   (export <virtual-input-port>
           <virtual-output-port>
+          <virtual-io-port>
           <buffered-input-port>
           <buffered-output-port>
           open-input-uvector
+          open-input-bytevector
           open-output-uvector get-output-uvector
+          open-output-bytevector get-output-bytevector
           open-input-limited-length-port
           open-input-char-list open-input-byte-list get-remaining-input-list
           open-input-char-generator open-input-byte-generator
           get-remaining-input-generator
+          open-output-char-accumulator open-output-byte-accumulator
+          open-output-accumulator
           ))
 (select-module gauche.vport)
 
@@ -82,6 +87,10 @@
       index)
     (make <buffered-input-port> :fill filler :seek seeker)))
 
+(define (open-input-bytevector bv) ; R7RS
+  (assume-type bv <u8vector>)
+  (open-input-uvector bv))
+
 ;; For output uvector, we keep backing storage info in port attributes so that
 ;; we can retrieve it by get-output-uvector.
 ;; The info is #(<buffer> <max-index> <target-class>), where <buffer> is
@@ -173,6 +182,12 @@
             vec
             (uvector-copy vec)))))))
 
+(define (open-output-bytevector) (open-output-uvector)) ;R7RS
+(define (get-output-bytevector port)
+  (or (get-output-uvector port)
+      (error "get-output-bytevector needs a output uvector port, but got:" 
+             port)))  
+
 ;;=======================================================
 ;; A port with limited-length input/output
 ;;
@@ -275,3 +290,25 @@
     eof-object))
 
 (define (%gprepend lis gen) (gunfold null? car cdr lis (^_ gen)))
+
+;;=======================================================
+;; A port write to accumulator
+;;
+
+(define (open-output-char-accumulator acc)
+  (define (putc c) (acc c))
+  (define (puts s) (string-for-each acc s))
+  (define (close) (acc (eof-object)))
+  (make <virtual-output-port> :putc putc :puts puts :close close))
+
+(define (open-output-byte-accumulator acc)
+  (define (putb b) (acc b))
+  (define (close) (acc (eof-object)))
+  (make <virtual-output-port> :putb putb :close close))
+
+(define (open-output-accumulator acc)
+  (define (putb b) (acc b))
+  (define (putc c) (acc c))
+  (define (puts s) (acc s))
+  (define (close) (acc (eof-object)))
+  (make <virtual-output-port> :putb putb :putc putc :puts puts :close close))
