@@ -93,14 +93,15 @@ SCM_DEFINE_BASE_CLASS(Scm_ZlibVersionErrorClass, ScmZlibVersionError,
                       message_print, NULL, NULL,
                       zliberror_allocate, zlib_error_cpl);
 
-static ScmObj zliberror_allocate(ScmClass *klass, ScmObj initargs)
+static ScmObj zliberror_allocate(ScmClass *klass, ScmObj initargs SCM_UNUSED)
 {
     ScmZlibError *e = SCM_NEW_INSTANCE(ScmZlibError, klass);
     e->message = SCM_FALSE;
     return SCM_OBJ(e);
 }
 
-static void message_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
+static void message_print(ScmObj obj, ScmPort *port, 
+                          ScmWriteContext *ctx SCM_UNUSED)
 {
     ScmClass *k = Scm_ClassOf(obj);
     Scm_Printf(port, "#<%A \"%30.1A\">",
@@ -109,12 +110,12 @@ static void message_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
 }
 
 static ScmClassStaticSlotSpec zliberror_slots[] = {
-    { NULL }
+    SCM_CLASS_SLOT_SPEC_END()
 };
 
 ScmObj Scm_MakeZlibError(ScmObj message, int error_code)
 {
-    ScmClass *klass;
+    ScmClass *klass = NULL;
     switch (error_code) {
     case Z_NEED_DICT:
         klass = SCM_CLASS_ZLIB_NEED_DICT_ERROR;
@@ -162,11 +163,11 @@ void Scm_ZlibError(int error_code, const char *msg, ...)
         e = Scm_MakeError(SCM_MAKE_STR("Error occurred in error handler"));
     }
     SCM_END_PROTECT;
-    Scm_VMThrowException2(vm, e, 0);
+    Scm_VMThrowException(vm, e, 0);
     Scm_Panic("Scm_Error: Scm_VMThrowException returned.  something wrong.");
 }
 
-static ScmObj porterror_allocate(ScmClass *klass, ScmObj initargs)
+static ScmObj porterror_allocate(ScmClass *klass, ScmObj initargs SCM_UNUSED)
 {
     ScmPortError *e = SCM_NEW_INSTANCE(ScmPortError, klass);
     e->common.message = SCM_FALSE; /* set by initialize */
@@ -197,7 +198,7 @@ void Scm_ZlibPortError(ScmPort *port, int error_code, const char *msg, ...)
         e = Scm_MakeError(SCM_MAKE_STR("Error occurred in error handler"));
     }
     SCM_END_PROTECT;
-    Scm_VMThrowException(vm, e);
+    Scm_VMThrowException(vm, e, 0);
     Scm_Panic("Scm_Error: Scm_VMThrowException returned.  something wrong.");
 }
 
@@ -205,7 +206,7 @@ void Scm_ZlibPortError(ScmPort *port, int error_code, const char *msg, ...)
  * Common
  */
 
-static int fix_buffer_size(int siz)
+static ScmSize fix_buffer_size(ScmSize siz)
 {
     if (siz <= 0) return DEFAULT_BUFFER_SIZE;
     if (siz <= MINIMUM_BUFFER_SIZE) return MINIMUM_BUFFER_SIZE;
@@ -223,16 +224,16 @@ static ScmObj port_name(const char *type, ScmPort *source)
 /*================================================================
  * Deflating port
  */
-static int deflate_flusher(ScmPort *port, int cnt, int forcep)
+static ScmSize deflate_flusher(ScmPort *port, ScmSize cnt, int forcep)
 {
     ScmZlibInfo *info = SCM_PORT_ZLIB_INFO(port);
     z_streamp strm = SCM_PORT_ZSTREAM(port);
-    int total = 0;
-    unsigned char *inbuf = (unsigned char*)port->src.buf.buffer;
+    ScmSize total = 0;
+    unsigned char *inbuf = (u_char*)Scm_PortBufferStruct(port)->buffer;
     unsigned char outbuf[CHUNK];
 
     strm->next_in = inbuf;
-    strm->avail_in = SCM_PORT_BUFFER_AVAIL(port);
+    strm->avail_in = Scm_PortBufferAvail(port);
 
     if (info->flush == Z_NO_FLUSH && forcep) {
         info->flush = Z_SYNC_FLUSH;
@@ -246,8 +247,8 @@ static int deflate_flusher(ScmPort *port, int cnt, int forcep)
         if (strm->avail_out != 0) {
             info->flush = Z_NO_FLUSH;
         }
-        int nread = strm->next_in - inbuf;
-        int nwrite = strm->next_out - outbuf;
+        ScmSize nread = strm->next_in - inbuf;
+        ScmSize nwrite = strm->next_out - outbuf;
         total += nread;
         if (nwrite > 0) {
             Scm_Putz((char*)outbuf, nwrite, info->remote);
@@ -263,11 +264,11 @@ static void deflate_closer(ScmPort *port)
 {
     ScmZlibInfo *info = SCM_PORT_ZLIB_INFO(port);
     z_streamp strm = SCM_PORT_ZSTREAM(port);
-    unsigned char *inbuf = (unsigned char*)port->src.buf.buffer;
+    unsigned char *inbuf = (u_char*)Scm_PortBufferStruct(port)->buffer;
     unsigned char outbuf[CHUNK];
 
     strm->next_in = inbuf;
-    strm->avail_in = SCM_PORT_BUFFER_AVAIL(port);
+    strm->avail_in = Scm_PortBufferAvail(port);
     strm->next_out = outbuf;
     strm->avail_out = CHUNK;
 
@@ -300,7 +301,7 @@ static int zlib_fileno(ScmPort *port)
 ScmObj Scm_MakeDeflatingPort(ScmPort *source, int level,
                              int window_bits, int memlevel,
                              int strategy, ScmObj dict,
-                             int bufsiz, int ownerp)
+                             ScmSize bufsiz, int ownerp)
 {
     ScmZlibInfo *info = SCM_NEW(ScmZlibInfo);
     z_streamp strm = SCM_NEW_ATOMIC2(z_streamp, sizeof(z_stream));
@@ -364,18 +365,18 @@ ScmObj Scm_MakeDeflatingPort(ScmPort *source, int level,
  * Inflating port
  */
 
-static int inflate_filler(ScmPort *port, int mincnt)
+static ScmSize inflate_filler(ScmPort *port, ScmSize mincnt SCM_UNUSED)
 {
     ScmZlibInfo *info = SCM_PORT_ZLIB_INFO(port);
     z_streamp strm = SCM_PORT_ZSTREAM(port);
-    unsigned char *outbuf = (unsigned char*)port->src.buf.end;
+    unsigned char *outbuf = (u_char*)Scm_PortBufferStruct(port)->end;
     int r;
 
     if (info->stream_endp) return 0;
 
-    int nread = Scm_Getz(info->ptr,
-                         info->bufsiz - (info->ptr - info->buf),
-                         info->remote);
+    ScmSize nread = Scm_Getz(info->ptr,
+                             info->bufsiz - (info->ptr - info->buf),
+                             info->remote);
 
     if (nread <= 0) {
         /* input reached EOF */
@@ -390,7 +391,7 @@ static int inflate_filler(ScmPort *port, int mincnt)
 
     strm->next_in = (unsigned char*)info->buf;
     strm->next_out = outbuf;
-    strm->avail_out = SCM_PORT_BUFFER_ROOM(port);
+    strm->avail_out = Scm_PortBufferRoom(port);
 
   redo:
     r = inflate(strm, Z_SYNC_FLUSH);
@@ -447,12 +448,12 @@ static void inflate_closer(ScmPort *port)
     }
 }
 
-static int inflate_ready(ScmPort *port)
+static int inflate_ready(ScmPort *port SCM_UNUSED)
 {
     return 0;
 }
 
-ScmObj Scm_MakeInflatingPort(ScmPort *sink, int bufsiz,
+ScmObj Scm_MakeInflatingPort(ScmPort *sink, ScmSize bufsiz,
                              int window_bits, ScmObj dict,
                              int ownerp)
 {
@@ -513,7 +514,7 @@ ScmObj Scm_InflateSync(ScmPort *port)
 {
     ScmZlibInfo *info = SCM_PORT_ZLIB_INFO(port);
     z_streamp strm = SCM_PORT_ZSTREAM(port);
-    unsigned char *outbuf = (unsigned char*)port->src.buf.end;
+    unsigned char *outbuf = (u_char*)Scm_PortBufferStruct(port)->end;
     unsigned long curr_in = strm->total_in;
 
     if (info->stream_endp) return SCM_FALSE;
@@ -536,7 +537,7 @@ ScmObj Scm_InflateSync(ScmPort *port)
 
         strm->next_in = (unsigned char*)info->buf;
         strm->next_out = outbuf;
-        strm->avail_out = SCM_PORT_BUFFER_ROOM(port);
+        strm->avail_out = Scm_PortBufferRoom(port);
 
         r = inflateSync(strm);
         SCM_ASSERT(r != Z_STREAM_ERROR);
