@@ -31,10 +31,6 @@ STATIC GC_bool GC_alloc_reclaim_list(struct obj_kind *kind)
     return(TRUE);
 }
 
-GC_INNER GC_bool GC_collect_or_expand(word needed_blocks,
-                                      GC_bool ignore_off_page,
-                                      GC_bool retry); /* from alloc.c */
-
 /* Allocate a large block of size lb bytes.     */
 /* The block is not cleared.                    */
 /* Flags is 0 or IGNORE_OFF_PAGE.               */
@@ -48,7 +44,7 @@ GC_INNER ptr_t GC_alloc_large(size_t lb, int k, unsigned flags)
 
     GC_ASSERT(I_HOLD_LOCK());
     lb = ROUNDUP_GRANULE_SIZE(lb);
-    n_blocks = OBJ_SZ_TO_BLOCKS(lb);
+    n_blocks = OBJ_SZ_TO_BLOCKS_CHECKED(lb);
     if (!EXPECT(GC_is_initialized, TRUE)) {
       DCL_LOCK_STATE;
       UNLOCK(); /* just to unset GC_lock_holder */
@@ -89,22 +85,75 @@ GC_INNER ptr_t GC_alloc_large(size_t lb, int k, unsigned flags)
 STATIC ptr_t GC_alloc_large_and_clear(size_t lb, int k, unsigned flags)
 {
     ptr_t result;
-    word n_blocks = OBJ_SZ_TO_BLOCKS(lb);
 
     GC_ASSERT(I_HOLD_LOCK());
     result = GC_alloc_large(lb, k, flags);
     if (result != NULL
           && (GC_debugging_started || GC_obj_kinds[k].ok_init)) {
+        word n_blocks = OBJ_SZ_TO_BLOCKS(lb);
+
         /* Clear the whole block, in case of GC_realloc call. */
         BZERO(result, n_blocks * HBLKSIZE);
     }
     return result;
 }
 
-/* allocate lb bytes for an object of kind k.   */
-/* Should not be used to directly to allocate   */
-/* objects such as STUBBORN objects that        */
-/* require special handling on allocation.      */
+/* Fill in additional entries in GC_size_map, including the i-th one.   */
+/* Note that a filled in section of the array ending at n always        */
+/* has the length of at least n/4.                                      */
+STATIC void GC_extend_size_map(size_t i)
+{
+  size_t orig_granule_sz = ROUNDED_UP_GRANULES(i);
+  size_t granule_sz;
+  size_t byte_sz = GRANULES_TO_BYTES(orig_granule_sz);
+                        /* The size we try to preserve.         */
+                        /* Close to i, unless this would        */
+                        /* introduce too many distinct sizes.   */
+  size_t smaller_than_i = byte_sz - (byte_sz >> 3);
+  size_t low_limit; /* The lowest indexed entry we initialize.  */
+  size_t number_of_objs;
+
+  GC_ASSERT(I_HOLD_LOCK());
+  GC_ASSERT(0 == GC_size_map[i]);
+  if (0 == GC_size_map[smaller_than_i]) {
+    low_limit = byte_sz - (byte_sz >> 2); /* much smaller than i */
+    granule_sz = orig_granule_sz;
+    while (GC_size_map[low_limit] != 0)
+      low_limit++;
+  } else {
+    low_limit = smaller_than_i + 1;
+    while (GC_size_map[low_limit] != 0)
+      low_limit++;
+
+    granule_sz = ROUNDED_UP_GRANULES(low_limit);
+    granule_sz += granule_sz >> 3;
+    if (granule_sz < orig_granule_sz)
+      granule_sz = orig_granule_sz;
+  }
+
+  /* For these larger sizes, we use an even number of granules.         */
+  /* This makes it easier to, e.g., construct a 16-byte-aligned         */
+  /* allocator even if GRANULE_BYTES is 8.                              */
+  granule_sz = (granule_sz + 1) & ~1;
+  if (granule_sz > MAXOBJGRANULES)
+    granule_sz = MAXOBJGRANULES;
+
+  /* If we can fit the same number of larger objects in a block, do so. */
+  number_of_objs = HBLK_GRANULES / granule_sz;
+  GC_ASSERT(number_of_objs != 0);
+  granule_sz = (HBLK_GRANULES / number_of_objs) & ~1;
+
+  byte_sz = GRANULES_TO_BYTES(granule_sz) - EXTRA_BYTES;
+                        /* We may need one extra byte; do not always    */
+                        /* fill in GC_size_map[byte_sz].                */
+
+  for (; low_limit <= byte_sz; low_limit++)
+    GC_size_map[low_limit] = granule_sz;
+}
+
+/* Allocate lb bytes for an object of kind k.           */
+/* Should not be used to directly to allocate objects   */
+/* that require special handling on allocation.         */
 GC_INNER void * GC_generic_malloc_inner(size_t lb, int k)
 {
     void *op;
@@ -146,10 +195,11 @@ GC_INNER void * GC_generic_malloc_inner(size_t lb, int k)
         }
         *opp = obj_link(op);
         obj_link(op) = 0;
-        GC_bytes_allocd += GRANULES_TO_BYTES(lg);
+        GC_bytes_allocd += GRANULES_TO_BYTES((word)lg);
     } else {
         op = (ptr_t)GC_alloc_large_and_clear(ADD_SLOP(lb), k, 0);
-        GC_bytes_allocd += lb;
+        if (op != NULL)
+            GC_bytes_allocd += lb;
     }
 
     return op;
@@ -170,7 +220,8 @@ GC_INNER void * GC_generic_malloc_inner(size_t lb, int k)
     GC_ASSERT(k < MAXOBJKINDS);
     lb_adjusted = ADD_SLOP(lb);
     op = GC_alloc_large_and_clear(lb_adjusted, k, IGNORE_OFF_PAGE);
-    GC_bytes_allocd += lb_adjusted;
+    if (op != NULL)
+        GC_bytes_allocd += lb_adjusted;
     return op;
   }
 #endif
@@ -178,7 +229,11 @@ GC_INNER void * GC_generic_malloc_inner(size_t lb, int k)
 #ifdef GC_COLLECT_AT_MALLOC
   /* Parameter to force GC at every malloc of size greater or equal to  */
   /* the given value.  This might be handy during debugging.            */
-  size_t GC_dbg_collect_at_malloc_min_lb = (GC_COLLECT_AT_MALLOC);
+# if defined(CPPCHECK)
+    size_t GC_dbg_collect_at_malloc_min_lb = 16*1024; /* e.g. */
+# else
+    size_t GC_dbg_collect_at_malloc_min_lb = (GC_COLLECT_AT_MALLOC);
+# endif
 #endif
 
 GC_API GC_ATTR_MALLOC void * GC_CALL GC_generic_malloc(size_t lb, int k)
@@ -203,8 +258,6 @@ GC_API GC_ATTR_MALLOC void * GC_CALL GC_generic_malloc(size_t lb, int k)
 
         lg = ROUNDED_UP_GRANULES(lb);
         lb_rounded = GRANULES_TO_BYTES(lg);
-        if (lb_rounded < lb)
-            return((*GC_get_oom_fn())(lb));
         n_blocks = OBJ_SZ_TO_BLOCKS(lb_rounded);
         init = GC_obj_kinds[k].ok_init;
         LOCK();
@@ -222,8 +275,8 @@ GC_API GC_ATTR_MALLOC void * GC_CALL GC_generic_malloc(size_t lb, int k)
                 ((word *)result)[GRANULES_TO_WORDS(lg)-2] = 0;
 #           endif
           }
+          GC_bytes_allocd += lb_rounded;
         }
-        GC_bytes_allocd += lb_rounded;
         UNLOCK();
         if (init && !GC_debugging_started && 0 != result) {
             BZERO(result, n_blocks * HBLKSIZE);
@@ -238,16 +291,16 @@ GC_API GC_ATTR_MALLOC void * GC_CALL GC_generic_malloc(size_t lb, int k)
 
 GC_API GC_ATTR_MALLOC void * GC_CALL GC_malloc_kind_global(size_t lb, int k)
 {
-    void *op;
-    void **opp;
-    size_t lg;
-    DCL_LOCK_STATE;
-
     GC_ASSERT(k < MAXOBJKINDS);
     if (SMALL_OBJ(lb)) {
+        void *op;
+        void **opp;
+        size_t lg;
+        DCL_LOCK_STATE;
+
         GC_DBG_COLLECT_AT_MALLOC(lb);
-        lg = GC_size_map[lb];
         LOCK();
+        lg = GC_size_map[lb];
         opp = &GC_obj_kinds[k].ok_freelist[lg];
         op = *opp;
         if (EXPECT(op != NULL, TRUE)) {
@@ -262,13 +315,16 @@ GC_API GC_ATTR_MALLOC void * GC_CALL GC_malloc_kind_global(size_t lb, int k)
                 *opp = obj_link(op);
                 obj_link(op) = 0;
             }
-            GC_bytes_allocd += GRANULES_TO_BYTES(lg);
+            GC_bytes_allocd += GRANULES_TO_BYTES((word)lg);
             UNLOCK();
             return op;
         }
         UNLOCK();
     }
-    return GENERAL_MALLOC(lb, k);
+
+    /* We make the GC_clear_stack() call a tail one, hoping to get more */
+    /* of the stack.                                                    */
+    return GC_clear_stack(GC_generic_malloc(lb, k));
 }
 
 #if defined(THREADS) && !defined(THREAD_LOCAL_ALLOC)
@@ -294,28 +350,29 @@ GC_API GC_ATTR_MALLOC void * GC_CALL GC_generic_malloc_uncollectable(
                                                         size_t lb, int k)
 {
     void *op;
-    void **opp;
-    size_t lg;
     DCL_LOCK_STATE;
 
     GC_ASSERT(k < MAXOBJKINDS);
     if (SMALL_OBJ(lb)) {
+        void **opp;
+        size_t lg;
+
         GC_DBG_COLLECT_AT_MALLOC(lb);
         if (EXTRA_BYTES != 0 && lb != 0) lb--;
                   /* We don't need the extra byte, since this won't be  */
                   /* collected anyway.                                  */
-        lg = GC_size_map[lb];
         LOCK();
+        lg = GC_size_map[lb];
         opp = &GC_obj_kinds[k].ok_freelist[lg];
         op = *opp;
         if (EXPECT(op != NULL, TRUE)) {
             *opp = obj_link(op);
             obj_link(op) = 0;
-            GC_bytes_allocd += GRANULES_TO_BYTES(lg);
+            GC_bytes_allocd += GRANULES_TO_BYTES((word)lg);
             /* Mark bit was already set on free list.  It will be       */
             /* cleared only temporarily during a collection, as a       */
             /* result of the normal free list mark bit clearing.        */
-            GC_non_gc_bytes += GRANULES_TO_BYTES(lg);
+            GC_non_gc_bytes += GRANULES_TO_BYTES((word)lg);
             UNLOCK();
         } else {
             UNLOCK();
@@ -377,6 +434,12 @@ GC_API GC_ATTR_MALLOC void * GC_CALL GC_malloc_uncollectable(size_t lb)
   /* admittedly quite ugly.                                             */
 # define GC_debug_malloc_replacement(lb) GC_debug_malloc(lb, GC_DBG_EXTRAS)
 
+# if defined(CPPCHECK)
+#   define REDIRECT_MALLOC_F GC_malloc /* e.g. */
+# else
+#   define REDIRECT_MALLOC_F REDIRECT_MALLOC
+# endif
+
   void * malloc(size_t lb)
   {
     /* It might help to manually inline the GC_malloc call here.        */
@@ -389,7 +452,7 @@ GC_API GC_ATTR_MALLOC void * GC_CALL GC_malloc_uncollectable(size_t lb)
       /* inopportune times.                                             */
       if (!EXPECT(GC_is_initialized, TRUE)) return sbrk(lb);
 #   endif
-    return (void *)REDIRECT_MALLOC(lb);
+    return (void *)REDIRECT_MALLOC_F(lb);
   }
 
 # if defined(GC_LINUX_THREADS)
@@ -400,7 +463,10 @@ GC_API GC_ATTR_MALLOC void * GC_CALL GC_malloc_uncollectable(size_t lb)
 
     STATIC void GC_init_lib_bounds(void)
     {
+      IF_CANCEL(int cancel_state;)
+
       if (GC_libpthread_start != 0) return;
+      DISABLE_CANCEL(cancel_state);
       GC_init(); /* if not called yet */
       if (!GC_text_mapping("libpthread-",
                            &GC_libpthread_start, &GC_libpthread_end)) {
@@ -413,23 +479,15 @@ GC_API GC_ATTR_MALLOC void * GC_CALL GC_malloc_uncollectable(size_t lb)
       if (!GC_text_mapping("ld-", &GC_libld_start, &GC_libld_end)) {
           WARN("Failed to find ld.so text mapping: Expect crash\n", 0);
       }
+      RESTORE_CANCEL(cancel_state);
     }
 # endif /* GC_LINUX_THREADS */
 
-# include <limits.h>
-# ifdef SIZE_MAX
-#   define GC_SIZE_MAX SIZE_MAX
-# else
-#   define GC_SIZE_MAX (~(size_t)0)
-# endif
-
-# define GC_SQRT_SIZE_MAX ((1U << (WORDSZ / 2)) - 1)
-
   void * calloc(size_t n, size_t lb)
   {
     if ((lb | n) > GC_SQRT_SIZE_MAX /* fast initial test */
         && lb && n > GC_SIZE_MAX / lb)
-      return NULL;
+      return (*GC_get_oom_fn())(GC_SIZE_MAX); /* n*lb overflow */
 #   if defined(GC_LINUX_THREADS)
       /* libpthread allocated some memory that is only pointed to by    */
       /* mmapped thread stacks.  Make sure it is not collectible.       */
@@ -451,14 +509,14 @@ GC_API GC_ATTR_MALLOC void * GC_CALL GC_malloc_uncollectable(size_t lb)
         /* be a way to speed this up.                                   */
       }
 #   endif
-    return (void *)REDIRECT_MALLOC(n * lb);
+    return (void *)REDIRECT_MALLOC_F(n * lb);
   }
 
 # ifndef strdup
     char *strdup(const char *s)
     {
       size_t lb = strlen(s) + 1;
-      char *result = (char *)REDIRECT_MALLOC(lb);
+      char *result = (char *)REDIRECT_MALLOC_F(lb);
       if (result == 0) {
         errno = ENOMEM;
         return 0;
@@ -479,12 +537,13 @@ GC_API GC_ATTR_MALLOC void * GC_CALL GC_malloc_uncollectable(size_t lb)
       size_t len = strlen(str);
       if (len > size)
         len = size;
-      copy = (char *)REDIRECT_MALLOC(len + 1);
+      copy = (char *)REDIRECT_MALLOC_F(len + 1);
       if (copy == NULL) {
         errno = ENOMEM;
         return NULL;
       }
-      BCOPY(str, copy, len);
+      if (EXPECT(len > 0, TRUE))
+        BCOPY(str, copy, len);
       copy[len] = '\0';
       return copy;
     }
@@ -501,7 +560,6 @@ GC_API void GC_CALL GC_free(void * p)
     hdr *hhdr;
     size_t sz; /* In bytes */
     size_t ngranules;   /* sz in granules */
-    void **flh;
     int knd;
     struct obj_kind * ok;
     DCL_LOCK_STATE;
@@ -515,8 +573,11 @@ GC_API void GC_CALL GC_free(void * p)
     h = HBLKPTR(p);
     hhdr = HDR(h);
 #   if defined(REDIRECT_MALLOC) && \
-        (defined(GC_SOLARIS_THREADS) || defined(GC_LINUX_THREADS) \
+        ((defined(NEED_CALLINFO) && defined(GC_HAVE_BUILTIN_BACKTRACE)) \
+         || defined(GC_SOLARIS_THREADS) || defined(GC_LINUX_THREADS) \
          || defined(MSWIN32))
+        /* This might be called indirectly by GC_print_callers to free  */
+        /* the result of backtrace_symbols.                             */
         /* For Solaris, we have to redirect malloc calls during         */
         /* initialization.  For the others, this seems to happen        */
         /* implicitly.                                                  */
@@ -524,18 +585,20 @@ GC_API void GC_CALL GC_free(void * p)
         if (0 == hhdr) return;
 #   endif
     GC_ASSERT(GC_base(p) == p);
-    sz = hhdr -> hb_sz;
+    sz = (size_t)hhdr->hb_sz;
     ngranules = BYTES_TO_GRANULES(sz);
     knd = hhdr -> hb_obj_kind;
     ok = &GC_obj_kinds[knd];
     if (EXPECT(ngranules <= MAXOBJGRANULES, TRUE)) {
+        void **flh;
+
         LOCK();
         GC_bytes_freed += sz;
         if (IS_UNCOLLECTABLE(knd)) GC_non_gc_bytes -= sz;
                 /* Its unnecessary to clear the mark bit.  If the       */
                 /* object is reallocated, it doesn't matter.  O.w. the  */
                 /* collector will do it, since it's on a free list.     */
-        if (ok -> ok_init) {
+        if (ok -> ok_init && EXPECT(sz > sizeof(word), TRUE)) {
             BZERO((word *)p + 1, sz-sizeof(word));
         }
         flh = &(ok -> ok_freelist[ngranules]);
@@ -544,6 +607,7 @@ GC_API void GC_CALL GC_free(void * p)
         UNLOCK();
     } else {
         size_t nblocks = OBJ_SZ_TO_BLOCKS(sz);
+
         LOCK();
         GC_bytes_freed += sz;
         if (IS_UNCOLLECTABLE(knd)) GC_non_gc_bytes -= sz;
@@ -565,20 +629,21 @@ GC_API void GC_CALL GC_free(void * p)
     hdr *hhdr;
     size_t sz; /* bytes */
     size_t ngranules;  /* sz in granules */
-    void ** flh;
     int knd;
     struct obj_kind * ok;
 
     h = HBLKPTR(p);
     hhdr = HDR(h);
     knd = hhdr -> hb_obj_kind;
-    sz = hhdr -> hb_sz;
+    sz = (size_t)hhdr->hb_sz;
     ngranules = BYTES_TO_GRANULES(sz);
     ok = &GC_obj_kinds[knd];
     if (ngranules <= MAXOBJGRANULES) {
+        void ** flh;
+
         GC_bytes_freed += sz;
         if (IS_UNCOLLECTABLE(knd)) GC_non_gc_bytes -= sz;
-        if (ok -> ok_init) {
+        if (ok -> ok_init && EXPECT(sz > sizeof(word), TRUE)) {
             BZERO((word *)p + 1, sz-sizeof(word));
         }
         flh = &(ok -> ok_freelist[ngranules]);
@@ -601,6 +666,13 @@ GC_API void GC_CALL GC_free(void * p)
 #endif
 
 #if defined(REDIRECT_FREE) && !defined(REDIRECT_MALLOC_IN_HEADER)
+
+# if defined(CPPCHECK)
+#   define REDIRECT_FREE_F GC_free /* e.g. */
+# else
+#   define REDIRECT_FREE_F REDIRECT_FREE
+# endif
+
   void free(void * p)
   {
 #   ifndef IGNORE_FREE
@@ -619,7 +691,7 @@ GC_API void GC_CALL GC_free(void * p)
           return;
         }
 #     endif
-      REDIRECT_FREE(p);
+      REDIRECT_FREE_F(p);
 #   endif
   }
 #endif /* REDIRECT_FREE */
