@@ -1,7 +1,7 @@
 ;;;
 ;;;  data.heap - Heaps
 ;;;
-;;;   Copyright (c) 2014-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2014-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -150,7 +150,7 @@
                      [(and (integer? num-entries) (>= num-entries 0))
                       (min num-entries xlen)]
                      [else
-                      (error "invalid num-entris value for build-binary-heap:"
+                      (error "invalid num-entries value for build-binary-heap:"
                              num-entries)])])
     (receive (<: >:)
         (ecase (comparator-flavor comparator)
