@@ -1,7 +1,7 @@
 ;;;
 ;;; fsdbm - dbm on filesystem
 ;;;
-;;;   Copyright (c) 2003-2014  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2003-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -212,7 +212,7 @@
              (chunks '()))
     (if (eof-object? elt)
       (let ((s (string-concatenate-reverse chunks)))
-        (or (string-incomplete->complete! s) s))
+        (or (string-incomplete->complete s) s))
       (loop (read-block 1024 port) (cons elt chunks)))))
 
 (define (dir-perm file-perm)
