@@ -1,7 +1,7 @@
 ;;;
 ;;; file/filter.scm - utility to build filter programs
 ;;;
-;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -90,7 +90,7 @@
     (cond [(string? temporary-file)
            (process-with-tempfile output temporary-file)]
           [(eq? temporary-file #t)
-           (process-with-tempfile output #`",(sys-basename output).tmp")]
+           (process-with-tempfile output #"~(sys-basename output).tmp")]
           [(not temporary-file)
            (guard (e [else (unless keep-output? (sys-unlink output))
                            (raise e)])
