@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.cgen.literal - static literal data
 ;;;
-;;;   Copyright (c) 2004-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -119,6 +119,11 @@
         (or dl
             (rlet1 new (make <cgen-static-data-list>
                          :category category :c-type c-type)
+              (when (eq? c-type 'ScmPair)
+                ;; We need a sentinel at the beginning of ScmPair array
+                ;; See ScmExtendedPair description in gauche/priv/pairP.h.
+                (push! (~ new'init-thunks) "   { SCM_NIL, SCM_NIL }")
+                (inc! (~ new'count)))
               (push! (~ unit'static-data-list) new))))))
 
   (let ([dl (ensure-static-data-list category c-type)]
@@ -153,6 +158,9 @@
         (dolist [dl dls] (emit-initializers dl))
         (print "};"))))
 
+  ;; NB: We add SCM_UNUSED in the static data, for it could be empty
+  ;; depending on cpp conditions, and we don't want the compiler warn
+  ;; about unused static var.
   (define (emit-struct-def category dls)
     (let1 name (static-data-c-struct-name category)
       (format #t "static ~astruct ~aRec {\n"
@@ -160,10 +168,13 @@
               name)
       (dolist [dl dls]
         (for-each (cut print "#if "<>) (reverse (~ dl'cpp-conditions)))
-        (format #t "  ~a ~a[~a];\n" (~ dl'c-type) (~ dl'c-member-name)
-                (~ dl'count))
+        (format #t "  ~a ~a[~a]~a;\n"
+                (~ dl'c-type) (~ dl'c-member-name) (~ dl'count)
+                (if (eq? (~ dl'c-type) 'ScmPair)
+                  " SCM_ALIGN_PAIR"
+                  ""))
         (for-each (cut print "#endif /*"<>"*/") (~ dl'cpp-conditions)))
-      (format #t "} ~a = " name)))
+      (format #t "} ~a SCM_UNUSED = " name)))
 
   (define (emit-initializers dl)
     (for-each (cut print "#if "<>) (reverse (~ dl'cpp-conditions)))
@@ -371,7 +382,9 @@
      [(pair? val) (logand (+ (rec (car val)) (rec (cdr val))) mask)]
      [(vector? val) (fold (^(v r) (logand (+ (rec v) r) mask)) 0 val)]
      [(string? val) (logand (string-hash val) mask)]
-     [(identifier? val) (logand (+ (rec (~ val'name))(rec (~ val'module))) mask)]
+     ;; TRANSIENT: After 1.0, the condition below can be 'wrapped-identifier?'
+     [(and (identifier? val) (not (symbol? val)))
+      (logand (+ (rec (~ val'name))(rec (~ val'module))) mask)]
      [else (eqv-hash val)]))
   (modulo (rec literal) .literal-hash-size.))
 
@@ -386,8 +399,9 @@
                   (every?-ec (: i len)
                              (rec (vector-ref x i) (vector-ref y i))))))]
      [(string? x) (and (string? y) (string=? x y))]
-     [(identifier? x)
-      (and (identifier? y)
+     ;; TRANSIENT: After 1.0, the conditions below can be 'wrapped-identifier?'
+     [(and (identifier? x) (not (symbol? x)))
+      (and (identifier? y) (not (symbol? y))
            (eq? (~ x'name) (~ y'name))
            (eq? (~ x'module) (~ y'module)))]
      [else (and (eq? (class-of x) (class-of y)) (eqv? x y))]))
@@ -472,7 +486,7 @@
                'constant 'ScmString
                (format "  SCM_STRING_CONST_INITIALIZER(~a, ~a, ~a)"
                        (cgen-safe-string value)
-                       (string-size value) (string-length value)))
+                       (string-length value) (string-size value)))
       :value value))
   )
 
@@ -614,12 +628,14 @@
    (cdr :init-keyword :cdr))
   (make (value)
     (let* ([ca (cgen-literal (car value))]
-           [cd (cgen-literal (cdr value))]
-           [sobj (cgen-allocate-static-array
-                  'runtime 'ScmObj
-                  (list (get-literal-initializer ca)
-                        (get-literal-initializer cd)))])
-      (make <cgen-scheme-pair> :value value :car ca :cdr cd :c-name sobj)))
+           [cd (cgen-literal (cdr value))])
+      (define (init-thunk)
+        (format #t "   { ~a, ~a}"
+                (get-literal-initializer ca)
+                (get-literal-initializer cd)))
+      (make <cgen-scheme-pair> :value value :car ca :cdr cd
+            :c-name (cgen-allocate-static-datum 'runtime 'ScmPair
+                                                init-thunk))))
   (init (self)
     (let1 cname (cgen-cexpr self)
       (unless (cgen-literal-static? (~ self'car))
@@ -677,7 +693,12 @@
                   'runtime 'ScmObj
                   (list*
                    "SCM_OBJ(SCM_CLASS_STATIC_TAG(Scm_VectorClass)) /* <vector> */"
-                   (format "SCM_OBJ(~a)" (length literals))
+                   (format "SCM_VECTOR_SIZE_SLOT_INITIALIZER(~a, ~a)" 
+                           (length literals)
+                           ;; TRANSIENT: 0.9.9 doesn't have vector-immutable?
+                           (if-let1 pred (global-variable-ref 'gauche 'vector-immutable? #f)
+                             (if (pred value) "TRUE" "FALSE")
+                             "FALSE"))
                    (map (^[lit] (if (cgen-literal-static? lit)
                                   (cgen-cexpr lit)
                                   "SCM_UNDEFINED"))
@@ -717,8 +738,7 @@
       (print (uvector-class->c-type-name class)" "(~ self'elements)"[] = {")
       (dotimes [i (uvector-length value)]
         ($ uvector-class-emit-elt class
-           $ (with-module gauche.internal %uvector-ref)
-             value (uvector-class->type-enum class) i))
+           $ (with-module gauche.internal uvector-ref) value i))
       (print  "};"))]
   [static (self) #f]
   )
@@ -741,16 +761,35 @@
     [(eq? class <u8vector>) "unsigned char"]
     [(eq? class <s16vector>) "short"]
     [(eq? class <u16vector>) "unsigned short"]
-    [(eq? class <s32vector>) "ScmInt32"]
-    [(eq? class <u32vector>) "ScmUInt32"]
-    [(eq? class <s64vector>) "ScmInt64"]
-    [(eq? class <u64vector>) "ScmUInt64"]
+    [(eq? class <s32vector>) "int32_t"]
+    [(eq? class <u32vector>) "uint32_t"]
+    [(eq? class <s64vector>) "int64_t"]
+    [(eq? class <u64vector>) "uint64_t"]
     [(eq? class <f16vector>) "ScmHalfFloat"]
     [(eq? class <f32vector>) "float"]
     [(eq? class <f64vector>) "double"]
+    [(eq? class <c32vector>) "ScmHalfComplex"]
+    [(eq? class <c64vector>) "ScmFloatComplex"]
+    [(eq? class <c128vector>) "ScmDoubleComplex"]
     ))
 
 (define (uvector-class-emit-elt class v)
+  (define (pr-half-float v)
+    (format #t "0x~4,'0x" ((with-module gauche.internal flonum->f16bits) v)))
+  (define (pr-float v)
+    (cond [(nan? v) (display "SCM_FLT_NAN")]
+          [(infinite? v) (display (if (> v 0)
+                                    "SCM_FLT_POSITIVE_INFINITY"
+                                    "SCM_FLT_NEGATIVE_INFINITY"))]
+          [(-zero? v) (display "-0.0f")]
+          [else (format #t "~sf" v)]))
+  (define (pr-double v)
+    (cond [(nan? v) (display "SCM_DBL_NAN")]
+          [(infinite? v) (display (if (> v 0)
+                                    "SCM_DBL_POSITIVE_INFINITY"
+                                    "SCM_DBL_NEGATIVE_INFINITY"))]
+          [(-zero? v) (display "-0.0")]
+          [else (format #t "~s" v)]))    
   (cond
    [(memq class `(,<s8vector> ,<s16vector> ,<s32vector>))
     (format #t "~d,\n" v)]
@@ -759,29 +798,29 @@
    [(eq? class <s64vector>)
     (print "#if SIZEOF_LONG == 8")
     (format #t "~dl,\n" v)
-    (print "#else if SCM_EMULATE_INT64")
-    (print "#ifdef WORDS_BIGENDIAN")
-    (print "#else  /*!WORDS_BIGENDIAN*/")
-    (print "#endif /*!WORDS_BIGENDIAN*/")
-    (print "#else  /*SIZEOF_LONG == 4 && !SCM_EMULATE_INT64*/")
+    (print "#else  /*SIZEOF_LONG == 4*/")
     (format #t " (((int64_t)~dl << 32)|~dl),\n"
             (ash v -32) (logand v (- (%expt 2 32) 1)))
-    (print "#endif /*SIZEOF_LONG == 4 && !SCM_EMULATE_INT64*/")]
+    (print "#endif /*SIZEOF_LONG == 4*/")]
    [(eq? class <u64vector>)
     (print "#if SIZEOF_LONG == 8")
     (format #t "~dlu,\n" v)
-    (print "#else if SCM_EMULATE_INT64")
-    (print "#ifdef WORDS_BIGENDIAN")
-    (print "#else  /*!WORDS_BIGENDIAN*/")
-    (print "#endif /*!WORDS_BIGENDIAN*/")
-    (print "#else  /*SIZEOF_LONG == 4 && !SCM_EMULATE_INT64*/")
+    (print "#else  /*SIZEOF_LONG == 4*/")
     (format #t " (((int64_t)~dlu << 32)|~dlu),\n"
             (ash v -32) (logand v (- (%expt 2 32) 1)))
-    (print "#endif /*SIZEOF_LONG == 4 && !SCM_EMULATE_INT64*/")]
-   [(eq? class <f16vector>)
-    (error "gauche.cgen.literal: literal f16vector isn't supported yet")]
-   [(eq? class <f32vector>) (format #t "~sf,\n" v)]
-   [(eq? class <f64vector>) (format #t "~s,\n" v)]
+    (print "#endif /*SIZEOF_LONG == 4*/")]
+   [(eq? class <f16vector>) (pr-half-float v) (print ",")]
+   [(eq? class <f32vector>) (pr-float v) (print ",")]
+   [(eq? class <f64vector>) (pr-double v) (print ",")]
+   [(eq? class <c32vector>)
+    (display "{ ") (pr-half-float (real-part v))
+    (display ", ") (pr-half-float (imag-part v)) (print "},")]
+   [(eq? class <c64vector>)
+    (pr-float (real-part v)) (display " + ") 
+    (pr-float (imag-part v)) (print " * _Complex_I,")]
+   [(eq? class <c128vector>)
+    (pr-double (real-part v)) (display " + ") 
+    (pr-double (imag-part v)) (print " * _Complex_I,")]
    ))
 
 ;; char-set -----------------------------------------------------
