@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.cgen.precomp - Precompile Scheme into C data
 ;;;
-;;;   Copyright (c) 2004-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -63,17 +63,17 @@
 ;;  which contains forms like define-module, use, export, etc.
 ;;
 ;;  A generated C file contains an initialization function, named
-;;  Scm_Init_<foo> by default.   Typically it is called by "extention
+;;  Scm_Init_<foo> by default.   Typically it is called by "extension
 ;;  initializer", which is invoked when the DSO file is loaded by
 ;;  dynamic-load.
 ;;
 ;;    Example1:
 ;;
-;;     *  An extention extention.so is built from extension.c,
+;;     *  An extension extension.so is built from extension.c,
 ;;        foo.scm and bar.scm.
 ;;
 ;;     *  extension.c must contain a funciton Scm_Init_extension(),
-;;        which is the extention initializer.  It is called from
+;;        which is the extension initializer.  It is called from
 ;;        (dynamic-load "extension")
 ;;
 ;;     *  By processing foo.scm and bar.scm, you'll get foo.c and
@@ -85,20 +85,20 @@
 ;;
 ;;  Sometimes sources consist of Scheme files only.  In which case,
 ;;  giving true value to ext-initializer keyword argument makes
-;;  the inialization function work as an extention initializer.
+;;  the inialization function work as an extension initializer.
 ;;
 ;;    Example2:
 ;;
-;;     * An extension extension.so is built from extention.scm.
+;;     * An extension extension.so is built from extension.scm.
 ;;
 ;;     * Processing extension.scm with :ext-initializer #t
-;;       makes generated Scm_Init_extension() work as an extention
+;;       makes generated Scm_Init_extension() work as an extension
 ;;       initializer.
 ;;
 ;;  If there are more than one Scheme files and you want to make
-;;  one of its initializer funtion as an extention initializer,
+;;  one of its initializer funtion as an extension initializer,
 ;;  give :sub-initializers argument to the 'main' source whose
-;;  initialization function becomes an extention initializer.
+;;  initialization function becomes an extension initializer.
 ;;
 ;;    Example3:
 ;;
@@ -255,8 +255,10 @@
              (call-with-output-file out.sci
                (^p (display ";; generated automatically.  DO NOT EDIT\n" p)
                    (display "#!no-fold-case\n" p)
-                   (parameterize ([ext-module-file p])
-                     (do-it))))]
+                   (parameterize ([ext-module-file p]
+                                  [ext-module-forms '()])
+                     (do-it)
+                     (flush-ext-module))))]
             [else
              (parameterize ([ext-module-file #f])
                (do-it))]))
@@ -320,6 +322,10 @@
 ;; file, assuming the source file has standard layout.
 (define ext-module-file (make-parameter #f))
 
+;; The forms to be written out to ext-module file (sci) are pushed
+;; on this parameter.
+(define ext-module-forms (make-parameter '()))
+
 ;; list of private macros that should be included in the output.
 ;; (--keep-private-macro=name,name,...)
 ;; usually private macros (macros bound to a variable which isn't exported)
@@ -426,7 +432,12 @@
 ;; code to *.sci as a transient measure and eventually we should have a way
 ;; to serialize Scheme code, including indentifier info, to the file.
 (define (write-ext-module form)
-  (cond [(ext-module-file) => (^_ (write (unwrap-syntax form) _) (newline _))]))
+  (push! (ext-module-forms) (unwrap-syntax form)))
+
+(define (flush-ext-module)
+  (cond [(ext-module-file) =>
+         (^p (dolist [f (reverse (ext-module-forms))]
+               (write f p) (newline p)))]))
 
 (define (setup ext-init? subinits)
   (cgen-decl "#include <gauche/code.h>")
@@ -494,8 +505,11 @@
 (define-global-pred =export?          export)
 (define-global-pred =export-all?      export-all)
 (define-global-pred =export-if-defined? export-if-defined)
+(define-global-pred =extend?          extend)
 (define-global-pred =provide?         provide)
 (define-global-pred =lambda?          lambda)
+(define-global-pred =include?         include)
+(define-global-pred =begin?           begin)
 
 ;; A parameter that holds the list of 'omitted' #<compiled-code> - for
 ;; example, the cliche of (CLOSURE #<compiled-code> DEFINE #<identifier> RET)
@@ -514,9 +528,13 @@
     (match form
       ;; Module related stuff
       [((? =define-module?) mod . body)
-       (write-ext-module form)
-       (with-tmodule mod
-         (fold compile-toplevel-form seed body))]
+       (receive (seed extforms)
+           (parameterize ((ext-module-forms '()))
+             (let1 seed (with-tmodule mod
+                          (fold compile-toplevel-form seed body))
+               (values seed (ext-module-forms))))
+         (write-ext-module `(define-module ,mod ,@(reverse extforms)))
+         seed)]
       [((? =select-module?) mod)
        (write-ext-module form)
        (match (dso-name)
@@ -527,25 +545,33 @@
          [_ #f])
        (select-tmodule mod)
        seed]
-      [((? =use?) mod)
-       (eval-in-current-tmodule `(use ,mod)) seed]
+      [((? =use?) . _)
+       (write-ext-module form)
+       (eval-in-current-tmodule form)
+       seed]
       [((? =export?) . syms)
        (when (list? (compile-module-exports))
          (compile-module-exports
           (lset-union eq? syms (compile-module-exports))))
        (eval-in-current-tmodule `(export ,@syms))
-       (unless (ext-module-file)
-         ;; If generate .sci file, 'export' form will be in it (as a part of
-         ;; define-module form).  Otherwise we need to do export during
-         ;; initialization.
+       ;; If generate .sci file, 'export' form will be in it.
+       ;; Otherwise we need to do export during initialization.
+       (if (ext-module-file)
+         (write-ext-module form)
          (let1 exp-specs (cgen-literal syms)
            (cgen-init
             (format "  (void)Scm_ExportSymbols(Scm_CurrentModule(), ~a);"
                     (cgen-cexpr exp-specs)))))
        seed]
-      [((? =export-all?)) (compile-module-exports #t)]
+      [((? =export-all?)) (write-ext-module form) (compile-module-exports #t)]
       [((? =export-if-defined?) . _) (write-ext-module form) seed]
       [((? =provide?) arg) (write-ext-module form) seed]
+      [((? =extend?) . _) 
+       (write-ext-module form)
+       (eval-in-current-tmodule form)
+       seed]
+      [((? =begin?) . forms) (fold compile-toplevel-form seed forms)]
+      [((? =include?) . filenames) (compile-includes filenames #f seed)]
       ;; Finally, ordinary expressions.
       [else
        (let* ([compiled-code (compile-in-current-tmodule form)]
@@ -616,6 +642,23 @@
   (cgen-init (format "  Scm_VMExecuteToplevels(toplevels);"))
   )
 
+;; Handle include.  Ideally this whole routine should be shared with
+;; src/compile-1.scm.
+(define (compile-includes filenames case-fold? seed)
+  (define (do-include filename seed)
+    (unless (string? filename)
+      (error "include requires literal string, but got:" filename))
+    (with-input-from-port
+        ($ (with-module gauche.internal pass1/open-include-file)
+           filename 
+           (sys-dirname (port-name (current-input-port))))
+      (^[]
+        (port-case-fold-set! (current-input-port) case-fold?)
+        (unwind-protect
+            (generator-fold compile-toplevel-form seed read)
+          (close-port (current-input-port))))))
+  (fold do-include seed filenames))
+
 ;;================================================================
 ;; Special form handlers
 ;;
@@ -667,8 +710,8 @@
       ((with-module gauche.cgen.precomp handle-define-syntax) f))
     (define-macro (define-macro . f)
       ((with-module gauche.cgen.precomp handle-define-macro) f))
-    (define-macro (define-inline/syntax . f)
-      ((with-module gauche.cgen.precomp handle-define-inline/syntax) f))
+    (define-macro (define-hybrid-syntax . f)
+      ((with-module gauche.cgen.precomp handle-define-hybrid-syntax) f))
     ;; TODO - we need more general framework supporting various declarations.
     ;; for the time being, this ad-hoc solution suffice our needs.
     (define-macro (declare . f)
@@ -731,7 +774,7 @@
       `((with-module gauche define-syntax) ,@form))
      ;; If the macro needs to exported, check if we can put it in
      ;; precompiled file (it is, if the transformer is a closure).
-     ;; Othewise, we emit the form to *.sci file.
+     ;; Otherwise, we emit the form to *.sci file.
      (when (or (symbol-exported? name)
                (memq name (private-macros-to-keep)))
        (let1 val (global-variable-ref (~ (current-tmodule)'module) name #f)
@@ -741,11 +784,11 @@
                `((with-module gauche.internal %insert-syntax-binding)
                  (current-module) ',name
                  (let ((,name ,xformer-spec)) ,name))) ; attach name to closure
-             (write-ext-module `(define-syntax . ,form))
-             #f)))]
+             (begin (write-ext-module `(define-syntax . ,form))
+                    #f))))]
     [_ (error "Malformed define-syntax" form)]))
 
-(define (handle-define-inline/syntax form)
+(define (handle-define-hybrid-syntax form)
   (match form
     [(name expr xformer-spec)
      ;; KLUDGE: We treat NAME as a macro during compiling this unit, since
@@ -762,10 +805,10 @@
              `((with-module gauche define-inline) ,name
                ((with-module gauche.internal %with-inline-transformer)
                 ,expr ,xformer-spec)))
-           (error "Currently, you can't use define-inline/syntax with \
+           (error "Currently, you can't use define-hybrid-syntax with \
                    other than er-macro-transformer in precompiled file"
                   form)))]
-    [_ (error "Malformed define-inline/syntax" form)]))
+    [_ (error "Malformed define-hybrid-syntax" form)]))
 
 (define (handle-define-constant form)
   (match form
@@ -1010,8 +1053,13 @@
               :name name :name-literal (cgen-literal name)))))
   (init (self)
     (unless (predefined-module (~ self'value))
+      ;; Module literals are emitted only related with macro hygiene.  In that
+      ;; case, the module is created without loading its actual body, so
+      ;; we mark it as 'placeholding'.  The placeholding module won't show
+      ;; up in (all-modules).  Once (define-module ...) is evaluated by loading
+      ;; the module, it becomes 'real' module.
       (format #t "  ~a = SCM_OBJ(Scm_FindModule(SCM_SYMBOL(~a), \
-                                     SCM_FIND_MODULE_CREATE)); \
+                         SCM_FIND_MODULE_CREATE|SCM_FIND_MODULE_PLACEHOLDING)); \
                   /* module ~a */\n"
               (~ self'c-name)
               (cgen-cexpr (~ self'name-literal))
@@ -1035,10 +1083,7 @@
   ((id-name        :init-keyword :id-name)
    (module-literal :init-keyword :module-literal))
   (make (value)
-    ;; TRANSIENT: After 0.9.6 release, we can directly call identifier-toplevel?
-    (unless (if (global-variable-bound? 'gauche.internal 'identifier-toplevel?)
-              ((with-module gauche.internal identifier-toplevel?) value)
-              (null? (~ value'env)))
+    (unless ((with-module gauche.internal identifier-toplevel?) value)
       (error "identifier with compiler environment can't be compiled" value))
     (make <cgen-scheme-identifier> :value value
           :c-name (cgen-allocate-static-datum)
