@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.interactive - useful stuff in the interactive session
 ;;;
-;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -35,6 +35,7 @@
 
 (define-module gauche.interactive
   (export apropos d read-eval-print-loop print-mode
+          toplevel-reader-state
           ;; autoloaded symbols follow
           info info-page info-search reload ed
           reload-modified-modules module-reload-rules reload-verbose)
@@ -68,9 +69,7 @@
         [matcher (cond [(symbol? item)
                         (let1 substr (symbol->string item)
                           (^[name] (string-scan name substr)))]
-                       [(string? item)
-                        ;; Note: future extention
-                        (error "Bad object for item: " item)]
+                       [(string? item) (^[name] (string-scan name item))]
                        [(is-a? item <regexp>) (^[name] (rxmatch item name))]
                        [else
                         (error "Bad object for item: " item)])]
@@ -86,10 +85,14 @@
            (when (matcher (symbol->string symbol))
              (found mod symbol))))))
 
+    (define (visible? sym)
+      (global-variable-bound? module sym))
+    
     (define (found module symbol)
-      (set! result
-            (cons (format #f "~30s (~a)~%" symbol (module-name module))
-                  result)))
+      (push! result
+             (format #f "~30s (~a~a)~%" symbol 
+                     (if (visible? symbol) "" "*")
+                     (module-name module))))
 
     ;; mimics the Scm_FindBinding
     (if stay-in-module
@@ -162,6 +165,8 @@
               (if has-optional?
                 (append spnames '_) ; this works even spnames is ()
                 spnames))))
+  (and-let1 dis ((with-module gauche.object generic-dispatcher-info) g)
+    (format #t "dispatcher:\n  ~s\n" dis))
   (values))
 
 (define-method describe ((p <procedure>))
@@ -223,6 +228,19 @@
           (format "~a~a " *repl-name* delim)
           (format "~a[~a]~a " *repl-name* (module-name m) delim))))))
 
+
+;; can be
+;;  #f       - read edit isn't available at all
+;;  editable - read edit is being used
+;;  vanilla  - read edit is available, but turned off temporarily
+(define *read-edit-state* #f)
+
+(define toplevel-reader-state
+  (case-lambda
+    [() *read-edit-state*]
+    [(mode) (ecase mode
+              [(#f editable vanilla) (set! *read-edit-state* mode)])]))
+
 ;; Returns a reader procedure that can handle toplevel command.
 ;; READ - reads one sexpr from the REPL input
 ;; READ-LINE - read to the EOL from REPL input and returns a string.
@@ -241,24 +259,42 @@
           (unless (eof-object? expr) (skipper))
           expr)))))
 
-;; EXPERIMENTAL: Environment GAUCHE_READ_EDIT enables editing mode.
-;; Note that, at this moment, text.line-edit isn't complete; it doesn't
-;; handle multibyte characters nor the multiline expressions bigger
-;; than the screen height.  Once we complete text.line-edit, we make
-;; the feature available through command-line options of gosh.
+;; History file, used by input editor.
+;; The default history file is ~/.gosh_history.  The user can override
+;; it with the environment variable GAUCHE_HISTORY_FILE, unless
+;; the process is suid/sgid-ed.
+;; To prohibit history saving, set an empty string to "GAUCHE_HISTORY_FILE".
+(define (get-history-filename)
+  (let1 h (sys-getenv "GAUCHE_HISTORY_FILE")
+    (cond [(or (equal? h "") (sys-setugid?)) #f] ; do not save history file
+          [h    (sys-normalize-pathname h :absolute #t :expand #t)]
+          [else (sys-normalize-pathname "~/.gosh_history" :expand #t)])))
 
+;; The variable *read-edit* is #f by default, #t if env var 
+;; GAUCHE_READ_EDIT is set.  It is also controlled by -fread-edit or
+;; -fno-read-edit flag.
 (define-values (%prompter %reader %line-edit-ctx)
   (receive (r rl skipper ctx)
-      (if (sys-getenv "GAUCHE_READ_EDIT")
-        (make-editable-reader (^[] (default-prompt-string "$")))
+      (if (with-module gauche.internal *read-edit*)
+        (make-editable-reader (^[] (default-prompt-string "$"))
+                              (get-history-filename))
         (values #f #f #f #f))
+    (define vanilla-reader
+      (make-repl-reader read read-line consume-trailing-whitespaces))
     (if (and r rl skipper ctx)
-      (values (^[] #f)
-              (make-repl-reader r rl skipper)
-              ctx)
+      (let1 editing-reader (make-repl-reader r rl skipper)
+        (set! *read-edit-state* 'editable)
+        (values (^[]
+                  ;; if input editor is running, prompt is shown by the reader
+                  (when (eq? *read-edit-state* 'vanilla)
+                    (display (default-prompt-string)) (flush)))
+                (^[]
+                  (if (eq? *read-edit-state* 'editable)
+                    (editing-reader)
+                    (vanilla-reader)))
+                ctx))
       (values (^[] (display (default-prompt-string)) (flush))
-              (make-repl-reader read read-line
-                                consume-trailing-whitespaces)
+              vanilla-reader
               #f))))
 
 ;; error printing will be handled by the original read-eval-print-loop
@@ -270,7 +306,8 @@
 
 ;; <write-controls> used for the printer.
 (define-constant *default-controls*
-  (make-write-controls :length 50 :level 10 :width 79))
+  (make-write-controls :length 50 :level 10 :width 79
+                       :pretty (not (sys-getenv "GAUCHE_REPL_NO_PPRINT"))))
 (define *controls* *default-controls*)
 
 (define (%printer . exprs)
@@ -306,7 +343,7 @@
      reader evaluator printer prompter)))
 
 ;;;
-;;; for windows console
+;;; Misc. setup
 ;;;
 
 ;; EXPERIMENTAL: windows console code page support for text.line-edit
@@ -337,10 +374,6 @@
          (set! (~ ctx 'wide-char-pos-setting  'surrogate-char-width) 4)])))]
  [else])
 
-;;;
-;;; Misc. setup
-;;;
-
 ;; Autoload online info viewer
 (autoload gauche.interactive.info info info-page info-search)
 
@@ -349,7 +382,7 @@
                         module-reload-rules reload-verbose)
 
 ;; Autoload editor invoker
-(autoload gauche.interactive.ed ed ed-pick-file)
+(autoload text.external-editor ed ed-pick-file)
 
 ;; Autoload toplevel command handler
 (autoload gauche.interactive.toplevel handle-toplevel-command)
@@ -357,23 +390,6 @@
 ;; See (describe <symbol>) above
 (autoload gauche.modutil describe-symbol-bindings)
 
-;; Load user-specific default settings from ~/.gaucherc, if there's any.
-;; It is evaluated in the user module.
-(let ((dotfile (sys-normalize-pathname "~/.gaucherc" :expand #t)))
-  (when (sys-access dotfile F_OK)
-    (load dotfile :environment (find-module 'user))))
-
 ;; This might help first time users
 (define-in-module user help "Type ,help (comma and help) for help")
 
-;; If gosh is invoked with R7RS mode, import r7rs-small libraries
-;; into user module for the convenience.
-(when (global-variable-ref (find-module 'user) '*r7rs-mode* #f)
-  (eval '(import (scheme base) (scheme case-lambda) (scheme char)
-                 (scheme complex) (scheme cxr) (scheme eval)
-                 (scheme file) (scheme inexact) (scheme lazy)
-                 (scheme load) (scheme process-context) (scheme read)
-                 (scheme repl) (scheme time) (scheme write)
-                 (only (gauche base) *1 *1+ *2 *2+ *3 *3+ *e *history))
-        (find-module 'r7rs.user)))
-
