@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.lazy - Lazy sequences
 ;;;
-;;;   Copyright (c) 2011-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2011-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -40,7 +40,8 @@
 (define-module gauche.lazy
   (use srfi-1)
   (use gauche.generator)
-  (export x->lseq lunfold lmap lmap-accum lappend lappend-map lconcatenate
+  (export x->lseq lunfold literate
+          lmap lmap-accum lappend lappend-map lconcatenate
           linterweave lfilter lfilter-map lstate-filter
           ltake ltake-while lrxmatch lslices))
 (select-module gauche.lazy)
@@ -62,6 +63,9 @@
   ($ generator->lseq
      $ gunfold p f g seed (if tail (^s (list->generator (tail s))) #f)))
 
+(define (literate f seed)
+  (generator->lseq seed (giterate1 f seed)))
+
 (define lmap
   (case-lambda
     [(proc arg)
