@@ -1,7 +1,7 @@
 ;;;
 ;;; library utilities - to be autoloaded.
 ;;;
-;;;   Copyright (c) 2003-2014  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2003-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -65,7 +65,7 @@
                   (match? (car pats) base)
                   (file-is-directory? path))
              (fold (^[subfile seed]
-                     (search (cdr pats) prefix #`",|file|/,|subfile|"
+                     (search (cdr pats) prefix #"~|file|/~|subfile|"
                              subfile seed))
                    seed (readdir path))]
             [(and (null? (cdr pats))
