@@ -1,7 +1,7 @@
 ;;;
 ;;; numerical.scm - auxiliary numerical functions - to be autoloaded
 ;;;
-;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -159,7 +159,7 @@
 
 ;; modulus exponent
 (define (expt-mod n e m)  ; (modulo (expt n e) m)
-  (if (and (exact-integer? n) (exact-integer? e) (exact-integer? m) (> e 0))
+  (if (and (exact-integer? n) (exact-integer? e) (exact-integer? m) (>= e 0))
     (if (< (* (integer-length n) e) (fixnum-width))
       (modulo (expt n e) m) ; in fixnum range, it's fast enough
       (let loop ([b (ash 1 (- (integer-length e) 1))]
@@ -316,17 +316,13 @@
 (define (floor/ x y)
   (receive (q r) (quotient&remainder x y)
     (if (>= x 0)
-      (if (> y 0)
+      (if (or (> y 0) (zero? r))
         (values q r)
         (values (- q 1) (+ r y)))
-      (if (> y 0)
+      (if (and (> y 0) (not (zero? r)))
         (values (- q 1) (+ r y))
         (values q r)))))
-(define (floor-quotient x y)
-  (let1 q (quotient x y)
-    (if (>= x 0)
-      (if (> y 0) q (- q 1))
-      (if (> y 0) (- q 1) q))))
+(define (floor-quotient x y)     (values-ref (floor/ x y) 0))
 (define (floor-remainder x y)    (modulo x y))
 (define (truncate/ x y)          (quotient&remainder x y))
 (define (truncate-quotient x y)  (quotient x y))
@@ -352,17 +348,27 @@
               min-exponent max-exponent exponent))
     (case mantissa
       [(#f) +nan.0]
-      [(#t) (if (< (vector-ref vec 2) 0) -inf.0 +inf.0)]
+      [(#t) (if (< sign 0) -inf.0 +inf.0)]
       [else
        (unless (<= 0 mantissa max-mantissa)
          (error "Mantissa is out of range (must be between 0 and 2^53-1):"
                 mantissa))
-       (* (vector-ref vec 2) (ldexp mantissa (vector-ref vec 1)))])))
+       (* sign (ldexp mantissa exponent))])))
 
 ;; Nearly equal comparison
 ;;  (Unofficial yet; see how it works)
 
+(define (approx=? x y :optional (relative-tolerance (flonum-epsilon))
+                                (absolute-tolerance (flonum-min-denormalized)))
+  (cond [(eqv? x y) #t]
+        [(or (not (finite? x)) (not (finite? y))) #f]
+        [else
+         (<= (abs (- x y))
+             (max (* (max (abs x) (abs y)) relative-tolerance)
+                  absolute-tolerance))]))
+
 (define (nearly=? tolerance x y)
+  (warn "nearly=? is deprecated; use approx=?")
   (< (abs (- x y))
      (/ (max (abs x) (abs y)) tolerance)))
 
