@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.package.compile - compile extensions
 ;;;
-;;;   Copyright (c) 2005-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2005-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -41,7 +41,9 @@
   (use gauche.package.util)
   (use gauche.config)
   (use gauche.parameter)
+  (use gauche.cgen.cise)
   (use gauche.cgen.stub)
+  (use gauche.cgen.precomp)
   (use file.util)
   (export gauche-package-compile-and-link
           gauche-package-compile
@@ -67,6 +69,8 @@
                                           (cflags   #f)
                                           (cc #f)
                                           (gauche-builddir #f)
+                                          (keep-c #f)
+                                          (no-line #f)
                                           (ld #f)      ; dummy
                                           (ldflags #f) ; dummy
                                           (libs #f)    ; dummy
@@ -74,19 +78,29 @@
                                           ((:verbose verb?) #f))
   (parameterize ([dry-run dry?]
                  [verbose-run verb?]
-                 [in-place-dir gauche-builddir])
+                 [in-place-dir gauche-builddir]
+                 [cise-emit-source-line (not no-line)])
     (let1 ofile (or output (sys-basename (path-swap-extension file OBJEXT)))
       (unless (and (file-exists? ofile)
                    (file-mtime>? ofile file))
-        (if (equal? (path-extension file) "stub")
+        (cond
+         [(equal? (path-extension file) "scm")
+          (let1 cfile (path-swap-extension file "c")
+            (unwind-protect
+                (begin (cgen-precompile file)
+                       (do-compile (or cc CC) cfile ofile
+                                   (or cppflags "") (or cflags "")))
+              (unless keep-c (sys-unlink cfile))))]
+         [(equal? (path-extension file) "stub")
           (let1 cfile (path-swap-extension file "c")
             (unwind-protect
                 (begin (cgen-genstub file)
                        (do-compile (or cc CC) cfile ofile
                                    (or cppflags "") (or cflags "")))
-              (sys-unlink cfile)))
+              (unless keep-c (sys-unlink cfile))))]
+         [else
           (do-compile (or cc CC) file ofile
-                      (or cppflags "") (or cflags "")))))))
+                      (or cppflags "") (or cflags ""))])))))
 
 (define (do-compile cc cfile ofile cppflags cflags)
   (run #"~cc -c ~cppflags ~(INCDIR) ~cflags ~CFLAGS -o '~ofile' '~cfile'"))
@@ -95,6 +109,8 @@
                                                 (libs #f)
                                                 (ld #f)
                                                 (gauche-builddir #f)
+                                                (keep-c #f)   ; dummy
+                                                (no-line #f)  ; dummy
                                                 (output #f)   ; dummy
                                                 (cppflags #f) ; dummy
                                                 (cflags #f)   ; dummy
@@ -128,10 +144,6 @@
 
 (define (gauche-package-clean module-name files :key (output #f))
   (when module-name
-    (sys-unlink #"~|module-name|_head.c")
-    (sys-unlink #"~|module-name|_tail.c")
-    (sys-unlink #"~|module-name|_head.~|OBJEXT|")
-    (sys-unlink #"~|module-name|_tail.~|OBJEXT|")
     (sys-unlink #"~|module-name|.~|SOEXT|"))
   (when output
     (sys-unlink output))
@@ -139,6 +151,10 @@
     (unless (equal? (path-extension f) OBJEXT)
       (sys-unlink (sys-basename (path-swap-extension f OBJEXT))))))
 
+;; Adjust pathnames in INCDIR and LIBDIR
+;;   If we're running compiler in-place during testing, we replace
+;;   the paths 'gauche-config --incdir' or 'gauche-config --libdir' returns
+;;   with the source tree directories.
 (define (filter-dir flag olddirs dir-key)
   (define sep (cond-expand [gauche.os.windows ";"][else ":"]))
   (define dirs
@@ -152,4 +168,8 @@
             #"~|new|~|sep|~|to-dir|/gc/include"
             new))
         olddirs))
-  (string-join (map (^s #"'-~|flag|~|s|'") (string-split dirs sep)) " "))
+  ;; Return "-I<path> -I<path> ..." or "-L<path> -L<path> ...".
+  ;; We exclude nonexistent paths, for OSX complains about it.
+  (string-join (map (^s #"'-~|flag|~|s|'")
+                    (filter file-exists? (string-split dirs sep)))
+               " "))
