@@ -1,7 +1,7 @@
 ;;;
 ;;; procedure.scm - auxiliary procedure utilities.  to be autoloaded.
 ;;;
-;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -40,9 +40,6 @@
           arity procedure-arity-includes?
           <arity-at-least> arity-at-least? arity-at-least-value
           source-code source-location disasm
-          generator-fold generator-fold-right
-          generator-for-each generator-map
-          generator-find
           ;; for the backward compatibility
           port-fold port-fold-right port-for-each port-map
           ))
@@ -199,7 +196,8 @@
     (and (pair? x)
          ($ (with-module gauche.internal pair-attribute-get)
             x 'source-info #f)))
-  (or (extract (~ proc'info))
+  (or (and (slot-exists? proc 'info)
+           (extract (~ proc'info)))
       (extract (source-code proc))))
 
 ;; disassembler ------------------------------------------------
@@ -218,7 +216,12 @@
                  (dump (closure-code proc)))]
              info)))
   (cond
-   [(closure? proc) (print "CLOSURE " proc) (dump (closure-code proc))]
+   [(closure? proc) (print "CLOSURE " proc)
+    (dump (closure-code proc))
+    (dolist [e ((with-module gauche.internal %closure-env->list) proc)]
+      (when (closure? e)
+        (print "LIFTED CLOSURE " e)
+        (dump (closure-code e))))]
    [(is-a? proc <method>)
     (print "METHOD " proc)
     (cond [(method-code proc) => dump]
@@ -233,84 +236,10 @@
    [else (print "Disassemble not applicable for " proc)])
   (values))
 
-;; iterate over generated values -----------------------------------------
-;;   GEN is a procedure that yields a series of values, terminated by EOF.
-;;   For example, read-char can be used as GEN.
-
-;; Returns a list of new values generated by each of GENS, having tail
-;; as the tail of the returned list.  It returns #<eof> when any one of
-;; generator returns eof (generation is
-(define (%generate-values gens . tail)
-  (fold-right (^[g tail]
-                (let1 v (g)
-                  (if (or (eof-object? v) (eof-object? tail))
-                    (eof-object)
-                    (cons v tail))))
-              tail gens))
-
-(define (generator-fold fn knil gen . more)
-  (if (null? more)
-    (let loop ([item (gen)]
-               [r    knil])
-      (if (eof-object? item)
-        r
-        (let1 r (fn item r)
-          (loop (gen) r))))
-    (let1 gens (cons gen more)
-      (let loop ([knil knil])
-        (let1 items (%generate-values gens knil)
-          (if (eof-object? items)
-            knil
-            (loop (apply fn items))))))))
-
-;; This will consume large stack if input file is large.
-(define (generator-fold-right fn knil gen . more)
-  (if (null? more)
-    (let loop ([item (gen)])
-      (if (eof-object? item)
-        knil
-        (fn item (loop (gen)))))
-    (let1 gens (cons gen more)
-      (let loop ()
-        (let1 items (%generate-values gens)
-          (if (eof-object? items)
-            knil
-            (apply fn (append items `(,(loop))))))))))
-
-(define (generator-for-each fn gen . more)
-  (if (null? more)
-    (let loop ([item (gen)])
-      (unless (eof-object? item)
-        (fn item) (loop (gen))))
-    (let1 gens (cons gen more)
-      (let loop ()
-        (let1 items (%generate-values gens)
-          (unless (eof-object? items)
-            (apply fn items)
-            (loop)))))))
-
-(define (generator-map fn gen . more)
-  (if (null? more)
-    (let loop ([item (gen)] [r '()])
-      (if (eof-object? item)
-        (reverse r)
-        (let1 r (cons (fn item) r)
-          (loop (gen) r))))
-    (let1 gens (cons gen more)
-      (let loop ([r '()])
-        (let1 items (%generate-values gens)
-          (if (eof-object? items)
-            (reverse r)
-            (loop (cons (apply fn items) r))))))))
-
-(define (generator-find pred gen)
-  (let loop ([item (gen)])
-    (cond [(eof-object? item) #f]
-          [(pred item) item]
-          [else (loop (gen))])))
-
-;; For the backward compatibility
+;; For the backward compatibility ------------------------------------
+(select-module gauche)
 (define port-fold generator-fold)
 (define port-fold-right generator-fold-right)
 (define port-for-each generator-for-each)
 (define port-map generator-map)
+
