@@ -1,7 +1,7 @@
 ;;;
 ;;; redefutil.scm - class redefinition protocol (autoloaded)
 ;;;
-;;;   Copyright (c) 2003-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2003-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -34,7 +34,6 @@
 ;; This file is autoloaded
 
 (select-module gauche.object)
-(use gauche.parameter) ;; used by instance update protocol
 (use srfi-1)           ;; used by instance update protocol
 
 ;;----------------------------------------------------------------
@@ -52,7 +51,7 @@
   (%start-class-redefinition! old) ;; MT safety
   (guard (e [else
              (%commit-class-redefinition! old #f)
-             (warn "Class redefinition of ~S is aborted.  The state of the class may be inconsistent" old)])
+             (warn "Class redefinition of ~S is aborted.  The state of the class may be inconsistent: ~a\n" old (condition-message e))])
     (class-redefinition old new)
     (%commit-class-redefinition! old new)))
 
@@ -92,14 +91,6 @@
          [new-sub  (apply make metaclass
                           (fix-initargs initargs supers metaclass))])
     (redefine-class! sub new-sub)
-    ;; Trick: redefine-class! above removes subclass form original
-    ;; superclass's direct-subclass list, but we want to keep it.
-    ;; so we add to it again.  We can't do this within class-redefinition,
-    ;; for we don't know if it is called on the top of redefinition
-    ;; or in the subclasses' redefinition.
-    (for-each (^[sup] (%add-direct-subclass! sup sub))
-              (class-direct-supers sub))
-    ;; If the subclass has global bindings, replace them.
     (%replace-class-binding! sub new-sub)))
 
 ;;----------------------------------------------------------------
