@@ -1,7 +1,7 @@
 ;;;
 ;;; regexp.scm - auxiliary macros and procedures for regexp.  autoloaded.
 ;;;
-;;;   Copyright (c) 2000-2014  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -34,73 +34,10 @@
 ;; NB: refrain from using util.match, for it could cause nasty dependency
 ;; problem.  (Should be resolved once we incorporate match into the core).
 (define-module gauche.regexp
-  (export rxmatch-let rxmatch-if rxmatch-cond rxmatch-case
-          regexp-unparse <regexp-invalid-ast>))
+  (export regexp-unparse <regexp-invalid-ast>
+          rxmatch-substrings rxmatch-positions))
 (select-module gauche.regexp)
 
-(define-syntax rxmatch-bind*
-  (syntax-rules ()
-    [(rxmatch-bind* ?n ?match () ?form ...)
-     (begin ?form ...)]
-    [(rxmatch-bind* ?n ?match (#f ?vars ...) ?form ...)
-     (rxmatch-bind* (+ ?n 1) ?match (?vars ...) ?form ...)]
-    [(rxmatch-bind* ?n ?match (?var ?vars ...) ?form ...)
-     (let1 ?var (rxmatch-substring ?match ?n)
-       (rxmatch-bind* (+ ?n 1) ?match (?vars ...) ?form ...))]))
-
-(define-syntax rxmatch-let
-  (syntax-rules ()
-    [(rxmatch-let ?expr (?var ...) ?form ...)
-     (cond [?expr => (^(match) (rxmatch-bind* 0 match (?var ...) ?form ...))]
-           [else (error "rxmatch-let: match failed:" '?expr)])]))
-
-(define-syntax rxmatch-if
-  (syntax-rules ()
-    [(rxmatch-if ?expr (?var ...) ?then ?else)
-     (cond [?expr => (^(match) (rxmatch-bind* 0 match (?var ...) ?then))]
-           [else ?else])]))
-
-(define-syntax rxmatch-cond
-  (syntax-rules (test else =>)
-    [(rxmatch-cond) #f]
-    [(rxmatch-cond (else ?form ...))
-     (begin ?form ...)]
-    [(rxmatch-cond (test ?expr => ?obj) ?clause ...)
-     (cond (?expr => ?obj) (else (rxmatch-cond ?clause ...)))]
-    [(rxmatch-cond (test ?expr ?form ...) ?clause ...)
-     (if ?expr (begin ?form ...) (rxmatch-cond ?clause ...))]
-    [(rxmatch-cond (?matchexp ?bind ?form ...) ?clause ...)
-     (rxmatch-if ?matchexp ?bind
-       (begin ?form ...)
-       (rxmatch-cond ?clause ...))]))
-
-(define-syntax rxmatch-case
-  (syntax-rules (test else =>)
-    [(rxmatch-case #t ?temp ?strp) #f]
-    [(rxmatch-case #t ?temp ?strp (else => ?proc))
-     (?proc ?temp)]
-    [(rxmatch-case #t ?temp ?strp (else ?form ...))
-     (begin ?form ...)]
-    [(rxmatch-case #t ?temp ?strp (test ?proc => ?obj) ?clause ...)
-     (cond [(?proc ?temp) => ?obj]
-           [else (rxmatch-case #t ?temp ?strp ?clause ...)])]
-    [(rxmatch-case #t ?temp ?strp (test ?proc ?form ...) ?clause ...)
-     (if (?proc ?temp)
-       (begin ?form ...)
-       (rxmatch-case #t ?temp ?strp ?clause ...))]
-    [(rxmatch-case #t ?temp ?strp (?re ?bind ?form ...) ?clause ...)
-     (rxmatch-if (and ?strp (rxmatch ?re ?temp))
-         ?bind
-       (begin ?form ...)
-       (rxmatch-case #t ?temp ?strp ?clause ...))]
-    [(rxmatch-case #t ?temp ?strip ?clause ...)
-     (syntax-error "malformed rxmatch-case")]
-    ;; main entry
-    [(rxmatch-case ?str ?clause ...)
-     (let* ([temp ?str]
-            [strp (string? temp)])
-       (rxmatch-case #t temp strp ?clause ...))]))
-
 ;; Extract all matches (including entire match).  Allows match to be #f.
 (define (rxmatch-substrings match :optional (start 0) (end #f))
   (if match
@@ -219,10 +156,16 @@
                                [else (disp n)])]
           [(char-set? n) (disp (substring (write-to-string n) 1 -1))]
           [(eq? n 'any)  (disp #\.)]
+          [(eq? n 'bos)  (disp "^^")]   ; made up syntax for reporting only
+          [(eq? n 'eos)  (disp "$$")]   ; made up syntax for reporting only
           [(eq? n 'bol)  (disp #\^)]
           [(eq? n 'eol)  (disp #\$)]
           [(eq? n 'wb)   (disp "\\b")]
+          [(eq? n 'bow)  (disp "\\b(?=\\w)")] ; unsupported pcre syntax
+          [(eq? n 'eow)  (disp "\\b(?<=\\w)")] ; unsupported pcre syntax
           [(eq? n 'nwb)  (disp "\\B")]
+          [(eq? n 'bog)  (disp "<bog>")] ; unsupported syntax
+          [(eq? n 'eog)  (disp "<eog>")] ; unsupported syntax
           [(not (pair? n)) (err "invalid AST node" n)]
           [else (case (car n)
                   [(comp) (rlet1 s (write-to-string (cdr n))
