@@ -1,7 +1,7 @@
 ;;;
 ;;; vecutil.scm - Auxiliary vector utilities.  Autoloaded.
 ;;;
-;;;   Copyright (c) 2013-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2013-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -47,7 +47,7 @@
   (let* ([len (length lis)]
          [end (if (< end 0) len end)])
     (when (< len start)
-      (errorf "start arugment out of range [0-~s]: ~s" len start))
+      (errorf "start argument out of range [0-~s]: ~s" len start))
     (when (< end start)
       (errorf "start ~s is greater than end ~s" start end))
     (do ([vec (make-vector (- end start))]
