@@ -1,7 +1,7 @@
 ;;;
 ;;; http.scm - HTTP 1.1
 ;;;
-;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -50,10 +50,10 @@
   (use rfc.uri)
   (use rfc.base64)
   (use gauche.net)
-  (use gauche.parameter)
   (use gauche.charconv)
   (use gauche.sequence)
   (use gauche.uvector)
+  (use gauche.connection)
   (use util.match)
   (use text.tree)
   (export <http-error>
@@ -82,13 +82,8 @@
           mime-compose-message-string
           mime-compose-parameters
           mime-parse-content-type)
-(autoload rfc.tls
-          make-tls tls-destroy tls-connect tls-input-port
-          tls-output-port tls-close)
-(autoload gauche.process
-          run-process process-input process-output process-error
-          process-wait process-kill)
-
+(autoload rfc.tls make-tls tls-connect)
+(autoload rfc.http.tunnel probe-stunnel make-stunnel-connection)
 (autoload file.util file-size find-file-in-paths null-device)
 
 ;;==============================================================
@@ -238,8 +233,8 @@
   (define extra-headers
     ($ concatenate $ reverse
        $ rlet1 z '()
-       (doplist [(k v) opts]
-                (when v (push! z (list k v))))))
+       (do-plist [(k v) opts]
+         (when v (push! z (list k v))))))
 
   (define conn (ensure-connection server auth-handler auth-user auth-password
                                   proxy secure extra-headers))
@@ -497,10 +492,10 @@
 (define-class <http-connection> ()
   ;; All slots are private.
   ((server :init-keyword :server)       ; server[:port]
-   (socket :init-value #f)              ; A <socket> for persistent connection.
+   (socket :init-value #f)              ; A <connection> object; either <socket>
+                                        ; or wrapped by secure agent.
                                         ; If it is shutdown by the server,
                                         ; the APIs attempt to reconnect.
-   (secure-agent  :init-value #f)       ; See "secure agent handling" below
    (persistent    :init-keyword :persistent)   ; true for persistent connection.
    (auth-handler  :init-keyword :auth-handler  :init-value #f)
    (auth-user     :init-keyword :auth-user     :init-value #f)
@@ -534,10 +529,8 @@
     (unless (and (string=? orig-server new-server)
                  (or (and (equal? proto "https") (~ conn'secure))
                      (and (not (equal? proto "https")) (not (~ conn'secure)))))
-      (shutdown-secure-agent conn)
-      (and-let* ([s (~ conn'socket)])
-        (socket-shutdown s)
-        (socket-close s)
+      (and-let1 c (~ conn'socket)
+        (connection-shutdown (~ conn'socket) 'both)
         (set! (~ conn'socket) #f))
       (set! (~ conn'server) new-server)
       ;; NB: If the server redirects http connection to https, we don't have
@@ -546,6 +539,28 @@
       (set! (~ conn'secure) (and (equal? proto "https") 'tls))))
   conn)
 
+;; API
+(define (reset-http-connection conn)
+  (when (~ conn'socket)
+    (guard (e [(<system-error> e) #f])
+      (connection-shutdown (~ conn'socket) 'both)
+      (connection-close (~ conn'socket)))
+    (set! (~ conn'socket) #f)))
+
+
+;; API
+(define (http-secure-connection-available? :optional (type #t))
+  (case type
+    [(#t tls)
+     (cond-expand
+      [gauche.net.tls #t]
+      [else #f])]
+    [(stunnel)
+     (boolean (probe-stunnel))]
+    [else
+     (error "Unknown secure connection type (must be tls, stunnel or #t):"
+            type)]))
+
 ;;==============================================================
 ;; query and request body composition
 ;;
@@ -707,11 +722,7 @@
                                  either boolean, 'tls or 'stunnel, but got:"
                                 secure)]))))))
 
-(define (reset-http-connection conn)
-  (shutdown-socket-connection conn)
-  (shutdown-secure-agent conn))
-
-(define (start-socket-connection conn)
+(define (start-connection conn)
   ;; If address is given ipv6 format such as "[::1]:port", we have to
   ;; use "::1" part as the hostname, excluding [].
   (define (parse-address addr)
@@ -720,35 +731,34 @@
       [#/^\[([a-fA-F\d:]+)\](?::(\d+))?$/ (_ host port) (values host port #f)]
       [#/^([^:]+)(?::(\d+))?$/ (_ host port) (values host port #f)]
       [else (error "Unrecognized http server address:" addr)]))
-  (receive (host port path) (parse-address (or (~ conn'proxy) (~ conn'server)))
-    (set! (~ conn'socket)
-          (if path
-            (make-client-socket 'unix path)
-            (make-client-socket host (if port
-                                       (x->integer port)
-                                       (if (~ conn'secure) 443 80)))))))
-
-(define (shutdown-socket-connection conn)
-  (when (~ conn'socket)
-    (guard (e [(<system-error> e) #f])
-      (socket-shutdown (~ conn'socket)))
-    (socket-close (~ conn'socket))
-    (set! (~ conn'socket) #f)))
+  (define (connect-socket)
+    (receive (host port path) (parse-address (or (~ conn'proxy) (~ conn'server)))
+      (if path
+        (make-client-socket 'unix path)
+        (make-client-socket host (if port
+                                   (x->integer port)
+                                   (if (~ conn'secure) 443 80))))))
+  (ecase (~ conn'secure)
+    [(#f) (set! (~ conn'socket) (connect-socket))]
+    [(tls)
+     (let1 tls (make-tls :server-name (~ conn'server)
+                         :options (if (tls-ca-bundle-path)
+                                    0
+                                    SSL_SERVER_VERIFY_LATER))
+       (set! (~ conn'socket) (tls-connect tls (connect-socket))))]
+    [(stunnel)
+     (let* ([rhost      (or (~ conn'proxy) (~ conn'server))]
+            [rhost:port (if (string-index rhost #\:) rhost #"~|rhost|:https")])
+       (set! (~ conn'socket) (make-stunnel-connection rhost:port)))]))
 
 (define (with-connection conn proc)
   (unwind-protect
       (begin
-        (unless (~ conn'persistent)
-          (unless (~ conn'socket) (start-socket-connection conn))
-          (when (~ conn'secure) (start-secure-agent conn)))
-        (apply proc (if (~ conn'secure)
-                      `(,(secure-agent-input-port conn)
-                        ,(secure-agent-output-port conn))
-                      `(,(socket-input-port (~ conn'socket))
-                        ,(socket-output-port (~ conn'socket))))))
+        (unless (~ conn'socket) (start-connection conn))
+        (proc (connection-input-port (~ conn'socket))
+              (connection-output-port (~ conn'socket))))
     (unless (~ conn'persistent)
-      (when (~ conn'secure) (shutdown-secure-agent conn))
-      (shutdown-socket-connection conn))))
+      (reset-http-connection conn))))
 
 ;; canonicalize uri for the sake of redirection.
 ;; URI is a request-uri given to the API, or the redirect location specified
@@ -871,104 +881,6 @@
   (begin0 (receiver code headers total callback)
     (when condition (raise condition))))
 
-;;==============================================================
-;; secure agent handling
-;;
-
-;; We provide two options to handle secure connection.
-;;  tls     - using Gauche's rfc.tls module (default)
-;;  stunnel - using external stunnel process
-
-(define (shutdown-secure-agent conn)
-  (when (~ conn'secure-agent)
-    (ecase (~ conn'secure)
-      [(tls)
-       (tls-close (~ conn'secure-agent))
-       (tls-destroy (~ conn'secure-agent))]
-      [(stunnel)
-       (close-output-port (process-input (~ conn'secure-agent)))
-       (or (process-wait (~ conn'secure-agent) #t)
-           (begin (sys-nanosleep #e1e8)    ;0.1s
-                  (process-wait (~ conn'secure-agent) #t))
-           (begin (sys-nanosleep #e1e8)    ;0.1s
-                  (process-kill (~ conn'secure-agent))))])
-    (set! (~ conn'secure-agent) #f)))
-
-(define (start-secure-agent conn)
-  (when (~ conn'secure-agent) (shutdown-secure-agent conn))
-  (ecase (~ conn'secure)
-    [(tls)
-     (let1 tls (make-tls :server-name (~ conn'server))
-       (set! (~ conn'secure-agent) tls)
-       (tls-connect tls (socket-fd (~ conn'socket))))]
-    [(stunnel)
-     (let* ([rhost      (or (~ conn'proxy) (~ conn'server))]
-            [rhost:port (if (string-index rhost #\:) rhost #"~|rhost|:https")])
-       (if-let1 p ((probe-stunnel) rhost:port)
-         (set! (~ conn'secure-agent) p)
-         (error "Couldn't start stunnel subprocess")))]))
-
-(define (secure-agent-input-port conn)
-  (ecase (~ conn'secure)
-    [(tls)     (tls-input-port (~ conn'secure-agent))]
-    [(stunnel) (process-output (~ conn'secure-agent))]))
-
-(define (secure-agent-output-port conn)
-  (ecase (~ conn'secure)
-    [(tls)     (tls-output-port (~ conn'secure-agent))]
-    [(stunnel) (process-input (~ conn'secure-agent))]))
-
-;; for external api
-(define (http-secure-connection-available? :optional (type #t))
-  (case type
-    [(#t tls)
-     (cond-expand
-      [gauche.net.tls #t]
-      [else #f])]
-    [(stunnel)
-     (boolean (probe-stunnel))]
-    [else
-     (error "Unknown secure connection type (must be tls, stunnel or #t):"
-            type)]))
-
-(define probe-stunnel
-  (let1 result #f
-    (define (run-stunnel3 path)
-      (lambda (host:port)
-        (run-process `(,path "-c" "-r" ,host:port) :input :pipe :output :pipe
-                     :error :null :wait #f)))
-    (define (run-stunnel4 path)
-      (lambda (host:port)
-        (receive (in out) (sys-pipe)
-          (rlet1 p (run-process `(,path "-fd" 3)
-                                :redirects `((< 0 stdin)
-                                             (> 1 stdout)
-                                             (> 2 :null)
-                                             (< 3 ,(port-file-number in)))
-                                :wait #f)
-            (format out "client = yes\n")
-            (format out "connect = ~a" host:port)
-            (close-output-port out)))))
-
-    (lambda (:key (force #f))
-      (if (and result (not force))
-        result
-        (and-let* ([path (or (find-file-in-paths "stunnel4") ;ubuntu has this
-                             (find-file-in-paths "stunnel"))]
-                   [p (run-process `(,path "-version")
-                                   :error :pipe :output :pipe)]
-                   [vers (read-line (process-error p))])
-          (process-wait p)
-          (rlet1 proc (cond
-                       [(eof-object? vers) (run-stunnel3 path)]
-                       [(#/stunnel (\d+)\.\d+/ vers)
-                        => (^m (if (>= (x->integer (m 1)) 4)
-                                 (run-stunnel4 path)
-                                 (run-stunnel3 path)))]
-                       [(#/exec failed/i vers) #f]
-                       [else (run-stunnel3 path)])
-            (set! result proc)))))))
-
 ;;==============================================================
 ;; authentication handling
 ;;
