@@ -1,4 +1,4 @@
 ;;
 ;; This is a cushion module to use R7RS (scheme base) from Gauche
 ;;
-(define-module scheme.base (use r7rs))
+(define-module scheme.base (use r7rs-setup))
