@@ -1,7 +1,7 @@
 ;;;
 ;;; srfi-14.scm - character set
 ;;;
-;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -34,7 +34,7 @@
 ;; Basic operators are built in the Gauche kernel.  This module
 ;; defines auxiliary procedures.
 ;;
-;; Some of the procedures are implemented with no optimizaiton
+;; Some of the procedures are implemented with no optimization
 ;; effort.   I included them just for completeness.
 ;; If you think you need faster operation, please optimize it and
 ;; send me a patch.
@@ -51,7 +51,7 @@
 ;;    integer-range->char-set integer-range->char-set!
 
 (define-module srfi-14
-  (export char-set= char-set<= char-set-hash
+  (export char-set= char-set<=
           char-set-cursor char-set-ref char-set-cursor-next end-of-char-set?
           char-set-fold char-set-unfold char-set-unfold!
           char-set-for-each char-set-map
@@ -60,7 +60,7 @@
           ucs-range->char-set ucs-range->char-set!
           integer-range->char-set integer-range->char-set!
           ->char-set
-          char-set-size char-set-count char-set->list char-set->string
+          char-set-count char-set->list char-set->string
           char-set-every char-set-any
           char-set-adjoin char-set-adjoin! char-set-delete char-set-delete!
           char-set-complement char-set-complement!
@@ -78,6 +78,7 @@
 
           ;; The followings are defined in core
           char-set char-set? char-set-contains? char-set-copy
+          char-set-hash char-set-size
           char-set-complement char-set-complement!
           ))
 (select-module srfi-14)
@@ -97,28 +98,7 @@
 (define char-set-complement  (with-module gauche char-set-complement))
 (define char-set-complement! (with-module gauche char-set-complement!))
 
-;; Predefined charsets
-;;  NB: Should char-set:symbol be extended to full Unicode range?
-
-(define char-set:letter+digit (%char-set-predefined 0))  ; ALNUM
-(define char-set:letter      (%char-set-predefined 1))   ; ALPHA
-(define char-set:blank       (%char-set-predefined 2))   ; BLANK
-(define char-set:iso-control (%char-set-predefined 3))   ; CNTRL
-(define char-set:digit       (%char-set-predefined 4))   ; DIGIT
-(define char-set:graphic     (%char-set-predefined 5))   ; GRAPH
-(define char-set:lower-case  (%char-set-predefined 6))   ; LOWER
-(define char-set:printing    (%char-set-predefined 7))   ; PRINT
-(define char-set:punctuation (%char-set-predefined 8))   ; PUNCT
-(define char-set:whitespace  (%char-set-predefined 9))   ; SPACE
-(define char-set:upper-case  (%char-set-predefined 10))  ; UPPER
-(define char-set:title-case  (%char-set-predefined 10))  ; UPPER
-(define char-set:hex-digit   (%char-set-predefined 11))  ; XDIGITS
-
-(define char-set:symbol      (%char-set-add-chars! (char-set)
-                                                   (string->list "$+<=>^`|~")))
-(define char-set:ascii       (%char-set-add-range! (char-set) 0 128))
-(define char-set:empty       (char-set))
-(define char-set:full        (char-set-complement! (char-set)))
+;; Predefined charsets - built-in
 
 ;;-------------------------------------------------------------------
 ;; Comparison
@@ -130,11 +110,6 @@
   (case-lambda [() #t]
                [args (every %char-set<=? args (cdr args))]))
 
-;; I'm not sure this works well.  at least it won't break anything.
-(define (char-set-hash cs :optional (bound #x1fffffff))
-  (fold (^[range val] (modulo (hash (+ val (car range) (cdr range))) bound))
-        0 (%char-set-ranges cs)))
-
 ;;-------------------------------------------------------------------
 ;; Iteration
 ;;
