@@ -4,7 +4,5 @@
 
 ;; Procedures of SRFI-4 are now defined in gauche.uvector.
 
-(define-module srfi-4
-  (extend gauche.uvector)
-  )
+(define-module srfi-4  (extend gauche.uvector))
 
