@@ -43,7 +43,6 @@
 
 (define-module srfi-64
   (use gauche.record)
-  (use gauche.parameter)
   (use gauche.test :prefix test:)
   (export test-begin
           test-end test-assert test-eqv test-eq test-equal
@@ -133,7 +132,8 @@
   (%test-runner-fail-list! runner '())
   (%test-runner-skip-save! runner '())
   (%test-runner-fail-save! runner '())
-  (test-runner-group-stack! runner '()))
+  (test-runner-group-stack! runner '())
+  (test-runner-aux-value! runner #f))
 
 (define (test-runner-group-path runner)
   (reverse (test-runner-group-stack runner)))
@@ -183,9 +183,7 @@
 (define (test-runner-get)
   (let ((r (test-runner-current)))
     (if (not r)
-	(cond-expand
-	 (srfi-23 (error "test-runner not initialized - test-begin missing?"))
-	 (else #t)))
+      (error "test-runner not initialized - test-begin missing?"))
     r))
 
 (define (%test-specifier-matches spec runner)
@@ -481,10 +479,16 @@
 		(if source-line (display (cdr source-line)))
 		(display ": ")))
 	  (display (if (eq? kind 'xpass) "XPASS" "FAIL"))
-	  (if test-name
-	      (begin
-		(display " ")
-		(display (cdr test-name))))
+	  (if test-name	
+            (begin
+              (display " ")
+              (display (cdr test-name))))
+          (newline)
+          (display "  expected:")
+          (write (test-result-ref runner 'expected-value))
+          (newline)
+          (display "    actual:")
+          (write (test-result-ref runner 'actual-value))
 	  (newline)))
     (if (output-port? log)
 	(begin
@@ -553,7 +557,7 @@
 (define-syntax %test-evaluate-with-catch
   (syntax-rules ()
     ((%test-evaluate-with-catch test-expression)
-     (guard (err (else #f)) test-expression))))
+     (guard (err (else (print err) #f)) test-expression))))
 	    
 (define (%test-source-line2 form) ; can be something - later
   '())
@@ -638,7 +642,7 @@
        (%test-comp2body r comp expected expr)))
     ((%test-comp2 comp expected expr)
      (let* ((r (test-runner-get)))
-       (test-result-alist! r '())
+       (test-result-alist! r (list (cons 'test-name 'expr)))
        (%test-comp2body r comp expected expr)))))
 (define-syntax test-equal
   (syntax-rules ()
