@@ -1,7 +1,7 @@
 ;;;
 ;;; css.scm - Generate and parse CSS
 ;;;
-;;;   Copyright (c) 2014-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2014-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -31,11 +31,10 @@
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;
 
-;; EXPERIMENTAL : API may change
-
 (define-module www.css
   (use gauche.generator)
   (use gauche.lazy)
+  (use gauche.unicode)
   (use parser.peg)
   (use util.match)
   (use text.tree)
@@ -56,7 +55,7 @@
 ;;;               | (style-decls <declaration> ...)
 ;;;
 ;;;  <pattern>   : <selector> | (:or <selector> ...)
-;;;  <seletor>   : <simple-selector>
+;;;  <selector>  : <simple-selector>
 ;;;              | <chained-selector>
 ;;;  <chained-selector> : (<simple-selector> . (<op>? . <chained-selector>))
 ;;;  <op>        : > | + | ~
@@ -73,7 +72,7 @@
 ;;;              | ($= <ident> <attrib-value>)           ; E[attrib$=val]
 ;;;              | (:not <negation-arg>)                 ; E:not(s)
 ;;;              | (: <ident>)                           ; E:pseudo-class
-;;;              | (: (<fn> <ident> ...))                ; E:pseudl-class(arg)
+;;;              | (: (<fn> <ident> ...))                ; E:pseudo-class(arg)
 ;;;              | (:: <ident>)                          ; E::pseudo-element
 ;;;  <element-name> : <ident> | *
 ;;;  <attrib-value> : <ident> | <string>
@@ -98,7 +97,8 @@
 ;;;  <at-rule>    : <at-media-rule> | <at-import-rule>
 ;;;                     ; NB: Other at-rules are not supported yet
 ;;;  <at-media-rule>  : (@media (<symbol> ...) <style-rule> ...)
-;;;  <at-import-rule> : (@import <string> (<symbol> ...))
+;;;  <at-import-rule> : (@import <import-path> (<symbol> ...))
+;;;  <import-path>    : <string> | <url>
 ;;;
 ;;; NB: style-decls shouldn't appear in a complete stylesheet, but can
 ;;;     appear when the 'style' attribute of the document is parsed.
@@ -131,11 +131,31 @@
 (define (construct-css sexpr :optional (oport (current-output-port)))
   (define (render top)
     (match top
-      [('style-rule pattern . decls)
-       (write-tree (render-style-rule pattern decls) oport)
+      [((or 'style-rule 'style-decls) _ ...)
+       (write-tree (render-style-rule-or-decl top) oport)
        (newline oport)]
-      [('style-decls . decls)
-       (write-tree (intersperse ";" (map render-decl decls)))]
+      [('@media media . rules)
+       (write-tree `("@media "
+                     ,@(intersperse "," media)
+                     " {"
+                     ,@(map render-style-rule-or-decl rules)
+                     "}\n")
+                   oport)]
+      [('@import path mediaqueries)
+       (write-tree `("@import "
+                     ,(match path
+                        [('url s) (format "url(~s)" s)]
+                        [s        (format "~s" s)])
+                     ,@(if (null? mediaqueries)
+                         '("")
+                         `(" " ,@(intersperse "," mediaqueries)))
+                     ";\n")
+                   oport)]
+      [_ (error "invalid or unsupported sexpr css:" top)]))
+  (define (render-style-rule-or-decl top)
+    (match top
+      [('style-rule pattern . decls) (render-style-rule pattern decls)]
+      [('style-decls . decls) (intersperse ";" (map render-decl decls))]
       [_ (error "invalid or unsupported sexpr css:" top)]))
   (define (render-style-rule pattern decls)
     `(,(render-pattern pattern) "{"
@@ -164,7 +184,7 @@
       [((and (or '= '~= '^= '$= '*=) op) ident value)
        `("[" ,ident ,op ,(render-attrval value) "]")]
       [(':= ident value) `("[" ,ident "|=" ,(render-attrval value) "]")]
-      [(':not not-arg) `(":(" ,(if (symbol? not-arg)
+      [(':not not-arg) `(":not(" ,(if (symbol? not-arg)
                                  not-arg
                                  (render-options not-arg)) ")")]
       [(': (fn arg ...)) `(":" ,(render-fn fn arg))]
@@ -181,7 +201,10 @@
   (define (render-arg arg)
     (match arg
       [#(v ...)  (intersperse " " (map render-attrval v))]
-      [(/ v ...) (intersperse "/" (map render-attrval v))]
+      [(:an+b a b) `(,(if (= a 1) "" a)
+                     n
+                     ,(if (= b 0) "" (format "~@d" b)))]
+      [('/ v ...) (intersperse "/" (map render-attrval v))]
       [v (render-attrval v)]))
 
   (define (render-decl decl)
@@ -258,63 +281,63 @@
 ;;       | (COLON) | (SEMICOLON) | (COMMA) | (OPEN-BRACKET) | (CLOSE-BRACKET)
 ;;       | (OPEN-PAREN) | (CLOSE-PAREN) | (OPEN-BRACE) | (CLOSE-BRACE)
 
-(define %w ($skip-many ($one-of #[ \t\r\n\f])))
-(define %s ($skip-many1 ($one-of #[ \t\r\n\f])))
-(define %nl ($/ ($s "\n") ($s "\r\n") ($s "\n") ($s "\f")))
+(define %w ($many_  ($. #[ \t\r\n\f])))
+(define %s ($many1_ ($. #[ \t\r\n\f])))
+(define %nl ($or ($. "\n") ($. "\r\n") ($. "\n") ($. "\f")))
 ;; comment => vlid
 (define %comment
-  ($seq ($s "/*")
-        ($skip-many ($none-of #[*]))
-        ($skip-many1 ($c #\*))
-        ($skip-many ($seq ($none-of #[/*])
-                          ($skip-many ($none-of #[*]))
-                          ($skip-many1 ($c #\*))))
-        ($c #\/)))
+  ($seq ($. "/*")
+        ($many_ ($none-of #[*]))
+        ($many1_ ($. #\*))
+        ($many_ ($seq ($none-of #[/*])
+                      ($many_ ($none-of #[*]))
+                      ($many1_ ($. #\*))))
+        ($. #\/)))
 
 ;; nonascii [^\0-\237] => char
 (define %nonascii ($none-of #[\x00-\x9f]))
 ;; unicode \\[0-9a-f]{1,6}(\r\n|[ \n\r\t\f])?  => char
-(define %unicode ($do [ ($c #\\) ]
-                      [code ($many ($one-of #[0-9a-fA-F]) 1 6)]
-                      [ ($optional ($/ ($s "\r\n") ($one-of #[ \n\r\t\f]))) ]
-                      ($return ($ integer->char
-                                  $ (cut string->number <> 16)
-                                  $ list->string code))))
+(define %unicode ($let ([ ($. #\\) ]
+                        [code ($many ($. #[0-9a-fA-F]) 1 6)]
+                        [ ($optional ($or ($. "\r\n") ($. #[ \n\r\t\f]))) ])
+                   ($return ($ integer->char
+                               $ (cut string->number <> 16)
+                               $ list->string code))))
 ;; escape {unicode}|\\[^\n\r\f0-9a-f] => char
-(define %escape ($/ %unicode ($seq ($c #\\) ($none-of #[\n\r\f0-9a-f]))))
+(define %escape ($/ %unicode ($seq ($. #\\) ($none-of #[\n\r\f0-9a-f]))))
 ;; nmstart [_a-z]|{nonascii}|{escape} => char
-(define %nmstart ($/ ($one-of #[_a-zA-Z]) %nonascii %escape))
-  
+(define %nmstart ($/ ($. #[_a-zA-Z]) %nonascii %escape))
+
 ;; nonascii [^\0-\237] => char
 (define %nonascii ($none-of #[\x00-\x9f]))
 ;; unicode \\[0-9a-f]{1,6}(\r\n|[ \n\r\t\f])?  => char
-(define %unicode ($do [ ($c #\\) ]
-                      [code ($many ($one-of #[0-9a-fA-F]) 1 6)]
-                      [ ($optional ($/ ($s "\r\n") ($one-of #[ \n\r\t\f]))) ]
-                      ($return ($ integer->char
-                                  $ (cut string->number <> 16)
-                                  $ list->string code))))
+(define %unicode ($let ([ ($. #\\) ]
+                        [code ($many ($. #[0-9a-fA-F]) 1 6)]
+                        [ ($optional ($or ($. "\r\n") ($. #[ \n\r\t\f]))) ])
+                   ($return ($ integer->char
+                               $ (cut string->number <> 16)
+                               $ list->string code))))
 ;; escape {unicode}|\\[^\n\r\f0-9a-f] => char
 (define %escape ($/ %unicode
-                    ($seq ($c #\\) ($none-of #[\n\r\f0-9a-f]))))
+                    ($seq ($. #\\) ($none-of #[\n\r\f0-9a-f]))))
 
 ;; nmstart [_a-z]|{nonascii}|{escape} => char
-(define %nmstart ($/ ($one-of #[_a-zA-Z]) %nonascii %escape))
+(define %nmstart ($/ ($. #[_a-zA-Z]) %nonascii %escape))
 ;; nmchar [_a-z0-9-]|{nonascii}|{escape} => char
-(define %nmchar ($/ ($one-of #[_a-zA-Z0-9-]) %nonascii %escape))
+(define %nmchar ($/ ($. #[_a-zA-Z0-9-]) %nonascii %escape))
 
 ;; name {nmchar}+ => string
 (define %name ($->string ($many1 %nmchar)))
 
 ;; ident [-]?{nmstart}{nmchar}* => string
-(define %ident ($->symbol ($optional ($c #\-)) %nmstart ($many %nmchar)))
+(define %ident ($->symbol ($optional ($. #\-)) %nmstart ($many %nmchar)))
 
-(define %hexdigit ($one-of #[0-9a-fA-F]))
+(define %hexdigit ($. #[0-9a-fA-F]))
 
 ;; num [0-9]+|[0-9]*\.[0-9]+
 (define %num
-  (let* ([digs ($many1 ($one-of #[0-9]))]
-         [frac ($->rope ($c #\.) digs)])
+  (let* ([digs ($many1 ($. #[0-9]))]
+         [frac ($->rope ($. #\.) digs)])
     ($lift ($ string->number $ rope->string $)
            ($/ ($->rope digs ($optional frac))
                ($->rope frac)))))
@@ -324,49 +347,49 @@
 ;; string2 \'([^\n\r\f\\']|\\{nl}|{escape})*\'
 (define %string
   ($->rope
-   ($/ ($between ($char #\")
+   ($/ ($between ($. #\")
                  ($many ($/ ($none-of #[\n\r\f\"\\])
-                            ($try ($seq ($char #\\) %nl))
+                            ($seq ($. #\\) %nl)
                             %escape))
-                 ($or ($char #\") eof))
-       ($between ($char #\')
+                 ($or ($. #\") ($eos)))
+       ($between ($. #\')
                  ($many ($/ ($none-of #[\n\r\f\'\\])
-                            ($try ($seq ($char #\\) %nl))
+                            ($seq ($. #\\) %nl)
                             %escape))
-                 ($or ($char #\') eof)))))
+                 ($or ($. #\') ($eos))))))
 
 (define %bad-string
-  ($/ ($seq ($char #\")
-            ($skip-many ($/ ($none-of #[\n\r\f\"])
-                            ($try ($seq ($char #\\) %nl))
-                            %escape))
-            ($char #\newline))
-      ($seq ($char #\')
-            ($skip-many ($/ ($none-of #[\n\r\f\'])
-                            ($try ($seq ($char #\\) %nl))
-                            %escape))
-            ($char #\newline))))
+  ($/ ($seq ($. #\")
+            ($many_ ($/ ($none-of #[\n\r\f\"])
+                        ($seq ($. #\\) %nl)
+                        %escape))
+            ($. #\newline))
+      ($seq ($. #\')
+            ($many_ ($/ ($none-of #[\n\r\f\'])
+                        ($seq ($. #\\) %nl)
+                        %escape))
+            ($. #\newline))))
 
 (define %url
   ($lift (^[_ _ r _ _] (rope->string r))
-         ($string "url(") %w
+         ($. "url(") %w
          ($->rope ($/ %string
                       ($many ($/ ($one-of #[!#$%&*-~])
                                  %nonascii
                                  %escape))))
-         %w ($char #\))))
+         %w ($. #\))))
 
 (define %unicode-range
   (letrec [(code (^[cs] (string->number (list->string cs) 16)))]
-    ($seq ($one-of #[Uu]) ($c #\+)
+    ($seq ($. #[Uu]) ($. #\+)
           ($/ ($lift (^[cs1 _ cs2] `(UNICODE-RANGE ,(code cs1) ,(code cs2)))
-                     ($many %hexdigit 1 6) ($c #\-) ($many %hexdigit 1 6))
-              ($do [ds ($many %hexdigit 1 5)]
-                   [ws ($many ($c #\?) 1 (- 6 (length ds)))]
-                   ($return
-                    `(UNICODE-RANGE ,(code (append ds (map (^_ #\0) ws)))
-                                    ,(max (code (append ds (map (^_ #\f) ws)))
-                                          #x10ffff))))
+                     ($many %hexdigit 1 6) ($. #\-) ($many %hexdigit 1 6))
+              ($let* ([ds ($many %hexdigit 1 5)]
+                      [ws ($many ($. #\?) 1 (- 6 (length ds)))])
+                ($return
+                 `(UNICODE-RANGE ,(code (append ds (map (^_ #\0) ws)))
+                                 ,(max (code (append ds (map (^_ #\f) ws)))
+                                       #x10ffff))))
               ($lift (^[cs] `(UNICODE-RANGE ,(code cs) ,(code cs)))
                      ($many %hexdigit 1 6))))))
 
@@ -377,35 +400,35 @@
   ($/ ($seq %comment ($return :comment))
       ($seq %s ($return '(WHITESPACE)))
       %unicode-range
-      ($seq ($s "~=")   ($return '(INCLUDE-MATCH)))
-      ($seq ($s "|=")   ($return '(DASH-MATCH)))
-      ($seq ($s "^=")   ($return '(PREFIX-MATCH)))
-      ($seq ($s "$=")   ($return '(SUFFIX-MATCH)))
-      ($seq ($s "*=")   ($return '(SUBSTRING-MATCH)))
-      ($seq ($s "||")   ($return '(COLUMN)))
-      ($seq ($s "<!--") ($return '(CDO)))
-      ($seq ($s "-->")  ($return '(CDC)))
-      ($seq ($c #\:)    ($return '(COLON)))
-      ($seq ($c #\;)    ($return '(SEMICOLON)))
-      ($seq ($c #\,)    ($return '(COMMA)))
-      ($seq ($c #\[)    ($return '(OPEN-BRACKET)))
-      ($seq ($c #\])    ($return '(CLOSE-BRACKET)))
-      ($seq ($c #\()    ($return '(OPEN-PAREN)))
-      ($seq ($c #\))    ($return '(CLOSE-PAREN)))
-      ($seq ($c #\{)    ($return '(OPEN-BRACE)))
-      ($seq ($c #\})    ($return '(CLOSE-BRACE)))
+      ($seq ($. "~=")   ($return '(INCLUDE-MATCH)))
+      ($seq ($. "|=")   ($return '(DASH-MATCH)))
+      ($seq ($. "^=")   ($return '(PREFIX-MATCH)))
+      ($seq ($. "$=")   ($return '(SUFFIX-MATCH)))
+      ($seq ($. "*=")   ($return '(SUBSTRING-MATCH)))
+      ($seq ($. "||")   ($return '(COLUMN)))
+      ($seq ($. "<!--") ($return '(CDO)))
+      ($seq ($. "-->")  ($return '(CDC)))
+      ($seq ($. #\:)    ($return '(COLON)))
+      ($seq ($. #\;)    ($return '(SEMICOLON)))
+      ($seq ($. #\,)    ($return '(COMMA)))
+      ($seq ($. #\[)    ($return '(OPEN-BRACKET)))
+      ($seq ($. #\])    ($return '(CLOSE-BRACKET)))
+      ($seq ($. #\()    ($return '(OPEN-PAREN)))
+      ($seq ($. #\))    ($return '(CLOSE-PAREN)))
+      ($seq ($. #\{)    ($return '(OPEN-BRACE)))
+      ($seq ($. #\})    ($return '(CLOSE-BRACE)))
       ($lift (^[val] `(URL . ,val)) %url)
-      ($lift (^[ident _] `(FUNCTION . ,ident)) %ident ($c #\())
+      ($lift (^[ident _] `(FUNCTION . ,ident)) %ident ($. #\())
       ($lift (^[ident] `(IDENT . ,ident)) %ident)
-      ($lift (^[_ ident] `(AT-KEYWORD . ,ident)) ($c #\@) %ident)
-      ($lift (^[_ val] `(HASH ,val #t)) ($c #\#) %ident)
-      ($lift (^[_ val] `(HASH ,val #f)) ($c #\#) %name)
+      ($lift (^[_ ident] `(AT-KEYWORD . ,ident)) ($. #\@) %ident)
+      ($lift (^[_ val] `(HASH ,val #t)) ($. #\#) %ident)
+      ($lift (^[_ val] `(HASH ,val #f)) ($. #\#) %name)
       ($lift (^[val] `(STRING . ,val)) %string)
       ($lift (^[_] '(BAD-STRING)) %bad-string)
-      ($lift (^[val _] `(PERCENTAGE . ,val)) %num ($c #\%))
+      ($lift (^[val _] `(PERCENTAGE . ,val)) %num ($. #\%))
       ($lift (^[val unit] `(DIMENSION ,val ,unit)) %num %ident)
       ($lift (^[val] `(NUMBER . ,val)) %num)
-      ($do [c anychar] ($return `(DELIM . ,c)))))
+      ($let ([c ($any)]) ($return `(DELIM . ,c)))))
 
 (define (css-token-generator chars)
   (define cs chars)
@@ -442,7 +465,7 @@
     (return-failure/unexpect "EOF" s)
     (return-result (car s) (cdr s))))
 
-(define %WS* ($skip-many ($tok 'WHITESPACE)))
+(define %WS* ($many_ ($tok 'WHITESPACE)))
 
 (define %component-value
   ($lazy
@@ -453,7 +476,7 @@
 
 (define %component-value+  ; whitespace preserving
   ($lazy ($/ ($seq ($tok 'WHITESPACE) ($return '(WHITESPACE)))
-             %brace-block %paren-block %bracket-block %function-call
+             %brace-block %paren-block %bracket-block %function-call+
              %preserved-token)))
 
 (define (block-parser opener closer tag :optional (preserve-ws? #f))
@@ -476,6 +499,12 @@
          ($many ($seq ($not ($tok 'CLOSE-PAREN)) %component-value))
          ($tok 'CLOSE-PAREN)))
 
+(define %function-call+
+  ($lift (^[fn args _] `(funcall ,fn ,@args))
+         ($tok 'FUNCTION)
+         ($many ($seq ($not ($tok 'CLOSE-PAREN)) %component-value+))
+         ($tok 'CLOSE-PAREN)))
+
 (define %at-rule
   ($lift (^[keyword _ args block] `(,keyword ,args ,block))
          ($tok 'AT-KEYWORD) %WS*
@@ -495,13 +524,13 @@
          %brace-block %WS*))
 
 (define %important
-  ($do [ ($delim #\!) ]
-       [ %WS* ]
-       [v ($tok 'IDENT)]
-       (if (string-ci=? (symbol->string v) "important")
-         ($return '!important)
-         ($fail "!important expected"))))
-        
+  ($let ([ ($delim #\!) ]
+         [ %WS* ]
+         [v ($tok 'IDENT)])
+    (if (string-ci=? (symbol->string v) "important")
+      ($return '!important)
+      ($fail "!important expected"))))
+
 (define %declaration
   ($lift (^[name _ _ block important _] (list name block important))
          ($tok 'IDENT) %WS* ($tok 'COLON)
@@ -517,11 +546,11 @@
                   ($lift list %at-rule %declaration-list)))))
 
 (define %inter-rule-spaces
-  ($skip-many ($/ ($tok 'CDO) ($tok 'CDC) ($tok 'WHITESPACE))))
+  ($many_ ($/ ($tok 'CDO) ($tok 'CDC) ($tok 'WHITESPACE))))
 
 (define %rule-list
   ($many ($/ ($tok 'WHITESPACE) %qualified-rule %at-rule)))
-               
+
 ;;
 ;; Higher-level parser
 ;;
@@ -562,10 +591,10 @@
   ($lift (^[_ class] `(class ,class)) ($delim #\.) ($tok 'IDENT)))
 
 (define %id-selector
-  ($do [hashval ($tok 'HASH)]
-       (if (cadr hashval) ; hashval is ident
-         ($return `(id ,(car hashval)))
-         ($fail "identifier required for id selector"))))
+  ($let ([hashval ($tok 'HASH)])
+    (if (cadr hashval) ; hashval is ident
+      ($return `(id ,(car hashval)))
+      ($fail "identifier required for id selector"))))
 
 (define %pseudo-fn-arg
   ;; NB: To support an+b syntax
@@ -579,13 +608,100 @@
    ($/ %type-selector %attr-selector %class-selector %id-selector
        %pseudo-selector)))
 
+(define %sign
+  ($or ($seq ($delim #\+) ($return +))
+       ($seq ($delim #\-) ($return -))))
+
+(define %optional-sign
+  ($optional %sign +))
+
+(define-syntax %match1
+  (syntax-rules ()
+    [(_ pat body0 body ...)
+     (lambda (s)
+       (if (pair? s)
+         (match (car s)
+           [pat ((begin body0 body ...) (cdr s))]
+           [_ (return-failure/expect (write-to-string 'pat) s)])
+         (return-failure/expect (write-to-string 'pat) s)))]
+    [(_ pat)
+     (lambda (s)
+       (if (pair? s)
+         (match (car s)
+           [pat (return-result (car s) (cdr s))]
+           [_ (return-failure/expect (write-to-string 'pat) s)])
+         (return-failure/expect (write-to-string 'pat) s)))]))
+
+(define (symbol-foldcase sym)
+  (string->symbol
+   (string-foldcase
+    (symbol->string sym))))
+
+(define (%one-of-ident-ci idents)
+  (%match1 ('IDENT . (and ident
+                          (= symbol-foldcase
+                             (? (cut memq <> idents)))))
+           ($return ident)))
+
+(define (%%an+b a b)
+  ($return
+   (if (= a 0)
+       b
+       `(:an+b ,a ,b))))
+
+(define (%n-b a n-b)
+  (rxmatch-if (#/^n(-\d+)$/ (symbol->string n-b))
+      (_ b)
+     (%%an+b a (string->number b))
+    ($fail "n-b")))
+
+(define %an+b
+  ($between %WS*
+            ($/ ($let ([sign %optional-sign])
+                  ($or ($let ([a ($or (%match1 ('DIMENSION a (or 'n 'N))
+                                               ($return a))
+                                      ($seq (%one-of-ident-ci '(n))
+                                            ($return 1))
+                                      ($seq (%one-of-ident-ci '(-n))
+                                            ($return -1)))]
+                              [b ($optional
+                                  ($lift (cut <> <>)
+                                         ($between %WS* %sign %WS*)
+                                         ($tok 'NUMBER))
+                                  0)])
+                         (%%an+b (sign a) b))
+                       ($let ([a ($or (%match1 ('DIMENSION a (or 'n- 'N-))
+                                               ($return a))
+                                      ($seq (%one-of-ident-ci '(n-))
+                                            ($return 1))
+                                      ($seq (%one-of-ident-ci '(-n-))
+                                            ($return -1)))]
+                              %WS*
+                              [b ($tok 'NUMBER)])
+                         (%%an+b (sign a) (- b)))
+                       (%match1 ('DIMENSION a n-b)
+                                (%n-b (sign a) n-b))
+                       (%match1 ('IDENT . n-b)
+                                (if-let1 n-b* (symbol-sans-prefix n-b '-)
+                                  (%n-b -1 n-b*)
+                                  (%n-b 1 n-b)))))
+                ($lift (cut <> <>)
+                       %optional-sign
+                       ($tok 'NUMBER))
+                (%one-of-ident-ci '(even odd)))
+            %WS*))
+
 ;; NB: negation-selector is also treated in pseudo-fn
 (define (%pseudo-fn s)
   (match s
     [(('funcall name . arg-tokens) . rest)
-     (receive (r v s) (if (eq? name 'not)
-                        (%negation-selector-arg arg-tokens)
-                        (%pseudo-fn-arg arg-tokens))
+     (receive (r v s) (case (symbol-foldcase name)
+                        [(not)
+                         (%negation-selector-arg arg-tokens)]
+                        [(nth-child nth-last-child nth-of-type nth-last-of-type)
+                         (%an+b arg-tokens)]
+                        [else
+                         (%pseudo-fn-arg arg-tokens)])
        (if (and (parse-success? r)
                 (null? s))
          (return-result (list name v) rest)
@@ -597,7 +713,7 @@
            (match sel
              [('not not-arg) `(:not ,not-arg)]
              [(? symbol?) `(,(if elem? ':: ':) ,sel)]
-             [_ `(,(if elem? ':: ':) ,@sel)]))
+             [_ `(,(if elem? ':: ':) ,sel)]))
          ($tok 'COLON) ($optional ($tok 'COLON))
          ($/ ($tok 'IDENT) %pseudo-fn)))
 
@@ -628,8 +744,8 @@
                                      ($seq ($delim #\+) ($return '+))
                                      ($seq ($delim #\~) ($return '~)))
                                  %WS*)
-                       ($followed-by ($seq ($tok 'WHITESPACE) ($return #f))
-                                     %WS*))
+                       ($seq0 ($seq ($tok 'WHITESPACE) ($return #f))
+                              %WS*))
                    %selector-seq))))))
 
 (define %selector-group  ;; section 5
@@ -644,8 +760,9 @@
 ;; it as a selector and returns <pattern> structure of S-expr CSS.
 ;; If it can't be parsed, returns #f.
 (define (parse-selectors tokens)
-  (receive (val next) (peg-run-parser %selector-group tokens)
-    (and (null? next) val)))
+  (guard (e [(<parse-error> e) #f])
+    (receive (val next) (peg-run-parser %selector-group tokens)
+      (and (null? next) val))))
 
 ;; default declaration value parser
 ;; This part roughly follows CSS2.1 spec appendix G.
@@ -703,8 +820,8 @@
 (define (%decl-value property)
   (if (memq property *juxta-properties*)
     %juxtaposed-exprs
-    ($do [exprs %comma-separated-exprs]
-         ($return (match exprs [(x) x] [(xs ...) `(:or ,@xs)])))))
+    ($let ([exprs %comma-separated-exprs])
+      ($return (match exprs [(x) x] [(xs ...) `(:or ,@xs)])))))
 
 ;; val :: (property-name . tokens)
 (define (parse-declaration val)
@@ -770,15 +887,15 @@
       (and (eq? at-keyword 'import)
            (match args
              [(url . media)
-              (and-let* ([url-string (match url
-                                       [('STRING . s) s]
-                                       [('URL . s) s]
-                                       [_ #f])]
+              (and-let* ([import-path (match url
+                                        [('STRING . s) s]
+                                        [('URL . s) `(url ,s)]
+                                        [_ #f])]
                          [media-kinds (filter-map (^x (match x
                                                         [('IDENT . s) s]
                                                         [_ #f]))
                                                   media)])
-                `(@import ,url-string ,media-kinds))]
+                `(@import ,import-path ,media-kinds))]
              [_ #f]))
       (begin
         (css-parser-warn "Ignored unsupported at-rule: ~s\n"
@@ -788,7 +905,7 @@
 ;; Integrated stylesheet parser
 (define (make-stylesheet-parser qualified-rule-handler at-rule-handler)
   ($seq %inter-rule-spaces
-        ($many ($followed-by
+        ($many ($seq0
                 ($/ ($lift (^[a] (at-rule-handler (car a) (cadr a) (caddr a)
                                                   qualified-rule-handler))
                            %at-rule)
@@ -830,7 +947,7 @@
 (define %selector-only
   ($seq %WS*
         ($many ($seq ($not ($tok 'OPEN-BRACE))
-                     ($/ %paren-block %bracket-block %function-call
+                     ($/ %paren-block %bracket-block %function-call+
                          %preserved-token ($tok 'WHITESPACE))))))
 
 ;; TODO: make this customizable
