@@ -1,7 +1,7 @@
 ;;;
 ;;; file/util.scm - filesystem utility functions
 ;;;
-;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -39,7 +39,6 @@
 #!no-fold-case
 
 (define-module file.util
-  (use srfi-1)
   (use srfi-13)
   (use util.match)
   (use gauche.parameter)
@@ -47,6 +46,7 @@
           home-directory temporary-directory
           make-directory* create-directory* remove-directory* delete-directory*
           copy-directory*
+          call-with-temporary-file call-with-temporary-directory
           create-directory-tree check-directory-tree
           build-path resolve-path expand-path simplify-path decompose-path
           absolute-path? relative-path? find-file-in-paths
@@ -66,11 +66,14 @@
           remove-files delete-files
           null-device console-device
           file->string file->string-list file->list file->sexp-list
+          file->bytevector
           string->file string-list->file list->file sexp-list->file
           <lock-file-failure> with-lock-file
           ))
 (select-module file.util)
 
+(autoload gauche.uvector port->uvector)
+
 ;; Common util.  Returns #f if PATH does not exist.
 
 (define (safe-stat path follow-link?)
@@ -112,6 +115,21 @@
 (define temporary-directory
   (make-parameter (sys-tmpdir)))
 
+(define (call-with-temporary-file proc
+                                  :key (directory (temporary-directory))
+                                       (prefix "gtemp"))
+  (receive (oport name) (sys-mkstemp (build-path directory prefix))  
+    (unwind-protect (proc oport name)
+      (close-output-port oport)
+      (sys-unlink name))))
+
+(define (call-with-temporary-directory proc 
+                                       :key (directory (temporary-directory))
+                                            (prefix "gtemp"))
+  (let1 dir (sys-mkdtemp (build-path directory prefix))
+    (unwind-protect (proc dir)
+      (remove-directory* dir))))
+
 (define (null-device)
   (cond-expand
    [gauche.os.windows "NUL"]
@@ -184,18 +202,26 @@
 
 ;; mkdir -p
 (define (make-directory* dir :optional (mode #o755))
+  (define (err-notdir p)
+    (errorf "non-directory ~s is found while creating a directory ~s"
+            (sys-basename p) dir))
+    
   (define (rec p)
     (if (file-exists? p)
-      (unless (file-is-directory? p)
-        (errorf "non-directory ~s is found while creating a directory ~s"
-                (sys-basename p) dir))
+      (unless (file-is-directory? p) (err-notdir p))
       (let1 d (sys-dirname p)
         (rec d)
         (unless (file-is-writable? d)
           (errorf "directory ~s unwritable during creating a directory ~s"
                   d dir))
         (unless (equal? (sys-basename p) ".") ; omit the last component in "/a/b/c/."
-          (sys-mkdir p mode))
+          ;; Directory p might have been created by other process after
+          ;; we checked it doesn't exist.  We rescue such case.
+          (guard (e [(and (is-a? e <system-error>)
+                          (eqv? (~ e'errno) EEXIST))
+                     (unless (file-is-directory? p) (err-notdir p))]
+                    [else (raise e)])
+            (sys-mkdir p mode)))
         )))
   ;; some platform complains the last "/"
   (rec (string-trim-right dir #[/])))
@@ -819,6 +845,9 @@
 (define (file->string file . opts)
   (apply call-with-input-file file (%maybe port->string) opts))
 
+(define (file->bytevector file . opts)
+  (apply call-with-input-file file (%maybe port->uvector) opts))
+
 (define (file->list reader file . opts)
   (apply call-with-input-file file (%maybe (cut port->list reader <>)) opts))
 
@@ -885,7 +914,7 @@
     (case type
       [(file)
        (with-output-to-file path
-         (^() (print (sys-getpid)) (sys-chmod path perms))
+         (^() (print (sys-getpid)) (sys-chmod path perms) #t)
          :if-exists #f)]
       [(directory)
        (guard (e [(and (<system-error> e) (eqv? (~ e'errno) EEXIST)) #f])
