@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.generator - generator framework
 ;;;
-;;;   Copyright (c) 2011-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2011-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -32,7 +32,6 @@
 ;;;
 
 (define-module gauche.generator
-  (use srfi-1)
   (use gauche.sequence)
   (use gauche.partcont)
   (export list->generator vector->generator reverse-vector->generator
@@ -54,7 +53,8 @@
 
           null-generator gcons* gappend gflatten
           gconcatenate gmerge ggroup
-          circular-generator gunfold giota grange gindex gselect ginterval
+          circular-generator gunfold giterate giterate1
+          giota grange gindex gselect ginterval
           gmap gmap-accum gfilter gremove gdelete gdelete-neighbor-dups
           gfilter-map gstate-filter gbuffer-filter
           gtake gtake* gdrop gtake-while gdrop-while grxmatch gslices
@@ -163,7 +163,19 @@
 (define-method x->generator ((obj <vector>)) (vector->generator obj))
 (define-method x->generator ((obj <string>)) (string->generator obj))
 (define-method x->generator ((obj <integer>)) (bits->generator obj))
-
+(define %marker (cons #f #f))
+(define-method x->generator ((obj <hash-table>))
+  (let1 iter ((with-module gauche.internal %hash-table-iter) obj)
+    (^[] (receive (k v) (iter %marker)
+           (if (eq? k %marker)
+             (eof-object)
+             (cons k v))))))
+(define-method x->generator ((obj <tree-map>))
+  (let1 iter ((with-module gauche.internal %tree-map-iter) obj)
+    (^[] (receive (k v) (iter %marker #f)
+           (if (eq? k %marker)
+             (eof-object)
+             (cons k v))))))
 (define-method x->generator ((obj <collection>))
   (generate
    (^[yield]
@@ -219,6 +231,22 @@
                 (tail)]
                [else (%begin0 (f seed) (set! seed (g seed)))]))))
 
+;; special case of gunfold, for speed and convenience.
+;;  giterate f seed == gunfold (^_ #f) identity f seed
+;; basically, it generates [seed, (f seed), (f (f seed)), ...]
+;; cf. stream-iterate (scheme.stream)
+(define (giterate f seed)
+  (let ([seed seed]
+        [first? #t])
+    (^[] (if first?
+           (begin (set! first? #f) seed)
+           (begin (set! seed (f seed)) seed)))))
+
+;; variation of giterate, where seed itself is not included.
+;; this is ~10% faster than giterate, mainly because this one is so simple.
+(define (giterate1 f seed)
+  (^[] (set! seed (f seed)) seed))
+
 (define (giota :optional (count +inf.0) (start 0) (step 1))
   (if (and (exact? start) (exact? step))
     (let1 val start
@@ -240,15 +268,25 @@
 (define (grange :optional (start 0) (end +inf.0) (step 1))
   (if (and (exact? start) (exact? step))
     (let1 val start
-      (^[] (if (>= val end)
-             (eof-object)
-             (%begin0 val (inc! val step)))))
+      (if (>= step 0)
+        (^[] (if (>= val end)
+               (eof-object)
+               (%begin0 val (inc! val step))))
+        (^[] (if (<= val end)
+               (eof-object)
+               (%begin0 val (inc! val step))))))
     (let ([val (inexact start)]
           [k   0])
-      (^[] (let1 v (+ val (* k step))
-             (if (>= v end)
-               (eof-object)
-               (begin (inc! k) v)))))))
+      (if (>= step 0)
+        (^[] (let1 v (+ val (* k step))
+               (if (>= v end)
+                 (eof-object)
+                 (begin (inc! k) v))))
+        (^[] (let1 v (+ val (* k step))
+               (if (<= v end)
+                 (eof-object)
+                 (begin (inc! k) v))))))))
+        
 
 ;;;
 ;;; Generator operations
@@ -632,8 +670,7 @@
 ;;; Consumers
 ;;;
 
-;; NB: The following consumers are in autoloaded lib/gauche/procedure.scm
-;; for historical reasons:
+;; NB: The following consumers are built-in (libproc.scm)
 ;; generator-fold generator-fold-right
 ;; genergenerator-for-each generator-map generator-find 
 
