@@ -1,7 +1,7 @@
 ;;;
 ;;; sequence.scm - sequence operations
 ;;;
-;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -36,7 +36,6 @@
 ;; See also gauche.collection, that defines various mapping functions.
 
 (define-module gauche.sequence
-  (use srfi-1)
   (extend gauche.collection)
   (export referencer modifier subseq
           fold-right
@@ -57,6 +56,7 @@
 
 (define-method referencer ((obj <list>))   list-ref)
 (define-method referencer ((obj <vector>)) vector-ref)
+(define-method referencer ((obj <bitvector>)) bitvector-ref/int)
 (define-method referencer ((obj <weak-vector>)) weak-vector-ref)
 (define-method referencer ((obj <string>)) string-ref)
 
@@ -76,6 +76,7 @@
 
 (define-method modifier ((obj <list>)) list-set!)
 (define-method modifier ((obj <vector>)) vector-set!)
+(define-method modifier ((obj <bitvector>)) bitvector-set!)
 (define-method modifier ((obj <weak-vector>)) weak-vector-set!)
 (define-method modifier ((obj <string>)) string-set!)
 
@@ -102,7 +103,6 @@
   (subseq seq start (size-of seq)))
 
 (define-method subseq ((seq <sequence>) start end)
-  (when (< end 0) (set! end (modulo end (size-of seq))))
   (when (> start end)
     (errorf "start ~a must be smaller than or equal to end ~a" start end))
   (let1 size (- end start)
@@ -110,6 +110,11 @@
       (with-iterator (seq end? next :start start)
         (dotimes [i size (get)] (add! (next)))))))
 
+(define-method subseq ((seq <vector>) . args)
+  (apply vector-copy seq args))
+(define-method subseq ((seq <bitvector>) . args)
+  (apply bitvector-copy seq args))
+
 (define-method (setter subseq) ((seq <sequence>) start vals)
   (with-iterator (vals end? next)
     (do ([index start (+ index 1)])
@@ -123,6 +128,11 @@
       (when (end?) (error "not enough values for (setter subseq)" vals))
       (set! (ref seq index) (next)))))
 
+(define-method (setter subseq) ((seq <vector>) start end (vals <vector>))
+  (vector-copy! seq start vals 0 (- end start)))
+(define-method (setter subseq) ((seq <bitvector>) start end (vals <bitvector>))
+  (bitvector-copy! seq start vals 0 (- end start)))
+
 ;; fold-right --------------------------------------------
 
 ;;  (proc e1 (proc e2 ... (proc eN seed)))
