@@ -171,6 +171,9 @@
 (define match-define.  (%match-id 'match-define))
 (define match:length>=?. (%match-id 'match:length>=?))
 
+(define lambda.        ((with-module gauche.internal make-identifier) 'lambda
+                        (find-module 'null) '()))
+
 (define-inline (match:length>=? n) (cut length>=? <> n))
 
 ;;; [SK] End of black magic
@@ -191,12 +194,10 @@
 (define match:set-error-control
   (lambda (v) (set! match:error-control v)))
 
-(define-inline (symid? x) (or (symbol? x) (identifier? x)))
-
 (define match:disjoint-predicates
   (cons 'null
         '(pair?
-          symid?
+          identifier?
           boolean?
           number?
           string?
@@ -227,14 +228,14 @@
                                          (pair? (cadr c))
                                          (equal? (caadr c) '=>)
                                          (pair? (cdadr c))
-                                         (symid? (cadadr c))
+                                         (identifier? (cadadr c))
                                          (null? (cddadr c))
                                          (pair? (cddr c))
                                          (cadadr c)))
                               (bv2 (if fail (cons fail bv) bv))
                               (body (if fail (cddr c) (cdr c))))
                          (set! blist
-                               (cons `(,code (lambda ,bv2 ,@body))
+                               (cons `(,code (,lambda. ,bv2 ,@body))
                                      (append bindings blist)))
                          (list p code bv (and fail (gensym)) #f)))
                      clauses))
@@ -257,7 +258,7 @@
     (unreachable plist match-expr)
     `(letrec (,@(map (lambda (v) `(,v #f)) bv)
               (,x ,exp)
-              (,code (lambda ,gs
+              (,code (,lambda. ,gs
                        ,@(map (lambda (v g) `(set! ,v ,g)) bv gs)
                        ,@body))
               ,@bindings
@@ -278,7 +279,7 @@
     (unreachable plist match-expr)
     `(begin ,@(map (lambda (v) `(define ,v #f)) bv)
             (let ((,x ,exp)
-                  (,code (lambda ,gs
+                  (,code (,lambda. ,gs
                            ,@(map (lambda (v g) `(set! ,v ,g)) bv gs)
                            (cond (#f #f))))
                   ,@bindings
@@ -286,20 +287,17 @@
               ,m))))
 
 (define (symbolize x)
-  (cond [(symbol? x) x]
-        [(identifier? x) (unwrap-syntax x)]
-        [else x]))
+  (if (identifier? x) (identifier->symbol x) x))
 
 (define (pattern-var? x)
-  (and-let* ([x (cond [(symbol? x) x]
-                      [(identifier? x) (unwrap-syntax x)]
-                      [else #f])]
+  (and-let* ([x (and (identifier? x)
+                     (identifier->symbol x))]
              [ (not (dot-dot-k? x)) ])
     (not (memq x '(quasiquote quote unquote unquote-splicing
                    ? _ $ struct @ object = and or not set! get! ... ___)))))
 
 (define (dot-dot-k? s)
-  (and (symid? s)
+  (and (identifier? s)
        (if (member s '(... ___))
          0
          (and-let* ([s (if (symbol? s) s (unwrap-syntax s))]
@@ -315,7 +313,7 @@
    ((eq? match:error-control 'match)
     (let ((errf (gensym))
           (arg (gensym)))
-      (cons `((,errf (lambda (,arg)
+      (cons `((,errf (,lambda. (,arg)
                        (,match:error. ,arg ',match-expr))))
             (lambda (x) `(,errf ,x)))))
    (else (match:syntax-err
@@ -373,11 +371,11 @@
              `(not ,@(map ordinary (cdr p)))
              (cons-ordinary (car p) (cdr p))))
           (($ struct)
-           (if (and (pair? (cdr p)) (symid? (cadr p)) (list? (cddr p)))
+           (if (and (pair? (cdr p)) (identifier? (cadr p)) (list? (cddr p)))
              `($ ,(cadr p) ,@(map ordinary (cddr p)))
              (cons-ordinary (car p) (cdr p))))
           ((@ object)
-           (if (and (pair? (cdr p)) (symid? (cadr p)) (list? (cddr p))
+           (if (and (pair? (cdr p)) (identifier? (cadr p)) (list? (cddr p))
                     (every (lambda (p) (and (list? p) (= (length p) 2)))
                            (cddr p)))
              `(object ,(cadr p) ,@(map (lambda (p)
@@ -414,7 +412,7 @@
     (let ((cons-quasi (lambda (x y) (cons (quasi x) (quasi y)))))
       (cond
        ((simple? p) p)
-       ((symid? p) `',p)
+       ((identifier? p) `',p)
        ((pair? p)
         (cond
          ((eq? (car p) 'unquote)
@@ -457,7 +455,7 @@
   (define (bound p a k)
     (cond
      ((eq? '_ p) (k p a))
-     ((symid? p)
+     ((identifier? p)
       (when (memq p a)
         (match:syntax-err pattern "duplicate variable in pattern"))
       (k p (cons p a)))
@@ -467,7 +465,7 @@
       (cond
        ((not (null? (cddr p)))
         (bound `(and (? ,(cadr p)) ,@(cddr p)) a k))
-       ((or (not (symid? (cadr p)))
+       ((or (not (identifier? (cadr p)))
             (memq (cadr p) a))
         (let ((g (gensym)))
           (push! pred-bodies `(,g ,(cadr p)))
@@ -475,7 +473,7 @@
        (else (k p a))))
      ((and (pair? p) (eq? '= (car p)))
       (cond
-       ((or (not (symid? (cadr p)))
+       ((or (not (identifier? (cadr p)))
             (memq (cadr p) a))
         (let ((g (gensym)))
           (push! pred-bodies `(,g ,(cadr p)))
@@ -623,11 +621,11 @@
                         (if fail-sym
                           (let ((ap `(,code ,fail-sym ,@(map val bv))))
                             `(call-with-current-continuation
-                              (lambda (,fail-sym)
+                              (,lambda. (,fail-sym)
                                 (let ((,fail-sym
-                                       (lambda ()
+                                       (,lambda. ()
                                          (call-with-values
-                                             (lambda () ,(fail sf))
+                                             (,lambda. () ,(fail sf))
                                            ,fail-sym))))
                                   ,ap))))
                           `(,code ,@(map val bv)))))))
@@ -647,7 +645,7 @@
                  See the ``Keyword and symbol integration'' section \
                  of the manual for the details.\n" p x)
           (emit `(equal? ,e ,p) sf kf ks))
-         ((symid? p) (set! v (cons (cons p e) v))
+         ((identifier? p) (set! v (cons (cons p e) v))
           (ks sf))
          ((null? p) (emit `(null? ,e) sf kf ks))
          ((equal? p ''()) (emit `(null? ,e) sf kf ks))
@@ -729,16 +727,16 @@
                                                       (lambda (sf) #f)
                                                       (lambda (sf) #t)))
                                           (tst (if (and (pair? ptst)
-                                                        (symid? (car ptst))
+                                                        (identifier? (car ptst))
                                                         (pair? (cdr ptst))
                                                         (eq? eta (cadr ptst))
                                                         (null? (cddr ptst)))
                                                  (car ptst)
-                                                 `(lambda (,eta) ,ptst))))
+                                                 `(,lambda. (,eta) ,ptst))))
                                      (assm `(match:every ,tst ,e)
                                            (kf sf)
                                            (ks sf))))
-                                  ((and (symid? (car p))
+                                  ((and (identifier? (car p))
                                         (equal? (list (car p)) bound))
                                    (next (car p) e sf kf ks))
                                   (else
@@ -921,7 +919,7 @@
          (equal? (car s) 'call-with-current-continuation)
          (pair? (cdr s))
          (pair? (cadr s))
-         (equal? (caadr s) 'lambda)
+         (equal? (caadr s) lambda.)
          (pair? (cdadr s))
          (pair? (cadadr s))
          (null? (cdr (cadadr s)))
@@ -933,7 +931,7 @@
          (pair? (caadar (cddadr s)))
          (pair? (cdr (caadar (cddadr s))))
          (pair? (cadr (caadar (cddadr s))))
-         (equal? (caadr (caadar (cddadr s))) 'lambda)
+         (equal? (caadr (caadar (cddadr s))) lambda.)
          (pair? (cdadr (caadar (cddadr s))))
          (null? (cadadr (caadar (cddadr s))))
          (pair? (cddadr (caadar (cddadr s))))
@@ -941,7 +939,7 @@
          (equal? (caar (cddadr (caadar (cddadr s)))) 'call-with-values)
          (pair? (cdar (cddadr (caadar (cddadr s)))))
          (pair? (cadar (cddadr (caadar (cddadr s)))))
-         (equal? (caadar (cddadr (caadar (cddadr s)))) 'lambda)
+         (equal? (caadar (cddadr (caadar (cddadr s)))) lambda.)
          (pair? (cdadar (cddadr (caadar (cddadr s)))))
          (null? (car (cdadar (cddadr (caadar (cddadr s))))))
          (pair? (cdr (cdadar (cddadr (caadar (cddadr s))))))
@@ -961,7 +959,7 @@
           (s2 (caddar (cddadr s))))
       `(call-with-current-continuation
         (lambda (,k)
-          (let ((,fail (lambda () (call-with-values (lambda () ,f) ,k))))
+          (let ((,fail (,lambda. () (call-with-values (,lambda. () ,f) ,k))))
             ,(assm tst `(,fail) s2))))))
    ((and #f
          (pair? s)
@@ -983,7 +981,7 @@
          (equal? (caddar (cdaadr s)) f))
     (let ((fail (caaadr s))
           (s2 (caddr s)))
-      `(let ((,fail (lambda () ,f)))
+      `(let ((,fail (,lambda. () ,f)))
          ,(assm tst `(,fail) s2))))
    (else `(if ,tst ,s ,f))))
 
@@ -1000,7 +998,7 @@
                  (loop (cadddr code)))))
        ((equal? (car code) 'lambda) #f)
        ((and (eq? (car code) 'let)
-             (symid? (cadr code)))
+             (identifier? (cadr code)))
         #f)
        (else (or (loop (car code))
                  (loop (cdr code))))))))
@@ -1061,7 +1059,7 @@
                 (pair? (cdr p))
                 (null? (cddr p))
                 (equal? 'quote (car p))
-                (symid? (cadr p)))
+                (identifier? (cadr p)))
            'symbol?)
           (else #f)))))
 
@@ -1113,8 +1111,8 @@
             (lambda (y) (and (list? y) (<= 2 (length y))))
             (cdr args))) (let* ((exp (car args))
                                 (clauses (cdr args))
-                                (e (if (symid? exp) exp (gensym))))
-                           (if (symid? exp)
+                                (e (if (identifier? exp) exp (gensym))))
+                           (if (identifier? exp)
                                ((car match:expanders)
                                 e
                                 clauses
@@ -1135,7 +1133,7 @@
                    #f))
              args))
       ((lambda ()
-         (let ((e (gensym))) `(lambda (,e) (,match. ,e ,@args)))))
+         (let ((e (gensym))) `(,lambda. (,e) (,match. ,e ,@args)))))
       ((lambda ()
          (match:syntax-err
            `(match-lambda ,@args)
@@ -1150,7 +1148,7 @@
                    #f))
              args))
       ((lambda ()
-         (let ((e (gensym))) `(lambda ,e (,match. ,e ,@args)))))
+         (let ((e (gensym))) `(,lambda. ,e (,match. ,e ,@args)))))
       ((lambda ()
          (match:syntax-err
            `(match-lambda* ,@args)
@@ -1172,7 +1170,7 @@
                      (,match. (cons ,g1 ,g2) ((,p1 . ,p2) ,@body))))))
         (g136 (cadddr match:expanders)))
     (if (pair? args)
-        (if (symid? (car args))
+        (if (identifier? (car args))
             (if (and (pair? (cdr args)) (list? (cadr args)))
                 (let g161 ((g162 (cadr args)) (g160 '()) (g159 '()))
                   (if (null? g162)
