@@ -1,7 +1,7 @@
 /*
  * bignum.c - multiple precision exact integer arithmetic
  *
- *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -300,32 +300,8 @@ u_long Scm_BignumToUI(const ScmBignum *b, int clamp, int *oor)
 
 #if SIZEOF_LONG == 4
 /* we need special routines for int64 */
-ScmInt64 Scm_BignumToSI64(const ScmBignum *b, int clamp, int *oor)
+int64_t Scm_BignumToSI64(const ScmBignum *b, int clamp, int *oor)
 {
-#if SCM_EMULATE_INT64
-    ScmInt64 r = {0, 0};
-    if (clamp == SCM_CLAMP_NONE && oor != NULL) *oor = FALSE;
-    if (b->sign > 0) {
-        if (b->size > 2 || b->values[1] > LONG_MAX) {
-            if (!(clamp & SCM_CLAMP_HI)) goto err;
-            SCM_SET_INT64_MAX(r);
-        } else {
-            r.lo = b->values[0];
-            if (b->size == 2) r.hi = b->values[1];
-        }
-    } else if (b->sign < 0) {
-        if (b->size > 2 || b->values[1] > (u_long)LONG_MAX + 1) {
-            if (!(clamp&SCM_CLAMP_LO)) goto err;
-            SCM_SET_INT64_MIN(r);
-        } else {
-            b = SCM_BIGNUM(Scm_BignumComplement(b));
-            r.lo = b->values[0];
-            if (b->size == 2) r.hi = b->values[1];
-            else              r.hi = -1;
-        }
-    }
-    return r;
-#else  /*!SCM_EMULATE_INT64*/
     int64_t r = 0;
     if (clamp == SCM_CLAMP_NONE && oor != NULL) *oor = FALSE;
     if (b->sign > 0) {
@@ -348,7 +324,6 @@ ScmInt64 Scm_BignumToSI64(const ScmBignum *b, int clamp, int *oor)
         }
     }
     return r;
-#endif /*!SCM_EMULATE_INT64*/
   err:
     if (clamp == SCM_CLAMP_NONE && oor != NULL) {
         *oor = TRUE;
@@ -358,24 +333,8 @@ ScmInt64 Scm_BignumToSI64(const ScmBignum *b, int clamp, int *oor)
     return r;
 }
 
-ScmUInt64 Scm_BignumToUI64(const ScmBignum *b, int clamp, int *oor)
+uint64_t Scm_BignumToUI64(const ScmBignum *b, int clamp, int *oor)
 {
-#if SCM_EMULATE_INT64
-    ScmInt64 r = {0, 0};
-    if (clamp == SCM_CLAMP_NONE && oor != NULL) *oor = FALSE;
-    if (b->sign > 0) {
-        if (b->size > 2) {
-            if (!(clamp&SCM_CLAMP_HI)) goto err;
-            SCM_SET_UINT64_MAX(r);
-        } else {
-            r.lo = b->values[0];
-            if (b->size == 2) r.hi = b->values[1];
-        }
-    } else if (b->sign < 0) {
-        if (!(clamp&SCM_CLAMP_LO)) goto err;
-    }
-    return r;
-#else  /*!SCM_EMULATE_INT64*/
     uint64_t r = 0;
     if (clamp == SCM_CLAMP_NONE && oor != NULL) *oor = FALSE;
     if (b->sign > 0) {
@@ -391,7 +350,6 @@ ScmUInt64 Scm_BignumToUI64(const ScmBignum *b, int clamp, int *oor)
         if (!(clamp&SCM_CLAMP_LO)) goto err;
     }
     return r;
-#endif /*!SCM_EMULATE_INT64*/
   err:
     if (clamp == SCM_CLAMP_NONE && oor != NULL) {
         *oor = TRUE;
@@ -402,7 +360,6 @@ ScmUInt64 Scm_BignumToUI64(const ScmBignum *b, int clamp, int *oor)
 }
 #endif /* SIZEOF_LONG == 4 */
 
-
 extern double Scm__EncodeDouble(u_long, u_long, int, int);
 
 /* Converts a bignum b to a double.  b must be normalized.
@@ -440,7 +397,7 @@ double Scm_BignumToDouble(const ScmBignum *b)
         }
     }
     if (exponent > 2046) return Scm__EncodeDouble(0, 0, 2047, (b->sign < 0));
-    else return Scm__EncodeDouble(0, dst[0], exponent, (b->sign < 0));
+    else return Scm__EncodeDouble(dst[0], 0, exponent, (b->sign < 0));
 #else  /*SIZEOF_LONG == 4 */
     dst[0] = dst[1] = 0;
     if (maxbit < 53) {
@@ -457,7 +414,7 @@ double Scm_BignumToDouble(const ScmBignum *b)
             if (dst[1] >= (1UL<<(52-32))) {
                 /* Overflow.  We mask the hidden bit, then shift. */
                 dst[1] &= ~(1UL<<(52-32));
-                dst[0] = (dst[0] >> 1) | (dst[1]&1 << 31);
+                dst[0] = (dst[0] >> 1) | ((u_long)(dst[1]&1) << 31);
                 dst[1] >>= 1;
                 exponent++;
             }
@@ -1104,7 +1061,7 @@ ScmObj Scm_BignumDivRem(const ScmBignum *dividend, const ScmBignum *divisor)
 }
 
 /*-----------------------------------------------------------------------
- * Logical (bitwise) opertaions
+ * Logical (bitwise) operations
  */
 
 ScmObj Scm_BignumAsh(const ScmBignum *x, int cnt)
@@ -1283,7 +1240,7 @@ ScmObj Scm_BignumToString(const ScmBignum *b, int radix, int use_upper)
     static const char utab[] = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
     const char *tab = use_upper? utab : ltab;
     ScmObj h = SCM_NIL, t = SCM_NIL;
-    if (radix < 2 || radix > 36)
+    if (radix < SCM_RADIX_MIN || radix > SCM_RADIX_MAX)
         Scm_Error("radix out of range: %d", radix);
     ScmBignum *q = SCM_BIGNUM(Scm_BignumCopy(b));
     for (;q->size > 0;) {
