@@ -1,7 +1,7 @@
 /*
  * bits.c - Bit manipulation utilities
  *
- *   Copyright (c) 2007-2018  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2007-2020  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -41,7 +41,10 @@
 
 ScmBits *Scm_MakeBits(int numbits)
 {
-    return SCM_NEW_ATOMIC_ARRAY(ScmBits, SCM_BITS_NUM_WORDS(numbits));
+    size_t nw = SCM_BITS_NUM_WORDS(numbits);
+    ScmBits *bits = SCM_NEW_ATOMIC_ARRAY(ScmBits, nw);
+    memset(bits, 0, nw * SIZEOF_LONG);
+    return bits;
 }
 
 void Scm_BitsCopyX(ScmBits *target, int tstart,
