@@ -1,7 +1,7 @@
 /*
  * char.c - character and character set operations
  *
- *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -38,6 +38,9 @@
 #include "gauche/priv/charP.h"
 #include "gauche/priv/vectorP.h"
 
+static ScmObj predef_sets[SCM_CHAR_SET_NUM_PREDEFINED_SETS];
+static ScmObj predef_sets_complement[SCM_CHAR_SET_NUM_PREDEFINED_SETS];
+
 #include "char_attr.c"          /* generated tables */
 
 /*=======================================================================
@@ -140,8 +143,10 @@ void Scm__InstallCharconvHooks(ScmChar (*u2c)(int), int (*c2u)(ScmChar))
 
 /* TRANSIENT: These two variables are no longer used, but kept here for the
    ABI compatibility.  Remove them on 1.0 release. */
+#if GAUCHE_API_VERSION < 1000
 ScmChar (*Scm_UcsToCharHook)(int ucs4) = NULL;
 int (*Scm_CharToUcsHook)(ScmChar ch) = NULL;
+#endif /*GAUCHE_API_VERSION < 1000*/
 
 ScmChar Scm_UcsToChar(int n)
 {
@@ -421,7 +426,8 @@ static void charset_print_ch(ScmPort *out, ScmChar ch, int firstp)
     }
 }
 
-static void charset_print(ScmObj obj, ScmPort *out, ScmWriteContext *ctx)
+static void charset_print(ScmObj obj, ScmPort *out, 
+                          ScmWriteContext *ctx SCM_UNUSED)
 {
     int prev, code, first = TRUE;
     ScmCharSet *cs = SCM_CHAR_SET(obj);
@@ -450,7 +456,7 @@ static void charset_print(ScmObj obj, ScmPort *out, ScmWriteContext *ctx)
 
     if (cs->flags & SCM_CHAR_SET_IMMUTABLE) {
         const uint32_t *v = cs->large.frozen.vec;
-        for (int i = 0; i < cs->large.frozen.size; i += 2) {
+        for (ScmSize i = 0; i < cs->large.frozen.size; i += 2) {
             charset_print_ch(out, (int)v[i], FALSE);
             if (v[i] != v[i+1]) {
                 if (v[i+1] - v[i] > 2) Scm_Printf(out, "-");
@@ -480,7 +486,7 @@ typedef struct cs_iter_rec {
     int end;
     union {
         ScmTreeIter ti;
-        size_t vi;
+        ScmSize vi;
     } iter;
 } cs_iter;
 
@@ -531,7 +537,7 @@ static int cs_iter_next(cs_iter *ci,
 /*-----------------------------------------------------------------
  * Constructors
  */
-static int cmp(ScmTreeCore *tc, intptr_t a, intptr_t b)
+static int cmp(ScmTreeCore *tc SCM_UNUSED, intptr_t a, intptr_t b)
 {
     if (a > b) return 1;
     if (a < b) return -1;
@@ -558,11 +564,12 @@ ScmObj Scm_MakeImmutableCharSet(const ScmBits *small,
                                 const uint32_t *vec,
                                 size_t size)
 {
-    SCM_ASSERT(size > 0 && size % 2 == 0);
+    SCM_ASSERT(size % 2 == 0);
     ScmCharSet *cs = SCM_NEW(ScmCharSet);
+    SCM_SET_CLASS(cs, SCM_CLASS_CHARSET);
     cs->flags |= SCM_CHAR_SET_IMMUTABLE;
     memcpy(cs->small, small, sizeof(cs->small));
-    if (vec != NULL) {
+    if (vec != NULL && size > 0) {
         set_large(cs, TRUE);
         if ((cs->large.frozen.size = size) == 2) {
             cs->large.frozen.ivec[0] = vec[0];
@@ -583,7 +590,7 @@ ScmObj Scm_CharSetCopy(ScmCharSet *src)
     if (SCM_CHAR_SET_IMMUTABLE_P(src)) {
         /* The destination is mutable */
         const uint32_t *vec = src->large.frozen.vec;
-        for (size_t k = 0; k < src->large.frozen.size; k += 2) {
+        for (ScmSize k = 0; k < src->large.frozen.size; k += 2) {
             ScmDictEntry *e = Scm_TreeCoreSearch(&dst->large.tree,
                                                  vec[k], SCM_DICT_CREATE);
             e->value = vec[k+1];
@@ -599,7 +606,7 @@ ScmObj Scm_CharSetCopy(ScmCharSet *src)
    The caller must provide uint32_t[2] buffer for ivec. */
 static uint32_t *char_set_freeze_vec(ScmCharSet *src,
                                      uint32_t *ivec,
-                                     size_t *size /*out*/)
+                                     ScmSize *size /*out*/)
 {
     SCM_ASSERT(!SCM_CHAR_SET_IMMUTABLE_P(src));
     size_t s = (size_t)Scm_TreeCoreNumEntries(&src->large.tree) * 2;
@@ -640,13 +647,17 @@ ScmObj Scm_CharSetFreezeX(ScmCharSet *src)
 {
     if (SCM_CHAR_SET_IMMUTABLE_P(src)) return SCM_OBJ(src);
     if (SCM_CHAR_SET_LARGE_P(src)) {
-        size_t s;
+        ScmSize s;
         uint32_t iv[2];
         uint32_t *v = char_set_freeze_vec(src, iv, &s);
         src->large.frozen.size = s;
-        src->large.frozen.vec = (s == 2)? src->large.frozen.ivec : v;
-        src->large.frozen.ivec[0] = iv[0];
-        src->large.frozen.ivec[1] = iv[1];
+        if (s == 2) {
+            src->large.frozen.vec = src->large.frozen.ivec;
+            src->large.frozen.ivec[0] = iv[0];
+            src->large.frozen.ivec[1] = iv[1];
+        } else {
+            src->large.frozen.vec = v;
+        }
     }
     src->flags |= SCM_CHAR_SET_IMMUTABLE;
     return SCM_OBJ(src);
@@ -711,9 +722,9 @@ static int cs_contains_range(ScmCharSet *s, ScmChar lo, ScmChar hi)
         ye = Scm_BinarySearchU32(s->large.frozen.vec, s->large.frozen.size,
                                  (uint32_t)lo, 1, &yl, NULL);
         if (ye != (size_t)-1) { /* case 1 */
-            if (s->large.frozen.vec[ye+1] < hi) return FALSE;
+            if (s->large.frozen.vec[ye+1] < (unsigned)hi) return FALSE;
         } else if (yl != (size_t)-1) { /* case 2 */
-            if (s->large.frozen.vec[yl+1] < hi) return FALSE;
+            if (s->large.frozen.vec[yl+1] < (unsigned)hi) return FALSE;
         } else {
             return FALSE;
         }
@@ -804,15 +815,26 @@ ScmObj Scm_CharSetAdd(ScmCharSet *dst, ScmCharSet *src)
 
     if (dst == src) return SCM_OBJ(dst);  /* precaution */
 
-    set_large(dst, SCM_CHAR_SET_LARGE_P(src));
+    if (SCM_CHAR_SET_LARGE_P(src)) {
+        set_large(dst, TRUE);
+    }
     
     ScmTreeIter iter;
     ScmDictEntry *e;
     Scm_BitsOperate(dst->small, SCM_BIT_IOR, dst->small, src->small,
                     0, SCM_CHAR_SET_SMALL_CHARS);
-    Scm_TreeIterInit(&iter, &src->large.tree, NULL);
-    while ((e = Scm_TreeIterNext(&iter)) != NULL) {
-        Scm_CharSetAddRange(dst, SCM_CHAR(e->key), SCM_CHAR(e->value));
+    if (SCM_CHAR_SET_IMMUTABLE_P(src)) {
+        ScmSize k;
+        for (k = 0; k < src->large.frozen.size; k += 2) {
+            Scm_CharSetAddRange(dst,
+                                SCM_CHAR(src->large.frozen.vec[k]),
+                                SCM_CHAR(src->large.frozen.vec[k+1]));
+        }
+    } else {
+        Scm_TreeIterInit(&iter, &src->large.tree, NULL);
+        while ((e = Scm_TreeIterNext(&iter)) != NULL) {
+            Scm_CharSetAddRange(dst, SCM_CHAR(e->key), SCM_CHAR(e->value));
+        }
     }
     return SCM_OBJ(dst);
 }
@@ -823,24 +845,26 @@ ScmObj Scm_CharSetComplement(ScmCharSet *cs)
     
     ScmDictEntry *e, *n;
 
-    set_large(cs, !SCM_CHAR_SET_LARGE_P(cs));
-    
     Scm_BitsOperate(cs->small, SCM_BIT_NOT1, cs->small, NULL,
                     0, SCM_CHAR_SET_SMALL_CHARS);
     int last = SCM_CHAR_SET_SMALL_CHARS-1;
+    int largep = FALSE;
     /* we can't use treeiter, since we modify the tree while traversing it. */
     while ((e = Scm_TreeCoreNextEntry(&cs->large.tree, last)) != NULL) {
         Scm_TreeCoreSearch(&cs->large.tree, e->key, SCM_DICT_DELETE);
         if (last < e->key-1) {
             n = Scm_TreeCoreSearch(&cs->large.tree, last+1, SCM_DICT_CREATE);
             n->value = e->key-1;
+            largep = TRUE;
         }
         last = (int)e->value;
     }
     if (last < SCM_CHAR_MAX) {
         n = Scm_TreeCoreSearch(&cs->large.tree, last+1, SCM_DICT_CREATE);
         n->value = SCM_CHAR_MAX;
+        largep = TRUE;
     }
+    set_large(cs, largep);
     return SCM_OBJ(cs);
 }
 
@@ -890,7 +914,7 @@ int Scm_CharSetContains(ScmCharSet *cs, ScmChar c)
                                            (uint32_t)c,
                                            1, &lo, NULL);
             if ((k != (size_t)-1)
-                || (lo != (size_t)-1 && c <= cs->large.frozen.vec[lo+1]))
+                || (lo != (size_t)-1 && (unsigned)c <= cs->large.frozen.vec[lo+1]))
                 return TRUE;
             else
                 return FALSE;
@@ -979,123 +1003,157 @@ void Scm_CharSetDump(ScmCharSet *cs, ScmPort *port)
    whether complement character (caret in the beginning) appeared or not.
    In that case, the returned charset is not complemented. */
 
-static ScmObj read_predef_charset(const char**, int);
-static int read_charset_syntax(ScmPort *input, int bracket_syntax,
-                               ScmDString *buf, int *complementp);
+static ScmObj read_predef_charset(ScmPort*, int);
 
 ScmObj Scm_CharSetRead(ScmPort *input, int *complement_p,
                        int error_p, int bracket_syntax)
 {
-    int complement = FALSE;
-    ScmDString buf;
-
+#define REAL_BEGIN 1
+#define CARET_BEGIN 2
+    ScmCharSet *set = make_charset();
+    int begin = REAL_BEGIN;
+    int complement = FALSE;     /* Flag for the initial ^ */
+    int inrange = FALSE;        /* The range '-' is being read */
+    ScmChar lastchar = SCM_CHAR_INVALID; /* The char before '-' range */
+    const char *prefetched = NULL; /* \x notation requires lookahead.  After
+                                      it reads extra characters, this points
+                                      to them.  The pointed string is guaranteed
+                                      to have only hexadecimal characters. */
+    ScmDString buf;             /* Save read characters for error message */
     Scm_DStringInit(&buf);
-    if (read_charset_syntax(input, bracket_syntax, &buf, &complement)) {
-        int lastchar = -1, inrange = FALSE, moreset_complement = FALSE;
-        ScmCharSet *set = SCM_CHAR_SET(Scm_MakeEmptyCharSet());
-        int size;
-        const char *cp = Scm_DStringPeek(&buf, &size, NULL);
-        const char *end = cp + size;
+    int ch;
 
-        while (cp < end) {
-            ScmChar ch;
-            SCM_CHAR_GET(cp, ch);
-            if (ch == SCM_CHAR_INVALID) goto err;
-            cp += SCM_CHAR_NBYTES(ch);
+    for (;;) {
+        if (prefetched) {
+            ch = *prefetched++;
+            if (*prefetched == '\0') prefetched = NULL;
+        } else {
+            ch = Scm_Getc(input);
+            if (ch == EOF) goto err;
+        }
 
-            ScmObj moreset;
+        Scm_DStringPutc(&buf, ch);
+
+        if (begin == REAL_BEGIN && ch == '^') {
+            complement = TRUE;
+            begin = CARET_BEGIN;
+            continue;
+        }
+
+        ScmObj moreset;
+        switch (ch) {
+        case '^':
+            if (begin == REAL_BEGIN) {
+                complement = TRUE;
+                begin = CARET_BEGIN;
+                continue;
+            } else {
+                goto ordchar;
+            }
+        case ']':
+            if (begin && bracket_syntax) goto ordchar;
+            else break;
+        case '-':
+            if (begin || inrange) goto ordchar;
+            inrange = TRUE;
+            begin = FALSE;
+            continue;
+        case '\\':
+            ch = Scm_Getc(input);
+            if (ch == EOF) goto err;
+            Scm_DStringPutc(&buf, ch);
             switch (ch) {
-            case '-':
-                if (inrange) goto ordchar;
-                inrange = TRUE;
-                continue;
-            case '\\':
-                if (cp >= end) goto err;
+            case 'a': ch = 7; goto ordchar;
+            case 'b': ch = 8; goto ordchar;
+            case 'n': ch = '\n'; goto ordchar;
+            case 'r': ch = '\r'; goto ordchar;
+            case 't': ch = '\t'; goto ordchar;
+            case 'f': ch = '\f'; goto ordchar;
+            case 'e': ch = 0x1b; goto ordchar;
+            case 'x': case 'u': case 'U': {
+                ScmDString xbuf;
+                Scm_DStringInit(&xbuf);
+                ScmObj mode = Scm_GetPortReaderLexicalMode(input);
+                ScmObj z = Scm_ReadXdigitsFromPort(input, ch, mode, FALSE, &xbuf);
+                if (SCM_STRINGP(z)) {
+                    /* parse failure.  z contains the prefetched string */
+                    Scm_DStringAdd(&buf, SCM_STRING(z));
+                    goto err;
+                }
+                /* xbuf contains the character that was hex-encoded,
+                   plus any hex digits that are prefetched. */
+                Scm_DStringPutc(&xbuf, '\0');
+                const char *cp = Scm_DStringPeek(&xbuf, NULL, NULL);
                 SCM_CHAR_GET(cp, ch);
-                if (ch == SCM_CHAR_INVALID) goto err;
-                cp += SCM_CHAR_NBYTES(ch);
-                switch (ch) {
-                case 'a': ch = 7; goto ordchar;
-                case 'b': ch = 8; goto ordchar;
-                case 'n': ch = '\n'; goto ordchar;
-                case 'r': ch = '\r'; goto ordchar;
-                case 't': ch = '\t'; goto ordchar;
-                case 'f': ch = '\f'; goto ordchar;
-                case 'e': ch = 0x1b; goto ordchar;
-                case 'x': case 'u': case 'U':
-                    ch = Scm_ReadXdigitsFromString(cp, end-cp, ch,
-                                                   Scm_GetPortReaderLexicalMode(input),
-                                                   TRUE, &cp);
-                    if (ch == SCM_CHAR_INVALID) goto err;
-                    goto ordchar;
-                case 'd':
-                    moreset_complement = FALSE;
-                    moreset = Scm_GetStandardCharSet(SCM_CHAR_SET_DIGIT);
-                    break;
-                case 'D':
-                    moreset_complement = TRUE;
-                    moreset = Scm_GetStandardCharSet(SCM_CHAR_SET_DIGIT);
-                    break;
-                case 's':
-                    moreset_complement = FALSE;
-                    moreset = Scm_GetStandardCharSet(SCM_CHAR_SET_SPACE);
-                    break;
-                case 'S':
-                    moreset_complement = TRUE;
-                    moreset = Scm_GetStandardCharSet(SCM_CHAR_SET_SPACE);
-                    break;
-                case 'w':
-                    moreset_complement = FALSE;
-                    moreset = Scm_GetStandardCharSet(SCM_CHAR_SET_WORD);
-                    break;
-                case 'W':
-                    moreset_complement = TRUE;
-                    moreset = Scm_GetStandardCharSet(SCM_CHAR_SET_WORD);
-                    break;
-                default:
-                    goto ordchar;
+                cp += SCM_CHAR_NFOLLOWS(*cp)+1;
+                if (*cp != '\0') {
+                    prefetched = cp;
                 }
-                if (moreset_complement) {
-                    moreset = Scm_CharSetComplement(SCM_CHAR_SET(Scm_CharSetCopy(SCM_CHAR_SET(moreset))));
-                }
-                Scm_CharSetAdd(set, SCM_CHAR_SET(moreset));
-                continue;
-            case '[':
-                moreset = read_predef_charset(&cp, error_p);
-                if (!SCM_CHAR_SET_P(moreset)) goto err;
-                Scm_CharSetAdd(set, SCM_CHAR_SET(moreset));
-                continue;
-            ordchar:
+                goto ordchar;
+            }
+            case 'd':
+                moreset = Scm_GetStandardCharSet(SCM_CHAR_SET_ASCII_DIGIT);
+                goto addset;
+            case 'D':
+                moreset = Scm_GetStandardCharSet(-SCM_CHAR_SET_ASCII_DIGIT);
+                goto addset;
+            case 's':
+                moreset = Scm_GetStandardCharSet(SCM_CHAR_SET_ASCII_WHITESPACE);
+                goto addset;
+            case 'S':
+                moreset = Scm_GetStandardCharSet(-SCM_CHAR_SET_ASCII_WHITESPACE);
+                goto addset;
+            case 'w':
+                moreset = Scm_GetStandardCharSet(SCM_CHAR_SET_ASCII_WORD);
+                goto addset;
+            case 'W':
+                moreset = Scm_GetStandardCharSet(-SCM_CHAR_SET_ASCII_WORD);
+                goto addset;
+            case 'p': case 'P':
+                moreset = Scm_GetStandardCharSet(Scm_CharSetParseCategory(input, ch));
+                goto addset;
             default:
-                if (inrange) {
-                    if (lastchar < 0) {
-                        Scm_CharSetAddRange(set, '-', '-');
-                        Scm_CharSetAddRange(set, ch, ch);
-                        lastchar = ch;
-                    } else {
-                        Scm_CharSetAddRange(set, lastchar, ch);
-                        lastchar = -1;
-                    }
-                    inrange = FALSE;
-                } else {
+                goto ordchar;
+            }
+        case '[':
+            moreset = read_predef_charset(input, error_p);
+            if (!SCM_CHAR_SET_P(moreset)) goto err;
+        addset:
+            Scm_CharSetAdd(set, SCM_CHAR_SET(moreset));
+            begin = FALSE;
+            continue;
+        ordchar:
+        default:
+            if (inrange) {
+                if (lastchar < 0) {
+                    Scm_CharSetAddRange(set, '-', '-');
                     Scm_CharSetAddRange(set, ch, ch);
                     lastchar = ch;
+                } else {
+                    Scm_CharSetAddRange(set, lastchar, ch);
+                    lastchar = -1;
                 }
-                continue;
+                inrange = FALSE;
+            } else {
+                Scm_CharSetAddRange(set, ch, ch);
+                lastchar = ch;
             }
-            break;
-        }
-        if (inrange) {
-            Scm_CharSetAddRange(set, '-', '-');
-            if (lastchar >= 0) Scm_CharSetAddRange(set, lastchar, lastchar);
-        }
-        if (complement_p) {
-            *complement_p = complement;
-            return SCM_OBJ(set);
-        } else {
-            if (complement) Scm_CharSetComplement(set);
-            return SCM_OBJ(set);
+            begin = FALSE;
+            continue;
         }
+        break;
+    }
+
+    if (inrange) {
+        Scm_CharSetAddRange(set, '-', '-');
+        if (lastchar >= 0) Scm_CharSetAddRange(set, lastchar, lastchar);
+    }
+    if (complement_p) {
+        *complement_p = complement;
+        return SCM_OBJ(set);
+    } else {
+        if (complement) Scm_CharSetComplement(set);
+        return SCM_OBJ(set);
     }
   err:
     if (error_p) {
@@ -1107,106 +1165,193 @@ ScmObj Scm_CharSetRead(ScmPort *input, int *complement_p,
     return SCM_FALSE;
 }
 
-/* Read till the end of char-set syntax and store the chars in BUF.
-   The surrounding brackets are not stored.  The complementing caret
-   ('^' right after opening '[') isn't stored either, but *complementp
-   holds whether it appeared or not.
-   Returns TRUE on success.
- */
-int read_charset_syntax(ScmPort *input, int bracket_syntax, ScmDString *buf,
-                        int *complementp)
-{
-#define REAL_BEGIN 1
-#define CARET_BEGIN 2
-    int begin = REAL_BEGIN, complement = FALSE, brackets = 0;
+/* Predefined charset name table */
+struct predef_charset_posix_name_rec {
+    const char *name;
+    int cset;           /* default cset */
+    int cset_unicode;   /* 'unicode' mode cset (:alnum: for unicode range)
+                           not used yet. */
+};
 
-    for (;;) {
-        int ch;
-        SCM_GETC(ch, input);
-        if (ch == EOF) return FALSE;
+#define PREDEF_ENTRY(n, cs, csu) \
+    { n, SCM_CPP_CAT(SCM_CHAR_SET_, cs), SCM_CPP_CAT(SCM_CHAR_SET_, csu) }
 
-        if (begin == REAL_BEGIN && ch == '^') {
-            complement = TRUE;
-            begin = CARET_BEGIN;
-            continue;
-        }
-        if (bracket_syntax && begin && ch == ']') {
-            begin = FALSE;
-            Scm_DStringPutc(buf, ch);
-            continue;
-        }
-        if (ch == ']' && brackets <= 0) break;
-        begin = FALSE;
+static struct predef_charset_posix_name_rec predef_charset_posix_names[] = {
+    PREDEF_ENTRY("alpha:", ASCII_LETTER, LETTER),
+    PREDEF_ENTRY("alnum:", ASCII_LETTER_DIGIT, LETTER_DIGIT),
+    PREDEF_ENTRY("blank:", ASCII_BLANK, BLANK),
+    PREDEF_ENTRY("cntrl:", ASCII_CONTROL, ISO_CONTROL),
+    PREDEF_ENTRY("digit:", ASCII_DIGIT, DIGIT),
+    PREDEF_ENTRY("graph:", ASCII_GRAPHIC, GRAPHIC),
+    PREDEF_ENTRY("lower:", ASCII_LOWER, LOWER),
+    PREDEF_ENTRY("print:", ASCII_PRINTING, PRINTING),
+    PREDEF_ENTRY("punct:", ASCII_PUNCTUATION, PUNCTUATION),
+    PREDEF_ENTRY("space:", ASCII_WHITESPACE, WHITESPACE),
+    PREDEF_ENTRY("upper:", ASCII_UPPER, UPPER),
+    PREDEF_ENTRY("word:",  ASCII_WORD, WORD),
+    PREDEF_ENTRY("xdigit:", HEX_DIGIT, HEX_DIGIT),
+    PREDEF_ENTRY("ascii:", ASCII, ASCII), /* like Go */
 
-        Scm_DStringPutc(buf, ch);
-
-        switch (ch) {
-        case ']': brackets--; break;
-        case '[': brackets++; break;
-        case '\\':
-            SCM_GETC(ch, input);
-            if (ch == EOF) return FALSE;
-            Scm_DStringPutc(buf, ch);
-            break;
-        }
-    }
-    *complementp = complement;
-    return TRUE;
-}
+    /* Gauche extension - explicitly unicode range */
+    PREDEF_ENTRY("ALPHA:", LETTER, LETTER),
+    PREDEF_ENTRY("ALNUM:", LETTER_DIGIT, LETTER_DIGIT),
+    PREDEF_ENTRY("BLANK:", BLANK, BLANK),
+    PREDEF_ENTRY("CNTRL:", ISO_CONTROL, ISO_CONTROL),
+    PREDEF_ENTRY("DIGIT:", DIGIT, DIGIT),
+    PREDEF_ENTRY("GRAPH:", GRAPHIC, GRAPHIC),
+    PREDEF_ENTRY("LOWER:", LOWER, LOWER),
+    PREDEF_ENTRY("PRINT:", PRINTING, PRINTING),
+    PREDEF_ENTRY("PUNCT:", PUNCTUATION, PUNCTUATION),
+    PREDEF_ENTRY("SPACE:", WHITESPACE, WHITESPACE),
+    PREDEF_ENTRY("UPPER:", UPPER, UPPER),
+    PREDEF_ENTRY("TITLE:", TITLE, TITLE),
+    PREDEF_ENTRY("WORD:",  WORD, WORD),
+    PREDEF_ENTRY("XDIGIT:", HEX_DIGIT, HEX_DIGIT),
 
+    { NULL, 0, 0 }
+};
 
 /* Read posix [:alpha:] etc.  The first '[' is already read.
-   Return #f on error if errorp is FALSE.
-   Set reverse list of read chars in *chars */
-#define MAX_CHARSET_NAME_LEN  10
-ScmObj read_predef_charset(const char **cp, int error_p)
+   Return #f on error if errorp is FALSE. */
+static ScmObj read_predef_charset(ScmPort *input, int error_p)
 {
-    int i;
+#define MAX_CHARSET_NAME_LEN  11
     char name[MAX_CHARSET_NAME_LEN+1];
-    for (i=0; i<MAX_CHARSET_NAME_LEN; i++) {
-        ScmChar ch;
-        SCM_CHAR_GET(*cp, ch);
-        if (ch == SCM_CHAR_INVALID) return SCM_FALSE;
-        *cp += SCM_CHAR_NBYTES(ch);
-        if (!SCM_CHAR_ASCII_P(ch)) break;
-        if (ch != ']') {
-            name[i] = (char)ch;
-            continue;
+    int namecnt = 0;
+    for (; namecnt < MAX_CHARSET_NAME_LEN; namecnt++) {
+        int ch = Scm_Getc(input);
+        if (ch == EOF && !SCM_CHAR_ASCII_P(ch)) {
+            name[namecnt] = '\0';
+            goto err;
         }
-        if (strncmp(name, ":alnum:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHAR_SET_ALNUM);
-        } else if (strncmp(name, ":alpha:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHAR_SET_ALPHA);
-        } else if (strncmp(name, ":blank:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHAR_SET_BLANK);
-        } else if (strncmp(name, ":cntrl:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHAR_SET_CNTRL);
-        } else if (strncmp(name, ":digit:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHAR_SET_DIGIT);
-        } else if (strncmp(name, ":graph:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHAR_SET_GRAPH);
-        } else if (strncmp(name, ":lower:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHAR_SET_LOWER);
-        } else if (strncmp(name, ":print:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHAR_SET_PRINT);
-        } else if (strncmp(name, ":punct:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHAR_SET_PUNCT);
-        } else if (strncmp(name, ":space:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHAR_SET_SPACE);
-        } else if (strncmp(name, ":upper:", 7) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHAR_SET_UPPER);
-        } else if (strncmp(name, ":xdigit:", 8) == 0) {
-            return Scm_GetStandardCharSet(SCM_CHAR_SET_XDIGIT);
-        } else break;
+        if (ch == ']') break;
+        name[namecnt] = (char)ch;
     }
+    if (namecnt == MAX_CHARSET_NAME_LEN) goto err;
+    name[namecnt] = '\0';
+
+    int complement = FALSE;
+    const char *start = name+1;
+
+    if (*start == '^') {
+        complement = TRUE;
+        start++;
+    }
+
+    struct predef_charset_posix_name_rec *e = predef_charset_posix_names;
+    while (e->name != NULL) {
+        if (strcmp(start, e->name) == 0) {
+            if (!complement) {
+                return Scm_GetStandardCharSet(e->cset);
+            } else {
+                return Scm_GetStandardCharSet(-e->cset);
+            }
+        }
+        e++;
+    }
+ err:
     /* here we got invalid charset name */
     if (error_p) {
-        name[i] = '\0';
         Scm_Error("invalid or unsupported POSIX charset '[%s]'", name);
     }
     return SCM_FALSE;
 }
 
+static struct predef_charset_category_name_rec {
+    const char *cat;
+    int cset;
+} predef_charset_category_name[] = {
+    { "L",  SCM_CHAR_SET_L },
+    { "LC", SCM_CHAR_SET_LC },
+    { "Lu", SCM_CHAR_SET_Lu },
+    { "Ll", SCM_CHAR_SET_Ll },
+    { "Lt", SCM_CHAR_SET_Lt },
+    { "Lm", SCM_CHAR_SET_Lm },
+    { "M",  SCM_CHAR_SET_M },
+    { "Mn", SCM_CHAR_SET_Mn },
+    { "Mc", SCM_CHAR_SET_Mc },
+    { "Me", SCM_CHAR_SET_Me },
+    { "N",  SCM_CHAR_SET_N },
+    { "Nd", SCM_CHAR_SET_Nd },
+    { "Nl", SCM_CHAR_SET_Nl },
+    { "No", SCM_CHAR_SET_No },
+    { "P",  SCM_CHAR_SET_P },
+    { "Pc", SCM_CHAR_SET_Pc },
+    { "Pd", SCM_CHAR_SET_Pd },
+    { "Ps", SCM_CHAR_SET_Ps },
+    { "Pe", SCM_CHAR_SET_Pe },
+    { "Pi", SCM_CHAR_SET_Pi },
+    { "Pf", SCM_CHAR_SET_Pf },
+    { "Po", SCM_CHAR_SET_Po },
+    { "S",  SCM_CHAR_SET_S },
+    { "Sm", SCM_CHAR_SET_Sm },
+    { "Sc", SCM_CHAR_SET_Sc },
+    { "Sk", SCM_CHAR_SET_Sk },
+    { "So", SCM_CHAR_SET_So },
+    { "Z",  SCM_CHAR_SET_Z },
+    { "Zs", SCM_CHAR_SET_Zs },
+    { "Zl", SCM_CHAR_SET_Zl },
+    { "Zp", SCM_CHAR_SET_Zp },
+    { "C",  SCM_CHAR_SET_C },
+    { "Cc", SCM_CHAR_SET_Cc },
+    { "Cf", SCM_CHAR_SET_Cf },
+    { "Cs", SCM_CHAR_SET_Cs },
+    { "Co", SCM_CHAR_SET_Co },
+    { "Cn", SCM_CHAR_SET_Cn },
+    { NULL, 0 }
+};
+
+/* Read \p{Category}, \P{Category}.   INPUT must point right after 'p' or
+   'P'.  KEY is either 'p' or 'P'.  On successful reading,  Returns the
+   charset number and update *cp to point right after the syntax. 
+   Otherwise, throws an error.
+*/
+int Scm_CharSetParseCategory(ScmPort *input, char key)
+{
+    int ch = Scm_Getc(input);
+    if (ch != '{') {
+        Scm_Error("\\%c must followed by '{'", key);
+    }
+    char name[3];
+    
+    ch = Scm_Getc(input);
+    if (ch == EOF || !SCM_CHAR_ASCII_P(ch)) {
+        name[0] = '\0';
+        goto bad; 
+    }
+    name[0] = (char)ch;
+
+    ch = Scm_Getc(input);
+    if (ch == EOF || !SCM_CHAR_ASCII_P(ch)) {
+        name[1] = '\0';
+        goto bad; 
+    }
+    if (ch == '}') {
+        name[1] = '\0';
+    } else {
+        name[1] = (char)ch;
+        ch = Scm_Getc(input);
+        name[2] = '\0';
+        if (ch != '}') {
+            goto bad;
+        }
+    }
+
+    for (int j=0; predef_charset_category_name[j].cat; j++) {
+        if (strcmp(name, predef_charset_category_name[j].cat) == 0) {
+            if (key == 'p') {
+                return predef_charset_category_name[j].cset;
+            } else {
+                return -predef_charset_category_name[j].cset;
+            }
+        }
+    }
+ bad:
+    Scm_Error("Bad charset category name near \\%c{%s...", key, name);
+    return 0;                   /* dummy */
+}
+
+
 /*-----------------------------------------------------------------
  * Character attributes
  */
@@ -1400,59 +1545,120 @@ ScmChar Scm_CharFoldcase(ScmChar ch)
 /*-----------------------------------------------------------------
  * Pre-defined charset
  */
-/* TODO: We need different definitions of character classes for different
- * character sets.  For now, I prepare the predefined table only for
- * ASCII range, that all character sets agree on.
- */
 
-static ScmCharSet *predef_charsets[SCM_CHAR_SET_NUM_PREDEFINED_SETS] = {NULL};
-static ScmInternalMutex predef_charsets_mutex;
-
-static void install_charsets(void)
-{
-    SCM_INTERNAL_MUTEX_LOCK(predef_charsets_mutex);
-
-#define CS(n)  predef_charsets[n]
-    for (int i = 0; i < SCM_CHAR_SET_NUM_PREDEFINED_SETS; i++) {
-        CS(i) = SCM_CHAR_SET(Scm_MakeEmptyCharSet());
-    }
-    for (int code = 0; code < SCM_CHAR_SET_SMALL_CHARS; code++) {
-        if (isalnum(code)) MASK_SET(CS(SCM_CHAR_SET_ALNUM), code);
-        if (isalpha(code)) MASK_SET(CS(SCM_CHAR_SET_ALPHA), code);
-        if (iscntrl(code)) MASK_SET(CS(SCM_CHAR_SET_CNTRL), code);
-        if (isdigit(code)) MASK_SET(CS(SCM_CHAR_SET_DIGIT), code);
-        if (isgraph(code)) MASK_SET(CS(SCM_CHAR_SET_GRAPH), code);
-        if (islower(code)) MASK_SET(CS(SCM_CHAR_SET_LOWER), code);
-        if (isprint(code)) MASK_SET(CS(SCM_CHAR_SET_PRINT), code);
-        if (ispunct(code)) MASK_SET(CS(SCM_CHAR_SET_PUNCT), code);
-        if (isspace(code)) MASK_SET(CS(SCM_CHAR_SET_SPACE), code);
-        if (isupper(code)) MASK_SET(CS(SCM_CHAR_SET_UPPER), code);
-        if (isxdigit(code)) MASK_SET(CS(SCM_CHAR_SET_XDIGIT), code);
-        /* Default word constituent chars #[\w].  NB: in future versions,
-           a parameter might be introduced to customize this set. */
-        if (isalnum(code)||code=='_')
-            MASK_SET(CS(SCM_CHAR_SET_WORD), code);
-        /* isblank() is not in posix.  for now, I hardcode it. */
-        if (code == ' ' || code == '\t')
-            MASK_SET(CS(SCM_CHAR_SET_BLANK), code);
-    }
-#undef CS
-    SCM_INTERNAL_MUTEX_UNLOCK(predef_charsets_mutex);
-}
+/* Most predefined charset are pre-generated as static immutable data.
+   See gen-unicode.scm for the generation code. */
 
 ScmObj Scm_GetStandardCharSet(int id)
 {
-    if (id < 0 || id >= SCM_CHAR_SET_NUM_PREDEFINED_SETS)
+    if (id == 0 
+        || id >= SCM_CHAR_SET_NUM_PREDEFINED_SETS
+        || id <= -SCM_CHAR_SET_NUM_PREDEFINED_SETS) {
         Scm_Error("bad id for predefined charset index: %d", id);
-    if (predef_charsets[id] == NULL) {
-        install_charsets();
     }
-    return SCM_OBJ(predef_charsets[id]);
+
+    if (id > 0) {
+        return predef_sets[id];
+    } else {
+        if (!SCM_CHAR_SET_P(predef_sets_complement[-id])) {
+            ScmObj cs = Scm_CharSetCopy(SCM_CHAR_SET(predef_sets[-id]));
+            cs = Scm_CharSetComplement(SCM_CHAR_SET(cs));
+            Scm_CharSetFreezeX(SCM_CHAR_SET(cs));
+            predef_sets_complement[-id] = cs;
+        }
+        return predef_sets_complement[-id];
+    }
 }
 
 void Scm__InitChar(void)
 {
-    SCM_INTERNAL_MUTEX_INIT(predef_charsets_mutex);
+    ScmModule *mod = Scm_GaucheModule();
+
+    init_predefined_charsets();
+    predef_sets[SCM_CHAR_SET_FULL] = Scm_CharSetComplement(make_charset());
+    
+#define DEFCS(name, id) \
+    Scm_Define(mod, SCM_SYMBOL(SCM_INTERN("char-set:" name)), predef_sets[SCM_CPP_CAT(SCM_CHAR_SET_, id)])
+
+    DEFCS("L",  L);
+    DEFCS("LC", LC);
+    DEFCS("Lu", Lu);
+    DEFCS("Ll", Ll);
+    DEFCS("Lt", Lt);
+    DEFCS("Lm", Lm);
+    DEFCS("Lo", Lo);
+    DEFCS("M",  M);
+    DEFCS("Mn", Mn);
+    DEFCS("Mc", Mc);
+    DEFCS("Me", Me);
+    DEFCS("N",  N);
+    DEFCS("Nd", Nd);
+    DEFCS("Nl", Nl);
+    DEFCS("No", No);
+    DEFCS("P",  P);
+    DEFCS("Pc", Pc);
+    DEFCS("Pd", Pd);
+    DEFCS("Ps", Ps);
+    DEFCS("Pe", Pe);
+    DEFCS("Pi", Pi);
+    DEFCS("Pf", Pf);
+    DEFCS("Po", Po);
+    DEFCS("S",  S);
+    DEFCS("Sm", Sm);
+    DEFCS("Sc", Sc);
+    DEFCS("Sk", Sk);
+    DEFCS("So", So);
+    DEFCS("Z",  Z);
+    DEFCS("Zs", Zs);
+    DEFCS("Zl", Zl);
+    DEFCS("Zp", Zp);
+    DEFCS("C",  C);
+    DEFCS("Cc", Cc);
+    DEFCS("Cf", Cf);
+    DEFCS("Cs", Cs);
+    DEFCS("Co", Co);
+    DEFCS("Cn", Cn);
+
+    DEFCS("lower-case", LOWER);
+    DEFCS("ascii-lower-case", ASCII_LOWER);
+    DEFCS("upper-case", UPPER);
+    DEFCS("ascii-upper-case", ASCII_UPPER);
+    DEFCS("title-case", TITLE);
+    DEFCS("letter", LETTER);
+    DEFCS("ascii-letter", ASCII_LETTER);
+    DEFCS("digit", DIGIT);
+    DEFCS("ascii-digit", ASCII_DIGIT);
+    DEFCS("letter+digit", LETTER_DIGIT);
+    DEFCS("ascii-letter+digit", ASCII_LETTER_DIGIT);
+    DEFCS("graphic", GRAPHIC);
+    DEFCS("ascii-graphic", ASCII_GRAPHIC);
+    DEFCS("printing", PRINTING);
+    DEFCS("ascii-printing", ASCII_PRINTING);
+    DEFCS("whitespace", WHITESPACE);
+    DEFCS("ascii-whitespace", ASCII_WHITESPACE);
+    DEFCS("iso-control", ISO_CONTROL);
+    DEFCS("ascii-control", ASCII_CONTROL);
+    DEFCS("punctuation", PUNCTUATION);
+    DEFCS("ascii-punctuation", ASCII_PUNCTUATION);
+    DEFCS("symbol", SYMBOL);
+    DEFCS("ascii-symbol", ASCII_SYMBOL);
+    DEFCS("hex-digit", HEX_DIGIT);
+    DEFCS("blank", BLANK);
+    DEFCS("ascii-blank", ASCII_BLANK);
+    DEFCS("ascii", ASCII);
+    DEFCS("word", WORD);
+    DEFCS("ascii-word", ASCII_WORD);
+    DEFCS("empty", EMPTY);
+    DEFCS("full", FULL);
+
+    /* We initialize complement charset on demand, except EMPTY and FULL. */
+    for (int i=0; i<SCM_CHAR_SET_NUM_PREDEFINED_SETS; i++) {
+        predef_sets_complement[i] = SCM_FALSE;
+    }
+    predef_sets_complement[SCM_CHAR_SET_EMPTY]
+        = predef_sets_complement[SCM_CHAR_SET_FULL];
+    predef_sets_complement[SCM_CHAR_SET_FULL]
+        = predef_sets_complement[SCM_CHAR_SET_EMPTY];
 
     /* Expose internal charset */
 #if defined(GAUCHE_CHAR_ENCODING_EUC_JP)
