@@ -1,7 +1,7 @@
 /*
  * compare.c - comparison & sort
  *
- *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -221,7 +221,7 @@ int Scm_Compare(ScmObj x, ScmObj y)
     else return 1;
 }
 
-/* NB: It turns out that calling back Scheme funtion from sort routine
+/* NB: It turns out that calling back Scheme function from sort routine
    is very inefficient and runs much slower than Scheme version, if
    a Scheme comarison function is given.
    So, as of 0.7.2, the C function is only used when a comparison
@@ -326,12 +326,12 @@ static int cmp_scm(ScmObj x, ScmObj y, ScmObj fn)
         return 1;
 }
 
-static int cmp_int(ScmObj x, ScmObj y, ScmObj dummy)
+static int cmp_int(ScmObj x, ScmObj y, ScmObj dummy SCM_UNUSED)
 {
     return Scm_Compare(x, y);
 }
 
-void Scm_SortArray(ScmObj *elts, int nelts, ScmObj cmpfn)
+void Scm_SortArray(ScmObj *elts, int nelts, ScmObj cmpfn SCM_UNUSED)
 {
     int limit, i;
     if (nelts <= 1) return;
@@ -353,13 +353,13 @@ void Scm_SortArray(ScmObj *elts, int nelts, ScmObj cmpfn)
 static ScmObj sort_list_int(ScmObj objs, ScmObj fn, int destructive)
 {
     ScmObj starray[STATIC_SIZE];
-    int len = STATIC_SIZE;
+    ScmSize len = STATIC_SIZE;
     ScmObj *array = Scm_ListToArray(objs, &len, starray, TRUE);
     Scm_SortArray(array, len, fn);
     if (destructive) {
         ScmObj cp = objs;
-        for (int i=0; i<len; i++, cp = SCM_CDR(cp)) {
-            SCM_SET_CAR(cp, array[i]);
+        for (ScmSize i=0; i<len; i++, cp = SCM_CDR(cp)) {
+            Scm_SetCar(cp, array[i]);
         }
         return objs;
     } else {
