@@ -1,7 +1,7 @@
 ;;;
 ;;; compile-1.scm - The compiler: Pass 1
 ;;;
-;;;   Copyright (c) 2004-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -59,7 +59,7 @@
 ;; we need to treat PROGRAM as a special form or an ordinary procedure.
 ;; It would be a large change, so this is a compromise...
 (define-inline (pass1/lookup-head head cenv)
-  (or (and (variable? head)
+  (or (and (identifier? head)
            (cenv-lookup-syntax cenv head))
       (and (pair? head)
            (module-qualified-variable? head cenv)
@@ -95,7 +95,7 @@
     (cond
      [(pass1/lookup-head (car program) cenv)
       => (^h (cond
-              [(identifier? h) (pass1/global-call h)]
+              [(wrapped-identifier? h) (pass1/global-call h)]
               [(lvar? h) (pass1/call program ($lref h) (cdr program) cenv)]
               [(macro? h) ;; local macro
                (pass1 (call-macro-expander h program cenv) cenv)]
@@ -117,10 +117,10 @@
                            (cdr program) cenv)))]
      [else (pass1/call program (pass1 (car program) (cenv-sans-name cenv))
                        (cdr program) cenv)])]
-   [(variable? program)                 ; variable reference
+   [(identifier? program)               ; variable reference
     (let1 r (cenv-lookup-variable cenv program)
       (cond [(lvar? r) ($lref r)]
-            [(identifier? r)
+            [(wrapped-identifier? r)
              (or (and-let* ([const (find-const-binding r)]) ($const const))
                  ($gref r))]
             [else (error "[internal] cenv-lookup returned weird obj:" r)]))]
@@ -134,9 +134,9 @@
        (not (vm-compiler-flag-is-set? SCM_COMPILE_NOINLINE_SETTERS))
        (global-identifier=? (car op) setter.)
        (and-let* ([var (cadr op)]
-                  [ (variable? var) ]   ;ok, <var> is variable
+                  [ (identifier? var) ] ;ok, <var> is variable
                   [hd (pass1/lookup-head var cenv)]
-                  [ (identifier? hd) ]
+                  [ (wrapped-identifier? hd) ]
                   [gloc (id->bound-gloc hd)] ; <var> has inlinable binding
                   [val (gloc-ref gloc)]
                   [ (procedure? val) ])
@@ -157,7 +157,7 @@
              [opt?  (slot-ref proc 'optional)])
          (unless (argcount-ok? args (slot-ref proc 'required) opt?)
            (errorf "wrong number of arguments: ~a requires ~a, but got ~a"
-                   (if (variable? name) (variable-name name) name)
+                   (if (identifier? name) (identifier->symbol name) name)
                    (slot-ref proc 'required) nargs))
          ;; We might get away with this limit by transforming inline calls
          ;; to apply or something.  Maybe in future.
@@ -191,7 +191,7 @@
 ;; thus we can afford the time.
 (define (module-qualified-variable? expr cenv)
   (match expr
-    [((? variable? wm) mod (? variable? v))
+    [((? identifier? wm) mod (? identifier? v))
      (and-let* ([var (cenv-lookup-syntax cenv wm)]
                 [ (identifier? var) ])
        (global-identifier=? var with-module.))]
@@ -249,7 +249,8 @@
              (pass1/body-finish exprs mframe vframe cenv)]
             [(and (pair? head) (eq? (car head) :rec))
              (pass1/body-finish exprs mframe vframe cenv)]
-            [(not (identifier? head)) (error "[internal] pass1/body" head)]
+            [(not (wrapped-identifier? head)) 
+             (error "[internal] pass1/body" head)]
             [(or (global-identifier=? head define.)
                  (global-identifier=? head define-inline.)
                  (global-identifier=? head r5rs-define.))
@@ -257,6 +258,10 @@
                          [((name . formals) . body)
                           `(,name :rec (,lambda. ,formals ,@body) . ,src)]
                          [(var init) `(,var :rec ,init . ,src)]
+                         [(var)
+                          (if (global-identifier=? head r5rs-define.)
+                            (error "define without expression is not allowed in R7RS" (caar exprs))
+                            `(,var :rec ,(undefined) . ,src))]
                          [_ (error "malformed internal define:" (caar exprs))])
                (if (not mframe)
                  (let* ([cenv (cenv-extend cenv '() SYNTAX)]
@@ -298,7 +303,7 @@
             [(global-identifier=? head include-ci.)
              (let1 sexpr&srcs (pass1/expand-include args cenv #t)
                (pass1/body-rec (append sexpr&srcs rest) mframe vframe cenv))]
-            [(identifier? head)
+            [(wrapped-identifier? head)
              (or (and-let* ([gloc (id->bound-gloc head)]
                             [gval (gloc-ref gloc)]
                             [ (macro? gval) ])
@@ -359,9 +364,9 @@
 
 ;; get symbol or id, and returns identiier.
 (define (ensure-identifier sym-or-id cenv)
-  (if (identifier? sym-or-id)
-    sym-or-id
-    (make-identifier sym-or-id (cenv-module cenv) (cenv-frames cenv))))
+  (if (symbol? sym-or-id)
+    (make-identifier sym-or-id (cenv-module cenv) (cenv-frames cenv))
+    sym-or-id))
 
 ;; Does the given argument list satisfy procedure's reqargs/optarg?
 (define (argcount-ok? args reqargs optarg?)
@@ -376,15 +381,14 @@
 
 ;; returns a module specified by THING.
 (define (ensure-module thing name create?)
-  (let1 mod (cond [(symbol? thing) (find-module thing)]
-                  [(identifier? thing) (find-module (unwrap-syntax thing))]
+  (let1 mod (cond [(identifier? thing) (find-module (identifier->symbol thing))]
                   [(module? thing) thing]
                   [else
                    (errorf "~a requires a module name or a module, but got: ~s"
                            name thing)])
     (or mod
         (if create?
-          (make-module (if (identifier? thing) (unwrap-syntax thing) thing))
+          (make-module (identifier->symbol thing))
           (errorf "~a: no such module: ~s" name thing)))))
 
 ;; IFORM must be a $LAMBDA node.  This expands the application of IFORM
@@ -420,9 +424,10 @@
                [(:internal) 'gauche.internal])]
         ;; a trick to assign comprehensive name to body:
         [name (string->symbol #"syntax/~(car formals)")])
-    `(let ((,name (^ ,(cdr formals) ,@body)))
-       (%insert-syntax-binding (find-module ',mod) ',(car formals)
-                               (make-syntax ',(car formals) ,name)))))
+    `(let ((,name (^ ,(cdr formals) ,@body))
+           (m (find-module ',mod)))
+       (%insert-syntax-binding m ',(car formals)
+                               (make-syntax ',(car formals) m ,name)))))
 
 (define (global-id id) (make-identifier id (find-module 'gauche) '()))
 (define (global-id% id) (make-identifier id (find-module 'gauche.internal) '()))
@@ -438,6 +443,7 @@
 (define eager.          (global-id 'eager))
 (define values.         (global-id 'values))
 (define begin.          (global-id 'begin))
+(define let.            (global-id 'let))
 (define include.        (global-id 'include))
 (define include-ci.     (global-id 'include-ci))
 (define else.           (global-id 'else))
@@ -447,7 +453,10 @@
 (define quasiquote.     (global-id 'quasiquote))
 (define unquote.        (global-id 'unquote))
 (define unquote-splicing. (global-id 'unquote-splicing))
+(define let-optionals*. (global-id 'let-optionals*))
+(define let-keywords*.  (global-id 'let-keywords*))
 
+(define make-case-lambda. (global-id% 'make-case-lambda))
 (define %make-er-transformer.          (global-id% '%make-er-transformer))
 (define %make-er-transformer/toplevel. (global-id% '%make-er-transformer/toplevel))
 (define %with-inline-transformer.      (global-id% '%with-inline-transformer))
@@ -501,9 +510,18 @@
                         `(,(if extended? lambda. r5rs-lambda.) ,args ,@body)
                         oform))
                    oform flags extended? module cenv)]
+
+    [(_ name)
+     (if extended?
+       ;; Gauche's define allows R6RS-style (define <name>).
+       (pass1/define `(define ,name ,(undefined))
+                     oform flags #t module cenv)
+       ;; R7RS define doesn't allow it.
+       (error "define without an expression is not allowed in R7RS (it is in R6RS):" oform))]
     [(_ name expr)
-     (unless (variable? name) (error "syntax-error:" oform))
+     (unless (identifier? name) (error "syntax-error:" oform))
      (let1 cenv (cenv-add-name cenv (variable-name name))
+       ;; Hygiene alert
        ;; If NAME is an identifier, it is inserted by macro expander; we
        ;; can't simply place it in $define, since it would insert a toplevel
        ;; definition into the toplevel of macro-definition environment---
@@ -513,9 +531,15 @@
        ;; other code except the code generated in the same macro expansion.
        ;; A trick - we directly modify the identifier, so that other forms
        ;; referring to the same (eq?) identifier can keep referring it.
-       (let1 id (if (identifier? name)
+       (let1 id (if (wrapped-identifier? name)
                   (%rename-toplevel-identifier! name)
                   (make-identifier name module '()))
+         ;; Insert dummy binding at compile time, if we don't have one yet.
+         ;; This matters when we compile multiple modules at once, and
+         ;; one need to import from another with qualifiers.
+         (unless (vm-compiler-flag-is-set? SCM_COMPILE_LEGACY_DEFINE)
+           (%insert-binding module (unwrap-syntax name) 
+                            (%uninitialized) '(fresh)))
          ($define oform flags id (pass1 expr cenv))))]
     [_ (error "syntax-error:" oform)]))
 
@@ -530,7 +554,7 @@
 ;;
 ;;   Processing define-inline involves two actions.
 ;;   (1) Process the lambda node to be inlined.  A packed IForm should be
-;;       attached, and if the lambda node closes enviornment, some code
+;;       attached, and if the lambda node closes environment, some code
 ;;       transformation is required.
 ;;   (2) Bind the resulting node to the global name, and mark the binding
 ;;       'inlinable'.
@@ -561,7 +585,7 @@
     [(_ (name . args) . body)
      (pass1/define-inline form name `(,lambda. ,args ,@body) cenv)]
     [(_ name expr)
-     (unless (variable? name) (error "syntax-error:" form))
+     (unless (identifier? name) (error "syntax-error:" form))
      (pass1/define-inline form name expr cenv)]
     [_ (error "syntax-error: malformed define-inline:" form)]))
 
@@ -589,7 +613,7 @@
                      ,(pass1/make-inlinable-binding form name closure cenv)))))]
        ))))
 
-;; If IFORM generats a closure with local environment, returns
+;; If IFORM generates a closure with local environment, returns
 ;; the closure itself ($lambda node) and the environment
 ;; ((lvar . init) ...).
 ;; Typical case is ($let ... ($lambda ...)).  In such case this
@@ -652,7 +676,7 @@
 
 (define (pass1/make-inlinable-binding form name iform cenv)
   ;; See the comment in pass1/define about renaming the toplevel identifier.
-  (let1 id (if (identifier? name)
+  (let1 id (if (wrapped-identifier? name)
              (%rename-toplevel-identifier! name)
              (make-identifier name (cenv-module cenv) '()))
     ($define form '(inlinable) id iform)))
@@ -676,14 +700,19 @@
     [_ (error "syntax-error:" form)]))
 
 (define (pass1/define-macro src name expr cenv)
-  (unless (variable? name) (error "syntax-error:" src))
+  (unless (identifier? name) (error "syntax-error:" src))
   ;; TODO: macro autoload
   (let* ([proc (eval expr (cenv-module cenv))]
          [trans (%make-macro-transformer name
                                          (^[form env] (apply proc (cdr form)))
-                                         expr #f)])
-    ;; See the "Hygiene alert" in pass1/define.
-    (%insert-syntax-binding (cenv-module cenv) (unwrap-syntax name) trans)
+                                         expr #f)]
+         ;; See the "Hygiene alert" in pass1/define.
+         [id (if (wrapped-identifier? name)
+               (%rename-toplevel-identifier! name)
+               (make-identifier name (cenv-module cenv) '()))])
+    (%insert-syntax-binding (identifier-module id)
+                            (unwrap-syntax name)
+                            trans)
     ($const-undef)))
 
 (define-pass1-syntax (define-syntax form cenv) :null
@@ -693,25 +722,34 @@
   (match form
     [(_ name expr)
      (let* ([cenv (cenv-add-name cenv (variable-name name))]
-            [transformer (pass1/eval-macro-rhs 'define-syntax expr cenv)])
-       ;; See the "Hygiene alert" in pass1/define.
-       (%insert-syntax-binding (cenv-module cenv) (unwrap-syntax name)
-                               transformer)
+            [trans (pass1/eval-macro-rhs 'define-syntax expr cenv)]
+            ;; See the "Hygiene alert" in pass1/define.
+            [id (if (wrapped-identifier? name)
+                  (%rename-toplevel-identifier! name)
+                  (make-identifier name (cenv-module cenv) '()))])
+       (%insert-syntax-binding (identifier-module id)
+                               (unwrap-syntax name)
+                               trans)
        ($const-undef))]
     [_ (error "syntax-error: malformed define-syntax:" form)]))
 
 ;; Experimental
-(define-pass1-syntax (define-inline/syntax form cenv) :gauche
+(define-pass1-syntax (define-hybrid-syntax form cenv) :gauche
+  (pass1-define-hybrid-syntax form cenv))
+(define-pass1-syntax (define-inline/syntax form cenv) :gauche ;deprecated
+  (pass1-define-hybrid-syntax form cenv))
+
+(define (pass1-define-hybrid-syntax form cenv)
   (check-toplevel form cenv)
   (match form
     [(_ name expr macro-expr)
      (let* ([cenv (cenv-add-name cenv (variable-name name))]
-            [xformer (pass1/eval-macro-rhs 'define-inline/syntax
+            [xformer (pass1/eval-macro-rhs 'define-hybrid-syntax
                                            macro-expr cenv)]
             [body (pass1/call expr ($gref %with-inline-transformer.)
                               (list expr xformer) cenv)])
        (pass1/make-inlinable-binding form name body cenv))]
-    [_ (error "syntax-error: define-inline/syntax")]))
+    [_ (error "syntax-error: define-hybrid-syntax")]))
             
 ;; Returns either <syntax> or <macro>
 (define (pass1/eval-macro-rhs who expr cenv)
@@ -751,9 +789,22 @@
                                                 ,(cenv-module cenv)
                                                 ',(cenv-exp-name cenv))
               cenv)
-       (pass1 `(,%make-er-transformer. ,xformer ,cenv) cenv))]
+       (pass1 `(,%make-er-transformer. ,xformer ,cenv #f) cenv))]
     [_ (error "syntax-error: malformed er-macro-transformer:" form)]))
 
+(define-pass1-syntax (eri-macro-transformer form cenv) :gauche
+  (match form
+    [(_ xformer)
+     ;; See the comment above in er-macro-transformer
+     (if (cenv-toplevel? cenv)
+       (pass1 `(,%make-er-transformer/toplevel. ,xformer
+                                                ,(cenv-module cenv)
+                                                ',(cenv-exp-name cenv)
+                                                #t)
+              cenv)
+       (pass1 `(,%make-er-transformer. ,xformer ,cenv #t) cenv))]
+    [_ (error "syntax-error: malformed eri-macro-transformer:" form)]))
+
 (define-pass1-syntax (%macroexpand form cenv) :gauche
   (match form
     [(_ expr) ($const (%internal-macro-expand expr cenv #f))]
@@ -767,10 +818,10 @@
 (define (%internal-macro-expand expr cenv once?)
   (define (xpand expr)
     (match expr
-      [((? variable? op) . args)
+      [((? identifier? op) . args)
        (let1 var (cenv-lookup-syntax cenv op)
          (cond [(macro? var) (call-macro-expander var expr cenv)]
-               [(identifier? var)
+               [(wrapped-identifier? var)
                 (if-let1 gval (and-let* ([gloc (id->bound-gloc var)]
                                          [gval (gloc-ref gloc)]
                                          [ (macro? gval) ])
@@ -990,34 +1041,42 @@
     (match binds
       [() (pass1/body body cenv)]
       [((exp) . more)
-       ($if form (pass1 exp (cenv-sans-name cenv))
-            (process-binds more body cenv)
-            ($it))]
-      [([? variable? var] . more)
-       ($if form (pass1 var (cenv-sans-name cenv))
-            (process-binds more body cenv)
-            ($it))]
-      [(([? variable? var] init) . more)
-       (let* ([lvar (make-lvar var)]
-              [newenv (cenv-extend cenv `((,var . ,lvar)) LEXICAL)]
-              [itree (pass1 init (cenv-add-name cenv var))])
-         (lvar-initval-set! lvar itree)
-         ($let form 'let
-               (list lvar)
-               (list itree)
-               ($if form ($lref lvar)
-                    (process-binds more body newenv)
-                    ($it))))]
+       (if (and (null? more) (null? body))
+         (pass1 exp (cenv-sans-name cenv))
+         ($if form (pass1 exp (cenv-sans-name cenv))
+              (process-binds more body cenv)
+              ($it)))]
+      [([? identifier? var] . more)
+       (if (and (null? more) (null? body))
+         (pass1 var (cenv-sans-name cenv))
+         ($if form (pass1 var (cenv-sans-name cenv))
+              (process-binds more body cenv)
+              ($it)))]
+      [(([? identifier? var] init) . more)
+       (if (and (null? more) (null? body))
+         (pass1 init (cenv-add-name cenv var))
+         (let* ([lvar (make-lvar var)]
+                [newenv (cenv-extend cenv `((,var . ,lvar)) LEXICAL)]
+                [itree (pass1 init (cenv-add-name cenv var))])
+           (lvar-initval-set! lvar itree)
+           ($let form 'let
+                 (list lvar)
+                 (list itree)
+                 ($if form ($lref lvar)
+                      (process-binds more body newenv)
+                      ($it)))))]
       [_ (error "syntax-error: malformed and-let*:" form)]))
 
   (match form
+    [(_ ()) ($const #t)]                ;special base case
     [(_ binds . body) (process-binds binds body cenv)]
     [_ (error "syntax-error: malformed and-let*:" form)]))
 
 ;; Quote and quasiquote ................................
 
 (define (pass1/quote obj)
-  ($const (unwrap-syntax obj)))
+  ($const ($ unwrap-syntax obj 
+             $ not $ vm-compiler-flag-is-set? SCM_COMPILE_MUTABLE_LITERALS)))
 
 (define-pass1-syntax (quote form cenv) :null
   (match form
@@ -1025,6 +1084,27 @@
     [else (error "syntax-error: malformed quote:" form)]))
 
 (define-pass1-syntax (quasiquote form cenv) :null
+  (match form
+    [(_ obj) (quasi-expand obj cenv)]
+    [_ (error "syntax-error: malformed quasiquote:" form)]))
+
+(define-pass1-syntax (unquote form cenv) :null
+  (error "unquote appeared outside quasiquote:" form))
+
+(define-pass1-syntax (unquote-splicing form cenv) :null
+  (error "unquote-splicing appeared outside quasiquote:" form))
+
+;; We need these to be bound so that scheme.base can export them
+;; as specified in R7RS.
+(define-pass1-syntax (_ form cenv) :null
+  ($const-undef))
+
+(define-pass1-syntax (... form cenv) :null
+  ($const-undef))
+
+;; quasiquote expander
+
+(define (quasi-expand obj cenv)
   ;; We want to avoid unnecessary allocation as much as possible.
   ;; Current code generates constants not only the obvious constant
   ;; case, e.g. `(a b c), but also folds constant variable references,
@@ -1032,10 +1112,10 @@
   ;; This extends as far as the pass-1 constant folding goes, so `(,(+ x 1))
   ;; also becomes '(4).
   ;; NB: The current code allocates lots of intermediate $const node.
-
   (define (quasiquote? v)       (global-eq? v quasiquote. cenv))
   (define (unquote? v)          (global-eq? v unquote. cenv))
   (define (unquote-splicing? v) (global-eq? v unquote-splicing. cenv))
+  (define (unquote*? v) (or (unquote? v) (unquote-splicing? v)))
 
   ;; In the context where there's no outer list to which we intersperse to.
   (define (quasi obj level)
@@ -1045,19 +1125,23 @@
          (if ($const? xx)
            ($const (list 'quasiquote ($const-value xx)))
            ($list obj (list ($const 'quasiquote) xx))))]
-      [((and (or (? unquote?) (? unquote-splicing?)) op) . xs)
+      [((? unquote?) x)
        (if (zero? level)
-         (if (and (unquote? op)
-                  (pair? xs) (null? (cdr xs)))
-           (pass1 (car xs) cenv)
-           (errorf "invalid ~a form in this context: ~s" op obj))
+         (pass1 x cenv)
+         (let1 xx (quasi x (- level 1))
+           (if ($const? xx)
+             ($const (list 'unquote ($const-value xx)))
+             ($list obj (list ($const 'unquote) xx)))))]
+      [((? unquote*? op) . xs) ;valid unquote is already handled
+       (if (zero? level)
+         (errorf "invalid ~a form in this context: ~s" op obj)
          (let1 xx (quasi* xs (- level 1))
            (if ($const? xx)
              ($const (cons op ($const-value xx)))
              ($cons obj ($const op) xx))))]
       [(? pair?)       (quasi* obj level)]
       [(? vector?)     (quasi-vector obj level)]
-      [(? identifier?) ($const (unwrap-syntax obj))]
+      [(? wrapped-identifier?) ($const (unwrap-syntax obj))]
       [() ($const-nil)]
       [_  ($const obj)]))
 
@@ -1065,7 +1149,7 @@
   (define (quasi* objs level)
     ;; NB: we already excluded toplevel quasiquote and unquote
     (match objs
-      [(((and (or (? unquote?) (? unquote-splicing?)) op) . xs) . ys)
+      [(((? unquote*? op) . xs) . ys)
        (let1 yy (quasi* ys level)
          (if (zero? level)
            ((if (unquote? op) build build@)
@@ -1074,7 +1158,7 @@
              (if (and ($const? xx) ($const? yy))
                ($const (acons op ($const-value xx) ($const-value yy)))
                ($cons objs ($cons (car objs) ($const op) xx) yy)))))]
-      [((or (? unquote?) (? unquote-splicing?)) . _) ;`(... . ,xs) `(... . ,@xs)
+      [((? unquote*?) . _) ;`(... . ,xs) `(... . ,@xs)
        (quasi objs level)]
       [((? vector? x) . ys) (quasi-cons objs quasi-vector x ys level)]
       [(x . ys)             (quasi-cons objs quasi x ys level)]
@@ -1126,29 +1210,46 @@
             [(and (pair? (vector-ref obj i))
                   (unquote-splicing? (car (vector-ref obj i))))]
             [else (loop (+ i 1))])))
+  
+  (quasi obj 0))
 
-  (match form
-    [(_ obj) (quasi obj 0)]
-    [_ (error "syntax-error: malformed quasiquote:" form)]))
-
-(define-pass1-syntax (unquote form cenv) :null
-  (error "unquote appeared outside quasiquote:" form))
-
-(define-pass1-syntax (unquote-splicing form cenv) :null
-  (error "unquote-splicing appeared outside quasiquote:" form))
 
 ;; Lambda family (binding constructs) ...................
 
-(define-pass1-syntax (lambda form cenv) :null
+(define-pass1-syntax (lambda form cenv) :null ;RnRS lambda
   (match form
     [(_ formals . body)
-     (pass1/lambda (add-arg-info form formals) formals body cenv #f)]
+     (receive (reqs rest) 
+         (let loop ((xs formals) (ys '()))
+           (cond [(null? xs) (values (reverse ys) #f)]
+                 [(identifier? xs) (values (reverse ys) xs)]
+                 [(pair? xs) 
+                  (unless (identifier? (car xs))
+                    (error "Invalid formal parameter:" (car xs)))
+                  (loop (cdr xs) (cons (car xs) ys))]
+                 [else (error "Invalid formal parameter:" formals)]))
+       (pass1/vanilla-lambda (add-arg-info form formals) 
+                             (if rest (append reqs (list rest)) reqs)
+                             (length reqs)
+                             (if rest 1 0)
+                             body cenv))]
     [_ (error "syntax-error: malformed lambda:" form)]))
 
-(define-pass1-syntax (lambda form cenv) :gauche
+(define-pass1-syntax (lambda form cenv) :gauche ;Extended lambda
   (match form
     [(_ formals . body)
-     (pass1/lambda (add-arg-info form formals) formals body cenv #t)]
+     (receive (args nreqs nopts kargs) (parse-extended-lambda-args formals)
+       (if (null? kargs)
+         (pass1/vanilla-lambda (add-arg-info form formals)
+                               args nreqs nopts body cenv)
+         ;; Convert extended lambda into vanilla lambda
+         (let1 restarg (gensym "rest")
+           (pass1/vanilla-lambda (add-arg-info form formals)
+                                 (append args (list restarg))
+                                 nreqs 1
+                                 (pass1/extended-lambda-body form cenv restarg
+                                                             kargs body)
+                                 cenv))))]
     [_ (error "syntax-error: malformed lambda:" form)]))
 
 ;; Add formals list as 'arg-info attributes of the source form
@@ -1158,41 +1259,30 @@
                  (extended-cons (car form) (cdr form)))
     (pair-attribute-set! xform 'arg-info formals)))
 
-(define (pass1/lambda form formals body cenv extended?)
-  (receive (args reqargs optarg kargs) (parse-lambda-args formals)
-    (cond [(null? kargs)
-           (let* ([lvars (imap make-lvar+ args)]
-                  [intform ($lambda form (cenv-exp-name cenv)
-                                    reqargs optarg lvars #f #f)]
-                  [newenv (cenv-extend/proc cenv (%map-cons args lvars)
-                                            LEXICAL intform)])
-             (vector-set! intform 6 (pass1/body body newenv))
-             intform)]
-          [(not extended?)
-           (error "syntax-error: extended formals aren't allowed in R5RS \
-                   lambda:" form)]
-          [else
-           (let1 g (gensym)
-             (pass1/lambda form (append args g)
-                           (pass1/extended-lambda form cenv g kargs body)
-                           cenv #f))])))
+(define (pass1/vanilla-lambda form formals nreqs nopts body cenv) ; R7RS lambda
+  (let* ([lvars (imap make-lvar+ formals)]
+         [intform ($lambda form (cenv-exp-name cenv) nreqs nopts lvars #f #f)]
+         [newenv (cenv-extend/proc cenv (%map-cons formals lvars)
+                                   LEXICAL intform)])
+    (vector-set! intform 6 (pass1/body body newenv))
+    intform))
 
 (define-pass1-syntax (receive form cenv) :gauche
   (match form
     [(_ formals expr body ...)
-     (receive (args reqargs optarg kargs) (parse-lambda-args formals)
+     (receive (args nreqs nopts kargs) (parse-extended-lambda-args formals)
        (unless (null? kargs)
          (error "syntax-error: extended lambda list isn't allowed in receive:"
                 form))
        (let* ([lvars (imap make-lvar+ args)]
               [newenv (cenv-extend cenv (%map-cons args lvars) LEXICAL)])
-         ($receive form reqargs optarg lvars (pass1 expr cenv)
+         ($receive form nreqs nopts lvars (pass1 expr cenv)
                    (pass1/body body newenv))))]
     [_ (error "syntax-error: malformed receive:" form)]))
 
 ;; Returns <list of args>, <# of reqargs>, <has optarg?>, <kargs>
 ;; <kargs> is like (:optional (x #f) (y #f) :rest k) etc.
-(define (parse-lambda-args formals)
+(define (parse-extended-lambda-args formals)
   (let loop ([formals formals] [args '()] [n 0])
     (match formals
       [()      (values (reverse args) n 0 '())]
@@ -1202,7 +1292,7 @@
 
 ;; Handles extended lambda list.  garg is a gensymed var that receives
 ;; restarg.
-(define (pass1/extended-lambda form cenv garg kargs body)
+(define (pass1/extended-lambda-body form cenv garg kargs body)
   (define (collect-args xs r)
     (match xs
       [() (values (reverse r) '())]
@@ -1240,17 +1330,16 @@
   (define (expand-opt os ks r a)
     (if (null? os)
       (if r
-        `(((with-module gauche let) ((,r ,garg)) ,@(expand-key ks garg a)))
+        `((,let. ((,r ,garg)) ,@(expand-key ks garg a)))
         (expand-key ks garg a))
       (let ([binds (map (match-lambda
-                          [[? symbol? o] o]
                           [[? identifier? o] o]
                           [(o init) `(,o ,init)]
                           [_ (error "illegal optional argument spec in " kargs)])
                         os)]
             [rest (or r (gensym))])
-        `(((with-module gauche let-optionals*) ,garg ,(append binds rest)
-           ,@(if (and (not r) (null? ks))
+        `((,let-optionals*. ,garg ,(append binds rest)
+           ,@(if (and (not r) (null? ks) (not a))
                ;; TODO: better error message!
                `((unless (null? ,rest)
                    (error "too many arguments for" ',form))
@@ -1258,19 +1347,22 @@
                (expand-key ks rest a)))))))
   (define (expand-key ks garg a)
     (if (null? ks)
-      body
+      (if a
+        ;; The case when we have :allow-other-keys without :key.
+        ;; We don't deal with specific keyword arguments, but expecting
+        ;; the user provides some.  Using let-keywords* checks the
+        ;; argument list is even.
+        `((,let-keywords*. ,garg ,(if (boolean? a) (gensym) a) ,@body))
+        body)
       (let1 args (map (match-lambda
-                        [[? symbol? o] o]
                         [[? identifier? o] o]
                         [(([? keyword-like? key] o) init)
-                         (let1 k (if (identifier? key)
-                                   (identifier->symbol key)
-                                   key)
+                         (let1 k (unwrap-syntax-1 key)
                            `(,o ,k ,init))]
                         [(o init) `(,o ,init)]
                         [_ (error "illegal keyword argument spec in " kargs)])
                       ks)
-        `(((with-module gauche let-keywords*) ,garg
+        `((,let-keywords*. ,garg
            ,(if a (append args a) args)
            ,@body)))))
 
@@ -1285,9 +1377,7 @@
     [(_) (error "syntax-error: malformed case-lambda:" form)]
     [(_ (formals . body) ...)
      (receive (min-req max-req) (find-argcount-minmax formals)
-       (pass1 `(,(make-identifier 'make-case-lambda
-                                  (find-module 'gauche.internal)
-                                  '())
+       (pass1 `(,make-case-lambda.
                 ,min-req ,max-req ',formals
                 (list ,@(map (^(f b) `(,lambda. ,f ,@b)) formals body))
                 ',(cenv-exp-name cenv))
@@ -1318,7 +1408,7 @@
                   expr lvars)
              (pass1/body body newenv)))]
     [(_ name ((var expr) ...) body ...)
-     (unless (variable? name) (error "bad name for named let:" name))
+     (unless (identifier? name) (error "bad name for named let:" name))
      ;; Named let.  (let name ((var exp) ...) body ...)
      ;;
      ;;  We don't use the textbook expansion here
@@ -1426,7 +1516,7 @@
      ;; have a chance of optimization.
      (pass1 (with-original-source `((,setter. ,op) ,@args ,expr) form) cenv)]
     [(_ name expr)
-     (unless (variable? name)
+     (unless (identifier? name)
        (error "syntax-error: malformed set!:" form))
      (let ([var (cenv-lookup-variable cenv name)]
            [val (pass1 expr cenv)])
@@ -1511,8 +1601,8 @@
       [((? symbol-but-not-keyword? a) (? symbol-but-not-keyword? b) . r)
        ;;likely to be an r7rs-style import
        (error "This import form looks like R7RS `import', as opposed to \
-               Gauche `import'.  If you're in REPL, type (use r7rs) \
-               and (select-module r7rs.user) to enter the R7RS namespace.")]
+               Gauche `import'.  If you're in REPL, type (use scheme.base) \
+               then (select-module r7rs.user) to enter the R7RS namespace.")]
       [(m . r) (process-import (cenv-module cenv) (ensure m) r)]
       [m       (process-import (cenv-module cenv) (ensure m) '())]))
   ($values0))
