@@ -1,7 +1,7 @@
 ;;;
 ;;; compile-2.scm - The compiler: Pass 2
 ;;;
-;;;   Copyright (c) 2004-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -432,7 +432,8 @@
       ($call-flag-set! (car call) 'local))
     ;; We clear the calls list, just in case if the lambda-node is
     ;; traversed more than once.
-    ($lambda-calls-set! lambda-node '())))
+    ($lambda-calls-set! lambda-node '())
+    #t))
 
 ;; Called when the local function (lambda-node) isn't needed to be a closure
 ;; and can be embedded.
@@ -454,7 +455,8 @@
         ($call-args-set! jcall (adjust-arglist nreqs nopts ($call-args jcall)
                                                name))
         ($call-proc-set! jcall call)
-        ($call-flag-set! jcall 'jump)))))
+        ($call-flag-set! jcall 'jump)))
+    #t))
 
 ;; Called when the local function (lambda-node) doesn't have recursive
 ;; calls, can be inlined, and called from multiple places.
@@ -475,7 +477,8 @@
   ($lambda-flag-set! lambda-node 'dissolved)
   (let loop ([calls calls])
     (cond [(null? (cdr calls))
-           (inline-it (car calls) lambda-node)]
+           (inline-it (car calls) lambda-node)
+           #t]
           [else
            (inline-it (car calls) (iform-copy lambda-node '()))
            (loop (cdr calls))])))
@@ -495,10 +498,6 @@
   ;; to do it again.
   iform)
 
-(define (pass2/$PROMISE iform penv tail?)
-  ($promise-expr-set! iform (pass2/rec ($promise-expr iform) penv #f))
-  iform)
-
 (define (pass2/$SEQ iform penv tail?)
   (if (null? ($seq-body iform))
     iform
@@ -618,9 +617,91 @@
 (define (pass2/self-recursing? node penv)
   (find (cut eq? node <>) penv))
 
+;; Recognize the pattern like:
+;;   ($let ([args ($LIST x y z ...)])
+;;      ...
+;;      ($asm (TAIL-APPLY n) proc ... a ... args))
+;; We can replace $asm node with a simple $CALL:
+;;      ($call proc a ... x y z ...)
+;; This pattern typically appears after inline expansion.
+;;
+;; We only do this transformation if every x y z ... is either $lref
+;; or $const.  If there's more complex expression, we can't change the order
+;; of evaluation, so we'd have to modify the outer $let form---which isn't 
+;; trivial, so we just give up such a case.
+;;
+;; If the argument list is constructed (as opposed to be a constant),
+;; we also need to consider the possibility that the list can be mutated
+;; before passed to apply.  We check if the intermediate variables are
+;; used elsewhere, and give up if they are passed to other procedures.
+;;
+;; IForm must be an $ASM node.
+;; Returns (potentially updated) iform.
+(define (pass2/dissolve-apply iform penv tail?)
+  ;; Returns a list of arguments as list of IForms, or #f if the arguments
+  ;; isn't a proper list.  We follow immutable local bindings.
+  (define (expand-restarg rarg)
+    (cond [($const? rarg)
+           (and (list? ($const-value rarg))
+                (imap (^v ($const v)) ($const-value rarg)))]
+          [($lref? rarg)
+           (and-let* ([lvar ($lref-lvar rarg)]
+                      [val (lvar-const-value lvar)]
+                      [r (expand-restarg val)])
+             ;; Record lvar.  If we decide to dissolve, we need to adjust
+             ;; refcount of lvar later. (We can't do it now, for we may abandon
+             ;; inlining later.
+             (push! freed-lvars lvar)
+             r)]
+          [(has-tag? rarg $LIST) (check-safe ($list-args rarg))]
+          [(has-tag? rarg $CONS)
+           (and-let* ([ (safe? ($cons-arg0 rarg)) ]
+                      [z (expand-restarg ($cons-arg1 rarg))])
+             (cons ($cons-arg0 rarg) z))]
+          [(has-tag? rarg $LIST*)
+           (and-let* ([z (expand-restarg (last ($list*-args rarg)))]
+                      [y (check-safe (drop-right ($list*-args rarg) 1))])
+             (append y z))]
+          [(has-tag? rarg $ASM)
+           (case/unquote 
+            (car ($asm-insn rarg))
+            [(LIST) (check-safe ($asm-args rarg))]
+            [(LIST-STAR) 
+             (and-let* ([z (expand-restarg (last ($asm-args rarg)))]
+                        [y (check-safe (drop-right ($asm-args rarg) 1))])
+               (append y z))]
+            [else #f])]
+          [else #f]))
+  (define freed-lvars '())
+  (define (safe? x) (or ($lref? x) ($const? x)))
+  (define (check-safe xs) ;; returns xs or #f
+    (cond [(null? xs) xs]
+          [(safe? (car xs)) (and-let1 z (check-safe (cdr xs))
+                              (cons (car xs) z))]
+          [else #f]))
+  (define args ($asm-args iform))
+  (assume (length>=? args 2))
+  (if (vm-compiler-flag-is-set? SCM_COMPILE_NODISSOLVE_APPLY)
+    iform
+    (or (and-let* ([iargs (expand-restarg (last args))]
+                   [ (every (^v (= (lvar-ref-count v) 1)) freed-lvars) ])
+          (ifor-each lvar-ref--! freed-lvars)
+          (ifor-each (^x (when ($lref? x) (lvar-ref++! ($lref-lvar x)))) iargs)
+          (pass2/rec ($call ($*-src iform) (car args)
+                            (append (drop-right (cdr args) 1) iargs))
+                     penv tail?))
+        iform)))
+
 (define (pass2/$ASM iform penv tail?)
-  (let1 args (imap (cut pass2/rec <> penv #f) ($asm-args iform))
-    (pass2/check-constant-asm iform args)))
+  (define (do-asm)
+    ($ pass2/check-constant-asm iform
+       (imap (cut pass2/rec <> penv #f) ($asm-args iform))))
+  (if (eqv? (car ($asm-insn iform)) TAIL-APPLY)
+    (let1 iform2 (pass2/dissolve-apply iform penv tail?)
+      (if (eq? iform iform2) ; didn't resolve
+        (do-asm)
+        iform2))
+    (do-asm)))
 
 (define (pass2/check-constant-asm iform args)
   (or (and (every $const? args)
@@ -648,6 +729,7 @@
             [(VEC-LEN) (pass2/const-veclen args)]
             [(EQ)      (pass2/const-op2 eq? args)]
             [(EQV)     (pass2/const-op2 eqv? args)]
+            [(NUMEQ2)  (pass2/const-numop2 = args)]
             [(NUMADD2) (pass2/const-numop2 + args)]
             [(NUMSUB2) (pass2/const-numop2 - args)]
             [(NUMMUL2) (pass2/const-numop2 * args)]
@@ -717,13 +799,13 @@
        (let1 tag (iform-tag val)
          (or (and (eqv? tag $LIST) (not (null? ($*-args val))))
              (and (eqv? tag $LIST*) (not (null? ($*-args val))))
-             (memv tag `(,$LAMBDA ,$PROMISE ,$CONS ,$EQ? ,$EQV?
+             (memv tag `(,$LAMBDA ,$CONS ,$EQ? ,$EQV?
                                   ,$VECTOR ,$LIST->VECTOR))))))
 
 (define (initval-never-false? val)
   (and (vector? val)
        (let1 tag (iform-tag val)
-         (memv tag `(,$LAMBDA ,$PROMISE ,$CONS ,$VECTOR
+         (memv tag `(,$LAMBDA ,$CONS ,$VECTOR
                      ,$LIST->VECTOR ,$LIST)))))
 
 (define (initval-always-pair? val)
