@@ -1,7 +1,7 @@
 ;;;
 ;;; compile-3.scm - The compiler: Pass 3
 ;;;
-;;;   Copyright (c) 2004-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -42,7 +42,7 @@
 
 ;; Dispatch pass3 handler.
 ;; Each handler is called with IForm and a list of label nodes.
-;; Returs IForm.
+;; Returns IForm.
 ;; *pass3-dispatch-table* is defined below, after all handlers are defined.
 (define-inline (pass3/rec iform labels)
   ((vector-ref *pass3-dispatch-table* (iform-tag iform)) iform labels))
@@ -174,9 +174,67 @@
     (pass2/shrink-let-frame iform lvars (pass3/rec ($let-body iform) labels))))
 
 (define (pass3/$RECEIVE iform labels)
+  ;; There's a common idiom of "save the result, do something, then return
+  ;; the result."  Considering that the result may be more than one value,
+  ;; it takes this pattern, frequently generated by macros:
+  ;;
+  ;;  (receive var expr
+  ;;     .... do something without var
+  ;;    (apply values var))
+  ;;
+  ;; This incurs overhead of packing and unpacking multiple values, especially
+  ;; if expr yields only one value, which is majority of the case.
+  ;; If we know expr can only yield single value, we can simplify it with
+  ;; the following:
+  ;;
+  ;;  (let ((var expr))
+  ;;    .... do something without var...
+  ;;    var)
+  ;;
+
+  ;; #t iff we definitely know iform yields a single value.
+  (define (single-value-iform? iform)
+    (or (memv (iform-tag iform)
+              `(,$CONST ,$LREF ,$GREF ,$LAMBDA ,$CONS ,$APPEND ,$VECTOR
+                        ,$LIST->VECTOR ,$LIST ,$LIST* ,$MEMV ,$EQ? ,$EQV?))
+        (and (has-tag? iform $IF)
+             (single-value-iform? ($if-then iform))
+             (single-value-iform? ($if-else iform)))
+        (and (has-tag? iform $ASM)
+             (not (insn-multi-value? (car ($asm-insn iform)))))
+        ))
+
+  ;; first, do the business in the subtree.
   ($receive-expr-set! iform (pass3/rec ($receive-expr iform) labels))
   ($receive-body-set! iform (pass3/rec ($receive-body iform) labels))
-  iform)
+  ;; see if we can optimize
+  (or (and-let* ([ (= ($receive-reqargs iform) 0) ]
+                 [ (= ($receive-optarg iform) 1) ]
+                 [lvar (car ($receive-lvars iform))]
+                 [ (= (lvar-ref-count lvar) 1) ]
+                 [ (single-value-iform? ($receive-expr iform)) ]
+                 [body ($receive-body iform)]
+                 [ (has-tag? body $SEQ) ]
+                 [last-expr (last ($seq-body body))]
+                 [ (has-tag? last-expr $ASM) ]
+                 [ (eq? (car ($asm-insn last-expr)) TAIL-APPLY) ]
+                 [last-args ($asm-args last-expr)]
+                 [ (length=? last-args 2) ]
+                 [ (has-tag? (car last-args) $GREF) ]
+                 [ (free-identifier=? ($gref-id (car last-args)) values.)]
+                 [ ($lref? (cadr last-args)) ]
+                 [ (eq? ($lref-lvar (cadr last-args)) lvar) ]
+                 )
+        ;; OK, we can.
+        (let ([newbody ($seq (append (drop-right ($seq-body body) 1)
+                                     (cdr last-args)))]) ;; $LREF
+          (pass3/rec ($let ($receive-src iform)
+                       'let
+                       (list lvar)
+                       (list ($receive-expr iform))
+                       newbody)
+                     labels)))
+      iform))
 
 (define (pass3/$LAMBDA iform labels)
   ($lambda-body-set! iform (pass3/rec ($lambda-body iform) labels))
@@ -188,10 +246,6 @@
     ($label-body-set! iform (pass3/rec ($label-body iform) labels)))
   iform)
 
-(define (pass3/$PROMISE iform labels)
-  ($promise-expr-set! iform (pass3/rec ($promise-expr iform) labels))
-  iform)
-
 ;; We may have a dead code in $SEQ as the result of pass2 main.
 ;; We eliminate if the value of subtree is not used, and it is
 ;; referentially transparent.
@@ -206,8 +260,14 @@
                       ($seq-body-set! iform
                                       (reverse! (cons (pass3/rec x labels) r)))
                       iform])]
-          [(x . xs) (let1 x. (pass3/rec x labels)
-                      (loop (if (transparent? x.) r (cons x. r)) xs))])))))
+          [(x . (and (y . ys) xs))
+           ;; Skip empty $seq if it immediately follows x.
+           ;; This prevents the constant expression followed by empty $seq as
+           ;; the last expression from being removed as the dead code.
+           (if (and (has-tag? y $SEQ) (null? ($seq-body y)))
+             (loop r (cons x ys))
+             (let1 x. (pass3/rec x labels)
+               (loop (if (transparent? x.) r (cons x. r)) xs)))])))))
 
 ;; Some extra optimization on $CALL.  We need to run this here, since
 ;; $CALL classifications needs to be done by the surrounding $LET.
@@ -247,9 +307,9 @@
                       [p (gref-inlinable-proc proc)])
              (or (and (%procedure-inliner p)
                       (pass3/late-inline iform proc p labels))
-                 (and (slot-ref p 'constant)
-                      (every $const? args)
-                      (pass3/precompute-constant p args))))]
+                 (and-let* ([ (procedure-constant? p) ]
+                            [constargs (pass3/constant-arguments args)])
+                   (pass3/precompute-constant p constargs))))]
           [(and-let* ([ (has-tag? proc $GREF) ]
                       [ (pair? args) ]
                       [ (null? (cdr args)) ]
@@ -258,7 +318,7 @@
                              (car args))])
              (pass3/deduce-predicate-result proc val))]
           [;; Like above, but we follow $LREFs.
-           ;; NB: We tempted to inline calles if the $lref is bound to $lambda
+           ;; NB: We tempted to inline calls if the $lref is bound to $lambda
            ;; node and it has reference count 1---in a non-redundant program,
            ;; it means the $lambda is no recursive call.  Unfortunately it
            ;; breaks when $lambda does have recursive call to itself, and
@@ -288,6 +348,19 @@
              [ (procedure? val) ])
     val))
 
+;; Get the constant or inlinable value for each iform in the list; if
+;; we can't get such value from any one, returns #f.
+(define (pass3/constant-arguments iforms)
+  (let loop ([iforms iforms] [args '()])
+    (cond 
+     [(null? iforms) (reverse args)]
+     [($const? (car iforms)) 
+      (loop (cdr iforms) (cons ($const-value (car iforms)) args))]
+     [(has-tag? (car iforms) $GREF)
+      (and-let1 gloc (gref-inlinable-gloc (car iforms))
+        (loop (cdr iforms) (cons (gloc-ref gloc) args)))]
+     [else #f])))
+
 ;; An ad-hoc table of builtin predicates that we can deduce its value
 ;; from what we know about its argument at compile-time.  Even the argument
 ;; is not a constant, we sometimes know its type and thus we know how
@@ -355,9 +428,9 @@
 ;; we let the compiler fail, it will be confusing since even a call in
 ;; a dead code can be the cause.  So if we get an error, we give up this
 ;; optimization and let the runtime fail.
-(define (pass3/precompute-constant proc arg-nodes)
+(define (pass3/precompute-constant proc args)
   (guard (e [else #f])                  ; give up optimization
-    (receive r (apply proc (imap (^[a] ($const-value a)) arg-nodes))
+    (receive r (apply proc args)
       (match r
         [()  ($const-undef)]
         [(r) ($const r)]
