@@ -1,7 +1,7 @@
 ;;;
 ;;; compile.scm - The compiler
 ;;;
-;;;   Copyright (c) 2004-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -72,7 +72,7 @@
 ;;;     - Global constant variables are substituted to its value.
 ;;;     - Variable bindings are resolved.  Local variables are marked
 ;;;       according to its usage (# of reference count and set count).
-;;;     - Constant expressons are folded.
+;;;     - Constant expressions are folded.
 ;;;
 ;;;   Pass 2 (Variable and closure optimization):
 ;;;     - Traverses IForm and modify the tree to optimize it.
@@ -153,7 +153,6 @@
   $RECEIVE
   $LAMBDA
   $LABEL
-  $PROMISE
   $SEQ
   $CALL
   $ASM
@@ -175,13 +174,18 @@
 (eval-when (:compile-toplevel)
   (use gauche.vm.insn)
   (define-macro (define-insn-constants)
-    (let1 name&codes
-        (map (^[insn] (cons (car insn) (ref (cdr insn)'code)))
-             (class-slot-ref <vm-insn-info> 'all-insns))
+    (let ([name&codes
+           (map (^[insn] (cons (car insn) (ref (cdr insn)'code)))
+                (class-slot-ref <vm-insn-info> 'all-insns))]
+          [mv-insn-codes
+           (filter-map (^[insn] (and (~ (cdr insn)'multi-value)
+                                     (~ (cdr insn)'code)))
+                       (class-slot-ref <vm-insn-info> 'all-insns))])
       `(begin
          ,@(map (^[n&c] `(define-constant ,(car n&c) ,(cdr n&c)))
                 name&codes)
          (define-constant .insn-alist. ',name&codes)
+         (define-constant .insn-multi-value-codes. ',mv-insn-codes)
          )))
   (define-insn-constants)
   )
@@ -189,9 +193,9 @@
 ;; Maximum size of $LAMBDA node we allow to duplicate and inline.
 (define-constant SMALL_LAMBDA_SIZE 12)
 
-(define-inline (variable? arg) (or (symbol? arg) (identifier? arg)))
+;; In case keyword is disjoint from symbols
 (define-inline (variable-or-keyword? arg)
-  (or (symbol? arg) (keyword? arg) (identifier? arg)))
+  (or (identifier? arg) (keyword? arg)))
 
 ;;============================================================
 ;; Data structures
@@ -201,14 +205,14 @@
 ;;
 ;;   Slots:
 ;;     name  - name of the variable (symbol)
-;;     initval   - initialized value
-;;     ref-count - in how many places this variable is referefnced?
+;;     initval   - initialized value (Maybe IForm)
+;;     ref-count - in how many places this variable is referenced?
 ;;     set-count - in how many places this variable is set!
 ;;
 
 (define-simple-struct lvar 'lvar make-lvar
   (name
-   (initval (undefined))
+   (initval #f)
    (ref-count 0)
    (set-count 0)))
 
@@ -248,7 +252,7 @@
        (let* ([v (Scm_MakeVector LVAR_SIZE '0)])
          (set! (SCM_VECTOR_ELEMENT v LVAR_OFFSET_TAG) 'lvar
                (SCM_VECTOR_ELEMENT v LVAR_OFFSET_NAME) name
-               (SCM_VECTOR_ELEMENT v LVAR_OFFSET_INITVAL) SCM_UNDEFINED)
+               (SCM_VECTOR_ELEMENT v LVAR_OFFSET_INITVAL) SCM_FALSE)
          (SCM_APPEND1 h t v)))
      (return h)))
 
@@ -281,7 +285,7 @@
 ;;                local binding).   This slot may be #f.
 ;;
 ;;     current-proc - Holds the information of the current
-;;                compilig procedure.  It accumulates information needed
+;;                compiling procedure.  It accumulates information needed
 ;;                in later stages for the optimization.  This slot may
 ;;                be #f.
 ;;
@@ -293,7 +297,7 @@
 (define (make-cenv module :optional (frames '()) (exp-name #f))
   (%make-cenv module frames exp-name))
 
-;; Some cenv-related proceduers are in C for better performance.
+;; Some cenv-related procedures are in C for better performance.
 (inline-stub
  ;; env-lookup-int :: Name, LookupAs, Module, [Frame] -> Var
  ;;         where Var = Lvar | Identifier | Macro
@@ -423,7 +427,6 @@
 ;;    #($lambda <src> <name> <reqarg> <optarg> (<lvar> ...) <iform> <flag>)
 ;;                           ;; closure
 ;;    #($label <src> <label> <iform>) ;; merge point of local call.  see below.
-;;    #($promise <src> <expr>) ;; promise
 ;;    #($seq (<iform> ...))   ;; sequencing
 ;;    #($call <src> <proc-expr> (<arg-expr> ...) <flag>) ;; procedure call
 ;;    #($asm <src> <insn> (<arg> ...)) ;; inline assembler
@@ -590,7 +593,7 @@
    ))
 
 ;; $seq <body>
-;;    Sequensing.  <body> is a list of IForms.
+;;    Sequencing.  <body> is a list of IForms.
 ;;    The compile tries to avoid creating $seq node if <body> has only
 ;;    one expression, but optimization paths may introduce such a $seq node.
 ;;    There can also be an empty $seq node, ($seq '()).
@@ -627,15 +630,6 @@
    args      ; list of IForms
    ))
 
-;; $promise <src> <expr>
-;;    OBSOLETED.  We keep $promise during 0.9.x releases since it
-;;    can appear in the packed IForm of the inlined procedures.
-;;    Will go in 1.0.
-(define-simple-struct $promise $PROMISE $promise
-  (src       ; original source for debugging
-   expr      ; IForm
-   ))
-
 ;; $it
 ;;   A special node.  See the explanation of $if above.
 (define $it (let ((c `#(,$IT))) (^[] c)))
@@ -695,6 +689,9 @@
           [(eqv? (cdar p) code) (caar p)]
           [else (loop (cdr p))])))
 
+(define (insn-multi-value? code)
+  (memv code .insn-multi-value-codes.))
+
 ;; prettyprinter of intermediate form
 (define (pp-iform iform :optional (lines +inf.0))
   (let/cc return
@@ -786,9 +783,6 @@
             [else (for-each (^n (nl (+ ind 2)) (rec (+ ind 2) n))
                             ($asm-args iform))])
           (display ")"))]
-       [($PROMISE) (display "($promise ")
-        (rec (+ ind 10) ($promise-expr iform))
-        (display ")")]
        [($IT)      (display "($it)")]
        [($CONS $APPEND $MEMV $EQ? $EQV?)
         (let* ([s (format "(~a " (iform-tag-name (iform-tag iform)))]
@@ -817,7 +811,10 @@
 ;;    every time it is used.
 ;;
 ;; Packed IForm is a vector, with the references are represented by indices.
-
+;;
+;; We also replace numeric tags of IForm for symbols.  The numeric value
+;; may change across Gauche versions, while the packed IForms are recorded
+;; in the precompiled source and survive across versions.
 (define (pack-iform iform)
 
   (define dict (make-hash-table 'eq?))
@@ -829,67 +826,74 @@
       (hash-table-put! dict iform head)
       (dolist [obj objs] (push! r obj) (inc! c))))
 
+  (define (put!-2ary iform tag-symbol) ; $CONS $APPEND $MEMV $EQ? $EQV?
+    (put! iform tag-symbol
+          ($*-src iform)
+          (get-ref ($*-arg0 iform))
+          (get-ref ($*-arg1 iform))))
+
+  (define (put!-nary iform tag-symbol) ; $VECTOR $LIST $LIST*
+    (put! iform tag-symbol
+          ($*-src iform)
+          (map get-ref ($*-args iform))))
+
   (define (get-ref iform)
     (or (hash-table-get dict iform #f) (pack-iform-rec iform)))
 
   (define (pack-iform-rec iform)
     (case/unquote
      (iform-tag iform)
-     [($DEFINE) (put! iform $DEFINE ($*-src iform)
+     [($DEFINE) (put! iform '$DEFINE ($*-src iform)
                       ($define-flags iform) ($define-id iform)
                       (get-ref ($define-expr iform)))]
-     [($LREF) (put! iform $LREF (get-ref ($lref-lvar iform)))]
-     [($LSET) (put! iform $LSET
+     [($LREF) (put! iform '$LREF (get-ref ($lref-lvar iform)))]
+     [($LSET) (put! iform '$LSET
                     (get-ref ($lset-lvar iform)) (get-ref ($lset-expr iform)))]
-     [($GREF) (put! iform $GREF ($gref-id iform))]
-     [($GSET) (put! iform $GSET ($gset-id iform) (get-ref ($gset-expr iform)))]
-     [($CONST)(put! iform $CONST ($const-value iform))]
-     [($IF)   (put! iform $IF ($*-src iform)
+     [($GREF) (put! iform '$GREF ($gref-id iform))]
+     [($GSET) (put! iform '$GSET ($gset-id iform) (get-ref ($gset-expr iform)))]
+     [($CONST)(put! iform '$CONST ($const-value iform))]
+     [($IF)   (put! iform '$IF ($*-src iform)
                     (get-ref ($if-test iform))
                     (get-ref ($if-then iform))
                     (get-ref ($if-else iform)))]
-     [($LET)  (put! iform (iform-tag iform) ($*-src iform) ($let-type iform)
+     [($LET)  (put! iform '$LET ($*-src iform) ($let-type iform)
                     (map get-ref ($let-lvars iform))
                     (map get-ref ($let-inits iform))
                     (get-ref ($let-body iform)))]
-     [($RECEIVE) (put! iform $RECEIVE ($*-src iform)
+     [($RECEIVE) (put! iform '$RECEIVE ($*-src iform)
                        ($receive-reqargs iform) ($receive-optarg iform)
                        (map get-ref ($receive-lvars iform))
                        (get-ref ($receive-expr iform))
                        (get-ref ($receive-body iform)))]
-     [($LAMBDA) (put! iform $LAMBDA ($*-src iform)
+     [($LAMBDA) (put! iform '$LAMBDA ($*-src iform)
                       ($lambda-name iform) ($lambda-reqargs iform)
                       ($lambda-optarg iform)
                       (map get-ref ($lambda-lvars iform))
                       (get-ref ($lambda-body iform))
                       ($lambda-flag iform))]
-     [($LABEL)  (put! iform $LABEL ($*-src iform) #f
+     [($LABEL)  (put! iform '$LABEL ($*-src iform) #f
                       (get-ref ($label-body iform)))]
-     [($SEQ)    (put! iform $SEQ (map get-ref ($seq-body iform)))]
-     [($CALL)   (put! iform $CALL ($*-src iform)
+     [($SEQ)    (put! iform '$SEQ (map get-ref ($seq-body iform)))]
+     [($CALL)   (put! iform '$CALL ($*-src iform)
                       (get-ref ($call-proc iform))
                       (map get-ref ($call-args iform))
                       ($call-flag iform))]
-     [($ASM)    (put! iform $ASM ($*-src iform)
+     [($ASM)    (put! iform '$ASM ($*-src iform)
                       ($asm-insn iform)
                       (map get-ref ($asm-args iform)))]
-     [($IT)     (put! iform $IT)]
-     [($PROMISE)(put! iform $PROMISE ($*-src iform)
-                      (get-ref ($promise-expr iform)))]
-     [($CONS $APPEND $MEMV $EQ? $EQV?)
-      (put! iform (iform-tag iform) ($*-src iform)
-            (get-ref ($*-arg0 iform))
-            (get-ref ($*-arg1 iform)))]
-     [($VECTOR $LIST $LIST*)
-      (put! iform (iform-tag iform) ($*-src iform)
-            (map get-ref ($*-args iform)))]
-     [($LIST->VECTOR)
-      (put! iform (iform-tag iform) ($*-src iform)
-            (get-ref ($*-arg0 iform)))]
-     [('lvar)
-      (put! iform 'lvar (lvar-name iform))]
-     [else
-      (errorf "[internal-error] unknown IForm in pack-iform: ~S" iform)]
+     [($IT)     (put! iform '$IT)]
+     [($CONS)   (put!-2ary iform '$CONS)]
+     [($APPEND) (put!-2ary iform '$APPEND)]
+     [($MEMV)   (put!-2ary iform '$MEMV)]
+     [($EQ?)    (put!-2ary iform '$EQ?)]
+     [($EQV?)   (put!-2ary iform '$EQV?)]
+     [($VECTOR) (put!-nary iform '$VECTOR)]
+     [($LIST)   (put!-nary iform '$LIST)]
+     [($LIST*)  (put!-nary iform '$LIST*)]
+     [($LIST->VECTOR) (put! iform '$LIST->VECTOR ($*-src iform)
+                            (get-ref ($*-arg0 iform)))]
+     [('lvar)   (put! iform 'lvar (lvar-name iform))]
+     [else (errorf "[internal-error] unknown IForm in pack-iform: ~S" iform)]
      ))
 
   ;; main body of pack-iform
@@ -902,6 +906,7 @@
     (vector-set! vec 0 start)
     vec))
 
+;; Recover IForm from a vector.
 (define (unpack-iform ivec)
   (let-syntax ([V (syntax-rules ()
                     [(V ix) (vector-ref ivec ix)]
@@ -912,42 +917,50 @@
             [else (rlet1 body (unpack-body ref)
                     (hash-table-put! dict ref body))]))
     (define (unpack-body i)
-      (case/unquote
-       (V i)
+      ;; NB: We dispatch by symbol, not the numeric tag value.
+      ;; TRANSIENT: Due to a bug, 0.9.7 precompiled IForm contains numeric
+      ;; tag 7 in place of $LET.  To keep binary compatibility, we have to
+      ;; support it until the next ABI bump.
+      (case (V i)
        [($DEFINE) ($define (V i 1) (V i 2) (V i 3) (unpack-rec (V i 4)))]
        [($LREF)   ($lref (unpack-rec (V i 1)))]
        [($LSET)   ($lset (unpack-rec (V i 1)) (unpack-rec (V i 2)))]
        [($GREF)   ($gref (V i 1))]
        [($GSET)   ($gset (V i 1) (unpack-rec (V i 2)))]
        [($CONST)  ($const (V i 1))]
-       [($IF)     ($if (V i 1) (unpack-rec (V i 2))
-                       (unpack-rec (V i 3)) (unpack-rec (V i 4)))]
-       [($LET)    (rlet1 unpacked
-                      ($let (V i 1) (V i 2)
-                            (map unpack-rec (V i 3)) (map unpack-rec (V i 4))
-                            (unpack-rec (V i 5)))
-                    (ifor-each2 (^(lv in) (lvar-initval-set! lv in))
-                                ($let-lvars unpacked) ($let-inits unpacked)))]
+       [($IF)  ($if (V i 1) (unpack-rec (V i 2))
+                    (unpack-rec (V i 3)) (unpack-rec (V i 4)))]
+       [($LET 7) (rlet1 unpacked
+                     ($let (V i 1) (V i 2)
+                           (map unpack-rec (V i 3)) (map unpack-rec (V i 4))
+                           (unpack-rec (V i 5)))
+                   (ifor-each2 (^(lv in) (lvar-initval-set! lv in))
+                               ($let-lvars unpacked) ($let-inits unpacked)))]
        [($RECEIVE) ($receive (V i 1) (V i 2) (V i 3)
                              (map unpack-rec (V i 4)) (unpack-rec (V i 5))
                              (unpack-rec (V i 6)))]
-       [($LAMBDA)  ($lambda (V i 1) (V i 2) (V i 3) (V i 4)
-                            (map unpack-rec (V i 5))
-                            (unpack-rec (V i 6)) (V i 7))]
-       [($LABEL)   ($label (V i 1) (V i 2) (unpack-rec (V i 3)))]
-       [($SEQ)     ($seq (map unpack-rec (V i 1)))]
-       [($CALL)    ($call (V i 1) (unpack-rec (V i 2))
-                          (map unpack-rec (V i 3)) (V i 4))]
-       [($ASM)     ($asm (V i 1) (V i 2) (map unpack-rec (V i 3)))]
-       [($PROMISE) ($promise (V i 1) (unpack-rec (V i 2)))]
-       [($IT) ($it)]
-       [($CONS $APPEND $MEMV $EQ? $EQV?)
-        (vector (V i) (V i 1) (unpack-rec (V i 2)) (unpack-rec (V i 3)))]
-       [($VECTOR $LIST $LIST*) (vector (V i) (V i 1) (map unpack-rec (V i 2)))]
-       [($LIST->VECTOR) (vector (V i) (V i 1) (unpack-rec (V i 2)))]
-       [('lvar)    (make-lvar (V i 1))]
-       [else (errorf "[internal error] unpack-iform: ivec broken at ~a: ~S"
-                     i ivec)]))
+       [($LAMBDA) ($lambda (V i 1) (V i 2) (V i 3) (V i 4)
+                           (map unpack-rec (V i 5))
+                           (unpack-rec (V i 6)) (V i 7))]
+       [($LABEL)  ($label (V i 1) (V i 2) (unpack-rec (V i 3)))]
+       [($SEQ)    ($seq (map unpack-rec (V i 1)))]
+       [($CALL)   ($call (V i 1) (unpack-rec (V i 2))
+                         (map unpack-rec (V i 3)) (V i 4))]
+       [($ASM)    ($asm (V i 1) (V i 2) (map unpack-rec (V i 3)))]
+       [($IT)     ($it)]
+       [($CONS)   ($cons (V i 1) (unpack-rec (V i 2)) (unpack-rec (V i 3)))]
+       [($APPEND) ($append (V i 1) (unpack-rec (V i 2)) (unpack-rec (V i 3)))]
+       [($MEMV)   ($memv (V i 1) (unpack-rec (V i 2)) (unpack-rec (V i 3)))]
+       [($EQ?)    ($eq? (V i 1) (unpack-rec (V i 2)) (unpack-rec (V i 3)))]
+       [($EQV?)   ($eqv? (V i 1) (unpack-rec (V i 2)) (unpack-rec (V i 3)))]
+       [($VECTOR) ($vector (V i 1) (map unpack-rec (V i 2)))]
+       [($LIST)   ($list (V i 1) (map unpack-rec (V i 2)))]
+       [($LIST*)  ($list* (V i 1) (map unpack-rec (V i 2)))]
+       [($LIST->VECTOR) ($list->vector (V i 1) (unpack-rec (V i 2)))]
+       [(lvar)    (make-lvar (V i 1))]
+       [else 
+        (errorf "[internal error] unpack-iform: ivec broken at ~a (tag=~s): ~S"
+                i (V i) ivec)]))
 
     (unpack-rec (V 0))))
 
@@ -979,7 +992,6 @@
        [($SEQ)    (sum-items cnt (* ($seq-body iform)))]
        [($CALL) (sum-items (+ cnt 1) ($call-proc iform) (* ($call-args iform)))]
        [($ASM)    (sum-items (+ cnt 1) (* ($asm-args iform)))]
-       [($PROMISE)(sum-items (+ cnt 1) ($promise-expr iform))]
        [($CONS $APPEND $MEMV $EQ? $EQV?)
         (sum-items (+ cnt 1) ($*-arg0 iform) ($*-arg1 iform))]
        [($VECTOR $LIST $LIST*) (sum-items (+ cnt 1) (* ($*-args iform)))]
@@ -1067,8 +1079,6 @@
                                            ($lambda-calls target)))))]
    [($ASM) ($asm ($*-src iform) ($asm-insn iform)
                  (imap (cut iform-copy <> lv-alist) ($asm-args iform)))]
-   [($PROMISE)($promise ($*-src iform)
-                        (iform-copy ($promise-expr iform) lv-alist))]
    [($CONS)   ($cons ($*-src iform)
                      (iform-copy ($*-arg0 iform) lv-alist)
                      (iform-copy ($*-arg1 iform) lv-alist))]
@@ -1155,7 +1165,6 @@
                         (cons (ref (cdr target-insn-info)'code)
                               (cdr host-insn))
                         (imap rec ($asm-args iform))))]
-     [($PROMISE) ($promise ($*-src iform) (rec ($promise-expr iform)))]
      [($CONS)    ($cons ($*-src iform)
                         (rec ($*-arg0 iform))
                         (rec ($*-arg1 iform)))]
@@ -1185,44 +1194,48 @@
 ;; won't change even if we move iform to a different place in the subtree.
 ;; NB: This may be called after pass2, so $LABEL node may have circular
 ;; reference.  We have to be careful not to diverge.
-;; TODO: we lift transparent?/rec manually to avoid closure allocation
-;; because of the unsophisticated compiler.  Fix this in future.
-(define (transparent? iform) (transparent?/rec iform (make-label-dic #f)))
-(define (transparent?/rec iform labels)
-  (case/unquote
-   (iform-tag iform)
-   [($LREF)   (lvar-immutable? ($lref-lvar iform))]
-   [($GREF)   (gref-inlinable-gloc iform)]
-   [($CONST)  #t]
-   [($IF)     (and (transparent?/rec ($if-test iform) labels)
-                   (transparent?/rec ($if-then iform) labels)
-                   (transparent?/rec ($if-else iform) labels))]
-   [($LET)    (and (everyc transparent?/rec ($let-inits iform) labels)
-                   (transparent?/rec ($let-body iform) labels))]
-   [($RECEIVE)(and (transparent?/rec ($receive-expr iform) labels)
-                   (transparent?/rec ($receive-body iform) labels))]
-   [($LAMBDA) #t]
-   [($LABEL)  (or (label-seen? labels iform)
-                  (begin (label-push! labels iform)
-                         (transparent?/rec ($label-body iform) labels)))]
-   [($SEQ)    (everyc transparent?/rec ($seq-body iform) labels)]
-   [($CALL)   (and (side-effect-free-proc? ($call-proc iform))
-                   (everyc transparent?/rec ($call-args iform) labels))]
-   [($ASM)    (and (side-effect-free-insn? ($asm-insn iform))
-                   (everyc transparent?/rec ($asm-args iform) labels))]
-   [($PROMISE) #t]
-   [($CONS $APPEND $MEMV $EQ? $EQV?)
-    (and (transparent?/rec ($*-arg0 iform) labels)
-         (transparent?/rec ($*-arg1 iform) labels))]
-   [($VECTOR $LIST $LIST*) (everyc transparent?/rec ($*-args iform) labels)]
-   [($LIST->VECTOR) (transparent?/rec ($*-arg0 iform) labels)]
-   [($IT) #t] ; this branch is only executed when $if-test of the parent is
-              ; transparent, thus this node is also transparent.
-   [else #f]))
+(define (transparent? iform) 
+  (define (side-effect-free-proc? iform) #f) ;for now
 
-(define (side-effect-free-proc? iform) #f) ;for now
+  ;; This is used when eliminating dead $ASM node.  We don't need to consider
+  ;; combined insns, which won't be introduced until pass5.
+  (define (side-effect-free-insn? insn)
+    (memq (car insn) `(,EQ ,EQV ,CONS ,LIST ,LIST-STAR
+                       ,NULLP ,PAIRP ,CHARP ,EOFP ,STRINGP ,SYMBOLP
+                       ,VECTORP ,IDENTIFIERP ,NUMBERP ,REALP
+                       ,VEC)))
 
-(define (side-effect-free-insn? insn)  #f) ;for now
+  (define (rec iform labels)
+    (case/unquote
+     (iform-tag iform)
+     [($LREF)   (lvar-immutable? ($lref-lvar iform))]
+     [($GREF)   (gref-inlinable-gloc iform)]
+     [($CONST)  #t]
+     [($IF)     (and (rec ($if-test iform) labels)
+                     (rec ($if-then iform) labels)
+                     (rec ($if-else iform) labels))]
+     [($LET)    (and (everyc rec ($let-inits iform) labels)
+                     (rec ($let-body iform) labels))]
+     [($RECEIVE)(and (rec ($receive-expr iform) labels)
+                     (rec ($receive-body iform) labels))]
+     [($LAMBDA) #t]
+     [($LABEL)  (or (label-seen? labels iform)
+                    (begin (label-push! labels iform)
+                           (rec ($label-body iform) labels)))]
+     [($SEQ)    (everyc rec ($seq-body iform) labels)]
+     [($CALL)   (and (side-effect-free-proc? ($call-proc iform))
+                     (everyc rec ($call-args iform) labels))]
+     [($ASM)    (and (side-effect-free-insn? ($asm-insn iform))
+                     (everyc rec ($asm-args iform) labels))]
+     [($CONS $APPEND $MEMV $EQ? $EQV?)
+      (and (rec ($*-arg0 iform) labels)
+           (rec ($*-arg1 iform) labels))]
+     [($VECTOR $LIST $LIST*) (everyc rec ($*-args iform) labels)]
+     [($LIST->VECTOR) (rec ($*-arg0 iform) labels)]
+     [($IT) #t] ; this branch is only executed when $if-test of the parent is
+                                        ; transparent, thus this node is also transparent.
+     [else #f]))
+  (rec iform (make-label-dic #f)))
 
 ;; Reset lvar reference count.  This is called in pass3,
 ;; when a subgraph of IForm is eliminated.
@@ -1257,7 +1270,6 @@
                (reset-lvars/rec ($call-proc iform) labels))
              (reset-lvars/rec* ($call-args iform) labels)]
   [($ASM)    (reset-lvars/rec* ($asm-args iform) labels)]
-  [($PROMISE)(reset-lvars/rec ($promise-expr iform) labels)]
   [($CONS $APPEND $MEMV $EQ? $EQV?)
              (reset-lvars/rec ($*-arg0 iform) labels)
              (reset-lvars/rec ($*-arg1 iform) labels)]
@@ -1277,7 +1289,7 @@
 ;; h is defined.   A straight transplantation of IForm wouldn't work, since
 ;; the $LAMBDA node for the closure bound to 'h' refers to the outer
 ;; environment.
-;; What we do is to eliminate the reference to the outer enviornment by
+;; What we do is to eliminate the reference to the outer environment by
 ;; beta-substitution at the time of definition of 'h', as follows:
 ;;
 ;;  (define-inline h (f (g) 4))
@@ -1355,8 +1367,6 @@
                        ($call-flag iform))]
      [($ASM)    ($asm ($*-src iform) ($asm-insn iform)
                       (imap (cut subst <> mapping dict) ($asm-args iform)))]
-     [($PROMISE)($promise ($*-src iform)
-                          (subst ($promise-expr iform) mapping dict))]
      [($CONS $APPEND $MEMV $EQ? $EQV?) (subst/2 iform mapping dict)]
      [($VECTOR $LIST $LIST*) (subst/* iform mapping dict)]
      [($LIST->VECTOR) ($list->vector ($*-src iform)
@@ -1371,7 +1381,6 @@
             (imap (cut subst <> mapping dict) ($*-args iform))))
   (subst iform mapping (make-hash-table 'eq?)))
 
-;; EXPERIMENTAL
 ;; Convert IForm back to S-expr.
 ;; This is used by macroexpand-all.  Note that pass2 can make IForm DG, and
 ;; it's not generally possible to convert it back to S-expr.  You can only
@@ -1381,7 +1390,7 @@
   (define (get-lvar lvar)
     (or (hash-table-get lvar-dict lvar #f)
         (let* ([name (lvar-name lvar)]
-               [name (if (identifier? name) (identifier->symbol name) name)])
+               [name (identifier->symbol name)])
           (rlet1 s ($ string->symbol $ format "~a.~d" name
                       $ hash-table-num-entries lvar-dict)
             (hash-table-put! lvar-dict lvar s)))))
@@ -1394,7 +1403,12 @@
                        ,(rec ($lset-expr iform)))]
      [($GREF)   ($gref-id iform)]
      [($GSET)   `(set! ,($gset-id iform) ,(rec ($gset-expr iform)))]
-     [($CONST)  `',($const-value iform)]
+     [($CONST)  (let1 v ($const-value iform)
+                  ;; This is just for better readability
+                  (if (or (number? v) (string? v) (char? v) (boolean? v)
+                          (undefined? v) (port? v) (procedure? v))
+                    v
+                    `',v))]
      [($IF)     (cond [(has-tag? ($if-then iform) $IT)
                        `(or ,(rec ($if-test iform))
                             ,(rec ($if-else iform)))]
@@ -1524,7 +1538,8 @@
          ;; If at least either one is unbound, we just compare their names.
          (if (and g1 g2)
            (return (SCM_EQ g1 g2))
-           (return (SCM_EQ (Scm_UnwrapSyntax id1) (Scm_UnwrapSyntax id2)))))
+           (return (SCM_EQ (Scm_UnwrapSyntax2 id1 FALSE)
+                           (Scm_UnwrapSyntax2 id2 FALSE)))))
        ;; At least one of id1 and id2 is lvar or macro.
        (return (SCM_EQ b1 b2)))))
 
@@ -1541,7 +1556,7 @@
 ;; identifier for those occurrences.
 (define (er-rename form dict module env)
   (cond
-   [(variable? form)
+   [(identifier? form)
     (if-let1 id (assq-ref dict form)
       (values id dict)
       (let1 id (make-identifier form module env)
@@ -1586,56 +1601,49 @@
    Scm__ERCompare)
  )
 
-;; xformer :: (Sexpr, (Sym -> Sym), (Sym, Sym -> Bool)) -> Sexpr
-(define (%make-er-transformer xformer def-env)
+;; xformer  :: (Sexpr, (Sym -> Sym), (Sym, Sym -> Bool)) -> Sexpr
+;; xformer+ :: (Sexpr, (Sym -> Sym), (Sym, Sym -> Bool), (Sym -> Sym)) -> Sexpr
+(define (%make-er-transformer xformer def-env :optional (has-inject? #f))
   (define def-module (cenv-module def-env))
   (define def-frames (cenv-frames def-env))
   (define (expand form use-env)
     (define use-module (cenv-module use-env))
     (define use-frames (cenv-frames use-env))
     (let1 dict '()
-      (xformer form
-               (^[sym]
-                 (receive [id dict_] (er-rename sym dict def-module def-frames)
-                   (set! dict dict_)
-                   id))
-               (^[a b] (er-compare a b use-module use-frames)))))
+      (define (%rename sym)
+        (receive [id dict_] (er-rename sym dict def-module def-frames)
+          (set! dict dict_)
+          id))
+      (define (%compare a b) (er-compare a b use-module use-frames))
+      (define (%inject sym)
+        (receive [id dict_] (er-rename sym dict use-module use-frames)
+          (set! dict dict_)
+          id))
+      (if has-inject?
+        (xformer form %rename %compare %inject)
+        (xformer form %rename %compare))))
   (%make-macro-transformer (cenv-exp-name def-env) expand))
 
 ;; Call to this procedure can be inserted in the output of er-macro expander
 ;; and emitted in the precompiled file, so do not change this API unless
 ;; you're sure no existing precompiled code refers to this.
-(define (%make-er-transformer/toplevel xformer def-module def-name)
-  (%make-er-transformer xformer (%make-cenv def-module '() def-name)))
+(define (%make-er-transformer/toplevel xformer def-module def-name 
+                                       :optional (has-inject? #f))
+  (%make-er-transformer xformer 
+                        (%make-cenv def-module '() def-name)
+                        has-inject?))
 
-;; TRANSIENT: We no longer use these procedures, but reference to them
-;; might be inserted in the file precompiled by 0.9.5 compiler as the
-;; result of expansion of er macro.
-;; Remove this on 1.0 release.
-(define (%make-toplevel-cenv name) ;internal
-  (%make-cenv (vm-current-module) '() name))
-(define (%make-primitive-transformer xformer def-env)
-  (%make-macro-transformer (cenv-exp-name def-env)
-                           (^[form use-env] (xformer form def-env use-env))))
-(define (%expand-er-transformer form cenv)
-  (match form
-    [(_ xformer)
-     ;; See the discussion in primitive-macro-transformer above.
-     (let1 def-env-form (if (null? (cenv-frames cenv))
-                          `(,%make-toplevel-cenv. ',(cenv-exp-name cenv))
-                          cenv)
-       `(,%make-er-transformer. ,xformer ,def-env-form))]
-    [_ (error "Invalid er-macro-transformer form:" form)]))
-
-
-;; EXPERIMENTAL
 ;; Returns an S-expr all macros in which are expanded.
 ;; The resulting form may not be equivalent to the input form, though,
 ;; since we strip off identifier information so toplevel hygiene isn't
 ;; kept.  (Local variables are renamed so they won't conflict with each
 ;; other.)
-(define-in-module gauche (macroexpand-all form)
-  (let1 flags-save (vm-compiler-flag)
+(define-in-module gauche (macroexpand-all form :optional (env #f))
+  (let ([flags-save (vm-compiler-flag)]
+        [env (cond
+              [(boolean? env) (vm-current-module)]
+              [(module? env) env]
+              [else (error "a boolean or a module required but got:" env)])])
     (unwind-protect
         (begin
           ;; TODO: We suppress global inline expansion, otherwise we'll see
@@ -1644,7 +1652,7 @@
           ;; We'll think of it later.
           (vm-compiler-flag-set! SCM_COMPILE_NOINLINE_GLOBALS)
           ($ unwrap-syntax $ iform->sexpr
-             $ pass1 form (make-bottom-cenv (vm-current-module))))
+             $ pass1 form (make-bottom-cenv env)))
       (begin
         (vm-compiler-flag-clear! SCM_COMPILE_NOINLINE_GLOBALS)
         (vm-compiler-flag-set! flags-save)))))
@@ -1653,10 +1661,10 @@
 ;; Customizable inliner interface
 ;;
 
-;; This is invoked from the expansion of define-inline/syntax.
+;; This is invoked from the expansion of define-hybrid-syntax.
 (define (%with-inline-transformer proc xformer)
   (unless (procedure? proc)
-    (error "define-inline/syntax: expression yields non-procedure value" proc))
+    (error "define-hybrid-syntax: expression yields non-procedure value" proc))
   (unless (macro? xformer)
     (errorf "macro required, but got" xformer))
   ;; Copy PROC, in case the given proc refers to already defined global
@@ -1692,8 +1700,7 @@
        (<= 0 obj #x7ffff)))
 
 (define (variable-name arg)
-  (cond [(symbol? arg) arg]
-        [(identifier? arg) (unwrap-syntax arg)]
+  (cond [(identifier? arg) (identifier->symbol arg)]
         [(lvar? arg) (lvar-name arg)]
         [else (error "variable required, but got:" arg)]))
 
@@ -1746,6 +1753,8 @@
    (let* ([mod::ScmModule* (-> (SCM_IDENTIFIER id) module)]
           [gloc::ScmGloc* (Scm_IdentifierGlobalBinding (SCM_IDENTIFIER id))])
      (set! SCM_RESULT0 '#f SCM_RESULT1 '#f)
+     (cast void cenv) ; suppress unused var warning
+     (cast void mod)  ; suppress unused var warning
      (when gloc
        (let* ([gval (SCM_GLOC_GET gloc)])
          (cond [(SCM_MACROP gval)
@@ -1781,12 +1790,12 @@
 ;; TODO: We should also check iff k isn't bound to something else!
 (define (keyword-like? k)
   (or (keyword? k)
-      (and (identifier? k) (keyword? (identifier->symbol k)))))
+      (and (wrapped-identifier? k) (keyword? (identifier->symbol k)))))
 
 (define (global-eq? var id cenv)  ; like free-identifier=?, used in pass1.
-  (and (variable? var)
+  (and (identifier? var)
        (let1 v (cenv-lookup-variable cenv var)
-         (and (identifier? v)
+         (and (wrapped-identifier? v)
               (global-identifier=? v id)))))
 
 ;; TRANSIENT: To compare keyword hygienically.  We need to treat keywords
@@ -1794,7 +1803,7 @@
 (define (global-keyword-eq? var key cenv)
   (or (eq? var key)
       (and (symbol? key)
-           (identifier? var)
+           (wrapped-identifier? var)
            (global-eq? var (global-id key) cenv))))
 
 (define (everyc proc lis c)             ;avoid closure allocation
@@ -1819,7 +1828,7 @@
 (define (global-eq?? sym srcmod modgen)
   (let1 id-gloc (find-binding (find-module srcmod) sym #f)
     (^[var] (eq? id-gloc
-                 (if (identifier? var)
+                 (if (wrapped-identifier? var)
                    (find-binding (slot-ref var'module) (slot-ref var'name) #f)
                    (find-binding (modgen) var #f))))))
 
@@ -1921,7 +1930,7 @@
  (define-cproc vm-compiler-flag-clear! (flag::<uint>) ::<void>
    (SCM_VM_COMPILER_FLAG_CLEAR (Scm_VM) flag))
  (define-cproc vm-compiler-flag () ::<uint>
-   (result (-> (Scm_VM) compilerFlags)))
+   (return (-> (Scm_VM) compilerFlags)))
 
  (define-cproc vm-compiler-flag-noinline-locals? () ::<boolean>
    (return (SCM_VM_COMPILER_FLAG_IS_SET (Scm_VM) SCM_COMPILE_NOINLINE_LOCALS)))
@@ -1940,7 +1949,11 @@
  (define-enum SCM_COMPILE_NO_POST_INLINE_OPT)
  (define-enum SCM_COMPILE_NO_LIFTING)
  (define-enum SCM_COMPILE_INCLUDE_VERBOSE)
- (define-enum SCM_COMPILE_ENABLE_CEXPR)
+ (define-enum SCM_COMPILE_NOINLINE_SETTERS)
+ (define-enum SCM_COMPILE_NODISSOLVE_APPLY)
+ (define-enum SCM_COMPILE_LEGACY_DEFINE)
+ (define-enum SCM_COMPILE_MUTABLE_LITERALS)
+ (define-enum SCM_COMPILE_SRFI_FEATURE_ID)
 
  ;; Set/get VM's current module info. (temporary)
  (define-cproc vm-current-module () (return (SCM_OBJ (-> (Scm_VM) module))))
