@@ -1,7 +1,7 @@
 /*
  * core.c - core kernel interface
  *
- *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -33,12 +33,11 @@
 
 #define LIBGAUCHE_BODY
 #include "gauche.h"
-#include "gauche/paths.h"
 #include "gauche/priv/builtin-syms.h"
 
 /* GC_print_static_roots() is declared in private/gc_priv.h.  It is too much
    hassle to include it with other GC internal baggages, so we just declare
-   it.  It's a private funciton of GC, so watch out the changes in GC. */
+   it.  It's a private function of GC, so watch out the changes in GC. */
 extern void GC_print_static_roots(void);
 
 /*
@@ -55,9 +54,10 @@ static void *oom_handler(size_t bytes)
  * Features list used by cond-expand macro
  */
 static struct {
-    ScmObj alist;
+    ScmObj alist;               /* list of (feature) or (feature module) */
+    ScmObj dlist;               /* list of disabled features */
     ScmInternalMutex mutex;
-} cond_features = { SCM_NIL };
+} cond_features = { SCM_NIL, SCM_NIL, SCM_INTERNAL_MUTEX_INITIALIZER };
 
 /*=============================================================
  * Program initialization
@@ -69,6 +69,7 @@ extern void Scm__InitSymbol(void);
 extern void Scm__InitNumber(void);
 extern void Scm__InitChar(void);
 extern void Scm__InitClass(void);
+extern void Scm__InitList(void);
 extern void Scm__InitExceptions(void);
 extern void Scm__InitPort(void);
 extern void Scm__InitWrite(void);
@@ -85,6 +86,7 @@ extern void Scm__InitVM(void);
 extern void Scm__InitAutoloads(void);
 extern void Scm__InitCollection(void);
 extern void Scm__InitComparator(void);
+extern void Scm__InitExecenv(void);
 
 extern void Scm_Init_libalpha(void);
 extern void Scm_Init_libbool(void);
@@ -98,7 +100,6 @@ extern void Scm_Init_libfmt(void);
 extern void Scm_Init_libio(void);
 extern void Scm_Init_liblazy(void);
 extern void Scm_Init_liblist(void);
-extern void Scm_Init_libmac(void);
 extern void Scm_Init_libmisc(void);
 extern void Scm_Init_libmod(void);
 extern void Scm_Init_libnum(void);
@@ -110,7 +111,10 @@ extern void Scm_Init_libstr(void);
 extern void Scm_Init_libsym(void);
 extern void Scm_Init_libsys(void);
 extern void Scm_Init_libvec(void);
+extern void Scm_Init_libmacbase(void);
 extern void Scm_Init_compile(void);
+extern void Scm_Init_libmacro(void);
+extern void Scm_Init_libparam(void);
 extern void Scm_Init_libomega(void);
 
 static void finalizable(void);
@@ -118,7 +122,9 @@ static void init_cond_features(void);
 
 #ifdef GAUCHE_USE_PTHREADS
 /* a trick to make sure the gc thread object is linked */
-static int (*ptr_pthread_create)(void) = NULL;
+static int (*ptr_pthread_create)(pthread_t *,
+                                 GC_PTHREAD_CREATE_CONST pthread_attr_t *,
+                                 void *(*)(void *), void * /* arg */) = NULL;
 #endif
 
 /* flag to see if Scheme infrastructure is fully initialized or not */
@@ -130,6 +136,16 @@ static int scheme_initialized = FALSE;
 void Scm_Init(const char *signature)
 {
     if (scheme_initialized) return;
+
+    int debug_init = FALSE;
+    if (Scm_GetEnv("GAUCHE_DEBUG_INITIALIZATION") != NULL) {
+        debug_init = TRUE;
+    }
+#define CALL_INIT(f)                                            \
+    do {                                                        \
+        if (debug_init) fprintf(stderr, "Calling %s...\n", #f); \
+        f();                                                    \
+    } while (0)
     
     /* make sure the main program links the same version of libgauche */
     if (strcmp(signature, GAUCHE_SIGNATURE) != 0) {
@@ -152,67 +168,71 @@ void Scm_Init(const char *signature)
 
     /* Initialize components.  The order is important, for some components
        rely on the other components to be initialized. */
-    Scm__InitParameter();
-    Scm__InitVM();
-    Scm__InitHash();
-    Scm__InitSymbol();
-    Scm__InitModule();
-    Scm__InitNumber();
-    Scm__InitChar();
-    Scm__InitClass();
-    Scm__InitCollection();
-    Scm__InitExceptions();
-    Scm__InitProc();
-    Scm__InitPort();
-    Scm__InitWrite();
-    Scm__InitMacro();
-    Scm__InitLoad();
-    Scm__InitRegexp();
-    Scm__InitRead();
-    Scm__InitSignal();
-    Scm__InitSystem();
-    Scm__InitComparator();
+    CALL_INIT(Scm__InitParameter);
+    CALL_INIT(Scm__InitVM);
+    CALL_INIT(Scm__InitHash);
+    CALL_INIT(Scm__InitSymbol);
+    CALL_INIT(Scm__InitModule);
+    CALL_INIT(Scm__InitNumber);
+    CALL_INIT(Scm__InitChar);
+    CALL_INIT(Scm__InitClass);
+    CALL_INIT(Scm__InitList);
+    CALL_INIT(Scm__InitCollection);
+    CALL_INIT(Scm__InitExceptions);
+    CALL_INIT(Scm__InitProc);
+    CALL_INIT(Scm__InitPort);
+    CALL_INIT(Scm__InitWrite);
+    CALL_INIT(Scm__InitMacro);
+    CALL_INIT(Scm__InitLoad);
+    CALL_INIT(Scm__InitRegexp);
+    CALL_INIT(Scm__InitRead);
+    CALL_INIT(Scm__InitSignal);
+    CALL_INIT(Scm__InitSystem);
+    CALL_INIT(Scm__InitComparator);
+    CALL_INIT(Scm__InitExecenv);
 
-    Scm_Init_libalpha();
-    Scm_Init_libbool();
-    Scm_Init_libchar();
-    Scm_Init_libcode();
-    Scm_Init_libcmp();
-    Scm_Init_libdict();
-    Scm_Init_libeval();
-    Scm_Init_libexc();
-    Scm_Init_libfmt();
-    Scm_Init_libio();
-    Scm_Init_liblazy();
-    Scm_Init_liblist();
-    Scm_Init_libmac();
-    Scm_Init_libmisc();
-    Scm_Init_libmod();
-    Scm_Init_libnum();
-    Scm_Init_libobj();
-    Scm_Init_libproc();
-    Scm_Init_librx();
-    Scm_Init_libsrfis();
-    Scm_Init_libstr();
-    Scm_Init_libsym();
-    Scm_Init_libsys();
-    Scm_Init_libvec();
-    Scm_Init_compile();
-    Scm_Init_libomega();
+    CALL_INIT(Scm_Init_libalpha);
+    CALL_INIT(Scm_Init_libbool);
+    CALL_INIT(Scm_Init_libchar);
+    CALL_INIT(Scm_Init_libcode);
+    CALL_INIT(Scm_Init_libcmp);
+    CALL_INIT(Scm_Init_libdict);
+    CALL_INIT(Scm_Init_libeval);
+    CALL_INIT(Scm_Init_libexc);
+    CALL_INIT(Scm_Init_libfmt);
+    CALL_INIT(Scm_Init_libio);
+    CALL_INIT(Scm_Init_liblazy);
+    CALL_INIT(Scm_Init_liblist);
+    CALL_INIT(Scm_Init_libmisc);
+    CALL_INIT(Scm_Init_libmod);
+    CALL_INIT(Scm_Init_libnum);
+    CALL_INIT(Scm_Init_libobj);
+    CALL_INIT(Scm_Init_libproc);
+    CALL_INIT(Scm_Init_librx);
+    CALL_INIT(Scm_Init_libsrfis);
+    CALL_INIT(Scm_Init_libstr);
+    CALL_INIT(Scm_Init_libsym);
+    CALL_INIT(Scm_Init_libsys);
+    CALL_INIT(Scm_Init_libvec);
+    CALL_INIT(Scm_Init_libmacbase);
+    CALL_INIT(Scm_Init_compile);
+    CALL_INIT(Scm_Init_libmacro);
+    CALL_INIT(Scm_Init_libparam);
+    CALL_INIT(Scm_Init_libomega);
 
-    Scm__InitCompaux();
+    CALL_INIT(Scm__InitCompaux);
 
     Scm_SelectModule(Scm_GaucheModule());
-    Scm__InitAutoloads();
+    CALL_INIT(Scm__InitAutoloads);
 
     Scm_SelectModule(Scm_UserModule());
 
     /* Final setup of cond-features alist. */
-    init_cond_features();
+    CALL_INIT(init_cond_features);
 
 #ifdef GAUCHE_USE_PTHREADS
     /* a trick to make sure the gc thread object is linked */
-    ptr_pthread_create = (int (*)(void))GC_pthread_create;
+    ptr_pthread_create = GC_pthread_create;
 #endif
 
     scheme_initialized = TRUE;
@@ -434,8 +454,8 @@ Scm_GetFeatures()
 }
 
 /*
- * Append FEATURE to the cond-features alist.  Once added, the symbol FEATURE
- * will be recognized by cond-expand.
+ * Enables FEATURE.  Once added, the symbol FEATURE will be recognized
+ * by cond-expand.
  *
  *  (cond-expand (FEATURE body ...))
  *
@@ -443,25 +463,40 @@ Scm_GetFeatures()
  * such module name can be specified in MODULE argument.  It can be NULL
  * if the feature is built-in.
  *
+ * If FEATURE is already in the disabled list, it is not added.
+ *
  * This API is meant to expose configuration information to a Scheme
  * level.
  */
 void
 Scm_AddFeature(const char *feature, const char *module)
 {
-    ScmObj cell;
-
-    if (module) {
-        cell = SCM_LIST2(SCM_INTERN(feature), SCM_INTERN(module));
-    } else {
-        cell = SCM_LIST1(SCM_INTERN(feature));
-    }
+    ScmObj f = SCM_INTERN(feature);
+    ScmObj cell = module? SCM_LIST2(f, SCM_INTERN(module)): SCM_LIST1(f);
 
     (void)SCM_INTERNAL_MUTEX_LOCK(cond_features.mutex);
-    cond_features.alist = Scm_Cons(cell, cond_features.alist);
+    if (SCM_FALSEP(Scm_Memq(f, cond_features.dlist))) {
+        cond_features.alist = Scm_Cons(cell, cond_features.alist);
+    }
     (void)SCM_INTERNAL_MUTEX_UNLOCK(cond_features.mutex);
 }
 
+/*
+ * Disables FEATURE.  If FEATURE is already enabled, it is removed
+ * from the features alist.
+ * Mainly used by gosh's -F-feature command-line option.
+ */
+void
+Scm_DisableFeature(const char *feature)
+{
+    ScmObj f = SCM_INTERN(feature);
+    (void)SCM_INTERNAL_MUTEX_LOCK(cond_features.mutex);
+    cond_features.dlist = Scm_Cons(f, cond_features.dlist);
+    cond_features.alist = Scm_AssocDelete(f, cond_features.alist, SCM_CMP_EQ);
+    (void)SCM_INTERNAL_MUTEX_UNLOCK(cond_features.mutex);
+}
+
+
 static void
 init_cond_features()
 {
@@ -504,6 +539,9 @@ init_cond_features()
         { "little-endian", NULL }, /* NB: r7rs say nothing on mixed endian */
 #endif
 
+        /* SRFIs that are not libraries */
+        { "srfi-22", NULL },    /* Scheme scripts */
+        
         /* Threads */
 #if   defined(GAUCHE_USE_PTHREADS)
         { "gauche.sys.threads", "gauche.threads" },
@@ -531,6 +569,19 @@ init_cond_features()
            try to use <mbed-tls>. */
         { "gauche.net.tls.mbedtls", NULL },
 #endif
+
+        /* zlib */
+#if defined(USE_ZLIB)
+        { "gauche.sys.zlib", "rfc.zlib" },
+#endif
+
+        { "regexp-non-greedy", "srfi.115" },
+        { "regexp-look-around", "srfi.115" },
+        { "regexp-backrefs", "srfi.115" },
+        { "regexp-unicode", "srfi.115" },
+
+#include "features.c"
+        
         { NULL, NULL }
     };
 
@@ -539,95 +590,36 @@ init_cond_features()
     }
 }
 
-#ifndef PATH_MAX
-#define PATH_MAX 4096
-#endif
-
-ScmObj Scm_LibraryDirectory(void)
+/* TRANSIENT: These used to be in paths.c but no longer needed.  We keep
+   the entries just for ABI compatibility.  These shouldn't be called.
+   Remove on 1.0 release.
+*/
+#if GAUCHE_API_VERSION < 1000
+const char *Scm_GetLibraryDirectory(void (*errfn)(const char *, ...) SCM_UNUSED)
 {
-    static ScmObj dir = SCM_FALSE;
-    if (SCM_FALSEP(dir)) {
-        char buf[PATH_MAX];
-        Scm_GetLibraryDirectory(buf, PATH_MAX, Scm_Error);
-        dir = Scm_MakeString(buf, -1, -1,
-                             SCM_STRING_COPYING|SCM_STRING_IMMUTABLE);
-    }
-    return dir;
+    Scm_Abort("Scm_GetLibraryDirectory is called (it shouldn't).");
 }
 
-ScmObj Scm_ArchitectureDirectory(void)
+const char *Scm_GetArchitectureDirectory(void (*errfn)(const char *, ...) SCM_UNUSED)
 {
-    static ScmObj dir = SCM_FALSE;
-    if (SCM_FALSEP(dir)) {
-        char buf[PATH_MAX];
-        Scm_GetArchitectureDirectory(buf, PATH_MAX, Scm_Error);
-        dir = Scm_MakeString(buf, -1, -1,
-                             SCM_STRING_COPYING|SCM_STRING_IMMUTABLE);
-    }
-    return dir;
+    Scm_Abort("Scm_GetArchitectureDirectory is called (it shouldn't).");
 }
 
-ScmObj Scm_SiteLibraryDirectory(void)
+const char *Scm_GetSiteLibraryDirectory(void (*errfn)(const char *, ...) SCM_UNUSED)
 {
-    static ScmObj dir = SCM_FALSE;
-    if (SCM_FALSEP(dir)) {
-        char buf[PATH_MAX];
-        Scm_GetSiteLibraryDirectory(buf, PATH_MAX, Scm_Error);
-        dir = Scm_MakeString(buf, -1, -1,
-                             SCM_STRING_COPYING|SCM_STRING_IMMUTABLE);
-    }
-    return dir;
+    Scm_Abort("Scm_GetSiteLibraryDirectory is called (it shouldn't).");
 }
 
-ScmObj Scm_SiteArchitectureDirectory(void)
+const char *Scm_GetSiteArchitectureDirectory(void (*errfn)(const char *, ...) SCM_UNUSED)
 {
-    static ScmObj dir = SCM_FALSE;
-    if (SCM_FALSEP(dir)) {
-        char buf[PATH_MAX];
-        Scm_GetSiteArchitectureDirectory(buf, PATH_MAX, Scm_Error);
-        dir = Scm_MakeString(buf, -1, -1,
-                             SCM_STRING_COPYING|SCM_STRING_IMMUTABLE);
-    }
-    return dir;
+    Scm_Abort("Scm_GetSiteArchitectureDirectory is called (it shouldn't).");
 }
 
-/* Temporary hack - this only works on Windows native and Darwin.
-   Returns a pathname that can be replaced for '@' prefix in various
-   configuration paths.  Note that replacement is already done in
-   the returned values of other Scm_*Directory functions above.
-   This is needed to augument the returned values of gauche.config
-   module.  Some refactorization is needed in future.
-
-   Aside from the current purpose, it will be useful to have a procedure
-   to obtain the directory of the running executable.  This function
-   can eventually evlove to it.
- */
-ScmObj Scm__RuntimeDirectory(void)
+const char *Scm_GetRuntimeDirectory(void (*errfn)(const char *, ...) SCM_UNUSED)
 {
-    static ScmObj dir = SCM_FALSE;
-    if (SCM_FALSEP(dir)) {
-        char buf[PATH_MAX];
-        Scm_GetRuntimeDirectory(buf, PATH_MAX, Scm_Error);
-        dir = Scm_MakeString(buf, -1, -1,
-                             SCM_STRING_COPYING|SCM_STRING_IMMUTABLE);
-    }
-    return dir;
-}
-
-/*=============================================================
- * Command line arguments
- */
-
-ScmObj Scm_InitCommandLine(int argc, const char *argv[])
-{
-    static ScmObj command_line_proc = SCM_UNDEFINED;
-    ScmObj args = Scm_CStringArrayToList(argv, argc, SCM_STRING_IMMUTABLE);
-    SCM_BIND_PROC(command_line_proc, "command-line", Scm_GaucheModule());
-    Scm_ApplyRec1(command_line_proc, args);
-    SCM_DEFINE(Scm_UserModule(), "*program-name*", SCM_CAR(args));
-    SCM_DEFINE(Scm_UserModule(), "*argv*", SCM_CDR(args));
-    return args;
+    Scm_Abort("Scm_GetRuntimeDirectory is called (it shouldn't).");
 }
+#endif /*GAUCHE_API_VERSION < 1000*/
 
 /*=============================================================
  * 'Main'
@@ -650,10 +642,10 @@ int main(void)
  */
 
 void Scm_SimpleMain(int argc, const char *argv[],
-                    const char *script, u_long flags)
+                    const char *script, u_long flags SCM_UNUSED)
 {
     SCM_ASSERT(argc > 0);
-    ScmObj args = Scm_InitCommandLine(argc, argv);
+    ScmObj args = Scm_InitCommandLine2(argc, argv, SCM_COMMAND_LINE_BOTH);
 
     if (script) {
         ScmObj s = SCM_MAKE_STR(script);
