@@ -1,7 +1,7 @@
 /*
  * error.c - error handling
  *
- *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -43,7 +43,6 @@
 #include <ctype.h>
 
 static ScmObj condition_allocate(ScmClass *klass, ScmObj initargs);
-static void   message_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx);
 static ScmObj message_allocate(ScmClass *klass, ScmObj initargs);
 static ScmObj syserror_allocate(ScmClass *klass, ScmObj initargs);
 static ScmObj sigerror_allocate(ScmClass *klass, ScmObj initargs);
@@ -79,18 +78,23 @@ SCM_DEFINE_BASE_CLASS(Scm_ConditionClass, ScmInstance,
                       NULL, NULL, NULL, 
                       condition_allocate, SCM_CLASS_DEFAULT_CPL);
 SCM_DEFINE_BASE_CLASS(Scm_MessageConditionClass, ScmMessageCondition,
-                      message_print, NULL, NULL,
+                      Scm_MessageConditionPrint, NULL, NULL,
                       message_allocate, condition_cpl);
 SCM_DEFINE_BASE_CLASS(Scm_SeriousConditionClass, ScmSeriousCondition,
                       NULL, NULL, NULL,
                       condition_allocate, condition_cpl);
+SCM_DEFINE_BASE_CLASS(Scm_MixinConditionClass, ScmCondition,
+                      NULL, NULL, NULL,
+                      condition_allocate, condition_cpl);
 
-static ScmObj condition_allocate(ScmClass *klass, ScmObj initargs)
+static ScmObj condition_allocate(ScmClass *klass, ScmObj initargs SCM_UNUSED)
 {
     return SCM_OBJ(SCM_NEW_INSTANCE(ScmCondition, klass));
 }
 
-static void message_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
+/* We expose this, for other condition subclasses may share this. */
+void Scm_MessageConditionPrint(ScmObj obj, ScmPort *port, 
+                               ScmWriteContext *ctx SCM_UNUSED)
 {
     ScmClass *k = Scm_ClassOf(obj);
     Scm_Printf(port, "#<%A \"%30.1A\">",
@@ -98,7 +102,7 @@ static void message_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
                SCM_ERROR_MESSAGE(obj));
 }
 
-static ScmObj message_allocate(ScmClass *klass, ScmObj initargs)
+static ScmObj message_allocate(ScmClass *klass, ScmObj initargs SCM_UNUSED)
 {
     ScmError *e = SCM_NEW_INSTANCE(ScmError, klass);
     e->message = SCM_FALSE;     /* would be set by initialize */
@@ -117,7 +121,7 @@ static ScmObj message_get(ScmMessageCondition *obj)
 static void message_set(ScmMessageCondition *obj, ScmObj val)
 {
     ScmObj msglist = obj->message;
-    if (SCM_PAIRP(msglist)) SCM_SET_CAR(msglist, val);
+    if (SCM_PAIRP(msglist)) SCM_SET_CAR_UNCHECKED(msglist, val);
     else SCM_MESSAGE_CONDITION(obj)->message = SCM_LIST2(val, val);
 }
 
@@ -135,7 +139,7 @@ static void message_prefix_set(ScmMessageCondition *obj, ScmObj val)
 {
     ScmObj msglist = obj->message;
     if (SCM_PAIRP(msglist) && SCM_PAIRP(SCM_CDR(msglist))) {
-        SCM_SET_CAR(SCM_CDR(msglist), val);
+        SCM_SET_CAR_UNCHECKED(SCM_CDR(msglist), val);
     } else {
         obj->message = SCM_LIST2(msglist, val);
     }
@@ -155,7 +159,7 @@ static void message_args_set(ScmMessageCondition *obj, ScmObj val)
 {
     ScmObj msglist = obj->message;
     if (SCM_PAIRP(msglist) && SCM_PAIRP(SCM_CDR(msglist))) {
-        SCM_SET_CDR(SCM_CDR(msglist), val);
+        SCM_SET_CDR_UNCHECKED(SCM_CDR(msglist), val);
     } else {
         obj->message = Scm_Cons(msglist, Scm_Cons(msglist, val));
     }
@@ -184,39 +188,63 @@ static ScmClass *porterror_cpl[] = {
     NULL
 };
 
+static ScmClass *decoding_error_cpl[] = {
+    SCM_CLASS_STATIC_PTR(Scm_IOReadErrorClass),
+    SCM_CLASS_STATIC_PTR(Scm_PortErrorClass),
+    SCM_CLASS_STATIC_PTR(Scm_IOErrorClass),
+    ERROR_CPL,
+    NULL
+};
+    
+static ScmClass *encoding_error_cpl[] = {
+    SCM_CLASS_STATIC_PTR(Scm_IOWriteErrorClass),
+    SCM_CLASS_STATIC_PTR(Scm_PortErrorClass),
+    SCM_CLASS_STATIC_PTR(Scm_IOErrorClass),
+    ERROR_CPL,
+    NULL
+};
 
 SCM_DEFINE_BASE_CLASS(Scm_ErrorClass, ScmError,
-                      message_print, NULL, NULL,
+                      Scm_MessageConditionPrint, NULL, NULL,
                       message_allocate, error_cpl+1);
 SCM_DEFINE_BASE_CLASS(Scm_SystemErrorClass, ScmSystemError,
-                      message_print, NULL, NULL,
+                      Scm_MessageConditionPrint, NULL, NULL,
                       syserror_allocate, error_cpl);
 SCM_DEFINE_BASE_CLASS(Scm_UnhandledSignalErrorClass, ScmUnhandledSignalError,
-                      message_print, NULL, NULL,
+                      Scm_MessageConditionPrint, NULL, NULL,
                       sigerror_allocate, error_cpl);
 SCM_DEFINE_BASE_CLASS(Scm_ReadErrorClass, ScmReadError,
-                      message_print, NULL, NULL,
+                      Scm_MessageConditionPrint, NULL, NULL,
                       readerror_allocate, error_cpl);
 SCM_DEFINE_BASE_CLASS(Scm_IOErrorClass, ScmIOError,
-                      message_print, NULL, NULL,
+                      Scm_MessageConditionPrint, NULL, NULL,
                       message_allocate, error_cpl);
 SCM_DEFINE_BASE_CLASS(Scm_PortErrorClass, ScmPortError,
-                      message_print, NULL, NULL,
+                      Scm_MessageConditionPrint, NULL, NULL,
                       porterror_allocate, porterror_cpl+1);
 SCM_DEFINE_BASE_CLASS(Scm_IOReadErrorClass, ScmIOReadError,
-                      message_print, NULL, NULL,
+                      Scm_MessageConditionPrint, NULL, NULL,
                       porterror_allocate, porterror_cpl);
 SCM_DEFINE_BASE_CLASS(Scm_IOWriteErrorClass, ScmIOWriteError,
-                      message_print, NULL, NULL,
+                      Scm_MessageConditionPrint, NULL, NULL,
                       porterror_allocate, porterror_cpl);
 SCM_DEFINE_BASE_CLASS(Scm_IOClosedErrorClass, ScmIOClosedError,
-                      message_print, NULL, NULL,
+                      Scm_MessageConditionPrint, NULL, NULL,
                       porterror_allocate, porterror_cpl);
 SCM_DEFINE_BASE_CLASS(Scm_IOUnitErrorClass, ScmIOUnitError,
-                      message_print, NULL, NULL,
+                      Scm_MessageConditionPrint, NULL, NULL,
+                      porterror_allocate, porterror_cpl);
+SCM_DEFINE_BASE_CLASS(Scm_IODecodingErrorClass, ScmIODecodingError,
+                      Scm_MessageConditionPrint, NULL, NULL,
+                      porterror_allocate, decoding_error_cpl);
+SCM_DEFINE_BASE_CLASS(Scm_IOEncodingErrorClass, ScmIOEncodingError,
+                      Scm_MessageConditionPrint, NULL, NULL,
+                      porterror_allocate, encoding_error_cpl);
+SCM_DEFINE_BASE_CLASS(Scm_IOInvalidPositionErrorClass, ScmIOInvalidPositionError,
+                      Scm_MessageConditionPrint, NULL, NULL,
                       porterror_allocate, porterror_cpl);
 
-static ScmObj syserror_allocate(ScmClass *klass, ScmObj initargs)
+static ScmObj syserror_allocate(ScmClass *klass, ScmObj initargs SCM_UNUSED)
 {
     ScmSystemError *e = SCM_NEW_INSTANCE(ScmSystemError, klass);
     e->common.message = SCM_FALSE; /* set by initialize */
@@ -224,7 +252,7 @@ static ScmObj syserror_allocate(ScmClass *klass, ScmObj initargs)
     return SCM_OBJ(e);
 }
 
-static ScmObj sigerror_allocate(ScmClass *klass, ScmObj initargs)
+static ScmObj sigerror_allocate(ScmClass *klass, ScmObj initargs SCM_UNUSED)
 {
     ScmUnhandledSignalError *e = SCM_NEW_INSTANCE(ScmUnhandledSignalError,
                                                   klass);
@@ -233,7 +261,7 @@ static ScmObj sigerror_allocate(ScmClass *klass, ScmObj initargs)
     return SCM_OBJ(e);
 }
 
-static ScmObj readerror_allocate(ScmClass *klass, ScmObj initargs)
+static ScmObj readerror_allocate(ScmClass *klass, ScmObj initargs SCM_UNUSED)
 {
     ScmReadError *e = SCM_NEW_INSTANCE(ScmReadError, klass);
     e->common.message = SCM_FALSE; /* set by initialize */
@@ -242,11 +270,12 @@ static ScmObj readerror_allocate(ScmClass *klass, ScmObj initargs)
     return SCM_OBJ(e);
 }
 
-static ScmObj porterror_allocate(ScmClass *klass, ScmObj initargs)
+static ScmObj porterror_allocate(ScmClass *klass, ScmObj initargs SCM_UNUSED)
 {
     ScmPortError *e = SCM_NEW_INSTANCE(ScmPortError, klass);
     e->common.message = SCM_FALSE; /* set by initialize */
     e->port = NULL;                /* set by initialize */
+    e->auxinfo = SCM_NIL;          /* set by initialize */
     return SCM_OBJ(e);
 }
 
@@ -308,12 +337,13 @@ static void readerror_line_set(ScmReadError *obj, ScmObj val)
     obj->line = SCM_INT_VALUE(val);
 }
 
-static ScmObj readerror_dummy_get(ScmReadError *obj)
+static ScmObj readerror_dummy_get(ScmReadError *obj SCM_UNUSED)
 {
     return SCM_FALSE;
 }
 
-static void readerror_dummy_set(ScmReadError *obj, ScmObj val)
+static void readerror_dummy_set(ScmReadError *obj SCM_UNUSED,
+                                ScmObj val SCM_UNUSED)
 {
     /* nothing */
 }
@@ -331,6 +361,44 @@ static void porterror_port_set(ScmPortError *obj, ScmObj val)
     obj->port = SCM_FALSEP(val)? NULL : SCM_PORT(val);
 }
 
+static ScmObj porterror_auxinfo_getter(ScmPortError *obj, ScmObj key)
+{
+    ScmObj p = Scm_Assq(key, obj->auxinfo);
+    if (SCM_PAIRP(p)) return SCM_CDR(p);
+    return SCM_FALSE;
+}
+
+static void porterror_auxinfo_setter(ScmPortError *obj, ScmObj key, ScmObj val)
+{
+    ScmObj p = Scm_Assq(key, obj->auxinfo);
+    if (SCM_PAIRP(p)) SCM_SET_CDR(p, val);
+    else obj->auxinfo = Scm_Acons(key, val, obj->auxinfo);
+}
+
+static ScmObj sym_offending_char; /* 'offending-char */
+
+static ScmObj porterror_offending_char_get(ScmPortError *obj)
+{
+    return porterror_auxinfo_getter(obj, sym_offending_char);
+}
+
+static void porterror_offending_char_set(ScmPortError *obj, ScmObj val)
+{
+    porterror_auxinfo_setter(obj, sym_offending_char, val);
+}
+
+static ScmObj sym_position;     /* 'position */
+
+static ScmObj porterror_position_get(ScmPortError *obj)
+{
+    return porterror_auxinfo_getter(obj, sym_position);
+}
+
+static void porterror_position_set(ScmPortError *obj, ScmObj val)
+{
+    porterror_auxinfo_setter(obj, sym_position, val);
+}
+
 static ScmClassStaticSlotSpec syserror_slots[] = {
     SCM_CLASS_SLOT_SPEC("errno", syserror_number_get, syserror_number_set),
     SCM_CLASS_SLOT_SPEC_END()
@@ -355,6 +423,23 @@ static ScmClassStaticSlotSpec porterror_slots[] = {
     SCM_CLASS_SLOT_SPEC_END()
 };
 
+static ScmClassStaticSlotSpec encodingerror_slots[] = {
+    SCM_CLASS_SLOT_SPEC("port", porterror_port_get, porterror_port_set),
+    SCM_CLASS_SLOT_SPEC("offending-char", 
+                        porterror_offending_char_get, 
+                        porterror_offending_char_set),
+    SCM_CLASS_SLOT_SPEC_END()
+};
+    
+static ScmClassStaticSlotSpec invalidpositionerror_slots[] = {
+    SCM_CLASS_SLOT_SPEC("port", porterror_port_get, porterror_port_set),
+    SCM_CLASS_SLOT_SPEC("position", 
+                        porterror_position_get, 
+                        porterror_position_set),
+    SCM_CLASS_SLOT_SPEC_END()
+};
+    
+
 /*------------------------------------------------------------
  * Compound conditions
  */
@@ -366,7 +451,8 @@ static ScmClass *compound_cpl[] = {
     NULL
 };
 
-static void compound_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
+static void compound_print(ScmObj obj, ScmPort *port, 
+                           ScmWriteContext *ctx SCM_UNUSED)
 {
     ScmClass *k = Scm_ClassOf(obj);
     Scm_Printf(port, "#<%A", Scm_ShortClassName(k));
@@ -385,7 +471,7 @@ SCM_DEFINE_BASE_CLASS(Scm_SeriousCompoundConditionClass, ScmCompoundCondition,
                       compound_print, NULL, NULL,
                       compound_allocate, compound_cpl);
 
-static ScmObj compound_allocate(ScmClass *klass, ScmObj initargs)
+static ScmObj compound_allocate(ScmClass *klass, ScmObj initargs SCM_UNUSED)
 {
     ScmCompoundCondition *e = SCM_NEW_INSTANCE(ScmCompoundCondition, klass);
     e->conditions = SCM_NIL;
@@ -616,14 +702,18 @@ ScmObj Scm_ConditionTypeName(ScmObj c)
  * check that.  Ideally a common single API should handle it,   
  * but for the time being, we add the check at the beginning
  * of Scm_*Error APIs.
- * The SCM_ERROR_BEING_HANDLED flag is cleared in Scm_VMThrowException2().
+ * The SCM_ERROR_BEING_HANDLED flag is cleared in Scm_VMThrowException().
  */
-#define SCM_ERROR_DOUBLE_FAULT_CHECK(vm)                                \
+#define SCM_ERROR_DOUBLE_FAULT_CHECK(vm, msg)                           \
     do {                                                                \
         if (SCM_VM_RUNTIME_FLAG_IS_SET(vm, SCM_ERROR_BEING_HANDLED)) {  \
-            ScmObj e =                                                  \
-                Scm_MakeError(SCM_MAKE_STR("Error occurred in error handler")); \
-            Scm_VMThrowException2(vm, e, SCM_RAISE_NON_CONTINUABLE);    \
+            ScmDString ds;                                              \
+            Scm_DStringInit(&ds);                                       \
+            Scm_DStringPutz(&ds, "Error occurred in error handler (", -1); \
+            Scm_DStringPutz(&ds, msg, -1);                              \
+            Scm_DStringPutz(&ds, ")", -1);                              \
+            ScmObj e = Scm_MakeError(Scm_DStringGet(&ds, 0));           \
+            Scm_VMThrowException(vm, e, SCM_RAISE_NON_CONTINUABLE);     \
         }                                                               \
         SCM_VM_RUNTIME_FLAG_SET(vm, SCM_ERROR_BEING_HANDLED);           \
     } while (0)
@@ -659,11 +749,11 @@ ScmObj Scm_ConditionTypeName(ScmObj c)
 void Scm_Error(const char *msg, ...)
 {
     ScmVM *vm = Scm_VM();
-    SCM_ERROR_DOUBLE_FAULT_CHECK(vm);
+    SCM_ERROR_DOUBLE_FAULT_CHECK(vm, msg);
     ScmObj ostr;
     SCM_ERROR_MESSAGE_FORMAT(ostr, msg);
     ScmObj e = Scm_MakeError(Scm_GetOutputString(SCM_PORT(ostr), TRUE));
-    Scm_VMThrowException2(vm, e, SCM_RAISE_NON_CONTINUABLE);
+    Scm_VMThrowException(vm, e, SCM_RAISE_NON_CONTINUABLE);
     Scm_Panic("Scm_Error: Scm_VMThrowException returned.  something wrong.");
 }
 
@@ -730,12 +820,12 @@ void Scm_SysError(const char *msg, ...)
 {
     int en = get_errno();       /* must take this before Scm_VM() */
     ScmVM *vm = Scm_VM();
-    SCM_ERROR_DOUBLE_FAULT_CHECK(vm);
+    SCM_ERROR_DOUBLE_FAULT_CHECK(vm, msg);
 
     ScmObj ostr;
     SCM_SYSERROR_MESSAGE_FORMAT(ostr, msg, en);
     ScmObj e = Scm_MakeSystemError(Scm_GetOutputString(SCM_PORT(ostr), TRUE), en);
-    Scm_VMThrowException2(vm, e, SCM_RAISE_NON_CONTINUABLE);
+    Scm_VMThrowException(vm, e, SCM_RAISE_NON_CONTINUABLE);
     Scm_Panic("Scm_Error: Scm_VMThrowException returned.  something wrong.");
 }
 
@@ -753,22 +843,21 @@ void Scm_TypeError(const char *what, const char *expected, ScmObj got)
 
 /*
  * A convenience function to raise port-related errors.
- * It creates either one of <port-error>, <io-read-error>,
- * <io-write-error>, <io-closed-error>, or <io-unit-error>,
- * depending on the 'reason' argument being
- * SCM_PORT_ERROR_{OTHER,INPUT,OUTPUT,CLOSED,UNIT}, respectively.
+ * It creates either one of port error instance,
+ * depending on the 'reason' argument.
  * If errno isn't zero, it also creates a <system-error> and throws
  * a compound condition of both.
  */
-void Scm_PortError(ScmPort *port, int reason, const char *msg, ...)
+void raise_port_error(ScmVM *vm, ScmPort *port, int reason, ScmObj auxinfo,
+                      int orig_errno, const char *msg, va_list args)
 {
-    int en = get_errno();       /* must take this before Scm_VM() */
-    ScmVM *vm = Scm_VM();
-    SCM_ERROR_DOUBLE_FAULT_CHECK(vm);
+    Scm_SetPortErrorOccurred(port, TRUE);
 
-    ScmObj ostr;
-    if (en != 0) SCM_SYSERROR_MESSAGE_FORMAT(ostr, msg, en);
-    else         SCM_ERROR_MESSAGE_FORMAT(ostr, msg);
+    ScmObj ostr = Scm_MakeOutputStringPort(TRUE);
+    Scm_Vprintf(SCM_PORT(ostr), msg, args, TRUE);
+    ScmObj syserr = get_syserrmsg(orig_errno);
+    Scm_Putz(": ", -1, SCM_PORT(ostr));
+    Scm_Puts(SCM_STRING(syserr), SCM_PORT(ostr));
     ScmObj smsg = Scm_GetOutputString(SCM_PORT(ostr), TRUE);
 
     ScmClass *peclass;
@@ -781,6 +870,14 @@ void Scm_PortError(ScmPort *port, int reason, const char *msg, ...)
         peclass = SCM_CLASS_IO_CLOSED_ERROR; break;
     case SCM_PORT_ERROR_UNIT:
         peclass = SCM_CLASS_IO_UNIT_ERROR; break;
+    case SCM_PORT_ERROR_DECODING:
+        peclass = SCM_CLASS_IO_DECODING_ERROR; break;
+    case SCM_PORT_ERROR_ENCODING:
+        peclass = SCM_CLASS_IO_ENCODING_ERROR; break;
+    case SCM_PORT_ERROR_SEEK:
+        peclass = SCM_CLASS_PORT_ERROR; break;
+    case SCM_PORT_ERROR_INVALID_POSITION:
+        peclass = SCM_CLASS_IO_INVALID_POSITION_ERROR; break;
     default:
         peclass = SCM_CLASS_PORT_ERROR; break;
     }
@@ -788,13 +885,40 @@ void Scm_PortError(ScmPort *port, int reason, const char *msg, ...)
     ScmObj pe = porterror_allocate(peclass, SCM_NIL);
     SCM_ERROR(pe)->message = SCM_LIST2(smsg, smsg);
     SCM_PORT_ERROR(pe)->port = port;
+    SCM_PORT_ERROR(pe)->auxinfo = auxinfo;
 
     ScmObj e = pe;
-    if (en != 0) {
-        e = Scm_MakeCompoundCondition(SCM_LIST2(Scm_MakeSystemError(smsg, en),
+    if (orig_errno != 0) {
+        e = Scm_MakeCompoundCondition(SCM_LIST2(Scm_MakeSystemError(smsg, orig_errno),
                                                 pe));
     }
-    Scm_VMThrowException2(vm, e, SCM_RAISE_NON_CONTINUABLE);
+    Scm_VMThrowException(vm, e, SCM_RAISE_NON_CONTINUABLE);
+}
+
+void Scm_PortErrorWithAux(ScmPort *port, int reason, ScmObj auxinfo,
+                          const char *msg, ...)
+{
+    int en = get_errno();       /* must take this before Scm_VM() */
+    ScmVM *vm = Scm_VM();
+    SCM_ERROR_DOUBLE_FAULT_CHECK(vm, msg);
+
+    va_list ap;
+    va_start(ap, msg);
+    raise_port_error(vm, port, reason, auxinfo, en, msg, ap);
+    va_end(ap);
+    Scm_Panic("Scm_Error: Scm_VMThrowException returned.  something wrong.");
+}
+
+void Scm_PortError(ScmPort *port, int reason, const char *msg, ...)
+{
+    int en = get_errno();       /* must take this before Scm_VM() */
+    ScmVM *vm = Scm_VM();
+    SCM_ERROR_DOUBLE_FAULT_CHECK(vm, msg);
+
+    va_list ap;
+    va_start(ap, msg);
+    raise_port_error(vm, port, reason, SCM_NIL, en, msg, ap);
+    va_end(ap);
     Scm_Panic("Scm_Error: Scm_VMThrowException returned.  something wrong.");
 }
 
@@ -814,7 +938,7 @@ void Scm_Warn(const char *msg, ...)
 }
 
 /* OBSOLETED: 'warn' is now in Scheme. */
-void Scm_FWarn(ScmString *fmt, ScmObj args)
+void Scm_FWarn(ScmString *fmt SCM_UNUSED, ScmObj args SCM_UNUSED)
 {
     Scm_Error("Scm_FWarn is obsoleted");
 }
@@ -824,22 +948,10 @@ void Scm_FWarn(ScmString *fmt, ScmObj args)
  */
 
 /* An external API to hide Scm_VMThrowException. */
-#if  GAUCHE_API_0_95
 ScmObj Scm_Raise(ScmObj condition, u_long flags)
 {
     return Scm_VMThrowException(Scm_VM(), condition, flags);
 }
-#else  /*!GAUCHE_API_0_95*/
-ScmObj Scm_Raise(ScmObj condition)
-{
-    return Scm_Raise2(condition, 0);
-}
-
-ScmObj Scm_Raise2(ScmObj condition, u_long flags)
-{
-    return Scm_VMThrowException2(Scm_VM(), condition, flags);
-}
-#endif /*!GAUCHE_API_0_95*/
 
 
 /* A convenient API---allows to call user-defined condition easily,
@@ -916,7 +1028,8 @@ ScmObj Scm_RaiseCondition(ScmObj condition_type, ...)
  *   format   - SCM_STACK_TRACE_FORMAT_* enum value.  No longer used.
  */
 void Scm_ShowStackTrace(ScmPort *out, ScmObj stacklite,
-                        int maxdepth, int skip, int offset, int format)
+                        int maxdepth, int skip, int offset, 
+                        int format SCM_UNUSED)
 {
     static ScmObj show_stack_trace = SCM_UNDEFINED;
     SCM_BIND_PROC(show_stack_trace,
@@ -935,14 +1048,14 @@ void Scm_DumpStackTrace(ScmVM *vm, ScmPort *port)
     if (vm == NULL) vm = Scm_VM();
     if (port == NULL) port = SCM_VM_CURRENT_ERROR_PORT(vm);
     ScmObj stack = Scm_VMGetStackLite(vm);
+    ScmObj calls = Scm_VMGetCallTraceLite(vm);
     SCM_PUTZ("Stack Trace:\n", -1, port);
     SCM_PUTZ("_______________________________________\n", -1, port);
     Scm_ShowStackTrace(port, stack, 0, 0, 0, 0);
-    if (vm->callTrace) {
-        ScmObj trace = Scm_VMGetCallTraceLite(vm);
+    if (SCM_PAIRP(calls)) {
         SCM_PUTZ("Call Trace:\n", -1, port);
         SCM_PUTZ("_______________________________________\n", -1, port);
-        Scm_ShowStackTrace(port, trace, 0, 0, 0, 0);
+        Scm_ShowStackTrace(port, calls, 0, 0, 0, 0);
     }
     SCM_FLUSH(port);
 }
@@ -982,13 +1095,11 @@ static void Scm_PrintDefaultErrorHeading(ScmObj e, ScmPort *out)
    port, and #f for string port.  If it's output port, use it.  For any
    other objects, we use current error port.  This permissive behavior is
    intentional - report-error is usually called during error handling,
-   and raising an error there masks the original error.
+   and raising an error there masks the original error.  However, the
+   caller should explicitly pass SCM_OBJ(SCM_CURERR) if that's what it intends,
+   instead of relying on the permissive behavior.
 */
-#if GAUCHE_API_0_95
 ScmObj Scm_ReportError(ScmObj e, ScmObj out)
-#else  /*!GAUCHE_API_0_95*/
-ScmObj Scm_ReportError2(ScmObj e, ScmObj out)
-#endif /*!GAUCHE_API_0_95*/
 {
     ScmVM *vm = Scm_VM();
     ScmPort *port = SCM_VM_CURRENT_ERROR_PORT(vm);
@@ -1005,13 +1116,6 @@ ScmObj Scm_ReportError2(ScmObj e, ScmObj out)
     else return SCM_UNDEFINED;
 }
 
-#if !GAUCHE_API_0_95
-ScmObj Scm_ReportError(ScmObj e)
-{
-    return Scm_ReportError2(e, SCM_UNBOUND);
-}
-#endif /*!GAUCHE_API_0_95*/
-
 /*
  * Initialization
  */
@@ -1036,6 +1140,9 @@ void Scm__InitExceptions(void)
     Scm_InitStaticClassWithMeta(SCM_CLASS_MESSAGE_CONDITION,
                                 "<message-condition>",
                                 mod, cond_meta, SCM_FALSE, message_slots, 0);
+    Scm_InitStaticClassWithMeta(SCM_CLASS_MIXIN_CONDITION,
+                                "<mixin-condition>",
+                                mod, cond_meta, SCM_FALSE, NULL, 0);
 
     Scm_InitStaticClassWithMeta(SCM_CLASS_ERROR,
                                 "<error>",
@@ -1077,7 +1184,18 @@ void Scm__InitExceptions(void)
                                 "<io-unit-error>",
                                 mod, cond_meta, SCM_FALSE,
                                 porterror_slots, 0);
-
+    Scm_InitStaticClassWithMeta(SCM_CLASS_IO_DECODING_ERROR,
+                                "<io-decoding-error>",
+                                mod, cond_meta, SCM_FALSE,
+                                porterror_slots, 0);
+    Scm_InitStaticClassWithMeta(SCM_CLASS_IO_ENCODING_ERROR,
+                                "<io-encoding-error>",
+                                mod, cond_meta, SCM_FALSE,
+                                encodingerror_slots, 0);
+    Scm_InitStaticClassWithMeta(SCM_CLASS_IO_INVALID_POSITION_ERROR,
+                                "<io-invalid-position-error>",
+                                mod, cond_meta, SCM_FALSE,
+                                invalidpositionerror_slots, 0);
 
     Scm_InitStaticClassWithMeta(SCM_CLASS_COMPOUND_CONDITION,
                                 "<compound-condition>",
@@ -1087,4 +1205,7 @@ void Scm__InitExceptions(void)
                                 "<serious-compound-condition>",
                                 mod, cond_meta, com_ser_supers,
                                 compound_slots, 0);
+
+    sym_offending_char = SCM_INTERN("offending-char");
+    sym_position = SCM_INTERN("position");
 }
