@@ -1,7 +1,7 @@
 /*
  * gauche.h - Gauche scheme system header
  *
- *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -34,6 +34,14 @@
 #ifndef GAUCHE_H
 #define GAUCHE_H
 
+/* GAUCHE_API_VERSION is GAUCHE_MAJOR_VERSION*1000 + revision.
+   The revision is only incremented when we change API, which we expect
+   rare during the same major revision. */
+#ifndef GAUCHE_API_VERSION
+#define GAUCHE_API_VERSION  97
+//#define GAUCHE_API_VERSION 1000
+#endif
+
 /* Read config.h _before_ other headers, for it may affect the behavior
    of system header files.  Currently the only known instance of it is
    sigwait() on Solaris---we need to define _POSIX_PTHREAD_SEMANTICS to
@@ -42,14 +50,6 @@
 #include <gauche/config.h>
 #include <gauche/config_threads.h>
 
-#if     GAUCHE_API_0_95         /* A provisional API towards 1.0 */
-#define GAUCHE_API_0_9   0
-#endif  /*GAUCHE_API_0_95*/
-
-#ifndef GAUCHE_API_0_9
-#define GAUCHE_API_0_9   1      /* 0.9 API */
-#endif
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
@@ -61,8 +61,8 @@
 #include <string.h>
 #include <errno.h>
 #include <stdint.h>
+#include <inttypes.h>
 #include <gauche/int64.h>
-#include <gauche/float.h>
 
 #ifdef TIME_WITH_SYS_TIME
 # include <sys/time.h>
@@ -75,6 +75,8 @@
 # endif
 #endif
 
+typedef ssize_t ScmSize;
+
 /* For Windows platforms, we need some compatibility tricks.
    This defines GAUCHE_WINDOWS preprocessor symbol.
    (This should come before including gc.h) */
@@ -82,6 +84,9 @@
 #include <gauche/win-compat.h>
 #endif /* MINGW32 || WINDOWS */
 
+/* Defines SCM_EXTERN magic. */
+#include <gauche/extern.h>
+
 #if defined(LIBGAUCHE_BODY)
 #if !defined(GC_DLL)
 #define GC_DLL    /* for gc.h to handle Win32 crazyness */
@@ -108,8 +113,8 @@ SCM_DECL_BEGIN
 #include <unistd.h>
 #endif /*HAVE_UNISTD_H*/
 
-/* Defines SCM_EXTERN magic. */
-#include <gauche/extern.h>
+/* This must come after gauche/extern.h */
+#include <gauche/float.h>
 
 /* Some useful macros */
 #ifndef FALSE
@@ -119,18 +124,28 @@ SCM_DECL_BEGIN
 #define TRUE (!FALSE)
 #endif
 
+
+
 /* Define this to 0 to turn off fast flonum extension.  See the comment in
    gauche/number.h for the details. */
 #define GAUCHE_FFX 1
 
-/* Define this to 0 to turn off lazy-pair feature. */
-#define GAUCHE_LAZY_PAIR 1
+/* Temporary - to test alignment of pairs */
+#define GAUCHE_CHECK_PAIR_ALIGNMENT 0
+
+/* TRANSIENT: Define this to 1 to include (obsoleted) string pointer functions.
+   It will be completely gone soon. */
+#define GAUCHE_STRING_POINTER 0
 
 /* Enable an option to make keywords and symbols disjoint.
    (Transient: Will be gone once we completely migrate to
    unified keyword-symbol system */
 #define GAUCHE_KEEP_DISJOINT_KEYWORD_OPTION 1
 
+/* Experimental: Enable lightweight continuation capturing in exception
+   handling. */
+#define GAUCHE_SPLIT_STACK 0
+
 /* Include appropriate threading interface.  Threading primitives are
    abstracted with SCM_INTERNAL_* macros and ScmInternal* typedefs.
    See gauche/uthread.h for the semantics of these primitives. */
@@ -151,7 +166,6 @@ SCM_DECL_BEGIN
 #define SCM_IGNORE_RESULT(expr)  do { if(expr) {} } while(0)
 
 /* ScmFlonum and ScmClass must always be aligned in 8-byte boundaries.
-   (All other Scheme objects can be in 4-byte boundary.)
    Some platform doesn't align static double in 8-byte boundaries, so
    we try this as well.  */
 #ifdef __GNUC__
@@ -160,6 +174,15 @@ SCM_DECL_BEGIN
 #define SCM_ALIGN8  /*empty*/
 #endif /* !__GNUC__ */
 
+/* Statically allocated ScmPair must be aligned in two ScmWords boundary.*/
+#ifdef __GNUC__
+#define SCM_PAIR_ALWAYS_ALIGNED_EVEN_WORDS  1
+#define SCM_ALIGN_PAIR  __attribute__ ((aligned(sizeof(ScmWord)*2)))
+#else  /* !__GNUC__ */
+#define SCM_PAIR_ALWAYS_ALIGNED_EVEN_WORDS  0
+#define SCM_ALIGN_PAIR  /*empty*/
+#endif /* !__GNUC__ */
+
 /* 'No return' attribute */
 #ifdef __GNUC__
 #define SCM_NORETURN  __attribute__((__noreturn__))
@@ -167,6 +190,21 @@ SCM_DECL_BEGIN
 #define SCM_NORETURN  /*empty*/
 #endif /*__GNUC__*/
 
+/* 'unused' attribute */
+#ifdef __GNUC__
+#define SCM_UNUSED   __attribute__((__unused__))
+#else  /*__GNUC__*/
+#define SCM_UNUSED  /*empty*/
+#endif /*__GNUC__*/
+
+/* 'noinline' attribute */
+#ifdef __GNUC__
+#define SCM_NOINLINE __attribute__((__noinline__))
+#else  /*__GNUC__*/
+#define SCM_NOINLINE  /*empty*/
+#endif /*__GNUC__*/
+
+
 /*-------------------------------------------------------------
  * BASIC TYPES
  */
@@ -223,12 +261,16 @@ typedef struct ScmClassRec ScmClass;
  *
  * [Miscellaneous]
  *      -------- -------- -------- 00001011
- *      #f, #t, '(), eof-object, undefined
+ *      #f, #t, '(), eof-object, undefined, uninitialized
  *
  * [Pattern variable]
  *      -------- -------- -------- 00010011
  *      Used in macro expander.
  *
+ * [String cursor]
+ *      -------- -------- -------- 00011011
+ *      Represent short string cursors.
+ *
  * [Heap object]
  *      -------- -------- -------- -----111
  *      Only appears at the first word of heap-allocated
@@ -276,13 +318,15 @@ typedef struct ScmClassRec ScmClass;
 #define SCM_EOF             SCM_OBJ(SCM__MAKE_ITAG(3)) /* eof-object */
 #define SCM_UNDEFINED       SCM_OBJ(SCM__MAKE_ITAG(4)) /* #undefined */
 #define SCM_UNBOUND         SCM_OBJ(SCM__MAKE_ITAG(5)) /* unbound value */
+#define SCM_UNINITIALIZED   SCM_OBJ(SCM__MAKE_ITAG(6)) /* uninitialized */
 
-#define SCM_FALSEP(obj)     ((obj) == SCM_FALSE)
-#define SCM_TRUEP(obj)      ((obj) == SCM_TRUE)
-#define SCM_NULLP(obj)      ((obj) == SCM_NIL)
-#define SCM_EOFP(obj)       ((obj) == SCM_EOF)
-#define SCM_UNDEFINEDP(obj) ((obj) == SCM_UNDEFINED)
-#define SCM_UNBOUNDP(obj)   ((obj) == SCM_UNBOUND)
+#define SCM_FALSEP(obj)         ((obj) == SCM_FALSE)
+#define SCM_TRUEP(obj)          ((obj) == SCM_TRUE)
+#define SCM_NULLP(obj)          ((obj) == SCM_NIL)
+#define SCM_EOFP(obj)           ((obj) == SCM_EOF)
+#define SCM_UNDEFINEDP(obj)     ((obj) == SCM_UNDEFINED)
+#define SCM_UNBOUNDP(obj)       ((obj) == SCM_UNBOUND)
+#define SCM_UNINITIALIZEDP(obj) ((obj) == SCM_UNINITIALIZED)
 
 /*
  * BOOLEAN
@@ -497,41 +541,41 @@ SCM_EXTERN void Scm_UnregisterFinalizer(ScmObj z);
 /* Safe coercer */
 #define SCM_OBJ_SAFE(obj)     ((obj)?SCM_OBJ(obj):SCM_UNDEFINED)
 
-typedef struct ScmVMRec        ScmVM;
-typedef struct ScmPairRec      ScmPair;
-typedef struct ScmExtendedPairRec ScmExtendedPair;
-typedef struct ScmLazyPairRec  ScmLazyPair;
-typedef struct ScmCharSetRec   ScmCharSet;
-typedef struct ScmStringRec    ScmString;
-typedef struct ScmDStringRec   ScmDString;
-typedef struct ScmVectorRec    ScmVector;
-typedef struct ScmBignumRec    ScmBignum;
-typedef struct ScmRatnumRec    ScmRatnum;
-typedef struct ScmCompnumRec   ScmCompnum;
-typedef struct ScmPortRec      ScmPort;
-typedef struct ScmHashTableRec ScmHashTable;
-typedef struct ScmTreeMapRec   ScmTreeMap;
-typedef struct ScmModuleRec    ScmModule;
-typedef struct ScmSymbolRec    ScmSymbol;
-typedef struct ScmGlocRec      ScmGloc;
-typedef struct ScmProcedureRec ScmProcedure;
-typedef struct ScmClosureRec   ScmClosure;
-typedef struct ScmSubrRec      ScmSubr;
-typedef struct ScmGenericRec   ScmGeneric;
-typedef struct ScmMethodRec    ScmMethod;
-typedef struct ScmNextMethodRec ScmNextMethod;
-typedef struct ScmSyntaxRec    ScmSyntax;
-typedef struct ScmMacroRec     ScmMacro;
-typedef struct ScmPromiseRec   ScmPromise;
-typedef struct ScmRegexpRec    ScmRegexp;
-typedef struct ScmRegMatchRec  ScmRegMatch;
+typedef struct ScmVMRec             ScmVM;
+typedef struct ScmPairRec           ScmPair;
+typedef struct ScmExtendedPairRec   ScmExtendedPair;
+typedef struct ScmLazyPairRec       ScmLazyPair;
+typedef struct ScmCharSetRec        ScmCharSet;
+typedef struct ScmStringRec         ScmString;
+typedef struct ScmDStringRec        ScmDString;
+typedef struct ScmVectorRec         ScmVector;
+typedef struct ScmBignumRec         ScmBignum;
+typedef struct ScmRatnumRec         ScmRatnum;
+typedef struct ScmCompnumRec        ScmCompnum;
+typedef struct ScmPortRec           ScmPort;
+typedef struct ScmHashTableRec      ScmHashTable;
+typedef struct ScmTreeMapRec        ScmTreeMap;
+typedef struct ScmModuleRec         ScmModule;
+typedef struct ScmSymbolRec         ScmSymbol;
+typedef struct ScmGlocRec           ScmGloc;
+typedef struct ScmProcedureRec      ScmProcedure;
+typedef struct ScmClosureRec        ScmClosure;
+typedef struct ScmSubrRec           ScmSubr;
+typedef struct ScmGenericRec        ScmGeneric;
+typedef struct ScmMethodRec         ScmMethod;
+typedef struct ScmNextMethodRec     ScmNextMethod;
+typedef struct ScmSyntaxRec         ScmSyntax;
+typedef struct ScmMacroRec          ScmMacro;
+typedef struct ScmPromiseRec        ScmPromise;
+typedef struct ScmRegexpRec         ScmRegexp;
+typedef struct ScmRegMatchRec       ScmRegMatch;
 typedef struct ScmWriteControlsRec  ScmWriteControls;  /* see writerP.h */
 typedef struct ScmWriteContextRec   ScmWriteContext;   /* see writerP.h */
 typedef struct ScmWriteStateRec     ScmWriteState;     /* see wrtierP.h */
-typedef struct ScmAutoloadRec  ScmAutoload;
-typedef struct ScmComparatorRec ScmComparator;
-typedef struct ScmDLObjRec     ScmDLObj;           /* see load.c */
-typedef struct ScmReadContextRec ScmReadContext;   /* see read.c */
+typedef struct ScmAutoloadRec       ScmAutoload;
+typedef struct ScmComparatorRec     ScmComparator;
+typedef struct ScmDLObjRec          ScmDLObj;          /* see load.c */
+typedef struct ScmReadContextRec    ScmReadContext;    /* see read.c */
 
 typedef ScmObj ScmSubrProc(ScmObj *, int, void*);
 
@@ -599,6 +643,7 @@ SCM_EXTERN ScmObj Scm_Values4(ScmObj val0, ScmObj val1, ScmObj val2,
                               ScmObj val3);
 SCM_EXTERN ScmObj Scm_Values5(ScmObj val0, ScmObj val1, ScmObj val2,
                               ScmObj val3, ScmObj val4);
+SCM_EXTERN ScmObj Scm_ValuesFromArray(ScmObj *argv, ScmSmallInt argc);
 
 /* CPS API for evaluating Scheme fragments on VM. */
 SCM_EXTERN ScmObj Scm_VMApply(ScmObj proc, ScmObj args);
@@ -614,6 +659,7 @@ SCM_EXTERN ScmObj Scm_VMCall(ScmObj *args, int argcnt, void *data);
 
 SCM_EXTERN ScmObj Scm_VMCallCC(ScmObj proc);
 SCM_EXTERN ScmObj Scm_VMCallPC(ScmObj proc);
+SCM_EXTERN ScmObj Scm_VMReset(ScmObj proc);
 SCM_EXTERN ScmObj Scm_VMDynamicWind(ScmObj pre, ScmObj body, ScmObj post);
 SCM_EXTERN ScmObj Scm_VMDynamicWindC(ScmSubrProc *before,
                                      ScmSubrProc *body,
@@ -623,7 +669,7 @@ SCM_EXTERN ScmObj Scm_VMDynamicWindC(ScmSubrProc *before,
 SCM_EXTERN ScmObj Scm_VMWithErrorHandler(ScmObj handler, ScmObj thunk);
 SCM_EXTERN ScmObj Scm_VMWithGuardHandler(ScmObj handler, ScmObj thunk);
 SCM_EXTERN ScmObj Scm_VMWithExceptionHandler(ScmObj handler, ScmObj thunk);
-SCM_EXTERN ScmObj Scm_VMReraise(ScmObj condition);
+SCM_EXTERN ScmObj Scm_VMReraise();
 
 /* Miscellaneous stuff */
 SCM_EXTERN int    Scm_VMGetNumResults(ScmVM *vm);
@@ -632,9 +678,8 @@ SCM_EXTERN ScmObj Scm_VMGetStackLite(ScmVM *vm);
 SCM_EXTERN ScmObj Scm_VMGetCallTraceLite(ScmVM *vm);
 SCM_EXTERN ScmObj Scm_VMGetStack(ScmVM *vm);
 
-/* A box is to keep a reference.  It is mainly used for mutable local variables.
- */
-
+/* A box is to keep a reference.  Internally, it is used for mutable
+   local variables.  srfi-111 defines Scheme interface. */
 typedef struct ScmBoxRec {
     SCM_HEADER;
     ScmObj value;
@@ -649,6 +694,26 @@ SCM_CLASS_DECL(Scm_BoxClass);
 
 SCM_EXTERN ScmBox *Scm_MakeBox(ScmObj value);
 
+/* An mv-box is multi-valued box.  Srfi-195 extends srfi-111 to support
+   arbitrary number of values in a box.  We use a different type <mv-box>,
+   in order to keep the one-value box lightweight. */
+typedef struct ScmMVBoxRec {
+    SCM_HEADER;
+    ScmSmallInt size;
+    ScmObj values[1];            /* variable length */
+} ScmMVBox;
+
+SCM_CLASS_DECL(Scm_MVBoxClass);
+#define SCM_CLASS_MVBOX            (&Scm_MVBoxClass)
+#define SCM_MVBOX(obj)             ((ScmMVBox*)(obj))
+#define SCM_MVBOXP(obj)            (SCM_XTYPEP(obj, SCM_CLASS_MVBOX))
+#define SCM_MVBOX_SIZE(obj)        (SCM_MVBOX(obj)->size)
+#define SCM_MVBOX_VALUES(obj)      (SCM_MVBOX(obj)->values)
+#define SCM_MVBOX_SET(obj, k, val) (SCM_MVBOX(obj)->values[k] = (val))
+
+SCM_EXTERN ScmMVBox *Scm_MakeMVBox(ScmSmallInt size, ScmObj init);
+SCM_EXTERN ScmMVBox *Scm_ListToMVBox(ScmObj elts);
+
 /*---------------------------------------------------------
  * CLASS
  */
@@ -657,11 +722,17 @@ typedef void (*ScmClassPrintProc)(ScmObj obj,
                                   ScmPort *sink,
                                   ScmWriteContext *mode);
 typedef int  (*ScmClassCompareProc)(ScmObj x, ScmObj y, int equalp);
-typedef int  (*ScmClassSerializeProc)(ScmObj obj,
-                                      ScmPort *sink,
-                                      ScmObj context);
+typedef ScmSmallInt (*ScmClassHashProc)(ScmObj obj, ScmSmallInt salt,
+                                        u_long flags);
 typedef ScmObj (*ScmClassAllocateProc)(ScmClass *klass, ScmObj initargs);
 
+/* Flags value for ScmClassHashProc */
+enum {
+    SCM_HASH_PORTABLE = 1L<<0  /* must calculate a portable hash value,
+                                  can be used for portable-hash. */
+};
+    
+
 /* See class.c for the description of function pointer members.
    There's a lot of voodoo magic in class structure, so don't touch
    those fields casually.  Also, the order of these fields must be
@@ -677,9 +748,12 @@ struct ScmClassRec {
 #if defined(GAUCHE_BROKEN_LINKER_WORKAROUND)
     ScmClass **classPtr;
 #endif
+    /* Some type-specific primitive methods.  Note that these take precedence
+       than the generic function verison (write-object, object-compare etc.)
+    */
     ScmClassPrintProc     print;
     ScmClassCompareProc   compare;
-    ScmClassSerializeProc serialize;
+    ScmClassHashProc      hash;
     ScmClassAllocateProc  allocate;
     ScmClass **cpa;             /* class precedence array, NULL terminated */
     int numInstanceSlots;       /* # of instance slots */
@@ -919,6 +993,7 @@ extern ScmClass *Scm_ObjectCPL[];
         SCM_FALSE, /*redefined*/                 \
         SCM_INTERNAL_MUTEX_INITIALIZER,          \
         SCM_INTERNAL_COND_INITIALIZER,           \
+        NULL       /* data */                    \
     } SCM__CLASS_PTR_BODY(cname)
     
 /* Define built-in class statically -- full-featured version */
@@ -1018,23 +1093,31 @@ SCM_EXTERN ScmObj Scm_ForeignPointerAttrSet(ScmForeignPointer *fp,
 
 #include <gauche/collection.h>
 
+/*--------------------------------------------------------
+ * CONNECTION INTERFACE
+ */
+
+SCM_CLASS_DECL(Scm_ConnectionClass);
+#define SCM_CLASS_CONNECTION         (&Scm_ConnectionClass)
+
 /*--------------------------------------------------------
  * PAIR AND LIST
  */
 
-/* Ordinary pair uses two words.  It can be distinguished from
+/* An ordinary pair uses two words.  It can be distinguished from
  * other heap allocated objects by checking the first word doesn't
- * have "11" in the lower bits.
+ * have "111" in the lower bits.
  */
 struct ScmPairRec {
     ScmObj car;                 /* should be accessed via macros */
     ScmObj cdr;                 /* ditto */
 };
 
-/* To keep extra information such as source-code info, some pairs
- * actually have one extra word for attribute assoc-list.  Checking
- * whether a pair is an extended one or not isn't a very lightweight
- * operation, so the use of extended pair should be kept minimal.
+/* An extended pair behaves like an ordinary pair for read operations,
+ * but can keep extra information in attributes.  It also has
+ * hidden field, and can behave differently on mutating operations.
+ * Immutable pairs are implemented on that mechanism.
+ * See priv/pairP.h for the real structure of an extended pair.
  */
 struct ScmExtendedPairRec {
     ScmObj car;                 /* should be accessed via macros */
@@ -1042,12 +1125,13 @@ struct ScmExtendedPairRec {
     ScmObj attributes;          /* should be accessed via API func. */
 };
 
-#if GAUCHE_LAZY_PAIR
-#define SCM_PAIRP(obj)  \
-    (SCM_HPTRP(obj)&&(SCM_HTAG(obj)!=7||Scm_PairP(SCM_OBJ(obj))))
-#else  /*!GAUCHE_LAZY_PAIR*/
-#define SCM_PAIRP(obj)          (SCM_HPTRP(obj)&&(SCM_HTAG(obj)!=7))
-#endif /*!GAUCHE_LAZY_PAIR*/
+#if GAUCHE_CHECK_PAIR_ALIGNMENT
+#  define SCM_PAIRP(obj)  (Scm_CheckingPairP(SCM_OBJ(obj)))
+SCM_EXTERN int Scm_CheckingPairP(ScmObj);
+#else
+#  define SCM_PAIRP(obj)                                                  \
+     (SCM_HPTRP(obj)&&(SCM_HTAG(obj)!=7||Scm_PairP(SCM_OBJ(obj))))
+#endif
 
 #define SCM_PAIR(obj)           ((ScmPair*)(obj))
 #define SCM_CAR(obj)            (SCM_PAIR(obj)->car)
@@ -1057,11 +1141,26 @@ struct ScmExtendedPairRec {
 #define SCM_CDAR(obj)           (SCM_CDR(SCM_CAR(obj)))
 #define SCM_CDDR(obj)           (SCM_CDR(SCM_CDR(obj)))
 
-#define SCM_SET_CAR(obj, value) (SCM_CAR(obj) = (value))
-#define SCM_SET_CDR(obj, value) (SCM_CDR(obj) = (value))
+#define SCM_SET_CAR(obj, value) Scm_SetCar(obj, value)
+#define SCM_SET_CDR(obj, value) Scm_SetCdr(obj, value)
 
+/* Use these only if you know OBJ is a mutable pair */
+#define SCM_SET_CAR_UNCHECKED(obj, value) (SCM_CAR(obj) = (value))
+#define SCM_SET_CDR_UNCHECKED(obj, value) (SCM_CDR(obj) = (value))
+
+#if SIZEOF_INTPTR_T == 4
+#define SCM_ODD_WORD_POINTER_P(p) (SCM_WORD(p) & 0x4)
+#else /*SIZEOF_INTPTR_T == 8*/
+#define SCM_ODD_WORD_POINTER_P(p) (SCM_WORD(p) & 0x8)
+#endif
+
+#if SCM_PAIR_ALWAYS_ALIGNED_EVEN_WORDS
 #define SCM_EXTENDED_PAIR_P(obj) \
-    (SCM_PAIRP(obj)&&GC_base(obj)&&GC_size(obj)>=sizeof(ScmExtendedPair))
+    (SCM_ODD_WORD_POINTER_P(obj)&&SCM_PAIRP(obj))
+#else  /*!SCM_PAIR_ALWAYS_ALIGNED_EVEN_WORDS*/
+#define SCM_EXTENDED_PAIR_P(obj) \
+    (SCM_ODD_WORD_POINTER_P(obj)&&SCM_PAIRP(obj)&&SCM_HOBJP(((ScmObj*)(obj))-1))
+#endif /*!SCM_PAIR_ALWAYS_ALIGNED_EVEN_WORDS*/
 #define SCM_EXTENDED_PAIR(obj)  ((ScmExtendedPair*)(obj))
 
 
@@ -1121,13 +1220,15 @@ enum {
 
 SCM_EXTERN ScmObj Scm_Cons(ScmObj car, ScmObj cdr);
 SCM_EXTERN ScmObj Scm_Acons(ScmObj caar, ScmObj cdar, ScmObj cdr);
+SCM_EXTERN ScmObj Scm_MakeImmutablePair(ScmObj car, ScmObj cdr);
 SCM_EXTERN ScmObj Scm_List(ScmObj elt, ...);
 SCM_EXTERN ScmObj Scm_Conses(ScmObj elt, ...);
 SCM_EXTERN ScmObj Scm_VaList(va_list elts);
 SCM_EXTERN ScmObj Scm_VaCons(va_list elts);
-SCM_EXTERN ScmObj Scm_ArrayToList(ScmObj *elts, int nelts);
-SCM_EXTERN ScmObj Scm_ArrayToListWithTail(ScmObj *elts, int nelts, ScmObj tail);
-SCM_EXTERN ScmObj *Scm_ListToArray(ScmObj list, int *nelts, ScmObj *store,
+SCM_EXTERN ScmObj Scm_ArrayToList(ScmObj *elts, ScmSize nelts);
+SCM_EXTERN ScmObj Scm_ArrayToListWithTail(ScmObj *elts, ScmSize nelts,
+                                          ScmObj tail);
+SCM_EXTERN ScmObj *Scm_ListToArray(ScmObj list, ScmSize *nelts, ScmObj *store,
                                    int alloc);
 
 SCM_EXTERN ScmObj Scm_Car(ScmObj obj);
@@ -1137,7 +1238,11 @@ SCM_EXTERN ScmObj Scm_Cadr(ScmObj obj);
 SCM_EXTERN ScmObj Scm_Cdar(ScmObj obj);
 SCM_EXTERN ScmObj Scm_Cddr(ScmObj obj);
 
-SCM_EXTERN int    Scm_Length(ScmObj obj);
+SCM_EXTERN int    Scm_ImmutablePairP(ScmObj obj);
+SCM_EXTERN void   Scm_SetCar(ScmObj pair, ScmObj value);
+SCM_EXTERN void   Scm_SetCdr(ScmObj pair, ScmObj value);
+
+SCM_EXTERN ScmSize Scm_Length(ScmObj obj);
 SCM_EXTERN ScmObj Scm_CopyList(ScmObj list);
 SCM_EXTERN ScmObj Scm_MakeList(ScmSmallInt len, ScmObj fill);
 SCM_EXTERN ScmObj Scm_Append2X(ScmObj list, ScmObj obj);
@@ -1166,17 +1271,20 @@ SCM_EXTERN ScmObj Scm_AssocDeleteX(ScmObj elt, ScmObj alist, int cmpmode);
 SCM_EXTERN ScmObj Scm_DeleteDuplicates(ScmObj list, int cmpmode);
 SCM_EXTERN ScmObj Scm_DeleteDuplicatesX(ScmObj list, int cmpmode);
 
-SCM_EXTERN ScmObj Scm_MonotonicMerge(ScmObj start, ScmObj sequences);
-SCM_EXTERN ScmObj Scm_MonotonicMerge1(ScmObj sequences);
-SCM_EXTERN ScmObj Scm_Union(ScmObj list1, ScmObj list2);
-SCM_EXTERN ScmObj Scm_Intersection(ScmObj list1, ScmObj list2);
-
 SCM_EXTERN ScmObj Scm_MakeExtendedPair(ScmObj car, ScmObj cdr, ScmObj attrs);
 SCM_EXTERN ScmObj Scm_ExtendedCons(ScmObj car, ScmObj cdr);
 SCM_EXTERN ScmObj Scm_PairAttr(ScmPair *pair);
 SCM_EXTERN ScmObj Scm_PairAttrGet(ScmPair *pair, ScmObj key, ScmObj fallback);
 SCM_EXTERN ScmObj Scm_PairAttrSet(ScmPair *pair, ScmObj key, ScmObj value);
 
+#if GAUCHE_API_VERSION >= 1000
+SCM_EXTERN ScmObj Scm_MonotonicMerge(ScmObj sequences);
+#define Scm_MonotonicMerge1(x) Scm_MonotonicMerge(x)
+#else  /* GAUCHE_API_VERSION < 1000 */
+SCM_EXTERN ScmObj Scm_MonotonicMerge(ScmObj start, ScmObj sequences);
+SCM_EXTERN ScmObj Scm_MonotonicMerge1(ScmObj sequences);
+#endif /* GAUCHE_API_VERSION < 1000 */
+
 /*--------------------------------------------------------
  * CHARACTERS
  */
@@ -1313,7 +1421,7 @@ struct ScmProcedureRec {
    and all excessive arguments are 'folded' in a list.
 
    This slot may have a value more than 1.  If it is N (>1), then up to N-1
-   optional arguments are passed wihtout being folded (that is, passed
+   optional arguments are passed without being folded (that is, passed
    'on the stack'.  Only when the given argument is more than or equal to
    N + reqargs, the excessive arguments are folded and passed in a list.
    Thus, such procedure may get between reqargs values and N+reqargs values
@@ -1327,14 +1435,23 @@ struct ScmProcedureRec {
  */
 
 /* About 'constant' flag:
-   This flag being TRUE means this procedure returns the same constant
-   value if given same constant arguments, and it does not have any other
-   external effects.   The compiler may use this info to replace a call
-   of this proc with the resulting value, if all the arguments are known
-   at compile-time.  The resulting value must be serializable to the
+
+   For a <procedure> and <method>, this flag being TRUE means it returns
+   the same constant value if given same constant arguments, and it does
+   not have any other external effects.   The compiler may use this info
+   to replace a call of this proc with the resulting value, 
+   if all the arguments are known at compile-time.  
+   The resulting value must be serializable to the
    precompiled file.  The result shouldn't be affected
    by the timing of the compile, architecture on which the compiler runs,
    or the compiler configuration (e.g. internal encoding).
+
+   If <generic> has this flag, it tells the compiler that it can calculate
+   applicable method at the compile time.  It is independent from method's
+   constantness---the selected method may or may not be used as a compile-time
+   calculation; but it is safe to pre-select that method, given that
+   enough information is available at the compile time.
+   We warn if a new method is added to a 'constant' generic.
  */
 
 /* About 'leaf' flag:
@@ -1368,6 +1485,11 @@ struct ScmProcedureRec {
                    insert reference to the proc, for it may not be
                    serializable for AOT compilation).
 
+           - A <primitive-parameter> or <parameter> object.  R7RS requires
+             parameters to be a procedure, responding #t to procedure?.
+             We need to adapt Gauche parameter into that, saving the
+             actual parameter instance here.
+
            - Subr's name, as a string or a symbol.  This is the old format.
              It may also the case that subr is created from C function
              Scm_MakeSubr(), for it's cumbersome in C routine to construct
@@ -1437,6 +1559,7 @@ enum ScmProcedureType {
 #define SCM_PROCEDURE_REQUIRED(obj) SCM_PROCEDURE(obj)->required
 #define SCM_PROCEDURE_OPTIONAL(obj) SCM_PROCEDURE(obj)->optional
 #define SCM_PROCEDURE_TYPE(obj)     SCM_PROCEDURE(obj)->type
+#define SCM_PROCEDURE_CONSTANT(obj) SCM_PROCEDURE(obj)->constant
 #define SCM_PROCEDURE_CURRYING(obj) SCM_PROCEDURE(obj)->currying
 #define SCM_PROCEDURE_INFO(obj)     SCM_PROCEDURE(obj)->info
 #define SCM_PROCEDURE_SETTER(obj)   SCM_PROCEDURE(obj)->setter
@@ -1471,7 +1594,7 @@ SCM_CLASS_DECL(Scm_ProcedureClass);
 
 /* This is internal - should never be used directly */
 #define SCM__PROCEDURE_INITIALIZER(klass, req, opt, typ, cst, lef, inf, inl) \
-    { { klass }, (req), (opt), (typ), FALSE, FALSE, cst, lef, 0,             \
+    { { klass, NULL }, (req), (opt), (typ), FALSE, FALSE, cst, lef, 0,       \
       (inf), SCM_FALSE, (inl) }
 
 SCM_EXTERN ScmObj Scm_CopyProcedure(ScmProcedure *proc);
@@ -1528,11 +1651,6 @@ struct ScmSubrRec {
 #define SCM_DEFINE_SUBRX(cvar, req, opt, cst, inf, flags, func, inliner, data) \
     SCM__DEFINE_SUBR_INT(cvar, req, opt, cst, inf, flags, func, inliner, data)
 
-/* TRANSIENT: old interface.  will be gone. */
-#define SCM_DEFINE_SUBRI(cvar, req, opt, inf, func, inliner, data) \
-    SCM__DEFINE_SUBR_INT(cvar, req, opt, 0, inf, SCM_SUBR_IMMEDIATE_ARG, \
-                         func, inliner, data)
-
 SCM_EXTERN ScmObj Scm_MakeSubr(ScmSubrProc *func,
                                void *data,
                                int required, int optional,
@@ -1551,10 +1669,9 @@ struct ScmGenericRec {
     int   maxReqargs;           /* maximum # of args required to select
                                    applicable methods */
     ScmObj (*fallback)(ScmObj *argv, int argc, ScmGeneric *gf);
+    void *dispatcher;
     void *data;
     ScmInternalMutex lock;
-    void *dispatcher;           /* To accelerate dispatching. See dispatch.c
-                                   TRANSIENT: Move this up in 1.0 release */
 };
 
 SCM_CLASS_DECL(Scm_GenericClass);
@@ -1563,14 +1680,16 @@ SCM_CLASS_DECL(Scm_GenericClass);
 #define SCM_GENERIC(obj)           ((ScmGeneric*)obj)
 #define SCM_GENERIC_DATA(obj)      (SCM_GENERIC(obj)->data)
 
+/* we share 'constant' flag for sealed generic */
+#define SCM_GENERIC_SEALED_P(obj)  SCM_PROCEDURE_CONSTANT(obj)
+
 #define SCM_DEFINE_GENERIC(cvar, cfunc, data)                           \
     ScmGeneric cvar = {                                                 \
         SCM__PROCEDURE_INITIALIZER(SCM_CLASS_STATIC_TAG(Scm_GenericClass),\
                                    0, 0, SCM_PROC_GENERIC, 0, 0,        \
                                    SCM_FALSE, NULL),                    \
-        SCM_NIL, 0, cfunc, data,                                        \
-        SCM_INTERNAL_MUTEX_INITIALIZER,                                 \
-        NULL                                                            \
+        SCM_NIL, 0, cfunc, NULL, data,                                  \
+        SCM_INTERNAL_MUTEX_INITIALIZER                                  \
     }
 
 SCM_EXTERN void Scm_InitBuiltinGeneric(ScmGeneric *gf, const char *name,
@@ -1675,24 +1794,13 @@ SCM_EXTERN ScmObj Scm_Map(ScmObj proc, ScmObj arg1, ScmObj args);
 SCM_CLASS_DECL(Scm_SyntaxClass);
 #define SCM_CLASS_SYNTAX            (&Scm_SyntaxClass)
 
-SCM_EXTERN ScmObj Scm_MakeSyntax(ScmSymbol *name, ScmObj handler);
-
 #define SCM_MACRO(obj)             ((ScmMacro*)(obj))
 #define SCM_MACROP(obj)            SCM_XTYPEP(obj, SCM_CLASS_MACRO)
 SCM_CLASS_DECL(Scm_MacroClass);
 #define SCM_CLASS_MACRO            (&Scm_MacroClass)
 
-/* TRANSIENT: Scm_MakeMacroFull is to keep ABI compatibility.  Will be gone
-   in 1.0.  */
-#if GAUCHE_API_0_95
 SCM_EXTERN ScmObj Scm_MakeMacro(ScmObj name, ScmObj transformer,
                                 ScmObj src, ScmObj describer);
-#define Scm_MakeMacroFull(a,b,c,d) Scm_MakeMacro(a,b,c,d)
-#else
-SCM_EXTERN ScmObj Scm_MakeMacroFull(ScmObj name, ScmObj transformer,
-                                    ScmObj src, ScmObj describer);
-ScmObj Scm_MakeMacro(ScmSymbol *name, ScmObj transformer);
-#endif
 SCM_EXTERN ScmObj Scm_MacroTransformer(ScmMacro *mac);
 SCM_EXTERN ScmObj Scm_MacroName(ScmMacro *mac);
 
@@ -1701,7 +1809,13 @@ SCM_EXTERN ScmObj Scm_MakeMacroTransformer(ScmSymbol *name,
 SCM_EXTERN ScmObj Scm_MakeMacroAutoload(ScmSymbol *name,
                                         ScmAutoload *al);
 
+#if GAUCHE_API_VERSION >= 1000
+SCM_EXTERN ScmObj Scm_UnwrapSyntax(ScmObj form, int immutablep);
+#define Scm_UnwrapSyntax2(form, imm) Scm_UnwrapSyntax(form, imm)
+#else  /* GAUCHE_API_VERSION < 1000 */
 SCM_EXTERN ScmObj Scm_UnwrapSyntax(ScmObj form);
+SCM_EXTERN ScmObj Scm_UnwrapSyntax2(ScmObj form, int immutablep);
+#endif /* GAUCHE_API_VERSION < 1000 */
 
 /*--------------------------------------------------------
  * PROMISE
@@ -1719,9 +1833,10 @@ SCM_CLASS_DECL(Scm_PromiseClass);
 #define SCM_PROMISEP(obj)           SCM_XTYPEP(obj, SCM_CLASS_PROMISE)
 
 SCM_EXTERN ScmObj Scm_MakePromise(int forced, ScmObj code);
+SCM_EXTERN ScmObj Scm_VMForce(ScmObj p); /* CPS, lightweight */
 SCM_EXTERN ScmObj Scm_Force(ScmObj p);
 
-/* Lazy pair structur is opaque to public.  Whenever you apply to an
+/* Lazy pair structure is opaque to public.  Whenever you apply to an
    ScmObj SCM_PAIRP, a lazy pair morphs itself to a pair, so the normal
    code never see lazy pairs. */
 
@@ -1748,15 +1863,23 @@ enum {
     SCM_PORT_ERROR_OUTPUT,
     SCM_PORT_ERROR_CLOSED,
     SCM_PORT_ERROR_UNIT,
+    SCM_PORT_ERROR_DECODING,
+    SCM_PORT_ERROR_ENCODING,
+    SCM_PORT_ERROR_SEEK,
+    SCM_PORT_ERROR_INVALID_POSITION,
     SCM_PORT_ERROR_OTHER
 };
 
 /* Throwing error */
-SCM_EXTERN void Scm_Error(const char *msg, ...);
-SCM_EXTERN void Scm_SysError(const char *msg, ...);
+SCM_EXTERN void Scm_Error(const char *msg, ...) SCM_NORETURN;
+SCM_EXTERN void Scm_SysError(const char *msg, ...) SCM_NORETURN;
 SCM_EXTERN void Scm_TypeError(const char *what,
-                              const char *expected, ScmObj got);
-SCM_EXTERN void Scm_PortError(ScmPort *port, int reason, const char *msg, ...);
+                              const char *expected, ScmObj got) SCM_NORETURN;
+SCM_EXTERN void Scm_PortError(ScmPort *port, int reason, 
+                              const char *msg, ...) SCM_NORETURN;
+SCM_EXTERN void Scm_PortErrorWithAux(ScmPort *port, int reason,
+                                     ScmObj auxinfo,
+                                     const char *msg, ...) SCM_NORETURN;
 
 /* common pattern */
 #define SCM_TYPE_ERROR(arg, expected)  Scm_TypeError(#arg, expected, arg)
@@ -1764,15 +1887,7 @@ SCM_EXTERN void Scm_PortError(ScmPort *port, int reason, const char *msg, ...);
 SCM_EXTERN void Scm_Warn(const char *msg, ...);
 SCM_EXTERN void Scm_FWarn(ScmString *fmt, ScmObj args);
 
-/* TRANSIENT: Scm_Raise2 is to keep ABI compatibility.  Will be gone
-   in 1.0.  */
-#if    GAUCHE_API_0_95
 SCM_EXTERN ScmObj Scm_Raise(ScmObj exception, u_long flags);
-#define Scm_Raise2(e, f)  Scm_Raise(e, f)
-#else  /*!GAUCHE_API_0_95*/
-SCM_EXTERN ScmObj Scm_Raise(ScmObj exception);
-SCM_EXTERN ScmObj Scm_Raise2(ScmObj exception, u_long flags);
-#endif /*!GAUCHE_API_0_95*/
 
 /* flags for Scm_Raise */
 enum {
@@ -1801,14 +1916,7 @@ SCM_EXTERN void Scm_ShowStackTrace(ScmPort *out, ScmObj stacklite,
 
 SCM_EXTERN void Scm_SetCallTraceSize(u_long size);
 
-/* TRANSIENT: Scm_ReportErrr2 is to keep ABI compatibility.  Will be gone
-   in 1.0.  */
-#if    GAUCHE_API_0_95
 SCM_EXTERN ScmObj Scm_ReportError(ScmObj e, ScmObj out);
-#else  /*!GAUCHE_API_0_95*/
-SCM_EXTERN ScmObj Scm_ReportError(ScmObj e);
-SCM_EXTERN ScmObj Scm_ReportError2(ScmObj e, ScmObj out);
-#endif /*!GAUCHE_API_0_95*/
 
 /*--------------------------------------------------------
  * REGEXP
@@ -1825,11 +1933,19 @@ SCM_CLASS_DECL(Scm_RegexpClass);
 /* flags */
 #define SCM_REGEXP_CASE_FOLD      (1L<<0)
 #define SCM_REGEXP_PARSE_ONLY     (1L<<1)
+/* bits 2 and 3 are used internally */
+#define SCM_REGEXP_MULTI_LINE     (1L<<4)
 
 SCM_EXTERN ScmObj Scm_RegComp(ScmString *pattern, int flags);
+#if GAUCHE_API_VERSION >= 1000
+SCM_EXTERN ScmObj Scm_RegCompFromAST(ScmObj ast, int flags);
+#define Scm_RegCompFromAST2(a,b) Scm_RegCompFromAST(a,b)
+#else   /* GAUCHE_API_VERSION < 1000 */
 SCM_EXTERN ScmObj Scm_RegCompFromAST(ScmObj ast);
+SCM_EXTERN ScmObj Scm_RegCompFromAST2(ScmObj ast, int flags);
+#endif  /* GAUCHE_API_VERSION < 1000 */
 SCM_EXTERN ScmObj Scm_RegOptimizeAST(ScmObj ast);
-SCM_EXTERN ScmObj Scm_RegExec(ScmRegexp *rx, ScmString *input);
+SCM_EXTERN ScmObj Scm_RegExec(ScmRegexp *rx, ScmString *input, ScmObj start, ScmObj end);
 SCM_EXTERN void Scm_RegDump(ScmRegexp *rx);
 
 SCM_CLASS_DECL(Scm_RegMatchClass);
@@ -1888,8 +2004,10 @@ SCM_EXTERN void   Scm_SetMasterSigmask(sigset_t *set);
 SCM_EXTERN ScmObj Scm_SignalName(int signum);
 SCM_EXTERN void   Scm_ResetSignalHandlers(sigset_t *mask);
 
+#if GAUCHE_API_VERSION < 1000
 SCM_EXTERN void   Scm_GetSigmask(sigset_t *mask);
 SCM_EXTERN void   Scm_SetSigmask(sigset_t *mask);
+#endif /*GAUCHE_API_VERSION < 1000*/
 
 /*---------------------------------------------------
  * SYSTEM
@@ -1923,7 +2041,22 @@ SCM_EXTERN void Scm_Cleanup(void);
 SCM_EXTERN void Scm_Exit(int code) SCM_NORETURN;
 SCM_EXTERN void Scm_Abort(const char *msg) SCM_NORETURN;
 SCM_EXTERN void Scm_Panic(const char *msg, ...) SCM_NORETURN;
+
+/* 'kind' argument of Scm_InitCommandLine */
+enum {
+    SCM_COMMAND_LINE_SCRIPT = 1,    /* for (command-line) */
+    SCM_COMMAND_LINE_OS = 2,        /* for (os-command-line) */
+    SCM_COMMAND_LINE_BOTH = (SCM_COMMAND_LINE_SCRIPT|SCM_COMMAND_LINE_OS)
+};
+
+#if GAUCHE_API_VERSION >= 1000
+SCM_EXTERN ScmObj Scm_InitCommandLine(int argc, const char *argv[],
+                                      int kind);
+#define Scm_InitCommandLine2(ac, av, kind) Scm_InitCommandLine(ac, av, kind)
+#else  /* GAUCHE_API_VERSION < 1000 */
 SCM_EXTERN ScmObj Scm_InitCommandLine(int argc, const char *argv[]);
+SCM_EXTERN ScmObj Scm_InitCommandLine2(int argc, const char *argv[], int kind);
+#endif /* GAUCHE_API_VERSION < 1000 */
 
 SCM_EXTERN void Scm_SimpleMain(int argc, const char *argv[],
                                const char *script, u_long flags);
@@ -1936,6 +2069,7 @@ SCM_EXTERN void Scm_GCSentinel(void *obj, const char *name);
 
 SCM_EXTERN ScmObj Scm_GetFeatures(void);
 SCM_EXTERN void   Scm_AddFeature(const char *feature, const char *mod);
+SCM_EXTERN void   Scm_DisableFeature(const char *feature);
 
 SCM_EXTERN void *Scm_AddCleanupHandler(void (*proc)(void *data), void *data);
 SCM_EXTERN void  Scm_DeleteCleanupHandler(void *handle);
@@ -1951,7 +2085,9 @@ SCM_EXTERN ScmObj Scm_LibraryDirectory(void);
 SCM_EXTERN ScmObj Scm_ArchitectureDirectory(void);
 SCM_EXTERN ScmObj Scm_SiteLibraryDirectory(void);
 SCM_EXTERN ScmObj Scm_SiteArchitectureDirectory(void);
-SCM_EXTERN ScmObj Scm__RuntimeDirectory(void); /* private */
+SCM_EXTERN ScmObj Scm_RuntimeDirectory(void); /* may return SCM_FALSE */
+SCM_EXTERN ScmObj Scm_LibgauchePath(void);    /* may return SCM_FALSE */
+SCM_EXTERN ScmObj Scm_ExecutablePath(void);   /* may return SCM_FALSE */
 
 /* Compare and Sort */
 
@@ -1986,6 +2122,7 @@ SCM_EXTERN ScmObj Scm__RuntimeDirectory(void); /* private */
 #endif /* !GAUCHE_RECKLESS */
 
 #include <gauche/scmconst.h>
+#include <gauche/endian.h>
 
 SCM_DECL_END
 
