@@ -1,7 +1,7 @@
 /*
  * charset.h - Character set implementation
  *
- *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -52,12 +52,6 @@
  * Mutable char-set uses ScmTreeCore for the large characters.  Immutable
  * char-set uses flat u32vector, sorted by the key and accessed by binary
  * search.
- *
- * Immutable charset is a new addition.  Since we have to keep ABI
- * compatibility, the layout is a bit weird during 0.9 series;
- * the existing binary checks Scm_TreeCoreNumEntries(cs->large.tree) == 0
- * for SCM_CHAR_SET_SMALLP, so after we introduce immutable set, we have
- * to make sure immutable set 
  */
 
 #define SCM_CHAR_SET_SMALL_CHARS 128
@@ -69,7 +63,7 @@ struct ScmCharSetRec {
     union {
         ScmTreeCore tree;
         struct {
-            size_t size;      /* size of vec.  # of entries is half of this */
+            ScmSize size; /* size of vec.  # of entries is half of this */
             const uint32_t *vec; 
             uint32_t ivec[2]; /* if size==2, vec points here */
         } frozen;
@@ -113,29 +107,9 @@ SCM_EXTERN ScmObj Scm_CharSetCaseFold(ScmCharSet *cs);
 SCM_EXTERN ScmObj Scm_CharSetRanges(ScmCharSet *cs);
 SCM_EXTERN ScmObj Scm_CharSetRead(ScmPort *input, int *complement_p,
                                   int error_p, int bracket_syntax);
+SCM_EXTERN int    Scm_CharSetParseCategory(ScmPort *input, char ch);
 
 SCM_EXTERN int    Scm_CharSetContains(ScmCharSet *cs, ScmChar c);
 SCM_EXTERN void   Scm_CharSetDump(ScmCharSet *cs, ScmPort *port);
 
-/* predefined character set API */
-enum {
-    SCM_CHAR_SET_ALNUM,
-    SCM_CHAR_SET_ALPHA,
-    SCM_CHAR_SET_BLANK,
-    SCM_CHAR_SET_CNTRL,
-    SCM_CHAR_SET_DIGIT,
-    SCM_CHAR_SET_GRAPH,
-    SCM_CHAR_SET_LOWER,
-    SCM_CHAR_SET_PRINT,
-    SCM_CHAR_SET_PUNCT,
-    SCM_CHAR_SET_SPACE,
-    SCM_CHAR_SET_UPPER,
-    SCM_CHAR_SET_XDIGIT,
-    SCM_CHAR_SET_WORD,           /* internal use: word constituent char. */
-    SCM_CHAR_SET_NUM_PREDEFINED_SETS
-};
-SCM_EXTERN ScmObj Scm_GetStandardCharSet(int id);
-
-
-
 #endif /*GAUCHE_CHARSET_H*/
