@@ -1,7 +1,7 @@
 /*
  * macro.h - structures used internally in macro expander
  *
- *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -41,9 +41,14 @@ SCM_DECL_BEGIN
 struct ScmSyntaxRec {
     SCM_HEADER;
     ScmSymbol *name;         /* for debugging.  can be NULL */
+    ScmModule *mod;          /* for debugging.  can be NULL */
     ScmObj     handler;      /* syntax handler.  (Sexpr, Env) -> IForm */
 };
 
+SCM_EXTERN ScmObj Scm_MakeSyntax(ScmSymbol *name,
+                                 ScmModule *mod,
+                                 ScmObj handler);
+
 /* Macro */
 struct ScmMacroRec {
     SCM_HEADER;
