@@ -1,7 +1,7 @@
 /*
  * reader.h - Reader API
  *
- *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -66,9 +66,10 @@ SCM_EXTERN ScmObj Scm_ReaderLexicalMode(void);
 SCM_EXTERN ScmObj Scm_SetReaderLexicalMode(ScmObj);
 
 /* An object to keep unrealized circular reference (e.g. #N=) during
- * 'read'.  It is replaced by the reference value before exitting 'read',
+ * 'read'.  It is replaced by the reference value before exiting 'read',
  * and it shouldn't leak out to the normal Scheme program, except the
  * code that handles it explicitly (like read-time constructor).
+ * This object is also used in Scm_UnwrapSyntax to track circular structures.
  */
 typedef struct ScmReadReferenceRec {
     SCM_HEADER;
@@ -109,9 +110,9 @@ SCM_EXTERN ScmObj  Scm_ReadXdigitsFromPort(ScmPort *port, int key, ScmObj mode,
 
 SCM_EXTERN long Scm_ReadDigitsAsLong(ScmPort *port, ScmChar ch, int radix,
                                      ScmChar *next,
-                                     int *numread);
-SCM_EXTERN long Scm_ParseDigitsAsLong(const char *buf, size_t len, int radix,
-                                      int *numread);
+                                     ScmSize *numread);
+SCM_EXTERN long Scm_ParseDigitsAsLong(const char *buf, ScmSize len, int radix,
+                                      ScmSize *numread);
 
 /*
  * SRFI-10 hash-comma syntax
