@@ -1,7 +1,7 @@
 /*
  * vm.h - Virtual machine
  *
- *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -50,7 +50,6 @@
 #define GAUCHE_PROFILE
 #endif /* (defined(ITIMER_PROF) && defined(SIGPROF)) || defined(GAUCHE_WINDOWS) */
 
-
 #ifdef __GNUC__
 /* in GNUC, we inline Scm_VMReturnFlonum using statement expressions. */
 # if GAUCHE_FFX
@@ -69,13 +68,15 @@ SCM_EXTERN void Scm_VMFlushFPStack(ScmVM *vm);
 #   define Scm_VMReturnFlonum(val)  Scm_MakeFlonum(val)
 #endif /* !__GNUC__ */
 
-
 /* Actual structure is defined in code.h */
 typedef struct ScmCompiledCodeRec ScmCompiledCode;
 
 /* Actual structure is defined in priv/vmP.h */
 typedef struct ScmCallTraceRec ScmCallTrace;
 
+/* Actual structure is defined in priv/parameterP.h */
+typedef struct ScmVMParameterTableRec ScmVMParameterTable;
+
 /*
  * Environment frame
  *
@@ -113,6 +114,7 @@ typedef struct ScmEnvFrameRec {
  *   |  base  |
  *   |   pc   |
  *   |  cpc   |
+ *   | marker |
  *   | size=N |
  *   |  env   |
  *   |..prev..|<--- ScmContFrame* cont
@@ -131,6 +133,7 @@ typedef struct ScmContFrameRec {
     struct ScmContFrameRec *prev; /* previous frame */
     ScmEnvFrame *env;             /* saved environment */
     int size;                     /* size of argument frame */
+    int marker;                   /* end marker of partial continuation */
     SCM_PCTYPE cpc;               /* current PC (for debugging info) */
     SCM_PCTYPE pc;                /* next PC */
     ScmCompiledCode *base;        /* base register value */
@@ -226,90 +229,8 @@ typedef struct ScmCStackRec {
     sigset_t mask;
 } ScmCStack;
 
-/*
- * Escape point
- *
- *  EscapePoint (EP) structure keeps certain point of continuation chain
- *  where control can be transferred.   This structure is used for
- *  saved continuations, as well as error handlers.
- *
- *  Normally EPs forms a single list, linked by the prev pointer.
- *  vm->escapePoint points a 'current' EP, whose ehandler is the current
- *  error handler.
- *
- *  However, this simple structure does not work in all cases.
- *  When an error is signalled, we pop EP before executing the error
- *  handler so that an error raised within the error handler will be
- *  handled by the handler of outer EP.
- *
- *    Suppose the current EP is EP0.
- *
- *    (with-error-handler    ;; <- (1)this installs EP1. EP1->cont captures
- *                           ;;        one-shot continuation of this expr.
- *       (lambda (e) ...)    ;; <- (3)this is executed while EP0 is current
- *      (lambda () ...)      ;; <- (2)this is executed while EP1 is current
- *
- *  If the error handler returns, we pass its result to the continuation of
- *  with-error-handler, which is kept in EP1->cont.  The problem arises
- *  if a stack overflow occurs within the error handler, continuation frames
- *  in the stack are relocated to the heap, but EP1->cont isn't updated
- *  since it is out of vm->escapePoint chain.
- *
- *  'Floating' pointer is used to catch such case.  When an EP is popped
- *  before an error handler is ivoked, EP0's floating pointer is set to
- *  point EP1.  When a new EP is pushed, it inherits the previous EP's
- *  floating pointer.  With this scheme, active floating EPs are always
- *  reachable from vm->esapePoint->floating chain.  (NB: the chain length
- *  can be more than 1, if with-error-handler is used within an error handler
- *  and an error is signalled in its body.
- */
-typedef struct ScmEscapePointRec {
-    struct ScmEscapePointRec *prev;
-    struct ScmEscapePointRec *floating;
-    ScmObj ehandler;            /* handler closure */
-    ScmContFrame *cont;         /* saved continuation */
-    ScmObj handlers;            /* saved dynamic handler chain */
-    ScmCStack *cstack;          /* vm->cstack when escape point is created.
-                                   this will be used to rewind cstack.
-                                   this is NULL for partial continuations,
-                                   for they can be executed on anywhere
-                                   w.r.t. cstack. */
-    ScmObj xhandler;            /* saved exception handler */
-    int errorReporting;         /* state of SCM_VM_ERROR_REPORTING flag
-                                   when this ep is captured.  The flag status
-                                   should be restored when the control
-                                   transferred to this escape point. */
-    int rewindBefore;           /* EXPERIMENTAL: if TRUE, dynamic handlers
-                                   are rewound after an exception is raised
-                                   and before the exception handler is called.
-                                   If FALSE, the exception handler is called
-                                   first, then the dynamic handlers are
-                                   rewound.   SRFI-18 model and legacy
-                                   with-error-handler uses the latter model,
-                                   but SRFI-34's guard needs the former model.
-                                */
-    int reraised;               /* EXPERIMENTAL: if exception is reraised,
-                                   this flag is set to TRUE and the exception
-                                   handler can return to the caller. */
-} ScmEscapePoint;
-
-/* Link management */
-#define SCM_VM_FLOATING_EP(vm) \
-    ((vm)->escapePoint? (vm)->escapePoint->floating : vm->escapePointFloating)
-#define SCM_VM_FLOATING_EP_SET(vm, ep)          \
-    do {                                        \
-        if ((vm)->escapePoint) {                \
-            (vm)->escapePoint->floating = (ep); \
-        } else {                                \
-            (vm)->escapePointFloating = (ep);   \
-        }                                       \
-    } while (0)
-
-/* Escape types */
-#define SCM_VM_ESCAPE_NONE   0
-#define SCM_VM_ESCAPE_ERROR  1
-#define SCM_VM_ESCAPE_CONT   2
-#define SCM_VM_ESCAPE_EXIT   3
+/* ScmEscapePoint definition is in vmP.h */
+typedef struct ScmEscapePointRec ScmEscapePoint;
 
 /*
  * Signal queue
@@ -336,9 +257,6 @@ typedef struct ScmEscapePointRec {
 #define SCM_NSIG NSIG
 #endif /*!_SIG_MAXSIG*/
 
-/* The following #ifdef is to maintain binary compatibility during 0.9 series.
- */
-#if defined(GAUCHE_API_0_9)
 typedef struct ScmSignalQueueRec {
     union {
         unsigned char dummy[NSIG];
@@ -346,12 +264,6 @@ typedef struct ScmSignalQueueRec {
     };
     ScmObj pending;        /* pending signal handlers */
 } ScmSignalQueue;
-#else  /*!GAUCHE_API_0_9*/
-typedef union ScmSignalQueueRec {
-    unsigned char sigcounts[SCM_NSIG];
-    ScmObj pending;        /* pending signal handlers */
-} ScmSignalQueue;
-#endif /*!GAUCHE_API_0_9*/
 
 SCM_EXTERN void Scm_SignalQueueInit(ScmSignalQueue* q);
 SCM_EXTERN int  Scm_GetSignalPendingLimit(void);
@@ -441,8 +353,14 @@ struct ScmVMRec {
                                    is used only in compilation. */
     ScmCStack *cstack;          /* current escape point.  see the comment of
                                    "C stack rewinding" below. */
+    /* TRANSIENT: Change these flags to u_long in 1.0 */
+#if GAUCHE_API_VERSION < 1000    
     unsigned int runtimeFlags;  /* Runtime flags */
     unsigned int compilerFlags; /* Compiler flags */
+#else /*GAUCHE_API_VERSION >= 1000*/
+    u_long runtimeFlags;  /* Runtime flags */
+    u_long compilerFlags; /* Compiler flags */
+#endif /*GAUCHE_API_VERSION >= 1000*/
     intptr_t attentionRequest;  /* Flag if VM needs to check signal queue,
                                    finalizer queue, or stop request.
                                    This flag can be turned on asynchronously.
@@ -465,7 +383,7 @@ struct ScmVMRec {
     ScmPort *curin;             /* current input port */
     ScmPort *curout;            /* current output port */
     ScmPort *curerr;            /* current error port */
-    ScmVMParameterTable parameters; /* parameter table */
+    ScmVMParameterTable *parameters; /* parameter table */
 
     /* Registers */
     ScmCompiledCode *base;      /* Current executing closure's code packet. */
@@ -487,6 +405,14 @@ struct ScmVMRec {
     ScmObj *stack;              /* bottom of allocated stack area */
     ScmObj *stackBase;          /* base of current stack area  */
     ScmObj *stackEnd;           /* end of current stack area */
+#if GAUCHE_SPLIT_STACK
+    /* EXPERIMENTAL: Save the continuation when an error occurs, used for
+       better error diagnostics.  Reset by the "cross the border" APIs
+       such as Scm_Eval().  This can point into the stack, but must be
+       saved when save_cont() is called.
+     */
+    ScmContFrame *lastErrorCont;
+#endif /*GAUCHE_SPLIT_STACK*/
 
 #if GAUCHE_FFX
     ScmFlonum *fpsp;            /* flonum stack pointer.  we call it 'stack'
@@ -520,11 +446,6 @@ struct ScmVMRec {
                                    it per thread.
                                  */
 
-    /* TRANSIENT: For the ABI compatibility.  Remove on 1.0 */
-    ScmObj dummy0;
-    ScmObj dummy1;
-    ScmObj dummy2;
-
     /* Program information */
     int    evalSituation;       /* eval situation (related to eval-when) */
 
@@ -547,6 +468,15 @@ struct ScmVMRec {
                                    Set by vm_register. */
 
     ScmCallTrace *callTrace;
+
+    /* for reset/shift */
+    ScmObj resetChain;          /* list of reset information,
+                                   where reset information is
+                                   (delimited . <dynamic handlers chain>).
+                                   the delimited flag is set when 'shift'
+                                   appears in 'reset' and the end marker of
+                                   partial continuation is set. */
+
 };
 
 SCM_EXTERN ScmVM *Scm_NewVM(ScmVM *proto, ScmObj name);
@@ -554,15 +484,7 @@ SCM_EXTERN int    Scm_AttachVM(ScmVM *vm);
 SCM_EXTERN void   Scm_DetachVM(ScmVM *vm);
 SCM_EXTERN void   Scm_VMDump(ScmVM *vm);
 SCM_EXTERN ScmObj Scm_VMDefaultExceptionHandler(ScmObj exc);
-/* TRANSIENT: Scm_VMThrowException2 is to keep ABI compatibility.  Will be
-   gone in 1.0 */
-#if    GAUCHE_API_0_95
 SCM_EXTERN ScmObj Scm_VMThrowException(ScmVM *vm, ScmObj exc, u_long flags);
-#define Scm_VMThrowException2(v, e, f)  Scm_VMThrowException(v, e, f)
-#else  /*!GAUCHE_API_0_95*/
-SCM_EXTERN ScmObj Scm_VMThrowException(ScmVM *vm, ScmObj exc);
-SCM_EXTERN ScmObj Scm_VMThrowException2(ScmVM *vm, ScmObj exc, u_long flags);
-#endif /*!GAUCHE_API_0_95*/
 SCM_EXTERN ScmObj Scm_VMGetSourceInfo(ScmCompiledCode *code, SCM_PCTYPE pc);
 SCM_EXTERN ScmObj Scm_VMGetBindInfo(ScmCompiledCode *code, SCM_PCTYPE pc);
 SCM_EXTERN void   Scm_VMSetResult(ScmObj obj);
@@ -680,8 +602,12 @@ enum {
                                            module */
     SCM_COLLECT_VM_STATS     = (1L<<5), /* enable statistics collection
                                            (incurs runtime overhead) */
-    SCM_COLLECT_LOAD_STATS   = (1L<<6)  /* log the stats of file load
+    SCM_COLLECT_LOAD_STATS   = (1L<<6), /* log the stats of file load
                                            timings (incurs runtime overhead) */
+    SCM_CHECK_UNDEFINED_TEST = (1L<<7), /* check if #<undef> appears as
+                                           the test value in branch */
+    SCM_SAFE_STRING_CURSORS = (1L<<8)   /* Always use large cursors for
+                                           extra validation. */
 };
 
 #define SCM_VM_RUNTIME_FLAG_IS_SET(vm, flag) ((vm)->runtimeFlags & (flag))
@@ -717,8 +643,14 @@ enum {
     SCM_COMPILE_NO_LIFTING = (1L<<7),      /* Do not run lambda lifting pass
                                               (pass4). */
     SCM_COMPILE_INCLUDE_VERBOSE = (1L<<8), /* Report expansion of 'include' */
-    SCM_COMPILE_ENABLE_CEXPR = (1L<<9),    /* Support C-expressions by reader */
-    SCM_COMPILE_NOINLINE_SETTERS = (1L<<10) /* Do not inline setters */
+    SCM_COMPILE_NOINLINE_SETTERS = (1L<<9), /* Do not inline setters */
+    SCM_COMPILE_NODISSOLVE_APPLY = (1L<<10),/* Do not dissolve APPLY
+                                              (pass2/dissolve-apply) */
+    SCM_COMPILE_LEGACY_DEFINE = (1L<<11),  /* Do not insert toplevel binding
+                                              at compile-time. */
+    SCM_COMPILE_MUTABLE_LITERALS = (1L<<12),/* Literal pairs are mutable */
+    SCM_COMPILE_SRFI_FEATURE_ID = (1L<<13)  /* Allow srfi-N feature id in
+                                               cond-expand */
 };
 
 #define SCM_VM_COMPILER_FLAG_IS_SET(vm, flag) ((vm)->compilerFlags & (flag))
