@@ -51,6 +51,7 @@
 ;; needs to load from source
 (add-load-path "../lib" :relative)
 (use text.unicode.ucd)
+(use text.unicode.codeset)
 
 ;; rfc.http is only required by '--fetch' operation.  We don't want to load
 ;; it during build process, so let's autoload it.
@@ -177,7 +178,9 @@
 ;;              9    MidNumLet
 ;;              a    Numeric
 ;;              b    ExtendNumLet
-;;              c    Other
+;;              c    WSegSpace
+;;              d    ZWJ
+;;              e    Other
 ;;    bit3-0:  Graphene_Break property
 ;;             10    CR (not stored in the table)
 ;;             11    LF (not stored in the table)
@@ -191,7 +194,8 @@
 ;;              7    T
 ;;              8    LV
 ;;              9    LVT
-;;              a    Other
+;;              a    ZWJ
+;;              b    Other
 ;;
 ;;   Codepoints below U+20000 are looked up by two-staged tables.
 ;;   First, look up this table with (codepoint >> 8).
@@ -292,10 +296,11 @@
 (define (generate-category-tables db)
   (define (emit-entry e)
     (if e
-      (cond [(ucd-entry-uppercase e)  (format #t " U(~a)," (ucd-entry-category e))]
-            [(ucd-entry-lowercase e)  (format #t " L(~a)," (ucd-entry-category e))]
-            [(ucd-entry-alphabetic e) (format #t " A(~a)," (ucd-entry-category e))]
-            [else                 (format #t "    ~a," (ucd-entry-category e))])
+      (cond
+       [(ucd-entry-uppercase e)  (format #t " U(~a)," (ucd-entry-category e))]
+       [(ucd-entry-lowercase e)  (format #t " L(~a)," (ucd-entry-category e))]
+       [(ucd-entry-alphabetic e) (format #t " A(~a)," (ucd-entry-category e))]
+       [else                 (format #t "    ~a," (ucd-entry-category e))])
       (format #t "    Cn,")))
   (define (emit-columns start end p)
     (do-ec (:parallel (: k start end)
@@ -318,74 +323,114 @@
   (print "#define L(x) ((x)|SCM_CHAR_LOWERCASE_BITS)")
 
   ;; utf-8 ('none' is also here, for we treat it as latin-1.)
-  (print "#if !defined(GAUCHE_CHAR_ENCODING_EUC_JP) && !defined(GAUCHE_CHAR_ENCODING_SJIS)")
-  ;;   U+0000 - U+1ffff : direct table lookup
-  (display "static unsigned char ucs_general_category_00000[] = {")
-  (emit-columns 0 #x20000 (^i (emit-entry (ucs->entry i))))
-  (print "};")
-  ;;   Over U+20000 - binary search
-  (print)
-  (print "static unsigned char ucs_general_category_20000(ScmChar code)")
-  (print "{")
-  (print "  /*")
-  (for-each (^e (format #t "    ~6x ~a\n" (car e) (cdr e)))
-            (ucd-get-category-ranges db))
-  (print "  */")
-  (generate-bisect (coerce-to <vector> (ucd-get-category-ranges db))
-                   (^e (format "return ~a;"
-                               (case (cdr e)
-                                 [(Lo) "A(Lo)"]
-                                 [(#f) "Cn"]
-                                 [else => identity]))))
-  (print "}")
-  (print "#endif /*defined(GAUCHE_CHAR_ENCODING_UTF_8)*/")
-  (print)
+  (let ([code-sets (build-code-sets db 'utf8 (ucd-general-categories))])
+    (print "#if !defined(GAUCHE_CHAR_ENCODING_EUC_JP) && !defined(GAUCHE_CHAR_ENCODING_SJIS)")
+    ;;   U+0000 - U+1ffff : direct table lookup
+    (display "static unsigned char ucs_general_category_00000[] = {")
+    (emit-columns 0 #x20000 (^i (emit-entry (ucs->entry i))))
+    (print "};")
+    ;;   Over U+20000 - binary search
+    (print)
+    (print "static unsigned char ucs_general_category_20000(ScmChar code)")
+    (print "{")
+    (print "  /*")
+    (for-each (^e (format #t "    ~6x ~a\n" (car e) (cdr e)))
+              (ucd-get-category-ranges db))
+    (print "  */")
+    (generate-bisect (coerce-to <vector> (ucd-get-category-ranges db))
+                     (^e (format "return ~a;"
+                                 (case (cdr e)
+                                   [(Lo) "A(Lo)"]
+                                   [(#f) "Cn"]
+                                   [else => identity]))))
+    (print "}")
+    (for-each
+     (^s (dump-code-set-in-C (cdr s)))
+     (sort (hash-table->alist code-sets) string<? (^x (x->string (car x)))))
+    (print "#endif /*defined(GAUCHE_CHAR_ENCODING_UTF_8)*/")
+    (print))
 
   ;; euc-jp
-  (print "#if defined(GAUCHE_CHAR_ENCODING_EUC_JP)")
-  (display "static unsigned char eucjp_general_category_G0[] = {")
-  (emit-columns 0 #x80 (^i (emit-entry (eucjp->ucd-entry db i))))
-  (print "};")
-  (display "static unsigned char eucjp_general_category_G1[] = {")
-  (dotimes [z (- #xff #xa1)]
-    (emit-columns (+ (ash (+ z #xa1) 8) #xa1)
-                  (+ (ash (+ z #xa1) 8) #xff)
-                  (^i (emit-entry (eucjp->ucd-entry db i)))))
-  (print "};")
-  (display "static unsigned char eucjp_general_category_G2[] = {")
-  (emit-columns #x8ea1 #x8ee0 (^i (emit-entry (eucjp->ucd-entry db i))))
-  (print "};")
-  (display "static unsigned char eucjp_general_category_G3[] = {")
-  (dotimes [z (- #xff #xa1)]
-    (emit-columns (+ (ash (+ z #xa1) 8) #x8f00a1)
-                  (+ (ash (+ z #xa1) 8) #x8f00ff)
-                  (^i (emit-entry (eucjp->ucd-entry db i)))))
-  (print "};")
-  (print "#endif /*defined(GAUCHE_CHAR_ENCODING_EUC_JP)*/")
-  (print)
+  (let ([code-sets (build-code-sets db 'eucjp (ucd-general-categories))])
+    (print "#if defined(GAUCHE_CHAR_ENCODING_EUC_JP)")
+    (display "static unsigned char eucjp_general_category_G0[] = {")
+    (emit-columns 0 #x80 (^i (emit-entry (eucjp->ucd-entry db i))))
+    (print "};")
+    (display "static unsigned char eucjp_general_category_G1[] = {")
+    (dotimes [z (- #xff #xa1)]
+      (emit-columns (+ (ash (+ z #xa1) 8) #xa1)
+                    (+ (ash (+ z #xa1) 8) #xff)
+                    (^i (emit-entry (eucjp->ucd-entry db i)))))
+    (print "};")
+    (display "static unsigned char eucjp_general_category_G2[] = {")
+    (emit-columns #x8ea1 #x8ee0 (^i (emit-entry (eucjp->ucd-entry db i))))
+    (print "};")
+    (display "static unsigned char eucjp_general_category_G3[] = {")
+    (dotimes [z (- #xff #xa1)]
+      (emit-columns (+ (ash (+ z #xa1) 8) #x8f00a1)
+                    (+ (ash (+ z #xa1) 8) #x8f00ff)
+                    (^i (emit-entry (eucjp->ucd-entry db i)))))
+    (print "};")
+    (for-each
+     (^s (dump-code-set-in-C (cdr s)))
+     (sort (hash-table->alist code-sets) string<? (^x (x->string (car x)))))
+    (print "#endif /*defined(GAUCHE_CHAR_ENCODING_EUC_JP)*/")
+    (print))
 
   ;; sjis
-  (print "#if defined(GAUCHE_CHAR_ENCODING_SJIS)")
-  (display "static unsigned char sjis_general_category_00[] = {")
-  (emit-columns 0 #x80 (^i (emit-entry (sjis->ucd-entry db i))))
-  (print "};")
-  (display "static unsigned char sjis_general_category_a0[] = {")
-  (emit-columns #xa0 #xdf (^i (emit-entry (sjis->ucd-entry db i))))
-  (print "};")
-  (display "static unsigned char sjis_general_category_8000[] = {")
-  (do-ec (: z #x8000 #xa000 256)
-         (emit-columns (+ z #x40)
-                       (+ z #xfd)
-                       (^i (emit-entry (sjis->ucd-entry db i)))))
-  (print "};")
-  (display "static unsigned char sjis_general_category_e000[] = {")
-  (do-ec (: z #xe000 #x10000 256)
-         (emit-columns (+ z #x40)
-                       (+ z #xfd)
-                       (^i (emit-entry (sjis->ucd-entry db i)))))
-  (print "};")
-  (print "#endif /*defined(GAUCHE_CHAR_ENCODING_SJIS)*/")
-  (print)
+  (let ([code-sets (build-code-sets db 'sjis (ucd-general-categories))])
+    (print "#if defined(GAUCHE_CHAR_ENCODING_SJIS)")
+    (display "static unsigned char sjis_general_category_00[] = {")
+    (emit-columns 0 #x80 (^i (emit-entry (sjis->ucd-entry db i))))
+    (print "};")
+    (display "static unsigned char sjis_general_category_a0[] = {")
+    (emit-columns #xa0 #xdf (^i (emit-entry (sjis->ucd-entry db i))))
+    (print "};")
+    (display "static unsigned char sjis_general_category_8000[] = {")
+    (do-ec (: z #x8000 #xa000 256)
+           (emit-columns (+ z #x40)
+                         (+ z #xfd)
+                         (^i (emit-entry (sjis->ucd-entry db i)))))
+    (print "};")
+    (display "static unsigned char sjis_general_category_e000[] = {")
+    (do-ec (: z #xe000 #x10000 256)
+           (emit-columns (+ z #x40)
+                         (+ z #xfd)
+                         (^i (emit-entry (sjis->ucd-entry db i)))))
+    (print "};")
+    (for-each
+     (^s (dump-code-set-in-C (cdr s)))
+     (sort (hash-table->alist code-sets) string<? (^x (x->string (car x)))))
+    (print "#endif /*defined(GAUCHE_CHAR_ENCODING_SJIS)*/")
+    (print))
+
+  ;; Bind predefined charset
+  ;; The CAT entry can be just a symbol or (alias symbol).
+  (print "static void init_predefined_charsets() {")
+  (dolist [cat (append (ucd-general-categories)
+                       '(L LC M N P S Z C
+                         (LETTER L) ASCII_LETTER
+                         (DIGIT Nd) ASCII_DIGIT
+                         LETTER_DIGIT ASCII_LETTER_DIGIT 
+                         (LOWER Ll) ASCII_LOWER 
+                         (UPPER Lu) ASCII_UPPER
+                         (TITLE Lt)
+                         GRAPHIC ASCII_GRAPHIC 
+                         PRINTING ASCII_PRINTING
+                         (PUNCTUATION P) ASCII_PUNCTUATION 
+                         (SYMBOL S) ASCII_SYMBOL
+                         (ISO_CONTROL Cc) ASCII_CONTROL
+                         HEX_DIGIT
+                         WHITESPACE ASCII_WHITESPACE 
+                         BLANK ASCII_BLANK
+                         ASCII EMPTY 
+                         ASCII_WORD (WORD ASCII_WORD)))]
+    (let1 setname (if (pair? cat) (car cat) cat)
+      (print #"  predef_sets[SCM_CHAR_SET_~|setname|] =")
+      (if (pair? cat)
+        (print #"    predef_sets[SCM_CHAR_SET_~(cadr cat)];")
+        (print #"    make_charset_~|setname|();"))))
+  (print "}")
 
   ;; Teardown
   (dolist [c (ucd-general-categories)]
@@ -532,6 +577,215 @@
                            "return -1;")))
     (print "}")))
 
+;; Predefined character sets
+
+(define (for-each-char-code/none proc)
+  (dotimes [n 256] (proc n n)))
+
+(define (for-each-char-code/ucs proc)
+  (dotimes [n #x2ffff] (proc n n))
+  (proc #xe0000 #xe007f) ; Language tags; Cf
+  (proc #xe0100 #xe01ef) ; Variation selectors; Mn
+  (proc #xf0000 #xffffd) ; Private use; Co
+  (proc #x100000 #x10fffd) ; Private use; Co
+  )
+
+(define (for-each-char-code/euc-jp proc)
+  (dotimes [n 256] (proc n n))
+  (do-ec (: n #x8ea1 #x8eff) (proc n n))
+  (do-ec (: n #xa1a1 #xfeff) (proc n n))
+  (do-ec (: n #x8fa1a1 #x8ffef7) (proc n n)))
+
+(define (for-each-char-code/sjis proc)
+  (dotimes [n 128] (proc n n))
+  (do-ec (: n #xa0 #xe0) (proc n n))
+  (do-ec (: n #xfd #x100) (proc n n))
+  (do-ec (: n #x8140 #x9ffd) (proc n n))
+  (do-ec (: n #xe040 #xfcfd) (proc n n)))
+
+(define (build-code-sets db encoding categories)
+  (define sets (make-hash-table 'eq?))
+  (define walker (ecase encoding
+                   [(none) for-each-char-code/none]
+                   [(utf8) for-each-char-code/ucs]
+                   [(eucjp) for-each-char-code/euc-jp]
+                   [(sjis) for-each-char-code/sjis]))
+  (define get-entry
+    (let ([eucjp-entry-cache (make-hash-table 'eqv?)]
+          [sjis-entry-cache (make-hash-table 'eqv?)])
+      (ecase encoding
+        [(none utf8) ucd-get-entry]
+        [(eucjp) (^[db n]
+                   (if-let1 e (hash-table-get eucjp-entry-cache n #f)
+                     (if (eq? e 'nothing) #f e)
+                     (rlet1 e (eucjp->ucd-entry db n)
+                       (hash-table-put! eucjp-entry-cache n (or e 'nothing)))))]
+        [(sjis) (^[db n]
+                  (if-let1 e (hash-table-get sjis-entry-cache n #f)
+                    (if (eq? e 'nothing) #f e)
+                    (rlet1 e (sjis->ucd-entry db n)
+                      (hash-table-put! sjis-entry-cache n (or e 'nothing)))))]
+        )))
+  (define (register set cat n m)
+    (and-let1 e (get-entry db n)
+      (if (eq? cat (ucd-entry-category e))
+        (if (= n m)
+          (add-code! set n)
+          (add-code-range! set n m)))))
+  ;; general category charsets (char-set:Lt etc.)
+  (dolist [cat categories]
+    (let1 cs (make <char-code-set> :name cat)
+      (walker (cut register cs cat <> <>))
+      (hash-table-put! sets cat cs)))
+  ;; general category class charsets (char-set:L etc.)
+  (dolist [gcats (group-collection categories
+                                  :key (^c (string-ref (symbol->string c) 0)))]
+    (let1 cs 
+        (make <char-code-set>
+          :name (string->symbol (substring (symbol->string (car gcats)) 0 1)))
+      (for-each (^c (walker (cut register cs c <> <>))) gcats)
+      (hash-table-put! sets (~ cs'name) cs)))
+  ;; srfi-14 charsets (the ones that has equivalent set in general category set
+  ;; is handled implicitly.
+  (hash-table-put! sets 'ASCII_UPPER
+                   (rlet1 cs (make <char-code-set> :name 'ASCII_UPPER)
+                     (add-code-range! cs 
+                                      (char->integer #\A)
+                                      (char->integer #\Z))))
+  (hash-table-put! sets 'ASCII_LOWER
+                   (rlet1 cs (make <char-code-set> :name 'ASCII_LOWER)
+                     (add-code-range! cs 
+                                      (char->integer #\a)
+                                      (char->integer #\z))))
+  (hash-table-put! sets 'ASCII_LETTER
+                   (code-set-union 'ASCII_LETTER
+                                   (hash-table-ref sets 'ASCII_UPPER)
+                                   (hash-table-ref sets 'ASCII_LOWER)))
+  (hash-table-put! sets 'ASCII_DIGIT
+                   (rlet1 cs (make <char-code-set> :name 'ASCII_DIGIT)
+                     (add-code-range! cs 
+                                      (char->integer #\0)
+                                      (char->integer #\9))))
+  (hash-table-put! sets 'LETTER_DIGIT
+                   (code-set-union 'LETTER_DIGIT
+                                   (hash-table-ref sets 'L)
+                                   (hash-table-ref sets 'Nd)))
+  (hash-table-put! sets 'ASCII_LETTER_DIGIT
+                   (code-set-union 'ASCII_LETTER_DIGIT
+                                   (hash-table-ref sets 'ASCII_LETTER)
+                                   (hash-table-ref sets 'ASCII_DIGIT)))
+  (hash-table-put! sets 'ASCII_WHITESPACE
+                   (rlet1 cs (make <char-code-set> :name 'ASCII_WHITESPACE)
+                     (add-code-range! cs 9 13) ;TAB,LF,LTAB,FF,CR
+                     (add-code! cs (char->integer #\space))))
+  (hash-table-put! sets 'WHITESPACE
+                   (code-set-union 'WHITESPACE
+                                   (hash-table-ref sets 'ASCII_WHITESPACE)
+                                   (hash-table-ref sets 'Z)))
+  (hash-table-put! sets 'ASCII_BLANK
+                   (rlet1 cs (make <char-code-set> :name 'ASCII_BLANK)
+                     (add-code! cs 9) ;TAB
+                     (add-code! cs (char->integer #\space))))
+  (hash-table-put! sets 'BLANK
+                   (code-set-union 'BLANK
+                                   (hash-table-ref sets 'ASCII_BLANK)
+                                   (hash-table-ref sets 'Zs)))
+  (hash-table-put! sets 'ASCII_PUNCTUATION
+                   (rlet1 cs (make <char-code-set> :name 'ASCII_PUNCTUATION)
+                     (add-code! cs 33)  ;!
+                     (add-code! cs 34)  ;"
+                     (add-code! cs 35)  ;#
+                     (add-code! cs 37)  ;%
+                     (add-code! cs 38)  ;&
+                     (add-code! cs 39)  ;'
+                     (add-code! cs 40)  ;(
+                     (add-code! cs 41)  ;)
+                     (add-code! cs 42)  ;*
+                     (add-code! cs 44)  ;,
+                     (add-code! cs 45)  ;-
+                     (add-code! cs 46)  ;.
+                     (add-code! cs 47)  ;/
+                     (add-code! cs 58)  ;:
+                     (add-code! cs 59)  ;;
+                     (add-code! cs 63)  ;?
+                     (add-code! cs 64)  ;@
+                     (add-code! cs 91)  ;[
+                     (add-code! cs 92)  ;\
+                     (add-code! cs 93)  ;)
+                     (add-code! cs 95)  ;_
+                     (add-code! cs 123) ;{
+                     (add-code! cs 125) ;}
+                     ))
+  (hash-table-put! sets 'ASCII_SYMBOL
+                   (rlet1 cs (make <char-code-set> :name 'ASCII_SYMBOL)
+                     (add-code! cs 36)  ;$
+                     (add-code! cs 43)  ;+
+                     (add-code! cs 60)  ;<
+                     (add-code! cs 61)  ;=
+                     (add-code! cs 62)  ;>
+                     (add-code! cs 94)  ;^
+                     (add-code! cs 96)  ;`
+                     (add-code! cs 124) ;|
+                     (add-code! cs 126) ;~
+                     ))
+  (hash-table-put! sets 'GRAPHIC
+                   (code-set-union 'GRAPHIC
+                                   (hash-table-ref sets 'L)
+                                   (hash-table-ref sets 'N)
+                                   (hash-table-ref sets 'P)
+                                   (hash-table-ref sets 'S)))
+  (hash-table-put! sets 'ASCII_GRAPHIC
+                   (code-set-union 'ASCII_GRAPHIC
+                                   (hash-table-ref sets 'ASCII_LETTER)
+                                   (hash-table-ref sets 'ASCII_DIGIT)
+                                   (hash-table-ref sets 'ASCII_PUNCTUATION)
+                                   (hash-table-ref sets 'ASCII_SYMBOL)))
+  (hash-table-put! sets 'PRINTING
+                   (code-set-union 'PRINTING
+                                   (hash-table-ref sets 'GRAPHIC)
+                                   (hash-table-ref sets 'WHITESPACE)))
+  (hash-table-put! sets 'ASCII_PRINTING
+                   (code-set-union 'ASCII_PRINTING
+                                   (hash-table-ref sets 'ASCII_GRAPHIC)
+                                   (hash-table-ref sets 'ASCII_WHITESPACE)))
+  (hash-table-put! sets 'ASCII_CONTROL
+                   (rlet1 cs (make <char-code-set> :name 'ASCII_CONTROL)
+                     (add-code-range! cs #x00 #x1f)
+                     (add-code! cs #x7f)))
+  (hash-table-put! sets 'HEX_DIGIT
+                   (rlet1 cs (make <char-code-set> :name 'HEX_DIGIT)
+                     (add-code-range! cs 
+                                      (char->integer #\0)
+                                      (char->integer #\9))
+                     (add-code-range! cs 
+                                      (char->integer #\A)
+                                      (char->integer #\F))
+                     (add-code-range! cs 
+                                      (char->integer #\a)
+                                      (char->integer #\f))))
+  (hash-table-put! sets 'ASCII
+                   (rlet1 cs (make <char-code-set> :name 'ASCII)
+                     (add-code-range! cs 0 127)))
+  (hash-table-put! sets 'ASCII_WORD
+                   (rlet1 cs (make <char-code-set> :name 'ASCII_WORD)
+                     (add-code-range! cs 
+                                      (char->integer #\0)
+                                      (char->integer #\9))
+                     (add-code-range! cs 
+                                      (char->integer #\A)
+                                      (char->integer #\Z))
+                     (add-code-range! cs 
+                                      (char->integer #\a)
+                                      (char->integer #\z))
+                     (add-code! cs (char->integer #\_))))
+  (hash-table-put! sets 'LC
+                   (code-set-union 'LC
+                                   (hash-table-ref sets 'Lt)
+                                   (hash-table-ref sets 'Ll)
+                                   (hash-table-ref sets 'Lu)))
+  (hash-table-put! sets 'EMPTY (make <char-code-set> :name 'EMPTY))
+  sets)
+
 ;; Break property values
 ;; This goes to src/gauche/priv/unicode_attr.h
 ;;
