@@ -1,7 +1,7 @@
 /*
  * hash.c - hash table implementation
  *
- *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -33,9 +33,9 @@
 
 #define LIBGAUCHE_BODY
 #include <math.h>
-#include "atomic_ops.h"
 #include "gauche.h"
 #include "gauche/class.h"
+#include "gauche/priv/atomicP.h"
 
 /*============================================================
  * Internal structures
@@ -78,17 +78,17 @@ static u_int round2up(unsigned int val);
    mechanism.
 */
 
-static ScmParameterLoc hash_salt; /* initialized by Scm__InitHash() */
+static ScmPrimitiveParameter *hash_salt; /* initialized by Scm__InitHash() */
 
 ScmSmallInt Scm_HashSaltRef()
 {
-    return SCM_INT_VALUE(Scm_ParameterRef(Scm_VM(), &hash_salt));
+    return SCM_INT_VALUE(Scm_PrimitiveParameterRef(Scm_VM(), hash_salt));
 }
 
 ScmSmallInt Scm_HashSaltSet(ScmSmallInt newval) /* returns old value */
 {
-    return SCM_INT_VALUE(Scm_ParameterSet(Scm_VM(), &hash_salt,
-                                          SCM_MAKE_INT(newval)));
+    return SCM_INT_VALUE(Scm_PrimitiveParameterSet(Scm_VM(), hash_salt,
+                                                   SCM_MAKE_INT(newval)));
 }
 
 /*============================================================
@@ -230,6 +230,112 @@ static u_long internal_string_hash(ScmString *str, u_long salt, int portable)
     }
 }
 
+static u_long internal_uvector_hash(ScmUVector *u, u_long salt, int portable)
+{
+    if (portable) {
+        ScmUVectorType uvtype = Scm_UVectorType(Scm_ClassOf(SCM_OBJ(u)));
+        u_long r, seed;
+        size_t s = SCM_UVECTOR_SIZE(u);
+
+        switch (uvtype) {
+            /* We can use siphash if u is u8vector or s8vector. */ 
+        case SCM_UVECTOR_S8:
+        case SCM_UVECTOR_U8:
+            return Scm__DwSipPortableHash((uint8_t*)SCM_UVECTOR_ELEMENTS(u),
+                                          (uint32_t)Scm_UVectorSizeInBytes(u),
+                                          salt, salt^uvtype);
+            /* We need to avoid depending on endianness of multibyte numbers.
+               Using siphash after canonicalizing byte order can be expensive,
+               for we need to allocate a buffer.
+               (Unless we directly access siphash setup/round function
+               and feed one word at a time--which is tedious.)
+               The current code may not be ideal, but just as good as our 
+               other primitive portable hash functions. */
+
+            /* Initial hash value.  The seed value for each uvector type
+               is just a random value I generated. */
+#define INIT_R(r, seed) SMALL_INT_HASH(r, seed^salt)
+                                                         
+        case SCM_UVECTOR_S16:
+            seed = 3499211612ul;
+            goto do16;
+        case SCM_UVECTOR_U16:
+            seed = 3890346734ul;
+            goto do16;
+        case SCM_UVECTOR_F16:
+            seed = 545404204ul;
+            do16:
+            {
+                INIT_R(r, seed);
+                for (size_t i=0; i<s; i++) {
+                    u_long e = SCM_U16VECTOR_ELEMENT(u, i);
+                    SMALL_INT_HASH(e, e);
+                    r = COMBINE(r, e);
+                }
+                return r;
+            }
+        case SCM_UVECTOR_S32:
+            seed = 4161255391ul;
+            goto do32;
+        case SCM_UVECTOR_U32:
+            seed = 3922919429ul;
+            goto do32;
+        case SCM_UVECTOR_F32:
+            seed = 949333985ul;
+            do32:
+            {
+                INIT_R(r, seed);
+                for (size_t i=0; i<s; i++) {
+                    u_long e = SCM_U32VECTOR_ELEMENT(u, i);
+                    SMALL_INT_HASH(e, e);
+                    r = COMBINE(r, e);
+                }
+                return r;
+            }
+        case SCM_UVECTOR_S64:
+            seed = 2715962298ul;
+            goto do64;
+        case SCM_UVECTOR_U64:
+            seed = 1323567403ul;
+            do64: 
+            {
+                INIT_R(r, seed);
+                for (size_t i=0; i<s; i++) {
+                    uint64_t e = SCM_U64VECTOR_ELEMENT(u, i);
+                    u_long z = (e >> 32) ^ e;
+                    SMALL_INT_HASH(z, z);
+                    r = COMBINE(r, z);
+                }
+                return r;
+            }
+        case SCM_UVECTOR_F64: {
+            /* We can't reinterpret f64 as u64, since ARM mixed-endian would
+               yield nonportable result. */
+            seed = 2350294565ul;
+            INIT_R(r, seed);
+            for (size_t i=0; i<s; i++) {
+                double e = SCM_F64VECTOR_ELEMENT(u, i);
+                r = COMBINE(r, flonum_hash(e, salt, TRUE));
+            }
+            return r;
+        }
+#undef INIT_R            
+        case SCM_UVECTOR_RESERVED1:
+        case SCM_UVECTOR_C32:
+        case SCM_UVECTOR_C64:
+        case SCM_UVECTOR_C128:
+        case SCM_UVECTOR_RESERVED2:
+        case SCM_UVECTOR_INVALID:
+            Scm_Panic("invalid uvector class.");
+        }
+        return 0;           /* dummy */
+    } else {
+        return Scm__DwSipDefaultHash((uint8_t*)SCM_UVECTOR_ELEMENTS(u),
+                                     (uint32_t)Scm_UVectorSizeInBytes(u),
+                                     salt, salt);
+    }
+}
+
 /* equal-hash, which satisfies
      forall x, y: equal(x,y) => hash(x) = hash(y)
   
@@ -263,6 +369,9 @@ static u_long equal_hash_common(ScmObj obj, u_long salt, int portable)
             h = COMBINE(h, h2);
         }
         return h;
+    /* uvector hash support */
+    } else if (SCM_UVECTORP(obj)) {
+        return internal_uvector_hash(SCM_UVECTOR(obj), salt, portable);
 #if GAUCHE_KEEP_DISJOINT_KEYWORD_OPTION
     } else if (SCM_KEYWORDP(obj)) {
         if (portable) {
@@ -297,54 +406,60 @@ static u_long equal_hash_common(ScmObj obj, u_long salt, int portable)
             ADDRESS_HASH(hashval, obj);
             return hashval;
         }
-    } else {
-        /* Call specialized object-hash method
-           We need some trick; See libomega.scm for the details. */
-        static ScmObj call_object_hash_proc = SCM_UNDEFINED;
-        static ScmObj portable_hash_proc = SCM_UNDEFINED;
-        static ScmObj default_hash_proc = SCM_UNDEFINED;
-        SCM_BIND_PROC(call_object_hash_proc, "%call-object-hash",
-                      Scm_GaucheInternalModule());
-        SCM_BIND_PROC(portable_hash_proc, "portable-hash", Scm_GaucheModule());
-        SCM_BIND_PROC(default_hash_proc, "default-hash", Scm_GaucheModule());
-        ScmObj r = Scm_ApplyRec3(call_object_hash_proc, obj,
-                                 (portable
-                                  ? portable_hash_proc
-                                  : default_hash_proc),
-                                 (portable
-                                  ? Scm_MakeIntegerU(salt)
-                                  : SCM_FALSE));
-        if (SCM_INTP(r)) {
-            return (u_long)SCM_INT_VALUE(r);
-        }
-        if (SCM_BIGNUMP(r)) {
-            /* NB: Scm_GetUInteger clamps the result to [0, ULONG_MAX],
-               so taking the LSW would give better distribution. */
-            return SCM_BIGNUM(r)->values[0];
-        }
-        Scm_Error("object-hash returned non-integer: %S", r);
-        return 0;               /* dummy */
     }
+
+    ScmClass *k = SCM_CLASS_OF(obj);
+    if (k->hash) {
+        return (u_long)k->hash(obj, salt,
+                               (portable? SCM_HASH_PORTABLE:0));
+    }
+
+    /* Call specialized object-hash method
+       We need some trick; See libomega.scm for the details. */
+    static ScmObj call_object_hash_proc = SCM_UNDEFINED;
+    static ScmObj portable_hash_proc = SCM_UNDEFINED;
+    static ScmObj default_hash_proc = SCM_UNDEFINED;
+    SCM_BIND_PROC(call_object_hash_proc, "%call-object-hash",
+                  Scm_GaucheInternalModule());
+    SCM_BIND_PROC(portable_hash_proc, "portable-hash", Scm_GaucheModule());
+    SCM_BIND_PROC(default_hash_proc, "default-hash", Scm_GaucheModule());
+    ScmObj r = Scm_ApplyRec3(call_object_hash_proc, obj,
+                             (portable
+                              ? portable_hash_proc
+                              : default_hash_proc),
+                             (portable
+                              ? Scm_MakeIntegerU(salt)
+                              : SCM_FALSE));
+    if (SCM_INTP(r)) {
+        return (u_long)SCM_INT_VALUE(r);
+    }
+    if (SCM_BIGNUMP(r)) {
+        /* NB: Scm_GetUInteger clamps the result to [0, ULONG_MAX],
+           so taking the LSW would give better distribution. */
+        return SCM_BIGNUM(r)->values[0];
+    }
+    Scm_Error("object-hash returned non-integer: %S", r);
+    return 0;               /* dummy */
 }
 
 /* For recursive call to the current hash function - see call-object-hash
    and object-hash definitions in libomega.scm. */
-static ScmParameterLoc current_recursive_hash;
+static ScmPrimitiveParameter *current_recursive_hash;
 
 ScmObj Scm_CurrentRecursiveHash(ScmObj newval)
 {
-    ScmObj val = Scm_ParameterRef(Scm_VM(), &current_recursive_hash);
-    if (newval != SCM_UNBOUND) {
-        Scm_ParameterSet(Scm_VM(), &current_recursive_hash, newval);
+    if (newval == SCM_UNBOUND) {
+        return Scm_PrimitiveParameterRef(Scm_VM(), current_recursive_hash);
+    } else {
+        return Scm_PrimitiveParameterSet(Scm_VM(), current_recursive_hash, newval);
     }
-    return val;
 }
 
 /* 'Portable' general hash function.
    
    It is guaranteed that the hash value won't change for the same objects
    (roughly, indistinguishable in their external representation)
-   accross the runs of the program, and among different platforms.
+   across the runs of the program, and among different platforms.
    That is, the value can be used in persistent stores.
  */
 u_long Scm_PortableHash(ScmObj obj, u_long salt)
@@ -358,6 +473,36 @@ ScmSmallInt Scm_DefaultHash(ScmObj obj)
     return equal_hash_common(obj, Scm_HashSaltRef(), FALSE) & HASHMASK;
 }
 
+/* This is to be called from ScmClass->hash if it needs to compute
+   hash value of field values. */
+ScmSmallInt Scm_RecursiveHash(ScmObj obj, ScmSmallInt salt, u_long flags)
+{
+    if (flags & SCM_HASH_PORTABLE) {
+        return equal_hash_common(obj, salt, TRUE) & PORTABLE_HASHMASK;
+    } else {
+        return equal_hash_common(obj, salt, FALSE) & HASHMASK;
+    }
+}
+
+ScmSmallInt Scm_SmallIntHash(ScmSmallInt val, 
+                             ScmSmallInt salt SCM_UNUSED,
+                             u_long flags SCM_UNUSED)
+{
+    ScmSmallInt v;
+    SMALL_INT_HASH(v, val);
+    return v;
+}
+
+ScmSmallInt Scm_Int64Hash(int64_t val,
+                          ScmSmallInt salt SCM_UNUSED,
+                          u_long flags SCM_UNUSED)
+{
+    ScmSmallInt v;
+    SMALL_INT_HASH(v, val);
+    return v;
+}
+
+
 /* This is to expose string hash function.  Modulo is for the compatibility
    of srfi-13; just give 0 as modulo if you don't need it.  */
 u_long Scm_HashString(ScmString *str, u_long modulo)
@@ -506,14 +651,15 @@ static Entry *address_access(ScmHashCore *table,
     NOTFOUND(table, op, key, hashval, index);
 }
 
-static u_long address_hash(const ScmHashCore *ht, intptr_t obj)
+static u_long address_hash(const ScmHashCore *ht SCM_UNUSED, intptr_t obj)
 {
     u_long hashval;
     ADDRESS_HASH(hashval, obj);
     return hashval;
 }
 
-static int address_cmp(const ScmHashCore *ht, intptr_t key, intptr_t k2)
+static int address_cmp(const ScmHashCore *ht SCM_UNUSED, 
+                       intptr_t key, intptr_t k2)
 {
     return (key == k2);
 }
@@ -522,22 +668,22 @@ static int address_cmp(const ScmHashCore *ht, intptr_t key, intptr_t k2)
  * Accessor function for equal and eqv-hash.
  * We assume KEY is ScmObj.
  */
-static u_long eqv_hash(const ScmHashCore *table, intptr_t key)
+static u_long eqv_hash(const ScmHashCore *ht SCM_UNUSED, intptr_t key)
 {
     return Scm_EqvHash(SCM_OBJ(key));
 }
 
-static int eqv_cmp(const ScmHashCore *table, intptr_t key, intptr_t k2)
+static int eqv_cmp(const ScmHashCore *ht SCM_UNUSED, intptr_t key, intptr_t k2)
 {
     return Scm_EqvP(SCM_OBJ(key), SCM_OBJ(k2));
 }
 
-static u_long equal_hash(const ScmHashCore *table, intptr_t key)
+static u_long equal_hash(const ScmHashCore *ht SCM_UNUSED, intptr_t key)
 {
     return Scm_DefaultHash(SCM_OBJ(key));
 }
 
-static int equal_cmp(const ScmHashCore *table, intptr_t key, intptr_t k2)
+static int equal_cmp(const ScmHashCore *ht SCM_UNUSED, intptr_t key, intptr_t k2)
 {
     return Scm_EqualP(SCM_OBJ(key), SCM_OBJ(k2));
 }
@@ -572,12 +718,12 @@ static Entry *string_access(ScmHashCore *table, intptr_t k, ScmDictOp op)
     NOTFOUND(table, op, k, hashval, index);
 }
 
-static u_long string_hash(const ScmHashCore *table, intptr_t key)
+static u_long string_hash(const ScmHashCore *ht SCM_UNUSED, intptr_t key)
 {
     return Scm_HashString(SCM_STRING(key), 0);
 }
 
-static int string_cmp(const ScmHashCore *table, intptr_t k1, intptr_t k2)
+static int string_cmp(const ScmHashCore *ht SCM_UNUSED, intptr_t k1, intptr_t k2)
 {
     const ScmStringBody *b1 = SCM_STRING_BODY(k1);
     const ScmStringBody *b2 = SCM_STRING_BODY(k2);
@@ -709,9 +855,9 @@ void Scm_HashCoreInitSimple(ScmHashCore *core,
                             unsigned int initSize,
                             void *data)
 {
-    SearchProc  *accessfn;
-    ScmHashProc *hashfn;
-    ScmHashCompareProc *cmpfn;
+    SearchProc  *accessfn = NULL;
+    ScmHashProc *hashfn = NULL;
+    ScmHashCompareProc *cmpfn = NULL;
 
     if (hash_core_predef_procs(type, &accessfn, &hashfn, &cmpfn) == FALSE) {
         Scm_Error("[internal error]: wrong TYPE argument passed to Scm_HashCoreInitSimple: %d", type);
@@ -750,6 +896,7 @@ void Scm_HashCoreCopy(ScmHashCore *dst, const ScmHashCore *src)
             e->key = s->key;
             e->value = s->value;
             e->next = NULL;
+            e->hashval = s->hashval;
             if (p) p->next = e;
             else   b[i] = e;
             p = e;
@@ -967,7 +1114,8 @@ ScmObj Scm_HashTableStat(ScmHashTable *table)
  * Printer
  */
 
-static void hash_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
+static void hash_print(ScmObj obj, ScmPort *port, 
+                       ScmWriteContext *ctx SCM_UNUSED)
 {
     ScmHashTable *ht = (ScmHashTable*)obj;
     char *str = "";
@@ -1020,14 +1168,23 @@ void Scm__InitHash()
     u_long salt = ((u_long)getpid() * ((u_long)t.tv_sec^(u_long)t.tv_usec));
     ADDRESS_HASH(salt, salt);
     salt &= SCM_SMALL_INT_MAX;
-    Scm_InitParameterLoc(Scm_VM(), &hash_salt, Scm_MakeIntegerU(salt));
-    Scm_InitParameterLoc(Scm_VM(), &current_recursive_hash, SCM_FALSE);
+    /* 
+     * We can't use Scm_BindPrimitiveParameter here, since symbol table
+     * is not initialized yet (symbol table uses hashtable!)
+     */
+    hash_salt = 
+        Scm_MakePrimitiveParameter(SCM_CLASS_PRIMITIVE_PARAMETER, SCM_FALSE, 
+                                   Scm_MakeIntegerU(salt), 0);
+    current_recursive_hash = 
+        Scm_MakePrimitiveParameter(SCM_CLASS_PRIMITIVE_PARAMETER, SCM_FALSE, 
+                                   SCM_FALSE, 0);
 }
 
 /*====================================================================
  * For backward compatibility
  */
 
+#if GAUCHE_API_VERSION < 1000
 /* Backward compatibility.
    NB: Casting ScmDictEntry* to ScmHashEntry* would be invalid if
    sizeof(intptr_t) and sizeof(void*) differ.  I know only one
@@ -1091,7 +1248,7 @@ ScmObj Scm_MakeHashTableMultiWord(int keysize, int initsize)
 
 /* Legacy constructor.  DEPRECATED.  Will go away soon. */
 ScmObj Scm_MakeHashTable(ScmHashProc *hashfn,
-                         ScmHashCompareProc *cmpfn,
+                         ScmHashCompareProc *cmpfn SCM_UNUSED,
                          unsigned int initSize)
 {
     if (hashfn == (ScmHashProc*)SCM_HASH_EQ) {
@@ -1112,6 +1269,7 @@ ScmObj Scm_MakeHashTable(ScmHashProc *hashfn,
     return SCM_UNDEFINED;
 #endif
 }
+#endif /*GAUCHE_API_VERSION < 1000*/
 
 /* Legacy hash function.
  *
@@ -1121,7 +1279,7 @@ ScmObj Scm_MakeHashTable(ScmHashProc *hashfn,
  *
  * There are several drawbacks, though.  The guaranteed hash value means
  * we can't change hash function.   The quality of the original hash
- * functon wasn't good (it behaves terrible on flonums and compnums);
+ * function wasn't good (it behaves terrible on flonums and compnums);
  * it's vulnerable to collision attacks; and it had a few bugs in the
  * number hash that broke the 'portable' guarantee between platforms.
  *
