@@ -1,7 +1,7 @@
 /*
  * lazy.c - lazy evaluation constructs
  *
- *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -84,7 +84,8 @@ typedef struct ScmPromiseContentRec {
  * class stuff
  */
 
-static void promise_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
+static void promise_print(ScmObj obj, ScmPort *port,
+                          ScmWriteContext *ctx SCM_UNUSED)
 {
     ScmPromise *p = (ScmPromise*)obj;
     const char *forced = p->content->forced? " (forced)" : "";
@@ -120,7 +121,9 @@ ScmObj Scm_MakePromise(int forced, ScmObj code)
  * force
  */
 
-static ScmObj release_promise(ScmObj *args, int nargs, void *data)
+static ScmObj release_promise(ScmObj *args SCM_UNUSED,
+                              int nargs SCM_UNUSED,
+                              void *data)
 {
     ScmPromise *p = SCM_PROMISE(data);
     p->content->owner = NULL;
@@ -166,10 +169,10 @@ static ScmObj force_cc(ScmObj result, void **data)
         SCM_INTERNAL_MUTEX_UNLOCK(p->content->mutex);
     }
     Scm_VM()->handlers = handlers;
-    SCM_RETURN(Scm_Force(SCM_OBJ(p)));
+    SCM_RETURN(Scm_VMForce(SCM_OBJ(p)));
 }
 
-ScmObj Scm_Force(ScmObj obj)
+ScmObj Scm_VMForce(ScmObj obj)
 {
     if (!SCM_PROMISEP(obj)) {
         SCM_RETURN(obj);
@@ -208,7 +211,20 @@ ScmObj Scm_Force(ScmObj obj)
     }
 }
 
-#if GAUCHE_LAZY_PAIR
+ScmObj Scm_Force(ScmObj obj)
+{
+    if (!SCM_PROMISEP(obj)) {
+        SCM_RETURN(obj);
+    } else {
+        ScmPromiseContent *c = SCM_PROMISE(obj)->content;
+        if (c->forced) SCM_RETURN(c->code);
+
+        static ScmObj force = SCM_UNDEFINED;
+        SCM_BIND_PROC(force, "force", Scm_SchemeModule());
+        return Scm_ApplyRec1(force, obj);
+    }
+}
+
 /*=================================================================
  * Lazy pairs
  *
@@ -255,7 +271,7 @@ ScmObj Scm_Force(ScmObj obj)
   +---------------+
 
 
-  (2) If generator yields a non-nil value, owner first creates a
+  (2) If generator yields a non-EOF value, owner first creates a
       new LazyPair...
 
   +---------------+
@@ -402,13 +418,13 @@ struct ScmLazyPairRec {
     SCM_HEADER;
     ScmObj item;
     ScmObj generator;
-    AO_t owner;
+    ScmAtomicVar owner;
 };
 
 ScmObj Scm_MakeLazyPair(ScmObj item, ScmObj generator)
 {
     ScmLazyPair *z = SCM_NEW(ScmLazyPair);
-    z->owner = (AO_t)0;
+    z->owner = (ScmAtomicWord)0;
     SCM_SET_CLASS(z, SCM_CLASS_LAZY_PAIR);
     z->generator = generator;
     z->item = item;
@@ -428,9 +444,10 @@ ScmObj Scm_ForceLazyPair(volatile ScmLazyPair *lp)
     static const ScmTimeSpec req = {0, 1000000};
     ScmTimeSpec rem;
     ScmVM *vm = Scm_VM();
+    ScmAtomicWord zero = 0;	/* Need to use C11 intrinsic */
 
     do {
-        if (AO_compare_and_swap_full(&lp->owner, 0, SCM_WORD(vm))) {
+        if (AO_compare_and_swap_full(&lp->owner, zero, SCM_WORD(vm))) {
             /* Here we own the lazy pair. */
             ScmObj item = lp->item;
             /* Calling generator might change VM state, so we protect
@@ -450,11 +467,11 @@ ScmObj Scm_ForceLazyPair(volatile ScmLazyPair *lp)
                     lp->generator = SCM_NIL;
                 }
                 AO_nop_full();
-                SCM_SET_CAR(lp, item);
+                SCM_SET_CAR_UNCHECKED(lp, item);
                 /* We don't need barrier here. */
-                lp->owner = (AO_t)1;
+                lp->owner = (ScmAtomicWord)1;
             } SCM_WHEN_ERROR {
-                lp->owner = (AO_t)0; /*NB: See above about error handling*/
+                lp->owner = (ScmAtomicWord)0; /*NB: See above about error handling*/
                 SCM_NEXT_HANDLER;
             } SCM_END_PROTECT;
             if (extra_frame_pushed) {
@@ -467,7 +484,7 @@ ScmObj Scm_ForceLazyPair(volatile ScmLazyPair *lp)
            Since generators are supposed to be called every time to yield
            a new value, so it is ambiguous what value should be returned
            if a generator calls itself recursively. */
-        if (lp->owner == SCM_WORD(vm)) {
+        if (SCM_WORD(lp->owner) == SCM_WORD(vm)) {
             /* NB: lp->owner will be reset by the original caller of
                the generator. */
             Scm_Error("Attempt to recursively force a lazy pair.");
@@ -487,7 +504,9 @@ ScmObj Scm_ForceLazyPair(volatile ScmLazyPair *lp)
    but forced during execution of Scm_DecomposeLazyPair), returns its CAR
    and a generator that returns its CDR.
    Otherwise, returns FALSE.  */
-static ScmObj dummy_gen(ScmObj *args, int nargs, void *data)
+static ScmObj dummy_gen(ScmObj *args SCM_UNUSED,
+                        int nargs SCM_UNUSED,
+                        void *data)
 {
     ScmObj item;
     ScmObj generator;
@@ -505,18 +524,19 @@ int Scm_DecomposeLazyPair(ScmObj obj, ScmObj *item, ScmObj *generator)
         static const ScmTimeSpec req = {0, 1000000};
         ScmTimeSpec rem;
         ScmVM *vm = Scm_VM();
+        ScmAtomicWord zero = 0;	/* Need to use C11 intrinsic */
 
         for (;;) {
-            if (AO_compare_and_swap_full(&lp->owner, 0, SCM_WORD(vm))) {
+            if (AO_compare_and_swap_full(&lp->owner, zero, SCM_WORD(vm))) {
                 *item = lp->item;
                 *generator = lp->generator;
                 AO_nop_full();
                 lp->owner = 0;
                 return TRUE;
             }
-            if (lp->owner == (AO_t)1) {
+            if (lp->owner == (ScmAtomicWord)1) {
                 /* Somebody else has forced OBJ.  In the typical cases
-                   where we call this funtion for co-recursive lazy
+                   where we call this function for co-recursive lazy
                    algorithms, this situation rarely happens.   We fallthrough
                    to the SCM_PAIRP check below to return appropriate
                    values. */
@@ -548,4 +568,3 @@ int Scm_PairP(ScmObj x)
     }
 }
 
-#endif /* GAUCHE_LAZY_PAIR */
