@@ -1,7 +1,7 @@
 ;;;
 ;;; libchar.scm - builtin character procedures
 ;;;
-;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -36,6 +36,7 @@
 (inline-stub
  (declcode (.include <gauche/vminsn.h>
                      <gauche/char_attr.h>
+                     <gauche/priv/charP.h>
                      <ctype.h>))
  )
 
@@ -121,8 +122,9 @@
 (define-cproc digit->integer (ch::<char> :optional (radix::<fixnum> 10) (extended-range?::<boolean> #f))
   :constant
   (let* ([r::int])
-    (when (or (< radix 2) (> radix 36))
-      (Scm_Error "radix must be between 2 and 36, but got %d" radix))
+    (when (or (< radix SCM_RADIX_MIN) (> radix SCM_RADIX_MAX))
+      (Scm_Error "radix must be an integer between %d and %d, but got %d"
+                 SCM_RADIX_MIN SCM_RADIX_MAX radix))
     (when (and extended-range? (> radix 10))
       (Scm_Error "for extended range, radix can't exceed 10" radix))
     (set! r (Scm_DigitToInt ch radix extended-range?))
@@ -131,8 +133,9 @@
 (define-cproc integer->digit (n::<fixnum> :optional (radix::<fixnum> 10) (basechar1::<char> #\0) (basechar2::<char> #\a))
   :constant
   (let* ([r::ScmChar])
-    (when (or (< radix 2) (> radix 36))
-      (Scm_Error "radix must be between 2 and 36, but got %d" radix))
+    (when (or (< radix SCM_RADIX_MIN) (> radix SCM_RADIX_MAX))
+      (Scm_Error "radix must be an integer between %d and %d, but got %d"
+                 SCM_RADIX_MIN SCM_RADIX_MAX radix))
     (set! r (Scm_IntToDigit n radix basechar1 basechar2))
     (return (?: (== r SCM_CHAR_INVALID) '#f (SCM_MAKE_CHAR r)))))
 
@@ -188,7 +191,10 @@
     [(SCM_CHAR_CATEGORY_Cf) (return 'Cf)]
     [(SCM_CHAR_CATEGORY_Cs) (return 'Cs)]
     [(SCM_CHAR_CATEGORY_Co) (return 'Co)]
-    [(SCM_CHAR_CATEGORY_Cn) (return 'Cn)]))
+    [(SCM_CHAR_CATEGORY_Cn) (return 'Cn)]
+    [else (Scm_Panic "Scm_CharGeneralCategory returns unknown value \
+                      for character %d" c)
+          (return SCM_UNDEFINED)])) ; dummy
 
 
 ;;;
@@ -236,6 +242,10 @@
 (define-cproc char-set-immutable? (cs::<char-set>) ::<boolean>
   (return (SCM_CHAR_SET_IMMUTABLE_P cs)))
 
+(define (char-set-hash cs :optional (bound #x1fffffff))
+  (fold (^[range val] (modulo (hash (+ val (car range) (cdr range))) bound))
+        0 ((with-module gauche.internal %char-set-ranges) cs)))
+
 (select-module gauche.internal)
 
 (define-cproc %char-set-equal? (x::<char-set> y::<char-set>)
@@ -265,5 +275,7 @@
 (define-cproc %char-set-ranges (cs::<char-set>) Scm_CharSetRanges)
 (define-cproc %char-set-predefined (num::<fixnum>) Scm_GetStandardCharSet)
 
+(define-cproc %char-set-case-fold! (cs::<char-set>) Scm_CharSetCaseFold)
+
 (define-cproc %char-set-dump (cs::<char-set>) ::<void>
   (Scm_CharSetDump cs SCM_CUROUT))
