@@ -1,7 +1,7 @@
 ;;;
 ;;; libdict.scm - dictionary-like data structures (hashtables etc.)
 ;;;
-;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -97,6 +97,9 @@
            (return resultval_))]))])
  )
 
+;; An unique object, used to detect exhausted iterator
+(define *unique* (cons #f #f))
+
 ;;;
 ;;; Hashtables
 ;;;
@@ -218,10 +221,18 @@
        (error "make-hash-table requires a comparator or \
                one of the symbols in eq?, eqv?, equal? or string=?, but got:"
               comparator))
+     ;; If comparator's equality predicate is eq?/eqv?, we ignore
+     ;; the its hash function and force using eq-/eqv-hash,
+     ;; as permitted in srfi-125.  It allows object that doesn't have
+     ;; custom hash method can still be used with hashtables based on
+     ;; (make-eq[v]-comparator).
+     ;; https://github.com/shirok/Gauche/issues/708
      (cond
-      [(eq? comparator eq-comparator)
+      [(or (eq? comparator eq-comparator)
+           (eq? (comparator-equality-predicate comparator) eq?))
        (make-hash-table 'eq? init-size)]
-      [(eq? comparator eqv-comparator)
+      [(or (eq? comparator eqv-comparator)
+           (eq? (comparator-equality-predicate comparator) eqv?))
        (make-hash-table 'eqv? init-size)]
       [(eq? comparator equal-comparator)
        (make-hash-table 'equal? init-size)]
@@ -305,6 +316,7 @@
 
 (inline-stub
  (define-cfn hash-table-iter (args::ScmObj* nargs::int data::void*) :static
+   (cast void nargs) ; suppress unused var warning
    (let* ([iter::ScmHashIter* (cast ScmHashIter* data)]
           [e::ScmDictEntry* (Scm_HashIterNext iter)]
           [eofval (aref args 0)])
@@ -322,11 +334,22 @@
 (select-module gauche)
 (define-cproc hash-table-copy (ht::<hash-table> :optional mutable?)
   ;; we don't have immutable hash table, and ignore mutable? argument (srfi-125)
+  (cast void mutable?) ; suppress unused var warning
   (return (Scm_HashTableCopy ht)))
 (define-cproc hash-table-keys (ht::<hash-table>)   Scm_HashTableKeys)
 (define-cproc hash-table-values (ht::<hash-table>) Scm_HashTableValues)
 (define-cproc hash-table-stat (ht::<hash-table>)   Scm_HashTableStat)
 
+;; used in some internal routines
+(define (hash-table-fold ht kons knil)
+  (assume-type ht <hash-table>)
+  (let1 i ((with-module gauche.internal %hash-table-iter) ht)
+    (let loop ([r knil])
+      (receive [k v] (i (with-module gauche.internal *unique*))
+        (if (eq? k (with-module gauche.internal *unique*))
+          r
+          (loop (kons k v r)))))))
+
 ;; conversion to/from hash-table
 (define (alist->hash-table a . opt-cmpr)
   (rlet1 tb (apply make-hash-table opt-cmpr)
@@ -335,6 +358,276 @@
 (define (hash-table->alist h)
   (hash-table-map h cons))
 
+(define unique (cons #f #f))
+
+;; vararg constructor 'hash-table' has conflicting API between
+;; legacy Gauche and R7RS scheme.hash-table (srfi-125).
+;; R7RS's API is more convenient to use, so we eventually
+;; switch to it.  Meantime, we provide original hash-table
+;; with a different name, hash-table-from-pairs, and R7RS
+;; hash-table as hash-table-r7.
+(define (hash-table-from-pairs cmpr . kvs)
+  (rlet1 h (make-hash-table cmpr)
+    (for-each (^[kv] (hash-table-put! h (car kv) (cdr kv))) kvs)))
+
+(define hash-table hash-table-from-pairs) ; transient
+
+(define (hash-table-r7 cmpr . kvs) ; r7rs's hash-table
+  (rlet1 h (make-hash-table cmpr)
+    (doplist [(k v) kvs] (hash-table-put! h k v))))
+
+(define (hash-table-empty? h) (zero? (hash-table-num-entries h))) ; r7rs
+(define hash-table-contains? hash-table-exists?) ; r7rs
+(define hash-table-size hash-table-num-entries)  ; r7rs
+
+(define (hash-table-mutable? ht)      ; r7rs
+  (assume-type ht <hash-table>)
+  #t)
+
+(define (hash-table-empty-copy ht)      ; r7rs
+  (make-hash-table (hash-table-comparator ht)))
+
+(define (hash-table-entries ht)         ; r7rs
+  ;; can be more efficient
+  (values (hash-table-keys ht) (hash-table-values ht)))
+
+(define (hash-table-unfold p f g seed cmpr . args) ; r7rs
+  (rlet1 ht (apply make-hash-table cmpr args)
+    (do ([seed seed (g seed)])
+        [(p seed)]
+      (receive [k v] (f seed)
+        (hash-table-put! ht k v)))))
+
+;; r7rs accessors
+
+(define (hash-table-ref ht key :optional failure (success identity)) ;r7rs
+  ;; We avoid giving default value to failure, for the default closure
+  ;; needs to enclose key and it's costly.
+  (let1 v (hash-table-get ht key unique)
+    (if (eq? v unique)
+      (if (undefined? failure)
+        (error "hash table doesn't have an entry for key:" key)
+        (failure))
+      (success v))))
+
+(define (hash-table-ref/default ht key default) ; r7rs
+  (hash-table-get ht key default))
+
+(define (hash-table-set! ht . kvs) ; r7rs
+  (doplist [[k v] kvs]
+    (hash-table-put! ht k v)))
+
+(define (hash-table-intern!-r7 ht key failure) ; r7rs
+  ;; we can't use hash-table-adjoin!, for we have to evaluate failure thunk
+  ;; after we know the entry's not in the table.
+  (let1 v (hash-table-get ht key unique)
+    (if (eq? v unique)
+      (rlet1 v (failure)
+        (hash-table-put! ht key v))
+      v)))
+
+(define (hash-table-delete!-r7 ht . keys) ; r7rs
+  (fold (^[key count] (if (hash-table-delete! ht key) (+ count 1) count))
+        0 keys))
+        
+(define (hash-table-update!/default ht key updater default) ;r7rs
+  (hash-table-update! ht key updater default))
+
+(define (hash-table-update!-r7 ht key updater ; r7rs
+                               :optional failure (success identity))
+  (let1 v (hash-table-get ht key unique)
+    (if (eq? v unique)
+      (if (undefined? failure)
+        (error "hash table doesn't have an entry for key:" key)
+        (hash-table-put! ht key (updater (failure))))
+      (hash-table-put! ht key (updater (success v))))))
+
+(define (hash-table-pop!-r7 ht)         ; r7rs
+  (assume-type ht <hash-table>)
+  (let1 i ((with-module gauche.internal %hash-table-iter) ht)
+    (receive [k v] (i unique)
+      (when (eq? k unique)
+        (error "empty hash table can't be popped:" ht))
+      (hash-table-delete! ht k)
+      (values k v))))
+
+;; hash-table-map also disagree between Gauche and R7RS.
+;; We keep Gauche API, for we have consistent interface with other *-map
+;; procedures.  We add a check in case the caller mistook this with R7RS one.
+(define (hash-table-map ht proc :optional arg)
+  (when (not (undefined? arg))
+    (error "Gauche's built-in hash-table-map is called with R7RS interface. \
+            Use hash-table-map-r7, or say (use scheme.hash-table)."))
+  (assume-type ht <hash-table>)
+  (let1 i ((with-module gauche.internal %hash-table-iter) ht)
+    (let loop ([r '()])
+      (receive [k v] (i unique)
+        (if (eq? k unique)
+          r
+          (loop (cons (proc k v) r)))))))
+
+;; This is R7RS version of hash-table-map
+(define (hash-table-map-r7 proc cmpr ht) ; r7rs
+  (assume-type cmpr <comparator>)
+  (assume-type ht <hash-table>)
+  (rlet1 r (make-hash-table cmpr)
+    (hash-table-for-each ht (^[k v] (hash-table-put! r k (proc v))))))
+
+(define (hash-table-map!-r7 proc ht) ; r7rs
+  (assume-type ht <hash-table>)
+  (hash-table-for-each ht (^[k v] (hash-table-put! ht k (proc k v)))))
+
+(define (hash-table-map->list-r7 proc ht) ; r7rs
+  (hash-table-map ht proc))
+
+(define (hash-table-for-each ht proc)
+  (assume-type ht <hash-table>)
+  (let1 i ((with-module gauche.internal %hash-table-iter) ht)
+    (let loop ()
+      (receive [k v] (i unique)
+        (unless (eq? k unique)
+          (proc k v) (loop))))))
+
+(define (hash-table-for-each-r7 proc ht) ; r7rs
+  (hash-table-for-each ht proc))
+
+(define (hash-table-fold-r7 kons knil ht) ; r7rs
+  (hash-table-fold ht kons knil))
+
+(define (hash-table-prune!-r7 proc ht) ; r7rs
+  (hash-table-for-each ht (^[k v] (when (proc k v) (hash-table-delete! ht k)))))
+
+(define (hash-table-seek ht pred succ fail)
+  (assume-type ht <hash-table>)
+  (let1 i ((with-module gauche.internal %hash-table-iter) ht)
+    (let loop ()
+      (receive [k v] (i unique)
+        (cond [(eq? k unique) (fail)]
+              [(pred k v) => (^r (succ r k v))]
+              [else (loop)])))))
+
+;; hash-table-find.  This doesn't align with other '*-find' API in a way that
+;; it returns the result of PRED.
+(define (hash-table-find ht pred :optional (failure (^[] #f)))
+  (hash-table-seek ht pred (^[r k v] r) failure))
+  
+(define (hash-table-find-r7 pred ht failure) ; r7rs
+  (hash-table-seek ht pred (^[r k v] r) failure))
+
+(define (hash-table-count-r7 pred ht)   ; r7rs
+  ($ hash-table-fold ht
+     (^[k v count] (if (pred k v) (+ count 1) count)) 0))
+
+;; Set operations
+
+(define (hash-table-union! ht1 ht2)
+  (assume-type ht1 <hash-table>)
+  (assume-type ht2 <hash-table>)
+  ($ hash-table-for-each ht2
+     (^[k v] (unless (hash-table-exists? ht1 k)
+               (hash-table-put! ht1 k v))))
+  ht1)
+
+(define (hash-table-intersection! ht1 ht2)
+  (assume-type ht1 <hash-table>)
+  (assume-type ht2 <hash-table>)
+  ($ hash-table-for-each ht1
+     (^[k v] (unless (hash-table-exists? ht2 k)
+               (hash-table-delete! ht1 k))))
+  ht1)
+
+(define (hash-table-difference! ht1 ht2)
+  (assume-type ht1 <hash-table>)
+  (assume-type ht2 <hash-table>)
+  ($ hash-table-for-each ht1
+     (^[k v] (when (hash-table-exists? ht2 k)
+               (hash-table-delete! ht1 k))))
+  ht1)
+
+(define (hash-table-xor! ht1 ht2)
+  (assume-type ht1 <hash-table>)
+  (assume-type ht2 <hash-table>)
+  ($ hash-table-for-each ht2
+     (^[k v] (if (hash-table-exists? ht1 k)
+               (hash-table-delete! ht1 k)
+               (hash-table-put! ht1 k v))))
+  ht1)
+
+;; We delegate most hash calculation to the built-in default-hash.
+;; These type-specific hash functions are mostly
+;; for the compatibility of srfi-128.
+
+(define (boolean-hash obj)
+  (assume-type obj <boolean>)
+  (default-hash obj))
+
+(define (char-hash obj)
+  (assume-type obj <char>)
+  (default-hash obj))
+
+(define (char-ci-hash obj)
+  (assume-type obj <char>)
+  (default-hash (char-foldcase obj)))
+
+(define (string-hash obj)
+  (assume-type obj <string>)
+  (default-hash obj))
+
+(define (symbol-hash obj)
+  (assume-type obj <symbol>)
+  (default-hash obj))
+
+(define (number-hash obj)
+  (assume-type obj <number>)
+  (default-hash obj))
+
+;; This is a placeholder to conform srfi-128.
+(define (hash-bound) (greatest-fixnum))
+
+;; Compare two hash-tables as sets.
+(define (hash-table-compare-as-sets h1 h2
+                                    :optional (value=? equal?)
+                                              (fallback (undefined)))
+  (define unique (cons #f #f))
+  (define (fail)
+    (if (undefined? fallback)
+      (error "hash-tables can't be ordered:" h1 h2)
+      fallback))
+  ;; Returns #t iff smaller is a subset of larger.
+  (define (subset? smaller larger)
+    (hash-table-seek smaller
+                     (^[k v] (let1 w (hash-table-get larger k unique)
+                               (or (eq? unique w)
+                                   (not (value=? v w)))))
+                     (^[r k v] #f)
+                     (^[] #t)))
+
+  ;; Check comparator compatibility
+  (let ([c1 (hash-table-comparator h1)]
+        [c2 (hash-table-comparator h2)])
+    (cond [(and c1 c2 (equal? c1 c2)) c1]
+          [(or c1 c2) (error "hash-tables with different comparators can't be \
+                              compared:" h1 h2)]
+          [else (error "hash-tables don't have comparators and can't be \
+                        compared:" h1 h2)]))  
+  ;; Let's start
+  (if (eq? h1 h2)
+    0                 ;fast path
+    (let ([n1 (hash-table-num-entries h1)]
+          [n2 (hash-table-num-entries h2)])
+      (cond
+       [(= n1 n2) (if (subset? h1 h2) 0 (fail))]
+       [(< n1 n2) (if (subset? h1 h2) -1 (fail))]
+       [else      (if (subset? h2 h1) 1 (fail))]))))
+
+(define (hash-table=? value-cmpr h1 h2)
+  (or (eq? h1 h2)
+      (and (= (hash-table-num-entries h1) (hash-table-num-entries h2))
+           (equal? (hash-table-comparator h1) (hash-table-comparator h2))
+           (zero? (hash-table-compare-as-sets h1 h2
+                                              (cut =? value-cmpr <> <>)
+                                              100))))) ;; any non-zero number
+
 ;;;
 ;;; TreeMap
 ;;;
@@ -425,6 +718,7 @@
 
 (inline-stub
  (define-cfn tree-map-iter (args::ScmObj* nargs::int data::void*) :static
+   (cast void nargs) ; suppress unused var warning
    (let* ([iter::ScmTreeIter* (cast ScmTreeIter* data)]
           [e::ScmDictEntry*   (?: (SCM_FALSEP (aref args 1))
                                   (Scm_TreeIterNext iter)
@@ -459,18 +753,25 @@
  ;;
  (define-cise-stmt tree-map-closest-entry
    [(_ inclusive? lh make-result)
-    `(let* ([lo::ScmDictEntry* NULL]
-            [hi::ScmDictEntry* NULL]
-            [eq::ScmDictEntry*
-             (Scm_TreeCoreClosestEntries (SCM_TREE_MAP_CORE tm)
-                                         (cast intptr_t key)
-                                         (& lo) (& hi))])
-       (cond
-        ,@(if inclusive?
-            `([(!= eq NULL)  (,make-result (SCM_DICT_KEY eq) (SCM_DICT_VALUE eq))])
-            '())
-        [(!= ,lh NULL) (,make-result (SCM_DICT_KEY ,lh) (SCM_DICT_VALUE ,lh))]
-        [else          (,make-result key-fb val-fb)]))])
+    (if inclusive?
+      `(let* ([lo::ScmDictEntry* NULL]
+              [hi::ScmDictEntry* NULL]
+              [eq::ScmDictEntry*
+               (Scm_TreeCoreClosestEntries (SCM_TREE_MAP_CORE tm)
+                                           (cast intptr_t key)
+                                           (& lo) (& hi))])
+         (cond
+          [(!= eq NULL)  (,make-result (SCM_DICT_KEY eq) (SCM_DICT_VALUE eq))]
+          [(!= ,lh NULL) (,make-result (SCM_DICT_KEY ,lh) (SCM_DICT_VALUE ,lh))]
+          [else          (,make-result key-fb val-fb)]))
+      `(let* ([lo::ScmDictEntry* NULL]
+              [hi::ScmDictEntry* NULL])
+         (Scm_TreeCoreClosestEntries (SCM_TREE_MAP_CORE tm)
+                                     (cast intptr_t key)
+                                     (& lo) (& hi))
+         (if (!= ,lh NULL)
+           (,make-result (SCM_DICT_KEY ,lh) (SCM_DICT_VALUE ,lh))
+           (,make-result key-fb val-fb))))])
 
  (define-cise-stmt tree-map-closest-key-result [(_ k v) `(return ,k)])
  (define-cise-stmt tree-map-closest-val-result [(_ k v) `(return ,v)])
