@@ -1,7 +1,7 @@
 ;;;
 ;;; libeval.scm - eval, load and related stuff
 ;;;
-;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -70,7 +70,8 @@
                                          (suffixes *load-suffixes*)
                                          (error-if-not-found #t)
                                          (environment #f)
-                                         (ignore-coding #f))
+                                         (ignore-coding #f)
+                                         (main-script #f))
   ;; NB: 'File not found' error is handled in find-load-file.
   (and-let* ([r (find-load-file file paths suffixes
                                 :error-if-not-found error-if-not-found
@@ -80,6 +81,10 @@
            [hooked? (pair? (cddr r))]
            [opener (if hooked? (caddr r) open-input-file)]
            [port (guard (e [else e]) (opener path))])
+      (when main-script 
+        ;; record full path of the script
+        (script-file (sys-normalize-pathname path
+                                             :absolute #t :canonicalize #t)))
       (when (%load-verbose?)
         (format (current-error-port) ";;~aLoading ~a~a...\n"
                 (make-string (* (length (current-load-history)) 2) #\space)
@@ -110,7 +115,8 @@
         [prev-next    (current-load-next)]
         [prev-reader-lexical-mode (reader-lexical-mode)]
         [prev-eval-situation (vm-eval-situation)]
-        [prev-read-context (current-read-context)])
+        [prev-read-context (current-read-context)]
+        [load-read-context (%new-read-context-for-load)])
 
     (define (setup-load-context)
       (when (port-closed? port) (error "port already closed:" port))
@@ -124,7 +130,6 @@
                (list #f))
              prev-history))
       (vm-eval-situation SCM_VM_LOADING)
-      (current-read-context (%new-read-context-for-load))
       (%record-load-stat (or (current-load-path) "(unnamed source)")))
 
     (define (restore-load-context)
@@ -139,6 +144,14 @@
       (%record-load-stat #f)
       (%port-unlock! port))
 
+    ;; Read a source form with load-read-context.  We don't use dynamic-wind,
+    ;; for the error from read will be captured by the guard and the context
+    ;; will be restored anyway.
+    (define (read+ port)
+      (current-read-context load-read-context)
+      (begin0 (read port)
+        (current-read-context prev-read-context)))
+
     (guard (e [else (let1 e2 (if (condition? e)
                                ($ make-compound-condition e
                                   $ make <load-condition-mixin>
@@ -149,7 +162,11 @@
                       (restore-load-context)
                       (raise e2))])
       (setup-load-context)
-      (do ([s (read port) (read port)])
+      ;; Discard BOM
+      (when (eq? (gauche-character-encoding) 'utf-8)
+        (when (eqv? (peek-char port) #\ufeff)
+          (read-char port)))
+      (do ([s (read+ port) (read+ port)])
           [(eof-object? s)]
         (eval s #f)))
     (restore-load-context)
@@ -205,6 +222,7 @@
 (define-cproc dynamic-load (file::<string>
                             :key (init-function #f)
                             (export-symbols #f)); for backward compatibility
+  (cast void export-symbols) ; suppress unused var warning
   (return (Scm_DynLoad file init_function 0)))
 
 ;; API
@@ -250,6 +268,8 @@
 ;; more a procedure than a compiler syntax---any ideas?
 (select-module gauche)
 (define-macro (add-load-path path . args)
+  (unless (string? path)
+    (error "add-load-path requires a literal string as path, but got:" path))
   (let ([afterp (or (memq #t args) (memq :after args))]
         ;; If :relative is given, we trust the programmer to give a relative
         ;; pathname to PATH.
@@ -447,6 +467,11 @@
 ;;;
 (select-module gauche.internal)
 (define (%invoke-other-version version args)
+  ;; Catch a common mistake that the user run 'gosh -version' to
+  ;; see the version.
+  (when (equal? version "ersion")
+    (exit 1 "No such version `ersion'.  To display Gauche version, \
+             run 'gosh -V'.  To see all options, run 'gosh -h'."))
   ;; Note: We assume other versions of gauche is installed under the
   ;; same exec_prefix: $exec_prefix/lib/gauche-ABIVERSION/VERSION/ARCH/
   (let* ([prefix (sys-normalize-pathname
@@ -463,10 +488,8 @@
                        [(#/^-v/ (car args)) (loop (cdr args) r)]
                        [else (loop (cdr args) (cons (car args) r))]))])
     (unless (pair? goshes)
-      (format (current-error-port)
-              "No installed Gauche with version ~a under ~a.\n"
-              version prefix)
-      (exit 1))
+      (exit 1 "No installed Gauche with version ~a under ~a.\n"
+            version prefix))
     (sys-exec (car goshes) (cons (car goshes) args))))
 
 ;;;
@@ -483,21 +506,16 @@
     (Scm_Exit status)))
 
 ;; API
-;; exit handler.  we don't want to import the fluff with gauche.parameter,
-;; so we manually allocate parameter slot.
+;; exit handler.
 (select-module gauche.internal)
-(define-in-module gauche exit-handler
-  (let1 index (%vm-make-parameter-slot)
-    ;; set default exit handler
-    (%vm-parameter-set! index #f
-                        (^[code fmt args]
-                          (when fmt
-                            (apply format (standard-error-port) fmt args)
-                            (newline (standard-error-port)))))
-    (^ maybe-arg
-      (rlet1 old (%vm-parameter-ref index #f)
-        (when (pair? maybe-arg)
-          (%vm-parameter-set! index #f (car maybe-arg)))))))
+(inline-stub
+ (initcode
+  (Scm_BindPrimitiveParameter (Scm_GaucheModule) "exit-handler"
+                              SCM_FALSE 0)))
+(exit-handler (^[code fmt args]
+                (when fmt
+                  (apply format (standard-error-port) fmt args)
+                  (newline (standard-error-port)))))
 
 ;; API
 (define-in-module gauche (exit :optional (code 0) (fmt #f) :rest args)
@@ -534,9 +552,11 @@
 ;;; Some system introspection
 ;;;
 
-(select-module gauche)
 ;; API
 ;; Obtain info about gauche itself
+
+(select-module gauche)
+
 (define-cproc gauche-version () ::<const-cstring> (return GAUCHE_VERSION))
 (define-cproc gauche-architecture () ::<const-cstring> Scm_HostArchitecture)
 (define-cproc gauche-library-directory () Scm_LibraryDirectory)
@@ -545,21 +565,30 @@
 (define-cproc gauche-site-architecture-directory () Scm_SiteArchitectureDirectory)
 (define-cproc gauche-dso-suffix () ::<const-cstring> (return SHLIB_SO_SUFFIX))
 
-;; Temporary - only usable on Windows(mingw) and Darwin; used to replace
-;; '@' in the paths embedded in gauche.config.
-(define-cproc %gauche-runtime-directory () Scm__RuntimeDirectory)
-
-;; API
-;; Command line - R7RS adds 'command-line' procedure.  We provide it as
-;; a predefined parameter.  Like exit-handler, we manually allocate a
-;; parametre slot to avoid importing gauche.parameter.
 (select-module gauche.internal)
-(define-in-module gauche command-line
-  (let1 index (%vm-make-parameter-slot)
-    (^ maybe-arg
-      (rlet1 old (%vm-parameter-ref index '())
-        (when (pair? maybe-arg)
-          (%vm-parameter-set! index #f (car maybe-arg)))))))
+(define-cproc %gauche-runtime-directory () Scm_RuntimeDirectory)
+(define-cproc %gauche-libgauche-path () Scm_LibgauchePath)
+(define-cproc %gauche-executable-path () Scm_ExecutablePath)
+
+;; srfi-176
+(define-in-module gauche (version-alist)
+  (let1 fs ((with-module gauche.internal cond-features))
+    `((version ,(gauche-version))
+      (command "gosh")
+      (scheme.id gauche)
+      (languages scheme r5rs r7rs)
+      (encodings ,(gauche-character-encoding))
+      (website "https://practical-scheme.net/gauche")
+      (build.platform ,(gauche-architecture))
+      (scheme.path ,@*load-path*)
+      (gauche.threads ,(cond
+                        [(assq 'gauche.sys.pthreads fs) 'pthreads]
+                        [(assq 'gauche.sys.wthreads fs) 'wthreads]
+                        [else 'none]))
+      (gauche.net.tls ,@(cond-list
+                         [(assq 'gauche.net.tls.axtls fs) 'axtls]
+                         [(assq 'gauche.net.tls.mbedtls fs) 'mbedtls])))))
+
 ;;
 ;; External view of VM.
 ;;
@@ -662,40 +691,17 @@
           (reverse r `((,(car si) ,(cadr si) ,obj)))))
       '())))
 
-;; Parameter internal API
-;; These will be called by the public API in gauche.parameter.  The protocol
-;; is a bit weird, for the Scheme-level parameter has its own instance
-;; definition distinct from C-level ScmParameterLoc.  Eventually it would
-;; be nicer if we could merge two.
-(select-module gauche.internal)
-
-(define-cproc %vm-make-parameter-slot () ::<int>
-  (let* ([loc::ScmParameterLoc])
-    (Scm_InitParameterLoc (Scm_VM) (& loc) SCM_FALSE)
-    (return (ref loc index))))
-
-(define-cproc %vm-parameter-ref (index::<int> init-value)
-  (let* ([loc::ScmParameterLoc])
-    (set! (ref loc index) index
-          (ref loc initialValue) init-value)
-    (return (Scm_ParameterRef (Scm_VM) (& loc)))))
-
-(define-cproc %vm-parameter-set! (index::<int> init-value new-value)
-  (let* ([loc::ScmParameterLoc])
-    (set! (ref loc index) index
-          (ref loc initialValue) init-value)
-    (return (Scm_ParameterSet (Scm_VM) (& loc) new-value))))
-
-;; TRANSIENT
-;; For the backward compatibility---files precompiled by 0.9.2 or before
-;; can contain reference to the old API (as the result of expansion of
-;; parameterize).  These definition converts them to the new API.
-;; Will be removed on 1.0 release.
-(select-module gauche)
-(define (%vm-make-parameter-slot)
-  (values ((with-module gauche.internal %vm-make-parameter-slot)) 0))
-(define (%vm-parameter-ref index id)
-  ((with-module gauche.internal %vm-parameter-ref) index #f))
-(define (%vm-parameter-set! index id value)
-  ((with-module gauche.internal %vm-parameter-set!) index #f value))
-
+;; Returns source file info attached to OBJ.  If there's no info
+;; attached, returns #f.
+;; The return value can be either (<filename> <line-no>) or
+;; (<filename> <line-no> <original-form>).  The latter happens when
+;; OBJ is a result of macro expansion, and <orginal-form> being
+;; the original source form.
+(define-in-module gauche (debug-source-info obj)
+  (and-let1 sis ((with-module gauche.internal %source-info) obj)
+    (any (^[si] ;; si :: (<file> <line> <form>)
+           (and (car si) (cadr si)
+                (if (eq? (caddr si) obj)
+                  `(,(car si) ,(cadr si))
+                  si)))
+         (reverse sis))))
