@@ -1,7 +1,7 @@
 ;;;
 ;;; libexc.scm - errors and exceptions
 ;;;
-;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -36,6 +36,9 @@
  (declcode (.include <gauche/class.h>
                      <gauche/exception.h>)))
 
+(use util.match)
+(declare (keep-private-macro define-condition-type condition))
+
 (define <exception> <condition>) ;; backward compatibility
 
 ;;;
@@ -52,15 +55,22 @@
   (unless (SCM_PROCEDURE_THUNK_P thunk)
     (Scm_Error "thunk required, but got %S" thunk))
   (if rewind-before
-    (result (Scm_VMWithGuardHandler handler thunk))
-    (result (Scm_VMWithErrorHandler handler thunk))))
+    (return (Scm_VMWithGuardHandler handler thunk))
+    (return (Scm_VMWithErrorHandler handler thunk))))
 
-(define-cproc report-error (exception :optional port)
-  ;; TRANSIENT: change this to Scm_ReportError when switching API to 0.95.
-  Scm_ReportError2)
+(define-cproc report-error (exception :optional (port (current-error-port)))
+  Scm_ReportError)
 
 (define-cproc condition-type-name (c) Scm_ConditionTypeName)
 
+;; A convenience procedure; safe for non-message condition objects.
+;; (cf. error-object-message of r7rs)
+(define (condition-message e :optional (fallback #f))
+  (if (and (condition? e)
+           (condition-has-type? e <message-condition>))
+    (condition-ref e 'message)
+    fallback))
+
 (define (print-default-error-heading exc out)
   (guard (e [else (display "*** ERROR:" out)
                   (display exc out)
@@ -94,7 +104,7 @@
 
 (select-module gauche)
 (define-cproc current-exception-handler ()
-  (result (-> (Scm_VM) exceptionHandler)))
+  (return (-> (Scm_VM) exceptionHandler)))
 
 (define-cproc with-exception-handler (handler thunk)
   Scm_VMWithExceptionHandler)
@@ -104,10 +114,16 @@
 (define-cproc %raise (exception :optional (non-continuable? #f))
   (let* ([flags::u_long
           (?: (SCM_FALSEP non-continuable?) 0 SCM_RAISE_NON_CONTINUABLE)])
-    (result (Scm_Raise2 exception flags))))
+    (return (Scm_Raise exception flags))))
 
-;; For r7rs compatible guard
-(define-cproc %reraise (exception) Scm_VMReraise)
+;; For r7rs compatible guard.
+;; %reraise must be called at the tail position of the error handler.
+;; What it does is to restore escape point and set ep->reraied flag, then
+;; return.  Subsequently the control returns from the error handler to
+;; Scm_VMDefaultExceptionHandler, which restores dynamic environment
+;; and returns to the original caller of raise.
+(define-cproc %reraise ()
+  (return (Scm_VMReraise)))
 
 ;; srfi-18 raise
 (define-in-module gauche (raise c) (%raise c))
@@ -125,6 +141,60 @@
 (define-cproc make-compound-condition (:rest conditions)
   Scm_MakeCompoundCondition)
 
+(define (make-condition-type name parent field-names)
+  (unless (condition-type? parent)
+    (error "condition-type required as a parent of make-condition-type, but got:" parent))
+  (make <condition-meta>
+    :name name
+    :supers (list parent)
+    :slots  field-names))
+
+(define (condition-type? obj)
+  (is-a? obj <condition-meta>))
+
+(define (make-condition type . initargs)
+  (unless (condition-type? type)
+    (error "make-condition requires a condition type, but got:" type))
+  (let ((c (make type)))
+    (let loop ((args initargs))
+      (cond ((null? args) c)
+            ((null? (cdr args))
+             (error "make-condition is given non-even initargs:" initargs))
+            ((slot-exists? c (car args))
+             (slot-set! c (car args) (cadr args))
+             (loop (cddr args)))
+            (else
+             (errorf "condition type ~s doesn't have a field ~s"
+                     type (car args)))))))
+
+(define (condition-ref c slot)
+  (slot-ref c slot))  ;; compound condition traps slot-missing
+
+(define (extract-condition c type)
+  (unless (condition-has-type? c type)
+    (errorf "cannot extract a condition of type ~s from a condition ~s"
+            type c))
+  (let ((cc (make type)))
+    (let loop ((slots (class-slots type)))
+      (if (null? slots)
+        cc
+        (let ((sn (slot-definition-name (car slots))))
+          (when (slot-bound? c sn)
+            (slot-set! cc sn (slot-ref c sn)))
+          (loop (cdr slots)))))))
+
+;; Aliases for srfi-35/srfi-36 compatibility
+(define &condition   <condition>)
+(define &message     <message-condition>)
+(define &serious     <serious-condition>)
+(define &error       <error>)
+(define &i/o-error   <io-error>)
+(define &i/o-port-error <port-error>)
+(define &i/o-read-error <io-read-error>)
+(define &i/o-write-error <io-write-error>)
+(define &i/o-closed-error <io-closed-error>)
+(define &read-error  <read-error>)
+
 ;;;
 ;;; Thread exception classes
 ;;;
@@ -134,10 +204,12 @@
 (select-module gauche)
 (inline-stub
  (define-cfn thread_exception_allocate (klass::ScmClass* initargs) :static
+   (cast void initargs)                 ;suppress unused warning
    (return (Scm_MakeThreadException klass NULL)))
 
  (define-cfn thread_exception_print (obj port::ScmPort* ctx::ScmWriteContext*)
    ::void :static
+   (cast void ctx)                      ;suppress unused warning
    (let* ([k::ScmClass* (SCM_CLASS_OF obj)]
           [exc::ScmThreadException* (SCM_THREAD_EXCEPTION obj)]
           [cname (Scm_ShortClassName k)])
@@ -148,12 +220,14 @@
 
  (define-cfn uncaught_exception_print (obj port::ScmPort* ctx::ScmWriteContext*)
    ::void :static
+   (cast void ctx)                      ;suppress unused warning
    (let* ([exc::ScmThreadException* (SCM_THREAD_EXCEPTION obj)])
      (Scm_Printf port "#<uncaught-exception in thread %S: %S>"
                  (SCM_OBJ_SAFE (-> exc thread)) (-> exc data))))
 
  (define-cfn terminated_thread_print (obj port::ScmPort* ctx::ScmWriteContext*)
    ::void :static
+   (cast void ctx)                      ;suppress unused warning
    (let* ([exc::ScmThreadException* (SCM_THREAD_EXCEPTION obj)])
      (Scm_Printf port "#<terminated-thread-exception: %S terminated by %S>"
                  (SCM_OBJ_SAFE (-> exc thread)) (-> exc data))))
@@ -228,12 +302,8 @@
    NULL
   };"
 
- (define-cclass <mixin-condition>
-   "ScmCondition*" "Scm_MixinConditionClass"
-   (c "mixin_condition_cpa+1")
-   () ())
- 
  (define-cfn load-condition-mixin-allocate (klass::ScmClass* initargs) :static
+   (cast void initargs)                 ;suppress unused warning
    (let* ([c::ScmLoadConditionMixin* (SCM_NEW_INSTANCE ScmLoadConditionMixin klass)])
      (set! (-> c history) SCM_FALSE)
      (set! (-> c port) SCM_FALSE)
@@ -247,6 +317,7 @@
    (allocator (c "load_condition_mixin_allocate")))
 
  (define-cfn compile-error-mixin-allocate (klass::ScmClass* initargs) :static
+   (cast void initargs)                 ;suppress unused warning
    (let* ([c::ScmCompileErrorMixin* (SCM_NEW_INSTANCE ScmCompileErrorMixin klass)])
      (set! (-> c expr) SCM_FALSE)
      (return (SCM_OBJ c))))
@@ -264,6 +335,7 @@
  ;; In C-level all filename mixin classes share one struct definition, and
  ;; one allocator.
  (define-cfn filename-error-mixin-allocate (klass::ScmClass* initargs) :static
+   (cast void initargs) ; suppress unused var warning
    (let* ([c::ScmFilenameErrorMixin* (SCM_NEW_INSTANCE ScmFilenameErrorMixin klass)])
      (set! (-> c filename) SCM_FALSE)
      (return (SCM_OBJ c))))
