@@ -11,6 +11,7 @@
  "ScmClass *fptr_simple, *fptr_unique, *fptr_maybe;"
 
  (define-cfn print-fptr (obj sink::ScmPort* mode::ScmWriteContext*) ::void
+   (cast void mode)                     ; suppress unused parameter warning
    (let* ([k::ScmClass* (Scm_ClassOf obj)])
      (Scm_Printf sink "#<fptr-%s %p>"
                  (?: (== k fptr_simple) "simple"
@@ -44,6 +45,20 @@
                      SCM_FOREIGN_POINTER_MAP_NULL))
   ))
 
+;; Pathname replacement
+(inline-stub
+ (declcode "#define PATH_ALLOC(n) SCM_MALLOC_ATOMIC(n)")
+ (declcode "#define PATH_ERROR(...) Scm_Error(__VA_ARGS__)")
+ (declcode "#include \"paths.c\"")
+
+ (define-cproc substitute-all (input::<const-cstring>
+                               mark::<const-cstring>
+                               subst::<const-cstring>)
+   ::<const-cstring>
+   (cast void replace_install_dir) ; to avoid unused warning
+   (cast void get_libgauche_path)  ; ditto
+   (return (substitute_all input mark subst))))
+
 ;; Entry point
 
 (define (main args)
