@@ -1,7 +1,7 @@
 ;;;
 ;;; libio.scm - builtin port and I/O procedures
 ;;;
-;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -36,6 +36,7 @@
 (inline-stub
  (declcode (.include <gauche/vminsn.h>
                      <gauche/class.h>
+                     <gauche/exception.h>
                      <gauche/priv/portP.h>
                      <gauche/priv/writerP.h>
                      <stdlib.h>
@@ -96,6 +97,11 @@
 (define-cproc standard-error-port (:optional (p::<output-port>? #f))
   (return (?: p (Scm_SetStderr p) (Scm_Stderr))))
 
+(inline-stub
+ (initcode
+  (Scm_BindPrimitiveParameter (Scm_GaucheModule) 
+                              "current-trace-port"
+                              (Scm_Stderr) 0)))
 
 ;;
 ;; Query and low-level properties
@@ -106,9 +112,15 @@
 (define-cproc port-name (port::<port>) Scm_PortName)
 (define-cproc port-current-line (port::<port>) ::<fixnum> Scm_PortLine)
 
-(define-cproc port-file-number (port::<port>)
+(define-cproc port-file-number (port::<port> :optional (dup?::<boolean> #f))
   (let* ([i::int (Scm_PortFileNo port)])
-    (return (?: (< i 0) SCM_FALSE (Scm_MakeInteger i)))))
+    (when (< i 0) (return SCM_FALSE))
+    (when dup?
+      (let* ([r::int 0])
+        (SCM_SYSCALL r (dup i))
+        (when (< r 0) (Scm_SysError "dup(2) failed"))
+        (set! i r)))
+    (return (Scm_MakeInteger i))))
 (define-cproc port-fd-dup! (dst::<port> src::<port>) ::<void> Scm_PortFdDup)
 
 (define-cproc port-attribute-set! (port::<port> key val)
@@ -185,16 +197,25 @@
 (define-cproc %open-input-file (path::<string>
                                 :key (if-does-not-exist :error)
                                 (buffering #f)
-                                (element-type :character))
-  (let* ([ignerr::int FALSE])
+                                (element-type :binary))
+  (let* ([ignerr::int FALSE]
+         [flags::int O_RDONLY])
     (cond [(SCM_FALSEP if-does-not-exist) (set! ignerr TRUE)]
           [(not (SCM_EQ if-does-not-exist ':error))
            (Scm_TypeError ":if-does-not-exist" ":error or #f"
                           if-does-not-exist)])
+    (unless (or (SCM_EQ element-type ':character)
+                (SCM_EQ element-type ':binary))
+      (Scm_Error "bad element-type argument: either :character or :binary \
+                  expected, but got %S" element-type))
+    (.if "defined(O_BINARY) && defined(O_TEXT)"
+         (if (SCM_EQ element-type ':character)
+           (logior= flags O_TEXT)
+           (logior= flags O_BINARY)))
     (let* ([bufmode::int (Scm_BufferingMode buffering SCM_PORT_INPUT
                                             SCM_PORT_BUFFER_FULL)]
            [o (Scm_OpenFilePort (Scm_GetStringConst path)
-                                O_RDONLY bufmode 0)])
+                                flags bufmode 0)])
       (when (and (SCM_FALSEP o) (not (%open/allow-noexist? ignerr)))
         (Scm_SysError "couldn't open input file: %S" path))
       (return o))))
@@ -205,10 +226,18 @@
                                  (if-does-not-exist :create)
                                  (mode::<fixnum> #o666)
                                  (buffering #f)
-                                 (element-type :character))
+                                 (element-type :binary))
   (let* ([ignerr-noexist::int FALSE]
          [ignerr-exist::int FALSE]
          [flags::int O_WRONLY])
+    (unless (or (SCM_EQ element-type ':character)
+                (SCM_EQ element-type ':binary))
+      (Scm_Error "bad element-type argument: either :character or :binary \
+                  expected, but got %S" element-type))
+    (.if "defined(O_BINARY) && defined(O_TEXT)"
+         (if (SCM_EQ element-type ':character)
+           (logior= flags O_TEXT)
+           (logior= flags O_BINARY)))
     ;; check if-exists flag
     (cond
      [(SCM_EQ if-exists ':append) (logior= flags O_APPEND)]
@@ -243,21 +272,41 @@
 
 (define-cproc open-input-fd-port (fd::<fixnum>
                                   :key (buffering #f)
-                                  (owner?::<boolean> #f)
+                                  (owner? #f)
                                   (name #f))
   (let* ([bufmode::int (Scm_BufferingMode buffering SCM_PORT_INPUT
                                           SCM_PORT_BUFFER_FULL)])
     (when (< fd 0) (Scm_Error "bad file descriptor: %ld" fd))
-    (return (Scm_MakePortWithFd name SCM_PORT_INPUT fd bufmode ownerP))))
+    (cond
+     [(SCM_EQ owner? 'dup) 
+      (let* ([r::int 0])
+        (SCM_SYSCALL r (dup fd))
+        (when (< r 0) (Scm_SysError "dup(2) failed"))
+        (set! fd r))]
+     [(not (SCM_BOOLP owner?))
+      (Scm_Error "owner? argument must be either #f, #t or a symbol dup, \n\
+                  but go t%S" owner?)])
+    (return (Scm_MakePortWithFd name SCM_PORT_INPUT fd bufmode
+                                (not (SCM_FALSEP owner?))))))
 
 (define-cproc open-output-fd-port (fd::<fixnum>
                                    :key (buffering #f)
-                                   (owner?::<boolean> #f)
+                                   (owner? #f)
                                    (name #f))
   (let* ([bufmode::int (Scm_BufferingMode buffering SCM_PORT_OUTPUT
                                           SCM_PORT_BUFFER_FULL)])
     (when (< fd 0) (Scm_Error "bad file descriptor: %d" fd))
-    (return (Scm_MakePortWithFd name SCM_PORT_OUTPUT fd bufmode owner?))))
+    (cond
+     [(SCM_EQ owner? 'dup) 
+      (let* ([r::int 0])
+        (SCM_SYSCALL r (dup fd))
+        (when (< r 0) (Scm_SysError "dup(2) failed"))
+        (set! fd r))]
+     [(not (SCM_BOOLP owner?))
+      (Scm_Error "owner? argument must be either #f, #t or a symbol dup, \n\
+                  but go t%S" owner?)])
+    (return (Scm_MakePortWithFd name SCM_PORT_OUTPUT fd bufmode 
+                                (not (SCM_FALSEP owner?))))))
 
 ;; Buffered port
 (select-module gauche)
@@ -266,26 +315,26 @@
  ;; NB: the interface may be changed soon!!
  (define-cfn bufport-closer (p::ScmPort*) ::void :static
    (when (== (SCM_PORT_DIR p) SCM_PORT_OUTPUT)
-     (let* ((scmflusher (SCM_OBJ (ref (-> p src) buf data)))
-            (siz::int (cast int (- (ref (-> p src) buf current)
-                                   (ref (-> p src) buf buffer)))))
+     (let* ((scmflusher (SCM_OBJ (-> (PORT_BUF p) data)))
+            (siz::int (cast int (- (-> (PORT_BUF p) current)
+                                   (-> (PORT_BUF p) buffer)))))
        (when (> siz 0)
          (Scm_ApplyRec1 scmflusher
-                        (Scm_MakeString (ref (-> p src) buf buffer) siz siz
+                        (Scm_MakeString (-> (PORT_BUF p) buffer) siz siz
                                         (logior SCM_STRING_INCOMPLETE
                                                 SCM_STRING_COPYING))))
        (Scm_ApplyRec1 scmflusher SCM_FALSE))))
 
- (define-cfn bufport-filler (p::ScmPort* cnt::int) ::int :static
-   (let* ([scmfiller (SCM_OBJ (ref (-> p src) buf data))]
+ (define-cfn bufport-filler (p::ScmPort* cnt::ScmSize) ::ScmSize :static
+   (let* ([scmfiller (SCM_OBJ (-> (PORT_BUF p) data))]
           [r (Scm_ApplyRec1 scmfiller (Scm_MakeInteger cnt))])
      (cond [(or (SCM_EOFP r) (SCM_FALSEP r)) (return 0)]
            [(not (SCM_STRINGP r))
             (Scm_Error "buffered port callback procedure returned non-string: %S" r)])
      (let* ([b::(const ScmStringBody*) (SCM_STRING_BODY r)]
-            [siz::int (SCM_STRING_BODY_SIZE b)])
+            [siz::ScmSize (SCM_STRING_BODY_SIZE b)])
        (when (> siz cnt) (set! siz cnt)) ; for safety
-       (memcpy (ref (-> p src) buf end) (SCM_STRING_BODY_START b) siz)
+       (memcpy (-> (PORT_BUF p) end) (SCM_STRING_BODY_START b) siz)
        (return (SCM_STRING_BODY_SIZE b)))))
  )
 
@@ -304,9 +353,11 @@
     (return (Scm_MakeBufferedPort SCM_CLASS_PORT SCM_FALSE SCM_PORT_INPUT TRUE (& bufrec)))))
 
 (inline-stub
- (define-cfn bufport-flusher (p::ScmPort* cnt::int forcep::int) ::int :static
-   (let* ([scmflusher (SCM_OBJ (ref (-> p src) buf data))]
-          [s (Scm_MakeString (ref (-> p src) buf buffer) cnt cnt
+ (define-cfn bufport-flusher (p::ScmPort* cnt::ScmSize forcep::int)
+   ::ScmSize :static
+   (cast void forcep) ; suppress unused var warning
+   (let* ([scmflusher (SCM_OBJ (-> (PORT_BUF p) data))]
+          [s (Scm_MakeString (-> (PORT_BUF p) buffer) cnt cnt
                              (logior SCM_STRING_INCOMPLETE SCM_STRING_COPYING))])
      (Scm_ApplyRec1 scmflusher s)
      (return cnt)))
@@ -336,22 +387,16 @@
 ;;   port source path.  Until then, be careful to name the ports.
 (select-module gauche)
 
-(define-cproc open-input-string (string::<string> :key (private?::<boolean> #f)
-                                                       name)
-  (let* ([p (Scm_MakeInputStringPort string private?)])
-    ;; NB: We don't have an API to set port name in the constructor; but
-    ;; this is a bad manner.  Don't duplicate this.
-    (unless (SCM_UNBOUNDP name)
-      (set! (-> (SCM_PORT p) name) name))
-    (return p)))
+(define-cproc open-input-string (string::<string>
+                                 :key (private?::<boolean> #f)
+                                      (name "(input string port)"))
+  (let* ([flags::u_long (?: private? SCM_PORT_STRING_PRIVATE 0)])
+    (return (Scm_MakeInputStringPortFull string name flags))))
 
-(define-cproc open-output-string (:key (private?::<boolean> #f) name)
-  (let* ([p (Scm_MakeOutputStringPort private?)])
-    ;; NB: We don't have an API to set port name in the constructor; but
-    ;; this is a bad manner.  Don't duplicate this.
-    (unless (SCM_UNBOUNDP name)
-      (set! (-> (SCM_PORT p) name) name))
-    (return p)))
+(define-cproc open-output-string (:key (private?::<boolean> #f)
+                                       (name "(output string port)"))
+  (let* ([flags::u_long (?: private? SCM_PORT_STRING_PRIVATE 0)])
+    (return (Scm_MakeOutputStringPortFull name flags))))
 
 (define-cproc get-output-string (oport::<output-port>) ;SRFI-6
   (return (Scm_GetOutputString oport 0)))
@@ -372,6 +417,17 @@
 ;; Miscellaneous
 ;;
 
+;; srfi-191
+(define-cproc port-has-port-position? (port::<port>) ::<boolean>
+  (return (Scm_PortPositionable port FALSE)))
+(define-cproc port-has-set-port-position!? (port::<port>) ::<boolean>
+  (return (Scm_PortPositionable port TRUE)))
+
+(define-cproc port-position (port::<port>)
+  (return (Scm_GetPortPosition port)))
+(define-cproc set-port-position! (port::<port> pos)
+  (return (Scm_SetPortPosition port pos)))
+
 (select-module gauche)
 (inline-stub
  (define-enum SEEK_SET)
@@ -389,12 +445,14 @@
 
 ;; useful for error messages
 (define (port-position-prefix port)
-  (if-let1 n (port-name port)
-    (let1 l (port-current-line port)
-      (if (positive? l)
-        (format #f "~s:line ~a: " n l)
-        (format #f "~s: " n))
-      "")))
+  (if (port? port)
+    (if-let1 n (port-name port)
+      (let1 l (port-current-line port)
+        (if (positive? l)
+          (format #f "~s:line ~a: " n l)
+          (format #f "~s: " n))
+        ""))
+    "???"))
 
 (select-module gauche.internal)
 
@@ -414,10 +472,13 @@
 
 (inline-stub
  (define-cfn write_state_allocate (klass::ScmClass* initargs) :static
+   (cast void klass)                    ; suppress unused var warning
+   (cast void initargs)                 ; suppress unused var warning
    (return (SCM_OBJ (Scm_MakeWriteState NULL))))
 
  (define-cfn write_state_print (obj port::ScmPort* ctx::ScmWriteContext*)
    ::void :static
+   (cast void ctx)                      ; suppress unused var warning
    (Scm_Printf port "#<write-state %p>" obj))
 
  (define-cclass <write-state>
@@ -432,11 +493,10 @@
 (define-cproc %port-write-state (port::<port>)
   (setter (port::<port> obj) ::<void>
           (if (SCM_WRITE_STATE_P obj)
-            (set! (-> port writeState) (SCM_WRITE_STATE obj))
-            (set! (-> port writeState) NULL)))
-  (return (?: (-> port writeState)
-              (SCM_OBJ (-> port writeState))
-              SCM_FALSE)))
+            (Scm_PortWriteStateSet port (SCM_WRITE_STATE obj))
+            (Scm_PortWriteStateSet port NULL)))
+  (let* ([r::ScmWriteState* (Scm_PortWriteState port)])
+    (return (?: r (SCM_OBJ r) SCM_FALSE))))
 
 (define-cproc %port-lock! (port::<port>) ::<void>
   (let* ([vm::ScmVM* (Scm_VM)])
@@ -508,15 +568,21 @@
 (define-cproc byte-ready? (:optional (port::<input-port> (current-input-port)))
   ::<boolean> Scm_ByteReady)
 
+(define u8-ready? byte-ready?)          ;R7RS
+
 (define-cproc read-byte (:optional (port::<input-port> (current-input-port)))
   (let* ([b::int])
     (SCM_GETB b port)
     (return (?: (< b 0) SCM_EOF (SCM_MAKE_INT b)))))
 
+(define read-u8 read-byte)              ;R7RS
+
 (define-cproc peek-byte (:optional (port::<input-port> (current-input-port)))
   (let* ([b::int (Scm_Peekb port)])
     (return (?: (< b 0) SCM_EOF (SCM_MAKE_INT b)))))
 
+(define peek-u8 peek-byte)              ;R7RS
+
 (define-cproc read-line (:optional (port::<input-port> (current-input-port))
                                    (allowbytestr #f))
   (let* ([r (Scm_ReadLine port)])
@@ -595,6 +661,23 @@
             (Scm_DStringPutz (& ds) buf nbytes)))
     (return (Scm_DStringGet (& ds) SCM_STRING_INCOMPLETE))))
 
+(define (port->string port)
+  (let1 out (open-output-string :private? #t)
+    (copy-port port out :unit 'byte)
+    (get-output-string out)))
+
+(define (port->list reader port)
+  (with-port-locking port
+    (^[]
+      (let loop ([obj (reader port)]
+                 [result '()])
+        (if (eof-object? obj)
+          (reverse! result)
+          (loop (reader port) (cons obj result)))))))
+
+(define (port->string-list port) (port->list (cut read-line <> #t) port))
+(define (port->sexp-list port)   (port->list read port))
+
 ;; Reader parameters
 (define-cproc reader-lexical-mode (:optional k)
   (if (SCM_UNBOUNDP k)
@@ -632,7 +715,8 @@
     (return (SCM_OBJ (Scm_CurrentReadContext)))
     (if (SCM_READ_CONTEXT_P ctx)
       (return (SCM_OBJ (Scm_SetCurrentReadContext (SCM_READ_CONTEXT ctx))))
-      (Scm_Error "<read-context> required, but got:" ctx))))
+      (begin (Scm_Error "<read-context> required, but got:" ctx)
+             (return SCM_UNDEFINED))))) ;dummy
 
 (define-cproc read-reference? (obj) ::<boolean> SCM_READ_REFERENCE_P)
 
@@ -648,27 +732,6 @@
 (define-in-module gauche read-with-shared-structure read)
 (define-in-module gauche read/ss read)
 
-;; srfi-105
-(select-module gauche.internal)
-(define (%xform-cexpr cex)
-  (define (simple-cexpr? op tail)
-    (cond [(not (pair? tail)) #f]
-          [(null? (cdr tail)) #t]
-          [(and (pair? (cdr tail)) (eq? op (cadr tail)))
-           (simple-cexpr? op (cddr tail))]
-          [else #f]))
-  (define (gather-args args) ; args is a list with even elements
-    (if (null? (cdr args))
-      args
-      (cons (car args) (gather-args (cddr args)))))
-  (cond [(not (pair? cex)) cex]         ;includes {} -> ()
-        [(null? (cdr cex)) (car cex)]   ; {e} -> e
-        [(and (pair? (cdr cex)) (null? (cddr cex))) cex] ; {x y} -> (x y)
-        [(and (pair? (cdr cex)) (pair? (cddr cex)) (symbol? (cadr cex))
-              (simple-cexpr? (cadr cex) (cddr cex)))
-         (cons* (cadr cex) (car cex) (gather-args (cddr cex)))]
-        [else (cons 'nfx cex)]))
-
 ;;;
 ;;; Output
 ;;;
@@ -744,6 +807,8 @@
   (SCM_PUTB byte port)
   (return 1))
 
+(define write-u8 write-byte)            ;R7RS
+
 (define-cproc write-limited (obj limit::<fixnum>
                                  :optional (port (current-output-port)))
   ::<int> (return (Scm_WriteLimited obj port SCM_WRITE_WRITE limit)))
@@ -812,6 +877,8 @@
 (select-module gauche)
 (inline-stub
  (define-cfn write_controls_allocate (klass::ScmClass* initargs) :static
+   (cast void klass)                    ; suppress unused var warning
+   (cast void initargs)                 ; suppress unused var warning
    (return (SCM_OBJ (Scm_MakeWriteControls NULL))))
 
  ;; TODO: We want to treat <write-controls> as immutable structure, but
@@ -839,11 +906,13 @@
                        obj->printWidth = SCM_INT_VALUE(value); \
                      else obj->printWidth = -1;")
     (base   :type <int>     :c-name "printBase"
-            :setter "if (SCM_INTP(value) && SCM_INT_VALUE(value) >= 2 \
-                         && SCM_INT_VALUE(value) <= 36) \
+            :setter "if (SCM_INTP(value) \
+                         && SCM_INT_VALUE(value) >= SCM_RADIX_MIN \
+                         && SCM_INT_VALUE(value) <= SCM_RADIX_MAX) \
                        obj->printBase = SCM_INT_VALUE(value); \
                      else Scm_Error(\"print-base must be an integer \
-                                    between 2 and 36, but got: %S\", value);")
+                                    between %d and %d, but got: %S\", \
+                                    SCM_RADIX_MIN, SCM_RADIX_MAX, value);")
     (radix  :type <boolean> :c-name "printRadix"
             :setter "obj->printRadix = !SCM_FALSEP(value);")
     (pretty :type <boolean> :c-name "printPretty"
@@ -1050,10 +1119,3 @@
   (^[sym port ctx]
     (port-attribute-set! port 'reader-lexical-mode 'strict-r7)
     (values)))
-
-;; HIGHLY EXPERIMENTAL
-(define-reader-directive 'c-expr
-  (^[sym port ctx]
-    ((with-module gauche.internal vm-compiler-flag-set!)
-     SCM_COMPILE_ENABLE_CEXPR)
-    (values)))
