@@ -1,7 +1,7 @@
 ;;;
 ;;;liblazy.scm - lazy constructs
 ;;;
-;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -43,16 +43,16 @@
 ;; expanded into PROMISE instruction.
 
 (select-module scheme)
-(define-cproc force (p) Scm_Force)
+(define-cproc force (p) Scm_VMForce)
 
 (select-module gauche)
 (define-cproc promise? (obj) ::<boolean> :constant
-  (result (SCM_XTYPEP obj SCM_CLASS_PROMISE)))
+  (return (SCM_XTYPEP obj SCM_CLASS_PROMISE)))
 (define-cproc eager (obj)              ;srfi-45
-  (result (Scm_MakePromise TRUE obj)))
+  (return (Scm_MakePromise TRUE obj)))
 (define-cproc promise-kind (p::<promise>)
   (setter (p::<promise> obj) ::<void> (set! (-> p kind) obj))
-  (result (-> p kind)))
+  (return (-> p kind)))
 
 ;;;
 ;;; lazy sequence
@@ -65,18 +65,18 @@
 (define-cproc %decompose-lazy-pair (obj) :: (<top> <top>)
   (let* ([item] [generator])
     (let* ([r::int (Scm_DecomposeLazyPair obj (& item) (& generator))])
-      (cond [r (result item generator)]
+      (cond [r (return item generator)]
             ;; NB: there's a possibility that obj has been forced by
             ;; some other thread; handle it.
-            [else (result SCM_EOF SCM_FALSE)]))))
+            [else (return SCM_EOF SCM_FALSE)]))))
 
 (define-cproc %force-lazy-pair (lp)
   (if (SCM_LAZY_PAIR_P lp)
-    (result (Scm_ForceLazyPair (SCM_LAZY_PAIR lp)))
-    (result lp)))
+    (return (Scm_ForceLazyPair (SCM_LAZY_PAIR lp)))
+    (return lp)))
 
 ;; A primitive for corecursion.
-;; See lib/gauche/common-macros.scm for the lcons macro.
+;; See libmacro.scm for lcons macro.
 (define (%lcons item thunk)
   (%make-lazy-pair item (^[] (%decompose-lazy-pair (thunk)))))
 
@@ -100,30 +100,27 @@
 
 ;; For convenience.
 (define-in-module gauche (lrange start :optional (end +inf.0) (step 1))
-  ;; Exact numbers.  Fast way.
-  (define (gen-exacts)
-    (set! start (+ start step))
-    (if (if (> step 0)
-          (< start end)
-          (> start end))
-      start
-      (eof-object)))
-  ;; Inexact numbers.  We use multiplication to avoid accumulating errors
-  (define c 0)
-  (define (gen-inexacts)
-    (set! c (+ c 1))
-    (let1 r (+ start (* c step))
-      (if (if (> step 0)
-            (< r end)
-            (> r end))
-        r
-        (eof-object))))
-
   (cond [(or (and (> step 0) (>= start end))
              (and (< step 0) (<= start end))) '()]
         [(= step 0) (generator->lseq (^[] start))]
-        [(and (exact? start) (exact? step)) (generator->lseq start gen-exacts)]
-        [else (generator->lseq (inexact start) gen-inexacts)]))
+        [(and (exact? start) (exact? step))
+         (generator->lseq start 
+                          (if (> step 0)
+                            (^[] (inc! start step)
+                              (if (< start end) start (eof-object)))
+                            (^[] (inc! start step)
+                              (if (> start end) start (eof-object)))))]
+        [else
+         (generator->lseq (inexact start)
+                          (let1 c 0
+                            (if (> step 0)
+                              (^[] (inc! c)
+                                (let1 r (+ start (* c step))
+                                  (if (< r end) r (eof-object))))
+                              (^[] (inc! c)
+                                (let1 r (+ start (* c step))
+                                  (if (> r end) r (eof-object))))
+                              )))]))
 
 (define-in-module gauche (liota :optional (count +inf.0) (start 0) (step 1))
   (let1 count (if (< count 0) +inf.0 count) ; like stream-iota
@@ -150,30 +147,3 @@
   (generator->lseq (cut read-line port)))
 (define-in-module gauche (port->sexp-lseq :optional (port (current-input-port)))
   (generator->lseq (cut read port)))
-
-(select-module gauche)
-(define-macro (lcons a b)
-  ;; poor man's explicit renaming.
-  ;; don't copy---we'll have real ER-transformer in future.
-  (let1 %lcons ((with-module gauche.internal make-identifier)
-                '%lcons
-                (find-module 'gauche.internal)
-                '())
-    `(,%lcons ,a (lambda () ,b))))
-
-(define-macro (lcons* x . args)
-  (let1 %lcons ((with-module gauche.internal make-identifier)
-                '%lcons
-                (find-module 'gauche.internal)
-                '())
-    (cond [(null? args) x]
-          [(null? (cdr args)) `(,%lcons ,x (lambda () ,(car args)))]
-          [else `(cons ,x (lcons* ,@args))])))
-(define-macro (llist* . args) `(lcons* ,@args))
-
-;; Once we are able to precompile hygineic macros ...
-;; (define-syntax lcons*
-;;   (syntax-rules ()
-;;     [(_ x) x]
-;;     [(_ x y) (lcons x y)]
-;;     [(_ x y z ...) (cons x (lcons* y z ...))]))
