@@ -1,7 +1,7 @@
 ;;;
 ;;; liblist.scm - builtin list procedures
 ;;;
-;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -48,8 +48,8 @@
   (inliner CAR) (setter set-car!) SCM_CAR)
 (define-cproc cdr (obj::<pair>) :constant
   (inliner CDR) (setter set-cdr!) SCM_CDR)
-(define-cproc set-car! (obj::<pair> value) ::<void> SCM_SET_CAR)
-(define-cproc set-cdr! (obj::<pair> value) ::<void> SCM_SET_CDR)
+(define-cproc set-car! (obj value) ::<void> Scm_SetCar)
+(define-cproc set-cdr! (obj value) ::<void> Scm_SetCdr)
 
 (inline-stub
  "#define CXR_SETTER(PRE, pre, tail) \
@@ -107,6 +107,23 @@
 (%define-cxr cdddar cddr cdar)
 (%define-cxr cddddr cddr cddr)
 
+;; primitives for immutable pars
+(define-cproc ipair? (obj) ::<boolean> Scm_ImmutablePairP)
+(define-cproc ipair (car cdr) Scm_MakeImmutablePair)
+(define-cproc ilist (:rest args)
+  (if (SCM_NULLP args)
+    (return SCM_NIL)
+    (let* ([h SCM_NIL] [t SCM_NIL])
+      (dopairs (cp args)
+        (if (SCM_NULLP t)
+          (set! h (Scm_MakeImmutablePair (SCM_CAR cp) SCM_NIL)
+                t h)
+          (let* ([p (Scm_MakeImmutablePair (SCM_CAR cp) SCM_NIL)])
+            (SCM_SET_CDR_UNCHECKED t p)
+            (set! t p))))
+      (return h))))
+
+(select-module scheme)
 (define-cproc null? (obj) ::<boolean> :fast-flonum :constant
   (inliner NULLP) SCM_NULLP)
 (define-cproc list? (obj) ::<boolean> :fast-flonum :constant
@@ -117,15 +134,22 @@
   (let* ([len::long (Scm_Length list)])
     (when (< len 0) (Scm_Error "bad list: %S" list))
     (return len)))
+
+(select-module gauche)
 (define-cproc length<=? (list k::<integer>) ::<boolean> :constant
   (if (SCM_INTP k)
-    (let* ([n::int (SCM_INT_VALUE k)])
+    (let* ([n::ScmSmallInt (SCM_INT_VALUE k)])
       (dolist [_ list] (when (<= (post-- n) 0) (return FALSE)))
       (return (<= 0 n)))
-    (return TRUE)))  ; k is bignum. it is impossible to have that long list.
+    ;; k is bignum. it is impossible to have that long list, but list
+    ;; can be circular, so we need to scan list entirely anyway.
+    (if (< (Scm_Sign k) 0)
+      (return FALSE)
+      (let* ([ln::ScmSmallInt (Scm_Length list)])
+        (return (>= ln 0))))))
 (define-cproc length=? (list k::<integer>) ::<boolean> :constant
   (if (SCM_INTP k)
-    (let* ([n::int (SCM_INT_VALUE k)])
+    (let* ([n::ScmSmallInt (SCM_INT_VALUE k)])
       (dolist [_ list] (when (<= (post-- n) 0) (return FALSE)))
       (return (== 0 n)))
     (return FALSE)))
@@ -133,7 +157,7 @@
 (define (length>? list k) (not (length<=? list k)))
 (define (length>=? list k) (not (length<? list k)))
                   
-
+(select-module scheme)
 (define-cproc append (:rest lists) (inliner APPEND) Scm_Append)
 (define-cproc reverse (list::<list> :optional (tail ())) Scm_Reverse2)
 
@@ -145,7 +169,7 @@
 (define-cproc list-set! (lis k::<fixnum> v) ::<void>
   (let* ([p (Scm_ListTail lis k SCM_FALSE)])
     (if (SCM_PAIRP p)
-      (SCM_SET_CAR p v)
+      (Scm_SetCar p v)
       (Scm_Error "list-set!: index out of bound: %d" k))))
 (define-cproc list-ref (list k::<fixnum> :optional fallback) :constant
   (setter list-set!)
@@ -188,7 +212,7 @@
     (let* ([head (SCM_LIST1 arg)] [tail head])
       (dopairs [cp args]
         (unless (SCM_PAIRP (SCM_CDR cp))
-          (SCM_SET_CDR tail (SCM_CAR cp))
+          (SCM_SET_CDR_UNCHECKED tail (SCM_CAR cp))
           (break))
         (SCM_APPEND1 head tail (SCM_CAR cp)))
       (return head))))
@@ -196,12 +220,10 @@
 (define-cproc append! (:rest list)
   (let* ([h '()] [t '()])
     (dopairs [cp list]
-      ;; allow non-list argument at the last position
-      (when (and (not (SCM_PAIRP (SCM_CAR cp)))
-                 (SCM_NULLP (SCM_CDR cp)))
+      (when (SCM_NULLP (SCM_CDR cp))
         (if (SCM_NULLP h)
           (set! h (SCM_CAR cp))
-          (SCM_SET_CDR t (SCM_CAR cp)))
+          (Scm_SetCdr t (SCM_CAR cp)))
         (break))
       (SCM_APPEND h t (SCM_CAR cp))
       (unless (or (SCM_NULLP t) (SCM_NULLP (SCM_CDR t)))
@@ -260,7 +282,7 @@
           [(<= c 0) r]))
     ;; for inexact numbers, we use multiplication to avoid error accumulation.
     (do ([c count (- c 1)]
-         [r '() (cons (+ start (* (- c 1) step)) r)])
+         [r '() (cons (+ start (*. (- c 1) step)) r)])
         [(<= c 0) r])))
 
 (select-module gauche.internal)
@@ -731,6 +753,31 @@
 (define assq-set!  (cut assoc-set! <> <> <> eq?))
 (define assv-set!  (cut assoc-set! <> <> <> eqv?))
 
+(define (assoc-adjoin alist key val :optional (eq equal?))
+  (define (rec alis)
+    (cond [(null? alis) '()]
+          [(eq key (caar alis)) (acons key val (cdr alis))]
+          [else (let1 tail (rec (cdr alis))
+                  (if (eq? tail (cdr alis))
+                    alis
+                    (cons (car alis) tail)))]))
+  (let1 r (rec alist)
+    (if (eq? r alist)
+      (acons key val alist)
+      r)))
+
+(define (assoc-update-in alist keys proc :optional (default #f) (eq equal?))
+  (define (rec alist key keys)
+    (if (null? keys)
+      (let1 val (assoc-ref alist key default eq)
+        (assoc-adjoin alist key (proc val) eq))
+      (let1 val (assoc-ref alist key '() eq)
+        (assoc-adjoin alist key (rec val (car keys) (cdr keys)) eq))))
+  (if (null? keys)
+    (error "nees at least one key in assoc-update")
+    (rec alist (car keys) (cdr keys))))
+
+
 ;;;
 ;;; Extended pairs
 ;;;
