@@ -1,7 +1,7 @@
 ;;;
 ;;; libmod.scm - modules, bindings, GLOCs
 ;;;
-;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -92,19 +92,19 @@
              (error "Invalid module name (component can't be \".\" or \"..\"):"
                     name)])))
   (with-input-from-string
-   (cond [(symbol? name) (symbol->string name)]
-         [(identifier? name) (symbol->string (identifier->symbol name))]
-         [else (error "symbol or identifier expected, but got:" name)])
-   (^[] (let loop ([c (read-char)] [p '()] [ps '()])
-          (cond
-           [(eof-object? c)
-            (string-join (reverse (cons (path-comp p) ps)) "/")]
-           [(eqv? c #\.)
-            (let1 c2 (read-char)
-              (if (eqv? c2 #\.)
-                (loop (read-char) (cons c2 p) ps)
-                (loop c2 '() (cons (path-comp p) ps))))]
-           [else (loop (read-char) (cons c p) ps)])))))
+      (if (identifier? name)
+        (symbol->string (identifier->symbol name))
+        (error "identifier expected, but got:" name))
+    (^[] (let loop ([c (read-char)] [p '()] [ps '()])
+           (cond
+            [(eof-object? c)
+             (string-join (reverse (cons (path-comp p) ps)) "/")]
+            [(eqv? c #\.)
+             (let1 c2 (read-char)
+               (if (eqv? c2 #\.)
+                 (loop (read-char) (cons c2 p) ps)
+                 (loop c2 '() (cons (path-comp p) ps))))]
+            [else (loop (read-char) (cons c p) ps)])))))
 
 (define (path->module-name path)
   (unless (string? path) (error "string required, but got:" path))
@@ -148,14 +148,28 @@
 (define-cproc %export-all (module::<module>) Scm_ExportAll)
 (define-cproc %extend-module (module::<module> supers::<list>)
   Scm_ExtendModule)
+
+;; Insert binding into the given module.  flags may be a list of
+;; one or more of the following symbols:
+;;   const     - SCM_BINDING_CONST
+;;   inlinable - SCM_BINDING_INLINABLE
+;;   fresh     - insert only if there's no binding yet (this isn't an
+;;               attribute for the binding itself)
+;; returns GLOC or #f
 (define-cproc %insert-binding (mod::<module> name::<symbol> value
                                              :optional (flags '()))
   (let* ([z::int 0])
-    (unless (SCM_FALSEP (Scm_Memq 'const flags))
-      (logior= z SCM_BINDING_CONST))
-    (unless (SCM_FALSEP (Scm_Memq 'inlinable flags))
-      (logior= z SCM_BINDING_INLINABLE))
-    (set! SCM_RESULT (SCM_OBJ (Scm_MakeBinding mod name value z)))))
+    (if (and (not (SCM_FALSEP (Scm_Memq 'fresh flags)))
+             (not (SCM_UNBOUNDP 
+                   (Scm_GlobalVariableRef mod name 
+                                          SCM_BINDING_STAY_IN_MODULE))))
+      (return SCM_FALSE)
+      (begin
+        (unless (SCM_FALSEP (Scm_Memq 'const flags))
+          (logior= z SCM_BINDING_CONST))
+        (unless (SCM_FALSEP (Scm_Memq 'inlinable flags))
+          (logior= z SCM_BINDING_INLINABLE))
+        (return (SCM_OBJ (Scm_MakeBinding mod name value z)))))))
 
 ;; Insert binding as a syntactic keyword.  VALUE must be #<macro> or #<syntax>.
 ;; Currently Gauche conflates toplevel variable bindings and syntax bindings,
@@ -195,6 +209,7 @@
 (inline-stub
  (define-cfn module-print (obj port::ScmPort* ctx::ScmWriteContext*)
    ::void :static
+   (cast void ctx)                      ; suppress unused var warning
    (if (SCM_MODULEP (-> (SCM_MODULE obj) origin))
      (Scm_Printf port "#<module %A$%A @%p>"
                  (-> (SCM_MODULE obj) name)
@@ -243,9 +258,9 @@
       ;; Some black magic needed here to inject the imported module to
       ;; r7rs.user.  We do this here instead of expanding into forms
       ;; to do the work, for "select-module" thingy is tricky.
-      (unless (provided? "r7rs")
-        (load "r7rs")
-        (provide "r7rs"))
+      (unless (provided? "r7rs-setup")
+        (load "r7rs-setup")
+        (provide "r7rs-setup"))
       (eval `(import ,@args) (find-module 'r7rs.user))
       '(select-module r7rs.user))))
 
@@ -330,8 +345,8 @@
 ;; Like free-identifier=? but we know id1 and id2 are both toplevel and
 ;; at least one is bound, so we skip local binding lookup.
 (define (global-identifier=? id1 id2)
-  (and-let* ([ (identifier? id1) ]
-             [ (identifier? id2) ]
+  (and-let* ([ (wrapped-identifier? id1) ]
+             [ (wrapped-identifier? id2) ]
              [g1 (id->bound-gloc id1)]
              [g2 (id->bound-gloc id2)])
     (eq? g1 g2)))
