@@ -1,7 +1,7 @@
 ;;;
 ;;; libnum.stub - builtin number libraries
 ;;;
-;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -81,6 +81,10 @@
 (define-cproc finite?   (x::<number>) ::<boolean> :fast-flonum Scm_FiniteP)
 (define-cproc infinite? (x::<number>) ::<boolean> :fast-flonum Scm_InfiniteP)
 (define-cproc nan?      (x::<number>) ::<boolean> :fast-flonum Scm_NanP)
+(define-cproc -zero?    (x::<number>) ::<boolean> :fast-flonum :constant
+  (return (and (SCM_FLONUMP x)          ;only flonums have -0.0
+               (== (SCM_FLONUM_VALUE x) 0.0)
+               (!= (signbit (SCM_FLONUM_VALUE x)) 0))))
 
 (define-cproc exact-integer? (obj) ::<boolean> :fast-flonum :constant
   SCM_INTEGERP)
@@ -95,15 +99,33 @@
 (define-cproc least-fixnum ()    ::<long> (return SCM_SMALL_INT_MIN))
 (define-cproc greatest-fixnum () ::<long> (return SCM_SMALL_INT_MAX))
 
-;; default-endian is defined in Scm__InitNumber().
+(inline-stub
+ (initcode
+  ;; default-endian is defined in number.c.  This call ensures it is
+  ;; initialized.
+  (Scm_DefaultEndian)))
+
 (define-cproc native-endian () Scm_NativeEndian)
 
-;; DBL_EPSILON
+;; DBL_EPSILON, etc.
 (define-cproc flonum-epsilon () 
-  (let* ([eps::(static ScmObj) SCM_UNBOUND])
-    (when (== eps SCM_UNBOUND)
-      (set! eps (Scm_MakeFlonum DBL_EPSILON)))
-    (return eps)))
+  (let* ([x::(static ScmObj) SCM_UNBOUND])
+    (when (== x SCM_UNBOUND)
+      (set! x (Scm_MakeFlonum DBL_EPSILON)))
+    (return x)))
+(define-cproc flonum-min-normalized () 
+  (let* ([x::(static ScmObj) SCM_UNBOUND])
+    (when (== x SCM_UNBOUND)
+      (set! x (Scm_MakeFlonum DBL_MIN)))
+    (return x)))
+(define-cproc flonum-min-denormalized ()
+  (let* ([x::(static ScmObj) SCM_UNBOUND])
+    (when (== x SCM_UNBOUND)
+      (let* ([m::double (Scm_EncodeFlonum (SCM_MAKE_INT 1) -1074 1)])
+        (if (== m 0.0) ; architecture doesn't support denormalized number
+          (set! x (Scm_MakeFlonum DBL_MIN))
+          (set! x (Scm_MakeFlonum m)))))
+    (return x)))
 
 (select-module gauche.internal)
 (define-cproc %bignum-dump (obj) ::<void>
@@ -190,6 +212,9 @@
   (let* ([f::u_long 0]
          [fmt::ScmNumberFormat]
          [o::ScmPort* (SCM_PORT (Scm_MakeOutputStringPort TRUE))])
+    (when (or (< radix SCM_RADIX_MIN) (> radix SCM_RADIX_MAX))
+      (Scm_Error "radix must be an integer between %d and %d, but got %d"
+                 SCM_RADIX_MIN SCM_RADIX_MAX radix))  
     (cond [(or (SCM_FALSEP flags) (SCM_NULLP flags)) (set! f 0)]
           [(SCM_TRUEP flags) (set! f SCM_NUMBER_FORMAT_USE_UPPER)];compatibility
           [(SCM_PAIRP flags)
@@ -212,8 +237,18 @@
     (Scm_PrintNumber o obj (& fmt))
     (return (Scm_GetOutputString o 0))))
 
-(define-cproc string->number (obj::<string> :optional (radix::<fixnum> 10))
-  (return (Scm_StringToNumber obj radix 0)))
+(define-cproc string->number (obj::<string>
+                              :optional (radix::<fixnum> 10)
+                                        (default-exactness #f))
+  (let* ([flags::u_long 0])
+    (cond
+     [(SCM_EQ default-exactness 'exact) 
+      (set! flags SCM_NUMBER_FORMAT_EXACT)]
+     [(SCM_EQ default-exactness 'inexact) 
+      (set! flags SCM_NUMBER_FORMAT_INEXACT)]
+     [(SCM_FALSEP default-exactness)]
+     [else (Scm_Error "default-exactness must be either #f, exact or inexact, but got: %S" default-exactness)])
+    (return (Scm_StringToNumber obj radix flags))))
 
 (select-module gauche)
 (define-cproc floor->exact (num) :fast-flonum :constant
@@ -264,6 +299,9 @@
  (if "defined(HAVE_LGAMMA)"
    (define-cproc %lgamma (x::<real>) ::<real> :fast-flonum :constant
      lgamma)))
+;; Returns the HalfFloat representation as integer.  For now,
+;; we keep it in gauche.internal.
+(define-cproc flonum->f16bits (x::<real>) ::<int> Scm_DoubleToHalf)
 
 ;;
 ;; Arithmetics
@@ -585,7 +623,7 @@
 
 (select-module gauche)
 (define-in-module scheme (cos z)
-  (cond [(real? z) (%cos z)]
+  (cond [(real? z) (if (eqv? 0 z) 1 (%cos z))]
         [(number? z)
          (let ([x (real-part z)]
                [y (imag-part z)])
@@ -594,7 +632,7 @@
         [else (error "number required, but got" z)]))
 
 (define (cosh z)
-  (cond [(real? z) (%cosh z)]
+  (cond [(real? z) (if (eqv? z 0) 1 (%cosh z))]
         [(number? z)
          (let ([x (real-part z)]
                [y (imag-part z)])
@@ -603,7 +641,7 @@
         [else (error "number required, but got" z)]))
 
 (define-in-module scheme (sin z)
-  (cond [(real? z) (%sin z)]
+  (cond [(real? z) (if (eqv? 0 z) 0 (%sin z))]
         [(number? z)
          (let ([x (real-part z)]
                [y (imag-part z)])
@@ -612,7 +650,7 @@
         [else (error "number required, but got" z)]))
 
 (define (sinh z)
-  (cond [(real? z) (%sinh z)]
+  (cond [(real? z) (if (eqv? 0 z) 0 (%sinh z))]
         [(number? z)
          (let ([x (real-part z)]
                [y (imag-part z)])
@@ -621,7 +659,7 @@
         [else (error "number required, but got" z)]))
 
 (define-in-module scheme (tan z)
-  (cond [(real? z) (%tan z)]
+  (cond [(real? z) (if (eqv? 0 z) 0 (%tan z))]
         [(number? z)
          (let1 iz (* +i z)
            (* -i
@@ -630,14 +668,14 @@
         [else (error "number required, but got" z)]))
 
 (define (tanh z)
-  (cond [(real? z) (%tanh z)]
+  (cond [(real? z) (if (eqv? 0 z) 0 (%tanh z))]
         [(number? z)
          (/ (- (exp z) (exp (- z)))
             (+ (exp z) (exp (- z))))]
         [else (error "number required, but got" z)]))
 
 (define-in-module scheme (asin z)
-  (cond [(real? z) (%asin z)]
+  (cond [(real? z) (if (eqv? z 0) 0 (%asin z))]
         [(number? z)
          ;; The definition of asin is
          ;;   (* -i (log (+ (* +i z) (sqrt (- 1 (* z z))))))
@@ -654,13 +692,15 @@
         [else (error "number required, but got" z)]))
 
 (define (asinh z)
-  (let1 zz (+ z (sqrt (+ (* z z) 1)))
-    (if (< (/. (magnitude zz) (magnitude z)) 1.0e-8)
-      (make-rectangular +nan.0 +nan.0)
-      (log (+ z (sqrt (+ (* z z) 1)))))))
+  (if (eqv? z 0)
+    0
+    (let1 zz (+ z (sqrt (+ (* z z) 1)))
+      (if (< (/. (magnitude zz) (magnitude z)) 1.0e-8)
+        (make-rectangular +nan.0 +nan.0)
+        (log (+ z (sqrt (+ (* z z) 1))))))))
 
 (define-in-module scheme (acos z)
-  (cond [(real? z) (%acos z)]
+  (cond [(real? z) (if (eqv? z 1) 0 (%acos z))]
         [(number? z)
          ;; The definition of acos is
          ;;  (* -i (log (+ z (* +i (sqrt (- 1 (* z z)))))))))
@@ -671,23 +711,29 @@
         [else (error "number required, but got" z)]))
 
 (define (acosh z)
-  ;; See the discussion of CLtL2, pp. 313-314
-  (* 2 (log (+ (sqrt (/ (+ z 1) 2))
-               (sqrt (/ (- z 1) 2))))))
+  (if (eqv? z 1)
+    0
+    ;; See the discussion of CLtL2, pp. 313-314
+    (* 2 (log (+ (sqrt (/ (+ z 1) 2))
+                 (sqrt (/ (- z 1) 2)))))))
 
 (define-in-module scheme (atan z . x)
-  (if (null? x)
-    (cond [(real? z) (%atan z)]
-          [(number? z)
-           (let1 iz (* z +i)
-             (/ (- (log (+ 1 iz))
-                   (log (- 1 iz)))
-                +2i))]
-          [else (error "number required, but got" z)])
-    (%atan z (car x))))
+  (if (eq? z 0)
+    0
+    (if (null? x)
+      (cond [(real? z) (%atan z)]
+            [(number? z)
+             (let1 iz (* z +i)
+               (/ (- (log (+ 1 iz))
+                     (log (- 1 iz)))
+                  +2i))]
+            [else (error "number required, but got" z)])
+      (%atan z (car x)))))
 
 (define (atanh z)
-  (/ (- (log (+ 1 z)) (log (- 1 z))) 2))
+  (if (eq? z 0)
+    0
+    (/ (- (log (+ 1 z)) (log (- 1 z))) 2)))
 
 
 (select-module gauche)
@@ -858,6 +904,3 @@
           (Scm_Error "clamp argument must be either 'both, 'high, 'low or #f, \
                     but got %S" clamp)])
    (return SCM_CLAMP_ERROR)))
-
-
-
