@@ -1,7 +1,7 @@
 ;;;
 ;;; object.scm - object system
 ;;;
-;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -94,20 +94,18 @@
           [other (delete-keyword :class opts)])
       (if getter-name
         (quasirename %id
-          (define ,true-name
-            (rlet1 ,true-name (make ,class ':name ',true-name ,@other)
-              (set! (setter ,getter-name) ,true-name))))
+          `(define ,true-name
+             (rlet1 ,true-name (make ,class ':name ',true-name ,@other)
+               (set! (setter ,getter-name) ,true-name))))
         (quasirename %id
-          (define ,true-name
-            (make ,class ':name ',true-name ,@other)))))))
+          `(define ,true-name
+             (make ,class ':name ',true-name ,@other)))))))
 
 ;; allow (setter name) type declaration
 (define (%check-setter-name name)
-  (cond [(symbol? name) (values name #f)]
-        [(identifier? name) (values name #f)]
+  (cond [(identifier? name) (values name #f)]
         [(and (pair? name) (eq? (car name) 'setter)
-              (pair? (cdr name)) (or (symbol? (cadr name))
-                                     (identifier? (cadr name)))
+              (pair? (cdr name)) (identifier? (cadr name))
               (null? (cddr name)))
          (values (%make-setter-name (cadr name)) (cadr name))]
         [else (error "Bad name for generic function or method" name)]))
@@ -115,6 +113,12 @@
 (define (%make-setter-name name)
   (string->symbol (format #f "setter of ~a" name)))
 
+;; These are hidden procedure in gauche.object, for now.
+(define-cproc generic-seal! (gf::<generic>) ::<void>
+  (set! (SCM_PROCEDURE_CONSTANT gf) TRUE))
+(define-cproc generic-unseal! (gf::<generic>) ::<void>
+  (set! (SCM_PROCEDURE_CONSTANT gf) FALSE))
+
 ;;----------------------------------------------------------------
 ;; Method
 ;;
@@ -147,24 +151,24 @@
                            `(,@reqargs next-method))]
            [real-body (if opts
                         (quasirename %id
-                          (lambda ,real-args
-                            (apply (lambda ,opts ,@body) ,rest)))
+                          `(lambda ,real-args
+                             (apply (lambda ,opts ,@body) ,rest)))
                         (quasirename %id
-                          (lambda ,real-args ,@body)))])
+                          `(lambda ,real-args ,@body)))])
       (receive (true-name getter-name) (%check-setter-name name)
-        (let1 gf (gensym)
+        (let1 gf (gensym "gf")
           (quasirename %id
-            (rlet1 ,gf (%ensure-generic-function ',true-name (current-module))
-              (add-method! ,gf (make <method>
-                                 ':generic ,gf
-                                 ':specializers (list ,@specializers)
-                                 ':lambda-list ',lambda-list
-                                 ':method-locked (boolean (memq ':locked ',quals))
-                                 ':body ,real-body))
-              ,@(cond-list [getter-name
-                            (quasirename %id
-                              (unless (has-setter? ,getter-name)
-                                (set! (setter ,getter-name) ,gf)))]))))))))
+            `(rlet1 ,gf (%ensure-generic-function ',true-name (current-module))
+               (add-method! ,gf (make <method>
+                                  ':generic ,gf
+                                  ':specializers (list ,@specializers)
+                                  ':lambda-list ',lambda-list
+                                  ':method-locked (boolean (memq ':locked ',quals))
+                                  ':body ,real-body))
+               ,@(cond-list [getter-name
+                             (quasirename %id
+                               `(unless (has-setter? ,getter-name)
+                                  (set! (setter ,getter-name) ,gf)))]))))))))
 
 (inline-stub
  ;; internal for %ensure-generic-function
@@ -186,7 +190,8 @@
    (let* ([val (Scm_GlobalVariableRef module name 0)])
      (cond
       [(Scm_TypeP val SCM_CLASS_GENERIC)
-       (unless (Scm_GlobalVariableRef module name SCM_BINDING_STAY_IN_MODULE)
+       (when (SCM_UNBOUNDP (Scm_GlobalVariableRef module name 
+                                                  SCM_BINDING_STAY_IN_MODULE))
          (Scm_Define module name val))]
       [else
        (if (or (SCM_SUBRP val) (SCM_CLOSUREP val))
@@ -196,9 +201,9 @@
      (return val)))
 
  (define-cproc %make-next-method (gf methods::<list> args::<list>)
-   (let* ([argv::ScmObj*] [argc::int])
+   (let* ([argv::ScmObj*] [argc::ScmSize])
      (unless (Scm_TypeP gf SCM_CLASS_GENERIC)
-       (Scm_Error "generic function requied, but got %S" gf))
+       (Scm_Error "generic function required, but got %S" gf))
      (dolist [mp methods]
        (unless (Scm_TypeP mp SCM_CLASS_METHOD)
          (Scm_Error "method required, but got %S" mp)))
@@ -211,6 +216,20 @@
 ;; Class
 ;;
 
+;; TEMPORARY: We need to detect keywords such as :metaclass, :slots
+;; or :supers at macro-expansion time.  If the define-class form
+;; is macro-generated, however, these keywords may be renamed, so
+;; we can't simply use get-keyword.  This will eventually addressed
+;; when we rewrote define-class expander as macro.  Meanwhile,
+;; this ad-hoc solution would work.
+
+(define (%get-keyword key args default)
+  (let loop ((as args))
+    (cond [(null? as) default]
+          [(null? (cdr as)) (error "Key-value list is not even:" args)]
+          [(eqv? key (unwrap-syntax-1 (car as))) (cadr as)]
+          [else (loop (cddr as))])))
+
 ;; TRANSIENT: We should employ er-macro-transformer to expand
 ;; define-class etc., but this file need to be compiled by 0.9.5
 ;; and we've improved the transformer since then.  So we postponed
@@ -221,29 +240,29 @@
 ;  (%expand-define-class name supers slots options))
 
 (define (%expand-define-class name supers slots options)
-  (let* ([metaclass (or (get-keyword :metaclass options #f)
+  (let* ([metaclass (or (%get-keyword :metaclass options #f)
                         (quasirename %id
-                          (%get-default-metaclass (list ,@supers))))]
+                          `(%get-default-metaclass (list ,@supers))))]
          [slot-defs (map %process-slot-definition slots)]
          [class     (gensym)]
          [slot      (gensym)])
     (quasirename %id
-      (define ,name
-        (rlet1 ,class (make ,metaclass
-                        ':name ',name ':supers (list ,@supers)
-                        ':slots (list ,@slot-defs)
-                        ':defined-modules (list (current-module))
-                        ,@options)
-          (when (%check-class-binding ',name (current-module))
-            (redefine-class! ,name ,class))
-          (for-each (lambda (,slot)
-                      (%make-accessor ,class ,slot (current-module)))
-                    (class-slots ,class)))))))
+      `(define ,name
+         (rlet1 ,class (make ,metaclass
+                         ':name ',name ':supers (list ,@supers)
+                         ':slots (list ,@slot-defs)
+                         ':defined-modules (list (current-module))
+                         ,@options)
+           (when (%check-class-binding ',name (current-module))
+             (redefine-class! ,name ,class))
+           (for-each (lambda (,slot)
+                       (%make-accessor ,class ,slot (current-module)))
+                     (class-slots ,class)))))))
 
 (define (%process-slot-definition sdef)
   (if (pair? sdef)
     (let loop ([opts (cdr sdef)] [r '()])
-      (cond [(null? opts) (quasirename %id (list ',(car sdef) ,@(reverse! r)))]
+      (cond [(null? opts) (quasirename %id `(list ',(car sdef) ,@(reverse! r)))]
             [(not (and (pair? opts) (pair? (cdr opts))))
              (error "bad slot specification:" sdef)]
             [else
@@ -253,14 +272,14 @@
                [(:initform :init-form)
                 (loop (cddr opts)
                       (quasirename %id
-                        ((lambda () ,(cadr opts)) ':init-thunk ,@r)))]
+                        `((lambda () ,(cadr opts)) ':init-thunk ,@r)))]
                [(:getter :setter :accessor)
                 (loop (cddr opts)
-                      (quasirename %id (',(cadr opts) ',(car opts) ,@r)))]
+                      (quasirename %id `(',(cadr opts) ',(car opts) ,@r)))]
                [else
                 (loop (cddr opts)
-                      (quasirename %id (,(cadr opts) ',(car opts) ,@r)))])]))
-    (quasirename %id '(,sdef))))
+                      (quasirename %id `(,(cadr opts) ',(car opts) ,@r)))])]))
+    (quasirename %id `'(,sdef))))
 
 ;; Determine default metaclass, that is a class inheriting all the metaclasses
 ;; of supers.  The idea is taken from stklos.  The difference is that
@@ -299,8 +318,8 @@
 
 (define-method initialize :locked ((class <class>) initargs)
   (next-method)
-  (let* ([slots  (get-keyword :slots  initargs '())]
-         [sup    (get-keyword :supers initargs '())]
+  (let* ([slots  (%get-keyword :slots  initargs '())]
+         [sup    (%get-keyword :supers initargs '())]
          ;;  NB: we always add <object> to the direct supers, for C defined
          ;;  base classes may not be inheriting from it.
          [supers (append sup (list <object>))])
@@ -362,9 +381,10 @@
     (when %setter
       (make-setter (%ensure-generic-function %setter module)))
     (when %accessor
-      (let ([gf  (%ensure-generic-function %accessor module)]
-            [gfs (%ensure-generic-function (%make-setter-name %accessor)
-                                           module)])
+      (let* ([gf  (%ensure-generic-function %accessor module)]
+             [gfs (or (setter gf)
+                      (%ensure-generic-function (%make-setter-name %accessor)
+                                                module))])
         (make-getter gf)
         (make-setter gfs)
         (set! (setter gf) gfs)
@@ -541,12 +561,9 @@
 
 ;; Internal API - undocumented
 (define-cproc instance-slot-ref (obj num::<fixnum> :optional fallback)
-  (let* ([v (Scm_InstanceSlotRef obj num)])
-    (if (SCM_UNBOUNDP v)
-      (if (SCM_UNBOUNDP fallback)
-        (Scm_Error "Slot #%d of object %S is unbound." num obj)
-        (return fallback))
-      (return v))))
+  (.if "GAUCHE_API_VERSION < 1000"
+       (return (Scm_InstanceSlotRef3 obj num fallback))
+       (return (Scm_InstanceSlotRef obj num fallback))))
 ;; Internal API - undocumented
 (define-cproc instance-slot-set! (obj num::<fixnum> value) ::<void>
   Scm_InstanceSlotSet)
@@ -574,7 +591,9 @@
 (define-cproc %record-ref (klass::<class> obj k::<fixnum>)
   (unless (SCM_ISA obj klass)
     (Scm_Error "record-ref: instance of %S expected, got %S" klass obj))
-  (return (Scm_InstanceSlotRef obj k)))
+  (.if "GAUCHE_API_VERSION < 1000"
+       (return (Scm_InstanceSlotRef3 obj k SCM_UNBOUND))
+       (return (Scm_InstanceSlotRef obj k SCM_UNBOUND))))
 
 (define-cproc %record-set! (klass::<class> obj k::<fixnum> val) ::<void>
   (unless (SCM_ISA obj klass)
@@ -664,8 +683,8 @@
 (define-cproc generic-build-dispatcher! (gf::<generic> axis::<fixnum>)
   Scm__GenericBuildDispatcher)
 
-(define-cproc generic-dispatcher-vector (gf::<generic>)
-  (return (-> gf dispatcher)))
+(define-cproc generic-dispatcher-info (gf::<generic>)
+  Scm__GenericDispatcherInfo)
 
 (define-cproc generic-invalidate-dispatcher! (gf::<generic>) ::<void>
   Scm__GenericInvalidateDispatcher)
@@ -755,18 +774,40 @@
 (define-method (setter ref) :locked ((obj <tree-map>) key value)
   (tree-map-put! obj key value))
 
+(define-method ref  ((obj <box>) field) ; srfi-123
+  (unless (eq? field '*)
+    (error "Box content can be accessed with a field name '*, but got:"
+           field))
+  (unbox obj))
+(define-method (setter ref) :locked ((obj <box>) field value)
+  (unless (eq? field '*)
+    (error "Box content can be accessed with a field name '*, but got:"
+           field))
+  (set-box! obj value))
+
 ;; gauche.sequence has the generic version for <sequence>, but these
 ;; shortcuts would be faster.
+;; NB: (ref <bitvector> k) has a choice of using /int or /bool.  We chose
+;; int, somewhat arbitrary but more intuitive with correspondence of the
+;; literal bitvector notation.
 (define-method ref :locked ((obj <list>) (index <integer>))
   (list-ref obj index))
 (define-method ref :locked ((obj <vector>) (index <integer>))
   (vector-ref obj index))
+(define-method ref :locked ((obj <uvector>) (index <integer>))
+  (uvector-ref obj index))
+(define-method ref :locked ((obj <bitvector>) (index <integer>))
+  (bitvector-ref/int obj index))
 (define-method ref :locked ((obj <string>) (index <integer>))
   (string-ref obj index))
 (define-method (setter ref) :locked ((obj <list>) (index <integer>) val)
   (set-car! (list-tail obj index) val))
 (define-method (setter ref) :locked ((obj <vector>) (index <integer>) val)
   (vector-set! obj index val))
+(define-method (setter ref) :locked ((obj <uvector>) (index <integer>) val)
+  (uvector-set! obj index val))
+(define-method (setter ref) :locked ((obj <bitvector>) (index <integer>) val)
+  (bitvector-set! obj index val))
 (define-method (setter ref) :locked ((obj <string>) (index <integer>) val)
   (string-set! obj index val))
 
@@ -813,6 +854,12 @@
 (define-method object-apply :locked ((type <condition-meta>) obj)
   (condition-has-type? obj type))
 
+;; Primitive parameters
+(define-method object-apply :locked ((p <primitive-parameter>) :optional val)
+  (if (undefined? val)
+    ((with-module gauche.internal %primitive-parameter-ref) p)
+    ((with-module gauche.internal %primitive-parameter-set!) p val)))
+
 ;; Regexp printer.  It doesn't need speed, and it's easier to write in Scheme.
 ;; This is defined here since we can only use it after define-method support.
 ;; NB: case-folding regexp is expressed by (?i:...) in regexp->string.
