@@ -1,7 +1,7 @@
 ;;;
 ;;; libomega.scm - the stuff to be run after other parts are initialized
 ;;;
-;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -31,17 +31,18 @@
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;
 
+(select-module gauche)
+
 ;; Register built-in modules as provided, so that (use <built-in-modue>) won't
 ;; complain.
-;; Kludge: util.match and gauche.interpolate are external files, but references
-;; to them are created in the compiled core code, since macros defined in them
-;; are expanded during compilation.  We need a better solution than this, for
-;; new code would introduce further dependency; but for the time being, this
-;; will do.
 (dolist [m (all-modules)]
   (let1 n (module-name m)
-    (unless (memq n '(util.match gauche.interpolate gauche.common-macros))
-      (provide (module-name->path n)))))
+    (provide (module-name->path n))))
+
+;;;
+;;; Object system finish-up
+;;;
+(select-module gauche)
 
 ;; A trick to allow slot-ref to be used for compound condition.
 ;; This is better to be in libexc.scm, but we need to evaluate this
@@ -74,6 +75,11 @@
               (car src-info) (cadr src-info) expr)
       (format port "    While compiling: ~,,,,90:s\n" expr))))
 
+
+;;;
+;;; Comparator finish-up
+;;;
+
 ;; Built-in comparators.  These are here instead of libcmp.scm, for
 ;; hash functions need to be defined before this.
 ;; NB: These are in srfi-114 but not in srfi-128.  We provide them
@@ -148,9 +154,24 @@
 ;; delegate to one-arg in order to maintain the backward compatibility.
 ;; See below for two-arg object-has base method.
 (define-method object-hash (a)
-  (if-let1 c (%choose-comparator-1 a)
-    (comparator-hash c a)
-    (errorf "Object ~s is not hashable" a)))
+  (or (and-let* ([c (%choose-comparator-1 a)]
+                 [ (comparator-hashable? c) ])
+        (comparator-hash c a))
+      ;; default-hash needs to accept arbitrary Scheme object, so object-hash
+      ;; must return something.  If we can know that the user-defined type
+      ;; uses eq? or eqv? for the comparison, we can do better (using
+      ;; eq-hash or eqv-hash) but with the layer of customiation it is
+      ;; quite difficult to know.  So here we are.
+      (let1 v (eq-hash 57)  ;Grothendieck prime, for no reason.
+        ;; If we're not computing a portable hash, sprinkle a grain of salt.
+        (if (eq? (%current-recursive-hash) default-hash)
+          (let1 v1 (* (hash-salt) v)
+            (logxor (ash v1 -32) v1))
+          v))))
+
+;;;
+;;; Hash function fixup
+;;;
 
 ;; Recursive hash function
 ;; This is here instead of in libdict.scm, for we need the rest of
@@ -180,6 +201,11 @@
 
 (select-module gauche.internal)
 
+;; %call-object-hash is called from equal_hash_common (hash.c) to invoke
+;; object-hash method.  HASH argument is either default-hash or portable-hash,
+;; and salt argument is #f (for default-hash, since it takes salt value from
+;; hash-salt parameter) or an exact integer (for portable-hash).
+;;
 ;; TODO: We may memoize hash&salt pair so that we can avoid
 ;; closure allocation for every recursive call; the memoization also
 ;; benefits bypassing dynamic-wind in object-hash.
@@ -213,34 +239,32 @@
        (subset? a b)
        (subset? b a)))
 
-;;; TEMPORARY for 0.9.x series
-;;; Remove this after 1.0 release!!!
+;; Make charset hashable
+(define-method object-hash ((c <char-set>) _) (char-set-hash c))
+
 ;;;
-;;; Add 0.9 directories, which doesn't follow the new directory structure,
-;;; to support the extension modules that are installed with 0.9.
-;;; A few extension packages installs files into the "sys" directory hierarchy
-;;; instead of "site" one.  It is banned after 0.9.1, but we need to add 0.9's
-;;; sys directories for the backward compatibility.
-;;; NB: we set! to *load-path* etc here, which is an emergency workaround.
-;;; Ordinary programs should never modify *load-path*/*dynamic-load-path*
-;;; directly.
-(select-module gauche)
-(let* ([archdir (gauche-architecture-directory)]
-       [m (rxmatch #/gauche-0\.9[\/\\]0\.9[^\/\\]*[\/\\]/ archdir)]
-       [oldsitedir (string-append (rxmatch-before m)
-                                  "gauche/site/0.9/"
-                                  (rxmatch-after m))]
-       [oldarchdir (string-append (rxmatch-before m)
-                                  "gauche/0.9/"
-                                  (rxmatch-after m))])
-  (set! *dynamic-load-path*
-        (append *dynamic-load-path* (list oldsitedir oldarchdir))))
-(let* ([libdir (gauche-library-directory)]
-       [m (rxmatch #/gauche-0\.9[\/\\]0\.9[^\/\\]*[\/\\]/ libdir)]
-       [oldsitedir (string-append (rxmatch-before m)
-                                  "gauche/site/"
-                                  (rxmatch-after m))]
-       [oldlibdir  (string-append (rxmatch-before m)
-                                  "gauche/0.9/"
-                                  (rxmatch-after m))])
-  (set! *load-path* (append *load-path* (list oldsitedir oldlibdir))))
+;;; Read-edit mode flag
+;;;
+
+(with-module gauche.internal)
+;; This variable affects hwo gauche.interactive handles input; if true,
+;; it sets up the input editor if possible.  Note that once gauche.interactive
+;; is loaded, this variable has no effect.
+(define *read-edit* (boolean (sys-getenv "GAUCHE_READ_EDIT")))
+
+;;;
+;;; Miscellaneous
+;;;
+
+;;
+;; Turn on generic dispatcher on selected gfs.
+;; Eventually, we automate attaching dispacher whenever gf meets certain
+;; criteria.  The dispatcher is so effective, (e.g. ref <vector>
+;; gets 8x speedup) so we manually turn it on for soem gfs.
+;; In case if bug is found in dispatcher mechanism, use -fno-generic-dispatcher
+;; option to turn off dispatchers.
+;;
+(with-module gauche.object
+  (generic-build-dispatcher! ref 0)
+  (generic-build-dispatcher! object-apply 0)
+  )
