@@ -1,7 +1,7 @@
 ;;;
 ;;; libproc.scm - procedure call & return, and other control stuff
 ;;;
-;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -36,8 +36,6 @@
  (declcode (.include <gauche/vminsn.h>
                      <gauche/prof.h>)))
 
-(declare (keep-private-macro let-keywords let-keywords* let-optionals*))
-
 ;;;
 ;;; R5RS
 ;;;
@@ -54,6 +52,8 @@
                       tail head)
                 (dopairs [cp args]
                   (when (SCM_NULLP (SCM_CDR cp))
+                    (when (< (Scm_Length (SCM_CAR cp)) 0)
+                      (Scm_Error "improper list not allowed: %S" (SCM_CAR cp)))
                     (SCM_APPEND head tail (SCM_CAR cp))
                     (break))
                   (unless (SCM_PAIRP (SCM_CDR cp))
@@ -73,138 +73,99 @@
 (select-module gauche.internal)
 ;; for partial continuation.  See lib/gauche/partcont.scm
 (define-cproc %call/pc (proc) (return (Scm_VMCallPC proc)))
+(define-cproc %reset (proc) (return (Scm_VMReset proc)))
 
 ;;;
-;;; Extended argument parsing
+;;; Useful gadgets
 ;;;
 
-;; Extended lambda formals (:optional, :key, :rest etc) are
-;; expanded into the call of let-optionals* and let-keywords*
-;; macros within the compiler.  Eventually the handling of the
-;; optional and keyword arguments will be built in the VM.
-
-;; NB: As of 0.9.2, Gauche cannot pre-compile R5RS macro, so
-;; we use legacy macros here.
-
 (select-module gauche)
-(define-macro (let-optionals* arg specs . body)
-  (define (rec arg vars&inits rest)
-    (cond
-     [(null? (cdr vars&inits))
-      `((let ((,(caar vars&inits)
-               (if (null? ,arg) ,(cdar vars&inits) (car ,arg)))
-              ,@(if (null? rest)
-                  '()
-                  `((,rest (if (null? ,arg) '() (cdr ,arg))))))
-          ,@body))]
-     [else
-      (let ([g (gensym)]
-            [v (caar vars&inits)]
-            [i (cdar vars&inits)])
-        ;; NB: if the compiler were more clever, we could use receive
-        ;; or apply to make (null? ,arg) test once.  For now, testing it
-        ;; twice is faster.
-        `((let ((,v (if (null? ,arg) ,i (car ,arg)))
-                (,g (if (null? ,arg) '() (cdr ,arg))))
-            ,@(rec g (cdr vars&inits) rest))))]))
-  (let1 g (gensym)
-    `(let ((,g ,arg))
-       ,@(rec g (map* (^s
-                        (cond [(and (pair? s) (pair? (cdr s)) (null? (cddr s)))
-                               (cons (car s) (cadr s))]
-                              [(or (symbol? s) (identifier? s))
-                               (cons s '(undefined))]
-                              [else (error "malformed let-optionals* bindings:"
-                                           specs)]))
-                      (^_ '()) ; ignore last cdr of dotted list
-                      specs)
-              (cdr (last-pair specs))))))
+;; 'values' could be used, but sometimes handy to be explicit that
+;; we're dealing with a single value.
+(define-inline (identity x) x)
 
+;; more verbose than its definition, but it shows the intention clearly.
+(define-inline (constantly val) (^ _ val))
 
-(define-macro (let-keywords arg specs . body)
-  ((with-module gauche.internal %let-keywords-rec)
-   arg specs body 'let 'errorf))
+;;;
+;;; Generator primitives
+;;;
 
-(define-macro (let-keywords* arg specs . body)
-  ((with-module gauche.internal %let-keywords-rec)
-   arg specs body 'let* 'errorf))
+;; Generators are just an ordinary procedure, so having these in core
+;; comes handy.  We used to have these as port-*.
 
 (select-module gauche.internal)
-(define (%let-keywords-rec arg specs body %let %error/warn)
-  (define (triplet var&default)
-    (or (and-let* ([ (list? var&default) ]
-                   [var (unwrap-syntax (car var&default))]
-                   [ (symbol? var) ])
-          (case (length var&default)
-            [(2) (values (car var&default)
-                         (make-keyword var)
-                         (cadr var&default))]
-            [(3) (values (car var&default)
-                         (unwrap-syntax (cadr var&default))
-                         (caddr var&default))]
-            [else #f]))
-        (and-let* ([var (unwrap-syntax var&default)]
-                   [ (symbol? var) ])
-          (values var (make-keyword var) (undefined)))
-        (error "bad binding form in let-keywords" var&default)))
-  (define (process-specs specs)
-    (let loop ((specs specs)
-               (vars '()) (keys '()) (defaults '()) (tmps '()))
-      (define (finish restvar)
-        (values (reverse! vars)
-                (reverse! keys)
-                (reverse! defaults)
-                (reverse! tmps)
-                restvar))
-      (cond [(null? specs) (finish #f)]
-            [(pair? specs)
-             (receive (var key default) (triplet (car specs))
-               (loop (cdr specs)
-                     (cons var vars)
-                     (cons key keys)
-                     (cons default defaults)
-                     (cons (gensym) tmps)))]
-            [else (finish (or specs #t))])))
+;; Returns a list of new values generated by each of GENS, having tail
+;; as the tail of the returned list.  It returns #<eof> when any one of
+;; generator returns eof (generation is
+(define (%generate-values gens . tail)
+  (fold-right (^[g tail]
+                (let1 v (g)
+                  (if (or (eof-object? v) (eof-object? tail))
+                    (eof-object)
+                    (cons v tail))))
+              tail gens))
 
-  (let ((argvar (gensym "args")) (loop (gensym "loop")))
-    (receive (vars keys defaults tmps restvar) (process-specs specs)
-      `(let ,loop ((,argvar ,arg)
-                   ,@(if (boolean? restvar) '() `((,restvar '())))
-                   ,@(map (cut list <> (undefined)) tmps))
-            (cond
-             [(null? ,argvar)
-              (,%let ,(map (^[var tmp default]
-                             `(,var (if (undefined? ,tmp) ,default ,tmp)))
-                           vars tmps defaults)
-                     ,@body)]
-             [(null? (cdr ,argvar))
-              (error "keyword list not even" ,argvar)]
-             [else
-              (case (car ,argvar)
-                ,@(map (^[key] `((,key)
-                                 (,loop (cddr ,argvar)
-                                        ,@(if (boolean? restvar)
-                                            '()
-                                            `(,restvar))
-                                        ,@(map (^[k t] (if (eq? key k)
-                                                         `(cadr ,argvar)
-                                                         t))
-                                               keys tmps))))
-                       keys)
-                (else
-                 ,(cond [(eq? restvar #t)
-                         `(,loop (cddr ,argvar) ,@tmps)]
-                        [(eq? restvar #f)
-                         `(begin
-                            (,%error/warn "unknown keyword ~S" (car ,argvar))
-                            (,loop (cddr ,argvar) ,@tmps))]
-                        [else
-                         `(,loop
-                           (cddr ,argvar)
-                           (list* (car ,argvar) (cadr ,argvar) ,restvar)
-                           ,@tmps)])))
-              ]))))
-  )
+(define-in-module gauche (generator-fold fn knil gen . more)
+  (if (null? more)
+    (let loop ([item (gen)]
+               [r    knil])
+      (if (eof-object? item)
+        r
+        (let1 r (fn item r)
+          (loop (gen) r))))
+    (let1 gens (cons gen more)
+      (let loop ([knil knil])
+        (let1 items (%generate-values gens knil)
+          (if (eof-object? items)
+            knil
+            (loop (apply fn items))))))))
+
+;; This will consume large stack if input file is large.
+(define-in-module gauche (generator-fold-right fn knil gen . more)
+  (if (null? more)
+    (let loop ([item (gen)])
+      (if (eof-object? item)
+        knil
+        (fn item (loop (gen)))))
+    (let1 gens (cons gen more)
+      (let loop ()
+        (let1 items (%generate-values gens)
+          (if (eof-object? items)
+            knil
+            (apply fn (append items `(,(loop))))))))))
+
+(define-in-module gauche (generator-for-each fn gen . more)
+  (if (null? more)
+    (let loop ([item (gen)])
+      (unless (eof-object? item)
+        (fn item) (loop (gen))))
+    (let1 gens (cons gen more)
+      (let loop ()
+        (let1 items (%generate-values gens)
+          (unless (eof-object? items)
+            (apply fn items)
+            (loop)))))))
+
+(define-in-module gauche (generator-map fn gen . more)
+  (if (null? more)
+    (let loop ([item (gen)] [r '()])
+      (if (eof-object? item)
+        (reverse r)
+        (let1 r (cons (fn item) r)
+          (loop (gen) r))))
+    (let1 gens (cons gen more)
+      (let loop ([r '()])
+        (let1 items (%generate-values gens)
+          (if (eof-object? items)
+            (reverse r)
+            (loop (cons (apply fn items) r))))))))
+
+(define-in-module gauche (generator-find pred gen)
+  (let loop ([item (gen)])
+    (cond [(eof-object? item) #f]
+          [(pred item) item]
+          [else (loop (gen))])))
 
 ;;;
 ;;; Profiler
@@ -269,8 +230,14 @@
 (define-cproc procedure-locked-setter (proc::<procedure>)
   (if (and (-> proc locked)
            (SCM_PROCEDUREP (-> proc setter)))
-    (result (-> proc setter))
-    (result SCM_FALSE)))
+    (return (-> proc setter))
+    (return SCM_FALSE)))
+;; Returns #t iff obj is a procedure AND constant
+;; Note: This also returns #t if obj is generic and sealed.  The behavior
+;; may change in future, though, so do not rely on it.
+(define-cproc procedure-constant? (obj) ::<boolean>
+  (return (and (SCM_PROCEDUREP obj)
+               (SCM_PROCEDURE_CONSTANT obj))))
 
 (define-cproc %procedure-copy (p::<procedure>) Scm_CopyProcedure)
 
