@@ -1,7 +1,7 @@
 ;;;
 ;;; librx.scm - builtin regular-expression procedures
 ;;;
-;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -32,6 +32,7 @@
 ;;;
 
 (select-module gauche)
+(use util.match)
 
 (inline-stub
  (declcode (.include <gauche/regexp.h>)))
@@ -39,17 +40,26 @@
 (define-cproc regexp? (obj)   ::<boolean> :constant SCM_REGEXPP)
 (define-cproc regmatch? (obj) ::<boolean> SCM_REGMATCHP)
 
-(define-cproc string->regexp (str::<string> :key (case-fold #f))
-  (let* ([flags::int (?: (SCM_BOOL_VALUE case-fold) SCM_REGEXP_CASE_FOLD 0)])
-    (return (Scm_RegComp str flags))))
+(define-cproc string->regexp (str::<string> :key (case-fold #f) (multi-line #f))
+  (return 
+   (Scm_RegComp str
+                (logior (?: (SCM_BOOL_VALUE case-fold) SCM_REGEXP_CASE_FOLD 0)
+                        (?: (SCM_BOOL_VALUE multi-line) SCM_REGEXP_MULTI_LINE 0)))))
 (define-cproc regexp-ast (regexp::<regexp>) (return (-> regexp ast)))
 (define-cproc regexp-case-fold? (regexp::<regexp>) ::<boolean>
   (return (logand (-> regexp flags) SCM_REGEXP_CASE_FOLD)))
 
-(define-cproc regexp-parse (str::<string> :key (case-fold #f))
-  (let* ([flags::int (?: (SCM_BOOL_VALUE case-fold) SCM_REGEXP_CASE_FOLD 0)])
-    (return (Scm_RegComp str (logior flags SCM_REGEXP_PARSE_ONLY)))))
-(define-cproc regexp-compile (ast)  Scm_RegCompFromAST)
+(define-cproc regexp-parse (str::<string> :key (case-fold #f) (multi-line #f))
+  (return 
+   (Scm_RegComp str
+                (logior (?: (SCM_BOOL_VALUE case-fold) SCM_REGEXP_CASE_FOLD 0)
+                        (?: (SCM_BOOL_VALUE multi-line) SCM_REGEXP_MULTI_LINE 0)
+                        SCM_REGEXP_PARSE_ONLY))))
+(define-cproc regexp-compile (ast :key (multi-line #f))
+  (return (Scm_RegCompFromAST2 ast
+                               (?: (SCM_BOOL_VALUE multi-line)
+                                   SCM_REGEXP_MULTI_LINE
+                                   0))))
 (define-cproc regexp-optimize (ast) Scm_RegOptimizeAST)
 
 (define-cproc regexp-num-groups (regexp::<regexp>) ::<int>
@@ -57,13 +67,13 @@
 (define-cproc regexp-named-groups (regexp::<regexp>)
   (return (-> regexp grpNames)))
 
-(define-cproc rxmatch (regexp str::<string>)
+(define-cproc rxmatch (regexp str::<string> :optional start end)
   (let* ([rx::ScmRegexp* NULL])
     (cond [(SCM_STRINGP regexp) (set! rx (SCM_REGEXP (Scm_RegComp
                                                       (SCM_STRING regexp) 0)))]
           [(SCM_REGEXPP regexp) (set! rx (SCM_REGEXP regexp))]
           [else (SCM_TYPE_ERROR regexp "regexp")])
-    (return (Scm_RegExec rx str))))
+    (return (Scm_RegExec rx str start end))))
 
 (inline-stub
  (define-cise-stmt rxmatchop
@@ -126,42 +136,56 @@
                        [else (loop2 (rxmatch-substring m 3))]))]
             [else (reverse (cons sub r))]))]
    [(procedure? sub) sub]
-   [else (error "string or procedure required, but got" sub)]))
+   [else (error "string, procedure or list required, but got" sub)]))
 
-;; internal loop
-(define (%regexp-replace-rec match subpat rec)
-  (display (rxmatch-before match))
-  (if (procedure? subpat)
-    (display (subpat match))
-    (dolist [pat subpat]
-      (display (if (or (number? pat) (symbol? pat))
-                 (rxmatch-substring match pat)
-                 pat))))
-  (rec (rxmatch-after match)))
+;; Skip the first subskip matches, then start replacing only up to
+;; subcount times (or infinite if subcount is #f).
+(define (%regexp-replace-rec rx string subpat subskip subcount)
+  (define (next-string match)
+    (let1 rest (rxmatch-after match)
+      (and (not (equal? rest ""))
+           (if (= (rxmatch-start match) (rxmatch-end match))
+             (begin (display (string-ref rest 0))
+                    (string-copy rest 1))
+             rest))))
+  (if (and subcount (zero? subcount))
+    (display string)
+    (let1 match (rxmatch rx string)
+      (cond
+       [(not match)
+        (display string)]
+       [(> subskip 0)
+        (display (rxmatch-before match))
+        (display (rxmatch-substring match))
+        (and-let1 next (next-string match)
+          (%regexp-replace-rec rx next subpat (- subskip 1) subcount))]
+       [else
+        (display (rxmatch-before match))
+        (if (procedure? subpat)
+          (display (subpat match))
+          (dolist [pat subpat]
+            (display (cond
+                      [(eq? pat 'pre) (rxmatch-before match)]
+                      [(eq? pat 'post) (rxmatch-after match)]
+                      [(or (number? pat) (symbol? pat))
+                       (rxmatch-substring match pat)]
+                      [else pat]))))
+        (and-let1 next (next-string match)
+          (%regexp-replace-rec rx next subpat subskip
+                               (and subcount (- subcount 1))))]))))
+
+(define (%regexp-replace rx string subpat subskip subcount)
+  (with-output-to-string
+      (^[]
+        (%regexp-replace-rec rx string subpat subskip subcount))))
 
 (define-in-module gauche (regexp-replace rx string sub)
-  (let ([subpat (%regexp-parse-subpattern sub)]
-        [match  (rxmatch rx string)])
-    (if match
-      (with-output-to-string (cut %regexp-replace-rec match subpat display))
-      string)))
+  (%regexp-replace rx string
+                   (%regexp-parse-subpattern sub) 0 1))
 
 (define-in-module gauche (regexp-replace-all rx string sub)
-  (let ([subpat (%regexp-parse-subpattern sub)]
-        [match  (rxmatch rx string)])
-    (if match
-      (with-output-to-string
-        (^[]
-          (define (loop str)
-            (unless (equal? str "")
-              (cond [(rxmatch rx str)
-                     => (^[match]
-                          (when (= (rxmatch-start match) (rxmatch-end match))
-                            (error "regexp-replace-all: matching zero-length string causes infinite loop:" rx))
-                          (%regexp-replace-rec match subpat loop))]
-                    [else (display str)])))
-          (%regexp-replace-rec match subpat loop)))
-      string)))
+  (%regexp-replace rx string
+                   (%regexp-parse-subpattern sub) 0 #f))
 
 (define (regexp-replace-driver name func-1)
   (^[string rx sub . more]
@@ -203,4 +227,3 @@
         [(eq? (car sel) 'before)
          (apply rxmatch-before (rxmatch rx str) (cdr sel))]
         [else (rxmatch-substring (rxmatch rx str) (car sel))]))
-
