@@ -19,12 +19,12 @@
 (Scm_AddFeature "srfi-22" NULL)
 (Scm_AddFeature "srfi-23" NULL)
 (Scm_AddFeature "srfi-25" "gauche.array")
-(Scm_AddFeature "srfi-26" "srfi-26")
+(Scm_AddFeature "srfi-26" NULL)
 (Scm_AddFeature "srfi-27" "srfi-27")
 (Scm_AddFeature "srfi-28" NULL)
 (Scm_AddFeature "srfi-29" "srfi-29")
 (Scm_AddFeature "srfi-30" NULL)
-(Scm_AddFeature "srfi-31" "srfi-31")
+(Scm_AddFeature "srfi-31" NULL)
 (Scm_AddFeature "srfi-34" NULL)
 (Scm_AddFeature "srfi-35" NULL)
 (Scm_AddFeature "srfi-36" NULL)
@@ -32,6 +32,7 @@
 (Scm_AddFeature "srfi-38" NULL)
 (Scm_AddFeature "srfi-39" "gauche.parameter")
 (Scm_AddFeature "srfi-40" "util.stream")
+(Scm_AddFeature "srfi-41" "srfi-41")
 (Scm_AddFeature "srfi-42" "srfi-42")
 (Scm_AddFeature "srfi-43" "srfi-43")
 (Scm_AddFeature "srfi-45" NULL)
@@ -50,31 +51,56 @@
 (Scm_AddFeature "srfi-96" "srfi-96")
 (Scm_AddFeature "srfi-98" "srfi-98")
 (Scm_AddFeature "srfi-99" "gauche.record")
+(Scm_AddFeature "srfi-101" NULL)
 (Scm_AddFeature "srfi-106" "srfi-106")
-(Scm_AddFeature "srfi-111" "srfi-111")
+(Scm_AddFeature "srfi-111" NULL)
 (Scm_AddFeature "srfi-112" "srfi-112")
 (Scm_AddFeature "srfi-113" "srfi-113")
 (Scm_AddFeature "srfi-114" "srfi-114")
+(Scm_AddFeature "srfi-115" "srfi-115")
+(Scm_AddFeature "srfi-116" "srfi-116")
 (Scm_AddFeature "srfi-117" "srfi-117")
 (Scm_AddFeature "srfi-118" "srfi-118")
 (Scm_AddFeature "srfi-121" "gauche.generator")
+(Scm_AddFeature "srfi-124" "srfi-124")
 (Scm_AddFeature "srfi-125" "srfi-125")
 (Scm_AddFeature "srfi-127" "srfi-127")
 (Scm_AddFeature "srfi-128" "srfi-128")
 (Scm_AddFeature "srfi-129" "srfi-129")
+(Scm_AddFeature "srfi-130" "srfi-130")
 (Scm_AddFeature "srfi-131" "gauche.record")
 (Scm_AddFeature "srfi-132" "srfi-132")
 (Scm_AddFeature "srfi-133" "srfi-133")
 (Scm_AddFeature "srfi-134" "data.ideque")
+(Scm_AddFeature "srfi-135" "srfi-135")
 (Scm_AddFeature "srfi-141" "srfi-141")
 (Scm_AddFeature "srfi-143" "srfi-143")
+(Scm_AddFeature "srfi-144" "srfi-144")
 (Scm_AddFeature "srfi-145" NULL)
-(Scm_AddFeature "srfi-146" NULL)
+(Scm_AddFeature "srfi-146" "srfi-146")
 (Scm_AddFeature "srfi-149" NULL)
 (Scm_AddFeature "srfi-151" "srfi-151")
 (Scm_AddFeature "srfi-152" "srfi-152")
+(Scm_AddFeature "srfi-154" "srfi-154")
 (Scm_AddFeature "srfi-158" "srfi-158")
+(Scm_AddFeature "srfi-159" "srfi-159")
+(Scm_AddFeature "srfi-160" "srfi-160")
+(Scm_AddFeature "srfi-162" "srfi-162")
+(Scm_AddFeature "srfi-170" "srfi-170")
+(Scm_AddFeature "srfi-173" "srfi-173")
+(Scm_AddFeature "srfi-174" "srfi-174")
+(Scm_AddFeature "srfi-175" "srfi-175")
+(Scm_AddFeature "srfi-176" "srfi-176")
+(Scm_AddFeature "srfi-178" "srfi-178")
+(Scm_AddFeature "srfi-180" "srfi-180")
+(Scm_AddFeature "srfi-181" "srfi-181")
+(Scm_AddFeature "srfi-185" "srfi-185")
+(Scm_AddFeature "srfi-189" "srfi-189")
+(Scm_AddFeature "srfi-192" "srfi-192")
+(Scm_AddFeature "srfi-193" "srfi-193")
+(Scm_AddFeature "srfi-195" NULL)
 ))
+(define-module srfi-0 (export cond-expand))
 (define-module srfi-2 (export and-let*))
 (define-module srfi-6 (export open-input-string open-output-string get-output-string))
 (define-module srfi-8 (export receive))
@@ -83,7 +109,9 @@
 (define-module srfi-16 (export case-lambda))
 (define-module srfi-17 (export setter getter-with-setter))
 (define-module srfi-23 (export error))
+(define-module srfi-26 (export cut cute))
 (define-module srfi-28 (export format))
+(define-module srfi-31 (export rec))
 (define-module srfi-34 (export with-exception-handler guard raise))
 (define-module srfi-35 (export make-condition-type condition-type? make-condition condition? condition-has-type? condition-ref make-compound-condition extract-condition define-condition-type condition &condition &message &serious &error))
 (define-module srfi-36 (export &i/o-error &i/o-port-error &i/o-read-error &i/o-write-error &i/o-closed-error &read-error))
@@ -96,3 +124,4 @@
 (define-module srfi-111 (export box box? unbox set-box!))
 (define-module srfi-145 (export assume))
 (define-module srfi-149 (export syntax-rules))
+(define-module srfi-195 (export box box? unbox set-box! box-arity unbox-value set-box-value!))
