@@ -1,7 +1,7 @@
 ;;;
 ;;; libsys.scm - builtin system inteface
 ;;;
-;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -37,7 +37,8 @@
 (select-module gauche)
 (inline-stub
  (declcode
-  (.include <stdlib.h> <locale.h> <math.h> <sys/types.h> <sys/stat.h>)
+  (.include <gauche/class.h>)
+  (.include <stdlib.h> <locale.h> <math.h> <sys/types.h> <sys/stat.h> <fcntl.h>)
   (.cond ["TIME_WITH_SYS_TIME" (.include <sys/time.h> <time.h>)]
          ["HAVE_SYS_TIME_H"    (.include <sys/time.h>)]
          [else                 (.include <time.h>)])
@@ -59,29 +60,82 @@
 
 (select-module gauche)
 (define-cproc sys-readdir (pathname::<string>) Scm_ReadDirectory)
-
-;; Bonus
-
-(define-cproc sys-normalize-pathname (pathname::<string>
-                                      :key (absolute #f)
-                                      (expand #f)
-                                      (canonicalize #f))
-  (let* ([flags::int 0])
-    (unless (SCM_FALSEP absolute) (logior= flags SCM_PATH_ABSOLUTE))
-    (unless (SCM_FALSEP expand)   (logior= flags SCM_PATH_EXPAND))
-    (unless (SCM_FALSEP canonicalize) (logior= flags SCM_PATH_CANONICALIZE))
-    (return (Scm_NormalizePathname pathname flags))))
-
 (define-cproc sys-tmpdir () Scm_TmpDir)
 (define-cproc sys-basename (pathname::<string>) Scm_BaseName)
 (define-cproc sys-dirname (pathname::<string>) Scm_DirName)
 
+(select-module gauche.internal)
+
+;; This isn't POSIX, but we need it in bootstrap so we have it here.
+(define-in-module gauche (sys-normalize-pathname pathname
+                                                 :key 
+                                                 (absolute #f)
+                                                 (expand #f)
+                                                 (canonicalize #f))
+  ;; NB: We can't use cond-expand, for this file is cross-precompiled.
+  (define windows? (assq 'gauche.os.windows (cond-features)))
+  (define separator (if windows? "\\" "/"))
+  (define (expand-tilde path)
+    (if-let1 m (and expand (rxmatch #/^~([^\\\/]*)/ path))
+      (let* ([user (m 1)]
+             [home (if (equal? user "")
+                     (if windows?
+                       (get-windows-home)
+                       (get-unix-home (sys-getpwuid (sys-getuid))))
+                     (if windows?
+                       (error "'~user' expansil isn't supported on Windows:"
+                              path)
+                       (get-unix-home (sys-getpwnam (m 1)))))])
+        (string-append home (m 'after)))
+      path))
+  (define (get-windows-home)
+    (or (sys-getenv "HOME") ; MSYS
+        (sys-getenv "HOMEDRIVE") ; cmd.exe
+        (sys-getenv "HOMEPATH")
+        "\\"))
+  (define (get-unix-home pw)
+    (if pw (~ pw'dir) (error "Couldn't obtain username for :" pathname)))
+  (define (absolute? path)
+    (if windows?
+      (rxmatch #/^([A-Za-z]:)?[\\\/]/ path)
+      (rxmatch #/^\// path)))
+  (define (abs-path path)
+    (if (and absolute (not (absolute? path)))
+      (string-append (sys-getcwd) separator path)
+      path))
+  (define (root? comps)
+    (or (equal? comps '(""))
+        (and windows?
+             (length=? comps 1)
+             (rxmatch #/^[A-Za-z]:$/ (car comps)))))
+  (define (canon-path path)
+    (if canonicalize
+      (let loop ([comps (string-split path #[\\/])]
+                 [r '()]
+                 [dir? #f])             ;whether we add '/' at end
+        (cond [(null? comps) 
+               (let1 r (if dir? (cons "" r) r)
+                 (string-join (reverse r) separator))]
+              [(equal? (car comps) ".") (loop (cdr comps) r #t)]
+              [(equal? (car comps) "..")
+               ;; If we've reached to the root dir, we go like "/../"
+               (if (or (null? r) (equal? (car r) "..") (root? r))
+                 (loop (cdr comps) (cons (car comps) r) #f)
+                 (loop (cdr comps) (cdr r) #t))]
+              [else (loop (cdr comps) (cons (car comps) r) #f)]))
+      (if windows?
+        (string-join (string-split path #\/) "\\")
+        path)))
+  ($ canon-path $ abs-path $ expand-tilde pathname))
+
 ;;---------------------------------------------------------------------
 ;; errno.h - error numbers
 
 ;; We won't (and can't) cover every possible errnos, including system
 ;; specific ones.  The following list is taken from Linux asm/errno.h.
 
+(select-module gauche)
+
 (inline-stub
  "ScmHashTable *errno_n2y;"             ;integer -> symbol
  "ScmHashTable *errno_y2n;"             ;symbol -> integer
@@ -363,10 +417,6 @@
 
 (define-cproc sys-sigwait (mask::<sys-sigset>) ::<int> Scm_SigWait)
 
-(inline-stub
- (initcode (.if "defined HAVE_SIGWAIT"
-                (Scm_AddFeature "gauche.sys.sigwait" NULL))))
-
 (define (sys-sigset . signals)
   (if (null? signals)
     (make <sys-sigset>)
@@ -469,15 +519,6 @@
 (define-cproc sys-unsetenv (name::<const-cstring>) ::<void> Scm_UnsetEnv)
 (define-cproc sys-clearenv () ::<void> Scm_ClearEnv)
 
-(inline-stub
- (initcode
-  (.if "defined(HAVE_PUTENV) || defined(HAVE_SETENV)"
-       (Scm_AddFeature "gauche.sys.setenv" NULL))
-  (.if "defined HAVE_UNSETENV"
-       (Scm_AddFeature "gauche.sys.unsetenv" NULL))
-  (.if "defined HAVE_CLEARENV"
-       (Scm_AddFeature "gauche.sys.clearenv" NULL))))
-
 (define (sys-environ->alist :optional (envlist (sys-environ)))
   (map (^[envstr] (receive (pre post) (string-scan envstr #\= 'both)
                     (if pre (cons pre post) (cons envstr ""))))
@@ -513,10 +554,11 @@
 ;; sys/loadavg.h
 
 (define-cproc sys-getloadavg (:optional (nsamples::<int> 3))
+  (cast void nsamples) ; suppress unused var warning
   (.if "defined HAVE_GETLOADAVG"
        (let* ([samples::(.array double [3])])
          (when (or (<= nsamples 0) (> nsamples 3))
-           (Scm_Error "sys-getloadavg: argument out of range: %d" samples))
+           (Scm_Error "sys-getloadavg: argument out of range: %d" nsamples))
          (let* ([count::int (getloadavg samples nsamples)])
            (if (< count 0)
              (return '#f)
@@ -525,11 +567,9 @@
                  (let* ([n (Scm_MakeFlonum (aref samples i))])
                    (SCM_APPEND1 h t n)))
                (return h)))))
-       (Scm_Error "sys-getloadavg isn't supported on this platform")))
-
-(inline-stub
- (initcode (.if "defined HAVE_GETLOADAVG"
-                (Scm_AddFeature "gauche.sys.getloadavg" NULL))))
+       (begin
+         (Scm_Error "sys-getloadavg isn't supported on this platform")
+         (return SCM_UNDEFINED))))      ;dummy
 
 ;;---------------------------------------------------------------------
 ;; sys/resource.h
@@ -568,8 +608,6 @@
        (SCM_SYSCALL ret (setrlimit rsrc (& limit)))
        (when (< ret 0) (Scm_SysError "setrlimit failed"))))
 
-   (initcode (Scm_AddFeature "gauche.sys.getrlimit" NULL))
-
    (define-constant RLIM_INFINITY (c "MAKERLIMIT(RLIM_INFINITY)"))
    (define-enum-conditionally RLIMIT_AS)
    (define-enum-conditionally RLIMIT_CORE)
@@ -689,18 +727,28 @@
  )
 
 ;; utime.h
+(define-cfn utime-ts (ts::ScmTimeSpec* arg) ::void :static
+  (cond [(SCM_FALSEP arg) (set! (-> ts tv_nsec) UTIME_NOW)]
+        [(SCM_TRUEP arg)  (set! (-> ts tv_nsec) UTIME_OMIT)]
+        [(SCM_REALP arg)
+         (let* ([s::double])
+           (set! (-> ts tv_nsec)
+                 (cast u_long (* (modf (Scm_GetDouble arg) (& s))
+                                 1.0e9)))
+           (set! (-> ts tv_sec) (cast u_long s))
+           (while (>= (-> ts tv_nsec) #e1e9)
+             (set! (-> ts tv_nsec) (- (-> ts tv_nsec) #e1e9))
+             (pre++ (-> ts tv_sec))))]
+        [(SCM_TIMEP arg) (Scm_GetTimeSpec arg ts)]
+        [else (Scm_Error "<time> object, real number or boolean required, but got: %S" arg)]))
+
 (define-cproc sys-utime
   (path::<const-cstring> :optional (atime #f) (mtime #f)) ::<void>
-  (let* ([tim::(struct utimbuf)] [r::int])
-    (cond [(and (SCM_FALSEP atime) (SCM_FALSEP mtime))
-           (SCM_SYSCALL r (utime path NULL))]
-          [else
-           (set! (ref tim actime)
-                 (?: (SCM_FALSEP atime) (time NULL) (Scm_GetUInteger atime)))
-           (set! (ref tim modtime)
-                 (?: (SCM_FALSEP mtime) (time NULL) (Scm_GetUInteger mtime)))
-           (SCM_SYSCALL r (utime path (& tim)))])
-    (when (< r 0) (Scm_SysError "utime failed on %s" path))))
+  (let* ([tss::(.array ScmTimeSpec [2])] [r::int])
+    (utime-ts (& (aref tss 0)) atime)
+    (utime-ts (& (aref tss 1)) mtime)
+    (SCM_SYSCALL r (utimensat AT_FDCWD path tss 0))
+    (when (< r 0) (Scm_SysError "utimensat failed on %s" path))))
 
 ;;---------------------------------------------------------------------
 ;; sys/times.h
@@ -753,7 +801,9 @@
 (define-cproc sys-wait-exit-status (status::<int>) ::<int> WEXITSTATUS)
 (define-cproc sys-wait-signaled? (status::<int>) ::<boolean> WIFSIGNALED)
 (define-cproc sys-wait-termsig (status::<int>) ::<int> WTERMSIG)
-(define-cproc sys-wait-stopped? (status::<int>) ::<boolean> WIFSTOPPED)
+(define-cproc sys-wait-stopped? (status::<int>) ::<boolean>
+  (cast void status) ; suppress unused var warning
+  (return (WIFSTOPPED status)))
 (define-cproc sys-wait-stopsig (status::<int>) ::<int> WSTOPSIG)
 
 ;;---------------------------------------------------------------------
@@ -807,17 +857,20 @@
 (define-cproc sys-asctime (tm::<sys-tm>)
   (return (SCM_MAKE_STR_COPYING (asctime (& (SCM_SYS_TM_TM tm))))))
 
-(define-cproc sys-ctime (time) ::<const-cstring>
-  (let* ([tim::time_t (Scm_GetSysTime time)]) (return (ctime (& tim)))))
+;; NB: For sys-ctime and sys-strftime, we don't use <const-cstring> return
+;; type to use autoboxing.
+;; See https://github.com/shirok/Gauche/issues/638#issuecomment-601777334
+(define-cproc sys-ctime (time)
+  (let* ([tim::time_t (Scm_GetSysTime time)]) 
+    (return (SCM_MAKE_STR_COPYING (ctime (& tim))))))
 
 (define-cproc sys-difftime (time1 time0) ::<double>
   (return (difftime (Scm_GetSysTime time1) (Scm_GetSysTime time0))))
 
 (define-cproc sys-strftime (format::<const-cstring> tm::<sys-tm>)
-  ::<const-cstring>
   (let* ([tmpbuf::(.array char [256])])
     (strftime tmpbuf (sizeof tmpbuf) format (& (SCM_SYS_TM_TM tm)))
-    (return tmpbuf)))
+    (return (SCM_MAKE_STR_COPYING tmpbuf))))
 
 (define-cproc sys-gmtime (time)
   (let* ([tim::time_t (Scm_GetSysTime time)])
@@ -870,6 +923,8 @@
 ;; chown
 (define-cproc sys-chown (path::<const-cstring> owner::<int> group::<int>)
   ::<int>
+  (cast void owner) ; suppress unused var warning
+  (cast void group) ; suppress unused var warning
   (.if "!defined(GAUCHE_WINDOWS)"
        (SCM_SYSCALL SCM_RESULT (chown path owner group))
        (set! SCM_RESULT 0))
@@ -882,7 +937,6 @@
      ::<int>
      (SCM_SYSCALL SCM_RESULT (lchown path owner group))
      (when (< SCM_RESULT 0) (Scm_SysError "lchown failed on %S" path)))
-   (initcode (Scm_AddFeature "gauche.sys.lchown" NULL))
    )
  )
 
@@ -941,6 +995,8 @@
 (define-cproc sys-geteuid () ::<int> geteuid)
 (define-cproc sys-getuid ()  ::<int> getuid)
 
+(define-cproc sys-setugid? () ::<boolean> Scm_IsSugid) ; xBSD's issetugid()
+
 (define-cproc sys-getpid ()  ::<int> getpid)
 (define-cproc sys-getppid () ::<int> getppid)
 
@@ -967,7 +1023,6 @@
      (define-cproc sys-getpgid (pid::<int>) ::<int>
        (SCM_SYSCALL SCM_RESULT (cast int (getpgid pid)))
        (when (< SCM_RESULT 0) (Scm_SysError "getpgid failed")))
-     (initcode (Scm_AddFeature "gauche.sys.getpgid" NULL))
      )
 
    (define-cproc sys-getpgrp () ::<int>
@@ -982,6 +1037,18 @@
      (SCM_SYSCALL SCM_RESULT (setuid uid))
      (when (< SCM_RESULT 0) (Scm_SysError "setuid failed")))
 
+   (define-cfn call-nice (inc::int errno_save::int*) ::int
+     (set! errno 0)
+     (let* ([r::int (nice inc)])
+       (set! (* errno_save) errno)
+       (return r)))
+
+   (define-cproc sys-nice (inc::<int>) ::<int>
+     (let* ([errno_save::int 0])
+       (SCM_SYSCALL SCM_RESULT (call-nice inc (& errno_save)))
+       (when (and (< SCM_RESULT 0) (!= errno 0))
+         (Scm_SysError "nice failed"))))
+   
    ;; some less-frequently used get-*
 
    (define-cproc sys-getgroups ()
@@ -1016,8 +1083,7 @@
                    gids)
          (SCM_SYSCALL r (setgroups ngid glist))
          (when (< r 0)
-           (Scm_SysError "setgroups failed with %S" gids))))
-     (initcode (Scm_AddFeature "gauche.sys.setgroups" NULL)))
+           (Scm_SysError "setgroups failed with %S" gids)))))
    ) ;; !defined(GAUCHE_WINDOWS)
  )
 
@@ -1057,13 +1123,16 @@
 ;; low-level file descriptor handling, and you know what you're doing.
 ;; closing a file descriptor that is still used by Scheme port would
 ;; result a disaster.
+;; NB: close() is not retried on EINTR.  By the time it returns EINTR the
+;; fd has actually been closed, and if other thread happens to grab the same
+;; fd, retrying close() inadvertently closes that one.
 (define-cproc sys-close (fd::<int>) ::<void>
-  (let* ([r::int])
-    (SCM_SYSCALL r (close fd))
+  (let* ([r::int (close fd)])
     (when (< r 0) (Scm_SysError "close failed on file descriptor %d" fd))))
 
 (define-cproc sys-mkdir (pathname::<const-cstring> mode::<int>) ::<void>
   (let* ([r::int])
+    (cast void mode) ; suppress unused var warning
     (.if "!defined(GAUCHE_WINDOWS)"
          (SCM_SYSCALL r (mkdir pathname mode))
          (SCM_SYSCALL r (mkdir pathname)))
@@ -1085,6 +1154,7 @@
 (define-cproc sys-sleep (seconds::<fixnum>
                          :optional (no-retry::<boolean> #f))
   ::<int>
+  (cast void no-retry) ; suppress unused var warning
   (.if "defined(GAUCHE_WINDOWS)"
        (begin (Sleep (* seconds 1000)) (return 0))
        (let* ([k::u_int (cast (u_int) seconds)]
@@ -1129,7 +1199,6 @@
        (if (and (== (ref rem tv_sec) 0) (== (ref rem tv_nsec) 0))
          (return '#f)
          (return (Scm_MakeTime '#f (ref rem tv_sec) (ref rem tv_nsec))))))
-   (initcode (Scm_AddFeature "gauche.sys.nanosleep" NULL))
    ) ; defined(HAVE_NANOSLEEP)||defined(GAUCHE_WINDOWS)
  )
 
@@ -1174,7 +1243,6 @@
  (when "defined(HAVE_CRYPT)"
    (define-cproc sys-crypt (key::<const-cstring> salt::<const-cstring>)
      ::<const-cstring> (return (cast (const char *) (crypt key salt))))
-   (initcode (Scm_AddFeature "gauche.sys.crypt" NULL))
    )
  )
 
@@ -1183,23 +1251,23 @@
 #define HOSTNAMELEN 1024
 #endif")
 
-(define-cproc sys-gethostname () ::<const-cstring>
+(define-cproc sys-gethostname ()
   (.if "defined HAVE_GETHOSTNAME"
        (let* ([buf::(.array char [HOSTNAMELEN])] [r::int])
          (SCM_SYSCALL r (gethostname buf HOSTNAMELEN))
          (when (< r 0) (Scm_SysError "gethostname failed"))
-         (return buf))
+         (return (SCM_MAKE_STR_COPYING buf)))
        ;; TODO: find better alternative
-       (return "localhost")))
+       (return (SCM_MAKE_STR_IMMUTABLE "localhost"))))
 
-(define-cproc sys-getdomainname () ::<const-cstring>
+(define-cproc sys-getdomainname ()
   (.if "defined HAVE_GETDOMAINNAME"
        (let* ([buf::(.array char [HOSTNAMELEN])] [r::int])
          (SCM_SYSCALL r (getdomainname buf HOSTNAMELEN))
          (when (< r 0) (Scm_SysError "getdomainame failed"))
-         (return buf))
+         (return (SCM_MAKE_STR_COPYING buf)))
        ;; TODO: find better alternative
-       (return "local")))
+       (return (SCM_MAKE_STR_IMMUTABLE "local"))))
 
 ;; not supported yet:
 ;;  fpathconf lseek pathconf read sysconf write
@@ -1216,7 +1284,6 @@
        (SCM_SYSCALL r (symlink existing newpath))
        (when (< r 0)
          (Scm_SysError "symlink from %s to %s failed" newpath existing))))
-   (initcode (Scm_AddFeature "gauche.sys.symlink" NULL))
    )
 
  (when "defined(HAVE_READLINK)"
@@ -1227,7 +1294,6 @@
        (when (< n 0) (Scm_SysError "readlink failed on %s" path))
        (when (== n 1024) (Scm_Error "readlink result too long on %s" path))
        (return (Scm_MakeString buf n -1 SCM_STRING_COPYING))))
-   (initcode (Scm_AddFeature "gauche.sys.readlink" NULL))
    )
  )
 
@@ -1291,7 +1357,6 @@
    (define-cproc sys-select! (rfds wfds efds :optional (timeout #f))
      Scm_SysSelectX)
 
-   (initcode (Scm_AddFeature "gauche.sys.select" NULL))
    ) ;; when defined(HAVE_SELECT)
  )
 
@@ -1319,6 +1384,7 @@
    (define-cfn handle-print (h p::ScmPort* c::ScmWriteContext*) ::void :static
      (let* ([type (Scm_ForeignPointerAttrGet (SCM_FOREIGN_POINTER h)
                                              'handle-type '#f)])
+       (cast void c) ; suppress unused var warning
        (cond
         [(SCM_EQ type 'process)
          (Scm_Printf p "#<win:handle process %d @%p>" (Scm_WinProcessPID h) h)]
@@ -1357,7 +1423,7 @@
      (return (Scm_WinHandleP obj 'process)))
 
    (define-cfn Scm_WinProcess (obj) ::HANDLE
-     (Scm_WinHandle obj 'process))
+     (return (Scm_WinHandle obj 'process)))
 
    (define-cproc sys-win-process? (obj) ::<boolean> Scm_WinProcessP)
    (define-cproc sys-win-process-pid (obj) ::<int> Scm_WinProcessPID)
