@@ -1,7 +1,7 @@
 ;;;
 ;;; libvec.scm - builtin vector procedures
 ;;;
-;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -66,11 +66,13 @@
         [else (return (SCM_VECTOR_ELEMENT vec (SCM_INT_VALUE k)))]))
 
 (define-cproc vector-set! (vec::<vector> k::<integer> obj) ::<void>
-  (if (or (SCM_BIGNUMP k)
-          (< (SCM_INT_VALUE k) 0)
-          (>= (SCM_INT_VALUE k) (SCM_VECTOR_SIZE vec)))
-    (Scm_Error "vector-set! index out of range: %S" k)
-    (set! (SCM_VECTOR_ELEMENT vec (SCM_INT_VALUE k)) obj)))
+  (begin
+    (SCM_VECTOR_CHECK_MUTABLE vec)
+    (if (or (SCM_BIGNUMP k)
+            (< (SCM_INT_VALUE k) 0)
+            (>= (SCM_INT_VALUE k) (SCM_VECTOR_SIZE vec)))
+      (Scm_Error "vector-set! index out of range: %S" k)
+      (set! (SCM_VECTOR_ELEMENT vec (SCM_INT_VALUE k)) obj))))
 
 (define-cproc vector->list
   (vec::<vector> :optional (start::<fixnum> 0) (end::<fixnum> -1))
@@ -92,6 +94,7 @@
   (t::<vector> tstart::<fixnum> s::<vector>
                :optional (sstart::<fixnum> 0) (send::<fixnum> -1)) ::<void>
   (let* ([tsize::long (SCM_VECTOR_SIZE t)])
+    (SCM_VECTOR_CHECK_MUTABLE t)
     (when (< send 0) (set! send (SCM_VECTOR_SIZE s)))
     (unless (and (<= 0 tstart) (<= tstart tsize))
       (Scm_Error "tstart out of range: %ld" tstart))
@@ -134,6 +137,10 @@
           (set! j (+ j k))))
       (return dst))))
 
+(select-module gauche)
+(define-cproc vector-immutable? (v::<vector>) ::<boolean>
+  SCM_VECTOR_IMMUTABLE_P)
+
 ;; TRANSIENT: :optional thing will be expanded unhygienically, inserting
 ;; reference to 'error'.  If we define these within #<module scheme> it
 ;; would cause 'error' to be unbound.  Better fix would be to make
@@ -169,19 +176,6 @@
 ;; Most public uniform vector APIs are defined in gauche.uvector.
 ;; We provide a handful basic APIs here for the performance.
 
-(select-module gauche.internal)
-;; TRANSIENT: This is inserted by old compiler macro, expecting to be
-;; futher expanded into VM instruction UVEC-REF by the compiler.
-;; Now we directly expands TAGvector-ref into VM insn so this is no longer
-;; called.  We keep this only for the modules precompiled by older versions.
-;; Will remove on 1.0.
-(define-cproc %uvector-ref (v::<uvector> t::<int> k::<fixnum>
-                                         :optional fallback)
-  :constant
-  (unless (== (Scm_UVectorType (SCM_CLASS_OF v)) t)
-    (Scm_TypeError "vec" (Scm_UVectorTypeName t) (SCM_OBJ v)))
-  (return (Scm_VMUVectorRef v t k fallback)))
-
 (select-module gauche)
 
 (inline-stub
@@ -275,6 +269,27 @@
   (setter f64vector-set!)
   (return (Scm_VMUVectorRef v SCM_UVECTOR_F64 i fallback)))
 
+(define-cproc c32vector-set! (v::<c32vector> i::<fixnum> val :optional clamp)
+  (return (Scm_UVectorSet v SCM_UVECTOR_C32 i val (Scm_ClampMode clamp))))
+(define-cproc c32vector-ref (v::<c32vector> i::<fixnum> :optional fallback)
+  :fast-flonum
+  (setter c32vector-set!)
+  (return (Scm_VMUVectorRef v SCM_UVECTOR_C32 i fallback)))
+
+(define-cproc c64vector-set! (v::<c64vector> i::<fixnum> val :optional clamp)
+  (return (Scm_UVectorSet v SCM_UVECTOR_C64 i val (Scm_ClampMode clamp))))
+(define-cproc c64vector-ref (v::<c64vector> i::<fixnum> :optional fallback)
+  :fast-flonum
+  (setter c64vector-set!)
+  (return (Scm_VMUVectorRef v SCM_UVECTOR_C64 i fallback)))
+
+(define-cproc c128vector-set! (v::<c128vector> i::<fixnum> val :optional clamp)
+  (return (Scm_UVectorSet v SCM_UVECTOR_C128 i val (Scm_ClampMode clamp))))
+(define-cproc c128vector-ref (v::<c128vector> i::<fixnum> :optional fallback)
+  :fast-flonum
+  (setter c128vector-set!)
+  (return (Scm_VMUVectorRef v SCM_UVECTOR_C128 i fallback)))
+
 (define-cproc uvector-set! (v::<uvector> i::<fixnum> val :optional clamp)
   ::<void> :fast-flonum 
   (Scm_UVectorSet v (Scm_UVectorType (Scm_ClassOf (SCM_OBJ v))) i val
@@ -291,13 +306,107 @@
 (define-cproc uvector? (obj) ::<boolean> :constant
   SCM_UVECTORP)
 
+(define-cproc u8vector? (x) ::<boolean> SCM_U8VECTORP)
+(define-cproc s8vector? (x) ::<boolean> SCM_S8VECTORP)
+(define-cproc u16vector? (x) ::<boolean> SCM_U16VECTORP)
+(define-cproc s16vector? (x) ::<boolean> SCM_U16VECTORP)
+(define-cproc u32vector? (x) ::<boolean> SCM_U32VECTORP)
+(define-cproc s32vector? (x) ::<boolean> SCM_U32VECTORP)
+(define-cproc u64vector? (x) ::<boolean> SCM_U64VECTORP)
+(define-cproc s64vector? (x) ::<boolean> SCM_U64VECTORP)
+(define-cproc f16vector? (x) ::<boolean> SCM_F16VECTORP)
+(define-cproc f32vector? (x) ::<boolean> SCM_F32VECTORP)
+(define-cproc f64vector? (x) ::<boolean> SCM_F64VECTORP)
+(define-cproc c32vector? (x) ::<boolean> SCM_C32VECTORP)
+(define-cproc c64vector? (x) ::<boolean> SCM_C64VECTORP)
+(define-cproc c128vector? (x) ::<boolean> SCM_C128VECTORP)
+
+;;;
+;;; Bitvectors
+;;;
+
+(define-cproc bitvector? (obj) ::<boolean> SCM_BITVECTORP) ;srfi-178
+(define-cproc bitvector-length (v::<bitvector>) ::<int>    ;srfi-178
+  SCM_BITVECTOR_SIZE)
+
+(define-cproc make-bitvector (len::<fixnum> :optional (init #f)) ;srfi-178
+  Scm_MakeBitvector)
+(define-cproc bitvector (:rest bits) Scm_ListToBitvector) ;srfi-178
+(define-cproc list->bitvector (bits) Scm_ListToBitvector) ;srfi-178
+
+(define-cproc string->bitvector (s::<string>) ;srfi-178
+  (return (Scm_StringToBitvector s TRUE)))
+(define-cproc bitvector->string (v::<bitvector>) ;srfi-178
+  (return (Scm_BitvectorToString v TRUE)))
+
+(define-cproc bit->integer (bit) ::<int> Scm_Bit2Int) ;srfi-178
+(define-cproc bit->boolean (bit) ::<boolean> Scm_Bit2Int) ;srfi-178
+
+(define-cproc bitvector-set! (v::<bitvector> i::<fixnum> b) ::<void> ;srfi-178
+  (when (or (< i 0) (>= i (SCM_BITVECTOR_SIZE v)))
+    (Scm_Error "bitvector index out of range: %ld" i))
+  (SCM_BITVECTOR_CHECK_MUTABLE v)
+  (if (Scm_Bit2Int b)
+    (SCM_BITS_SET (SCM_BITVECTOR_BITS v) i)
+    (SCM_BITS_RESET (SCM_BITVECTOR_BITS v) i)))
+
+(define-cproc bitvector-ref/int (v::<bitvector> i::<fixnum> ;srfi-178
+                                                :optional fallback)
+  (setter bitvector-set!)
+  (when (or (< i 0) (>= i (SCM_BITVECTOR_SIZE v)))
+    (if (SCM_UNBOUNDP fallback)
+      (Scm_Error "bitvector index out of range: %ld" i)
+      (return fallback)))
+  (return (?: (SCM_BITS_TEST (SCM_BITVECTOR_BITS v) i) 
+              (SCM_MAKE_INT 1)
+              (SCM_MAKE_INT 0))))
+
+(define-cproc bitvector-ref/bool (v::<bitvector> i::<fixnum> ;srfi-178
+                                                 :optional fallback)
+  (setter bitvector-set!)
+  (when (or (< i 0) (>= i (SCM_BITVECTOR_SIZE v)))
+    (if (SCM_UNBOUNDP fallback)
+      (Scm_Error "bitvector index out of range: %ld" i)
+      (return fallback)))
+  (return (?: (SCM_BITS_TEST (SCM_BITVECTOR_BITS v) i) SCM_TRUE SCM_FALSE)))
+
+(define-cproc bitvector-copy (v::<bitvector> 
+                              :optional (start::<fixnum> 0)
+                                        (end::<fixnum> -1)) ;srfi-178
+  Scm_BitvectorCopy)
+
+(define-cproc bitvector-copy! (dest::<bitvector>
+                               dstart::<fixnum>
+                               src::<bitvector>
+                               :optional (start::<fixnum> 0)
+                                         (end::<fixnum> -1)) ;srfi-178
+  Scm_BitvectorCopyX)
+
 ;;;
 ;;; Flat vector API (interact with underlying C array)
 ;;;
 
+(select-module gauche)
+
 (inline-stub
+ (.define common_eqv (a b) (== a b))
+ (.define common_lt  (a b) (< a b))
+
+ (.define hc_eqv (a b) (and (== (ref a r) (ref b r))
+                            (== (ref a i) (ref b i))))
+ (.define hc_lt (a b) (or (< (ref a r) (ref b r))
+                          (and (== (ref a r) (ref b r))
+                               (< (ref a i) (ref b i)))))
+
+ (.define fc_lt (a b) (or (< (crealf a) (crealf b))
+                          (and (== (crealf a) (crealf b))
+                               (< (cimagf a) (cimagf b)))))
+ (.define dc_lt (a b) (or (< (creal a) (creal b))
+                          (and (== (creal a) (creal b))
+                               (< (cimag a) (cimag b)))))
+ 
  (define-cise-stmt %binary-search
-   [(_ elttype)
+   [(_ elttype eq lt)
     `(let* ([esize::u_int  (+ skip 1)]
             [nume::size_t (/ len esize)]
             [k::size_t (/ nume 2)]
@@ -305,8 +414,8 @@
             [lo::size_t 0])
        (while (< lo hi)
          (let* ([v:: ,elttype (aref vec (* k esize))])
-           (cond [(== v key) (return (* k esize))]
-                 [(< v key)
+           (cond [(,eq v key) (return (* k esize))]
+                 [(,lt v key)
                   (set! lo k) (set! k (+ lo (/ (- hi lo) 2)))
                   (when (== lo k) (break))]
                  [else
@@ -327,7 +436,7 @@
                                  skip::u_int
                                  floor::size_t*
                                  ceil::size_t*)
-   ::size_t (%binary-search int8_t))
+   ::size_t (%binary-search int8_t common-eqv common-lt))
 
  (define-cfn Scm_BinarySearchU8 (vec::(const uint8_t*)
                                  len::size_t
@@ -335,7 +444,7 @@
                                  skip::u_int
                                  floor::size_t*
                                  ceil::size_t*)
-   ::size_t (%binary-search uint8_t))
+   ::size_t (%binary-search uint8_t common-eqv common-lt))
 
  (define-cfn Scm_BinarySearchS16 (vec::(const int16_t*)
                                   len::size_t
@@ -343,7 +452,7 @@
                                   skip::u_int
                                   floor::size_t*
                                   ceil::size_t*)
-   ::size_t (%binary-search int16_t))
+   ::size_t (%binary-search int16_t common-eqv common-lt))
 
  (define-cfn Scm_BinarySearchU16 (vec::(const uint16_t*)
                                   len::size_t
@@ -351,7 +460,7 @@
                                   skip::u_int
                                   floor::size_t*
                                   ceil::size_t*)
-   ::size_t (%binary-search uint16_t))
+   ::size_t (%binary-search uint16_t common-eqv common-lt))
 
  (define-cfn Scm_BinarySearchS32 (vec::(const int32_t*)
                                   len::size_t
@@ -359,7 +468,7 @@
                                   skip::u_int
                                   floor::size_t*
                                   ceil::size_t*)
-   ::size_t (%binary-search int32_t))
+   ::size_t (%binary-search int32_t common-eqv common-lt))
 
  (define-cfn Scm_BinarySearchU32 (vec::(const uint32_t*)
                                   len::size_t
@@ -367,23 +476,23 @@
                                   skip::u_int
                                   floor::size_t*
                                   ceil::size_t*)
-   ::size_t (%binary-search uint32_t))
+   ::size_t (%binary-search uint32_t common-eqv common-lt))
 
- (define-cfn Scm_BinarySearchS64 (vec::(const ScmInt64*)
+ (define-cfn Scm_BinarySearchS64 (vec::(const int64_t*)
                                   len::size_t
-                                  key::ScmInt64
+                                  key::int64_t
                                   skip::u_int
                                   floor::size_t*
                                   ceil::size_t*)
-   ::size_t (%binary-search ScmInt64))
+   ::size_t (%binary-search int64_t common-eqv common-lt))
 
- (define-cfn Scm_BinarySearchU64 (vec::(const ScmUInt64*)
+ (define-cfn Scm_BinarySearchU64 (vec::(const uint64_t*)
                                   len::size_t
-                                  key::ScmUInt64
+                                  key::uint64_t
                                   skip::u_int
                                   floor::size_t*
                                   ceil::size_t*)
-   ::size_t (%binary-search ScmUInt64))
+   ::size_t (%binary-search uint64_t common-eqv common-lt))
  
  (define-cfn Scm_BinarySearchF16 (vec::(const ScmHalfFloat*)
                                   len::size_t
@@ -391,7 +500,7 @@
                                   skip::u_int
                                   floor::size_t*
                                   ceil::size_t*)
-   ::size_t (%binary-search ScmHalfFloat))
+   ::size_t (%binary-search ScmHalfFloat common-eqv common-lt))
 
  (define-cfn Scm_BinarySearchF32 (vec::(const float*)
                                   len::size_t
@@ -399,7 +508,7 @@
                                   skip::u_int
                                   floor::size_t*
                                   ceil::size_t*)
-   ::size_t (%binary-search float))
+   ::size_t (%binary-search float common-eqv common-lt))
 
  (define-cfn Scm_BinarySearchF64 (vec::(const double*)
                                   len::size_t
@@ -407,5 +516,30 @@
                                   skip::u_int
                                   floor::size_t*
                                   ceil::size_t*)
-   ::size_t (%binary-search double))
+   ::size_t (%binary-search double common-eqv common-lt))
+
+ (define-cfn Scm_BinarySearchC32 (vec::(const ScmHalfComplex*)
+                                  len::size_t
+                                  key::ScmHalfComplex
+                                  skip::u_int
+                                  floor::size_t*
+                                  ceil::size_t*)
+   ::size_t (%binary-search ScmHalfComplex hc-eqv hc-lt))
+ 
+ (define-cfn Scm_BinarySearchC64 (vec::(const ScmFloatComplex*)
+                                  len::size_t
+                                  key::(ScmFloatComplex)
+                                  skip::u_int
+                                  floor::size_t*
+                                  ceil::size_t*)
+   ::size_t (%binary-search (ScmFloatComplex) common-eqv fc-lt))
+
+ (define-cfn Scm_BinarySearchC128 (vec::(const ScmDoubleComplex*)
+                                   len::size_t
+                                   key::(ScmDoubleComplex)
+                                   skip::u_int
+                                   floor::size_t*
+                                   ceil::size_t*)
+   ::size_t (%binary-search (ScmDoubleComplex) common-eqv dc-lt))
  )
+
