@@ -1,7 +1,7 @@
 /*
  * load.c - load a program
  *
- *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -66,13 +66,13 @@ static struct {
     ScmInternalMutex prov_mutex;
     ScmInternalCond  prov_cv;
 
-    /* Dynamic enviornments kept during specific `load'.  They are
+    /* Dynamic environments kept during specific `load'.  They are
        thread-specific, and we use ScmParameter mechanism. */
-    ScmParameterLoc load_history;      /* history of the nested load */
-    ScmParameterLoc load_next;         /* list of the directories to be
-                                          searched. */
-    ScmParameterLoc load_port;         /* current port from which we are
-                                          loading */
+    ScmPrimitiveParameter *load_history; /* history of the nested load */
+    ScmPrimitiveParameter *load_next;    /* list of the directories to be
+                                            searched. */
+    ScmPrimitiveParameter *load_port;    /* current port from which we are
+                                            loading */
     
     /* Dynamic linking */
     ScmObj dso_suffixes;
@@ -85,7 +85,7 @@ static struct {
                                      but we may change this design in future.
                                   */
     ScmInternalMutex dso_mutex;
-} ldinfo = { (ScmGloc*)&ldinfo, };  /* trick to put ldinfo in .data section */
+} ldinfo;
 
 /* keywords used for load and load-from-port surbs */
 static ScmObj key_error_if_not_found = SCM_UNBOUND;
@@ -93,8 +93,9 @@ static ScmObj key_macro              = SCM_UNBOUND;
 static ScmObj key_ignore_coding      = SCM_UNBOUND;
 static ScmObj key_paths              = SCM_UNBOUND;
 static ScmObj key_environment        = SCM_UNBOUND;
+static ScmObj key_main_script        = SCM_UNBOUND;
 
-#define PARAM_REF(vm, loc)      Scm_ParameterRef(vm, &ldinfo.loc)
+#define PARAM_REF(vm, loc)      Scm_PrimitiveParameterRef(vm, ldinfo.loc)
 
 /*
  * ScmLoadPacket is the way to communicate to Scm_Load facility.
@@ -195,6 +196,9 @@ ScmObj Scm_VMLoad(ScmString *filename, ScmObj paths, ScmObj env, int flags)
     if (flags&SCM_LOAD_IGNORE_CODING) {
         opts = Scm_Cons(key_ignore_coding, Scm_Cons(SCM_TRUE, opts));
     }
+    if (flags&SCM_LOAD_MAIN_SCRIPT) {
+        opts = Scm_Cons(key_main_script, Scm_Cons(SCM_TRUE, opts));
+    }
     if (SCM_NULLP(paths) || SCM_PAIRP(paths)) {
         opts = Scm_Cons(key_paths, Scm_Cons(paths, opts));
     }
@@ -217,6 +221,9 @@ int Scm_Load(const char *cpath, u_long flags, ScmLoadPacket *packet)
     if (flags&SCM_LOAD_IGNORE_CODING) {
         opts = Scm_Cons(key_ignore_coding, Scm_Cons(SCM_TRUE, opts));
     }
+    if (flags&SCM_LOAD_MAIN_SCRIPT) {
+        opts = Scm_Cons(key_main_script, Scm_Cons(SCM_TRUE, opts));
+    }
 
     load_packet_prepare(packet);
     if (flags&SCM_LOAD_PROPAGATE_ERROR) {
@@ -394,7 +401,7 @@ void Scm_DeleteLoadPathHook(ScmObj proc)
  * functions.
  */
 
-/* The implementation of dynamic loader is a bit complicated in the presense
+/* The implementation of dynamic loader is a bit complicated in the presence
    of multiple threads and multiple initialization routines.
 
    We keep ScmDLObj record for each DYNAMIC-LOADed files (keyed
@@ -433,7 +440,8 @@ struct ScmDLObjRec {
     ScmInternalCond  cv;
 };
 
-static void dlobj_print(ScmObj obj, ScmPort *sink, ScmWriteContext *mode)
+static void dlobj_print(ScmObj obj, ScmPort *sink, 
+                        ScmWriteContext *mode SCM_UNUSED)
 {
     Scm_Printf(sink, "#<dlobj \"%s\">", SCM_DLOBJ(obj)->path);
 }
@@ -587,7 +595,7 @@ static void call_initfn(ScmDLObj *dlo, const char *name)
    already loaded from a pseudo pathname "@/DSONAME" (e.g. for
    "gauche--collection", we use "@/gauche--collection".) */
 
-/* Register DSONAME as prelinked.  DSONAME shoudn't have system's suffix.
+/* Register DSONAME as prelinked.  DSONAME shouldn't have system's suffix.
    INITFNS is an array of function pointers, NULL terminated.
    INITFN_NAMES should have prefixed with '_', for call_initfn() searches
    names with '_' first. */
@@ -632,7 +640,8 @@ static ScmObj find_prelinked(ScmString *dsoname)
    A DSO may contain multiple initialization functions (initfns), in
    which case each initfn is called at most once.
 */
-ScmObj Scm_DynLoad(ScmString *dsoname, ScmObj initfn, u_long flags/*reserved*/)
+ScmObj Scm_DynLoad(ScmString *dsoname, ScmObj initfn,
+                   u_long flags SCM_UNUSED /*reserved*/)
 {
     ScmObj dsopath = find_prelinked(dsoname);
     if (SCM_FALSEP(dsopath)) {
@@ -817,7 +826,7 @@ int do_require(ScmObj feature, int flags, ScmModule *base_mod,
     load_packet_prepare(packet);
     if (!SCM_STRINGP(feature)) {
         ScmObj e = Scm_MakeError(Scm_Sprintf("require: string expected, but got %S\n", feature));
-        if (flags&SCM_LOAD_PROPAGATE_ERROR) Scm_Raise(e);
+        if (flags&SCM_LOAD_PROPAGATE_ERROR) Scm_Raise(e, 0);
         else {
             if (packet) packet->exception = e;
             return -1;
@@ -858,7 +867,7 @@ int do_require(ScmObj feature, int flags, ScmModule *base_mod,
 
     if (loop) {
         ScmObj e = Scm_MakeError(Scm_Sprintf("a loop is detected in the require dependency involving feature %S", feature));
-        if (flags&SCM_LOAD_PROPAGATE_ERROR) Scm_Raise(e);
+        if (flags&SCM_LOAD_PROPAGATE_ERROR) Scm_Raise(e, 0);
         else {
             if (packet) packet->exception = e;
             return -1;
@@ -881,7 +890,7 @@ int do_require(ScmObj feature, int flags, ScmModule *base_mod,
         ldinfo.providing = Scm_AssocDeleteX(feature, ldinfo.providing, SCM_CMP_EQUAL);
         (void)SCM_INTERNAL_COND_BROADCAST(ldinfo.prov_cv);
         (void)SCM_INTERNAL_MUTEX_UNLOCK(ldinfo.prov_mutex);
-        if (flags&SCM_LOAD_PROPAGATE_ERROR) Scm_Raise(xresult.exception);
+        if (flags&SCM_LOAD_PROPAGATE_ERROR) Scm_Raise(xresult.exception, 0);
         else return -1;
     }
 
@@ -915,7 +924,7 @@ ScmObj Scm_Provide(ScmObj feature)
     ScmObj cp;
     SCM_FOR_EACH(cp, ldinfo.providing) {
         if (SCM_CADR(SCM_CAR(cp)) == SCM_OBJ(self)) {
-            SCM_SET_CDR(SCM_CDR(SCM_CAR(cp)), SCM_LIST1(feature));
+            SCM_SET_CDR_UNCHECKED(SCM_CDR(SCM_CAR(cp)), SCM_LIST1(feature));
             break;
         }
     }
@@ -936,7 +945,8 @@ int Scm_ProvidedP(ScmObj feature)
  * Autoload
  */
 
-static void autoload_print(ScmObj obj, ScmPort *out, ScmWriteContext *ctx)
+static void autoload_print(ScmObj obj, ScmPort *out, 
+                           ScmWriteContext *ctx SCM_UNUSED)
 {
     Scm_Printf(out, "#<autoload %A::%A (%A)>",
                SCM_AUTOLOAD(obj)->module->name,
@@ -1002,7 +1012,7 @@ void Scm_DefineAutoload(ScmModule *where,
 }
 
 
-ScmObj Scm_ResolveAutoload(ScmAutoload *adata, int flags)
+ScmObj Scm_ResolveAutoload(ScmAutoload *adata, int flags SCM_UNUSED)
 {
     int circular = FALSE;
     ScmVM *vm = Scm_VM();
@@ -1119,6 +1129,7 @@ ScmObj Scm_CurrentLoadPort()    { return PARAM_REF(Scm_VM(), load_port); }
  * Compatibility stuff
  */
 
+#if GAUCHE_API_VERSION < 1000
 /* TRANSIENT: Pre-0.9 Compatibility routine.  Kept for the binary compatibility.
    Will be removed on 1.0 */
 void Scm__LoadFromPortCompat(ScmPort *port, int flags)
@@ -1143,8 +1154,10 @@ ScmObj Scm__RequireCompat(ScmObj feature)
 
 /* TRANSIENT: This is entirely moved to Scheme (libeval.scm).  The entry is
    kept only for the binary compatibility. */
-ScmObj Scm_VMLoadFromPort(ScmPort *port, ScmObj next_paths,
-                          ScmObj env, int flags)
+ScmObj Scm_VMLoadFromPort(ScmPort *port SCM_UNUSED,
+                          ScmObj next_paths SCM_UNUSED,
+                          ScmObj env SCM_UNUSED,
+                          int flags SCM_UNUSED)
 {
     Scm_Error("[internal] Scm_VMLoadFromPort() is obsoleted; call load-from-port Scheme procedure.");
     return SCM_UNDEFINED;
@@ -1152,7 +1165,7 @@ ScmObj Scm_VMLoadFromPort(ScmPort *port, ScmObj next_paths,
 
 /* TRANSIENT: Kept for the binary compatibility; the feature
    is in libeval.scm now. */
-void Scm__RecordLoadStart(ScmObj load_file_path/*ARGSUSED*/)
+void Scm__RecordLoadStart(ScmObj load_file_path SCM_UNUSED)
 {
 }
 
@@ -1161,6 +1174,8 @@ ScmObj Scm_LoadMainScript()
 {
     return SCM_UNDEFINED;
 }
+#endif /*GAUCHE_API_VERSION < 1000*/
+
 
 /*------------------------------------------------------------------
  * Initialization
@@ -1196,6 +1211,7 @@ void Scm__InitLoad(void)
     key_ignore_coding = SCM_MAKE_KEYWORD("ignore-coding");
     key_paths = SCM_MAKE_KEYWORD("paths");
     key_environment = SCM_MAKE_KEYWORD("environment");
+    key_main_script = SCM_MAKE_KEYWORD("main-script");
 
     Scm_InitStaticClass(SCM_CLASS_DLOBJ, "<dlobj>",
                         m, dlobj_slots, 0);
@@ -1218,7 +1234,7 @@ void Scm__InitLoad(void)
     ldinfo.dso_table = SCM_HASH_TABLE(Scm_MakeHashTableSimple(SCM_HASH_STRING,0));
     ldinfo.dso_prelinked = SCM_NIL;
 
-#define PARAM_INIT(var, name, val) Scm_DefinePrimitiveParameter(m, name, val, &ldinfo.var)
+#define PARAM_INIT(var, name, val) ldinfo.var = Scm_BindPrimitiveParameter(m, name, val, 0)
     PARAM_INIT(load_history, "current-load-history", SCM_NIL);
     PARAM_INIT(load_next, "current-load-next", SCM_NIL);
     PARAM_INIT(load_port, "current-load-port", SCM_FALSE);
