@@ -1,7 +1,7 @@
 /*
  * macro.c - macro implementation
  *
- *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -50,19 +50,28 @@
  * Syntax object
  */
 
-static void syntax_print(ScmObj obj, ScmPort *port, ScmWriteContext *mode)
+static void syntax_print(ScmObj obj, ScmPort *port, 
+                         ScmWriteContext *mode SCM_UNUSED)
 {
     ScmSymbol *name = SCM_SYNTAX(obj)->name;
-    Scm_Printf(port, "#<syntax %A>", (name ? SCM_OBJ(name) : SCM_FALSE));
+    ScmModule *mod = SCM_SYNTAX(obj)->mod;
+    if (mod == NULL) {
+        Scm_Printf(port, "#<syntax %A>", (name ? SCM_OBJ(name) : SCM_FALSE));
+    } else {
+        Scm_Printf(port, "#<syntax %A#%A>", 
+                   mod->name,
+                   (name ? SCM_OBJ(name) : SCM_FALSE));
+    }
 }
 
 SCM_DEFINE_BUILTIN_CLASS_SIMPLE(Scm_SyntaxClass, syntax_print);
 
-ScmObj Scm_MakeSyntax(ScmSymbol *name, ScmObj handler)
+ScmObj Scm_MakeSyntax(ScmSymbol *name, ScmModule *mod, ScmObj handler)
 {
     ScmSyntax *s = SCM_NEW(ScmSyntax);
     SCM_SET_CLASS(s, SCM_CLASS_SYNTAX);
     s->name = name;
+    s->mod = mod;
     s->handler = handler;
     return SCM_OBJ(s);
 }
@@ -71,8 +80,8 @@ ScmObj Scm_MakeSyntax(ScmSymbol *name, ScmObj handler)
  * Macro object
  */
 
-ScmObj Scm_MakeMacroFull(ScmObj name, ScmObj transformer,
-                         ScmObj src, ScmObj describer)
+ScmObj Scm_MakeMacro(ScmObj name, ScmObj transformer,
+                     ScmObj src, ScmObj describer)
 {
     ScmMacro *s = SCM_NEW(ScmMacro);
     SCM_SET_CLASS(s, SCM_CLASS_MACRO);
@@ -83,15 +92,6 @@ ScmObj Scm_MakeMacroFull(ScmObj name, ScmObj transformer,
     return SCM_OBJ(s);
 }
 
-#if GAUCHE_API_0_95
-/* Kept for the backward compatibility */
-ScmObj Scm_MakeMacro(ScmSymbol *name, ScmObj transformer)
-{
-    return Scm_MakeMacroFull(SCM_OBJ_SAFE(name), transformer,
-                             SCM_FALSE, SCM_FALSE);
-}
-#endif
-
 ScmObj Scm_MacroTransformer(ScmMacro *mac)
 {
     return mac->transformer;
@@ -130,7 +130,8 @@ SCM_CLASS_DECL(Scm_SyntaxPatternClass);
 #define SCM_SYNTAX_PATTERN(obj)   ((ScmSyntaxPattern*)(obj))
 #define SCM_SYNTAX_PATTERN_P(obj) SCM_XTYPEP(obj, SCM_CLASS_SYNTAX_PATTERN)
 
-static void pattern_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
+static void pattern_print(ScmObj obj, ScmPort *port, 
+                          ScmWriteContext *ctx SCM_UNUSED)
 {
     Scm_Printf(port, "#<pattern:%d%S %S%s>",
                SCM_SYNTAX_PATTERN(obj)->level,
@@ -157,7 +158,8 @@ ScmSyntaxPattern *make_syntax_pattern(int level, int numFollowing)
  *   Internal object to construct pattern matcher
  */
 
-static void synrule_print(ScmObj obj, ScmPort *port, ScmWriteContext *mode)
+static void synrule_print(ScmObj obj, ScmPort *port, 
+                          ScmWriteContext *mode SCM_UNUSED)
 {
     ScmSyntaxRules *r = SCM_SYNTAX_RULES(obj);
 
@@ -186,26 +188,16 @@ ScmSyntaxRules *make_syntax_rules(int nr)
  * Traditional Macro
  */
 
-/* TODO: how to retain debug info? */
-/* TODO: better error message on syntax error (macro invocation with
-   bad number of arguments) */
-
-static ScmObj macro_transform_old(ScmObj *argv, int argc, void *data)
-{
-    SCM_ASSERT(argc == 2);
-    ScmObj form = argv[0];      /* we ignore env (argv[1]) */
-    ScmObj proc = SCM_OBJ(data);
-    SCM_ASSERT(SCM_PAIRP(form));
-    return Scm_VMApply(proc, SCM_CDR(form));
-}
-
 /* TRANSIENT
    This used to be called via make-macro-transformer, but no longer.
    We leave stub here for ABI compatibility. */
-ScmObj Scm_MakeMacroTransformerOld(ScmSymbol *name, ScmProcedure *proc)
+#if GAUCHE_API_VERSION < 1000
+ScmObj Scm_MakeMacroTransformerOld(ScmSymbol *name SCM_UNUSED,
+                                   ScmProcedure *proc SCM_UNUSED)
 {
     Scm_Panic("Obsoleted Scm_MakeMacroTransformerOld called!  Something is wrong!");
 }
+#endif /*GAUCHE_API_VERSION < 1000*/
 
 static ScmMacro *resolve_macro_autoload(ScmAutoload *adata)
 {
@@ -241,8 +233,8 @@ ScmObj Scm_MakeMacroAutoload(ScmSymbol *name, ScmAutoload *adata)
                                      AUTOLOAD_MACRO_SUFFIX,
                                      sizeof(AUTOLOAD_MACRO_SUFFIX)-1,
                                      sizeof(AUTOLOAD_MACRO_SUFFIX)-1);
-    return Scm_MakeMacroFull(Scm_MakeSymbol(SCM_STRING(name1), FALSE),
-                             transformer, SCM_FALSE, SCM_FALSE);
+    return Scm_MakeMacro(Scm_MakeSymbol(SCM_STRING(name1), FALSE),
+                         transformer, SCM_FALSE, SCM_FALSE);
 }
 
 /*===================================================================
@@ -337,14 +329,6 @@ static inline ScmObj pvar_to_pvref(PatternContext *ctx,
     return pvref;
 }
 
-static inline ScmObj pvref_to_pvar(PatternContext *ctx, ScmObj pvref)
-{
-    int count = PVREF_COUNT(pvref);
-    ScmObj q = Scm_ListRef(ctx->pvars, count, SCM_UNBOUND);
-    SCM_ASSERT(SCM_PAIRP(q));
-    return SCM_CAR(q);
-}
-
 static int isEllipsis(PatternContext *ctx, ScmObj obj)
 {
     if (SCM_FALSEP(ctx->ellipsis)) return FALSE; /* inside (... TEMPLATE) */
@@ -715,17 +699,17 @@ static void grow_branch(MatchVar *rec, int level)
         }
         if (SCM_NULLP(SCM_CAR(trunc))) {
             for (i++; i<level-1; i++, trunc = SCM_CAR(trunc)) {
-                SCM_SET_CAR(trunc, SPROUT);
+                SCM_SET_CAR_UNCHECKED(trunc, SPROUT);
             }
             rec->sprout = SPROUT;
-            SCM_SET_CAR(trunc, rec->sprout);
+            SCM_SET_CAR_UNCHECKED(trunc, rec->sprout);
             return;
         }
     }
     SCM_FOR_EACH(trunc, trunc) {
         if (SCM_NULLP(SCM_CDR(trunc))) {
             rec->sprout = SPROUT;
-            SCM_SET_CDR(trunc, rec->sprout);
+            SCM_SET_CDR_UNCHECKED(trunc, rec->sprout);
             break;
         }
     }
@@ -751,8 +735,8 @@ static void exit_subpattern(ScmSyntaxPattern *subpat, MatchVar *mvec)
             if (subpat->level == 1) {
                 mvec[count].root = Scm_ReverseX(mvec[count].branch);
             } else {
-                SCM_SET_CAR(mvec[count].sprout,
-                            Scm_ReverseX(mvec[count].branch));
+                SCM_SET_CAR_UNCHECKED(mvec[count].sprout,
+                                      Scm_ReverseX(mvec[count].branch));
                 mvec[count].branch = SCM_NIL;
             }
         }
@@ -1017,7 +1001,7 @@ ScmObj Scm_CompileSyntaxRules(ScmObj name, ScmObj src, ScmObj ellipsis,
                                        SCM_MODULE(mod), env);
     ScmObj sr_xform = Scm_MakeSubr(synrule_transform, sr,
                                    2, 0, SCM_FALSE);
-    return Scm_MakeMacroFull(name, sr_xform, src, SCM_FALSE);
+    return Scm_MakeMacro(name, sr_xform, src, SCM_FALSE);
 }
 
 /*===================================================================
@@ -1028,11 +1012,15 @@ ScmObj Scm_CompileSyntaxRules(ScmObj name, ScmObj src, ScmObj ellipsis,
    Now it's in compile.scm (%internal-macro-expand).  This is kept
    for ABI compatibility, but nobody is supposed to call this.
  */
-ScmObj Scm_VMMacroExpand(ScmObj expr, ScmObj env, int oncep)
+#if GAUCHE_API_VERSION < 1000
+ScmObj Scm_VMMacroExpand(ScmObj expr SCM_UNUSED,
+                         ScmObj env SCM_UNUSED,
+                         int oncep SCM_UNUSED)
 {
     Scm_Error("Scm_VMMacroExpand is obsoleted.");
     return SCM_UNDEFINED;
 }
+#endif /*GAUCHE_API_VERSION < 1000*/
 
 ScmObj Scm_CallMacroExpander(ScmMacro *mac, ScmObj expr, ScmObj cenv)
 {
