@@ -1,9 +1,10 @@
+#!/bin/sh
 # Create MinGW binary distribution.
 # Run this script on topsrcdir
 # You need MinGW and MSYS.  (You no longer need Cygwin.)
 # The compiled binary is installed in ../Gauche-mingw-dist.  You can just
 # zip it and distribute.
-# See doc/HOWTO-mingw.txt for the details.
+# See doc/HOWTO-mingw.adoc for the details.
 
 set -e
 
@@ -48,12 +49,27 @@ while [ "$#" -gt 0 ]; do
   case $1 in
     --with-gl)   WITH_GL=yes; shift;;
     --with-installer) INSTALLER=yes; shift;;
+    --with-mbedtls) MBEDTLS=yes; shift;;
+    --with-mbedtls=dll) MBEDTLS=dll; shift;;
+    --with-zip) ZIP_ARCHIVE=yes; shift;;
     --skip-config) SKIP_CONFIG=yes; shift;;
     -*)
      echo "Options:"
      echo "  --with-gl: Include Gauche-gl.  Gauche-gl source must be in ../Gauche-gl."
      echo "  --with-installer:  Creates binary installer using Wix.  'candle.exe' and"
      echo "      'light.exe' must be visible in PATH."
+     echo "  --with-mbedtls: Include MbedTLS."
+     echo "      If you create an installer with this option, the binary"
+     echo "      will also be covered by Apache License 2.0 for MbedTLS."
+     echo "      This option comes with two flavors:"
+     echo "      * If you simply say --with-mbedtls, we build MbedTLS from source"
+     echo "        and bundle it with extension modules, so you don't need MbedTLS"
+     echo "        DLL on the system.  You need CMake, though."
+     echo "      * If you say --with-mbedtls=dll, we build to link with MbedTLS"
+     echo "        DLL installed on the system.  You need mingw-w64-{i686|x86_64}-mbedtls."
+     echo "        If you choose this option, those DLLs in turn depends on libgcc DLL."
+     echo "  --with-zip:  Creates Zip archive using p7zip. '7z.exe' must be visible"
+     echo "      in PATH."
      echo "  --skip-config:  Skip cleanup and configuration."
      exit 1;;
   esac
@@ -72,6 +88,14 @@ if [ "$INSTALLER" = yes ]; then
   fi
 fi
 
+if [ "$MBEDTLS" = yes ]; then
+  tlslibs=axtls,mbedtls-internal
+elif [ "$MBEDTLS" = dll ]; then
+  tlslibs=axtls,mbedtls
+else
+  tlslibs=axtls
+fi
+
 # check gosh
 gosh_path=`which gosh.exe 2> /dev/null`
 if test -e "$gosh_path"; then
@@ -101,8 +125,8 @@ fi
 if [ "$SKIP_CONFIG" != yes ]; then
   rm -rf $distdir
   ./configure --prefix=$distdir --enable-threads=win32 \
-              --enable-multibyte=utf8 --enable-ipv6=no \
-	      --with-tls=axtls \
+              --enable-multibyte=utf8 --enable-ipv6=yes \
+	      --with-tls=$tlslibs \
               --with-dbm=ndbm,odbm $buildopt
 fi
 make
@@ -119,17 +143,25 @@ make install-examples
 rm -rf $distdir/lib/libgauche.dll*
 case "$MSYSTEM" in
   MINGW64|MINGW32)
-    for dll in libwinpthread-1.dll; do
-      if [ -f $mingwdir/bin/$dll ]; then
-        cp $mingwdir/bin/$dll $distdir/bin
-      fi
-    done
-    ;;
+    mingw_dll="libwinpthread-1.dll";;
   *)
-    cp $mingwdir/bin/mingwm10.dll $distdir/bin
-    ;;
+    mingw_dll="mingwm10.dll";;
 esac
 
+# If we use external mbedtls, we need these DLLs.
+# NB: those dlls also depend on libgcc.
+if [ "$MBEDTLS" = dll ]; then
+  mingw_dll="$mingw_dll libmbedcrypto.dll libmbedtls.dll libmbedx509.dll"
+fi
+
+if [ -n "$mingw_dll" ]; then
+  for dll in $mingw_dll; do
+    if [ -f $mingwdir/bin/$dll ]; then
+      cp $mingwdir/bin/$dll $distdir/bin
+    fi
+  done
+fi
+
 # Enable using freshly-built gosh for the subsequent operations.
 PATH=$distdir/bin:$PATH
 
@@ -147,6 +179,11 @@ if [ "$INSTALLER" = "yes" ]; then
   cp winnt/wix/Gauche-mingw-*.msi ..
 fi
 
-# 'zip' isn't included in MinGW.
-#VERSION=`cat VERSION`
-#(cd $distdir; zip -r Gauche-mingw-$VERSION.zip Gauche)
+# Build zip archive
+if [ "$ZIP_ARCHIVE" = "yes" ]; then
+    VERSION=$(cat VERSION)
+    ZIP_FILE="Gauche-mingw-${VERSION}.zip"
+
+    (cd "${distdir}/../" && rm -f "./${ZIP_FILE}" && 7z a "./${ZIP_FILE}" $(basename "${distdir}"))
+    cp "${distdir}/../${ZIP_FILE}" ..
+fi
