@@ -1,7 +1,7 @@
 /*
  * module.c - module implementation
  *
- *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -59,7 +59,7 @@
  *  namespace---for example, a 'sandbox' environment to evaluate an
  *  expression sent over the network during a session.
  *  The anonymous namespace will be garbage-collected if nobody references
- *  it, recovering its resouces.
+ *  it, recovering its resources.
  */
 
 /* Note on mutex of module operation
@@ -128,7 +128,7 @@ static struct {
 
 /* Predefined modules - slots will be initialized by Scm__InitModule */
 #define DEFINE_STATIC_MODULE(cname) \
-    static ScmModule cname = { { NULL } }
+    static ScmModule cname;
 
 DEFINE_STATIC_MODULE(nullModule);     /* #<module null> */
 DEFINE_STATIC_MODULE(schemeModule);   /* #<module scheme> */
@@ -162,6 +162,7 @@ static void init_module(ScmModule *m, ScmObj name, ScmHashTable *internal)
     m->external = SCM_HASH_TABLE(Scm_MakeHashTableSimple(SCM_HASH_EQ, 0));
     m->origin = m->prefix = SCM_FALSE;
     m->sealed = FALSE;
+    m->placeholding = FALSE;
 }
 
 /* Internal */
@@ -381,10 +382,12 @@ static ScmGloc *search_binding(ScmModule *module, ScmSymbol *symbol,
         }
         ScmObj v = Scm_HashTableRef(external_only?m->external:m->internal,
                                     SCM_OBJ(symbol), SCM_FALSE);
+
         if (SCM_GLOCP(v)) {
             if (SCM_GLOC_PHANTOM_BINDING_P(SCM_GLOC(v))) {
+                symbol = SCM_GLOC(v)->name; /* in case it's renamed on export */                ScmGloc *g = search_binding(m, symbol, FALSE, FALSE, TRUE);
+                if (g) return g;
                 external_only = FALSE; /* See above comment */
-                symbol = SCM_GLOC(v)->name; /* in case it's renamed on export */
             } else {
                 return SCM_GLOC(v);
             }
@@ -560,7 +563,7 @@ int Scm_AliasBinding(ScmModule *target, ScmSymbol *targetName,
 ScmObj Scm_ImportModule(ScmModule *module,
                         ScmObj imported,
                         ScmObj prefix,
-                        u_long flags) /* reserved for future use */
+                        u_long flags SCM_UNUSED) /* reserved for future use */
 {
     if (module->sealed) err_sealed(SCM_OBJ(imported), module);
 
@@ -586,7 +589,7 @@ ScmObj Scm_ImportModule(ScmModule *module,
     (void)SCM_INTERNAL_MUTEX_LOCK(modules.mutex);
     {
         ScmObj ms, prev = p;
-        SCM_SET_CDR(p, module->imported);
+        SCM_SET_CDR_UNCHECKED(p, module->imported);
         /* Remove duplicate module, if any.
            NB: We allow to import the same module multiple times if they are
            qualified by :only, :prefix, etc.  Theoretically we should check
@@ -599,7 +602,7 @@ ScmObj Scm_ImportModule(ScmModule *module,
                 prev = ms;
                 continue;
             }
-            SCM_SET_CDR(prev, SCM_CDR(ms));
+            SCM_SET_CDR_UNCHECKED(prev, SCM_CDR(ms));
             break;
         }
         module->imported = p;
@@ -805,6 +808,15 @@ ScmModule *Scm_FindModule(ScmSymbol *name, int flags)
         int created;
         ScmModule *m = lookup_module_create(name, &created);
         SCM_ASSERT(m != NULL);
+        /* If the module is ever called with CREATE and PLACEHOLDING flag,
+           turn placeholding flag on.  The flag is cleared if the module
+           is ever called with CREATE but without PLACEHOLDING. */
+        if (created && (flags & SCM_FIND_MODULE_PLACEHOLDING)) {
+            m->placeholding = TRUE;
+        }
+        if (!(flags & SCM_FIND_MODULE_PLACEHOLDING)) {
+            m->placeholding = FALSE;
+        }
         return m;
     } else {
         ScmModule *m = lookup_module(name);
@@ -828,7 +840,10 @@ ScmObj Scm_AllModules(void)
     (void)SCM_INTERNAL_MUTEX_LOCK(modules.mutex);
     Scm_HashIterInit(&iter, SCM_HASH_TABLE_CORE(modules.table));
     while ((e = Scm_HashIterNext(&iter)) != NULL) {
-        SCM_APPEND1(h, t, SCM_DICT_VALUE(e));
+        ScmModule *m = SCM_MODULE(SCM_DICT_VALUE(e));
+        if (!m->placeholding) {
+            SCM_APPEND1(h, t, SCM_DICT_VALUE(e));
+        }
     }
     (void)SCM_INTERNAL_MUTEX_UNLOCK(modules.mutex);
     return h;
