@@ -1,7 +1,7 @@
 /*
  * read.c - reader
  *
- *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -69,6 +69,7 @@ static ScmObj read_keyword(ScmPort *port, ScmReadContext *ctx);
 static ScmObj read_regexp(ScmPort *port);
 static ScmObj read_charset(ScmPort *port);
 static ScmObj read_sharp_comma(ScmPort *port, ScmReadContext *ctx);
+static ScmObj read_sharp_asterisk(ScmPort *port, ScmReadContext *ctx);
 static ScmObj process_sharp_comma(ScmPort *port, ScmObj key, ScmObj args,
                                   ScmReadContext *ctx, int has_ref);
 static ScmObj read_shebang(ScmPort *port, ScmReadContext *ctx);
@@ -79,16 +80,16 @@ static ScmObj read_sharp_word(ScmPort *port, char c, ScmReadContext *ctx);
 static struct {
     ScmHashTable *table;
     ScmInternalMutex mutex;
-} readCtorData = { NULL };
+} readCtorData;
 
 /* Table of hash-bang directive */
 static struct {
     ScmHashTable *table;
     ScmInternalMutex mutex;
-} hashBangData = { NULL };
+} hashBangData;
 
 /* Parameter location for default reader mode */
-ScmParameterLoc defaultReadContext;
+static ScmPrimitiveParameter *defaultReadContext;
 
 /*----------------------------------------------------------------
  * Entry points
@@ -185,14 +186,15 @@ ScmObj Scm_ReadList(ScmObj port, ScmChar closer)
 
 ScmReadContext *Scm_CurrentReadContext()
 {
-    ScmObj c = Scm_ParameterRef(Scm_VM(), &defaultReadContext);
+    ScmObj c = Scm_PrimitiveParameterRef(Scm_VM(), defaultReadContext);
     SCM_ASSERT(SCM_READ_CONTEXT_P(c));
     return SCM_READ_CONTEXT(c);
 }
 
 ScmReadContext *Scm_SetCurrentReadContext(ScmReadContext *ctx)
 {
-    ScmObj p = Scm_ParameterSet(Scm_VM(), &defaultReadContext, SCM_OBJ(ctx));
+    ScmObj p = Scm_PrimitiveParameterSet(Scm_VM(), defaultReadContext,
+                                         SCM_OBJ(ctx));
     SCM_ASSERT(SCM_READ_CONTEXT_P(p));
     return SCM_READ_CONTEXT(p);
 }
@@ -213,7 +215,7 @@ ScmReadContext *Scm_MakeReadContext(ScmReadContext *proto)
 }
 
 static void read_context_print(ScmObj obj, ScmPort *port,
-                               ScmWriteContext *ctx)
+                               ScmWriteContext *ctx SCM_UNUSED)
 {
     Scm_Printf(port, "#<read-context %p>", obj);
 }
@@ -234,7 +236,7 @@ void Scm_ReadError(ScmPort *port, const char *msg, ...)
 {
     ScmObj ostr = Scm_MakeOutputStringPort(TRUE);
     ScmObj name = Scm_PortName(port);
-    int line = Scm_PortLine(port);
+    ScmSize line = Scm_PortLine(port);
 
     Scm_Printf(SCM_PORT(ostr), "Read error at %S:",
                SCM_STRINGP(name)? name : SCM_OBJ(SCM_MAKE_STR("??")));
@@ -248,7 +250,7 @@ void Scm_ReadError(ScmPort *port, const char *msg, ...)
 
     ScmObj rerr = Scm_MakeReadError(Scm_GetOutputString(SCM_PORT(ostr), 0),
                                     port, line);
-    Scm_Raise(rerr);
+    Scm_Raise(rerr, 0);
 }
 
 /*----------------------------------------------------------------
@@ -270,8 +272,8 @@ ScmObj Scm_MakeReadReference(void)
     return SCM_OBJ(a);
 }
 
-static void read_reference_print(ScmObj obj, ScmPort *port,
-                                 ScmWriteContext *ctx)
+static void read_reference_print(ScmObj obj SCM_UNUSED, ScmPort *port,
+                                 ScmWriteContext *ctx SCM_UNUSED)
 {
     Scm_Printf(port, "#<read-reference>");
 }
@@ -317,11 +319,11 @@ static void read_context_flush(ScmReadContext *ctx)
         } else if (SCM_PAIRP(obj)) {
             SCM_FOR_EACH(ep, obj) {
                 if (SCM_READ_REFERENCE_P(SCM_CAR(ep))) {
-                    SCM_SET_CAR(ep, ref_val(SCM_CAR(ep)));
+                    SCM_SET_CAR_UNCHECKED(ep, ref_val(SCM_CAR(ep)));
                 }
                 if (SCM_READ_REFERENCE_P(SCM_CDR(ep))) {
                     /* in case we have (... . #N#) */
-                    SCM_SET_CDR(ep, ref_val(SCM_CDR(ep)));
+                    SCM_SET_CDR_UNCHECKED(ep, ref_val(SCM_CDR(ep)));
                     break;
                 }
             }
@@ -410,10 +412,11 @@ static int char_is_delimiter(ScmChar ch)
             || SCM_CHAR_EXTRA_WHITESPACE(ch));
 }
 
-static void read_nested_comment(ScmPort *port, ScmReadContext *ctx)
+static void read_nested_comment(ScmPort *port,
+                                ScmReadContext *ctx SCM_UNUSED)
 {
     int nesting = 0;
-    int line = Scm_PortLine(port);
+    ScmSize line = Scm_PortLine(port);
 
     for (ScmChar c = Scm_GetcUnsafe(port);;) {
         switch (c) {
@@ -452,16 +455,11 @@ static void read_comment(ScmPort *port) /* leading semicolon is already read */
         /* NB: comment may contain unexpected character code.
            for the safety, we read bytes here. */
         int c = Scm_GetbUnsafe(port);
-        if (c == '\n') {
-            /* oops.  ugly. */
-            port->line++;
-            break;
-        }
-        if (c == EOF) break;
+        if (c == '\n' || c == EOF) break;
     }
 }
 
-static int skipws(ScmPort *port, ScmReadContext *ctx)
+static int skipws(ScmPort *port, ScmReadContext *ctx SCM_UNUSED)
 {
     for (;;) {
         int c = Scm_GetcUnsafe(port);
@@ -478,7 +476,9 @@ static int skipws(ScmPort *port, ScmReadContext *ctx)
     }
 }
 
-static void reject_in_r7(ScmPort *port, ScmReadContext *ctx, const char *token)
+static void reject_in_r7(ScmPort *port,
+                         ScmReadContext *ctx SCM_UNUSED,
+                         const char *token)
 {
     if (SCM_EQ(Scm_GetPortReaderLexicalMode(port), SCM_SYM_STRICT_R7)) {
         Scm_ReadError(port,
@@ -501,10 +501,12 @@ static ScmObj read_internal(ScmPort *port, ScmReadContext *ctx)
             switch (c1) {
             case EOF:
                 Scm_ReadError(port, "premature #-sequence at EOF");
+                return SCM_UNDEFINED; /* dummy */
             case 't':; case 'T': return read_sharp_word(port, 't', ctx);
             case 'f':; case 'F': return read_sharp_word(port, 'f', ctx);
             case 's':; case 'S': return read_sharp_word(port, 's', ctx);
             case 'u':; case 'U': return read_sharp_word(port, 'u', ctx);
+            case 'c':; case 'C': return read_sharp_word(port, 'c', ctx);
             case '(':
                 return read_vector(port, ')', ctx);
             case '\\':
@@ -571,6 +573,7 @@ static ScmObj read_internal(ScmPort *port, ScmReadContext *ctx)
                     return SCM_EOF;
                 default:
                     Scm_ReadError(port, "unsupported #?-syntax: #?%C", c2);
+                    return SCM_UNDEFINED; /* dummy */
                 }
             }
             case '0': case '1': case '2': case '3': case '4':
@@ -579,11 +582,9 @@ static ScmObj read_internal(ScmPort *port, ScmReadContext *ctx)
                 return read_num_prefixed(port, c1, ctx);
             case '*': {
                 reject_in_r7(port, ctx, "#*");
-                /* #*"...." byte string
-                   #*01001001 for bit vector, maybe in future. */
-                int c2 = Scm_GetcUnsafe(port);
-                if (c2 == '"') return read_string(port, TRUE, ctx);
-                Scm_ReadError(port, "unsupported #*-syntax: #*%C", c2);
+                /* #**"...." byte string
+                   #*01001001 for bit vector. */
+                return read_sharp_asterisk(port, ctx);
             }
             case ':': {
                 reject_in_r7(port, ctx, "#:");
@@ -601,6 +602,7 @@ static ScmObj read_internal(ScmPort *port, ScmReadContext *ctx)
             }
             default:
                 Scm_ReadError(port, "unsupported #-syntax: #%C", c1);
+                return SCM_UNDEFINED; /* dummy */
             }
         }
     case '\'': return read_quoted(port, SCM_SYM_QUOTE, ctx);
@@ -617,6 +619,7 @@ static ScmObj read_internal(ScmPort *port, ScmReadContext *ctx)
             int c1 = Scm_GetcUnsafe(port);
             if (c1 == EOF) {
                 Scm_ReadError(port, "unterminated unquote");
+                return SCM_UNDEFINED; /* dummy */
             } else if (c1 == '@') {
                 return read_quoted(port, SCM_SYM_UNQUOTE_SPLICING, ctx);
             } else {
@@ -632,18 +635,7 @@ static ScmObj read_internal(ScmPort *port, ScmReadContext *ctx)
         return read_list(port, ']', ctx);
     case '{':
         reject_in_r7(port, ctx, "{}");
-        /* srfi-105 experimental support */
-        {
-            ScmObj r = read_list(port, '}', ctx);
-            if (SCM_VM_COMPILER_FLAG_IS_SET(Scm_VM(),SCM_COMPILE_ENABLE_CEXPR)){
-                static ScmObj xform_cexpr = SCM_UNDEFINED;
-                SCM_BIND_PROC(xform_cexpr, "%xform-cexpr",
-                              Scm_GaucheInternalModule());
-                return Scm_ApplyRec1(xform_cexpr, r);
-            } else {
-                return r;
-            }
-        }
+        return read_list(port, '}', ctx);
     case '+':; case '-':
         /* Note: R5RS doesn't permit identifiers beginning with '+' or '-',
            but some Scheme programs use such identifiers. */
@@ -664,6 +656,7 @@ static ScmObj read_internal(ScmPort *port, ScmReadContext *ctx)
         return read_symbol_or_number(port, c, ctx);
     case ')':; case ']':; case '}':;
         Scm_ReadError(port, "extra close parenthesis `%c'", c);
+        return SCM_UNDEFINED;   /* dummy */
     case EOF:
         return SCM_EOF;
     default:
@@ -712,7 +705,7 @@ static ScmObj read_item(ScmPort *port, ScmReadContext *ctx)
    be read.  The character is passed to KEY.  It also receives
    ScmReaderLexicalMode.
 
-   Character literal doesn't take terminating ';'.  TERMINATOR character
+   Character literal doesn't take terminating ';'.  TERMINATOR flag
    indicates whether we expect terminator or not.
 
    NEXTBUF is an output variable points to the next character on success.
@@ -801,7 +794,7 @@ ScmObj Scm_ReadXdigitsFromPort(ScmPort *port, int key, ScmObj mode,
         Scm_DStringPutc(&ds, ch);
     }
 
-    int numchars;
+    ScmSmallInt numchars;
     const char *chars = Scm_DStringPeek(&ds, &numchars, NULL);
     const char *next;
 
@@ -829,10 +822,10 @@ ScmObj Scm_ReadXdigitsFromPort(ScmPort *port, int key, ScmObj mode,
 /* NB: Call for refactoring, using Scm_ParseDigitsAsLong  */
 long Scm_ReadDigitsAsLong(ScmPort *port, ScmChar ch, int radix,
                           ScmChar *next, /*out*/
-                          int *numread /*out*/)
+                          ScmSize *numread /*out*/)
 {
-    u_long val = 0;
-    int nchars = 0;
+    long val = 0;
+    ScmSize nchars = 0;
     if (ch != SCM_CHAR_INVALID) {
         int v = Scm_DigitToInt(ch, radix, FALSE);
         if (v < 0) {
@@ -864,11 +857,11 @@ long Scm_ReadDigitsAsLong(ScmPort *port, ScmChar ch, int radix,
 
 /* Read long digits from BUF up to LEN.  BUF must only contain single-byte
    chars. */
-long Scm_ParseDigitsAsLong(const char *buf, size_t len, int radix,
-                           int *numread) /*out*/
+long Scm_ParseDigitsAsLong(const char *buf, ScmSize len, int radix,
+                           ScmSize *numread) /*out*/
 {
-    u_long val = 0;
-    int nchars = 0;
+    long val = 0;
+    ScmSize nchars = 0;
     for (; nchars < len; nchars++, buf++) {
         int v = Scm_DigitToInt((ScmChar)*buf, radix, FALSE);
         if (v < 0) {
@@ -923,7 +916,7 @@ static ScmObj read_list_int(ScmPort *port, ScmChar closer,
                 Scm_UngetcUnsafe(c2, port);
                 item = read_item(port, ctx);
                 if (SCM_READ_REFERENCE_P(item)) ref_seen = TRUE;
-                SCM_SET_CDR(last, item);
+                SCM_SET_CDR_UNCHECKED(last, item);
                 dot_seen = TRUE;
                 continue;
             }
@@ -953,7 +946,7 @@ static ScmObj read_list_int(ScmPort *port, ScmChar closer,
 static ScmObj read_list(ScmPort *port, ScmChar closer, ScmReadContext *ctx)
 {
     int has_ref;
-    int line = -1;
+    ScmSize line = -1;
 
     if (ctx->flags & RCTX_SOURCE_INFO) line = Scm_PortLine(port);
 
@@ -974,7 +967,7 @@ static ScmObj read_list(ScmPort *port, ScmChar closer, ScmReadContext *ctx)
 static ScmObj read_vector(ScmPort *port, ScmChar closer, ScmReadContext *ctx)
 {
     int has_ref;
-    int line = -1;
+    ScmSize line = -1;
     ScmObj r;
 
     if (ctx->flags & RCTX_SOURCE_INFO) line = Scm_PortLine(port);
@@ -986,7 +979,7 @@ static ScmObj read_vector(ScmPort *port, ScmChar closer, ScmReadContext *ctx)
 
 static ScmObj read_quoted(ScmPort *port, ScmObj quoter, ScmReadContext *ctx)
 {
-    int line = -1;
+    ScmSize line = -1;
     ScmObj r;
 
     if (ctx->flags & RCTX_SOURCE_INFO) line = Scm_PortLine(port);
@@ -1037,7 +1030,7 @@ static void read_string_xdigits(ScmPort *port, int key,
 }
 
 static ScmObj read_string(ScmPort *port, int incompletep,
-                          ScmReadContext *ctx)
+                          ScmReadContext *ctx SCM_UNUSED)
 {
     int c = 0;
     ScmDString ds;
@@ -1106,8 +1099,8 @@ static ScmObj read_string(ScmPort *port, int incompletep,
                     ACCUMULATE(c);
                     break;
                 }
-                /*FALLTHROUGH*/
             }
+                /*FALLTHROUGH*/
             cont_lf:
             case '\n': {
                 for (;;) {
@@ -1148,7 +1141,7 @@ static ScmObj read_string(ScmPort *port, int incompletep,
 
 static struct char_name {
     const char *name;
-    int size;
+    u_int size;
     ScmObj ch;
 } char_names[] = {
 #define DEFCHAR(name, char) \
@@ -1178,7 +1171,9 @@ static ScmObj read_char(ScmPort *port, ScmReadContext *ctx)
 
     int c = Scm_GetcUnsafe(port);
     switch (c) {
-    case EOF: Scm_ReadError(port, "EOF encountered in character literal");
+    case EOF:
+        Scm_ReadError(port, "EOF encountered in character literal");
+        break;
     case '(':; case ')':; case '[':; case ']':; case '{':; case '}':;
     case '"':; case ' ':; case '\\':; case '|':; case ';':;
     case '#':;
@@ -1196,7 +1191,7 @@ static ScmObj read_char(ScmPort *port, ScmReadContext *ctx)
             Scm_UngetcUnsafe(following, port);
         }
 
-        u_int namelen, namesize;
+        ScmSmallInt namelen, namesize;
         const char *cname = Scm_GetStringContent(name, &namesize, &namelen,
                                                  NULL);
         if (namelen == 1) {
@@ -1238,7 +1233,7 @@ static ScmObj read_char(ScmPort *port, ScmReadContext *ctx)
 
         struct char_name *cntab = char_names;
         while (cntab->name) {
-            if (cntab->size == namesize
+            if (cntab->size == (unsigned)namesize
                 && strncmp(cntab->name, cname, namesize) == 0) {
                 return cntab->ch;
             }
@@ -1261,7 +1256,8 @@ static ScmObj read_char(ScmPort *port, ScmReadContext *ctx)
    the read context setting.  INCLUDE_HASH_SIGN allows '#' to appear in
    the word.
 */
-static ScmObj read_word(ScmPort *port, ScmChar initial, ScmReadContext *ctx,
+static ScmObj read_word(ScmPort *port, ScmChar initial,
+                        ScmReadContext *ctx SCM_UNUSED,
                         int temp_case_fold, int include_hash_sign)
 {
     int case_fold = temp_case_fold || SCM_PORT_CASE_FOLDING(port);
@@ -1338,6 +1334,9 @@ static ScmObj read_number(ScmPort *port, ScmChar initial, int radix,
 {
     ScmString *s = SCM_STRING(read_word(port, initial, ctx, FALSE, TRUE));
     u_long flags = radix >=2 ? SCM_NUMBER_FORMAT_ALT_RADIX : 0;
+    if (SCM_EQ(Scm_GetPortReaderLexicalMode(port), SCM_SYM_STRICT_R7)) {
+        flags |= SCM_NUMBER_FORMAT_STRICT_R7RS;
+    }
     int default_radix = radix >= 2? radix : 10;
     ScmObj num = Scm_StringToNumber(s, default_radix, flags);
     if (num == SCM_FALSE) {
@@ -1354,7 +1353,11 @@ static ScmObj read_number(ScmPort *port, ScmChar initial, int radix,
 static ScmObj read_symbol_or_number(ScmPort *port, ScmChar initial, ScmReadContext *ctx)
 {
     ScmString *s = SCM_STRING(read_word(port, initial, ctx, FALSE, TRUE));
-    ScmObj num = Scm_StringToNumber(s, 10, 0);
+    u_long flags = 0;
+    if (SCM_EQ(Scm_GetPortReaderLexicalMode(port), SCM_SYM_STRICT_R7)) {
+        flags |= SCM_NUMBER_FORMAT_STRICT_R7RS;
+    }
+    ScmObj num = Scm_StringToNumber(s, 10, flags);
     if (num != SCM_FALSE) return num;
     check_valid_symbol(s);
     return Scm_Intern(s);
@@ -1375,7 +1378,7 @@ static ScmObj read_keyword(ScmPort *port, ScmReadContext *ctx)
 }
 
 static ScmObj read_escaped_symbol(ScmPort *port, ScmChar delim, int interned,
-                                  ScmReadContext *ctx)
+                                  ScmReadContext *ctx SCM_UNUSED)
 {
     ScmDString ds;
     Scm_DStringInit(&ds);
@@ -1480,7 +1483,7 @@ static ScmObj read_num_prefixed(ScmPort *port, ScmChar ch, ScmReadContext *ctx)
 {
     ScmObj e = SCM_UNBOUND;
     ScmChar ch2;
-    int nread;
+    ScmSize nread;
     long prefix = Scm_ReadDigitsAsLong(port, ch, 10, &ch2, &nread);
 
     if (ch2 == EOF) Scm_ReadError(port, "unterminated reference form (#digits)");
@@ -1526,8 +1529,13 @@ static ScmObj read_num_prefixed(ScmPort *port, ScmChar ch, ScmReadContext *ctx)
         }
     case 'r': case 'R':
         /* #digitR - radix */
-        if (prefix < 2 || prefix > 36) {
-            Scm_ReadError(port, "Radix prefix out of range: radix in #<radix>R must be between 2 and 36 inclusive, but got: %d", prefix);
+        if (SCM_EQ(Scm_GetPortReaderLexicalMode(port), SCM_SYM_STRICT_R7)) {
+            Scm_ReadError(port, "Radix prefix isn't allowed in strict R7RS mode.");
+        }
+        if (prefix < SCM_RADIX_MIN || prefix > SCM_RADIX_MAX) {
+            Scm_ReadError(port, "Radix prefix out of range: radix in #<radix>R must be between %d and %d inclusive, but got: %d",
+                          SCM_RADIX_MIN, SCM_RADIX_MAX, prefix);
+            return SCM_UNDEFINED; /* dummy */
         } else {
             int ch = Scm_GetcUnsafe(port);
             if (ch == EOF) {
@@ -1551,7 +1559,7 @@ static ScmObj read_num_prefixed(ScmPort *port, ScmChar ch, ScmReadContext *ctx)
    For now, it is not used and the caller should pass SCM_FALSE. */
 
 ScmObj Scm_DefineReaderCtor(ScmObj symbol, ScmObj proc, ScmObj finisher,
-                            ScmObj module /*reserved*/)
+                            ScmObj module SCM_UNUSED /*reserved*/)
 {
     if (!SCM_PROCEDUREP(proc)) {
         Scm_Error("procedure required, but got %S", proc);
@@ -1563,7 +1571,7 @@ ScmObj Scm_DefineReaderCtor(ScmObj symbol, ScmObj proc, ScmObj finisher,
     return SCM_UNDEFINED;
 }
 
-ScmObj Scm_GetReaderCtor(ScmObj symbol, ScmObj module /*reserved*/)
+ScmObj Scm_GetReaderCtor(ScmObj symbol, ScmObj module SCM_UNUSED/*reserved*/)
 {
     (void)SCM_INTERNAL_MUTEX_LOCK(readCtorData.mutex);
     ScmObj r = Scm_HashTableRef(readCtorData.table, symbol, SCM_FALSE);
@@ -1579,7 +1587,8 @@ static ScmObj read_sharp_comma(ScmPort *port, ScmReadContext *ctx)
                       next);
     }
 
-    int has_ref, line = -1;
+    int has_ref;
+    ScmSize line = -1;
     if (ctx->flags & RCTX_SOURCE_INFO) line = Scm_PortLine(port);
 
     ScmObj form = read_list_int(port, ')', ctx, &has_ref, line);
@@ -1607,7 +1616,7 @@ static ScmObj process_sharp_comma(ScmPort *port, ScmObj key, ScmObj args,
     return r;
 }
 
-static ScmObj reader_ctor(ScmObj *args, int nargs, void *data)
+static ScmObj reader_ctor(ScmObj *args, int nargs, void *data SCM_UNUSED)
 {
     ScmObj optarg = (nargs > 2? args[2] : SCM_FALSE);
     return Scm_DefineReaderCtor(args[0], args[1], optarg, SCM_FALSE);
@@ -1627,11 +1636,11 @@ ScmObj Scm_DefineReaderDirective(ScmObj symbol, ScmObj proc)
 
 static ScmObj read_shebang(ScmPort *port, ScmReadContext *ctx)
 {
-    /* If '#!' appears in the beginning of the input port, and it is
-       followed by '/' or a space, then we consider it as the
-       beginning of shebang and discards entire line.  Otherwise, we
-       take this as #!<identifier> directive as specified in R6RS, and
-       calls appropriate handler.
+    /* If '#!' appears at the beginning of the input port, and it is
+       followed by '/' or a space or a newline, then we consider it as
+       the beginning of shebang and discard the entire line.
+       Otherwise, we take this as #!<identifier> directive as
+       specified in R6RS, and calls appropriate handler.
 
        R6RS is actually not very clean at this corner.  It requires
        distinct modes to parse a script (which always begins with
@@ -1640,12 +1649,12 @@ static ScmObj read_shebang(ScmPort *port, ScmReadContext *ctx)
        parser that strictly covers both situation.
     */
     int c2 = Scm_GetcUnsafe(port);
-    if (port->bytes == 3 && (c2 == '/' || c2 == ' ')) {
+    if (Scm_PortBytes(port) == 3 && (c2 == '/' || c2 == ' ' || c2 == '\n')) {
         /* shebang */
         for (;;) {
-            c2 = Scm_GetcUnsafe(port);
             if (c2 == '\n') return SCM_UNDEFINED;
             if (c2 == EOF) return SCM_EOF;
+            c2 = Scm_GetcUnsafe(port);
         }
         /*NOTREACHED*/
     } else {
@@ -1729,6 +1738,13 @@ static ScmObj read_sharp_word_1(ScmPort *port, char ch, ScmReadContext *ctx)
     const char *tag = NULL;
 
     switch (ch) {
+    case 'c':
+        if (strcmp(w, "c32") == 0
+            || strcmp(w, "c64") == 0
+            || strcmp(w, "c128") == 0) {
+            tag = w;
+        }
+        break;
     case 'f':
         if (strcmp(w, "f16") == 0
             || strcmp(w, "f32") == 0
@@ -1774,10 +1790,87 @@ static ScmObj read_sharp_word(ScmPort *port, char ch, ScmReadContext *ctx)
     }
 }
 
+/*----------------------------------------------------------------
+ * #**"....", #*1010010...
+ */
+static ScmObj read_sharp_asterisk(ScmPort *port, ScmReadContext *ctx)
+{
+    /* We used to use #*"..." for incomplete string literals, but it turned
+       out it can be read as an zero-length bitvector #* followed by an
+       ordinary string (since a double quote is a <delimiter> in R7RS).
+       
+       So we changed the literal to #**"..." (yeah, ugly, but incomplete
+       strings are anomalies and shouldn't be used often). 
+
+       The reader behavior can be customized by reader-lexical-mode:
+
+         permissive (default) - For the next release, #*"..." is read as
+             an incomplete string, without warning.  In future, it warns.
+         legacy - #*"..." is read as an incomplete string, without warning.
+         warn-legacy - #*"..." is read as an incomplete string, but warns.
+         strict-r7 - #*"..." is read as a zero-length bitvector followed by
+                     a string.  #**"..." is rejected.
+    */
+
+    int c = Scm_GetcUnsafe(port);
+    if (c == '*') {
+        c = Scm_GetcUnsafe(port);
+        if (c == '"') return read_string(port, TRUE, ctx);
+        Scm_ReadError(port, "Invalid #* syntax: #**%C", c);
+    }   
+    if (c == '"') {
+        ScmObj m = Scm_GetPortReaderLexicalMode(port);
+        if (SCM_EQ(m, SCM_SYM_PERMISSIVE)) {
+            /* TRANSIENT: We switch this default behavior to warn in future */
+            return read_string(port, TRUE, ctx);
+        }
+        if (SCM_EQ(m, SCM_SYM_LEGACY)) {
+            return read_string(port, TRUE, ctx);
+        }
+        if (SCM_EQ(m, SCM_SYM_WARN_LEGACY)) {
+            Scm_Warn("Deprecated incomplete string syntax #*\"...\" at %A:%ld",
+                     Scm_PortName(port), Scm_PortLine(port));
+            return read_string(port, TRUE, ctx);
+        }
+        if (SCM_EQ(m, SCM_SYM_STRICT_R7)) {
+            Scm_UngetcUnsafe(c, port);
+            return Scm_MakeBitvector(0, SCM_FALSE);
+        }
+        Scm_Panic("invalid reader-lexical-mode");
+    }
+    
+    ScmDString ds;
+    Scm_DStringInit(&ds);
+    for (;;) {
+        switch (c) {
+        case EOF:
+            goto finish;
+        case '(': case ')': case '"': case '|': case ';':
+            Scm_UngetcUnsafe(c, port);
+            goto finish;
+        case '0': case '1':
+            Scm_DStringPutc(&ds, c);
+            break;
+        default:
+            if ((SCM_CHAR_ASCII_P(c) && isspace(c))
+                || (!SCM_CHAR_ASCII_P(c) && SCM_CHAR_EXTRA_WHITESPACE(c))) {
+                Scm_UngetcUnsafe(c, port);
+                goto finish;
+            }
+            Scm_ReadError(port, "Invalid char in bitvector literal: %C", c);
+        }
+        c = Scm_GetcUnsafe(port);
+    }
+
+ finish:;
+    ScmObj s = Scm_DStringGet(&ds, 0);
+    return Scm_StringToBitvector(SCM_STRING(s), FALSE);
+}
+
 /* OBSOLETED: gauche.uvector used to call this to set up reader pointer.
    Now it is read in src/vector.c.   We keep this entry for ABI compatibility.
    Remove on 1.0 release. */
-void Scm__InstallReadUvectorHook(ScmObj (*f)(ScmPort*, const char*, ScmReadContext *))
+void Scm__InstallReadUvectorHook(ScmObj (*f)(ScmPort*, const char*, ScmReadContext *) SCM_UNUSED)
 {
 }
 
@@ -1800,6 +1893,8 @@ void Scm__InitRead(void)
         SCM_HASH_TABLE(Scm_MakeHashTableSimple(SCM_HASH_EQ, 0));
     (void)SCM_INTERNAL_MUTEX_INIT(hashBangData.mutex);
 
-    Scm_InitParameterLoc(Scm_VM(), &defaultReadContext,
-                         SCM_OBJ(make_read_context(NULL)));
+    defaultReadContext =
+        Scm_MakePrimitiveParameter(SCM_CLASS_PRIMITIVE_PARAMETER, SCM_FALSE,
+                                   SCM_OBJ(make_read_context(NULL)),
+                                   0);
 }
