@@ -1,7 +1,7 @@
 /*
  * signal.c - signal handling
  *
- *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -89,7 +89,7 @@ static struct sigHandlersRec {
                                    for the system to handle.  These can be
                                    overridden by Scm_SetSignalHandler. */
     ScmInternalMutex mutex;
-} sigHandlers = {{NULL}};
+} sigHandlers;
 
 /* Maximum # of the same signals before it is processed by the VM loop.
    If any one of signals exceeds this count, Gauche exits with Scm_Abort.
@@ -242,7 +242,7 @@ static struct sigdesc {
 #ifdef SIGTHR
     SIGDEF(SIGTHR,  SIGDEF_NOHANDLE), /* Thread interrupt / AST (FreeBSD, OpenBSD) */
 #endif
-    { NULL, -1 }
+    { NULL, -1, 0 }
 };
 
 /*===============================================================
@@ -307,14 +307,15 @@ static ScmObj sigset_allocate(ScmClass *klass, ScmObj initargs);
 SCM_DEFINE_BUILTIN_CLASS(Scm_SysSigsetClass, sigset_print,
                          NULL, NULL, sigset_allocate, SCM_CLASS_DEFAULT_CPL);
 
-void sigset_print(ScmObj obj, ScmPort *out, ScmWriteContext *ctx)
+void sigset_print(ScmObj obj, ScmPort *out, 
+                  ScmWriteContext *ctx SCM_UNUSED)
 {
     Scm_Printf(out, "#<sys-sigset [");
     display_sigset(&SCM_SYS_SIGSET(obj)->set, out);
     Scm_Printf(out, "]>");
 }
 
-ScmObj sigset_allocate(ScmClass *klass, ScmObj initargs)
+ScmObj sigset_allocate(ScmClass *klass, ScmObj initargs SCM_UNUSED)
 {
     ScmSysSigset *s = SCM_NEW_INSTANCE(ScmSysSigset, klass);
     sigemptyset(&s->set);
@@ -375,9 +376,6 @@ void Scm_SigFillSetMostly(sigset_t *set) /* out */
 #ifdef SIGKILL
     sigdelset(set, SIGKILL);
 #endif
-#ifdef SIGCONT
-    sigdelset(set, SIGCONT);
-#endif
 #ifdef SIGSTOP
     sigdelset(set, SIGSTOP);
 #endif
@@ -396,6 +394,11 @@ void Scm_SigFillSetMostly(sigset_t *set) /* out */
     int sig_restart = GC_get_thr_restart_signal();
     if (sig_restart >= 0) sigdelset(set, sig_restart);
 #endif /*GC_THREADS*/
+
+    /* Exclude SIGPROF when we're using gperftools profiler. */
+#if defined(HAVE_GPERFTOOLS_PROFILER_H)
+    sigdelset(set, SIGPROF);
+#endif /*WITHGPERFTOOLS*/
 }
 
 /*=============================================================
@@ -440,9 +443,7 @@ void Scm_SignalQueueClear(ScmSignalQueue* q)
  */
 void Scm_SignalQueueInit(ScmSignalQueue* q)
 {
-#if defined(GAUCHE_API_0_9)
     q->sigcounts = SCM_NEW_ARRAY(unsigned char, SCM_NSIG);
-#endif /* GAUCHE_API_0_9 */
     Scm_SignalQueueClear(q);
     q->pending = SCM_NIL;
 }
@@ -492,9 +493,7 @@ void Scm_SigCheck(ScmVM *vm)
     }
 #endif  /* defined(GAUCHE_USE_PTHREADS) && defined(GAUCHE_PTHREAD_SIGNAL) */
 
-    /* Now, prepare queued signal handlers
-       If an error is thrown in this loop, the queued signals will be
-       lost---it doesn't look like so, but I may overlook something. */
+    /* Now, queue the signal handlers */
     ScmObj tail = q->pending;
     if (!SCM_NULLP(tail)) tail = Scm_LastPair(tail);
     for (int i=0; i<SCM_NSIG; i++) {
@@ -507,7 +506,7 @@ void Scm_SigCheck(ScmVM *vm)
             if (SCM_NULLP(tail)) {
                 q->pending = tail = cell;
             } else {
-                SCM_SET_CDR(tail, cell);
+                SCM_SET_CDR_UNCHECKED(tail, cell);
                 tail = SCM_CDR(tail);
             }
         }
@@ -526,6 +525,7 @@ void Scm_SigCheck(ScmVM *vm)
         ScmObj mask = SCM_CAR(SCM_CDDR(e));
         if (SCM_SYS_SIGSET_P(mask)) {
             sigset_t omask;
+            sigemptyset(&omask);
             SCM_UNWIND_PROTECT {
                 SIGPROCMASK(SIG_BLOCK, &SCM_SYS_SIGSET(mask)->set, &omask);
                 Scm_ApplyRec(handler, SCM_LIST1(num));
@@ -550,7 +550,7 @@ void Scm_SigCheck(ScmVM *vm)
  * Default Scheme-level handlers
  */
 /* For most signals, default handler raises an error. */
-static ScmObj default_sighandler(ScmObj *args, int nargs, void *data)
+static ScmObj default_sighandler(ScmObj *args, int nargs, void *data SCM_UNUSED)
 {
     SCM_ASSERT(nargs == 1 && SCM_INTP(args[0]));
     int signum = SCM_INT_VALUE(args[0]);
@@ -587,7 +587,9 @@ static SCM_DEFINE_SUBR(default_sighandler_stub, 1, 0,
 #define DEFAULT_SIGHANDLER    SCM_OBJ(&default_sighandler_stub)
 
 /* For some signals, exits. */
-static ScmObj exit_sighandler(ScmObj *args, int nargs, void *data)
+static ScmObj exit_sighandler(ScmObj *args SCM_UNUSED,
+                              int nargs SCM_UNUSED,
+                              void *data SCM_UNUSED)
 {
     Scm_Exit(0);
     return SCM_UNDEFINED;       /* dummy */
@@ -603,7 +605,9 @@ static SCM_DEFINE_SUBR(exit_sighandler_stub, 1, 0,
 #define EXIT_SIGHANDLER    SCM_OBJ(&exit_sighandler_stub)
 
 /* For some signals, gauche does nothing */
-static ScmObj indifferent_sighandler(ScmObj *args, int nargs, void *data)
+static ScmObj indifferent_sighandler(ScmObj *args SCM_UNUSED,
+                                     int nargs SCM_UNUSED,
+                                     void *data SCM_UNUSED)
 {
     return SCM_UNDEFINED;
 }
@@ -643,7 +647,8 @@ int sigaction(int signum, const struct sigaction *act,
     }
 }
 
-int sigprocmask_win(int how, const sigset_t *set, sigset_t *oldset)
+int sigprocmask_win(int how SCM_UNUSED, const sigset_t *set SCM_UNUSED,
+                    sigset_t *oldset SCM_UNUSED)
 {
     return 0;
 }
@@ -657,12 +662,12 @@ ScmObj Scm_SetSignalHandler(ScmObj sigs, ScmObj handler, ScmSysSigset *mask)
     sigset_t sigset;
     int badproc = FALSE, sigactionfailed = FALSE;
 
+    sigemptyset(&sigset);
     if (SCM_INTP(sigs)) {
         int signum = SCM_INT_VALUE(sigs);
         if (signum < 0 || signum >= SCM_NSIG) {
             Scm_Error("bad signal number: %d", signum);
         }
-        sigemptyset(&sigset);
         sigaddset(&sigset, signum);
     } else if (SCM_SYS_SIGSET_P(sigs)) {
         sigset = SCM_SYS_SIGSET(sigs)->set;
@@ -685,7 +690,7 @@ ScmObj Scm_SetSignalHandler(ScmObj sigs, ScmObj handler, ScmSysSigset *mask)
     }
 
     if (mask == NULL) {
-        /* If no mask is specified, block singals in SIGS. */
+        /* If no mask is specified, block signals in SIGS. */
         mask = make_sigset();
         mask->set = sigset;
     }
@@ -844,6 +849,7 @@ void Scm_SetMasterSigmask(sigset_t *set)
  * aren't used anywhere.  Scm_SysSigmask covers those functionalities,
  * so we'll drop them by 1.0.
  */
+#if GAUCHE_API_VERSION < 1000
 void Scm_GetSigmask(sigset_t *mask)
 {
     if (SIGPROCMASK(SIG_SETMASK, NULL, mask) != 0) {
@@ -857,6 +863,7 @@ void Scm_SetSigmask(sigset_t *mask)
         Scm_SysError("sigprocmask failed");
     }
 }
+#endif /*GAUCHE_API_VERSION < 1000*/
 
 /*
  * set signal mask
@@ -922,6 +929,7 @@ static void scm_sigsuspend(sigset_t *mask)
     SIGPROCMASK(SIG_SETMASK, &omask, NULL);
     SCM_SIGCHECK(vm);
 #else  /* GAUCHE_WINDOWS */
+    (void)mask; /* suppress unused var warning */
     Scm_Error("sigsuspend not supported on Windows port");
 #endif /* GAUCHE_WINDOWS */
 }
@@ -1030,6 +1038,7 @@ int Scm_SigWait(ScmSysSigset *mask)
     }
     return sig;
 #else  /* !HAVE_SIGWAIT */
+    (void)mask; /* suppress unused var warning */
     Scm_Error("sigwait not supported on this platform");
     return 0;
 #endif
