@@ -22,7 +22,9 @@
 
 ;; returns ((num module exports doc-en doc-ja) ...)
 (define (parse self)
-  (let1 gen ($ record-generator $ file->string-list self :encoding 'utf-8)
+  (let1 gen ($ record-generator 
+               $ remove #/^\;/
+               $ file->string-list self :encoding 'utf-8)
     ($ generator->list
        $ gmap ($ paragraphs->final-record $ split-by-empty-line $) gen)))
 
@@ -54,7 +56,7 @@
        ,(read-from-string (cadr entry-line))   ; list of exported symbols
        ,doc-en ,doc-ja)]
     [else
-     (error "Invalid entry for ~a" (car ps))]))
+     (error "Invalid entry for " (car ps))]))
 
 ;; "srfi-N, module" -> (N "module")
 ;; "srfi-N"  -> (N #f)
@@ -135,7 +137,7 @@
 ;; follows.
 
 srfi-0
-()
+(cond-expand)
 
 Feature-based conditional expansion construct.
 This has become a part of R7RS small.  Gauche supports this as
@@ -278,14 +280,14 @@ srfi-14, srfi-14
 Character-set library
 This has become a part of R7RS large.
 Character-set object and a few procedures are built-in.
-@xref{Character set}.
+@xref{Character sets}.
 Complete set of SRFI-14 is supported by the module @code{scheme.charset}.
 @xref{R7RS character sets}.
 
 文字集合のライブラリ
 これはR7RS largeの一部になりました。
 文字集合と基本的な手続きは組み込みになっています。
-@ref{Character set}参照。
+@ref{Character sets}参照。
 SRFI-14の完全なサポートはモジュール@code{scheme.charset}
 で提供されています。@ref{R7RS character sets}参照。
 
@@ -295,11 +297,11 @@ srfi-16
 
 Syntax for procedures of variable arity (case-lambda)
 This has become a part of R7RS small.
-Built-in.  @xref{Making Procedures}.
+Built-in.  @xref{Making procedures}.
 
 可変長引数手続き構文 (case-lambda)
 これはR7RS smallの一部になりました。
-組み込みです。@ref{Making Procedures}参照。
+組み込みです。@ref{Making procedures}参照。
 
 
 srfi-17
@@ -372,14 +374,14 @@ superset of SRFI-25.  @xref{Arrays}.
 なっています。@ref{Arrays}参照。
 
 
-srfi-26, srfi-26
-()
+srfi-26
+(cut cute)
 
 Notation for specializing parameters without currying.
-As an autoloaded macro.  @xref{Making Procedures}.
+Built-in.  @xref{Making procedures}.
 
-カリー化をともなわないパラメータの特殊化記法
-オートロードされるマクロとして定義されています。@ref{Making Procedures}参照。
+カリー化をともなわないパラメータの特殊化記法。組み込みです。
+@ref{Making procedures}参照。
 
 
 srfi-27, srfi-27
@@ -428,14 +430,14 @@ Supported by the native reader.  @xref{Lexical structure}.
 ネイティブのリーダでサポートされています。@ref{Lexical structure}参照。
 
 
-srfi-31, srfi-31
-()
+srfi-31
+(rec)
 
-A special form rec for recursive evaluation
-Defined as an autoloaded macro.  @xref{Binding constructs}.
+A special form rec for recursive evaluation.  Built-in.
+@xref{Binding constructs}.
 
-再帰評価用の特殊フォーム rec
-オートロードされるマクロとして定義されています。@ref{Binding constructs}参照。
+再帰評価用の特殊フォーム@code{rec}. 組み込みです。
+@ref{Binding constructs}参照。
 
 
 srfi-34
@@ -501,13 +503,11 @@ srfi-39, gauche.parameter
 
 Parameter objects
 This has become a part of R7RS small.
-Supported by the module @code{gauche.parameter}.
-@xref{Parameters}.
+Built-in (@pxref{Parameters}).
 
 パラメータオブジェクト
 これはR7RS smallの一部になりました。
-モジュール@code{gauche.parameter}でサポートされます。
-@ref{Parameters}参照。
+組み込みです。@ref{Parameters}参照。
 
 
 srfi-40, util.stream
@@ -522,6 +522,20 @@ Supported by the module @code{util.stream}.
 @xref{Stream library}.
 
 
+srfi-41, srfi-41
+()
+
+Streams
+This has become a part of R7RS large.  @xref{R7RS stream}.
+Most of stream procedures are also in @code{util.stream}
+(@pxref{Stream library}).
+
+ストリーム
+これはR7RS largeの一部になりました。@ref{R7RS stream}参照。
+ストリーム手続きのほぼ全ては@code{util.stream}からも提供されています
+(@ref{Stream library}参照)。
+
+
 srfi-42, srfi-42
 ()
 
@@ -593,11 +607,11 @@ srfi-60, srfi-60
 ()
 
 Integers as bits
-Most procedures are built-in: @xref{Bitwise operations}.
+Most procedures are built-in: @xref{R7RS bitwise operations}.
 The complete support is in @code{srfi-60} module: @xref{Integers as bits}.
 
 整数に対するビット操作
-ほとんどの手続きは組み込みになっています。@ref{Bitwise operations}参照。
+ほとんどの手続きは組み込みになっています。@ref{R7RS bitwise operations}参照。
 完全なサポートは@code{srfi-60}モジュールで提供されます。
 @ref{Integers as bits}参照。
 
@@ -628,10 +642,10 @@ srfi-64, srfi-64
 ()
 
 A Scheme API for test suites
-Supported by the module @code{srfi-64}.
+Supported by the module @code{srfi-64}. @xref{A Scheme API for test suites}.
 
 テストスイートのためのScheme API
-モジュール@code{srfi-64}でサポートされます。
+モジュール@code{srfi-64}でサポートされます。@ref{A Scheme API for test suites}参照。
 
 
 srfi-66, srfi-66
@@ -680,14 +694,11 @@ srfi-78, srfi-78
 
 Lightweight testing
 Supported by the module @code{srfi-78}.
-Since Gauche already has its own test framework (@pxref{Unit testing}),
-this is mainly for third-party modules that adopt srfi-78 for testing.
+It can work with @code{gauche.test}.  @xref{Lightweight testing}.
 
 軽いテストフレームワーク
 モジュール@code{srfi-78}でサポートされます。
-Gaucheは既に独自のテストフレームワークを持っているので(@ref{Unit testing}参照)、
-このモジュールは主としてサードパーティ製のモジュールがsrfi-78をテストに
-使っている場合をサポートするためのものです。
+@code{gauche.test}と一緒に使うこともできます。@ref{Lightweight testing}参照。
 
 
 srfi-87
@@ -746,6 +757,23 @@ Supported by the module @code{gauche.record}.  @xref{Record types}.
 ERR5RS レコード
 モジュール@code{gauche.record}でサポートされます。@ref{Record types}参照。
 
+
+;; We don't load srfi-101 on (cond-expand ((srfi-101) ...)) for now, since
+;; it would shadow many list primitives and may create wreck havoc.  Especialy
+;; it would make SLIB unusable, for it uses cond-expand to check library
+;; availability.
+srfi-101
+()
+
+Purely functional random-access pairs and lists
+This has become a part of R7RS large.
+Supported by the module @code{scheme.rlist}.
+
+純粋に関数的でランダムアクセス可能なペアとリスト
+これはR7RS largeの一部となりました。
+モジュール@code{scheme.rlist}でサポートされます。
+
+
 srfi-106, srfi-106
 ()
 
@@ -756,16 +784,16 @@ Supported by the module @code{srfi-106}.  @xref{Basic socket interface}.
 モジュール@code{srfi-106}でサポートされます。@ref{Basic socket interface}参照。
 
 
-srfi-111, srfi-111
+srfi-111
 (box box? unbox set-box!)
 
 Boxes
-This has become a part of R7RS large.
-Supported by the module @code{scheme.box}.  @xref{R7RS boxes}.
+This has become a part of R7RS large as @code{scheme.box}.
+Gauche has it as built-in.  @xref{Boxes}.
 
 ボックス
-これはR7RS largeの一部となりました。
-モジュール@code{scheme.box}でサポートされます。@ref{R7RS boxes}参照。
+これは@code{scheme.box}としてR7RS largeの一部となりました。
+Gaucheは組み込みでサポートしています。@ref{Boxes}参照。
 
 
 srfi-112, srfi-112
@@ -803,6 +831,33 @@ Some of the features are built-in (@pxref{Basic comparators}).  Full srfi spec i
 いくつかの機能は組み込みです(@ref{Basic comparators}参照)。srfiの完全な仕様はモジュール@code{srfi-114}でサポートされます(@ref{Comparators}参照)。
 
 
+srfi-115, srfi-115
+()
+
+Scheme Regular Expressions
+This has become a part of R7RS large.
+Supported by the module @code{scheme.regex}.  @xref{R7RS regular expressions}.
+
+Scheme正規表現
+これはR7RS largeの一部となりました。
+モジュール@code{scheme.regex}でサポートされます。@ref{R7RS regular expressions}参照。
+
+
+srfi-116, srfi-116
+()
+
+Immutable List Library
+This has become a part of R7RS large.
+Immutable pairs are supported natively (@pxref{Mutable and immutable pairs}).
+Full set of APIs are available
+in the module @code{scheme.ilist} (@pxref{R7RS immutable lists}).
+
+変更不可リストのライブラリ
+これはR7RS largeの一部となりました。
+変更不可なペアはGauche本体で直接サポートされます(@ref{Mutable and immutable pairs}参照)。
+フルセットのAPIは モジュール@code{scheme.ilist}でサポートされます(@ref{R7RS immutable lists}参照)。
+
+
 srfi-117, srfi-117
 ()
 
@@ -843,6 +898,22 @@ Gaucheの@code{gauche.generator}がsrfi-121のスーパーセットになって
 (@ref{Generators}参照)
 
 
+srfi-124, srfi-124
+()
+
+Ephemerons
+This has become a part of R7RS large.
+Supported by @code{scheme.ephemeron}.
+Note: Current Gauche's implementation isn't optimal.
+@xref{R7RS ephemerons}.
+
+Ephemeron
+これはR7RS largeの一部となりました。
+@code{scheme.ephemeron}モジュールでサポートされます
+(註: 現在のGaucheでの実装は理想的ではありません。)
+@ref{R7RS ephemerons}参照。
+
+
 srfi-125, srfi-125
 ()
 
@@ -903,6 +974,24 @@ Titlecase手続き
 この3つの手続きがインポートされます。
 
 
+srfi-130, srfi-130
+()
+
+Cursor-based string library
+String cursors are supported natively (@pxref{String cursors}).
+Most of built-in and srfi-13 string procedures accept cursors in addition
+to indexes.  The module srfi-130 provides several procedures that
+has the same name as srfi-13 but returns a string cursor instead
+of an index (@pxref{Cursor-based string library}).
+
+カーソルベースの文字列ライブラリ
+文字列カーソルは組み込みでサポートされています (@ref{String cursors}参照)。
+また、多くの組み込みおよびsrfi-13手続きはインデックスに加えてカーソルも
+取れるようになっています。モジュールsrfi-130は、srfi-13と同名だけれど
+インデックスの代わりにカーソルを返す手続きを提供します
+(@ref{Cursor-based string library}参照)。
+
+
 srfi-131, gauche.record
 ()
 
@@ -955,25 +1044,59 @@ The module @code{data.ideque} is compatible to srfi-134.
 @ref{変更不可な両端キュー}参照。
 
 
+srfi-135, srfi-135
+()
+
+Immutable Texts
+This has become a part of R7RS large.
+In Gauche, the text type is not disjoint from the string type.  Instead,
+a text is simply an immutable and indexed string.
+@xref{String indexing}, for the detail of indexed string.
+The API is described in @ref{R7RS immutable texts}.
+
+変更不可なテキスト
+これはR7RS largeの一部となりました。
+Gaucheではtext型はstring型と独立してはおらず、
+textは単に「変更不可でインデクスされた文字列」にすぎません。
+文字列のインデクシングについては@ref{String indexing}を参照してください。
+APIは@ref{R7RS immutable texts}で説明しています。
+
+
 srfi-141, srfi-141
 ()
 
 
 Integer division
-Supported by the module @code{srfi-141}.  @xref{Integer division}.
+This has become a part of R7RS large.
+Supported by the module @code{scheme.division}.  @xref{R7RS integer division}.
 
 整数除算
-モジュール@code{srfi-141}でサポートされます。@ref{Integer division}参照。
+これはR7RS largeの一部となりました。
+モジュール@code{scheme.division}でサポートされます。@ref{R7RS integer division}参照。
 
 
 srfi-143, srfi-143
 ()
 
 Finxums
-Supported by the module @code{srfi-143}.  @xref{Fixnums}.
+This has become a part of R7RS large.
+Supported by the module @code{scheme.fixnum}.  @xref{R7RS fixnum}.
 
-Fixnumx
-モジュール@code{srfi-143}でサポートされます。@ref{Fixnums}参照。
+Fixnums
+これはR7RS largeの一部となりました。
+モジュール@code{scheme.fixnum}でサポートされます。@ref{R7RS fixnum}参照。
+
+
+srfi-144, srfi-144
+()
+
+Flonums
+This has become a part of R7RS large.
+Supported by the module @code{scheme.flonum}.  @xref{R7RS flonum}.
+
+Flonums
+これはR7RS largeの一部となりました。
+モジュール@code{scheme.flonum}でサポートされます。@ref{R7RS flonum}参照。
 
 
 srfi-145
@@ -986,14 +1109,18 @@ Built-in.  @xref{Conditionals}.
 組み込みです。@ref{Conditionals}参照。
 
 
-srfi-146
+srfi-146, srfi-146
 ()
 
 Mappings
-Supported by the module @code{srfi-146}.  @xref{Mappings and hashmaps}.
+This has become a part of R7RS large.
+Supported by the module @code{scheme.mapping}.
+@xref{R7RS mappings}.
 
 マッピング
-モジュール@code{srfi-146}でサポートされます。 @ref{Mappings and hashmaps}参照。
+これはR7RS largeの一部となりました。
+モジュール@code{scheme.mapping}でサポートされます。
+@ref{R7RS mappings}参照。
 
 
 srfi-149
@@ -1010,12 +1137,12 @@ srfi-151, srfi-151
 ()
 
 Bitwise operations
-Supported by the module @code{srfi-151} (@pxref{Bitwise operations}).
+Supported by the module @code{srfi-151} (@pxref{R7RS bitwise operations}).
 Note that many equivalent procedures are provided built-in
 (@pxref{Basic bitwise operations}).
 
 ビット演算
-モジュール@code{srfi-151}でサポートされます (@ref{Bitwise operations}参照)。
+モジュール@code{srfi-151}でサポートされます (@ref{R7RS bitwise operations}参照)。
 なお、組み込みでも多くの手続きについては同等の手続きが提供されています
 (@ref{Basic bitwise operations}参照).
 
@@ -1030,17 +1157,228 @@ Supported by the module @code{srfi-152} (@pxref{String library (reduced)}).
 モジュール@code{srfi-152}でサポートされます (@ref{String library (reduced)}参照)。
 
 
+srfi-154, srfi-154
+()
+
+First-class dynamic extents
+Suppored by the module @code{srfi-154}.  (@pxref{First-class dynamic extents}).
+
+第一級の動的エクステント
+モジュール@code{srfi-154}でサポートされます。(@ref{First-class dynamic extents}参照)。
+
+
 srfi-158, srfi-158
 ()
 
 Generators and accumulators
-Supported by the module @code{srfi-158} (@pxref{Generators and accumulators}).
+This has become a part of R7RS large.
+Supported by the module @code{scheme.generator}
+(@pxref{R7RS generators}).
 Note that most of generator procedures are supported by @code{gauche.generator}
 (@pxref{Generators}).
 
 ジェネレータとアキュムレータ
-モジュール@code{srfi-158}でサポートされます
- (@ref{Generators and accumulators}参照)。
+これはR7RS largeの一部となりました。
+モジュール@code{scheme.generator}でサポートされます
+ (@ref{R7RS generators}参照)。
 ジェネレータ手続きのほとんどは、@code{gauche.generator}でもサポートされています
 (@ref{Generators}参照)。
 
+
+srfi-159, srfi-159
+()
+
+Combinator Formatting
+This has become a part of R7RS large.  @xref{R7RS combinator formatting}.
+
+コンビネータによるフォーマッタ
+これはR7RS largeの一部になりました。@ref{R7RS combinator formatting}参照。
+
+srfi-160, srfi-160
+()
+
+Homogeneous numeric vector libraries
+This has become a part of R7RS large,
+supported by the module @code{scheme.vector.@@} where
+@code{@@} is one of @code{base},
+@code{u8}, @code{s8}, @code{u16}, @code{s16},
+@code{u32}, @code{s32}, @code{u64}, @code{s64}, @code{f32}, @code{f64},
+@code{c64}, or @code{c128} (@pxref{R7RS uniform vectors}).
+
+均質な数値ベクタライブラリ
+これはR7RS largeの一部となりました。
+モジュール@code{scheme.vector.@@}でサポートされます
+(但し@code{@@}は@code{base},
+@code{u8}, @code{s8}, @code{u16}, @code{s16},
+@code{u32}, @code{s32}, @code{u64}, @code{s64}, @code{f32}, @code{f64},
+@code{c64}, @code{c128})。
+@ref{R7RS uniform vectors}参照。
+
+
+srfi-162, srfi-162
+()
+
+Comparators sublibrary
+Supported by the module @code{srfi-162}.  @xref{Comparator sublibrary}.
+
+比較器のサブライブラリ
+モジュール@code{srfi-162}でサポートされます。@ref{Comparator sublibrary}参照。
+
+
+srfi-170, srfi-170
+()
+
+POSIX API
+Supported by the module @code{srfi-170}.  @xref{POSIX API}.
+
+POSIX API
+モジュール@code{srfi-170}でサポートされます。@ref{POSIX API}参照。
+
+
+srfi-173, srfi-173
+()
+
+Hooks
+Supported by the module @code{srfi-173} (@pxref{Hooks (srfi)}),
+which is a thin layer on top of @code{gauche.hook} (@pxref{Hooks})
+
+フック
+モジュール@code{srfi-173}でサポートされます (@ref{Hooks (srfi)}参照) 。
+それ自体は@code{gauche.hook}の上にかぶせた薄いレイヤです (@ref{Hooks}参照)。
+
+
+srfi-174, srfi-174
+()
+
+POSIX Timespecs
+Supported by the module @code{srfi-174} (@pxref{POSIX timespecs}).
+In Gauche, the timespec type is the same as built-in @code{<time>}
+object, which is also the same as srfi-19 time.
+
+POSIX Timespec
+モジュール@code{srfi-174}でサポートされます (@ref{POSIX timespecs}参照)。
+Gaucheでは、timespecオブジェクトは組み込みの@code{<time>}オブジェクトであり、
+またsrfi-19のtimeオブジェクトとも共通です。
+
+
+srfi-175, srfi-175
+()
+
+ASCII character library
+Supported by the module @code{srfi-175} (@pxref{ASCII character library}).
+
+ASCII文字ライブラリ
+モジュール@code{srfi-175}でサポートされます(@ref{ASCII character library}参照)。
+
+
+srfi-176, srfi-176
+()
+
+Version flag
+Supported as a command-line flag of @code{gosh}.
+The @code{version-alist} procedure is built-in.
+
+バージョンフラグ
+@code{gosh}のコマンドライン引数としてサポートされます。
+@code{version-alist}手続きは組み込みです。
+
+
+srfi-178, srfi-178
+()
+
+Bitvector library
+The basic support is built-in (@pxref{Bitvectors}).
+Complete support is in the module @code{srfi-178} (@pxref{Bitvector library}.
+
+ビットベクタライブラリ
+基本的なサポートは組み込みです (@ref{Bitvectors}参照)。
+完全なサポートはモジュール@code{srfi-178}にあります(@ref{Bitvector library}参照)。
+
+
+srfi-180, srfi-180
+()
+
+JSON
+Supported by the module @code{srfi-180} (@pxref{JSON}).
+Note that Gauche also has @code{rfc.json},
+and @code{srfi-180} is implemented on top of it.
+
+JSON
+モジュール@code{srfi-180}でサポートされます (@ref{JSON}参照)。
+Gaucheには@code{rfc.json}もあって、@code{srfi-180}は
+@code{rfc.json}の上に実装されています。
+
+
+srfi-181, srfi-181
+()
+
+Custom ports
+Supported by the module @code{srfi-181} (@pxref{Custom ports}).
+Gauche has an original custom port mechanism (@pxref{Virtual ports}), and
+This srfi is built in top of it.
+
+カスタムポート
+モジュール@code{srfi-181}でサポートされます (@ref{Custom ports}参照)。
+Gaucheは独自のカスタムポートメカニズムを持っており(@ref{Virtual ports}参照)、
+このsrfiはその上に作られています。
+
+
+srfi-185, srfi-185
+()
+
+Linear adjustable-length strings
+Supported by the module @code{srfi-185}.
+@xref{Linear adjustable-length strings}.
+
+線形更新な可変長文字列
+モジュール@code{srfi-185}でサポートされます。
+@ref{Linear adjustable-length strings}参照。
+
+
+srfi-189, srfi-189
+()
+
+Maybe and Either: optional container types
+Supported by the module @code{srfi-189}.
+
+MaybeとEither: オプショナルなコンテナ型
+モジュール@code{srfi-189}でサポートされます。
+
+
+srfi-192, srfi-192
+()
+
+Port positioning
+Gauche's port already has positining mechanism, so main procedures
+are built-in (@pxref{Common port operations}).
+A few additional procedures are provided by the module @code{srfi-192}
+(@pxref{Port positioning}).
+
+ポート位置
+Gaucheのポートは既に位置の問い合わせと設定機能を持っており、
+主要な手続きは組み込みになっています(@ref{Common port operations}参照)。
+組み込みでない追加手続きはモジュール@code{srfi-192}にあります
+(@ref{Port positioning}参照)。
+
+
+srfi-193, srfi-193
+()
+
+Command line
+Two procedures, @code{command-line} and @code{script-file}, are
+built-in.  Other APIs are provided by the module @code{srfi-193}
+(@pxref{Command line}).
+
+コマンドライン
+@code{command-line}と@code{script-file}の二つの手続きは組み込みです。
+他のAPIはモジュール@code{srfi-193}で提供されます (@ref{Command line}参照)。
+
+
+srfi-195
+(box box? unbox set-box! box-arity unbox-value set-box-value!)
+
+Multiple-value boxes
+Built-in.  @xref{Boxes}.
+
+多値ボックス
+組み込みです。@ref{Boxes}参照。
