@@ -1,7 +1,7 @@
 /*
  * symbol.c - symbol implementation
  *
- *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -64,7 +64,7 @@ static ScmHashTable *obtable = NULL;
 static struct {
     ScmHashTable *table;
     ScmInternalMutex mutex;
-} keywords = { NULL };
+} keywords;
 
 static int keyword_disjoint_p = FALSE;
 #endif /*!GAUCHE_KEEP_DISJOINT_KEYWORD_OPTION*/
@@ -410,7 +410,7 @@ ScmObj Scm_DeleteKeywordX(ScmObj key, ScmObj list)
                 return Scm_DeleteKeywordX(key, SCM_CDR(SCM_CDR(cp)));
             } else {
                 ScmObj tail = Scm_DeleteKeywordX(key, SCM_CDR(SCM_CDR(cp)));
-                SCM_SET_CDR(prev, tail);
+                Scm_SetCdr(prev, tail);
                 return list;
             }
         }
@@ -486,7 +486,7 @@ void Scm__InitSymbol(void)
     } else if (issymbol != NULL) {
         keyword_disjoint_p = FALSE;
     } else {
-        keyword_disjoint_p = TRUE; /* This determines the default */
+        keyword_disjoint_p = FALSE; /* This determines the default */
     }
     if (keyword_disjoint_p) {
         Scm_KeywordClass.cpa = &(keyword_cpl[1]);
