@@ -1,7 +1,7 @@
 /*
  * vector.c - vector implementation
  *
- *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -35,6 +35,26 @@
 #include "gauche.h"
 #include "gauche/priv/writerP.h"
 
+/* Catch integer overflow.
+   NB: If total size is too big, GC_malloc aborts.  But we have to prevent
+   total size from overflowing before passed to GC_malloc.
+
+   We'll try to allocate size*eltsize (+ up to two words of header).
+   The GC's malloc routine first round it up to GC allocation unit boundary
+   (8 or 16 bytes).  If there's not enough heap, then it tries to expand
+   the heap by the size rounded up to the pagesize.  We don't want the final
+   value overflows signed long.
+   (In reality, expanding heap with close to LONG_MAX surely fails, so it  
+   should suffice to avoid overflow before calling GC_MALLOC. But it won't
+   harm to have a bit of margin here...)
+ */
+static void check_size(ScmSmallInt size, int eltsize)
+{
+    if (size >= (ScmSmallInt)((LONG_MAX - 0x400000)/eltsize)) {
+        Scm_Error("Size too big: %ld", size);
+    }
+}
+
 /*=====================================================================
  * Generic vectors
  */
@@ -81,10 +101,15 @@ SCM_DEFINE_BUILTIN_CLASS_FLAGS(Scm_VectorClass, vector_print, vector_compare,
 
 static ScmVector *make_vector(ScmSmallInt size)
 {
+    check_size(size, sizeof(ScmObj));
     ScmVector *v = SCM_NEW2(ScmVector *,
                             sizeof(ScmVector) + sizeof(ScmObj)*(size-1));
     SCM_SET_CLASS(v, SCM_CLASS_VECTOR);
+#if GAUCHE_API_VERSION >= 1000
+    v->size_flags = (size << 1);
+#else    
     v->size = size;
+#endif
     return v;
 }
 
@@ -140,19 +165,21 @@ ScmObj Scm_VectorToList(ScmVector *v, ScmSmallInt start, ScmSmallInt end)
 
 ScmObj Scm_VectorRef(ScmVector *vec, ScmSmallInt i, ScmObj fallback)
 {
-    if (i < 0 || i >= vec->size) return fallback;
+    if (i < 0 || i >= SCM_VECTOR_SIZE(vec)) return fallback;
     return vec->elements[i];
 }
 
 ScmObj Scm_VectorSet(ScmVector *vec, ScmSmallInt i, ScmObj obj)
 {
-    if (i >= 0 && i < vec->size) vec->elements[i] = obj;
+    SCM_VECTOR_CHECK_MUTABLE(vec);
+    if (i >= 0 && i < SCM_VECTOR_SIZE(vec)) vec->elements[i] = obj;
     return obj;
 }
 
 ScmObj Scm_VectorFill(ScmVector *vec, ScmObj fill,
                       ScmSmallInt start, ScmSmallInt end)
 {
+    SCM_VECTOR_CHECK_MUTABLE(vec);
     ScmSmallInt len = SCM_VECTOR_SIZE(vec);
     SCM_CHECK_START_END(start, end, len);
     for (ScmSmallInt i=start; i < end; i++) {
@@ -225,40 +252,49 @@ DEF_UVCLASS(U64, u64)
 DEF_UVCLASS(F16, f16)
 DEF_UVCLASS(F32, f32)
 DEF_UVCLASS(F64, f64)
+DEF_UVCLASS(C32, c32)
+DEF_UVCLASS(C64, c64)
+DEF_UVCLASS(C128, c128)
 
 /*
  * Some generic APIs
  */
 ScmUVectorType Scm_UVectorType(ScmClass *klass)
 {
-    if (SCM_EQ(klass, SCM_CLASS_S8VECTOR))  return SCM_UVECTOR_S8;
-    if (SCM_EQ(klass, SCM_CLASS_U8VECTOR))  return SCM_UVECTOR_U8;
-    if (SCM_EQ(klass, SCM_CLASS_S16VECTOR)) return SCM_UVECTOR_S16;
-    if (SCM_EQ(klass, SCM_CLASS_U16VECTOR)) return SCM_UVECTOR_U16;
-    if (SCM_EQ(klass, SCM_CLASS_S32VECTOR)) return SCM_UVECTOR_S32;
-    if (SCM_EQ(klass, SCM_CLASS_U32VECTOR)) return SCM_UVECTOR_U32;
-    if (SCM_EQ(klass, SCM_CLASS_S64VECTOR)) return SCM_UVECTOR_S64;
-    if (SCM_EQ(klass, SCM_CLASS_U64VECTOR)) return SCM_UVECTOR_U64;
-    if (SCM_EQ(klass, SCM_CLASS_F16VECTOR)) return SCM_UVECTOR_F16;
-    if (SCM_EQ(klass, SCM_CLASS_F32VECTOR)) return SCM_UVECTOR_F32;
-    if (SCM_EQ(klass, SCM_CLASS_F64VECTOR)) return SCM_UVECTOR_F64;
+    if (SCM_EQ(klass, SCM_CLASS_S8VECTOR))   return SCM_UVECTOR_S8;
+    if (SCM_EQ(klass, SCM_CLASS_U8VECTOR))   return SCM_UVECTOR_U8;
+    if (SCM_EQ(klass, SCM_CLASS_S16VECTOR))  return SCM_UVECTOR_S16;
+    if (SCM_EQ(klass, SCM_CLASS_U16VECTOR))  return SCM_UVECTOR_U16;
+    if (SCM_EQ(klass, SCM_CLASS_S32VECTOR))  return SCM_UVECTOR_S32;
+    if (SCM_EQ(klass, SCM_CLASS_U32VECTOR))  return SCM_UVECTOR_U32;
+    if (SCM_EQ(klass, SCM_CLASS_S64VECTOR))  return SCM_UVECTOR_S64;
+    if (SCM_EQ(klass, SCM_CLASS_U64VECTOR))  return SCM_UVECTOR_U64;
+    if (SCM_EQ(klass, SCM_CLASS_F16VECTOR))  return SCM_UVECTOR_F16;
+    if (SCM_EQ(klass, SCM_CLASS_F32VECTOR))  return SCM_UVECTOR_F32;
+    if (SCM_EQ(klass, SCM_CLASS_F64VECTOR))  return SCM_UVECTOR_F64;
+    if (SCM_EQ(klass, SCM_CLASS_C32VECTOR))  return SCM_UVECTOR_C32;
+    if (SCM_EQ(klass, SCM_CLASS_C64VECTOR))  return SCM_UVECTOR_C64;
+    if (SCM_EQ(klass, SCM_CLASS_C128VECTOR)) return SCM_UVECTOR_C128;
     else return SCM_UVECTOR_INVALID;
 }
 
 const char *Scm_UVectorTypeName(int type) /* for error msgs etc. */
 {
     switch (type) {
-    case SCM_UVECTOR_S8:  return "s8vector";
-    case SCM_UVECTOR_U8:  return "u8vector";
-    case SCM_UVECTOR_S16: return "s16vector";
-    case SCM_UVECTOR_U16: return "u16vector";
-    case SCM_UVECTOR_S32: return "s32vector";
-    case SCM_UVECTOR_U32: return "u32vector";
-    case SCM_UVECTOR_S64: return "s64vector";
-    case SCM_UVECTOR_U64: return "u64vector";
-    case SCM_UVECTOR_F16: return "f16vector";
-    case SCM_UVECTOR_F32: return "f32vector";
-    case SCM_UVECTOR_F64: return "f64vector";
+    case SCM_UVECTOR_S8:   return "s8vector";
+    case SCM_UVECTOR_U8:   return "u8vector";
+    case SCM_UVECTOR_S16:  return "s16vector";
+    case SCM_UVECTOR_U16:  return "u16vector";
+    case SCM_UVECTOR_S32:  return "s32vector";
+    case SCM_UVECTOR_U32:  return "u32vector";
+    case SCM_UVECTOR_S64:  return "s64vector";
+    case SCM_UVECTOR_U64:  return "u64vector";
+    case SCM_UVECTOR_F16:  return "f16vector";
+    case SCM_UVECTOR_F32:  return "f32vector";
+    case SCM_UVECTOR_F64:  return "f64vector";
+    case SCM_UVECTOR_C32:  return "c32vector";
+    case SCM_UVECTOR_C64:  return "c64vector";
+    case SCM_UVECTOR_C128: return "c128vector";
     default: return "invalid type of uvector (possibly implementation error)";
     }
 }
@@ -267,7 +303,11 @@ const char *Scm_UVectorTypeName(int type) /* for error msgs etc. */
 int Scm_UVectorElementSize(ScmClass *klass)
 {
     static const int sizes[] = { 1, 1, 2, 2, 4, 4, 8, 8,
-                                 2, sizeof(float), sizeof(double) };
+                                 2, sizeof(float), sizeof(double), -1,
+                                 sizeof(ScmHalfComplex),
+                                 sizeof(ScmFloatComplex),
+                                 sizeof(ScmDoubleComplex),
+                                 -1 };
     int ind = (int)Scm_UVectorType(klass);
     if (ind >= 0) return sizes[ind];
     return -1;
@@ -290,14 +330,10 @@ ScmObj Scm_MakeUVectorFull(ScmClass *klass, ScmSmallInt size, void *init,
     if (init) {
         vec->elements = init;   /* trust the caller */
     } else {
+        check_size(size, eltsize);
         vec->elements = SCM_NEW_ATOMIC2(void*, size*eltsize);
     }
-#if GAUCHE_API_0_95
     vec->size_flags = (size << 1)|(immutable?1:0);
-#else  /*!GAUCHE_API_0_95*/
-    vec->size = size;
-    vec->immutable = immutable;
-#endif /*!GAUCHE_API_0_95*/
     vec->owner = owner;
     return SCM_OBJ(vec);
 }
@@ -311,12 +347,15 @@ ScmObj Scm_ListToUVector(ScmClass *klass, ScmObj list, int clamp)
 {
     ScmUVectorType type = Scm_UVectorType(klass);
     if (type < 0) Scm_Error("uvector class required, but got: %S", klass);
-    ScmSmallInt length = Scm_Length(list);
+    ScmSize length = Scm_Length(list);
     if (length < 0) Scm_Error("improper list not allowed: %S", list);
+    if (length > SCM_SMALL_INT_MAX) Scm_Error("list is too long: %,,,,100S", list);
 
-    ScmUVector *v = (ScmUVector*)Scm_MakeUVector(klass, length, NULL);
+    ScmUVector *v = (ScmUVector*)Scm_MakeUVector(klass,
+                                                 (ScmSmallInt)length,
+                                                 NULL);
     ScmObj cp = list;
-    for (int i=0; i<length; i++, cp = SCM_CDR(cp)) {
+    for (ScmSize i=0; i<length; i++, cp = SCM_CDR(cp)) {
         switch (type) {
         case SCM_UVECTOR_S8:
             SCM_S8VECTOR_ELEMENTS(v)[i] =
@@ -344,11 +383,11 @@ ScmObj Scm_ListToUVector(ScmClass *klass, ScmObj list, int clamp)
             break;
         case SCM_UVECTOR_S64:
             SCM_S64VECTOR_ELEMENTS(v)[i] =
-                (ScmInt64)Scm_GetInteger64Clamp(SCM_CAR(cp), clamp, NULL);
+                (int64_t)Scm_GetInteger64Clamp(SCM_CAR(cp), clamp, NULL);
             break;
         case SCM_UVECTOR_U64:
             SCM_U64VECTOR_ELEMENTS(v)[i] =
-                (ScmUInt64)Scm_GetIntegerU64Clamp(SCM_CAR(cp), clamp, NULL);
+                (uint64_t)Scm_GetIntegerU64Clamp(SCM_CAR(cp), clamp, NULL);
             break;
         case SCM_UVECTOR_F16:
             SCM_F16VECTOR_ELEMENTS(v)[i] =
@@ -359,8 +398,16 @@ ScmObj Scm_ListToUVector(ScmClass *klass, ScmObj list, int clamp)
                 (float)Scm_GetDouble(SCM_CAR(cp));
             break;
         case SCM_UVECTOR_F64:
-            SCM_F64VECTOR_ELEMENTS(v)[i] =
-                Scm_GetDouble(SCM_CAR(cp));
+            SCM_F64VECTOR_ELEMENTS(v)[i] = Scm_GetDouble(SCM_CAR(cp));
+            break;
+        case SCM_UVECTOR_C32:
+            SCM_C32VECTOR_ELEMENTS(v)[i] = Scm_GetHalfComplex(SCM_CAR(cp));
+            break;
+        case SCM_UVECTOR_C64:
+            SCM_C64VECTOR_ELEMENTS(v)[i] = Scm_GetFloatComplex(SCM_CAR(cp));
+            break;
+        case SCM_UVECTOR_C128:
+            SCM_C128VECTOR_ELEMENTS(v)[i] = Scm_GetDoubleComplex(SCM_CAR(cp));
             break;
         default:
             Scm_Error("[internal error] unknown uvector type given to Scm_ListToUVector");
@@ -398,6 +445,12 @@ ScmObj Scm_VMUVectorRef(ScmUVector *v, int t, ScmSmallInt k, ScmObj fallback)
         return Scm_VMReturnFlonum((double)(SCM_F32VECTOR_ELEMENT(v, k)));
     case SCM_UVECTOR_F64:
         return Scm_VMReturnFlonum(SCM_F64VECTOR_ELEMENT(v, k));
+    case SCM_UVECTOR_C32:
+        return Scm_HalfComplexToComplex(SCM_C32VECTOR_ELEMENT(v, k));
+    case SCM_UVECTOR_C64:
+        return Scm_FloatComplexToComplex(SCM_C64VECTOR_ELEMENT(v, k));
+    case SCM_UVECTOR_C128:
+        return Scm_DoubleComplexToComplex(SCM_C128VECTOR_ELEMENT(v, k));
     default:
         Scm_Error("[internal error] unknown uvector type given to Scm_VMUVectorRef");
         return SCM_UNDEFINED;   /* dummy */
@@ -446,6 +499,15 @@ ScmObj Scm_UVectorSet(ScmUVector *v, int t, ScmSmallInt k, ScmObj val, int clamp
     case SCM_UVECTOR_F64:
         SCM_F64VECTOR_ELEMENTS(v)[k] = Scm_GetDouble(val);
         break;
+    case SCM_UVECTOR_C32:
+        SCM_C32VECTOR_ELEMENTS(v)[k] = Scm_GetHalfComplex(val);
+        break;
+    case SCM_UVECTOR_C64:
+        SCM_C64VECTOR_ELEMENTS(v)[k] = Scm_GetFloatComplex(val);
+        break;
+    case SCM_UVECTOR_C128:
+        SCM_C128VECTOR_ELEMENTS(v)[k] = Scm_GetDoubleComplex(val);
+        break;
     default:
         Scm_Error("[internal error] unknown uvector type given to Scm_VMUVectorRef");
     }
@@ -470,6 +532,7 @@ ScmObj SCM_CPP_CAT3(Scm_Make,tag,Vector)(ScmSmallInt size, T fill)      \
 ScmObj SCM_CPP_CAT3(Scm_Make,tag,VectorFromArray)(ScmSmallInt size,     \
                                                   const T array[])      \
 {                                                                       \
+    check_size(size, sizeof(T));                                        \
     T *z = SCM_NEW_ATOMIC_ARRAY(T, size);                               \
     memcpy(z, array, size*sizeof(T));                                   \
     return Scm_MakeUVector(SCM_CPP_CAT3(SCM_CLASS_,tag,VECTOR),         \
@@ -504,11 +567,14 @@ DEF_UVCTOR_FILL(S16, int16_t)
 DEF_UVCTOR_FILL(U16, uint16_t)
 DEF_UVCTOR_FILL(S32, int32_t)
 DEF_UVCTOR_FILL(U32, uint32_t)
-DEF_UVCTOR_FILL(S64, ScmInt64)
-DEF_UVCTOR_FILL(U64, ScmUInt64)
+DEF_UVCTOR_FILL(S64, int64_t)
+DEF_UVCTOR_FILL(U64, uint64_t)
 DEF_UVCTOR_FILL(F16, ScmHalfFloat)
 DEF_UVCTOR_FILL(F32, float)
 DEF_UVCTOR_FILL(F64, double)
+DEF_UVCTOR_FILL(C32, ScmHalfComplex)
+DEF_UVCTOR_FILL(C64, ScmFloatComplex)
+DEF_UVCTOR_FILL(C128,ScmDoubleComplex)
 
 DEF_UVCTOR_ARRAY(S8,  int8_t)
 DEF_UVCTOR_ARRAY(U8,  uint8_t)
@@ -516,11 +582,14 @@ DEF_UVCTOR_ARRAY(S16, int16_t)
 DEF_UVCTOR_ARRAY(U16, uint16_t)
 DEF_UVCTOR_ARRAY(S32, int32_t)
 DEF_UVCTOR_ARRAY(U32, uint32_t)
-DEF_UVCTOR_ARRAY(S64, ScmInt64)
-DEF_UVCTOR_ARRAY(U64, ScmUInt64)
+DEF_UVCTOR_ARRAY(S64, int64_t)
+DEF_UVCTOR_ARRAY(U64, uint64_t)
 DEF_UVCTOR_ARRAY(F16, ScmHalfFloat)
 DEF_UVCTOR_ARRAY(F32, float)
 DEF_UVCTOR_ARRAY(F64, double)
+DEF_UVCTOR_ARRAY(C32, ScmHalfComplex)
+DEF_UVCTOR_ARRAY(C64, ScmFloatComplex)
+DEF_UVCTOR_ARRAY(C128,ScmDoubleComplex)
 
 /*
  * Reader
@@ -532,24 +601,27 @@ ScmObj Scm_ReadUVector(ScmPort *port, const char *tag, ScmReadContext *ctx)
     if (c != '(') Scm_Error("bad uniform vector syntax for %s", tag);
     ScmObj list = Scm_ReadList(SCM_OBJ(port), ')');
     ScmClass *klass = NULL;
-    if (strcmp(tag, "s8") == 0)       klass = SCM_CLASS_S8VECTOR;
-    else if (strcmp(tag, "u8") == 0)  klass = SCM_CLASS_U8VECTOR;
-    else if (strcmp(tag, "s16") == 0) klass = SCM_CLASS_S16VECTOR;
-    else if (strcmp(tag, "u16") == 0) klass = SCM_CLASS_U16VECTOR;
-    else if (strcmp(tag, "s32") == 0) klass = SCM_CLASS_S32VECTOR;
-    else if (strcmp(tag, "u32") == 0) klass = SCM_CLASS_U32VECTOR;
-    else if (strcmp(tag, "s64") == 0) klass = SCM_CLASS_S64VECTOR;
-    else if (strcmp(tag, "u64") == 0) klass = SCM_CLASS_U64VECTOR;
-    else if (strcmp(tag, "f16") == 0) klass = SCM_CLASS_F16VECTOR;
-    else if (strcmp(tag, "f32") == 0) klass = SCM_CLASS_F32VECTOR;
-    else if (strcmp(tag, "f64") == 0) klass = SCM_CLASS_F64VECTOR;
+    if (strcmp(tag, "s8") == 0)        klass = SCM_CLASS_S8VECTOR;
+    else if (strcmp(tag, "u8") == 0)   klass = SCM_CLASS_U8VECTOR;
+    else if (strcmp(tag, "s16") == 0)  klass = SCM_CLASS_S16VECTOR;
+    else if (strcmp(tag, "u16") == 0)  klass = SCM_CLASS_U16VECTOR;
+    else if (strcmp(tag, "s32") == 0)  klass = SCM_CLASS_S32VECTOR;
+    else if (strcmp(tag, "u32") == 0)  klass = SCM_CLASS_U32VECTOR;
+    else if (strcmp(tag, "s64") == 0)  klass = SCM_CLASS_S64VECTOR;
+    else if (strcmp(tag, "u64") == 0)  klass = SCM_CLASS_U64VECTOR;
+    else if (strcmp(tag, "f16") == 0)  klass = SCM_CLASS_F16VECTOR;
+    else if (strcmp(tag, "f32") == 0)  klass = SCM_CLASS_F32VECTOR;
+    else if (strcmp(tag, "f64") == 0)  klass = SCM_CLASS_F64VECTOR;
+    else if (strcmp(tag, "c32") == 0)  klass = SCM_CLASS_C32VECTOR;
+    else if (strcmp(tag, "c64") == 0)  klass = SCM_CLASS_C64VECTOR;
+    else if (strcmp(tag, "c128") == 0) klass = SCM_CLASS_C128VECTOR;
     else Scm_Error("invalid unform vector tag: %s", tag);
 
     ScmObj uv = Scm_ListToUVector(klass, list, 0);
     
     /* If we are reading source file, let literal uvectors be immutable. */
     if (Scm_ReadContextLiteralImmutable(ctx)) {
-        SCM_UVECTOR_IMMUTABLE_P(uv) = TRUE;
+        SCM_UVECTOR_IMMUTABLE_SET(uv, TRUE);
     }
     return uv;
 }
@@ -566,7 +638,7 @@ static void SCM_CPP_CAT3(print_,tag,vector)(ScmObj obj,                 \
                                             ScmWriteContext *ctx)       \
 {                                                                       \
     const ScmWriteControls *wp =                                        \
-        Scm_GetWriteControls(ctx, out->writeState);                     \
+        Scm_GetWriteControls(ctx, Scm_PortWriteState(out));             \
     Scm_Printf(out, "#"#tag"(");                                        \
     for (int i=0; i<SCM_CPP_CAT3(SCM_,TAG,VECTOR_SIZE)(obj); i++) {     \
         T elt = SCM_CPP_CAT3(SCM_,TAG,VECTOR_ELEMENTS)(obj)[i];         \
@@ -583,12 +655,32 @@ static void SCM_CPP_CAT3(print_,tag,vector)(ScmObj obj,                 \
 #define spr(out, elt) Scm_Printf(out, "%d", elt)
 #define upr(out, elt) Scm_Printf(out, "%u", elt)
 #define fpr(out, elt) Scm_PrintDouble(out, (double)elt, 0)
+#define c32pr(out, elt)                                                 \
+    do {                                                                \
+        Scm_PrintDouble(out, Scm_HalfToDouble(SCM_HALF_COMPLEX_REAL(elt)), 0); \
+        Scm_Putz("+", 1, out);                                          \
+        Scm_PrintDouble(out, Scm_HalfToDouble(SCM_HALF_COMPLEX_IMAG(elt)), 0); \
+        Scm_Putz("i", 1, out);                                          \
+    } while (0)
+#define c64pr(out, elt)                                 \
+    do {                                                \
+        Scm_PrintDouble(out, (double)crealf(elt), 0);   \
+        Scm_Putz("+", 1, out);                          \
+        Scm_PrintDouble(out, (double)cimagf(elt), 0);   \
+        Scm_Putz("i", 1, out);                          \
+    } while (0)
+#define c128pr(out, elt)                        \
+    do {                                        \
+        Scm_PrintDouble(out, creal(elt), 0);    \
+        Scm_Putz("+", 1, out);                  \
+        Scm_PrintDouble(out, cimag(elt), 0);    \
+        Scm_Putz("i", 1, out);                  \
+    } while (0)
+    
 
-static inline void s64pr(ScmPort *out, ScmInt64 elt)
+static inline void s64pr(ScmPort *out, int64_t elt)
 {
-#if SCM_EMULATE_INT64
-    Scm_Printf(out, "%S", Scm_MakeInteger64(elt));
-#elif SIZEOF_LONG == 4
+#if SIZEOF_LONG == 4
     char buf[50];
     snprintf(buf, 50, "%lld", elt);
     Scm_Printf(out, "%s", buf);
@@ -597,11 +689,9 @@ static inline void s64pr(ScmPort *out, ScmInt64 elt)
 #endif
 }
 
-static inline void u64pr(ScmPort *out, ScmUInt64 elt)
+static inline void u64pr(ScmPort *out, uint64_t elt)
 {
-#if SCM_EMULATE_INT64
-    Scm_Printf(out, "%S", Scm_MakeIntegerU64(elt));
-#elif SIZEOF_LONG == 4
+#if SIZEOF_LONG == 4
     char buf[50];
     snprintf(buf, 50, "%llu", elt);
     Scm_Printf(out, "%s", buf);
@@ -621,11 +711,14 @@ DEF_PRINT(S16, s16, int16_t, spr)
 DEF_PRINT(U16, u16, uint16_t, upr)
 DEF_PRINT(S32, s32, int32_t, spr)
 DEF_PRINT(U32, u32, uint32_t, upr)
-DEF_PRINT(S64, s64, ScmInt64, s64pr)
-DEF_PRINT(U64, u64, ScmUInt64, u64pr)
+DEF_PRINT(S64, s64, int64_t, s64pr)
+DEF_PRINT(U64, u64, uint64_t, u64pr)
 DEF_PRINT(F16, f16, ScmHalfFloat, f16pr)
 DEF_PRINT(F32, f32, float, fpr)
 DEF_PRINT(F64, f64, double, fpr)
+DEF_PRINT(C32, c32, ScmHalfComplex, c32pr)
+DEF_PRINT(C64, c64, ScmFloatComplex, c64pr)
+DEF_PRINT(C128, c128, ScmDoubleComplex, c128pr)
 
 
 /* comparer */
@@ -658,68 +751,223 @@ static int SCM_CPP_CAT3(compare_,tag,vector)(ScmObj x, ScmObj y, int equalp) \
 #define common_eqv(x, y)  ((x)==(y))
 #define common_lt(x, y)   ((x)<(y))
 
-static inline int int64eqv(ScmInt64 x, ScmInt64 y)
-{
-#if SCM_EMULATE_INT64
-    return (x.hi == y.hi && x.lo == y.lo);
-#else
-    return x == y;
-#endif
-}
-
-static inline int uint64eqv(ScmUInt64 x, ScmUInt64 y)
-{
-#if SCM_EMULATE_INT64
-    return (x.hi == y.hi && x.lo == y.lo);
-#else
-    return x == y;
-#endif
-}
-
-static inline int int64lt(ScmInt64 x, ScmInt64 y)
-{
-#if SCM_EMULATE_INT64
-    return (x.hi < y.hi || (x.hi == y.hi && x.lo < y.lo));
-#else
-    return x < y;
-#endif
-}
-
-static inline int uint64lt(ScmUInt64 x, ScmUInt64 y)
-{
-#if SCM_EMULATE_INT64
-    return (x.hi < y.hi || (x.hi == y.hi && x.lo < y.lo));
-#else
-    return x < y;
-#endif
-}
-
 #define f16eqv(a, b) SCM_HALF_FLOAT_CMP(==, a, b)
 #define f16lt(a, b)  SCM_HALF_FLOAT_CMP(<, a, b)
 
+static inline int c32eqv(ScmHalfComplex x, ScmHalfComplex y)
+{
+    return (SCM_HALF_FLOAT_CMP(==, x.r, y.r)
+            && SCM_HALF_FLOAT_CMP(==, x.i, y.i));
+}
+
+static inline int c32lt(ScmHalfComplex x, ScmHalfComplex y)
+{
+    return (SCM_HALF_FLOAT_CMP(<, x.r, y.r)
+            || (SCM_HALF_FLOAT_CMP(==, x.r, y.r)
+                && SCM_HALF_FLOAT_CMP(<, x.i, y.i)));
+}
+
+static inline int c64lt(ScmFloatComplex x, ScmFloatComplex y)
+{
+    return (crealf(x) < crealf(y)
+            || (crealf(x) == crealf(y)
+                && cimagf(x) < cimagf(y)));
+}
+
+static inline int c128lt(ScmDoubleComplex x, ScmDoubleComplex y)
+{
+    return (creal(x) < creal(y)
+            || (creal(x) == creal(y)
+                && cimag(x) < cimag(y)));
+}
+
+
 DEF_CMP(S8, s8,   int8_t, common_eqv, common_lt)
 DEF_CMP(U8, u8,   uint8_t, common_eqv, common_lt)
 DEF_CMP(S16, s16, int16_t, common_eqv, common_lt)
 DEF_CMP(U16, u16, uint16_t, common_eqv, common_lt)
 DEF_CMP(S32, s32, int32_t, common_eqv, common_lt)
 DEF_CMP(U32, u32, uint32_t, common_eqv, common_lt)
-DEF_CMP(S64, s64, ScmInt64, int64eqv, int64lt)
-DEF_CMP(U64, u64, ScmUInt64, uint64eqv, uint64lt)
+DEF_CMP(S64, s64, int64_t, common_eqv, common_lt)
+DEF_CMP(U64, u64, uint64_t, common_eqv, common_lt)
 DEF_CMP(F16, f16, ScmHalfFloat, f16eqv, f16lt)
 DEF_CMP(F32, f32, float, common_eqv, common_lt)
 DEF_CMP(F64, f64, double, common_eqv, common_lt)
+DEF_CMP(C32, c32, ScmHalfComplex, c32eqv, c32lt)
+DEF_CMP(C64, c64, ScmFloatComplex, common_eqv, c64lt)
+DEF_CMP(C128, c128, ScmDoubleComplex, common_eqv, c128lt)
+
+/*=====================================================================
+ * Bitvectors
+ */
+
+static void bitvector_write_int(ScmBitvector *v, int prefix, ScmPort *port)
+{
+    if (prefix) Scm_Putz("#*", -1, port);
+    for (int i=0; i<SCM_BITVECTOR_SIZE(v); i++) {
+        if (SCM_BITS_TEST(v->bits, i)) SCM_PUTC('1', port);
+        else SCM_PUTC('0', port);
+    }
+}
+
+static void bitvector_print(ScmObj obj, 
+                            ScmPort *port, 
+                            ScmWriteContext *ctx SCM_UNUSED)
+{
+    bitvector_write_int(SCM_BITVECTOR(obj), TRUE, port);
+}
+
+static int bitvector_compare(ScmObj x, ScmObj y, int equalp SCM_UNUSED)
+{
+    SCM_ASSERT(SCM_BITVECTORP(x)&&SCM_BITVECTORP(y));
+    ScmBits *bx = SCM_BITVECTOR_BITS(x);
+    ScmBits *by = SCM_BITVECTOR_BITS(y);
+    ScmWord xlen = SCM_BITVECTOR_SIZE(x);
+    ScmWord ylen = SCM_BITVECTOR_SIZE(y);
+
+    /* NB: The ordering is somewhat counterintuitive, for the bits are
+       stored in little-endian.  That is, #*0100 comes after #*1000.
+       This is a lot faster.  Srfi-178 doesn't define the ordering. */
+    if (xlen < ylen) return -1;
+    if (xlen > ylen) return 1;
+    size_t nw = SCM_BITS_NUM_WORDS(xlen);
+    for (size_t i = 0; i < nw; i++, bx++, by++) {
+        if (*bx < *by) return -1;
+        if (*bx > *by) return 1;
+    }
+    return 0;
+}
+
+SCM_DEFINE_BUILTIN_CLASS_FLAGS(Scm_BitvectorClass,
+                               bitvector_print, bitvector_compare,
+                               NULL, NULL, SCM_CLASS_SEQUENCE_CPL,
+                               SCM_CLASS_AGGREGATE);
+
+
+int Scm_Bit2Int(ScmObj bit)
+{
+    if (SCM_EQ(bit, SCM_TRUE) || SCM_EQ(bit, SCM_MAKE_INT(1))) return 1;
+    if (SCM_FALSEP(bit) || SCM_EQ(bit, SCM_MAKE_INT(0))) return 0;
+    Scm_Error("bit value must be 0, 1, #f or #t, but got: %S", bit);
+    return 0;                   /* dummy */
+}
+
+ScmObj Scm_Bit2Bool(ScmObj bit)
+{
+    if (SCM_EQ(bit, SCM_TRUE) || SCM_EQ(bit, SCM_MAKE_INT(1))) return SCM_TRUE;
+    if (SCM_FALSEP(bit) || SCM_EQ(bit, SCM_MAKE_INT(0))) return SCM_FALSE;
+    Scm_Error("bit value must be 0, 1, #f or #t, but got: %S", bit);
+    return SCM_UNDEFINED;       /* dummy */
+}
+
+/* init can be 0, 1, #f or #t. */
+ScmObj Scm_MakeBitvector(ScmSmallInt size, ScmObj init)
+{
+    if (size < 0) {
+        Scm_Error("bitvector size must be a positive integer, but got %d", size);
+    }
+    ScmBitvector *v = SCM_NEW(ScmBitvector);
+    SCM_SET_CLASS(v, SCM_CLASS_BITVECTOR);
+    v->size_flags = (size << 1);
+    v->bits = Scm_MakeBits(size);
+
+    int fill = Scm_Bit2Int(init);
+    Scm_BitsFill(v->bits, 0, size, fill);
+    return SCM_OBJ(v);
+}
+
+ScmObj Scm_ListToBitvector(ScmObj lis) 
+{
+    ScmSmallInt len = Scm_Length(lis);
+    if (len < 0) {
+        Scm_Error("proper list required, but got: %S", lis);
+    }
+    
+    ScmBitvector *v = SCM_BITVECTOR(Scm_MakeBitvector(len, SCM_FALSE));
+    ScmObj cp;
+    ScmSmallInt i = 0;
+    SCM_FOR_EACH(cp, lis) {
+        if (Scm_Bit2Int(SCM_CAR(cp))) SCM_BITS_SET(v->bits, i);
+        else                          SCM_BITS_RESET(v->bits, i);
+        i++;
+    }
+    return SCM_OBJ(v);
+}
+
+/* Parse string of 0's and 1's to a bitvector.  If PREFIX is true,      
+   "#*" prefix is assumed.  Returns #f if unparsable. */
+
+ScmObj Scm_StringToBitvector(ScmString *s, int prefix)
+{
+    const ScmStringBody *b = SCM_STRING_BODY(s);
+    const char *p = SCM_STRING_BODY_START(b);
+    /* multibyte string can't be a bitvector literal. */
+    if (SCM_STRING_BODY_SIZE(b) != SCM_STRING_BODY_LENGTH(b)) return SCM_FALSE;
+    ScmSmallInt len = SCM_STRING_BODY_LENGTH(b);
+    
+    if (prefix) {
+        if (len < 2) return SCM_FALSE;
+        if (strncmp(p, "#*", 2) != 0) return SCM_FALSE;
+        p += 2;
+        len -= 2;
+    }
+
+    ScmBitvector *v = SCM_BITVECTOR(Scm_MakeBitvector(len, SCM_FALSE));
+    int i = 0;
+    for (; i < len; p++, i++) {
+        if (*p == '0')      SCM_BITS_RESET(v->bits, i);
+        else if (*p == '1') SCM_BITS_SET(v->bits, i);
+        else return SCM_FALSE;
+    }
+    return SCM_OBJ(v);
+}
+
+ScmObj Scm_BitvectorToString(ScmBitvector *v, int prefix)
+{
+    ScmObj out = Scm_MakeOutputStringPort(TRUE);
+    bitvector_write_int(v, prefix, SCM_PORT(out));
+    return Scm_GetOutputString(SCM_PORT(out), 0);
+}
+
+ScmObj Scm_BitvectorCopy(ScmBitvector *v, ScmSmallInt start, ScmSmallInt end)
+{
+    ScmSmallInt size = SCM_BITVECTOR_SIZE(v);
+    SCM_CHECK_START_END(start, end, size);
+    ScmBitvector *vv = SCM_BITVECTOR(Scm_MakeBitvector(end-start, SCM_FALSE));
+    Scm_BitsCopyX(vv->bits, 0, v->bits, start, end);
+    return SCM_OBJ(vv);
+}
+
+ScmObj Scm_BitvectorCopyX(ScmBitvector *dest, ScmSmallInt dstart,
+                          ScmBitvector *src,
+                          ScmSmallInt sstart, ScmSmallInt send)
+{
+    SCM_BITVECTOR_CHECK_MUTABLE(dest);
+    ScmSmallInt ssize = SCM_BITVECTOR_SIZE(src);
+    SCM_CHECK_START_END(sstart, send, ssize);
+    ScmSmallInt dsize = SCM_BITVECTOR_SIZE(dest);
+    ScmSmallInt dend = dstart + send - sstart;
+    if (dstart > dsize || dstart < 0 || dend > dsize) {
+        Scm_Error("destination index out of range (size=%ld, start=%ld, end=%ld)",
+                  dsize, dstart, dend);
+    }
+
+    Scm_BitsCopyX(dest->bits, dstart, src->bits, sstart, send);
+    return SCM_OBJ(dest);
+}
+
 
 /*=====================================================================
  * Utility
  */
 
-const uint8_t *Scm_GetBytes(ScmObj obj, size_t *size)
+const uint8_t *Scm_GetBytes(ScmObj obj, ScmSize *size)
 {
     if (SCM_UVECTORP(obj)) {
         *size = Scm_UVectorSizeInBytes(SCM_UVECTOR(obj));
         return (const uint8_t*)SCM_UVECTOR_ELEMENTS(obj);
     } else if (SCM_STRINGP(obj)) {
-        unsigned int s;
+        ScmSmallInt s;
         const char *z = Scm_GetStringContent(SCM_STRING(obj), &s, 0, 0);
         *size = s;
         return (const uint8_t*)z;
