@@ -1,7 +1,7 @@
 ;;;
 ;;; vminsn.scm - Virtual machine instruction definition
 ;;;
-;;;   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -82,6 +82,7 @@
 ;;;                         of having LREF0-SOMETHING or LREF21-SOMETHING
 ;;;                         separately, we'll have LREF-SOMETHING(0,0) and
 ;;;                         LREF-SOMETHING(2,1), respectively.
+;;;           :multi-value - Can result multiple values.
 
 ;;;==============================================================
 ;;; Common Cise macros
@@ -305,10 +306,47 @@
 (define-cise-stmt $include
   [(_ var) `("\n" |#reset-line| "#include " ,(write-to-string var) "\n")])
 
+;; ($lref depth offset)
+;;   Common code in LREF and XLREF.
+(define-cise-stmt $lref
+  [(_ depth offset)
+   (let ([dep (gensym)]
+         [off (gensym)]
+         [e (gensym)])
+     `(let* ([,dep ::int ,depth]
+             [,off ::int ,offset]
+             [,e ::ScmEnvFrame* ENV])
+        (for [() (> ,dep 0) (post-- ,dep)]
+          (set! ,e (-> ,e up)))
+        ($result (ENV-DATA ,e ,off))))])
+
+;; ($lset depth offset)
+;;   Common code in LSETL and XLSET.
+(define-cise-stmt $lset
+  [(_ depth offset)
+   (let ([dep (gensym)]
+         [off (gensym)]
+         [e (gensym)]
+         [box (gensym)])
+     `(let* ([,dep ::int ,depth]
+             [,off ::int ,offset]
+             [,e ::ScmEnvFrame* ENV])
+        (for [() (> ,dep 0) (post-- ,dep)]
+          (VM-ASSERT (!= ,e NULL))
+          (set! ,e (-> ,e up)))
+        (VM-ASSERT (!= ,e NULL))
+        (VM-ASSERT (> (-> ,e size) ,off))
+        (SCM_FLONUM_ENSURE_MEM VAL0)
+        (let* ([,box (ENV-DATA ,e ,off)])
+          (VM_ASSERT (SCM_BOXP ,box))
+          (SCM_BOX_SET ,box VAL0))
+        (set! (-> vm numVals) 1)
+        NEXT))])
+
 ;;
 ;; ($lrefNN depth offset)
 ;;   Generate an expr to refer to the local variable, starting from ENV.
-;;
+;;   DEPTH *must* be a literal integer.  We unroll the loop.
 (define-cise-stmt $lrefNN
   [(_ depth offset)
    (let1 v (gensym)
@@ -413,14 +451,24 @@
     ($include "./vmcall.c")
     (label tail_apply_entry)
     ($define APPLY_CALL)
-    ($include "./vmcall.c")))
+    ($include "./vmcall.c"))
+  :multi-value)
 
 ;; TAIL-CALL(nargs)
 ;;  Call procedure in val0.  Same as CALL except this discards the
 ;;  caller's argument frame and shift the callee's argument frame.
 ;;
 (define-insn TAIL-CALL 1 none #f
-  (begin (DISCARD-ENV) ($goto-insn CALL)))
+  (begin
+    (let* ([ct::ScmCallTrace* (-> vm callTrace)])
+      (when (!= ct NULL)
+        (set! (ref (aref (-> ct entries) (-> ct top)) base) BASE)
+        (set! (ref (aref (-> ct entries) (-> ct top)) pc) PC)
+        (set! (-> ct top) (logand (+ (-> ct top) 1)
+                                  (- (-> ct size) 1)))))
+    (DISCARD-ENV)
+    ($goto-insn CALL))
+  :multi-value)
 
 ;; JUMP <addr>
 ;;  Jump to <addr>.
@@ -432,7 +480,8 @@
 ;;  Pop the continuation stack.
 ;;
 (define-insn RET       0 none #f
-  (begin (RETURN-OP) CHECK-INTR NEXT))
+  (begin (RETURN-OP) CHECK-INTR NEXT)
+  :multi-value)
 
 ;; DEFINE(flag) <symbol>
 ;;  Defines global binding of SYMBOL in the current module.
@@ -554,7 +603,8 @@
     (CHECK-STACK (-> vm base maxstack))
     CHECK-INTR
     (SCM_PROF_COUNT_CALL vm (SCM_OBJ (-> vm base)))
-    NEXT))
+    NEXT)
+  :multi-value)
 
 (define-insn LOCAL-ENV-TAIL-CALL 1 none #f
   (let* ([nargs::int (cast int (- SP ARGP))] [to::ScmObj*])
@@ -568,7 +618,8 @@
           (set! (* (post++ t)) (* (post++ a))))))
     (set! ARGP to)
     (set! SP (+ to nargs))
-    ($goto-insn LOCAL-ENV-CALL)))
+    ($goto-insn LOCAL-ENV-CALL))
+  :multi-value)
 
 ;; BF <else-offset>          ; branch if VAL0 is false
 ;; BT <else-offset>          ; branch if VAL0 is true
@@ -583,8 +634,8 @@
 ;;   Conditional branches.
 ;;   The combined operations leave the boolean value of the test result
 ;;   in VAL0.
-(define-insn BF      0 addr #f ($branch (SCM_FALSEP VAL0)))
-(define-insn BT      0 addr #f ($branch (not (SCM_FALSEP VAL0))))
+(define-insn BF      0 addr #f ($branch (SCM_CHECKED_FALSEP VAL0)))
+(define-insn BT      0 addr #f ($branch (not (SCM_CHECKED_FALSEP VAL0))))
 (define-insn BNEQ    0 addr #f ($w/argp z ($branch* (not (SCM_EQ VAL0 z)))))
 (define-insn BNEQV   0 addr #f ($w/argp z ($branch* (not (Scm_EqvP VAL0 z)))))
 (define-insn BNNULL  0 addr #f ($branch* (not (SCM_NULLP VAL0))))
@@ -630,8 +681,8 @@
 ;; RNEQ
 ;; RNEQV
 ;;   Conditional returns.
-(define-insn RF          0 none #f ($retc (SCM_FALSEP VAL0)))
-(define-insn RT          0 none #f ($retc (not (SCM_FALSEP VAL0))))
+(define-insn RF          0 none #f ($retc (SCM_CHECKED_FALSEP VAL0)))
+(define-insn RT          0 none #f ($retc (not (SCM_CHECKED_FALSEP VAL0))))
 (define-insn RNEQ        0 none #f ($w/argp v
                                      ($retc* (not (SCM_EQ VAL0 v)))))
 (define-insn RNEQV       0 none #f ($w/argp v
@@ -652,6 +703,9 @@
       (when (and (not restarg) (> (-> vm numVals) reqargs))
         ($vm-err "received more values than expected"))
       (set! argsize (+ reqargs (?: restarg 1 0)))
+      (cast void argsize) ; suppress unused variable warning
+      (cast void nextpc) ; suppress unused variable warning
+      (cast void size) ; suppress unused variable warning
       ,@stmts
       (cond [(> reqargs 0) (PUSH-ARG VAL0) (post++ i)]
             [(and restarg (> (-> vm numVals) 0))
@@ -721,31 +775,20 @@
 (define-insn VALUES-N 0 none #f
   (begin
     (VM-ASSERT ENV)
-    (let* ([nvals::int (cast int (-> ENV size))] [v])
+    (let* ([nvals::int (cast int (-> ENV size))])
       (set! (-> vm numVals) nvals)
       (for [() (> nvals 1) (post-- nvals)]
            (POP-ARG (aref (-> vm vals) (- nvals 1))))
       (POP-ARG VAL0)
-      NEXT)))
+      NEXT))
+  :multi-value)
 
 ;; LSET(depth, offset)
 ;;  Local set
 ;;
 (define-insn LSET        2 none #f
-  (let* ([dep::int (SCM_VM_INSN_ARG0 code)]
-         [off::int (SCM_VM_INSN_ARG1 code)]
-         [e::ScmEnvFrame* ENV])
-    (for [() (> dep 0) (post-- dep)]
-         (VM-ASSERT (!= e NULL))
-         (set! e (-> e up)))
-    (VM-ASSERT (!= e NULL))
-    (VM-ASSERT (> (-> e size) off))
-    (SCM_FLONUM_ENSURE_MEM VAL0)
-    (let* ([box (ENV-DATA e off)])
-      (VM_ASSERT (SCM_BOXP box))
-      (SCM_BOX_SET box VAL0))
-    (set! (-> vm numVals) 1)
-    NEXT))
+  ($lset (SCM_VM_INSN_ARG0 code)        ; depth
+         (SCM_VM_INSN_ARG1 code)))      ; offset
 
 ;; GSET <location>
 ;;  LOCATION may be a symbol or gloc
@@ -790,12 +833,8 @@
 ;;  Retrieve local value.
 ;;
 (define-insn LREF        2 none #f
-  (let* ([dep::int (SCM_VM_INSN_ARG0 code)]
-         [off::int (SCM_VM_INSN_ARG1 code)]
-         [e::ScmEnvFrame* ENV])
-    (for [() (> dep 0) (post-- dep)]
-         (set! e (-> e up)))
-    ($result (ENV-DATA e off))))
+  ($lref (SCM_VM_INSN_ARG0 code)   ; depth
+         (SCM_VM_INSN_ARG1 code))) ; offset 
 
 ;; Shortcut for the frequent depth/offset.
 ;; From statistics, we found that the following depth/offset combinations
@@ -828,6 +867,8 @@
 (define-insn LREF21-PUSH 0 none  (LREF21 PUSH))
 (define-insn LREF30-PUSH 0 none  (LREF30 PUSH))
 
+(define-insn-lref* LREF-RET 0 none (LREF RET))
+
 ;; GREF <location>
 ;;  LOCATION may be a symbol or GLOC object.
 ;;  Retrieve global value in the current module.
@@ -848,10 +889,6 @@
 (define-insn PUSH-GREF-CALL 1 obj   (PUSH GREF CALL))
 (define-insn PUSH-GREF-TAIL-CALL 1 obj (PUSH GREF TAIL-CALL))
 
-(define-insn LREF0-PUSH-GREF 0 obj  (LREF0 PUSH GREF) #f :obsoleted)
-(define-insn LREF0-PUSH-GREF-CALL 1 obj (LREF0 PUSH GREF CALL) #f :obsoleted)
-(define-insn LREF0-PUSH-GREF-TAIL-CALL 1 obj (LREF0 PUSH GREF TAIL-CALL) #f :obsoleted)
-
 ;; PROMISE
 ;;  Delay syntax emits this instruction.  Wrap a procedure into a promise
 ;;  object.
@@ -876,7 +913,8 @@
                   (aref (-> vm vals) (- SCM_VM_MAX_VALUES 1)))
         (break))
       (PUSH-ARG (aref (-> vm vals) i)))
-    ($goto-insn TAIL-CALL)))
+    ($goto-insn TAIL-CALL))
+  :multi-value)
 
 ;; Inlined operators
 ;;  They work the same as corresponding Scheme primitives, but they are
@@ -970,7 +1008,7 @@
        (set! cp (Scm_Append args))])
     ($result cp)))
 
-(define-insn NOT      0 none #f ($w/argr v ($result:b (SCM_FALSEP v))))
+(define-insn NOT      0 none #f ($w/argr v ($result:b (SCM_CHECKED_FALSEP v))))
 (define-insn REVERSE  0 none #f ($w/argr v ($result (Scm_Reverse v))))
 
 (define-insn APPLY       1 none #f
@@ -985,7 +1023,8 @@
     (POP-ARG VAL0)                      ; get proc
     (TAIL-CALL-INSTRUCTION)
     (set! VAL0 (Scm_VMApply VAL0 cp))
-    NEXT))
+    NEXT)
+  :multi-value)
 
 (define-insn TAIL-APPLY  1 none #f
   ;; Inlined apply.  Assumes the call is at the tail position.
@@ -1003,36 +1042,37 @@
   ;;
   ;; where N = SCMVM_INSN_ARG(code)-2.
   ;; rest contains a list of "all the rest arguments".
+  ;; We just push VAL0 onto stack, and move ARGP 
   ;; We "rotate" the stack and VAL0 and jump to the
-  ;; TAIL-CALL processing.  (The unfolding of rest
+  ;; tail_apply_entry.  (The unfolding of rest
   ;; argument will be done in ADJUST_ARGUMENT_FRAME later,
   ;; if necessary.)
   ;;
   ;;   SP  >|      |
-  ;;        | rest |
+  ;;        | rest |< original SP position
   ;;        | argN |
   ;;        |   :  |
   ;;   ARGP>| arg0 |        VAL0=proc
-  ;;
+  ;;        | proc |< original ARGP postiion (proc unused) 
   (let* ([rest VAL0]
-         [rargc::int (check_arglist_tail_for_apply vm rest)]
          [nargc::int (- (SCM_VM_INSN_ARG code) 2)]
          [proc (* (- SP nargc 1))])
-    (when (< rargc 0) ($vm-err "improper list not allowed: %S" rest))
-    (while (> nargc 0)
-      (set! (* (- SP nargc 1)) (* (- SP nargc)))
-      (post-- nargc))
-    ;; a micro-optimization: if VAL0 is (), we just omit it and
-    ;; pretend this is a normal TAIL-CALL.
-    (when (== rargc 0)
-      (post-- SP)
-      (set! VAL0 proc)
-      ($goto-insn TAIL-CALL))
-    ;; normal path
-    (set! (* (- SP 1)) rest)
     (set! VAL0 proc)
+    (post++ ARGP)
+    ;; a micro-optimization: if the rest arg is (), we just omit it and
+    ;; pretend this is a normal TAIL-CALL.
+    (when (SCM_NULLP rest) ($goto-insn TAIL-CALL))
+    ;; Checking if rest argument is proper is done after tail_apply_entry.
+    ;; Here we just ensure that rest is a normal pair (If rest is a lazy
+    ;; pair, it is forced by SCM_PAIRP).
+    (unless (SCM_PAIRP rest)
+      ($vm-err "Rest argument is not proper: %S" rest))
+    (set! VAL0 proc)  ;VAL0 may be broken by lazy pair evaluation
+    ;; normal path
+    (set! (* (post++ SP)) rest)
     (DISCARD-ENV)
-    (goto tail_apply_entry)))
+    (goto tail_apply_entry))
+  :multi-value)
 
 (define-insn IS-A        0 none #f      ; is-a?
   ($w/argp obj
@@ -1051,14 +1091,14 @@
 (define-insn STRINGP     0 none #f ($w/argr v ($result:b (SCM_STRINGP v))))
 (define-insn SYMBOLP     0 none #f ($w/argr v ($result:b (SCM_SYMBOLP v))))
 (define-insn VECTORP     0 none #f ($w/argr v ($result:b (SCM_VECTORP v))))
-(define-insn IDENTIFIERP 0 none #f ($w/argr v ($result:b (SCM_IDENTIFIERP v))))
 (define-insn NUMBERP     0 none #f ($w/argr v ($result:b (SCM_NUMBERP v))))
 (define-insn REALP       0 none #f ($w/argr v ($result:b (SCM_REALP v))))
+(define-insn IDENTIFIERP 0 none #f ($w/argr v ($result:b (or (SCM_SYMBOLP v) (SCM_IDENTIFIERP v)))))
 
 (define-insn SETTER      0 none #f ($w/argr v ($result (Scm_Setter v))))
 
-(define-insn VALUES      1 none #f (begin ($values) NEXT))
-(define-insn VALUES-RET  1 none (VALUES RET) (begin ($values) (RETURN-OP) NEXT))
+(define-insn VALUES      1 none #f (begin ($values) NEXT) :multi-value)
+(define-insn VALUES-RET  1 none (VALUES RET) (begin ($values) (RETURN-OP) NEXT) :multi-value)
 
 (define-insn VEC         1 none #f      ; vector
   (let* ([nargs::int (SCM_VM_INSN_ARG code)]
@@ -1111,6 +1151,8 @@
     (POP-ARG vec)
     ($type-check vec SCM_VECTORP "vector")
     ($type-check ind SCM_INTP "fixnum")
+    (when (SCM_VECTOR_IMMUTABLE_P vec)
+      ($vm-err "vector is immutable: %S" vec))
     (let* ([k::int (SCM_INT_VALUE ind)] [v VAL0])
       (when (or (< k 0) (>= k (SCM_VECTOR_SIZE vec)))
         ($vm-err "vector-set! index out of range: %d" k))
@@ -1130,6 +1172,8 @@
 (define-insn VEC-SETI    1 none #f
   ($w/argp vec
     ($type-check vec SCM_VECTORP "vector")
+    (when (SCM_VECTOR_IMMUTABLE_P vec)
+      ($vm-err "vector is immutable: %S" vec))
     (let* ([k::int (SCM_VM_INSN_ARG code)] [v VAL0])
       (when (or (< k 0) (>= k (SCM_VECTOR_SIZE vec)))
         ($vm-err "vector-set! index out of range: %d" k))
@@ -1198,10 +1242,25 @@
     ;; we take a shortcut if either one is flonum and the
     ;; other is real.  (if both are integers, the overflow check
     ;; would be cumbersome so we just call Scm_Mul).
-    (if (or (and (SCM_FLONUMP arg) (SCM_REALP VAL0))
-            (and (SCM_FLONUMP VAL0) (SCM_REALP arg)))
-      ($result:f (* (Scm_GetDouble arg) (Scm_GetDouble VAL0)))
-      ($result (Scm_Mul arg VAL0)))))
+    ;; NB: If one arg is inexact real and another arg is exact zero,
+    ;; the result should be an exact zero.
+    (if (SCM_FLONUMP arg)
+      (cond [(and (== (SCM_MAKE_INT 0) VAL0)
+                  (not (SCM_IS_INF (SCM_FLONUM_VALUE arg)))
+                  (not (SCM_IS_NAN (SCM_FLONUM_VALUE arg))))
+             ($result (SCM_MAKE_INT 0))]
+            [(SCM_REALP VAL0)
+             ($result:f (* (Scm_GetDouble arg) (Scm_GetDouble VAL0)))]
+            [else ($result (Scm_Mul arg VAL0))])
+      (if (SCM_FLONUMP VAL0)
+        (cond [(and (== (SCM_MAKE_INT 0) arg) 
+                    (not (SCM_IS_INF (SCM_FLONUM_VALUE VAL0)))
+                    (not (SCM_IS_NAN (SCM_FLONUM_VALUE VAL0))))
+               ($result (SCM_MAKE_INT 0))]
+              [(SCM_REALP arg)
+               ($result:f (* (Scm_GetDouble arg) (Scm_GetDouble VAL0)))]
+              [else ($result (Scm_Mul arg VAL0))])
+        ($result (Scm_Mul arg VAL0))))))
 
 (define-insn NUMDIV2 0 none #f          ; / (binary)
   ($w/argp arg
@@ -1263,6 +1322,14 @@
              ($result:f (- (cast double imm) (SCM_FLONUM_VALUE arg)))]
             [else           ($result (Scm_Sub (SCM_MAKE_INT imm) arg))]))))
 
+
+(define-insn NUMMODI      1 none #f
+  (let* ([divisor::ScmSmallInt (SCM_VM_INSN_ARG code)])
+    ($w/argr arg ($result (Scm_Modulo arg (SCM_MAKE_INT divisor) FALSE)))))
+(define-insn NUMREMI      1 none #f
+  (let* ([divisor::ScmSmallInt (SCM_VM_INSN_ARG code)])
+    ($w/argr arg ($result (Scm_Modulo arg (SCM_MAKE_INT divisor) TRUE)))))
+
 (define-insn ASHI         1 none #f
   (let* ([cnt::ScmSmallInt (SCM_VM_INSN_ARG code)])
     ($w/argr arg ($result (Scm_Ash arg cnt)))))
@@ -1353,9 +1420,9 @@
       (SCM_FLONUM_ENSURE_MEM VAL0)
       ($result (Scm_VMSlotSet obj slot VAL0)))))
 
-;;;
-;;; Additional instructions
-;;;
+;;
+;; Dynamic handlers
+;;
 
 ;; PUSH-HANDLERS
 ;; POP-HANDLERS
@@ -1377,11 +1444,9 @@
     NEXT))
 
 ;;
-;; Experimental
+;; Environment
 ;;
 
-(define-insn-lref* LREF-RET 0 none (LREF RET))
-
 ;; if param == 0, VAL0 <- box(VAL0)
 ;; else if param > 0,  ENV[param-1] <- box(ENV[param-1])
 ;; The second case is for arguments that are mutated.
@@ -1431,12 +1496,22 @@
     (local_env_shift vm (SCM_VM_INSN_ARG code))
     NEXT))
 
-;; TRANSIENT: These are better to be with ASHI; we put here to keep
-;; binary compatibility.  Should be moved on 1.0 release.
-(define-insn NUMMODI      1 none #f
-  (let* ([divisor::ScmSmallInt (SCM_VM_INSN_ARG code)])
-    ($w/argr arg ($result (Scm_Modulo arg (SCM_MAKE_INT divisor) FALSE)))))
-(define-insn NUMREMI      1 none #f
-  (let* ([divisor::ScmSmallInt (SCM_VM_INSN_ARG code)])
-    ($w/argr arg ($result (Scm_Modulo arg (SCM_MAKE_INT divisor) TRUE)))))
+;; TRANSIENT: TODO: Move these up below LREF on 1.0 release
 
+;; XLREF(offset) depth
+;;   LREF, but used when depth and/or offset can't fit in the params field.
+(define-insn XLREF 1 obj #f
+  (let* ([dep_s])
+    (FETCH-OPERAND dep_s)
+    INCR-PC
+    ($lref (SCM_INT_VALUE dep_s)          ;depth
+           (SCM_VM_INSN_ARG code))))      ;offset
+
+;; XLSET(offset) depth
+;;   LSET, but used when depth and/or offset can't fit in the params field.
+(define-insn XLSET 1 obj #f
+  (let* ([dep_s])
+    (FETCH-OPERAND dep_s)
+    INCR-PC
+    ($lset (SCM_INT_VALUE dep_s)        ; depth
+           (SCM_VM_INSN_ARG code))))    ; offset
