@@ -1,7 +1,7 @@
 /*
  * weak.c - weak vectors and tables
  *
- *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -63,7 +63,7 @@ static void weakvector_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
     SCM_PUTC(')', port);
 }
 
-static void weakvector_finalize(ScmObj obj, void *data)
+static void weakvector_finalize(ScmObj obj, void *data SCM_UNUSED)
 {
     ScmWeakVector *v = SCM_WEAK_VECTOR(obj);
     ScmObj *p = (ScmObj*)v->pointers;
@@ -142,7 +142,7 @@ ScmObj Scm_WeakVectorSet(ScmWeakVector *v, ScmSmallInt index, ScmObj value)
 /* ptr points to the target object weakly.
    Registered flag becomes TRUE whenever ptr points to a GC_malloced object,
    thus &wbox->ptr is registered as a disappearing link.
-   Note that we can distinguish a box that contaning NULL pointer, and
+   Note that we can distinguish a box that containing NULL pointer, and
    a box whose target has been GCed and hence ptr is cleared---in the
    former case registered is FALSE, while in the latter case it is TRUE. */
 struct ScmWeakBoxRec {
@@ -219,7 +219,8 @@ void *Scm_WeakBoxRef(ScmWeakBox *wbox)
 #define MARK_GONE_ENTRY(ht, e)  (ht->goneEntries++)
 
 
-static void weakhash_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
+static void weakhash_print(ScmObj obj, ScmPort *port, 
+                           ScmWriteContext *ctx SCM_UNUSED)
 {
     ScmWeakHashTable *ht = SCM_WEAK_HASH_TABLE(obj);
     char *type = "";
