@@ -1,7 +1,7 @@
 /*
  * write.c - writer
  *
- *   Copyright (c) 2000-2018  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2020  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -39,6 +39,7 @@
 #include "gauche/priv/macroP.h" /* PVREF stuff.  Will go in future */
 #include "gauche/priv/writerP.h"
 #include "gauche/priv/portP.h"
+#include "gauche/priv/stringP.h"
 #include "gauche/char_attr.h"
 
 #include <ctype.h>
@@ -83,7 +84,7 @@ static const ScmWriteControls *defaultWriteControls;
    (SCM_VM_RUNTIME_FLAG_IS_SET(Scm_VM(), SCM_CASE_FOLD)? \
     SCM_WRITE_CASE_FOLD:SCM_WRITE_CASE_NOFOLD)
 
-/* Whether we need 'walk' pass to find out shared and/or ciruclar
+/* Whether we need 'walk' pass to find out shared and/or circular
    structure.  Now we use two-pass writing by default, and use one-pass
    writing only when requested specifically. */
 #define WRITER_NEED_2PASS(ctx) (SCM_WRITE_MODE(ctx) != SCM_WRITE_SIMPLE)
@@ -151,7 +152,7 @@ const ScmWriteControls *Scm_GetWriteControls(ScmWriteContext *ctx,
 /* The class definition is in libio.scm  */
 
 /* NB: For the time being, proto argument is ignored. */
-ScmWriteState *Scm_MakeWriteState(ScmWriteState *proto)
+ScmWriteState *Scm_MakeWriteState(ScmWriteState *proto SCM_UNUSED)
 {
     ScmWriteState *z = SCM_NEW(ScmWriteState);
     SCM_SET_CLASS(z, SCM_CLASS_WRITE_STATE);
@@ -166,9 +167,9 @@ ScmWriteState *Scm_MakeWriteState(ScmWriteState *proto)
 static void cleanup_port_write_state(ScmPort *port)
 {
     port->flags &= ~(SCM_PORT_WALKING|SCM_PORT_WRITESS);
-    if (port->writeState != NULL) {
-        ScmWriteState *s = port->writeState;
-        port->writeState = NULL;
+    ScmWriteState *s = Scm_PortWriteState(port);
+    if (s != NULL) {
+        Scm_PortWriteStateSet(port, NULL);
         /* The table for recursive/shared detection should be GC'ed after
            we drop the reference to it.  However, the table can be quite big
            after doing write-shared on a large graph, and our implementation of
@@ -260,7 +261,7 @@ void Scm_WriteWithControls(ScmObj obj, ScmObj p, int mode,
  *
  *  Characters exceeding WIDTH are truncated.
  *  If the output fits within WIDTH, # of characters actually written
- *  is returned.  Othewise, -1 is returned.
+ *  is returned.  Otherwise, -1 is returned.
  *
  *  Currently this API is only used from Scm_Printf, for 'format' has been
  *  moved to libfmt.scm.  I don't like the way this is implemented and would
@@ -283,7 +284,7 @@ int Scm_WriteLimited(ScmObj obj, ScmObj p, int mode, int width)
     }
 
     ScmObj out = Scm_MakeOutputStringPort(TRUE);
-    SCM_PORT(out)->writeState = SCM_PORT(port)->writeState;
+    Scm_PortWriteStateSet(SCM_PORT(out), Scm_PortWriteState(port));
     ScmWriteContext ctx;
     write_context_init(&ctx, mode, 0, width);
 
@@ -337,14 +338,16 @@ static void write_general(ScmObj obj, ScmPort *out, ScmWriteContext *ctx)
    write-object.   We can't use VMApply here since this function can be
    called deep in the recursive stack of Scm_Write, so the control
    may not return to VM immediately. */
-static void write_object(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
+static void write_object(ScmObj obj, ScmPort *port, 
+                         ScmWriteContext *ctx SCM_UNUSED)
 {
     Scm_ApplyRec(SCM_OBJ(&Scm_GenericWriteObject),
                  SCM_LIST2(obj, SCM_OBJ(port)));
 }
 
 /* Default method for write-object */
-static ScmObj write_object_fallback(ScmObj *args, int nargs, ScmGeneric *gf)
+static ScmObj write_object_fallback(ScmObj *args, int nargs,
+                                    ScmGeneric *gf SCM_UNUSED)
 {
     if (nargs != 2 || (nargs == 2 && !SCM_OPORTP(args[1]))) {
         Scm_Error("No applicable method for write-object with %S",
@@ -384,7 +387,7 @@ static size_t write_char(ScmChar ch, ScmPort *port, ScmWriteContext *ctx)
 
         Scm_PutzUnsafe("#\\", -1, port);
         if (ch <= 0x20)       cname = char_names[ch];
-        else if (ch == 0x7f)  cname = "del";
+        else if (ch == 0x7f)  cname = "delete";
         else {
             switch (Scm_CharGeneralCategory(ch)) {
             case SCM_CHAR_CATEGORY_Mn:
@@ -425,7 +428,8 @@ static size_t write_char(ScmChar ch, ScmPort *port, ScmWriteContext *ctx)
  */
 ScmObj Scm__WritePrimitive(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
 {
-    const ScmWriteControls *wp = Scm_GetWriteControls(ctx, port->writeState);
+    const ScmWriteControls *wp = 
+        Scm_GetWriteControls(ctx, Scm_PortWriteState(port));
     
 #define CASE_ITAG_RET(obj, str)                 \
     case SCM_ITAG(obj):                         \
@@ -439,7 +443,10 @@ ScmObj Scm__WritePrimitive(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
             CASE_ITAG_RET(SCM_NIL,       "()");
             CASE_ITAG_RET(SCM_EOF,       "#<eof>");
             CASE_ITAG_RET(SCM_UNDEFINED, "#<undef>");
+            /* The following two values should never be seen by the user; but
+               internal bugs may reveal them. */
             CASE_ITAG_RET(SCM_UNBOUND,   "#<unbound>");
+            CASE_ITAG_RET(SCM_UNINITIALIZED, "#<uninitialized>");
         default:
             Scm_Panic("write: unknown itag object: %08x", SCM_WORD(obj));
         }
@@ -471,6 +478,13 @@ ScmObj Scm__WritePrimitive(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
         Scm_PutzUnsafe(buf, -1, port);
         return SCM_MAKE_INT(k);
     }
+    else if (SCM_STRING_CURSOR_SMALL_P(obj)) {
+        char buf[SPBUFSIZ];
+        int k = snprintf(buf, SPBUFSIZ, "#<string-cursor %ld>",
+			 SCM_STRING_CURSOR_SMALL_OFFSET(obj));
+        Scm_PutzUnsafe(buf, -1, port);
+        return SCM_MAKE_INT(k);
+    }
     return SCM_FALSE;
 }
 
@@ -527,8 +541,9 @@ ScmObj Scm__WritePrimitive(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
 static void write_walk(ScmObj obj, ScmPort *port)
 {
     static ScmObj proc = SCM_UNDEFINED;
-    SCM_ASSERT(port->writeState);
-    ScmHashTable *ht = port->writeState->sharedTable;
+    ScmWriteState *s = Scm_PortWriteState(port);
+    SCM_ASSERT(s);
+    ScmHashTable *ht = s->sharedTable;
     SCM_ASSERT(ht != NULL);
     SCM_BIND_PROC(proc, "%write-walk-rec", Scm_GaucheInternalModule());
     Scm_ApplyRec3(proc, obj, SCM_OBJ(port), SCM_OBJ(ht));
@@ -550,7 +565,7 @@ static void write_rec(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
 {
     char numbuf[50];  /* enough to contain long number */
     ScmObj stack = SCM_NIL;
-    ScmWriteState *st = port->writeState;
+    ScmWriteState *st = Scm_PortWriteState(port);
     ScmHashTable *ht = (st? st->sharedTable : NULL);
     const ScmWriteControls *wp = Scm_GetWriteControls(ctx, st);
     int stack_depth = 0;        /* only used when !ht */
@@ -584,7 +599,7 @@ static void write_rec(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
     for (;;) {
     write1:
         if (ctx->flags & WRITE_LIMITED) {
-            if (port->src.ostr.length >= ctx->limit) return;
+            if (PORT_OSTR(port)->length >= ctx->limit) return;
         }
 
         /* number may be heap allocated, but we don't use srfi-38 notation. */
@@ -612,7 +627,7 @@ static void write_rec(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
                 goto next;
             } else if (k > 1) {
                 /* This object will be seen again. Put a reference tag. */
-                ScmWriteState *s = port->writeState;
+                ScmWriteState *s = Scm_PortWriteState(port);
                 snprintf(numbuf, 50, "#%d=", s->sharedCounter);
                 Scm_HashTableSet(ht, obj, SCM_MAKE_INT(-s->sharedCounter), 0);
                 s->sharedCounter++;
@@ -705,7 +720,7 @@ static void write_rec(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
                 } else {
                     Scm_PutcUnsafe(' ', port);
                     obj = SCM_VECTOR_ELEMENT(v, i);
-                    SCM_SET_CAR(top, SCM_MAKE_INT(i+1));
+                    SCM_SET_CAR_UNCHECKED(top, SCM_MAKE_INT(i+1));
                     goto write1;
                 }
             } else {
@@ -731,8 +746,9 @@ static void write_rec(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
                         POP();
                     } else {
                         obj = v;
-                        SCM_SET_CAR(SCM_CDR(top), SCM_MAKE_INT(count+1));
-                        SCM_SET_CDR(SCM_CDR(top), SCM_NIL);
+                        SCM_SET_CAR_UNCHECKED(SCM_CDR(top), 
+                                              SCM_MAKE_INT(count+1));
+                        SCM_SET_CDR_UNCHECKED(SCM_CDR(top), SCM_NIL);
                         goto write1;
                     }
                 } else if (wp->printLength >= 0 && wp->printLength <= count) {
@@ -743,14 +759,14 @@ static void write_rec(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
                     /* cdr part is shared */
                     Scm_PutzUnsafe(" . ", -1, port);
                     obj = v;
-                    SCM_SET_CAR(SCM_CDR(top), SCM_MAKE_INT(count+1));
-                    SCM_SET_CDR(SCM_CDR(top), SCM_NIL);
+                    SCM_SET_CAR_UNCHECKED(SCM_CDR(top), SCM_MAKE_INT(count+1));
+                    SCM_SET_CDR_UNCHECKED(SCM_CDR(top), SCM_NIL);
                     goto write1;
                 } else {
                     Scm_PutcUnsafe(' ', port);
                     obj = SCM_CAR(v);
-                    SCM_SET_CAR(SCM_CDR(top), SCM_MAKE_INT(count+1));
-                    SCM_SET_CDR(SCM_CDR(top), SCM_CDR(v));
+                    SCM_SET_CAR_UNCHECKED(SCM_CDR(top), SCM_MAKE_INT(count+1));
+                    SCM_SET_CDR_UNCHECKED(SCM_CDR(top), SCM_CDR(v));
                     goto write1;
                 }
             }
@@ -770,7 +786,7 @@ static void write_rec(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
    by the caller even if we throw an error during write. */
 static void write_ss(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
 {
-    SCM_ASSERT(port->writeState == NULL);
+    SCM_ASSERT(Scm_PortWriteState(port) == NULL);
 
     /* pass 1 */
     port->flags |= SCM_PORT_WALKING;
@@ -778,7 +794,7 @@ static void write_ss(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
     ScmWriteState *s = Scm_MakeWriteState(NULL);
     s->sharedTable = SCM_HASH_TABLE(Scm_MakeHashTableSimple(SCM_HASH_EQ, 0));
     s->controls = ctx->controls;
-    port->writeState = s;
+    Scm_PortWriteStateSet(port, s);
 
     write_walk(obj, port);
     port->flags &= ~(SCM_PORT_WALKING|SCM_PORT_WRITESS);
@@ -797,7 +813,10 @@ static void write_ss(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
 
 /*OBSOLETED*/
 /*format is now in Scheme (libfmt.scm).*/
-void Scm_Format(ScmPort *out, ScmString *fmt, ScmObj args, int sharedp)
+void Scm_Format(ScmPort *out SCM_UNUSED,
+                ScmString *fmt SCM_UNUSED,
+                ScmObj args SCM_UNUSED,
+                int sharedp SCM_UNUSED)
 {
     Scm_Error("Scm_Format is obsoleted");
 }
@@ -833,11 +852,6 @@ void Scm_Format(ScmPort *out, ScmString *fmt, ScmObj args, int sharedp)
  *  move away from %A in future.
  */
 
-struct vprintf_ctx {
-    const char *fmt;
-    ScmObj args;
-};
-
 /* NB: Scm_Vprintf scans format string twice.  In the first pass, arguments
  * are retrieved from va_list variable and pushed to a list.  In the second
  * pass, they are printed according to the format string.
@@ -849,8 +863,12 @@ struct vprintf_ctx {
 /* Pass 1.  Pop vararg and make a list of arguments.
  * NB: If we're "walking" pass, and the argument is a Lisp object,
  * we recurse to it in this pass.
+ * NB: Without NOINLINE, gcc inlines this in Scm_Vprintf which causes warning
+ * of register variable clobbering by longjmp.
  */
-static ScmObj vprintf_pass1(ScmPort *out, const char *fmt, va_list ap)
+static ScmObj SCM_NOINLINE vprintf_pass1(ScmPort *out,
+                                         const char *fmt,
+                                         va_list ap)
 {
     ScmObj h = SCM_NIL, t = SCM_NIL;
     const char *fmtp = fmt;
@@ -1105,11 +1123,12 @@ static void vprintf_pass2(ScmPort *out, const char *fmt, ScmObj args)
 
 /* Public APIs */
 
-void Scm_Vprintf(ScmPort *out, const char *fmt, va_list ap, int sharedp)
+void Scm_Vprintf(ScmPort *out, const char *fmt, va_list ap, 
+                 int sharedp SCM_UNUSED)
 {
     if (!SCM_OPORTP(out)) Scm_Error("output port required, but got %S", out);
 
-    /* TOOD: handle sharedp */
+    /* TODO: handle sharedp */
     ScmObj args = vprintf_pass1(out, fmt, ap);
 
     ScmVM *vm = Scm_VM();
