@@ -11,9 +11,8 @@
 
 (test-section "testing with dbd-null")
 
-(let ((conn #f)
-      (query #f)
-      )
+(let ([conn #f]
+      [query #f])
   (test* "dbi-connect" '<null-connection>
          (begin (set! conn (dbi-connect "dbi:null"))
                 (and (dbi-open? conn)
@@ -74,8 +73,8 @@
 (test-section "testing conditions")
 
 (test* "<dbi-nonexistent-driver-error>" "nosuchdriver"
-       (guard (e ((<dbi-nonexistent-driver-error> e)
-                  (ref e 'driver-name)))
+       (guard (e [(<dbi-nonexistent-driver-error> e)
+                  (ref e 'driver-name)])
          (dbi-connect "dbi:nosuchdriver")))
 
 
