@@ -90,45 +90,75 @@
   )
 
 ;;-------------------------------------------------------------------
-(test-section "string-pointer")
-(define sp #f)
-(test* "make-string-pointer" #t
-       (begin (set! sp (make-string-pointer "Ϥhoؤ"))
-              (string-pointer? sp)))
-(test* "string-pointer-next!" #\ (string-pointer-next! sp))
-(test* "string-pointer-next!" #\ (string-pointer-next! sp))
-(test* "string-pointer-prev!" #\ (string-pointer-prev! sp))
-(test* "string-pointer-prev!" #\ (string-pointer-prev! sp))
-(test* "string-pointer-prev!" #t (eof-object? (string-pointer-prev! sp)))
-(test* "string-pointer-index" 0 (string-pointer-index sp))
-(test* "string-pointer-index" 8
-       (do ((x (string-pointer-next! sp) (string-pointer-next! sp)))
-           ((eof-object? x) (string-pointer-index sp))))
-(test* "string-pointer-substring" '("Ϥhoؤ" "")
-       (list (string-pointer-substring sp)
-             (string-pointer-substring sp :after #t)))
-(test* "string-pointer-substring" '("Ϥh" "oؤ")
-       (begin
-         (string-pointer-set! sp 5)
-         (list (string-pointer-substring sp)
-               (string-pointer-substring sp :after #t))))
-(test* "string-pointer-substring" '("" "Ϥhoؤ")
-       (begin
-         (string-pointer-set! sp 0)
-         (list (string-pointer-substring sp)
-               (string-pointer-substring sp :after #t))))
+(test-section "string-cursor")
 
-;; torturing backward pointer movement
-(define sp (make-string-pointer "a" -1))
-(test* "string-pointer-prev!" #\a (string-pointer-prev! sp))
-(test* "string-pointer-prev!" #\ (string-pointer-prev! sp))
-(test* "string-pointer-prev!" #t (eof-object? (string-pointer-prev! sp)))
-
-(define sp (make-string-pointer "ޤa" -1)) ;;dreaded jisx0201 kana
-(test* "string-pointer-prev!" #\a (string-pointer-prev! sp))
-(test* "string-pointer-prev!" #\ (string-pointer-prev! sp))
-(test* "string-pointer-prev!" #\ (string-pointer-prev! sp))
-(test* "string-pointer-prev!" #\ (string-pointer-prev! sp))
+(define str "Ϥhoؤ")
+(define ss #f)
+(define se #f)
+(test* "string-cursor-start" #t
+       (begin
+         (set! ss (string-cursor-start str))
+         (string-cursor? ss)))
+(test* "string-cursor-start" #t
+       (begin
+         (set! se (string-cursor-end str))
+         (string-cursor? se)))
+(test* "string-cursor-next" #\
+       (string-ref str (string-cursor-next str ss)))
+(test* "string-cursor-forward" #\
+       (string-ref str (string-cursor-forward str ss 3)))
+(test* "string-cursor-prev" #\
+       (string-ref str (string-cursor-prev str se)))
+(test* "string-cursor-back" #\
+       (string-ref str (string-cursor-back str se 5)))
+(test* "string-index->cursor" #\
+       (string-ref str (string-index->cursor str 6)))
+(test* "string-cursor->index" 0
+       (string-cursor->index str ss))
+(test* "string-cursor->index" 8
+       (string-cursor->index str se))
+(test* "substring with cursors" "Ϥho"
+       (substring str
+                  (string-cursor-forward str ss 2)
+                  (string-cursor-back str se 2)))
+(test* "substring with cursors" "Ϥho"
+       (substring str
+                  2
+                  (string-cursor-back str se 2)))
+(test* "substring with cursors" "Ϥh"
+       (substring str
+                  (string-cursor-forward str ss 2)
+                  5))
+(test* "string->list" '(#\ #\ #\ #\h #\o #\)
+       (string->list str
+                     (string-cursor-next str ss)
+                     (string-cursor-prev str se)))
+(test* "string->list" '(#\ #\ #\ #\h #\o #\)
+       (string->list str
+                     1
+                     (string-cursor-prev str se)))
+(test* "string->list" '(#\ #\ #\ #\h #\o #\)
+       (string->list str
+                     (string-cursor-next str ss)
+                     7))
+(test* "string-cursor=?" #t
+       (string-cursor=? ss (string-cursor-back str se 8)))
+(test* "string-cursor=?" #t
+       (string-cursor=? 0 0))
+(test* "string-cursor<?" #t
+       (string-cursor<? ss se))
+(test* "string-cursor<=?" #t
+       (string-cursor<=? ss se))
+(test* "string-cursor<=?" #t
+       (string-cursor<=? ss ss))
+(test* "string-cursor>?" #t
+       (string-cursor>? se ss))
+(test* "string-cursor>=?" #t
+       (string-cursor>=? se se))
+(test* "string-cursor-diff" 8
+       (string-cursor-diff str ss se))
+(test* "string-cursor-diff" -8
+       (string-cursor-diff str se ss))
 
 ;;-------------------------------------------------------------------
 (test-section "incomplete strings")
