@@ -5,6 +5,7 @@
 (use gauche.test)
 (use srfi-1)
 (use srfi-13)
+(use gauche.uvector)
 
 ;; Note: test/object.scm contains extra tests of hashtables using
 ;; object-equal? and object-hash overload.
@@ -57,6 +58,61 @@
                         (hash (car p))))
               data)))
 
+;; uvector hash support
+(let ([data '((#u8()       1137702107  103389108)
+              (#s8()       3183946422 1028203680)
+              (#u16()      4026203278 2385671743)
+              (#s16()       521471644 3175907405)
+              (#u32()      2096652405 3737183940)
+              (#s32()      1176995887 2817527422)
+              (#u64()      3594299835  939864074)
+              (#s64()      4097263002 2456731467)
+              (#f16()      2577174380  936642845)
+              (#f32()      2264331153 3904862688)
+              (#f64()       404970645 2045502180)
+
+              (#u8(0)      2711902886  356952918)
+              (#s8(0)       241632508 3086832956)
+              (#u16(0)     2951147206 3338424123)
+              (#s16(0)     2607358220 2994635137)
+              (#u32(0)     1893327433 1506050516)
+              (#s32(0)     1590012139 1202735222)
+              (#u64(0)      791629991  404353074)
+              (#s64(0)     3306445826 3693722743)
+              (#f16(0)         970012  388246929)
+              (#f32(0)     2731721173 2344444256)
+              (#f64(0)     3019130455 2631853538)
+
+              (#u8(1)       253287183  824885740)
+              (#s8(1)       312535652 1326439597)
+              (#u16(1)     1310615671 1697892588)
+              (#s16(1)      966826685 1354103602)
+              (#u32(1)      252795898 4160486277)
+              (#s32(1)     4244447900 3857170983)
+              (#u64(1)     3446065752 3058788835)
+              (#s64(1)     1665914291 2053191208)
+              (#f16(1)        9718044  396994961)
+              (#f32(1)      173195733 4080886112)
+              (#f64(1)     1555830517 1168553600)
+
+              (#u8(1 2 3)   241852752 4262278780)
+              (#s8(1 2 3)  3864380406 3888224286)
+              (#u16(1 2 3) 2848547228 3940535561)
+              (#s16(1 2 3) 2843757170 3935745503)
+              (#u32(1 2 3) 2172856679 1080868346)
+              (#s32(1 2 3) 3179908921 2087920588)
+              (#u64(1 2 3)  400224405 3603203368)
+              (#s64(1 2 3) 3141078136 4233066469)
+              (#f16(1 2 3) 3019376060 4111364393)
+              (#f32(1 2 3) 3998543309 2906554976)
+              (#f64(1 2 3)  529975387 3732954350)
+              )])
+  (dolist [d data]
+    (test* #"portable hash uvector ~(car d)" d
+           (list (car d)
+                 (portable-hash (car d) 0)
+                 (portable-hash (car d) 1)))))
+
 ;;------------------------------------------------------------------
 (test-section "eq?-hash")
 
@@ -293,8 +349,15 @@
          (hash-table-put! h-equal (vector (cons 'a 'b) 3+3i) 61)
          (length (hash-table-keys h-equal))))
 
+;; u8vector hash support
+(test* "equal? test" 9
+       (begin
+         (hash-table-put! h-equal (u8vector (char->integer #\d)) 70)
+         (hash-table-put! h-equal (u8vector (char->integer #\d)) 71)
+         (length (hash-table-keys h-equal))))
+
 (test* "hash-table-values(3)" #t
-       (lset= equal? (hash-table-values h-equal) '(8 "b" #\c -1 "E" 5 7 61)))
+       (lset= equal? (hash-table-values h-equal) '(8 "b" #\c -1 "E" 5 7 61 71)))
 
 (test* "delete!" #f
        (begin
@@ -391,6 +454,18 @@
          (map cdr x)
          (map (^p (hash-table-comparator (make-hash-table (car p)))) x)))
 
+;;------------------------------------------------------------------
+(test-section "hash-table-copy & rehash")
+
+;; https://github.com/shirok/Gauche/issues/400
+
+(test* "hash-table-copy & rehash" #t 
+  (let1 h (fold (^[n h] (rlet1 h (hash-table-copy h)
+                          (hash-table-put! h n 1) h))
+                (make-hash-table 'eqv?)
+                (iota 20))
+    (every (cut hash-table-contains? h <> ) (iota 20))))
+
 ;;------------------------------------------------------------------
 (test-section "iterators")
 
@@ -487,6 +562,4 @@
                                         #f #f)
                        c d)))
               
-(test-module 'gauche.hashutil) ; autoloaded module
-
 (test-end)
