@@ -137,16 +137,25 @@
          (port-attribute-set! p 'my-attribute3 20))
   (test* "port-attributes" '((my-attribute . ok)
                              (my-attribute2 . 103)
-                             (my-attribute3 . 12))
+                             (my-attribute3 . 12)
+                             (name . "tmp.o"))
          (alist-delete 'reader-lexical-mode
                        (sort (port-attributes p)
                              string<?
                              ($ symbol->string $ car $))))
-  (test* "port-attribute-delete!" '((my-attribute3 . 13))
+  (test* "port-attribute-delete!" '((my-attribute3 . 13) (name . "tmp.o"))
          (begin
            (port-attribute-delete! p 'my-attribute)
            (port-attribute-delete! p 'my-attribute2)
            (alist-delete 'reader-lexical-mode (port-attributes p))))
+
+  ;; make sure 'name' attribute is read-only and undeletable
+  (test* "name attribute is read-only" (test-error <error> #/read-only/)
+         (port-attribute-set! p 'name "foo"))
+  (test* "name attribute is undeletable" (test-error <error> #/not deletable/)
+         (port-attribute-delete! p 'name))
+  (test* "name attribute is overridable" (test-error <error> #/Couldn't create/)
+         (port-attribute-create! p 'name (^ _ #f)))
   )
 
 ;;-------------------------------------------------------------------
@@ -764,6 +773,7 @@
 (test* "format ~10f" "     abcde"  (format "~10,2f" 'abcde))
 (test* "format ~1,2f" "0.00"     (format "~1,2f" 0.003))
 (test* "format ~1,0f" "-1."      (format "~1,0f" -0.555))
+(test* "format ~,2f" "0.00"      (format "~,2f" 0))
 
 (test* "format carry over ~,4f" "0.0100" (format "~,4f" 0.00999))
 (test* "format carry over across decimal point ~,1f" "124.0"
@@ -774,11 +784,43 @@
 (test* "format ~,1000f" (string-append "0.01" (make-string 998 #\0))
        (format "~,1000f" 0.01))
 
+(test* "format ~f with non-numeric object"  "   abc" (format "~6f" 'abc))
+(test* "format ~f with complex"  "0.707+0.707i" (format "~1,3f" (sqrt +i)))
+(test* "format ~f with complex"  "0.707-0.707i" (format "~1,3f" (sqrt -i)))
+(test* "format ~f with complex"  "          1.10+0.46i"
+       (format "~20,2f" (sqrt 1+i)))
+(test* "format ~f with complex"  "         +1.10+0.46i"
+       (format "~20,2@f" (sqrt 1+i)))
+
+(test* "format ~$" "100.00"  (format "~$" 100))
+(test* "format ~$" "0.33"    (format "~$" 1/3))
+(test* "format ~$" "3.14"    (format "~$" 3.1415926))
+(test* "format ~$" "3.14"    (format "~2$" 3.1415926))
+(test* "format ~$" "3.142"   (format "~3$" 3.1415926))
+(test* "format ~$" "3.1416"  (format "~4$" 3.1415926))
+(test* "format ~$" "3.14159" (format "~5$" 3.1415926))
+(test* "format ~$ pre-digits" "00.00"   (format "~,2$" 0))
+(test* "format ~$ padding"    "   0.00" (format "~,,7$" 0))
+(test* "format ~$ padding"    "***0.00" (format "~,,7,'*$" 0))
+(test* "format ~$ sign"       "  -3.14" (format "~,,7$" -3.1416))
+(test* "format ~$ sign"       "  -3.14" (format "~,,7@$" -3.1416))
+(test* "format ~$ sign"       "  +3.14" (format "~,,7@$" 3.1416))
+(test* "format ~$ sign"       "  -3.14" (format "~,,7$" -3.1416))
+(test* "format ~$ sign"       "  -3.14" (format "~,,7@$" -3.1416))
+(test* "format ~$ sign"       "  +3.14" (format "~,,7@$" 3.1416))
+(test* "format ~$ sign"       "-  3.14" (format "~,,7:$" -3.1416))
+(test* "format ~$ sign"       "   3.14" (format "~,,7:$" 3.1416))
+(test* "format ~$ sign"       "-  3.14" (format "~,,7@:$" -3.1416))
+(test* "format ~$ sign"       "+  3.14" (format "~,,7@:$" 3.1416))
+(test* "format ~$ non-real"   "    zzz" (format "~,,7$" 'zzz))
+
 (test* "format ~nr" "wud0up"  (format "~36r" 1985913745))
 (test* "format ~nr" "    wud0up"  (format "~36,10r" 1985913745))
 (test* "format ~nr" "****wud0up"  (format "~36,10,'*r" 1985913745))
 (test* "format ~nr" "***+wud0up"  (format "~36,10,'*@r" 1985913745))
 (test* "format ~nR" "WUD0UP"  (format "~36R" 1985913745))
+(test* "format ~vr" "wud0up"  (format "~vr" 36 1985913745))
+(test* "format ~vr" "    wud0up"  (format "~v,10r" 36 1985913745))
 
 (test* "format v param" "     12345"
        (format "~vd" 10 12345))
@@ -798,6 +840,27 @@
 (test* "format ~@*" "1 2 5"
        (format "~a ~a ~4@*~a" 1 2 3 4 5))
 
+(test* "format single character"
+       '(("~" "~~~" "~~~~~")
+         ("\t" "\t\t\t" "\t\t\t\t\t")
+         ("\n" "\n\n\n" "\n\n\n\n\n")
+         ("\f" "\f\f\f" "\f\f\f\f\f"))
+       (map (^c (list (format (string #\~ c))
+                      (format (string #\~ #\3 c))
+                      (format (string #\~ #\v c) 5)))
+            '(#\~ #\t #\% #\|)))
+
+(test* "format ~?" "ding[\"dang\" dong]ping"
+       (format "~s~?~a" 'ding "[~s ~a]" '("dang" "dong") "ping"))
+(test* "format ~?" "ding[\"dang\" gong]ping" ; extra args for ~? are ignored
+       (format "~s~?~a" 'ding "[~s ~a]" '("dang" "gong" "dong") "ping"))
+(test* "format ~@?" "ding[\"dang\" dong]ping"
+       (format "~s~@?~a" 'ding "[~s ~a]" "dang" "dong" "ping"))
+(test* "format nested ~?" "<[<<{\"O\" O}>>]>"
+       (format "~s[~?]~s" '< "~s{~?}~s" '(<< "~s ~a" ("O" "O") >>) '>))
+(test* "format nested ~@?" "<[<<{\"O\" O}>>]>"
+       (format "~s[~@?]~s" '< "~s{~@?}~s" '<< "~s ~a" "O" "O" '>> '>))
+
 (test* "format incomplete tilde sequence" (test-error)
        (format "~"))
 (test* "format incomplete tilde sequence" (test-error)
@@ -977,5 +1040,14 @@
                          (loop (read-char wrap) (cons ch r)))))))
               tdata)))
 
+;;-------------------------------------------------------------------
+;; Literal uvector on load
+;; Cf. https://github.com/shirok/Gauche/issues/521
+
+(load (string-append (sys-dirname (current-load-path)) "/load-literal-test"))
+
+(test* "uvector immutability on load & read" '(#t #f #f)
+       (map uvector-immutable? (load-read-test-uvectors)))
+
 (test-end)
 
