@@ -1,7 +1,7 @@
-;; test for write/ss and read/ss
+;; test for advanced read/write features
 ;;
-;; this test is splitted from io.scm, since this one uses util.isomorph,
-;; and has to be done after the test of util.* module.
+;; These test is splitted from io.scm, since they depend on other modules
+;; has to be done after their tests.
 
 (use gauche.test)
 (use srfi-1)
@@ -687,4 +687,42 @@
        (call-with-output-string 
          (cut pprint '(a a) :width 3 :newline #f :port <>)))
 
+(test* "label (issue #484)" "#0=(a #1=(b #0# #1#))\n"
+       (call-with-output-string
+         (cut pprint '#0=(a #1=(b #0# #1#)) :port <>)))
+(test* "label (issue #713)" "#0=(1 . #1=((2 . #1#) 3 . #0#))\n"
+       (call-with-output-string
+         (cut pprint '#0=(1 . #1=((2 . #1#) 3 . #0#)) :port <>)))
+
+(test* "shorthand notation" "('a `b ,c ,@d `(e ,f 'g ,@h))\n"
+       (call-with-output-string
+         (cut pprint '('a `b ,c ,@d `(e ,f 'g ,@h)) :port <>)))
+(test* "shorthand notation" "'#0=(a . '#0#)\n"
+       (call-with-output-string
+         (cut pprint ''#0=(a . '#0#) :port <>)))
+(test* "shorthand notation" "'#0=(a ,@#1='(#0# . ,#1#))\n"
+       (call-with-output-string
+         (cut pprint ''#0=(a ,@#1='(#0# . ,#1#)) :port <>)))
+(test* "shorthand notation" "#0=(a (quote . #0#))\n"
+       (call-with-output-string
+         (cut pprint '#0=(a (quote . #0#)) :port <>)))
+(test* "shorthand notation" "#0=(a quote . #0#)\n"
+       (call-with-output-string
+         (cut pprint '#0=(a quote . #0#) :port <>)))
+(test* "shorthand notation" "'(,@(the quick brown fox\
+                           \n     jumps over the\
+                           \n     ,@lazy dog))\n"
+       (call-with-output-string
+         (cut pprint ''(,@(the quick brown fox jumps over the ,@lazy dog))
+              :port <> :width 25)))
+
+;;===============================================================
+;; utf-8 with BOM
+;;
+
+(test* "utf-8 with BOM" "foo!"
+       (begin
+         (load "data/utf-8-bom.scm" :paths `(,(sys-dirname (current-load-path))))
+         (foo)))
+
 (test-end)
