@@ -72,7 +72,28 @@
   (test-length '(a b c d e) 5))
 
 ;;--------------------------------------------------------------------------
-(test-section "append and reverse")
+(test-section "copy, append and reverse")
+
+(test* "copy" '((a b c d e) #f)
+       (let* ([x '(a b c d e)]
+              [y (list-copy x)])
+         (list x (eq? x y))))
+(test* "copy" '((a b c d . e) #f)
+       (let* ([x '(a b c d . e)]
+              [y (list-copy x)])
+         (list x (eq? x y))))
+(test* "copy" '(x #t)
+       (let* ([x 'x]
+              [y (list-copy x)])
+         (list x (eq? x y))))
+(test* "copy" '(() #t)
+       (let* ([x '()]
+              [y (list-copy x)])
+         (list x (eq? x y))))
+(test* "copy" (test-error)              ;detect circular list
+       (let* ([x '#0=(a b c . #0#)]
+              [y (list-copy x)])
+         (list x (eq? x y))))
 
 (test* "append (inlined)" '()           (append))
 (test* "append (inlined)" 1             (append 1))
@@ -225,25 +246,25 @@
 (test* "delete" '("b" "c" "d" "e")
        (delete "A" '("a" "b" "a" "c" "d" "a" "e") string-ci=?))
 (test* "delete!" '(b c d e)
-       (delete! 'a '(a b a c a d a e a) eq?))
+       (delete! 'a (list-copy '(a b a c a d a e a)) eq?))
 (test* "delete!" '(b c d e)
-       (delete! 'a '(a b a c a d a e) eq?))
+       (delete! 'a (list-copy '(a b a c a d a e)) eq?))
 (test* "delete!" '(b c d e)
-       (delete! 'a '(b a c a d a e a) eq?))
+       (delete! 'a (list-copy '(b a c a d a e a)) eq?))
 (test* "delete!" '(b c d e)
-       (delete! 'a '(b a c a d a e) eq?))
+       (delete! 'a (list-copy '(b a c a d a e)) eq?))
 (test* "delete!" '(b c d e)
-       (delete! 'a '(b c d e) eq?))
+       (delete! 'a (list-copy '(b c d e)) eq?))
 (test* "delete!" '(b c d e . f)
-       (delete! 'a '(b c a d e . f) eq?))
+       (delete! 'a (list-copy '(b c a d e . f)) eq?))
 (test* "delete!" '(2.0 3.0 4.0 5.0)
-       (delete! '1.0 '(1.0 2.0 1.0 3.0 4.0 1.0 5.0) eqv?))
+       (delete! '1.0 (list 1.0 2.0 1.0 3.0 4.0 1.0 5.0) eqv?))
 (test* "delete!" '("b" "c" "d" "e")
-       (delete! "a" '("a" "b" "a" "c" "d" "a" "e")))
+       (delete! "a" (list "a" "b" "a" "c" "d" "a" "e")))
 (test* "delete!" '("b" "c" "d" "e")
-       (delete! "a" '("a" "b" "a" "c" "d" "a" "e") string=?))
+       (delete! "a" (list "a" "b" "a" "c" "d" "a" "e") string=?))
 (test* "delete!" '("b" "c" "d" "e")
-       (delete! "A" '("a" "b" "a" "c" "d" "a" "e") string-ci=?))
+       (delete! "A" (list "a" "b" "a" "c" "d" "a" "e") string-ci=?))
 (test* "delete-duplicates" '(a b c d e)
        (delete-duplicates '(a b c d e) eq?))
 (test* "delete-duplicates" '(a b c d e)
@@ -259,17 +280,17 @@
 (test* "delete-duplicates" '("A" "b" "c" "d" "e")
        (delete-duplicates '("A" "b" "a" "B" "c" "d" "a" "e") string-ci=?))
 (test* "delete-duplicates!" '(a b c d e)
-       (delete-duplicates! '(a b c d e) eq?))
+       (delete-duplicates! (list-copy '(a b c d e)) eq?))
 (test* "delete-duplicates!" '(a b c d e)
-       (delete-duplicates! '(a b a c b a d d a e) eq?))
+       (delete-duplicates! (list-copy '(a b a c b a d d a e)) eq?))
 (test* "delete-duplicates!" '(1.0 2.0 3.0 4.0 5.0)
-       (delete-duplicates! '(1.0 2.0 1.0 2.0 3.0 3.0 4.0 1.0 5.0) eqv?))
+       (delete-duplicates! (list 1.0 2.0 1.0 2.0 3.0 3.0 4.0 1.0 5.0) eqv?))
 (test* "delete-duplicates!" '("a" "b" "c" "d" "e")
-       (delete-duplicates! '("a" "b" "b" "a" "b" "c" "d" "a" "e")))
+       (delete-duplicates! (list "a" "b" "b" "a" "b" "c" "d" "a" "e")))
 (test* "delete-duplicates!" '("a" "b" "c" "d" "e")
-       (delete-duplicates! '("a" "b" "a" "a" "c" "d" "a" "e") string=?))
+       (delete-duplicates! (list "a" "b" "a" "a" "c" "d" "a" "e") string=?))
 (test* "delete-duplicates!" '("A" "b" "c" "d" "e")
-       (delete-duplicates! '("A" "b" "a" "B" "c" "d" "a" "e") string-ci=?))
+       (delete-duplicates! (list "A" "b" "a" "B" "c" "d" "a" "e") string-ci=?))
 
 (test* "any" #f (any even? '()))
 
@@ -324,8 +345,8 @@
 (test* "drop-right" '(1)       (drop-right '(1 2 3 . d) 2))
 (test* "take-right" 'd         (take-right '(1 2 3 . d) 0))
 (test* "drop-right" '(1 2 3)   (drop-right '(1 2 3 . d) 0))
-(test* "take!"      '(1 2)     (take! '(1 2 3 . d) 2))
-(test* "drop-right!" '(1 2)    (drop-right! '(1 2 3 . d) 1))
+(test* "take!"      '(1 2)     (take! (list-copy '(1 2 3 . d)) 2))
+(test* "drop-right!"'(1 2)     (drop-right! (list-copy '(1 2 3 . d)) 1))
 (test* "split-at" '((a b c) (d e f g h))
        (call-with-values (^[] (split-at '(a b c d e f g h) 3))
          list))
@@ -492,6 +513,55 @@
 (test* "assv-set!" '((9 . c) (3 . a) (5 . b))
        (assv-set! (list (cons 3 'a) (cons 5 'b)) 9 'c))
 
+(test* "assoc-adjoin" '((c . 3) (a . 1) (b . 2))
+       (assoc-adjoin '((a . 1) (b . 2)) 'c 3))
+(test* "assoc-adjoin" '((a . 4) (b . 2))
+       (assoc-adjoin '((a . 1) (b . 2)) 'a 4))
+(test* "assoc-adjoin" '((a . 1) (b . 5))
+       (assoc-adjoin '((a . 1) (b . 2)) 'b 5))
+(test* "assoc-adjoin" '((a . 1))
+       (assoc-adjoin '() 'a 1))
+
+(test* "assoc-update-in" '((a (aa . 1) (ab . 2))
+                           (b (ba . 3) (bb (bba . 104) (bbb . 5))))
+       (assoc-update-in '((a (aa . 1) (ab . 2))
+                          (b (ba . 3) (bb (bba . 4) (bbb . 5))))
+                        '(b bb bba)
+                        (lambda (x) (+ x 100))
+                        0
+                        eq?))
+(test* "assoc-update-in" '((a (aa . 1) (ab . 2))
+                           (b (ba . 3) (bb (bbc . 100) (bba . 4) (bbb . 5))))
+       (assoc-update-in '((a (aa . 1) (ab . 2))
+                          (b (ba . 3) (bb (bba . 4) (bbb . 5))))
+                        '(b bb bbc)
+                        (lambda (x) (+ x 100))
+                        0
+                        eq?))
+(test* "assoc-update-in" '((c (d (e . 100)))
+                           (a (aa . 1) (ab . 2))
+                           (b (ba . 3) (bb (bba . 4) (bbb . 5))))
+       (assoc-update-in '((a (aa . 1) (ab . 2))
+                          (b (ba . 3) (bb (bba . 4) (bbb . 5))))
+                        '(c d e)
+                        (lambda (x) (+ x 100))
+                        0
+                        eq?))
+(test* "assoc-update-in" '((a (aa . 1)))
+       (assoc-update-in '()
+                        '(a aa)
+                        -
+                        -1))
+(test* "assoc-update-in" '(("a" ("aa" . 101) ("ab" . 2))
+                           ("b" ("ba" . 3)
+                            ("bb" ("bba" . 4) ("bbb" . 5))))
+       (assoc-update-in '(("a" ("aa" . 1) ("ab" . 2))
+                          ("b" ("ba" . 3) ("bb" ("bba" . 4) ("bbb" . 5))))
+                        '("a" "aa")
+                        (lambda (x) (+ x 100))
+                        0
+                        string=?))
+
 ;;--------------------------------------------------------------------------
 
 (test-section "circular list and equality")
@@ -554,7 +624,6 @@
   )
 
 ;;--------------------------------------------------------------------------
-
 (test-section "monotonic-merge")
 
 ;; monotonic-merge is a core function to implement Dylan-style class
@@ -583,4 +652,55 @@
               (hv-grid horizontal-grid vertical-grid grid-layout object)
               (vh-grid vertical-grid horizontal-grid grid-layout object)))))
 
+;;--------------------------------------------------------------------------
+(test-section "immutable pairs")
+
+(test* "ipair" #t (ipair? (ipair 1 2)))
+(test* "ipair" #f (ipair? '()))
+(test* "ipair" #f (ipair? (cons 1 2)))
+(test* "ipair" '(1 . 2)
+       (let1 p (ipair 1 2)
+         (cons (car p) (cdr p))))
+
+(test* "ilist" '(#t #t #t #f)
+       (let1 p (ilist 1 2 3)
+         (list (ipair? p)
+               (ipair? (cdr p))
+               (ipair? (cddr p))
+               (ipair? (cdddr p)))))
+(test* "ilist" '() (ilist))
+
+(test* "ipair immutability" (test-error)
+       (let1 p (ipair 1 2)
+         (set-car! p 5)))
+(test* "ipair immutability" (test-error)
+       (let1 p (ipair 1 2)
+         (set-cdr! p 5)))
+
+;;--------------------------------------------------------------------------
+(test-section "extended pairs")
+
+(let ([extended-cons  (with-module gauche.internal extended-cons)]
+      [extended-pair? (with-module gauche.internal extended-pair?)]
+      [pair-attributes (with-module gauche.internal pair-attributes)]
+      [pair-attribute-get (with-module gauche.internal pair-attribute-get)]
+      [pair-attribute-set! (with-module gauche.internal pair-attribute-set!)])
+  (test* "extended pair" #f (extended-pair? (cons 1 2)))
+  ;(test* "extended pair" #t (extended-pair? '(1 2)))
+  (test* "extended pair" #t (extended-pair? (extended-cons 1 2)))
+  (test* "pair-attributes" '() (pair-attributes (cons 1 2)))
+  (test* "pair-attributes" '() (pair-attributes (ipair 1 2)))
+  (test* "pair-attributes" '() (pair-attributes (extended-cons 1 2)))
+  (test* "pair-attributes-get/set!" 'hoi
+         (let1 z (extended-cons 1 2)
+           (pair-attribute-set! z 'foo 'hoi)
+           (pair-attribute-get z 'foo)))
+  (test* "set-car! to extended pair" '(3 . 2)
+         (rlet1 z (extended-cons 1 2)
+           (set-car! z 3)))
+  (test* "set-cdr! to extended pair" '(1 . 4)
+         (rlet1 z (extended-cons 1 2)
+           (set-cdr! z 4)))
+  )
+
 (test-end)
