@@ -79,8 +79,7 @@
          (let* ([first (car clauses)]
                 [rest  (cdr clauses)]
                 [test  (car first)])
-           (cond [(and (or (symbol? test)
-                           (identifier? test))
+           (cond [(and (identifier? test)
                        (c test (r 'else)))
                   `(,(r 'begin) ,@(cdr first))]
                  [else `(,(r 'if) ,test
@@ -110,8 +109,7 @@
            (let* ([first (car clauses)]
                   [rest  (cdr clauses)]
                   [test  (car first)])
-             (cond [(and (or (symbol? test)
-                             (identifier? test))
+             (cond [(and (identifier? test)
                          (c test (r 'else)))
                     `(,(r 'begin) ,@(cdr first))]
                    [else `(,(r 'if) ,test
@@ -141,7 +139,7 @@
 ;; cf. http://saito.hatenablog.jp/entry/2014/11/18/233209
 (define (er-test-traverse proc obj)
   (let loop ((obj obj))
-    (cond [(or (symbol? obj) (identifier? obj))    (proc obj)]
+    (cond [(identifier? obj) (proc obj)]
           [(pair? obj)   (cons (loop (car obj)) (loop (cdr obj)))]
           [(vector? obj) (vector-map loop obj)]
           [else obj])))
@@ -208,7 +206,6 @@
              (find-module 'er-macro-test-1)))
 
 ;; quasirename
-;; Note: currently quasirename depends on util.match, too.
 (let ((unquote list)
       (x 1)
       (y 2))
@@ -218,8 +215,8 @@
                              [b (caddr f)]
                              [all (cdr f)])
                          (quasirename r
-                           (list x ,a y ,b ,@all
-                                 '#(x ,a y ,b) ,@(reverse all))))))])
+                           `(list x ,a y ,b ,@all
+                                  '#(x ,a y ,b) ,@(reverse all))))))])
     (let ((list vector)
           (x 10)
           (y 20))
@@ -227,6 +224,59 @@
              '(1 3 2 4 3 4 #(x 3 y 4) 4 3)
              (foo 3 4)))))
 
+;; nested quasirename
+(let ()
+  (define (add-prefix p)
+    (^s (symbol-append p s)))
+  (define a 1)
+  (define b 2)
+  (define c 3)
+  (test* "nested quasirename"
+         '(p:quasirename p:x
+            `(p:a ,p:b ,(p:quote 3) ,d))
+         (unwrap-syntax
+          (quasirename (add-prefix 'p:)
+            `(quasirename x
+               `(a ,b ,',c ,,'d))))))
+
+(let-syntax ([def (er-macro-transformer
+                   (^[f r c]
+                     (quasirename r
+                       `(define-syntax ,(cadr f)
+                          (er-macro-transformer
+                           ;; we need to protect ff from being renamed,
+                           ;; for we have to refer to it inside quote
+                           ;; in (cadr ff).
+                           (^[,'ff rr cc]
+                             (quasirename rr
+                               `(define ,',(caddr f) ,,'(cadr ff)))))))))])
+  (test* "nested quasirename" 4
+         (let ()
+           (def foo bar)
+           (let ()
+             (foo 4)
+             bar))))
+
+;; Mixing syntax-rules and er-macro requires unhygienic identifiers to be
+;; explicitly "injected".
+;; (This does not work with the current compiler)
+
+;; (define-syntax eri-test-loop
+;;   (eri-macro-transformer
+;;    (lambda (x r c i)
+;;      (let ((body (cdr x)))
+;;        `(,(r 'call-with-current-continuation)
+;;          (,(r 'lambda) (,(i 'exiit))
+;;           (,(r 'let) ,(r 'f) () ,@body (,(r 'f)))))))))
+
+;; (define-syntax eri-test-foo
+;;   (syntax-rules ()
+;;     ((_ x) (eri-test-loop (exiit x)))))
+
+;; (test* "Mixing syntax-rules and eri-macro" 'yot
+;;        (let ((exiit 42))
+;;          (eri-test-foo exiit)))
+
 ;;----------------------------------------------------------------------
 ;; basic tests
 
@@ -493,7 +543,7 @@
 (test-macro "dot3" 3           (dot3 () 3))
 
 ;; see if effective quote introduced by quasiquote properly unwrap
-;; syntactic enviornment.
+;; syntactic environment.
 (define-syntax unwrap1 (syntax-rules ()
                          ((_ x) `(a ,x))))
 (test "unwrap1" '(a 3) (lambda () (unwrap1 3))
@@ -1233,6 +1283,35 @@
   (test "literal identifier comparison car car-alias" #t
         (lambda () (free-identifier=?? car car-alias))))
 
+;; macro defining macro from other module
+;; https://github.com/shirok/Gauche/issues/532
+
+(define-module macro-defining-macro-toplevel
+  (export x1)
+  (define-syntax x1
+    (syntax-rules ()
+      ((x1 y1)
+       (x2 x3 y1))))
+
+  (define-syntax x2
+    (syntax-rules ()
+      ((x2 x3 y1)
+       (begin
+         (define-syntax x3
+           (syntax-rules ()
+             ((x3 x4) x4)))
+         (define-syntax y1
+           (syntax-rules ()
+             ((y1 y2) (x3 y2)))))))))
+
+(define-module macro-defining-macro-toplevel-user
+  (use gauche.test)
+  (import macro-defining-macro-toplevel)
+  (x1 bar)
+  ;; without fix, (bar 1) fails with "unbound variable: #<identifier ... x3>"
+  (test "macro defining macro in other module" 1
+        (lambda () (eval '(bar 1) (current-module)))))
+
 ;;----------------------------------------------------------------------
 ;; identifier comparison
 
@@ -1241,9 +1320,9 @@
 ;; This is EXPERIMENTAL: may be changed in later release.
 (define-syntax expand-id-compare (syntax-rules () ((hoge foo ...) (cdr b))))
 (test "comparison of identifiers" '(cdr b)
-      (lambda () (macroexpand '(expand-id-compare bar))))
-(test "comparison of identifiers" (macroexpand '(expand-id-compare bar))
-      (lambda () (macroexpand '(expand-id-compare bar))))
+      (lambda () (macroexpand '(expand-id-compare bar) #t)))
+(test "comparison of identifiers" (macroexpand '(expand-id-compare bar) #t)
+      (lambda () (macroexpand '(expand-id-compare bar) #t)))
 
 ;;----------------------------------------------------------------------
 ;; keyword and extended lambda list
@@ -1548,9 +1627,9 @@
 ;;----------------------------------------------------------------------
 ;; compiler macros
 
-(test-section "define-inline/syntax")
+(test-section "define-hybrid-syntax")
 
-(define-inline/syntax cpm
+(define-hybrid-syntax cpm
   (lambda (a b) (+ a b))
   (er-macro-transformer
    (lambda (f r c) `(,(r '*) ,(cadr f) ,(caddr f)))))
@@ -1583,7 +1662,7 @@
       (lambda ()
         (guard [e (else (let1 xs (filter <compile-error-mixin>
                                          (slot-ref e '%conditions))
-                          (cons (slot-ref e 'message)
+                          (cons (condition-message e e)
                                 (map (lambda (x) (slot-ref x 'expr)) xs))))]
           (eval '(list (test-syntax-error x y z))
                 (interaction-environment)))))
@@ -1594,7 +1673,7 @@
       (lambda ()
         (guard [e (else (let1 xs (filter <compile-error-mixin>
                                          (slot-ref e '%conditions))
-                          (cons (slot-ref e 'message)
+                          (cons (condition-message e e)
                                 (map (lambda (x) (slot-ref x 'expr)) xs))))]
           (eval '(list (test-syntax-error x y))
                 (interaction-environment)))))
@@ -1669,9 +1748,9 @@
        (eval '(lit-test-2 temp 1) (interaction-environment)))
 
 ;;----------------------------------------------------------------------
-;; 'generate-underbar-1' test should output the following error.
-;;
-;; *** ERROR: unbound variable: #<identifier user#_.2d4ac00>
+;; 'generate-underbar-1' inserts global underbar into the macro output.
+;; It shouldn't be regarded as a pattern variable, so the underbar in
+;; the template refers to the global binding of '_'.
 
 (test-section "generate underbar")
 
@@ -1685,8 +1764,7 @@
               _))))
        (mac-sub1 'failed)))))
 
-(test* "generate-underbar-1"
-       (test-error <error> #/^unbound variable/)
+(test* "generate-underbar-1" _
        (gen-underbar))
 
 ;;----------------------------------------------------------------------
@@ -1748,6 +1826,54 @@
           (build-deep-nested-pattern 256 'x)
           (interaction-environment))))
 
+;;----------------------------------------------------------------------
+;; let-keyword* hygienic expansion
+;;
+
+(test-section "hygienic extened-lambda expansion")
+(define-module let-keyword-hygiene-def
+  (use gauche.base)
+  (use util.match)
+  (export klambda)
+  (extend scheme)
+  (define-syntax klambda
+    (er-macro-transformer
+     (^[f r c]
+       (match f
+         [(_ formals&keys . body)
+          (quasirename r
+            `(lambda (,@(drop-right formals&keys 1)
+                      ,(make-keyword 'key)
+                      ,@(map (^s `(,s #f)) (last formals&keys)))
+               ,@body))])))))
+
+(define-module let-keyword-hygeiene-use
+  (import let-keyword-hygiene-def)
+  (import gauche.keyword)
+  (export call-klambda)
+  (extend scheme)
+  (define (call-klambda a b c d)
+    ((klambda (a b (x y)) (list a b x y))
+     a b :x c :y d)))
+
+(test* "hygienic let-keyword expansion" '(1 2 3 4)
+       ((with-module let-keyword-hygeiene-use call-klambda) 1 2 3 4))
+
+;; Cf. http://chaton.practical-scheme.net/gauche/a/2020/11/05#entry-5fa3ba50-dc7d3
+(define-syntax let-keywords-hygiene-test-1-inner
+  (er-macro-transformer
+   (^[f r c]
+     (let-keywords (cdr f) ([a 1]
+                            [b 2])
+       (quasirename r `(+ ,a ,b))))))
+(define-syntax let-keywords-hygiene-test-1-outer
+  (syntax-rules ()
+    [(_ x) (let-keywords-hygiene-test-1-inner :b x)]))
+
+(test* "hygienic let-keyword match" 10
+       (let-keywords-hygiene-test-1-outer 9))
+
+
 ;;----------------------------------------------------------------------
 ;; srfi-147 begin
 ;; (not yest supported)
