@@ -548,6 +548,52 @@
          (check 'mpl-prefix-8 'a 'b 'X:a 'X:b 'Y:a 'Y:b))
   )
 
+;;------------------------------------------------------------------
+;; transitive export issue
+;; https://github.com/shirok/Gauche/issues/472
+
+(define-module tre1-A
+  (export a)
+  (define a 1))
+(define-module tre1-B
+  (import tre1-A)
+  (export a))
+(define-module tre1-C
+  (extend tre1-B))
+
+(test* "transitive export with only" 1
+       (let1 m (make-module #f)
+         (eval '(import (tre1-C :only (a))) m)
+         (eval 'a m)))
+
+(define-module tre2-A
+  (export a)
+  (define a 1))
+(define-module tre2-B
+  (import tre2-A)
+  (export (rename a aaa)))
+(define-module tre2-C
+  (extend tre2-B))
+
+(test* "transitive export with only" 1
+       (let1 m (make-module #f)
+         (eval '(import (tre2-C :only (aaa))) m)
+         (eval 'aaa m)))
+
+(define-module tre3-A
+  (export a)
+  (define a 1))
+(define-module tre3-B
+  (import tre3-A)
+  (export a))
+(define-module tre3-C
+  (extend tre3-B))
+
+(test* "transitive export with only" 1
+       (let1 m (make-module #f)
+         (eval '(import (tre3-C :rename ((a aaa)))) m)
+         (eval 'aaa m)))
+
 ;;------------------------------------------------------------------
 ;; creates modules on-the-fly
 
@@ -587,6 +633,71 @@
           (eval '(define x 13) m0)
           (eval 'x m1))))
 
+;;-------------------------------------------------------------------
+;; null and scheme module bindings (R5RS)
+
+(test "null module bindings" '()
+      (lambda ()
+        (let loop ((names '(and begin cond define define-syntax delay do
+                            if lambda let let* let-syntax
+                            letrec letrec-syntax or quasiquote quote
+                            set! syntax-rules else =>))
+                   (missing '()))
+          (cond [(null? names) missing]
+                [(global-variable-bound? 'null (car names))
+                 (loop (cdr names) missing)]
+                [else (loop (cdr names) (cons (car names) missing))]))))
+
+(test "scheme module bindings" '()
+      (lambda ()
+        (let loop ((names '(* + - / < <= = >= >
+                            abs acos angle append apply asin assoc assq
+                            assv atan boolean? call-with-current-continuation
+                            call-with-input-file call-with-output-file
+                            call-with-values car cdr caar cadr cdar cddr
+                            caaar caadr cadar caddr cdaar cdadr cddar cdddr
+                            caaaar caaadr caadar caaddr cadaar cadadr
+                            caddar cadddr cdaaar cdaadr cdadar cdaddr
+                            cddaar cddadr cdddar cddddr ceiling char->integer
+                            char-alphabetic? char-ci<=? char-ci<? char-ci=?
+                            char-ci>=? char-ci>? char-downcase char-lower-case?
+                            char-numeric? char-ready? char-upcase
+                            char-upper-case? char-whitespace?
+                            char<? char<=? char=? char>? char>=? char?
+                            close-input-port close-output-port complex?
+                            cons cos current-input-port current-output-port
+                            denominator display dynamic-wind eof-object?
+                            eq? equal? eqv? eval even? exact->inexact
+                            exact? exp expt floor for-each force gcd
+                            imag-part inexact->exact inexact? input-port?
+                            integer->char integer? interaction-environment
+                            lcm length list list->string list->vector
+                            list-ref list-tail list? load log magnitude
+                            make-polar make-rectangular make-string
+                            make-vector map max member memq memv min modulo
+                            negative? newline not null-environment null?
+                            number->string number? numerator odd?
+                            open-input-file open-output-file or output-port?
+                            pair? peek-char port? positive? procedure?
+                            quotient rational? rationalize read read-char
+                            real-part real? remainder reverse round
+                            scheme-report-environment set-car! set-cdr!
+                            sin sqrt string string->list string->number
+                            string->symbol string-append string-ci<=?
+                            string-ci<? string-ci=? string-ci>? string-ci>=?
+                            string-copy string-fill! string-length string-ref
+                            string-set! string<=? string<? string=? string>?
+                            string>=? substring symbol->string symbol?
+                            tan truncate vector vector->list vector-fill!
+                            vector-length vector-ref vector-set! vector?
+                            with-input-from-file with-output-to-file
+                            write write-char zero?))
+                   (missing '()))
+          (cond [(null? names) missing]
+                [(global-variable-bound? 'scheme (car names))
+                 (loop (cdr names) missing)]
+                [else (loop (cdr names) (cons (car names) missing))]))))
+
 ;;-------------------------------------------------------------------
 ;; Macro and builtin inliner
 ;; https://twitter.com/tk_riple/status/647865265154326528
@@ -607,8 +718,19 @@
 
 (test* "builtin inliner bug" 64
        ((with-module builtin-inliner-bug-C ash-3) 7 2))
-  
 
+;;-------------------------------------------------------------------
+;; Import qualifier & binding check
+;; https://github.com/shirok/Gauche/issues/549
 
+(test* "import qualifier binding check" 1
+       (eval '(begin
+                (define-module imp-A
+                  (export x)
+                  (define (x) 1))
+                (define-module imp-B
+                  (import (imp-A :only (x)))
+                  (x)))
+             (current-module)))
 
 (test-end)
