@@ -457,6 +457,43 @@
                (1 ,(undefined) ,(undefined) 0 1 ())
                ,(test-error)))
 
+;; :allow-other-keys without :key
+;; http://chaton.practical-scheme.net/gauche/a/2020/03/12#entry-5e6a02c5-9b99a
+(test-optkey ":allow-other-keys z"
+             (lambda (x :allow-other-keys z)
+               (cons x z))
+             '(1 :a 2 :b 3)
+             `((1 :b 3 :a 2)
+               ,(test-error)
+               (1 :a 2)
+               ,(test-error)
+               (1)
+               ,(test-error)))
+
+(test-optkey ":allow-other-keys"
+             (lambda (:allow-other-keys) 'ok)
+             '(:a 2 :b 3)
+             `(ok
+               ,(test-error)
+               ok
+               ,(test-error)
+               ok))
+
+;; If keyword is symbol, we should allow them as formals.
+(test-section "rnrs lambda formals")
+
+(define-syntax rnrs-lambda (with-module null lambda))
+(when (symbol? :a)
+  (test* "rnrs lambda formals" '(x y)
+         ((rnrs-lambda (:a :b) (list :a :b)) 'x 'y))
+  (test* "rnrs lambda formals" '(x z)
+         ((rnrs-lambda (a :key b) (list a b)) 'x 'y 'z))
+  (test* "rnrs lambda formals" '(x (z))
+         ((rnrs-lambda (a :optional . b) (list a b)) 'x 'y 'z))
+  (test* "rnrs lambda formals" '(x y z)
+         ((rnrs-lambda :rest :rest) 'x 'y 'z))
+  )
+
 ;;-----------------------------------------------------------------------
 ;; case-lambda
 (test-section "case-lambda")
@@ -567,4 +604,44 @@
                                              (cut read x) (cut read y)))
                  (^_ r)))
 
+;;-----------------------------------------------------------------------
+;; large env frame
+;; https://github.com/shirok/Gauche/issues/487
+(test-section "large env frame")
+
+(define-macro (biglet n)
+  (let1 syms (map (^_ (gensym)) (iota n))
+    `(let (,@(map (^s `(,s #f)) syms))
+       ,@(map (^[s i] `(set! ,s ,i)) syms (iota n))
+       (list ,@syms))))
+
+(test* "bigenv 1030" (iota 1030)
+       (biglet 1030))
+
+;; Test LREF-VAL0-NUMADD2 path
+(define-macro (bigsum n)
+  (let1 syms (map (^_ (gensym)) (iota n))
+    `(lambda ,syms (+ ,@syms))))
+
+(define bigsum-proc (bigsum 1029))
+
+(test* "bigsum 1029" (apply + (iota 1029))
+       (apply bigsum-proc (iota 1029)))
+
+;; Test LREF-VAL0-BNGT etc. path
+(define-macro (bigcond op n)
+  (let1 syms (map (^_ (gensym)) (iota n))
+    `(lambda (k ,@syms)
+       (cond ,@(map (^s `((,op k ,s) ,s)) syms)))))
+
+(define bigcond<-proc (bigcond < 1028))
+
+(test* "bigcond< 1028" 1025
+       (apply bigcond<-proc 1024 (iota 1028)))
+
+(define bigcond=-proc (bigcond = 1028))
+
+(test* "bigcond= 1028" 1027
+       (apply bigcond=-proc 1027 (iota 1028)))
+
 (test-end)
