@@ -412,69 +412,109 @@
        (substring #*"abcde" 0 2))
 
 ;;-------------------------------------------------------------------
-(test-section "string-pointer")
+(test-section "string-cursor")
 
-(define sp #f)
-(test* "make-string-pointer" #t
+(define str "abcdefg")
+(define ss #f)
+(define se #f)
+(test* "string-cursor-start" #t
        (begin
-         (set! sp (make-string-pointer "abcdefg"))
-         (string-pointer? sp)))
-(test* "string-pointer-next!" #\a
-       (string-pointer-next! sp))
-(test* "string-pointer-next!" #\b
-       (string-pointer-next! sp))
-(test* "string-pointer-prev!" #\b
-       (string-pointer-prev! sp))
-(test* "string-pointer-prev!" #\a
-       (string-pointer-prev! sp))
-(test* "string-pointer-prev!" #t
-       (eof-object? (string-pointer-prev! sp)))
-(test* "string-pointer-index" 0
-       (string-pointer-index sp))
-(test* "string-pointer-index" 7
-       (do ((x (string-pointer-next! sp) (string-pointer-next! sp)))
-           ((eof-object? x) (string-pointer-index sp))))
-(test* "string-pointer-substring" '("abcdefg" "")
-       (list (string-pointer-substring sp)
-             (string-pointer-substring sp :after #t)))
-(test* "string-pointer-substring" '("abcd" "efg")
+         (set! ss (string-cursor-start str))
+         (string-cursor? ss)))
+(test* "string-cursor-start" #t
        (begin
-         (string-pointer-set! sp 4)
-         (list (string-pointer-substring sp)
-               (string-pointer-substring sp :after #t))))
-(test* "string-pointer-substring" '("" "abcdefg")
-       (begin
-         (string-pointer-set! sp 0)
-         (list (string-pointer-substring sp)
-               (string-pointer-substring sp :after #t))))
-(test* "string-pointer-substring" '("" "")
-       (let ((sp (make-string-pointer "")))
-         (list (string-pointer-substring sp)
-               (string-pointer-substring sp :after #t))))
+         (set! se (string-cursor-end str))
+         (string-cursor? se)))
+(test* "string-cursor-next" #\b
+       (string-ref str (string-cursor-next str ss)))
+(test* "string-cursor-forward" #\d
+       (string-ref str (string-cursor-forward str ss 3)))
+(test* "string-cursor-prev" #\g
+       (string-ref str (string-cursor-prev str se)))
+(test* "string-cursor-back" #\d
+       (string-ref str (string-cursor-back str se 4)))
+(test* "string-index->cursor" #\f
+       (string-ref str (string-index->cursor str 5)))
+(test* "string-cursor->index" 0
+       (string-cursor->index str ss))
+(test* "string-cursor->index" 7
+       (string-cursor->index str se))
+(test* "substring with cursors" "cde"
+       (substring str
+                  (string-cursor-forward str ss 2)
+                  (string-cursor-back str se 2)))
+(test* "substring with cursors" "cde"
+       (substring str
+                  2
+                  (string-cursor-back str se 2)))
+(test* "substring with cursors" "cde"
+       (substring str
+                  (string-cursor-forward str ss 2)
+                  5))
+(test* "string->list" '(#\b #\c #\d #\e #\f)
+       (string->list str
+                     (string-cursor-next str ss)
+                     (string-cursor-prev str se)))
+(test* "string->list" '(#\b #\c #\d #\e #\f)
+       (string->list str
+                     1
+                     (string-cursor-prev str se)))
+(test* "string->list" '(#\b #\c #\d #\e #\f)
+       (string->list str
+                     (string-cursor-next str ss)
+                     6))
+(test* "string-cursor=?" #t
+       (string-cursor=? ss (string-cursor-back str se 7)))
+(test* "string-cursor=?" #t
+       (string-cursor=? 0 0))
+(test* "string-cursor<?" #t
+       (string-cursor<? ss se))
+(test* "string-cursor<=?" #t
+       (string-cursor<=? ss se))
+(test* "string-cursor<=?" #t
+       (string-cursor<=? ss ss))
+(test* "string-cursor>?" #t
+       (string-cursor>? se ss))
+(test* "string-cursor>=?" #t
+       (string-cursor>=? se se))
+(test* "string-cursor-diff" 7
+       (string-cursor-diff str ss se))
+(test* "string-cursor-diff" -7
+       (string-cursor-diff str se ss))
 
-(test* "make-string-pointer (bound)" #t
-       (begin
-         (set! sp (make-string-pointer "abcdefg" 1 2 5))
-         (string-pointer? sp)))
-(test* "string-pointer-next! (bound)" #\d
-       (string-pointer-next! sp))
-(test* "string-pointer-next! (bound)" #\e
-       (string-pointer-next! sp))
-(test* "string-pointer-next! (bound)" #t
-       (eof-object? (string-pointer-next! sp)))
-(test* "string-pointer-prev! (bound)" #\e
-       (string-pointer-prev! sp))
-(test* "string-pointer-prev! (bound)" #\d
-       (string-pointer-prev! sp))
-(test* "string-pointer-prev! (bound)" #\c
-       (string-pointer-prev! sp))
-(test* "string-pointer-prev! (bound)" #t
-       (eof-object? (string-pointer-prev! sp)))
-(test* "string-pointer-next! (bound)" #\c
-       (string-pointer-next! sp))
-(test* "string-pointer-substring (bound)" '("c" "de")
-       (list (string-pointer-substring sp)
-             (string-pointer-substring sp :after #t)))
+;;-------------------------------------------------------------------
+(test-section "string mappers")
+
+(test* "string-map, single arg" "O'AFU"
+       (string-map char-upcase "o'afu"))
+(test* "string-map, single arg" ""
+       (string-map char-upcase ""))
+(test* "string-map, multi arg" "orpng"
+       (string-map (^[a b] (if (char>? a b) a b))
+                   "apple" "orange"))
+(test* "string-map, srfi-13 style" "CALIFRAGILISTICEXPIALIDOCIOUS"
+       (string-map char-upcase "supercalifragilisticexpialidocious" 5))
+(test* "string-map, srfi-13 style" "CALIFRAGILISTIC"
+       (string-map char-upcase "supercalifragilisticexpialidocious" 5 20))
+(test* "string-map, bad arg" (test-error)
+       (string-map (^ _ #\space) "abc" 'def))
+(test* "string-map, bad arg" (test-error)
+       (string-map (^ _ #\space) 'abc))
+(test* "string-map, bad arg" (test-error)
+       (string-map (^ _ #\space) "abc" "def" 2))
+(test* "string-map, bad arg" (test-error)
+       (string-map (^ _ #\space) "abc" 1 2 3))
+(test* "string-for-each single arg" "maui"
+       (with-output-to-string
+         (cut string-for-each write-char "maui")))
+(test* "string-for-each multi arg" "Niuihi"
+       (with-output-to-string
+         (cut string-for-each (^[a b] (write-char (if (char>? a b) a b)))
+              "Kaua'i" "Ni'ihau")))
+(test* "string-for-each srfi-13 style" "califragilistic"
+       (with-output-to-string
+         (cut string-for-each write-char
+              "supercalifragilisticexpialidocious" 5 20)))
 
 ;;-------------------------------------------------------------------
 (test-section "input string port")
@@ -646,6 +686,19 @@
                    (* *dstr-init-size* (+ *dstr-incr-factor* 1))
                    )
 
+;;-------------------------------------------------------------------
+(test-section "immutablility")
+
+(test* "string-immutable?" '(#t #f)
+       (list (string-immutable? "abc")
+             (string-immutable? (string-copy "abc"))))
+
+(test* "string-copy-immutable" '(#t #t #t #t)
+       (list (string-immutable? (string-copy-immutable "abc"))
+             (string-immutable? (string-copy-immutable "abc" 0 1))
+             (string-immutable? (string-copy-immutable (string-copy "abc")))
+             (string-immutable? (string-copy-immutable (string-copy "abc") 0 1))))
+
 ;;-------------------------------------------------------------------
 (test-section "string interpolation")
 
