@@ -5,11 +5,6 @@
 (use gauche.test)
 (test-start "text utilities")
 
-;;-------------------------------------------------------------------
-(test-section "console")
-(use text.console)
-(test-module 'text.console)
-
 ;;-------------------------------------------------------------------
 (test-section "csv")
 (use text.csv)
@@ -169,109 +164,100 @@ fuga
        (with-output-to-string
          (lambda () (diff-report diff-a diff-b))))
 
+
 ;;-------------------------------------------------------------------
-(test-section "gap-buffer")
-(use text.gap-buffer)
+(test-section "edn")
+(use text.edn)
+(use srfi-13)
 (use gauche.uvector)
+(test-module 'text.edn)
 
-(test-module 'text.gap-buffer)
+;; simple stuff
+(let1 data `[(#t . "  true ")
+             (#f . "false")
+             (nil . "nil ")
+             (symbol . " symbol ")
+             (namespace/name . " namespace/name")
+             (:keyword . ":keyword")
+             ("string" . "\"string\"")
+             ("weird \" characters \t\n\r\\" 
+              . "\"weird \\\" characters \\t\\n\\r\\\\\"")
+             (#\a . "\\a")
+             (#\newline . "\\newline")
+             (#\return . "\\return")
+             (#\space . "\\space")
+             (#\tab . "\\tab")
+             (0 . "0")
+             (-1 . "-1")
+             (() . "()")
+             ((a b c) . "(a b c)")
+             (#(a b c) . "[a b c]")
+             (#((a b) (a c) #(b c)) . "[(a b) (a c) [b c]]")]
+  (test* "parser basics" (map car data)
+         (map ($ parse-edn-string $ cdr $) data))
+  (test* "writer basics" (map ($ string-trim-both $ cdr $) data)
+         (map ($ construct-edn-string $ car $) data)))
 
-;; stringify gap-buffer for test purpose
-(define (gap-buffer-visualize gbuf)
-  (with-output-to-string
-    (^[]
-      (let1 buf (~ gbuf'buffer) ;accessing internal
-        (dotimes [i (gap-buffer-gap-start gbuf)]
-          (display (integer->char (~ buf i))))
-        (dotimes [i (- (gap-buffer-gap-end gbuf) (gap-buffer-gap-start gbuf))]
-          (display #\_))
-        (dotimes [i (- (gap-buffer-capacity gbuf) (gap-buffer-gap-end gbuf))]
-          (display (integer->char (~ buf (+ i (gap-buffer-gap-end gbuf))))))))))
+;; numeric formats (round-trip is not guaranteed)
+;; NB: Edn page doesn't mention it, but Clojure reader also supports
+;; 0xNNNN (hexadecimal) and 0NNN (octal) representations.
+(let1 data `[(1234 . "1234")
+             (-1234 . "-1234N")
+             (1234 . "+1234N")
+             (1234.0 . "1234M")
+             (1234.5678 . "1234.5678")
+             (12.34 . "1234e-2")
+             (-1234.5678 . "-12.345678e2")
+             (1234.5678 . "12.345678e+2")
+             (1234.5678 . "+12.345678E+2")
+             (4660      . "0x1234")
+             (-4660      . "-0x1234")
+             (668        . "01234")
+             (-668       . "-01234")]
+  (test* "parser numeric" (map car data)
+         (map ($ parse-edn-string $ cdr $) data)))
 
-(test* "constuct" "abcde___"
-       (gap-buffer-visualize (string->gap-buffer "abcde")))
-(test* "constuct" "___abcde"
-       (gap-buffer-visualize (string->gap-buffer "abcde" 0 'beginning)))
-(test* "constuct" "bcd_"
-       (gap-buffer-visualize (string->gap-buffer "abcde" 0 'end 1 4)))
-  
-(let1 gbuf (string->gap-buffer "abcde")
-  (test* "move" "abcd___e"
-         (gap-buffer-visualize (gap-buffer-move! gbuf -1 'current)))
-  (test* "move" "a___bcde"
-         (gap-buffer-visualize (gap-buffer-move! gbuf 1)))
-  (test* "move" "abc___de"
-         (gap-buffer-visualize (gap-buffer-move! gbuf -2 'end)))
-  (test* "insert" "abcZ__de"
-         (gap-buffer-visualize (gap-buffer-insert! gbuf #\Z)))
-  (test* "insert" "abcZxyde"
-         (gap-buffer-visualize (gap-buffer-insert! gbuf "xy")))
-  (test* "insert" "abcZxyw_______de"
-         (gap-buffer-visualize (gap-buffer-insert! gbuf #\w)))
-  (test* "insert"
-         "abcZxyw012345678901234567890123456789_________________________de"
-         (gap-buffer-visualize
-          (gap-buffer-insert! gbuf "012345678901234567890123456789")))
-  (test* "delete"
-         "abcZxyw012345678901234567890123456789__________________________e"
-         (gap-buffer-visualize
-          (gap-buffer-delete! gbuf 1)))
-  (test* "delete"
-         "abc____________________________________678901234567890123456789e"
-         (begin (gap-buffer-move! gbuf 3)
-                (gap-buffer-visualize
-                 (gap-buffer-delete! gbuf 10))))
-  (test* "delete"
-         (test-error)
-         (gap-buffer-visualize (gap-buffer-delete! gbuf 100)))
-  (test* "change"
-         "abcdef______________________________________1234567890123456789e"
-         (gap-buffer-visualize (gap-buffer-change! gbuf 5 "def")))
-  )
+;; other aggregates (round-trip is not guaranteed)
+(let1 data `(("{:a 1, :b 2, :c 3}" . ,(edn-map :a 1 :b 2 :c '3))
+             ("#{a b c d e}" . ,(edn-set 'a 'b 'c 'd 'e))
+             ("#myobj{:a 1}" . ,(make-edn-object 'myobj (edn-map :a 1)))
+             ("#foo\"abc\"" .  ,(make-edn-object 'foo "abc"))
+             ("#bar[1 2 3]" .  ,(make-edn-object 'bar '#(1 2 3)))
+             ("#baz 2345" .    ,(make-edn-object 'baz 2345))
+             )
+  (dolist [d data]
+    (test* (format "parse ~s" d) (cdr d) (parse-edn-string (car d)) edn-equal?)
+    (test* (format "write ~s" d) (parse-edn-string (car d))
+           ($ parse-edn-string $ construct-edn-string
+              $ parse-edn-string $ car d)
+           edn-equal?)))
 
-(let1 gbuf (string->gap-buffer "abcde")
-  (test* "->string (gap at end)" "abcde"
-         (gap-buffer->string gbuf))
-  (test* "->string (gap at beginning)" "abcde"
-         (begin (gap-buffer-move! gbuf 0)
-                (gap-buffer->string gbuf)))
-  (test* "->string (gap at middle)" "abcde"
-         (begin (gap-buffer-move! gbuf 3 'current)
-                (gap-buffer->string gbuf)))
-  )
+;; custom object
+(register-edn-object-handler! 'u8vector 
+                              (^[tag vec] (vector->u8vector vec)))
+(define-method edn-write ((x <u8vector>))
+  (display "#u8vector")
+  (edn-write (u8vector->vector x)))
+(test* "custom parse" '#u8(1 2 3 4)
+       (parse-edn-string "#u8vector[1 2 3 4]"))
+(test* "custom writer" "#u8vector[1 2 3 4]"
+       (construct-edn-string '#u8(1 2 3 4)))
 
-(let1 gbuf (gap-buffer-copy (string->gap-buffer "abcde"))
-  (test* "undo and redo"
-         '("abXYZcde" "abXYde" "aQRSYde" "abXYde" "abXYZcde"
-           "abcde" "aQRSYde" "abcde")
-         (let* ([u0 (begin (gap-buffer-move! gbuf 2)
-                           (gap-buffer-edit! gbuf '(i #f "XYZ")))]
-                [s0 (gap-buffer->string gbuf)] ; abXYZcde
-                [u1 (begin (gap-buffer-move! gbuf -1 'current)
-                           (gap-buffer-edit! gbuf '(d #f 2)))]
-                [s1 (gap-buffer->string gbuf)] ; abXYde
-                [u2 (begin (gap-buffer-move! gbuf -3 'current)
-                           (gap-buffer-edit! gbuf '(c #f 2 "QRS")))]
-                [s2 (gap-buffer->string gbuf)] ; aQRSYde
-                [r2 (begin (gap-buffer-move! gbuf 0)
-                           (gap-buffer-edit! gbuf u2))] ; undo change!
-                [s3 (gap-buffer->string gbuf)] ; abXYde
-                [r1 (gap-buffer-edit! gbuf u1)] ; undo delete!
-                [s4 (gap-buffer->string gbuf)] ; abXYZcde
-                [r0 (begin (gap-buffer-move! gbuf -1 'end)
-                           (gap-buffer-edit! gbuf u0))] ; undo insert!
-                [s5 (gap-buffer->string gbuf)] ; abcde
-                [u0 (gap-buffer-edit! gbuf r0)] ; redo insert!
-                [u1 (gap-buffer-edit! gbuf r1)] ; redo delete!
-                [u2 (gap-buffer-edit! gbuf r2)] ; redo change!
-                [s6 (gap-buffer->string gbuf)] ; aQRSYde
-                [s7 (begin (gap-buffer-edit! gbuf u2)
-                           (gap-buffer-edit! gbuf u1)
-                           (gap-buffer-edit! gbuf u0)
-                           (gap-buffer->string gbuf))] ; abcde
-                )
-           (list s0 s1 s2 s3 s4 s5 s6 s7)))
-  )
+
+;; invalid
+(let1 invalids '("foo/bar/baz" "/foo" "foo/")
+  (dolist [d invalids]
+    (test* (format "parse (invalid) ~s" d) 
+           (test-error <edn-parse-error> #/invalid token/)
+           (parse-edn-string d))))
+(test* (format "valid symbol name") '(#f #f #f #f #t)
+       (map edn-valid-symbol-name? 
+            '("nil" "bar/nil" "true" "false" "nile")))
+
+;;-------------------------------------------------------------------
+(test-section "external-editor")
+(use text.external-editor)
+(test-module 'text.external-editor)
 
 ;;-------------------------------------------------------------------
 (test-section "html-lite")
@@ -573,50 +559,9 @@ fuga
        (sql-tokenize "?,?,:foo, :bar , ?"))
 
 ;;-------------------------------------------------------------------
-(test-section "line-edit")
-(use text.line-edit)
-(test-module 'text.line-edit)
-
-;; Test internal functions
-(let ()
-  (define (t s pos expect)
-    (test* #"buffer-find-matching-paren: ~s ~pos" expect
-           ((with-module text.line-edit buffer-find-matching-paren)
-            ((with-module text.gap-buffer string->gap-buffer) s)
-            0 pos)))
-
-  (t "(abc)" 4 0)
-  (t "aa(abc)zz" 6 2)
-  (t "(a(abc)z)" 6 2)
-  (t "(a)(abc)z" 7 3)
-  (t "(a\\(b)z" 5 0)
-  (t "(a\\(\\))z" 6 0)
-  (t "(a\\)\\()z" 6 0)
-  (t "(a(abc)(def)z)" 11 7)
-  (t "(a(abc)(def)z)" 13 0)
-  (t "aaaabc)zz" 6 #f)
-  (t "(a{b}c)" 4 2)
-  (t "(a{b}c)" 6 0)
-  (t "(a{b(}c)" 5 #f)
-  (t "(a{b(}c)" 7 4)
-  (t "(a{b)}c)" 4 #f)
-  (t "(a{b)}c)" 5 2)
-  (t "(a\"(\"b)" 6 0)
-  (t "(a\")\"b)" 6 0)
-  (t "(a\"\\\"(\"b)" 8 0)
-  (t "(a#/bc([d\\//)]" 12 0)
-  (t "(a#/bc([d\\//)]" 13 #f)
-  (t "(a#(b)c)" 5 3)
-  (t "(a#(b)c)" 7 0)
-  (t "(a#[b[:space:]]c)" 14 3)
-  (t "(a#[b[:space:]]c)" 13 5)
-  (t "(a#[b[:space:]]c)" 16 0)
-  (t "(a#{b}c)" 5 3)
-  (t "(a#{b)c}" 5 #f)
-  (t "(a#{b)c}" 7 3)
-  (t "(a\\#(b)c)" 6 4)
-  (t "(a#\\(b)c)" 6 0)
-  )
+(test-section "pager")
+(use text.pager)
+(test-module 'text.pager)
 
 ;;-------------------------------------------------------------------
 (test-section "template")
@@ -661,4 +606,113 @@ fuga
 (use text.unicode.ucd)
 (test-module 'text.unicode.ucd)
 
+(test-section "unicode.codeset")
+(use text.unicode.codeset)
+(test-module 'text.unicode.codeset)
+
+(let ([cs (make <char-code-set>)])
+  ;; testing overlapped ranges
+  (add-code-range! cs 64 127)
+  (add-code-range! cs 8 80)
+  (add-code-range! cs 100 1000)
+  (add-code-range! cs 200 210)   ; totally subsumed
+  (add-code-range! cs 1500 2000) ; independent
+
+  (test* "add-code-range! (bitmap)"
+         #xffff_ffff_ffff_ffff_ffff_ffff_ffff_ff00
+         (~ cs'small-map))
+
+  (test* "add-code-range! (subsumed)"
+         '((128 . 1000) (1500 . 2000))
+         (tree-map->alist (~ cs'large-map)))
+
+  (add-code-range! cs 1800 2100) ; partially overlap
+  (add-code-range! cs 1400 1600)
+  (test* "add-code-range! (partially overlap)"
+         '((128 . 1000) (1400 . 2100))
+         (tree-map->alist (~ cs'large-map)))
+
+  (add-code-range! cs 1300 1400) ; adjacent
+  (add-code-range! cs 1000 1100) ; adjacent
+  (add-code-range! cs 2100 2200) ; adjacent
+  (test* "add-code-range! (adjacent)"
+         '((128 . 1100) (1300 . 2200))
+         (tree-map->alist (~ cs'large-map)))
+
+  (add-code-range! cs 2500 2600)
+  (add-code-range! cs 2700 2800)
+  (add-code-range! cs 2900 3000)
+  (add-code-range! cs 3100 3200)
+  (add-code-range! cs 3300 3400)
+  (add-code-range! cs 2550 3050)
+  (test* "add-code-range! (multiple)"
+         '((128 . 1100) (1300 . 2200) (2500 . 3050) (3100 . 3200) (3300 . 3400))
+         (tree-map->alist (~ cs'large-map)))
+
+  (add-code-range! cs 3090 3300)
+  (test* "add-code-range! (multiple)"
+         '((128 . 1100) (1300 . 2200) (2500 . 3050) (3090 . 3400))
+         (tree-map->alist (~ cs'large-map)))
+
+  (add-code! cs 3401)
+  (add-code! cs 3089)
+  (test* "add-code! (next)"
+         '((128 . 1100) (1300 . 2200) (2500 . 3050) (3089 . 3401))
+         (tree-map->alist (~ cs'large-map)))
+
+  (add-code-range! cs 3402 3500)
+  (test* "add-code-range! (next)"
+         '((128 . 1100) (1300 . 2200) (2500 . 3050) (3089 . 3500))
+         (tree-map->alist (~ cs'large-map)))
+
+  (add-code-range! cs 3052 3088)
+  (test* "add-code-range! (next)"
+         '((128 . 1100) (1300 . 2200) (2500 . 3050) (3052 . 3500))
+         (tree-map->alist (~ cs'large-map)))
+
+  (add-code! cs 3051)
+  (test* "add-code! (fill)"
+         '((128 . 1100) (1300 . 2200) (2500 . 3500))
+         (tree-map->alist (~ cs'large-map)))
+
+  (add-code-range! cs 1101 1299)
+  (test* "add-code-range! (fill)"
+         '((128 . 2200) (2500 . 3500))
+         (tree-map->alist (~ cs'large-map)))
+
+  (add-code-range! cs 4000 4500)
+  (add-code-range! cs 2300 3999)
+  (test* "add-code-range! (fill)"
+         '((128 . 2200) (2300 . 4500))
+         (tree-map->alist (~ cs'large-map)))
+  )
+
+(let ([cs1 (make <char-code-set>)]
+      [cs2 (make <char-code-set>)])
+  (add-code-range! cs1 8 15)
+  (add-code-range! cs1 120 200)
+  (add-code-range! cs1 300 400)
+  (add-code! cs1 500)
+  (add-code! cs1 600)
+  (add-code-range! cs1 1000 2000)
+  (add-code-range! cs1 3000 4000)
+  (add-code-range! cs1 5000 6000)
+  (add-code-range! cs1 7000 8000)
+
+  (add-code-range! cs2 24 31)
+  (add-code-range! cs2 128 250)
+  (add-code-range! cs2 300 500)
+  (add-code-range! cs2 600 900)
+  (add-code-range! cs2 1100 5100)
+  (add-code-range! cs2 5800 9000)
+
+  (let1 cs (code-set-union #f cs1 cs2)
+    (test* "code-set-union (bitmap)"
+           #xff00_0000_0000_0000_0000_0000_ff00_ff00
+           (~ cs'small-map))
+    (test* "code-set-union (large-map"
+           '((128 . 250) (300 . 500) (600 . 900) (1000 . 9000))
+           (tree-map->alist (~ cs'large-map))))
+  )
+
 (test-end)
