@@ -90,40 +90,125 @@
   )
 
 ;;-------------------------------------------------------------------
-(test-section "string-pointer")
-(define sp #f)
-(test "make-string-pointer" #t
-      (lambda ()
-        (set! sp (make-string-pointer "いろはにhoへと"))
-        (string-pointer? sp)))
-(test "string-pointer-next!" #\い
-      (lambda () (string-pointer-next! sp)))
-(test "string-pointer-next!" #\ろ
-      (lambda () (string-pointer-next! sp)))
-(test "string-pointer-prev!" #\ろ
-      (lambda () (string-pointer-prev! sp)))
-(test "string-pointer-prev!" #\い
-      (lambda () (string-pointer-prev! sp)))
-(test "string-pointer-prev!" #t
-      (lambda () (eof-object? (string-pointer-prev! sp))))
-(test "string-pointer-index" 0
-      (lambda () (string-pointer-index sp)))
-(test "string-pointer-index" 8
-      (lambda () (do ((x (string-pointer-next! sp) (string-pointer-next! sp)))
-                     ((eof-object? x) (string-pointer-index sp)))))
-(test "string-pointer-substring" '("いろはにhoへと" "")
-      (lambda () (list (string-pointer-substring sp)
-                       (string-pointer-substring sp :after #t))))
-(test "string-pointer-substring" '("いろはにh" "oへと")
-      (lambda ()
-        (string-pointer-set! sp 5)
-        (list (string-pointer-substring sp)
-              (string-pointer-substring sp :after #t))))
-(test "string-pointer-substring" '("" "いろはにhoへと")
-      (lambda ()
-        (string-pointer-set! sp 0)
-        (list (string-pointer-substring sp)
-              (string-pointer-substring sp :after #t))))
+(test-section "string-cursor")
+
+(define str "いろはにhoへと")
+(define ss #f)
+(define se #f)
+(test* "string-cursor-start" #t
+       (begin
+         (set! ss (string-cursor-start str))
+         (string-cursor? ss)))
+(test* "string-cursor-start" #t
+       (begin
+         (set! se (string-cursor-end str))
+         (string-cursor? se)))
+(test* "string-cursor-next" #\ろ
+       (string-ref str (string-cursor-next str ss)))
+(test* "string-cursor-forward" #\に
+       (string-ref str (string-cursor-forward str ss 3)))
+(test* "string-cursor-prev" #\と
+       (string-ref str (string-cursor-prev str se)))
+(test* "string-cursor-back" #\に
+       (string-ref str (string-cursor-back str se 5)))
+(test* "string-index->cursor" #\へ
+       (string-ref str (string-index->cursor str 6)))
+(test* "string-cursor->index" 0
+       (string-cursor->index str ss))
+(test* "string-cursor->index" 8
+       (string-cursor->index str se))
+(test* "substring with cursors" "はにho"
+       (substring str
+                  (string-cursor-forward str ss 2)
+                  (string-cursor-back str se 2)))
+(test* "substring with cursors" "はにho"
+       (substring str
+                  2
+                  (string-cursor-back str se 2)))
+(test* "substring with cursors" "はにh"
+       (substring str
+                  (string-cursor-forward str ss 2)
+                  5))
+(test* "string->list" '(#\ろ #\は #\に #\h #\o #\へ)
+       (string->list str
+                     (string-cursor-next str ss)
+                     (string-cursor-prev str se)))
+(test* "string->list" '(#\ろ #\は #\に #\h #\o #\へ)
+       (string->list str
+                     1
+                     (string-cursor-prev str se)))
+(test* "string->list" '(#\ろ #\は #\に #\h #\o #\へ)
+       (string->list str
+                     (string-cursor-next str ss)
+                     7))
+(test* "string-cursor=?" #t
+       (string-cursor=? ss (string-cursor-back str se 8)))
+(test* "string-cursor=?" #t
+       (string-cursor=? 0 0))
+(test* "string-cursor<?" #t
+       (string-cursor<? ss se))
+(test* "string-cursor<=?" #t
+       (string-cursor<=? ss se))
+(test* "string-cursor<=?" #t
+       (string-cursor<=? ss ss))
+(test* "string-cursor>?" #t
+       (string-cursor>? se ss))
+(test* "string-cursor>=?" #t
+       (string-cursor>=? se se))
+(test* "string-cursor-diff" 8
+       (string-cursor-diff str ss se))
+(test* "string-cursor-diff" -8
+       (string-cursor-diff str se ss))
+
+;;-------------------------------------------------------------------
+(test-section "string index building")
+
+(let* ([make-cycle (^[start-char count]
+                     ($ circular-list
+                        $* map ucs->char (iota count (char->ucs start-char))))]
+       [cycle1 (make-cycle #\a 26)]
+       [cycle2 (make-cycle #\α 25)]
+       [cycle3 (make-cycle #\あ 85)])
+  (define (make-str max-size)
+    (let loop ([size 0] [r '()] 
+               [srcs (circular-list cycle1 cycle2 cycle3)]
+               [counts (circular-list 1 2 3)])
+      (let ([next-char (caar srcs)]
+            [next-count (car counts)])
+        (if (> (+ size next-count) max-size)
+          (list->string (reverse r))
+          (loop (+ size next-count)
+                (cons next-char r)
+                (circular-list (cadr srcs) (caddr srcs) (cdar srcs))
+                (cdr counts))))))
+  (define (test-string-index max-size)
+    (let1 s (string-build-index! (make-str max-size))
+      ;;((with-module gauche.internal %string-index-dump) s)
+      (test* #"string indexing (size=~max-size)"
+             #f
+             (any (^p (and (not (eqv? (car p) (string-ref s (cdr p))))
+                           (list 'pos (cdr p)
+                                 'expect (car p)
+                                 'actual (string-ref s (cdr p)))))
+                  (map cons (string->list s) (liota))))))
+  (test-string-index 0)
+  (test-string-index 15)
+  (test-string-index 33)
+  (test-string-index 34)
+  (test-string-index 49)
+  (test-string-index 63)
+  (test-string-index 64)
+  (test-string-index 65)
+  (test-string-index 255)
+  (test-string-index 256)
+  (test-string-index 257)
+  (test-string-index 8191)
+  (test-string-index 8193)
+  (test-string-index 65535)
+  (test-string-index 65537)
+  (test-string-index 131072)
+  (test-string-index 131073)
+  )
 
 ;;-------------------------------------------------------------------
 (test-section "incomplete strings")
@@ -771,5 +856,9 @@
        (cond ((rxmatch (string->regexp "о[А-Я]а") "она")
               => rxmatch-substring)
              (else #f)))
+(test* "regexp/unicode-ci" "σζ\x01C5;"
+       (cond ((rxmatch (string->regexp "[[:LOWER:]]*" :case-fold #t) "σζ\x01C5;")
+              => rxmatch-substring)
+             (else #f)))
 
 (test-end)
