@@ -800,6 +800,24 @@
 (test* "stream-delay" #t
        (stream? (stream-delay (error "Boo"))))
 
+(test* "stream-lambda/stream-take" '(1 3 5 7 9)
+       (letrec ([iter (stream-lambda (f x)
+                        (stream-cons x (iter f (f x))))])
+         (stream->list (stream-take (iter (pa$ + 2) 1) 5))))
+
+(test* "define-stream/stream-take" '(0 2 4 6 8)
+       (let ()
+         (define-stream (iter f x)
+           (stream-cons x (iter f (f x))))
+         (stream->list (stream-take (iter (pa$ + 2) 0) 5))))
+
+(test* "stream-unfold" '(0 1 4 9 16 25 36 49 64 81)
+       (stream->list 10
+                     (stream-unfold (cut expt <> 2)
+                                    (^_ #t)
+                                    (cut + <> 1)
+                                    0)))
+
 (test* "stream-unfoldn" '((0 2 4 6) (1 3 5 7))
        (receive (s0 s1)
            (stream-unfoldn (lambda (s)
@@ -814,6 +832,25 @@
                       (stream-fourth s)))
               (list s0 s1))))
 
+(test* "stream-unfolds" '((1 3 5 7) (0 2 4 6))
+       (receive (s0 s1)
+           (stream-unfolds (^s (if (stream-null? s)
+                                 (values s '() '())
+                                 (if (odd? (stream-car s))
+                                   (values (stream-cdr s) 
+                                           (list (stream-car s))
+                                           #f)
+                                   (values (stream-cdr s)
+                                           #f
+                                           (list (stream-car s))))))
+                           (stream-iota 10))
+         (map (lambda (s)
+                (list (stream-first s)
+                      (stream-second s)
+                      (stream-third s)
+                      (stream-fourth s)))
+              (list s0 s1))))
+
 (test* "stream-map/stream-ref" '(2 4 6 8)
        (let1 s
            (stream-map (cut * 2 <>)
@@ -829,14 +866,144 @@
                           (stream-iota -1 1))
          (reverse r)))
 
+(test* "stream-iota" '(1.0 1.5 2.0 2.5 3.0)
+       (stream->list (stream-iota 5 1 0.5)))
+
+(test* "stream-iota" '(1.0 1.0+1.0i 1.0+2.0i 1.0+3.0i)
+       (stream->list (stream-iota 4 1 +i)))
+
 (test* "stream-filter/take/drop" '(12 15 18 21)
        (let1 s (stream-filter (^x (zero? (modulo x 3)))
-                              (stream-iota #f))
+                              (stream-iota))
          (stream->list (stream-take (stream-drop s 4) 4))))
 
+(test* "stream-range" '(10 9 8 7)
+       (stream->list (stream-range 10 6)))
+
+(test* "stream-range" '(10.0 9.0 8.0 7.0)
+       (stream->list (stream-range 10 6.0)))
+
+(test* "stream-range" (lrange 0 1 0.1)
+       (stream->list (stream-range 0 1 0.1)))
+
+(test* "stream-drop-while/stream-range" '(5 6 7 8 9)
+       (stream->list 5 (stream-drop-while (cut < <> 5)
+                                          (stream-range 0))))
+
+(test* "stream-take-while/stream-range" '(0 1 2 3 4)
+       (stream->list (stream-take-while (cut < <> 5)
+                                        (stream-range 0))))
+
+(test* "stream-fold" '(4 3 2 1 0)
+       (stream-fold xcons '() (stream-iota 5)))
+
+(test* "stream-from" '(0 1 2 3 4)
+       (stream->list 5 (stream-from 0)))
+
+(test* "stream-from" '(1 3 5 7 9)
+       (stream->list 5 (stream-from 1 2)))
+
+(test* "stream-from" '(1 3/2 2 5/2 3)
+       (stream->list 5 (stream-from 1 1/2)))
+
+(test* "stream-from" '(1.0 1.1 1.2 1.3 1.4)
+       (stream->list 5 (stream-from 1 0.1)))
+
+(test* "stream-iterate" '(10 11 12 13 14)
+       (stream->list 5 (stream-iterate (cut + <> 1) 10)))
+
+(test* "stream-iterate" '(1 2 4 8 16)
+       (stream->list 5 (stream-iterate (cut * <> 2) 1)))
+
+(test* "stream-length" 6 (stream-length (stream 0 1 2 3 4 5)))
+(test* "stream-length" 0 (stream-length (stream)))
+
+(test* "stream-match" 5
+       (let ()
+         (define (len s)
+           (stream-match s
+             [() 0]
+             [(head . tail) (+ 1 (len tail))]))
+         (len (stream 0 1 2 3 4))))
+
+(test* "stream-match" 'ok
+       (stream-match (stream 1 1 2)
+         [(x y . _) (equal? x y) 'ok]
+         [else 'ng]))
+
+(test* "stream-match" 'ng
+       (stream-match (stream 1 2 1)
+         [(x y . _) (equal? x y) 'ok]
+         [else 'ng]))
+
+(test* "stream-of" '(0 4 16 36 64)
+       (stream->list
+        (stream-of (* x x)
+                   (x in (stream-range 0 10))
+                   (even? x))))
+
+(test* "stream-of" '((1 1) (1 2) (2 1) (2 2) (3 1) (3 2))
+       (stream->list
+        (stream-of (list a b)
+                   (a in (stream-range 1 4))
+                   (b in (stream-range 1 3)))))
+
+(test* "stream-of" '((3 4 5) (5 12 13) (6 8 10))
+       (stream->list
+        (stream-of (list a b c)
+                   (a in (stream-range 1 14))
+                   (b in (stream-range a 14))
+                   (c in (stream-range b 14))
+                   (d is (+ (* a a) (* b b)))
+                   (= (* c c) d))))
+
+(test* "stream-reverse" '(5 4 3 2 1)
+       (stream->list (stream-reverse (stream 1 2 3 4 5))))
+(test* "stream-reverse" '()
+       (stream->list (stream-reverse stream-null)))
+
+(test* "stream-scan" '(0 1 3 6 10 15)
+       (stream->list 6 (stream-scan + 0 (stream-from 1))))
+(test* "stream-scan"  '(1 1 2 3 5 8 13 21 34 55)
+       (letrec ((fibs (stream-scan + 1 (stream-cons 0 fibs))))
+         (stream->list 10 fibs)))
+(test* "stream-scan" '(() (a) (b a) (c b a) (d c b a) (e d c b a))
+       (stream->list (stream-scan xcons '() (stream 'a 'b 'c 'd 'e))))
+
+(test* "stream-zip" '((0 a) (1 b) (2 c))
+       (stream->list
+        (stream-zip (stream 0 1 2) (stream 'a 'b 'c))))
+
+(test* "stream-zip" '(0 3 5 7 10)
+       (let ()
+         (define-stream (stream-finds eq obj s)
+           (stream-of (car x)
+                      (x in (stream-zip (stream-from 0) s))
+                      (eq obj (cadr x))))
+         (stream->list
+          (stream-finds char=? #\a
+                        (string->stream "abracadabra")))))
+
 (test* "stream-remove" '(0 2 4 6)
        (stream->list (stream-remove odd? (stream-iota 8))))
 
+(test* "port->stream" '(#\a #\b #\c)
+       (let1 p (open-input-string "abc")
+         (stream->list (port->stream p))))
+
+(test* "stream+" '(1 1/2 1/3)
+       (stream->list (stream-take (stream+ 1 (/ 1 2) (/ 1 3) (/ 1 0)) 3)))
+
+(test* "stream-append" '(1 2 3 4 a b c d A B C D)
+       (stream->list
+        (stream-append (stream 1 2 3 4)
+                       (stream 'a 'b 'c 'd)
+                       (stream 'A 'B 'C 'D))))
+
+(test* "stream-constant" '(1 2 3 1 2 3 1 2)
+       (stream->list 8
+                     (stream-constant 1 2 3)))
+
 (test* "stream-count" 4
        (stream-count odd? (stream-iota 8)))
 
