@@ -12,9 +12,9 @@
     [(_ in out) (file-filter doit :input in :output out)]
     [_ (exit 1 "Usage: ~a <input-file> <output-file>" (car args))])
   0)
-    
+
 (define (doit in out)
-  (display "{\\rtf1\\ansi\\ansicpg1252\\deff0{\\fonttbl{\\f0\\fnil\\fcharset0 Arial Narrow;}}\r\n" out)
+  (display "{\\rtf1\\ansi\\ansicpg1252\\deff0{\\fonttbl{\\f0\\fnil\\fcharset0 Tahoma;}}\r\n" out)
   (display "\\viewkind4\\uc1\\pard\\lang1033\\f0\\fs20 " out)
   (let loop ([prev-line (read-line in)])
     (unless (eof-object? prev-line)
@@ -28,8 +28,5 @@
                (loop line)]
               [else
                (format out "~a\\par\r\n" prev-line)
-               (loop line)])))))
-
-      
-             
-                             
\ No newline at end of file
+               (loop line)]))))
+  (display "}" out))
